(function () {
    'use strict';

    class PanelRegister {
        static registerClass(pkgName, resName, cls, pkg = UrlUtils.UI) {
            let pkgUrl = UrlUtils.getFairyPkg(pkgName, pkg);
            if (pkgName && !fgui.UIPackage.getByName(resName)) {
                fgui.UIPackage.addPackage(pkgUrl);
            }
            let url = fgui.UIPackage.getItemURL(pkgName, resName);
            fgui.UIObjectFactory.setPackageItemExtension(url, cls);
        }
        static createGObject(pkgName, resName) {
            let obj = null;
            let _id = pkgName + "_" + resName;
            obj = Laya.Pool.getItem(_id);
            if (obj) {
                obj.displayObject.active = true;
                return obj;
            }
            let count = this.poolCount.hasOwnProperty(_id) ? this.poolCount[_id] : 0;
            let url = this.getUrl(pkgName, resName);
            obj = fgui.UIPackage.createObjectFromURL(url);
            obj._id = _id + "_" + count;
            count++;
            this.poolCount[_id] = count;
            obj.displayObject.active = true;
            return obj;
        }
        static recover(obj) {
            if (obj != null) {
                let _id = obj.id.substr(0, obj.id.lastIndexOf("_"));
                obj.displayObject.active = false;
                obj.removeFromParent();
                fgui.GTween.kill(obj);
                Laya.Pool.recover(_id, obj);
            }
        }
        static getUrl(pkgName, resName) {
            let url = fgui.UIPackage.getItemURL(pkgName, resName);
            return url;
        }
        static loadUrl(url, handler = null) {
            if (!url) {
                handler && handler.runWith(null);
                return;
            }
            let item = fgui.UIPackage.getItemByURL(url);
            let pkg = item && item.owner;
            let sprite = pkg && pkg["_sprites"][item.id];
            let atlasItem = sprite && sprite.atlas;
            if (atlasItem && atlasItem.texture == null) {
                let urls = [{ url: atlasItem.file, type: Laya.Loader.IMAGE }];
                fgui.AssetProxy.inst.load(urls, Laya.Handler.create(this, function (url, atlasItem, handler) {
                    atlasItem.load();
                    handler && handler.runWith(url);
                }, [url, atlasItem, handler]));
            }
            else {
                handler && handler.runWith(url);
            }
        }
        static getTextureBy(pkgName, resName) {
            let url = this.getUrl(pkgName, resName);
            return this.getTexture(url);
        }
        static getTexture(url) {
            let item = fgui.UIPackage.getItemByURL(url);
            if (item) {
                item.load();
            }
            return item ? item.texture : null;
        }
        static getItemAsset(pkgName, resName) {
            let url = fgui.UIPackage.getItemURL(pkgName, resName);
            return fgui.UIPackage.getItemAssetByURL(url);
        }
    }
    PanelRegister.poolCount = {};

    class GameDispatcher extends Laya.EventDispatcher {
        constructor() {
            super();
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new GameDispatcher();
            return this._Inst;
        }
    }
    GameDispatcher._Inst = null;

    class Logger {
        constructor() {
        }
        static log(obj, str) {
            if (Logger.isShowLog) {
                Logger.outPut("[log] [" + Logger.getClassName(obj) + "] " + str);
            }
        }
        static debug(obj, str) {
            Logger.outPut("[log] [" + Logger.getClassName(obj) + "] " + str);
        }
        static net(obj, str) {
            if (Logger.isShowLog && Logger.isNet) {
                Logger.outPut("[net] [" + Logger.getClassName(obj) + "] " + str);
            }
        }
        static error(obj, str) {
        }
        static warning(obj, str) {
            if (Logger.isShowLog) {
                Logger.outPut("[warning] [" + Logger.getClassName(obj) + "] " + str);
            }
        }
        static outPut(value) {
            console.log(value);
        }
        static getClassName(target) {
            if (typeof target === "string") {
                return target;
            }
            if (target && target.constructor.name === "Function") {
                if (target.name) {
                    return target.name;
                }
                else {
                    console.error("Logger.getClassName 错误!!!");
                }
            }
            return target && target.constructor.name;
        }
    }
    Logger.isNet = false;
    Logger.isShowLog = false;

    class Dictionary extends Object {
        constructor() {
            super();
        }
    }

    class Resource {
        constructor() {
            this.gcTime = Number.MAX_VALUE;
            this.count = 0;
            this.url = null;
            this.data = null;
            this.isRecover = true;
        }
        SetUrl(url) {
            this.url = url;
        }
        Decode(data) {
            this.data = data;
            this.resetGcTime();
        }
        GetRes() {
            this.count++;
            return this.data;
        }
        IsCanGc() {
            return this.isRecover && this.count <= 0 && Laya.timer.currTimer >= this.gcTime;
        }
        resetGcTime() {
            this.gcTime = Laya.timer.currTimer + Resource.CACHE_TIME;
        }
        Dispose() {
            this.count--;
            if (this.count == 0) {
                this.resetGcTime();
                Logger.warning(this, "资源达到回收条件：	" + this.url);
            }
            else if (this.count < 0) {
                Logger.error(this, "资源有多余销毁次数：	" + this.url);
            }
        }
        Destroy() {
            Resource.DestroyByName(this.url);
            this.GC();
        }
        static DestroyByName(url) {
            Laya.loader.clearRes(url);
            Logger.warning(this, "资源销毁成功：	" + url);
        }
        GC() {
        }
        Reset() {
            this.gcTime = Number.MAX_VALUE;
            this.url = null;
            this.data = null;
            this.count = 0;
            this.isRecover = true;
        }
    }
    Resource.CACHE_TIME = 5000;
    Resource.CACHE_OBJECT_TIME = 5000;
    Resource.CACHE_UI_TIME = 5000;

    class UISystem {
        constructor() {
            this.m_view = new Dictionary();
            this.m_gc_view = new Dictionary();
            Laya.timer.loop(Resource.CACHE_UI_TIME, UISystem, UISystem.CheckCache);
        }
        static CheckCache() {
            let viewDic = UISystem.Inst.GetGcViewCache();
            let view = null;
            for (let key in viewDic) {
                view = viewDic[key];
                if (view.IsCanGc()) {
                    view.Destroy();
                    delete viewDic[key];
                }
            }
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new UISystem();
            return this._Inst;
        }
        GetGcViewCache() {
            return this.m_gc_view;
        }
        HasView(cls) {
            return this.m_view[cls] != null;
        }
        GetViewByGc(cls) {
            let view = this.m_gc_view[cls];
            if (view != null) {
                delete this.m_gc_view[cls];
                return view;
            }
            return null;
        }
        CreateWindowView(cls, param = null) {
            if (cls == null) {
                return;
            }
            let view = this.m_view[cls];
            if (view != null) {
                return;
            }
            if (view == null) {
                view = this.GetViewByGc(cls);
                if (view == null) {
                    view = new cls();
                    view.cls = cls;
                }
                this.m_view[cls] = view;
            }
            view.Init(param);
            view.LoadAtlas();
            return view;
        }
        RemoveWindowClass(cls) {
            this.RemoveWindowView(this.m_view[cls]);
        }
        RemoveWindowView(view) {
            if (view == null || view.parent == null) {
                return;
            }
            if (view.cls in this.m_view) {
                delete this.m_view[view.cls];
                this.m_gc_view[view.cls] = view;
            }
            else {
                return;
            }
            view.removeFromParent();
            view.RemoveListener();
            view.Reset();
        }
        GetWindowView(cls) {
            return this.m_view[cls];
        }
        GetWindowByName(panelName) {
            let window;
            for (const key in this.m_view) {
                window = this.m_view[key];
                if (window && window.panelName == panelName) {
                    return window;
                }
            }
            return null;
        }
        GetOpenViews() {
            let views = [];
            let view;
            for (let key in this.m_view) {
                view = this.m_view[key];
                if (view) {
                    views.push(view);
                }
            }
            return views;
        }
        removeAllFromMain(exceptLayers = null) {
            let views = this.GetOpenViews();
            let view;
            for (let i = views.length - 1; i >= 0; i--) {
                view = views[i];
                if (view instanceof UIBaseWindow) {
                    if (view.GetViewStruct().layerType == LayerType.TYPE_SCENE
                        || view.GetViewStruct().layerType == LayerType.TYPE_MAIN_DOWN
                        || view.GetViewStruct().layerType == LayerType.TYPE_MAIN) {
                        continue;
                    }
                    if (exceptLayers == null || exceptLayers.indexOf(view.GetViewStruct().layerType) == -1) {
                        this.RemoveWindowView(view);
                    }
                }
            }
        }
        hasAnyViewOnMain(excepts = null) {
            let views = this.GetOpenViews();
            let view;
            for (let i = views.length - 1; i >= 0; i--) {
                view = views[i];
                if (view instanceof UIBaseWindow) {
                    if (view.GetViewStruct().layerType == LayerType.TYPE_SCENE
                        || view.GetViewStruct().layerType == LayerType.TYPE_MAIN_DOWN
                        || view.GetViewStruct().layerType == LayerType.TYPE_MAIN
                        || view.GetViewStruct().layerType == LayerType.TYPE_AD) {
                        continue;
                    }
                    if (excepts && excepts.indexOf(view) != -1) {
                        continue;
                    }
                    if (excepts && excepts.indexOf(view.cls) != -1) {
                        continue;
                    }
                    return true;
                }
            }
            return false;
        }
        GetPanelByName(panelName) {
            let views = this.GetOpenViews();
            let view;
            for (let i = views.length - 1; i >= 0; i--) {
                view = views[i];
                if (view instanceof UIBaseWindow && view.panelName == panelName) {
                    return view;
                }
            }
            return null;
        }
        getTopWindow(excetps = null) {
            let key;
            let prev = -1;
            let view = null;
            for (key in this.m_view) {
                let v = this.m_view[key];
                if (v instanceof UIBaseWindow && v.parent) {
                    if (v.GetViewStruct().layerType == LayerType.TYPE_AD)
                        continue;
                    if (v.panelName == "UITipView" || v.panelName == "UIGuideView" || v.panelName == "UILoadingLeafView") {
                        continue;
                    }
                    if (excetps && excetps.indexOf(v.cls) != -1) {
                        continue;
                    }
                    let index = fgui.GRoot.inst.getChildIndex(v.parent);
                    if (index > prev) {
                        view = v;
                        prev = index;
                    }
                }
            }
            return view;
        }
        hasAnyShowBanner() {
            let key;
            for (key in this.m_view) {
                let view = this.m_view[key];
                if (view && view.isShowBanner) {
                    return true;
                }
            }
            return false;
        }
        static showLoadingLeaf() {
            this.hideLoadingLeaf();
            if (UISystem.leafView == null) {
                let url = fgui.UIPackage.getItemURL("common", "UILoadingLeafView");
                if (!url) {
                    return;
                }
                UISystem.leafView = fgui.UIPackage.createObjectFromURL(url);
            }
            UISystem.leafView.InitData(null);
            fgui.GRoot.inst.addChild(UISystem.leafView);
        }
        static hideLoadingLeaf() {
            if (UISystem.leafView && UISystem.leafView.parent) {
                UISystem.leafView.Reset();
                UISystem.leafView.removeFromParent();
            }
        }
    }

    class GameLayer {
        static start() {
            Laya.stage.addChild(fgui.GRoot.inst.displayObject);
            GameLayer.fightLayer = new fgui.GComponent();
            GameLayer.mainDownLayer = new fgui.GComponent();
            GameLayer.mainLayer = new fgui.GComponent();
            GameLayer.windowLayer = new fgui.GComponent();
            GameLayer.alertLayer = new fgui.GComponent();
            GameLayer.topLayer = new fgui.GComponent();
            GameLayer.flayLayer = new fgui.GComponent();
            GameLayer.diversionLayer = new fgui.GComponent();
            GameLayer.adLayer = new fgui.GComponent();
            fgui.GRoot.inst.addChild(GameLayer.fightLayer);
            fgui.GRoot.inst.addChild(GameLayer.mainDownLayer);
            fgui.GRoot.inst.addChild(GameLayer.mainLayer);
            fgui.GRoot.inst.addChild(GameLayer.windowLayer);
            fgui.GRoot.inst.addChild(GameLayer.alertLayer);
            fgui.GRoot.inst.addChild(GameLayer.topLayer);
            fgui.GRoot.inst.addChild(GameLayer.flayLayer);
            fgui.GRoot.inst.addChild(GameLayer.diversionLayer);
            fgui.GRoot.inst.addChild(GameLayer.adLayer);
            fgui.GRoot.inst.name = "GRoot";
            GameLayer.fightLayer.name = "fightLayer";
            GameLayer.mainDownLayer.name = "mainDownLayer";
            GameLayer.mainLayer.name = "mainLayer";
            GameLayer.windowLayer.name = "windowLayer";
            GameLayer.alertLayer.name = "alertLayer";
            GameLayer.topLayer.name = "topLayer";
            GameLayer.flayLayer.name = "flayLayer";
            GameLayer.diversionLayer.name = "diversionLayer";
            GameLayer.adLayer.name = "adLayer";
            fgui.UIConfig.defaultFont = GlobalConfig.Font_Normal;
            fgui.UIConfig.packageFileExtension = "lm";
            Laya.stage.on(Laya.Event.RESIZE, GameLayer, GameLayer.onResize);
            this.onResize();
            if (GlobalConfig.stat)
                fgui.GRoot.inst.on(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
        }
        static onMouseDown(event) {
            this.checkNode(fgui.GRoot.inst, event.target, "");
        }
        static checkNode(parent, target, nodeName) {
            let length = parent.numChildren;
            for (let i = 0; i < length; i++) {
                const element = parent.getChildAt(i);
                if (element.displayObject == target) {
                    nodeName += element.name;
                    console.log(nodeName);
                    return;
                }
                if (element instanceof fgui.GComponent || element instanceof fgui.GMovieClip) {
                    GameLayer.checkNode(element, target, nodeName + element.name + "|");
                }
            }
        }
        static onResize() {
            fgui.GRoot.inst.setSize(Laya.stage.width, Laya.stage.height);
            GlobalConfig.viewWidth = fgui.GRoot.inst.viewWidth;
            GlobalConfig.viewHeight = fgui.GRoot.inst.viewHeight;
            GameLayer.onUpdate(GameLayer.mainLayer);
            GameLayer.onUpdate(GameLayer.windowLayer);
            GameLayer.onUpdate(GameLayer.alertLayer);
            GameLayer.onUpdate(GameLayer.flayLayer);
            GameLayer.onUpdate(GameLayer.topLayer);
            GameLayer.onUpdate(GameLayer.diversionLayer);
            GameLayer.onUpdate(GameLayer.adLayer);
        }
        static onUpdate(parent) {
            let length = parent.numChildren;
            var view = null;
            for (var i = 0; i < length; i++) {
                view = parent.getChildAt(i);
                if (view != null && view.isLoaded) {
                    view.onResize();
                }
            }
            if (UISystem.leafView && UISystem.leafView.parent) {
                UISystem.leafView.setSize(Laya.stage.width, Laya.stage.height);
            }
        }
    }

    class LayerType {
        static SetWindowLayout(view) {
            let vs = view.GetViewStruct();
            switch (vs.layout) {
                case LayerType.LAYOUT_LEFT_TOP:
                    {
                        view.setSize(fgui.GRoot.inst.width, fgui.GRoot.inst.height);
                        view.x = 0;
                        view.y = 0;
                        break;
                    }
                case LayerType.LAYOUT_CENTER_TOP:
                    {
                        view.x = (GlobalConfig.viewWidth - view.viewWidth) * 0.5;
                        view.y = 0;
                        break;
                    }
                case LayerType.LAYOUT_RIGHT_TOP:
                    {
                        view.x = GlobalConfig.viewWidth - view.viewWidth;
                        view.y = 0;
                        break;
                    }
                case LayerType.LAYOUT_LEFT_CENTER:
                    {
                        view.x = 0;
                        view.y = (GlobalConfig.viewHeight - view.viewHeight) * 0.5;
                        break;
                    }
                case LayerType.LAYOUT_CENTER:
                    {
                        view.x = (GlobalConfig.viewWidth - view.viewWidth) * 0.5;
                        view.y = (GlobalConfig.viewHeight - view.viewHeight) * 0.5;
                        break;
                    }
                case LayerType.LAYOUT_RIGHT_CENTER:
                    {
                        view.x = GlobalConfig.viewWidth - view.viewWidth;
                        view.y = (GlobalConfig.viewHeight - view.viewHeight) * 0.5;
                        break;
                    }
                case LayerType.LAYOUT_LEFT_BOTTOM:
                    {
                        view.x = 0;
                        view.y = GlobalConfig.viewHeight - view.viewHeight;
                        break;
                    }
                case LayerType.LAYOUT_CENTER_BOTTOM:
                    {
                        view.x = (GlobalConfig.viewWidth - view.viewWidth) * 0.5;
                        view.y = GlobalConfig.viewHeight - view.viewHeight;
                        break;
                    }
                case LayerType.LAYOUT_RIGHT_BOTTOM:
                    {
                        view.x = GlobalConfig.viewWidth - view.viewWidth;
                        view.y = GlobalConfig.viewHeight - view.viewHeight;
                        break;
                    }
            }
        }
        static SetWindowLayer(view) {
            let i = 0;
            let len = 0;
            let panel;
            let isExclusion = view.GetViewStruct().isExclusion;
            switch (view.GetViewStruct().layerType) {
                case LayerType.TYPE_SCENE:
                    {
                        if (isExclusion) {
                            len = GameLayer.fightLayer.numChildren;
                            for (i = len - 1; i >= 0; i--) {
                                panel = GameLayer.fightLayer.getChildAt(i);
                                if (panel && panel.GetViewStruct().isExclusion)
                                    UISystem.Inst.RemoveWindowView(panel);
                            }
                        }
                        GameLayer.fightLayer.addChild(view);
                        break;
                    }
                case LayerType.TYPE_MAIN_DOWN:
                    {
                        if (isExclusion) {
                            len = GameLayer.mainDownLayer.numChildren;
                            for (i = len - 1; i >= 0; i--) {
                                panel = GameLayer.mainDownLayer.getChildAt(i);
                                if (panel.GetViewStruct().isExclusion)
                                    UISystem.Inst.RemoveWindowView(panel);
                            }
                        }
                        GameLayer.mainDownLayer.addChild(view);
                        break;
                    }
                case LayerType.TYPE_MAIN:
                    {
                        if (isExclusion) {
                            len = GameLayer.mainLayer.numChildren;
                            for (i = len - 1; i >= 0; i--) {
                                panel = GameLayer.mainLayer.getChildAt(i);
                                if (panel.GetViewStruct().isExclusion)
                                    UISystem.Inst.RemoveWindowView(panel);
                            }
                        }
                        GameLayer.mainLayer.addChild(view);
                        break;
                    }
                case LayerType.TYPE_WINDOW:
                    {
                        if (isExclusion) {
                            len = GameLayer.windowLayer.numChildren;
                            for (i = len - 1; i >= 0; i--) {
                                panel = GameLayer.windowLayer.getChildAt(i);
                                if (panel.GetViewStruct().isExclusion)
                                    UISystem.Inst.RemoveWindowView(panel);
                            }
                        }
                        GameLayer.windowLayer.addChild(view);
                        break;
                    }
                case LayerType.TYPE_ALERT:
                    {
                        if (isExclusion) {
                            len = GameLayer.alertLayer.numChildren;
                            for (i = len - 1; i >= 0; i--) {
                                panel = GameLayer.alertLayer.getChildAt(i);
                                if (panel.GetViewStruct().isExclusion)
                                    UISystem.Inst.RemoveWindowView(panel);
                            }
                        }
                        GameLayer.alertLayer.addChild(view);
                        break;
                    }
                case LayerType.TYPE_TOP:
                    if (isExclusion) {
                        len = GameLayer.topLayer.numChildren;
                        for (i = len - 1; i >= 0; i--) {
                            panel = GameLayer.topLayer.getChildAt(i);
                            if (panel instanceof UIBaseWindow && panel.GetViewStruct().isExclusion) {
                                UISystem.Inst.RemoveWindowView(panel);
                            }
                        }
                    }
                    GameLayer.topLayer.addChild(view);
                    break;
                case LayerType.TYPE_DIVERSION:
                    if (isExclusion) {
                        len = GameLayer.diversionLayer.numChildren;
                        for (i = len - 1; i >= 0; i--) {
                            panel = GameLayer.diversionLayer.getChildAt(i);
                            if (panel.GetViewStruct().isExclusion)
                                UISystem.Inst.RemoveWindowView(panel);
                        }
                    }
                    GameLayer.diversionLayer.addChild(view);
                    break;
                case LayerType.TYPE_AD:
                    if (isExclusion) {
                        len = GameLayer.adLayer.numChildren;
                        for (i = len - 1; i >= 0; i--) {
                            panel = GameLayer.adLayer.getChildAt(i);
                            if (panel == view)
                                continue;
                            if (panel.GetViewStruct().isExclusion)
                                UISystem.Inst.RemoveWindowView(panel);
                        }
                    }
                    GameLayer.adLayer.addChild(view);
                    break;
            }
        }
    }
    LayerType.TYPE_SCENE = 1;
    LayerType.TYPE_MAIN_DOWN = 2;
    LayerType.TYPE_MAIN = 3;
    LayerType.TYPE_WINDOW = 4;
    LayerType.TYPE_ALERT = 5;
    LayerType.TYPE_TOP = 6;
    LayerType.TYPE_DIVERSION = 7;
    LayerType.TYPE_AD = 8;
    LayerType.LAYOUT_LEFT_TOP = 1;
    LayerType.LAYOUT_CENTER_TOP = 2;
    LayerType.LAYOUT_RIGHT_TOP = 3;
    LayerType.LAYOUT_LEFT_CENTER = 4;
    LayerType.LAYOUT_CENTER = 5;
    LayerType.LAYOUT_RIGHT_CENTER = 6;
    LayerType.LAYOUT_LEFT_BOTTOM = 7;
    LayerType.LAYOUT_CENTER_BOTTOM = 8;
    LayerType.LAYOUT_RIGHT_BOTTOM = 9;

    class UIGameEvent {
    }
    UIGameEvent.ON_ACTIVE = "ON_ACTIVE";
    UIGameEvent.SHOW_AD = "UIGameEvent.SHOW_AD";
    UIGameEvent.CACHE_NATIVE_AD = "CACHE_NATIVE_AD";
    UIGameEvent.CHANGE_GROUP_INFO = "CHANGE_GROUP_INFO";
    UIGameEvent.ADD_GROUP_SUCCESS = "ADD_GROUP_SUCCESS";
    UIGameEvent.GET_Subscribe_INFO = "GET_Subscribe_INFO";
    UIGameEvent.TT_START_LUZHI = "TT_START_LUZHI";
    UIGameEvent.TT_STOP_LUZHI = "TT_STOP_LUZHI";
    UIGameEvent.TT_UPDATE_LUZHI = "TT_UPDATE_LUZHI";
    UIGameEvent.JIANQIEBAN = "JIANQIEBAN";
    UIGameEvent.LOAD_SCENE_COMPLETE = "LOAD_SCENE_COMPLETE";
    UIGameEvent.ON_SHOW = "ON_SHOW";
    UIGameEvent.ON_HIDE = "ON_HIDE";
    UIGameEvent.LOGIN_FAIL = "LOGIN_FAIL";
    UIGameEvent.UI_SHOW_IMGBG = "UI_SHOW_IMGBG";
    UIGameEvent.UI_HIDE_IMGBG = "UI_HIDE_IMGBG";
    UIGameEvent.CLICK_BUILD = "CLICK_BUILD";
    UIGameEvent.CLICK_CAT = "CLICK_CAT";
    UIGameEvent.HIDE_BUILD_PROBAR = "HIDE_BUILD_PROBAR";
    UIGameEvent.CAMERA_MOVE_COMPLETE = "CAMERA_MOVE_COMPLETE";
    UIGameEvent.CAMERA_TWEEN_START = "CAMERA_TWEEN_START";
    UIGameEvent.UPDATE_CURRENCY_DATA = "UPDATE_CURRENCY_DATA";
    UIGameEvent.UPDATE_GOLD_RATE = "UPDATE_GOLD_RATE";
    UIGameEvent.MATOU_JIAN_CD_TIME = "MATOU_JIAN_CD_TIME";
    UIGameEvent.UPDATE_FPS = "UPDATE_FPS";
    UIGameEvent.ADD_SKIN = "ADD_SKIN";
    UIGameEvent.REMOVE_SKIN = "REMOVE_SKIN";
    UIGameEvent.GIVE_SKINS = "GIVE_SKINS";
    UIGameEvent.GET_BACK_SKINS = "GET_BACK_SKINS";
    UIGameEvent.ADD_ITEM = "ADD_ITEM";
    UIGameEvent.REMOVE_ITEM = "REMOVE_ITEM";
    UIGameEvent.ADD_BUILD_PARTS = "ADD_BUILD_PARTS";
    UIGameEvent.UNLOCK_POSE = "UNLOCK_POSE";
    UIGameEvent.CAT_UNLOCK = "CAT_UNLOCK";
    UIGameEvent.CAT_CHANGE_NAME = "CAT_CHANGE_NAME";
    UIGameEvent.CAT_UPDATE_QINMI = "CAT_UPDATE_QINMI";
    UIGameEvent.CAT_WEAR_CLOTH = "CAT_WEAR_CLOTH";
    UIGameEvent.CAT_TAKEOFF_CLOTH = "CAT_TAKEOFF_CLOTH";
    UIGameEvent.CAT_STATE_STROKE_CHANGE = "CAT_STATE_STROKE_CHANGE";
    UIGameEvent.CAT_CHANGE_POSE = "CAT_CHANGE_POSE";
    UIGameEvent.CAT_STROKED_COOLTIME_UPDATE = "CAT_STROKED_COOLTIME_UPDATE";
    UIGameEvent.CAT_START_ONLAND = "CAT_START_ONLAND";
    UIGameEvent.CAT_COMPLETE_ONLAND = "CAT_COMPLETE_ONLAND";
    UIGameEvent.CAT_SKIP_ONLAND = "CAT_SKIP_ONLAND";
    UIGameEvent.AWARD_CAT_GIFT = "AWARD_CAT_GIFT";
    UIGameEvent.ADD_GUIDE_CAT = "ADD_GUIDE_CAT";
    UIGameEvent.ADD_FISH = "ADD_FISH";
    UIGameEvent.REMOVE_FISH = "REMOVE_FISH";
    UIGameEvent.UPDATE_FISH_STATE = "UPDATE_FISH_STATE";
    UIGameEvent.START_MOVE_HOUSE = "START_MOVE_HOUSE";
    UIGameEvent.END_MOVE_HOUSE = "END_MOVE_HOUSE";
    UIGameEvent.BUILD_UPDATE_SERVER = "BUILD_UPDATE_SERVER";
    UIGameEvent.LAND_PROMPT_ICON = "LAND_PROMPT_ICON";
    UIGameEvent.BUILD_CANUNLOCK_ICON = "BUILD_CANUNLOCK_ICON";
    UIGameEvent.BUILD_ADD_OR_REMOVE_ICON = "BUILD_ADD_OR_REMOVE_ICON";
    UIGameEvent.CAT_ADD_OR_REMOVE_ICON = "CAT_ADD_OR_REMOVE_ICON";
    UIGameEvent.SCENE_RESIZE = "SCENE_RESIZE";
    UIGameEvent.ISHIDE_ICON = "ISHIDE_ICON";
    UIGameEvent.REMOVE_CAT = "REMOVE_CAT";
    UIGameEvent.BUILD_LEVEL_UP = "BUILD_LEVEL_UP";
    UIGameEvent.BUILD_STAGE_UP = "BUILD_STAGE_UP";
    UIGameEvent.BUILD_ADD = "BUILD_ADD";
    UIGameEvent.BUILD_REMOE = "BUILD_REMOE";
    UIGameEvent.BUILD_UNLOCK = "BUILD_UNLOCK";
    UIGameEvent.BUILD_LEVEL_LESS = "BUILD_LEVEL_LESS";
    UIGameEvent.SHOW_NATIVE = "SHOW_NATIVE";
    UIGameEvent.HIDE_NATIVE = "HIDE_NATIVE";
    UIGameEvent.CLICK_NATIVE_AD = "CLICK_NATIVE_AD";
    UIGameEvent.CLICK_SUIT_ITEM = "CLICK_SUIT_ITEM";
    UIGameEvent.CAT_STROKING_DOWN_OR_UP = "CAT_STROKING_DOWN_OR_UP";
    UIGameEvent.CAT_STROKING_COMPLETE = "CAT_STROKING_COMPLETE";
    UIGameEvent.FLY_GOLD = "FLY_GOLD";
    UIGameEvent.EXCHANGE_ADD = "EXCHANGE_ADD";
    UIGameEvent.EXCHANGE_REMOVE = "EXCHANGE_REMOVE";
    UIGameEvent.EXCHANGE_SUCCESS = "EXCHANGE_SUCCESS";
    UIGameEvent.UPDATE_TASK = "UPDATE_TASK";
    UIGameEvent.DAYTASK_COMP_PROP = "DAYTASK_COMP_PROP";
    UIGameEvent.MAINTASK_COMP_PROP = "MAINTASK_COMP_PROP";
    UIGameEvent.SHOW_TASKBTN_RED = "SHOW_TASKBTN_RED";
    UIGameEvent.SHOW_SIGINBTN_RED = "SHOW_SIGINBTN_RED";
    UIGameEvent.SHOW_BAGBTN_RED = "SHOW_BAGBTN_RED";
    UIGameEvent.SHOW_CATBTN_RED = "SHOW_CATBTN_RED";
    UIGameEvent.GET_ADLIXIAN = "GET_ADLIXIAN";
    UIGameEvent.SHOW_LIBAO_RED = "SHOW_LIBAO_RED";
    UIGameEvent.ON_BUYFREELIBAO = "ON_BUYFREELIBAO";
    UIGameEvent.SHOPBUYUPPERLIMIT = "SHOPBUYUPPERLIMIT";
    UIGameEvent.GUIDE_SHOW = "GUIDE_SHOW";
    UIGameEvent.GUIDE_COMPELTE = "GUIDE_COMPELTE";
    UIGameEvent.GUIDE_COMPELTE_ALL = "GUIDE_COMPELTE_ALL";
    UIGameEvent.MAIN_GUIDE_COMPELTE_ALL = "MAIN_GUIDE_COMPELTE_ALL";
    UIGameEvent.MAIN_TO_BLACK_TO_WHITE = "MAIN_TO_BLACK_TO_WHITE";
    UIGameEvent.BLACK_TO_WHITE = "BLACK_TO_WHITE";
    UIGameEvent.WHITE_TO_BLACK = "WHITE_TO_BLACK";
    UIGameEvent.WHITE_TO_MAIN = "WHITE_TO_MAIN";
    UIGameEvent.SHOW_MAIN_BTNS = "SHOW_MAIN_BTNS";
    UIGameEvent.CATADDSKIN = "CATADDSKIN";
    UIGameEvent.CAT_MATOUDOWN = "CAT_MATOUDOWN";
    UIGameEvent.SUMMER_ACTIVE = "SUMMER_ACTIVE";
    UIGameEvent.ONUNLOCKSUMMER = "ONUNLOCKSUMMER";
    UIGameEvent.GAME_HT_INIT = "GAME_HT_INIT";
    UIGameEvent.GAME_HT = "GAME_HT";
    UIGameEvent.ONGETDOUBLEAWARD = "ONGETDOUBLEAWARD";
    UIGameEvent.SKIP_DAY = "SKIP_DAY";
    UIGameEvent.SHOW_BANNER = "SHOW_BANNER";
    UIGameEvent.HIDE_BANNER = "HIDE_BANNER";
    UIGameEvent.ONMATOU_UPLV = "ONMATOU_UPLV";
    UIGameEvent.ONMAOWU_UPLV = "ONMAOWU_UPLV";
    UIGameEvent.ONMAOWU_UPSTAGE = "ONMAOWU_UPSTAGE";
    UIGameEvent.ONUPDATEYUYUEBTNSHOW = "ONUPDATEYUYUEBTNSHOW";
    UIGameEvent.SHOW_AD_COMPLETE = "SHOW_AD_COMPLETE";
    UIGameEvent.RESIZE_BANNER = "RESIZE_BANNER";
    UIGameEvent.UPDATE_GROTH_COUNT = "UPDATE_GROTH_COUNT";
    UIGameEvent.CLOSE_AD = "CLOSE_AD";
    UIGameEvent.ONADDTOFAVORITS = "ONADDTOFAVORITS";
    UIGameEvent.ON_LIHE_GETAWARD = "ON_LIHE_GETAWARD";
    UIGameEvent.ONCLOSEBOXVIEW = "ONCLOSEBOXVIEW";

    class MusicType {
        constructor() {
        }
    }
    MusicType.BGM = "BGM";
    MusicType.fx_congratulation_invite = "fx_congratulation_invite";
    MusicType.catGradeUp = "catGradeUp";
    MusicType.catLevelup = "catLevelup";
    MusicType.catMeal = "catMeal";
    MusicType.Click_calcel = "Click_calcel";
    MusicType.Click_OK = "Click_OK";
    MusicType.event_happy = "event_happy";
    MusicType.facilityUpgrade = "facilityUpgrade";
    MusicType.fx_congratulation_facility = "fx_congratulation_facility";
    MusicType.fx_UI_btn_Close = "fx_UI_btn_Close";
    MusicType.fx_UI_btn_Open = "fx_UI_btn_Open";
    MusicType.getGold = "getGold";
    MusicType.getHairBall = "getHairBall";
    MusicType.giveChuru = "giveChuru";
    MusicType.itemDrop = "itemDrop";
    MusicType.majorFacilityUpgrade = "majorFacilityUpgrade";
    MusicType.cat_sit_yawn1 = "cat_sit_yawn1";
    MusicType.cat_sit_yawn2 = "cat_sit_yawn2";
    MusicType.cat_sit_yawn3 = "cat_sit_yawn3";
    MusicType.cat_interaction_ready11 = "cat_interaction_ready11";
    MusicType.cat_interaction_ready12 = "cat_interaction_ready12";
    MusicType.cat_interaction_ready13 = "cat_interaction_ready13";
    MusicType.cat_interaction_ready21 = "cat_interaction_ready21";
    MusicType.cat_interaction_ready22 = "cat_interaction_ready22";
    MusicType.cat_interaction_ready23 = "cat_interaction_ready23";
    MusicType.cat_interaction_ready31 = "cat_interaction_ready31";
    MusicType.cat_interaction_ready32 = "cat_interaction_ready32";
    MusicType.cat_interaction_ready33 = "cat_interaction_ready33";
    MusicType.gachaEmbient_ongoing = "gachaEmbient_ongoing";
    MusicType.gachaEmbient_Intro = "gachaEmbient_Intro";

    class MathUtil {
        static bignumerToString(num, isInt = true) {
            let str = "";
            if (num instanceof BigNumber) {
                if (isInt) {
                    str = num ? num.integerValue(BigNumber.ROUND_CEIL).toFormat({ "prefix": "" }) : "";
                }
                else {
                    str = num ? num.toFormat({ "prefix": "" }) : "";
                }
            }
            else {
                str = num + "";
            }
            return str;
        }
        static goldFormat(val, fixed = 1) {
            if (typeof val === "string") {
                val = new BigNumber(val);
            }
            if (val instanceof BigNumber) {
                return this.BigNumberFormat(val, fixed);
            }
            else if (typeof val === "number") {
                let yy = Math.floor(val / 1000000000000000);
                if (yy > 0) {
                    return yy + "千兆";
                }
                let wy = Math.floor(val / 1000000000000);
                if (wy > 0) {
                    return wy + "兆";
                }
                let y = Math.floor(val / 100000000);
                if (y > 0) {
                    return y + "亿";
                }
                let w = Math.floor(val / 10000);
                if (w > 0) {
                    return w + "万";
                }
                let str = val.toFixed(fixed);
                if (str.split(".")[1] == "0") {
                    str = str.split(".")[0];
                }
                return str;
            }
        }
        static BigNumberFormat(bigNum, fixed = 1) {
            let fmt = {
                "decimalSeparator": '.',
                "groupSeparator": ',',
                "groupSize": 4,
                "secondaryGroupSize": 4
            };
            let str = bigNum.toFormat(fmt).split(".")[0];
            let wei = str.match(/,/g);
            let arr = str.split(",");
            let hou = fixed <= 0 ? "" : arr[1] ? "." + arr[1].substr(0, fixed) : "";
            if (hou == ".0") {
                hou = "";
            }
            if (wei == null || wei.length == 0) {
                return str;
            }
            else if (wei.length == 1) {
                str = `${arr[0]}${hou}万`;
            }
            else if (wei.length == 2) {
                str = `${arr[0]}${hou}亿`;
            }
            else if (wei.length == 3) {
                str = `${arr[0]}${hou}兆`;
            }
            else if (wei.length == 4) {
                str = `${arr[0]}${hou}京`;
            }
            else if (wei.length == 5) {
                str = `${arr[0]}${hou}万京`;
            }
            return str;
        }
        static calculateLinePoint(bpoint, direction, distance, out = null) {
            Laya.Vector3.normalize(direction, direction);
            if (out == null) {
                out = new Laya.Vector3();
            }
            out.setValue(bpoint.x + direction.x * distance, bpoint.y + direction.y * distance, bpoint.z + direction.z * distance);
            return out;
        }
        static GetForward(transform) {
            let out = new Laya.Vector3();
            transform.getForward(out);
            Laya.Vector3.scale(out, -1, out);
            return out;
        }
        static localToGlobal(sp, parent) {
            if (sp == null || sp.parent == parent)
                return;
            let position = sp.transform.position.clone();
            let rotationEuler = sp.transform.rotationEuler.clone();
            parent.addChild(sp);
            sp.transform.position = position;
            sp.transform.rotationEuler = rotationEuler;
        }
        static globalToLocal(sp, parent) {
            let position = sp.transform.localPosition.clone();
            let rotationEuler = sp.transform.localRotationEuler.clone();
            parent.addChild(sp);
            sp.transform.localPosition = MathUtil.InverseTransformPoint(parent.transform, position);
            sp.transform.localRotationEuler = rotationEuler;
        }
        static translate(ts, offset) {
            let pos = new Laya.Vector3();
            Laya.Matrix4x4.createFromQuaternion(ts.rotation, Laya.Transform3D['_tempMatrix0']);
            Laya.Vector3.transformCoordinate(offset, Laya.Transform3D['_tempMatrix0'], Laya.Transform3D['_tempVector30']);
            Laya.Vector3.add(ts.position, Laya.Transform3D['_tempVector30'], pos);
            return pos;
        }
        static InverseTransformPoint(parent, position) {
            let tempMatrix = new Laya.Matrix4x4();
            parent.worldMatrix.invert(tempMatrix);
            let v3 = new Laya.Vector3();
            Laya.Vector3.transformCoordinate(position, tempMatrix, v3);
            return v3;
        }
        static GetPointFrom3DWorld(camera, pos3d, ui = null) {
            let uiPos = new Laya.Vector4();
            camera.worldToViewportPoint(pos3d, uiPos);
            uiPos.x = Math.ceil(uiPos.x);
            uiPos.y = Math.ceil(uiPos.y);
            if (ui != null) {
                return ui.globalToLocal(uiPos.x, uiPos.y);
            }
            else {
                return new Laya.Point(uiPos.x, uiPos.y);
            }
        }
        static GetScalePos(sPos, ePos, scale) {
            let dir = new Laya.Vector3();
            Laya.Vector3.subtract(ePos, sPos, dir);
            Laya.Vector3.scale(dir, scale, dir);
            return new Laya.Vector3(sPos.x + dir.x, sPos.y + dir.y, sPos.z + dir.z);
        }
        static distance(px1, py1, px2, py2) {
            let dx = px1 - px2;
            let dy = py1 - py2;
            return Math.sqrt(dx * dx + dy * dy);
        }
        static distanceNS(sx, sy, tx, ty) {
            let dx = tx - sx;
            let dy = ty - sy;
            return dx * dx + dy * dy;
        }
        static random(jilv, maxJilv = MathUtil.max) {
            if (jilv == 0)
                return false;
            let base = Math.random() * maxJilv;
            return base <= jilv;
        }
        static makeRandom(min, max) {
            let num = min + Math.random() * (max - min);
            return num < max ? Math.floor(num) : max;
        }
        static angleToRadian(angle) {
            let l = angle * Math.PI / 180;
            return l;
        }
        static radianToAngle(l) {
            return l * 180 / Math.PI;
        }
        static GetMeshSprite3D(sp) {
            if (sp == null)
                return null;
            if (sp instanceof Laya.MeshSprite3D) {
                return sp;
            }
            let length = sp.numChildren;
            for (let i = 0; i < length; i++) {
                const element = sp.getChildAt(i);
                let node = MathUtil.GetMeshSprite3D(element);
                if (node != null)
                    return node;
            }
            return null;
        }
        static GetMaterial(sp) {
            if (sp == null)
                return null;
            if (sp instanceof Laya.MeshSprite3D) {
                return sp.meshRenderer.sharedMaterial;
            }
            let length = sp.numChildren;
            for (let i = 0; i < length; i++) {
                const element = sp.getChildAt(i);
                let mat = MathUtil.GetMaterial(element);
                if (mat != null)
                    return mat;
            }
            return null;
        }
        static FindSprite3DByName(parent, name) {
            if (parent == null)
                return null;
            if (parent.name == name) {
                return parent;
            }
            let length = parent.numChildren;
            for (let i = 0; i < length; i++) {
                const element = parent.getChildAt(i);
                let node = MathUtil.FindSprite3DByName(element, name);
                if (node != null)
                    return node;
            }
            return null;
        }
        static FindPhysicsCollider(parent) {
            if (parent == null)
                return null;
            let phy = parent.getComponent(Laya.PhysicsCollider);
            if (phy != null) {
                return phy;
            }
            let length = parent.numChildren;
            for (let i = 0; i < length; i++) {
                const element = parent.getChildAt(i);
                let node = MathUtil.FindPhysicsCollider(element);
                if (node != null)
                    return node;
            }
            return null;
        }
        static MultiPointBezier(points, t, resultPos) {
            resultPos.setTo(0, 0);
            var len = points.length;
            var erxiangshi = function (start, end) {
                var cs = 1, bcs = 1;
                while (end > 0) {
                    cs *= start;
                    bcs *= end;
                    start--;
                    end--;
                }
                return (cs / bcs);
            };
            for (var i = 0; i < len; i++) {
                var point = points[i];
                resultPos.x += point.x * Math.pow((1 - t), (len - 1 - i)) * Math.pow(t, i) * (erxiangshi(len - 1, i));
                resultPos.y += point.y * Math.pow((1 - t), (len - 1 - i)) * Math.pow(t, i) * (erxiangshi(len - 1, i));
            }
            return resultPos;
        }
        static angle(a, b) {
            let aa = new Laya.Vector3();
            Laya.Vector3.normalize(a, aa);
            let bb = new Laya.Vector3();
            Laya.Vector3.normalize(b, bb);
            let angle = Math.acos(Laya.Vector3.dot(aa, bb));
            return angle * (180 / Math.PI);
        }
        static changeToM180To180(sourceAngel) {
            let x = MathUtil.clampAngle(sourceAngel.x, -180);
            let y = MathUtil.clampAngle(sourceAngel.y, -180);
            let z = MathUtil.clampAngle(sourceAngel.z, -180);
            return new Laya.Vector3(x, y, z);
        }
        static ClampRotationEulerRange(curRot, targetRot) {
            curRot.x = MathUtil.clampAngle(curRot.x, targetRot.x - 180);
            curRot.x = MathUtil.MinNearAngle(curRot.x, targetRot.x);
            curRot.y = MathUtil.clampAngle(curRot.y, targetRot.y - 180);
            curRot.y = MathUtil.MinNearAngle(curRot.y, targetRot.y);
            curRot.z = MathUtil.clampAngle(curRot.z, targetRot.z - 180);
            curRot.z = MathUtil.MinNearAngle(curRot.z, targetRot.z);
            return curRot;
        }
        static clampAngle(angle, min) {
            if (angle < min) {
                angle += 360;
                angle = MathUtil.clampAngle(angle, min);
            }
            else if (angle > min + 360) {
                angle -= 360;
                angle = MathUtil.clampAngle(angle, min);
            }
            return angle;
        }
        static MinNearAngle(curAngle, angle) {
            if (angle > curAngle) {
                if (angle - curAngle < (curAngle + 360) - angle) {
                    return curAngle;
                }
                else {
                    return curAngle + 360;
                }
            }
            else {
                if (curAngle - angle < angle - (curAngle - 360)) {
                    return curAngle;
                }
                else {
                    return curAngle - 360;
                }
            }
        }
        static GetBezierPoint(points, t, retPos) {
            retPos.setTo(0, 0);
            var getPoint = function (start, end, t) {
                let x = start.x + (end.x - start.x) * t;
                let y = start.y + (end.y - start.y) * t;
                return new Laya.Point(x, y);
            };
            let newPointArr = [];
            for (let i = 0; i < points.length; i++) {
                newPointArr.push(points[i]);
            }
            while (newPointArr.length > 1) {
                let arr = [];
                for (let i = 0; i < newPointArr.length - 1; i++) {
                    arr.push(getPoint(newPointArr[i], newPointArr[i + 1], t));
                }
                newPointArr = arr;
            }
            retPos.x = newPointArr[0].x;
            retPos.y = newPointArr[0].y;
            return retPos;
        }
        static GetRandomItemByWeight(list, count = 1) {
            if (list == null || list.length <= 0) {
                return null;
            }
            let totalWeight = 0;
            for (let i = 0; i < list.length; i++) {
                let item = list[i];
                if (item.weight == null) {
                    item.weight = 1;
                }
                totalWeight += item.weight;
            }
            let list1 = list.slice();
            list1 = list1.sort((a, b) => {
                if (a.weight < b.weight)
                    return -1;
                else if (a.weight > b.weight)
                    return 1;
                else
                    return 0;
            });
            let retArr = [];
            for (let i = 0; i < count; i++) {
                let reItem = MathUtil.GetRandomItem2(list1, totalWeight);
                totalWeight -= reItem.weight;
                retArr.push(reItem);
            }
            return retArr;
        }
        static GetRandomItem(list, totalWeight = 10000) {
            if (list == null || list.length <= 0) {
                return null;
            }
            let list1 = list.slice();
            list1 = list1.sort((a, b) => {
                if (a.weight < b.weight)
                    return -1;
                else if (a.weight > b.weight)
                    return 1;
                else
                    return 0;
            });
            let random = MathUtil.makeRandom(0, totalWeight);
            let length = list1.length;
            let startJv = 0;
            let item = null;
            for (let i = 0; i < length; i++) {
                let roll = list1[i];
                startJv += roll.weight;
                if (random < startJv) {
                    item = roll;
                    list1.splice(i, 1);
                    break;
                }
            }
            return item;
        }
        static GetRandomItem2(list1, totalWeight = 10000) {
            if (list1 == null || list1.length <= 0) {
                return null;
            }
            let random = MathUtil.makeRandom(0, totalWeight);
            let length = list1.length;
            let startJv = 0;
            let item = null;
            for (let i = 0; i < length; i++) {
                let roll = list1[i];
                startJv += roll.weight;
                if (random < startJv) {
                    item = roll;
                    list1.splice(i, 1);
                    break;
                }
            }
            return item;
        }
        static rotateAroundAxis(V1, T, angle) {
            let retVector = new Laya.Vector3();
            let tempVector1 = new Laya.Vector3();
            let tempValue1 = Laya.Vector3.dot(T, V1);
            tempVector1.x = tempValue1 * T.x;
            tempVector1.y = tempValue1 * T.y;
            tempVector1.z = tempValue1 * T.z;
            Laya.Vector3.subtract(V1, tempVector1, retVector);
            retVector.x = Math.cos(angle) * retVector.x;
            retVector.y = Math.cos(angle) * retVector.y;
            retVector.z = Math.cos(angle) * retVector.z;
            let tempVector2 = new Laya.Vector3();
            Laya.Vector3.cross(T, V1, tempVector2);
            tempVector2.x = Math.sin(angle) * tempVector2.x;
            tempVector2.y = Math.sin(angle) * tempVector2.y;
            tempVector2.z = Math.sin(angle) * tempVector2.z;
            Laya.Vector3.add(retVector, tempVector2, retVector);
            Laya.Vector3.add(retVector, tempVector1, retVector);
            return retVector;
        }
        static WorldToScreen2(camera, point) {
            var pointA = this.InverseTransformPoint2(camera.transform, point);
            var distance = pointA.z;
            var out = new Laya.Vector3();
            camera.viewport.project(point, camera.projectionViewMatrix, out);
            var value = new Laya.Vector3(out.x / Laya.stage.clientScaleX, out.y / Laya.stage.clientScaleY, distance);
            return value;
        }
        static ScreenToWorld(camera, point) {
            var halfFOV = (camera.fieldOfView * 0.5) * Math.PI / 180;
            let height = point.z * Math.tan(halfFOV);
            let width = height * camera.aspectRatio;
            let lowerLeft = this.GetLowerLeft(camera.transform, point.z, width, height);
            let v = this.GetScreenScale(width, height);
            var value = new Laya.Vector3();
            var lowerLeftA = this.InverseTransformPoint2(camera.transform, lowerLeft);
            value = new Laya.Vector3(-point.x / v.x, point.y / v.y, 0);
            Laya.Vector3.add(lowerLeftA, value, value);
            value = this.TransformPoint(camera.transform, value);
            return value;
        }
        static GetScreenScale(width, height) {
            var v = new Laya.Vector3();
            v.x = Laya.stage.width / width / 2;
            v.y = Laya.stage.height / height / 2;
            return v;
        }
        static GetLowerLeft(transform, distance, width, height) {
            var lowerLeft = new Laya.Vector3();
            var right = new Laya.Vector3();
            transform.getRight(right);
            Laya.Vector3.normalize(right, right);
            var xx = new Laya.Vector3(right.x * width, right.y * width, right.z * width);
            Laya.Vector3.add(transform.position, xx, lowerLeft);
            var up = new Laya.Vector3();
            transform.getUp(up);
            Laya.Vector3.normalize(up, up);
            var yy = new Laya.Vector3(up.x * height, up.y * height, up.z * height);
            Laya.Vector3.subtract(lowerLeft, yy, lowerLeft);
            var forward = new Laya.Vector3();
            transform.getForward(forward);
            Laya.Vector3.normalize(forward, forward);
            var zz = new Laya.Vector3(forward.x * distance, forward.y * distance, forward.z * distance);
            Laya.Vector3.subtract(lowerLeft, zz, lowerLeft);
            return lowerLeft;
        }
        static InverseTransformPoint2(origin, point) {
            var xx = new Laya.Vector3();
            origin.getRight(xx);
            var yy = new Laya.Vector3();
            origin.getUp(yy);
            var zz = new Laya.Vector3();
            origin.getForward(zz);
            var zz1 = new Laya.Vector3(-zz.x, -zz.y, -zz.z);
            var x = this.ProjectDistance(point, origin.position, xx);
            var y = this.ProjectDistance(point, origin.position, yy);
            var z = this.ProjectDistance(point, origin.position, zz1);
            var value = new Laya.Vector3(x, y, z);
            return value;
        }
        static TransformPoint(origin, point) {
            var value = new Laya.Vector3();
            Laya.Vector3.transformQuat(point, origin.rotation, value);
            Laya.Vector3.add(value, origin.position, value);
            return value;
        }
        static ProjectDistance(A, C, B) {
            var CA = new Laya.Vector3();
            Laya.Vector3.subtract(A, C, CA);
            var angle = this.Angle2(CA, B) * Math.PI / 180;
            var distance = Laya.Vector3.distance(A, C);
            distance *= Math.cos(angle);
            return distance;
        }
        static Angle2(ma, mb) {
            var v1 = (ma.x * mb.x) + (ma.y * mb.y) + (ma.z * mb.z);
            var ma_val = Math.sqrt(ma.x * ma.x + ma.y * ma.y + ma.z * ma.z);
            var mb_val = Math.sqrt(mb.x * mb.x + mb.y * mb.y + mb.z * mb.z);
            var cosM = v1 / (ma_val * mb_val);
            if (cosM < -1)
                cosM = -1;
            if (cosM > 1)
                cosM = 1;
            var angleAMB = Math.acos(cosM) * 180 / Math.PI;
            return angleAMB;
        }
        static GetPosByStr(strArr) {
            return new Laya.Vector3(-parseFloat(strArr[0]), parseFloat(strArr[1]), parseFloat(strArr[2]));
        }
        static GetRotByStr(strArr) {
            return new Laya.Vector3(-parseFloat(strArr[0]), 180 - parseFloat(strArr[1]), parseFloat(strArr[2]));
        }
        static GetColorByStr(strArr) {
            return new Laya.Vector3(parseFloat(strArr[0]), parseFloat(strArr[1]), parseFloat(strArr[2]));
        }
        static replaceAll(str, replace1, replace2) {
            str = str.replace(replace1, replace2);
            if (str.indexOf(replace1) != -1) {
                str = MathUtil.replaceAll(str, replace1, replace2);
            }
            return str;
        }
    }
    MathUtil.max = 10000;

    class TimeModel {
        constructor() {
            this.initLocalTime = 0;
            this.initLayaTimer = 0;
        }
        static get Inst() {
            return this.inst = this.inst || new TimeModel();
        }
        init() {
            this.GetLocalTime();
        }
        GetLocalTime() {
            this.date = new Date();
            this.initLocalTime = this.date.getTime();
            this.initLayaTimer = Laya.timer.currTimer;
            return this.initLocalTime;
        }
        get LocalTime() {
            let time = this.initLocalTime + (Laya.timer.currTimer - this.initLayaTimer);
            return time;
        }
        getDate(time = NaN) {
            if (isNaN(time)) {
                time = this.LocalTime;
            }
            this.date.setTime(time);
            return this.date;
        }
        static judgeIsOneDay(time) {
            let date = new Date();
            let nowDay = date.getDay();
            let nowYear = date.getFullYear();
            let nowMonth = date.getMonth();
            date.setTime(time);
            let prevDay = date.getDay();
            let preYear = date.getFullYear();
            let preMonth = date.getMonth();
            if (nowYear != preYear || nowMonth != preMonth || nowDay != prevDay) {
                return false;
            }
            return true;
        }
    }
    TimeModel.inst = null;

    class StorageUtils {
        constructor() {
            this._dayData = null;
            this._foreverData = null;
            this.isNeedSave = false;
            this._user = null;
            this.LoginTime = 0;
            this.loginTime2 = 0;
            this.IsClearEveryDay = false;
            this.todayIsFirst = false;
            this.bigNum = new BigNumber(0);
            this.goldNum = new BigNumber(0);
            this._signTime = NaN;
            this.lineTimeTemp = -1;
            Laya.timer.frameLoop(1, this, this.CheckSave);
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new StorageUtils();
            return this._Inst;
        }
        enterGame() {
            this.setLoginTime();
        }
        initStorage(handler) {
            let lf = this;
            lf.loginTime2 = StorageUtils.getCurTimer();
            PlatformMgr.Inst.getStorage(StorageUtils.username, Laya.Handler.create(null, function (data) {
                lf._dayData = data["dayData"] || {};
                lf._foreverData = data["foreverData"] || {};
                data["dayData"] = lf._dayData;
                data["foreverData"] = lf._foreverData;
                lf._user = data;
                lf.isNeedSave = false;
                if (handler != null)
                    handler.run();
            }));
        }
        static getCurTimer() {
            return TimeModel.Inst.LocalTime;
        }
        setLoginTime() {
            let logtime = StorageUtils.getCurTimer();
            if (!(StorageUtils.todayFirstTime in this._user)) {
                this._user[StorageUtils.todayFirstTime] = logtime;
                this.todayIsFirst = true;
                this.todayFirstLoginInit();
            }
            else if (!TimeModel.judgeIsOneDay(this._user[StorageUtils.todayFirstTime])) {
                this._user[StorageUtils.todayFirstTime] = logtime;
                this.todayIsFirst = true;
                this.todayFirstLoginInit();
            }
            this._user[StorageUtils.loginTime] = logtime;
            this.LoginTime = logtime;
            this.SaveUser();
        }
        getGameTime() {
            return StorageUtils.getCurTimer() - this.LoginTime;
        }
        get todayFirstTime() {
            return this._user[StorageUtils.loginTime];
        }
        todayFirstLoginInit() {
            if (this._dayData != null) {
                this._dayData = {};
            }
            let lf = this;
            lf.SaveLoginDayCount();
            lf.ClearEveryDayTask();
            lf.ClearEveryCommodityAd();
            lf.AddNipAdCount(5, false);
            if (lf.bottleState == 3) {
                lf.bottleState = 4;
            }
            if (lf.canyuyue == 1) {
                lf.canyuyue = 2;
            }
            if (!lf.IsHasLogin()) {
                lf.isShuaXing = true;
            }
            lf.user[StorageUtils.onlineTime] = 0;
            lf.user["dayData"] = lf._dayData;
            lf.IsClearEveryDay = true;
        }
        SaveLoginDayCount() {
            if (this._foreverData[StorageUtils.loginDayNum]) {
                this._foreverData[StorageUtils.loginDayNum]++;
            }
            else {
                this._foreverData[StorageUtils.loginDayNum] = 1;
            }
            this.SaveUser();
        }
        get loginDayCount() {
            if (StorageUtils.loginDayNum in this._foreverData) {
                this._foreverData[StorageUtils.loginDayNum];
            }
            else {
                this._foreverData[StorageUtils.loginDayNum] = 1;
            }
            return this._foreverData[StorageUtils.loginDayNum];
        }
        ClearEveryDayTask() {
            this.setUserInfoData(null, false, StorageUtils.dayTask, true);
        }
        ClearEveryCommodityAd() {
            this.setUserInfoData(null, false, StorageUtils.commodityAdCount, true);
            this.setUserInfoData(null, false, StorageUtils.commodityAdTime, true);
        }
        AddNipAdCount(value, isAdd = true) {
            let cnt = this.setUserInfoData(value, isAdd, StorageUtils.nipViewAdCont, true, "", true);
            return cnt;
        }
        GetNipAdCount() {
            return this.getUserInfoData(StorageUtils.nipViewAdCont, true, "", 5);
        }
        GetUserInfo() {
            if (!(StorageUtils.userinfo in this.user)) {
                this.user[StorageUtils.userinfo] = { uid: this.LoginTime };
            }
            return this.user[StorageUtils.userinfo];
        }
        GetUid() {
            return this.GetUserInfo()["uid"];
        }
        CheckSave() {
            if (!this.isNeedSave)
                return;
            this.isNeedSave = false;
            PlatformMgr.Inst.setStorage(StorageUtils.username, JSON.stringify(this.user));
        }
        IsHasLogin() {
            return StorageUtils.firstLogin in this.user;
        }
        SetLogin() {
            this.user[StorageUtils.firstLogin] = true;
            this.SaveUser();
        }
        setGold(value) {
            value = this.bigNum.multipliedBy(0).plus(value);
            value = MathUtil.bignumerToString(value);
            this.setUserInfoData(value, false, StorageUtils.gold, false);
        }
        setDiamond(value) {
            value = this.bigNum.multipliedBy(0).plus(value);
            value = MathUtil.bignumerToString(value);
            this.setUserInfoData(value, false, StorageUtils.diamond, false);
        }
        GetGold() {
            let count = this.getUserInfoData(StorageUtils.gold, false, "", 0);
            this.goldNum = this.goldNum.multipliedBy(0).plus(count);
            return this.goldNum;
        }
        GetDiamond() {
            let count = this.getUserInfoData(StorageUtils.diamond, false);
            if (typeof count === "string") {
                count = parseInt(count);
            }
            return count;
        }
        GetSignInfo() {
            if (!(StorageUtils.signInfo in this._foreverData)) {
                this._foreverData[StorageUtils.signInfo] = {};
            }
            return this._foreverData[StorageUtils.signInfo];
        }
        get signCount() {
            let signInfo = this.GetSignInfo();
            if (!(StorageUtils.signDays in signInfo)) {
                signInfo[StorageUtils.signDays] = 0;
            }
            return signInfo[StorageUtils.signDays];
        }
        set signCount(val) {
            let signInfo = this.GetSignInfo();
            if (val > signInfo[StorageUtils.signDays]) {
                this.signTime = StorageUtils.getCurTimer();
            }
            signInfo[StorageUtils.signDays] = val;
            this.SaveUser();
        }
        get canGetDima() {
            return this.getUserInfoData(StorageUtils.canGetDima, true, "", true);
        }
        set canGetDima(bol) {
            if (bol == this.canGetDima)
                return;
            this.setUserInfoData(bol, false, StorageUtils.canGetDima, true, "", true);
        }
        get signTime() {
            if (isNaN(this._signTime)) {
                let signInfo = this.GetSignInfo();
                if (!(StorageUtils.signPrevTime in signInfo)) {
                    signInfo[StorageUtils.signPrevTime] = 0;
                }
                this._signTime = signInfo[StorageUtils.signPrevTime];
            }
            return this._signTime;
        }
        set signTime(val) {
            let signInfo = this.GetSignInfo();
            signInfo[StorageUtils.signPrevTime] = val;
            this._signTime = val;
            this.SaveUser();
        }
        get canSign() {
            if (this.signTime == 0)
                return true;
            return this.CheckYMDOneDay(this.signTime);
        }
        CheckYMDOneDay(time, interval = new Date(0, 0, 1).getTime() - new Date(0, 0, 0).getTime(), checkTime = "day") {
            if (time <= 0)
                return true;
            let curDate = new Date();
            let curTime = curDate.getTime();
            let preDate = new Date(time);
            let offestTime = curTime - time;
            if (offestTime <= 0) {
                return false;
            }
            else if (offestTime < interval) {
                let curDay = 0;
                let preDay = 0;
                if (checkTime == "day") {
                    curDay = curDate.getDay();
                    preDay = preDate.getDay();
                }
                return curDay != preDay;
            }
            else {
                return true;
            }
        }
        GetOnlineTime() {
            return this.user[StorageUtils.onlineTime];
        }
        AddOnlineTime(value) {
            let onlineTime = this.GetOnlineTime();
            onlineTime += value;
            this.user[StorageUtils.onlineTime] = onlineTime;
            this.SetOfflineTime();
            this.SaveUser();
        }
        GetOfflineInterValTime() {
            if (this.lineTimeTemp == -1) {
                let time = this.GetOfflineTime();
                if (!time) {
                    this.lineTimeTemp = 0;
                }
                else {
                    this.lineTimeTemp = this.loginTime2 - time;
                }
            }
            return this.lineTimeTemp;
        }
        ResetOffLine() {
            this.lineTimeTemp = -1;
        }
        GetOfflineTime() {
            return this.user[StorageUtils.offlineTime];
        }
        SetOfflineTime() {
            this.user[StorageUtils.offlineTime] = StorageUtils.getCurTimer();
            this.SaveUser();
        }
        GetlixianAd() {
            if (this.user[StorageUtils.lixianAdGet] == null) {
                this.user[StorageUtils.lixianAdGet] = 0;
            }
            return this.user[StorageUtils.lixianAdGet];
        }
        SetlixianAd(val) {
            this.user[StorageUtils.lixianAdGet] = val;
            this.SaveUser();
        }
        GetlixianGoldAward() {
            if (this.user[StorageUtils.lixianGoldAward] == null) {
                this.user[StorageUtils.lixianGoldAward] = 0;
            }
            this.goldNum = this.goldNum.multipliedBy(0).plus(this.user[StorageUtils.lixianGoldAward]);
            return this.goldNum;
        }
        SetlixianGoldAward(val) {
            val = this.bigNum.multipliedBy(0).plus(val);
            val = MathUtil.bignumerToString(val);
            this.user[StorageUtils.lixianGoldAward] = val;
            this.SaveUser();
        }
        setForeverRecord(data = 1, type = "", isAdd = true) {
            if (typeof data === "number" || typeof data === "string") {
            }
            else if (typeof data === "object") {
            }
            this.setUserInfoData(data, isAdd, StorageUtils.foreverRecord, false, type);
        }
        getForeverRecord(type = "") {
            let value = this.getUserInfoData(StorageUtils.foreverRecord, false, type);
            return value;
        }
        IsCloseMusic() {
            return this.GetMusic() != 1;
        }
        GetMusic() {
            let state = StorageUtils.music in this.user ? this.user[StorageUtils.music] : 1;
            return state;
        }
        SetMusic(value) {
            SoundMgr.Inst.isMusicPlay = value == 1;
            this.user[StorageUtils.music] = value;
            this.SaveUser();
        }
        GetSound() {
            let state = StorageUtils.sound in this.user ? this.user[StorageUtils.sound] : 1;
            return state;
        }
        SetSound(value) {
            SoundMgr.Inst.isSoundPlay = value == 1;
            this.user[StorageUtils.sound] = value;
            this.SaveUser();
        }
        GetShake() {
            let state = StorageUtils.shake in this.user ? this.user[StorageUtils.shake] : 1;
            return state;
        }
        SetShake(value) {
            this.user[StorageUtils.shake] = value;
            this.SaveUser();
        }
        HasGetDesktop() {
            return StorageUtils.desktop in this.user ? this.user[StorageUtils.desktop] : false;
        }
        SetDesktop() {
            this.user[StorageUtils.desktop] = true;
            this.SaveUser();
        }
        HasAddToFavorites() {
            return StorageUtils.addToFavorites in this.user ? this.user[StorageUtils.addToFavorites] : false;
        }
        SetAddToFavorites() {
            this.user[StorageUtils.addToFavorites] = true;
            this.SaveUser();
        }
        HasBuild(buildId) {
            let builds = this.GetBuilds();
            return buildId in builds;
        }
        GetBuilds() {
            return StorageUtils.buildvs in this.user ? this.user[StorageUtils.buildvs] : { 2: 1001 };
        }
        GetBuildStage(buildId) {
            let builds = this.GetBuilds();
            let build = builds[buildId];
            if (isNaN(build))
                return 0;
            return Math.floor(build / 1000);
        }
        GetBuildLv(buildId) {
            let builds = this.GetBuilds();
            let build = builds[buildId];
            if (isNaN(build))
                return 0;
            return build % 1000;
        }
        SetBuildStage(buildId, stage) {
            let builds = this.GetBuilds();
            let build = builds[buildId];
            let lv = build % 1000;
            if (isNaN(lv))
                lv = 0;
            builds[buildId] = stage * 1000 + lv;
            this.user[StorageUtils.buildvs] = builds;
            this.SaveUser();
        }
        SetBuildLv(buildId, lv) {
            let builds = this.GetBuilds();
            let build = builds[buildId];
            if (isNaN(build))
                build = 0;
            let stage = Math.floor(build / 1000);
            builds[buildId] = stage * 1000 + lv;
            this.user[StorageUtils.buildvs] = builds;
            this.SaveUser();
        }
        GetBuildZuoweis() {
            return StorageUtils.buildzws in this.user ? this.user[StorageUtils.buildzws] : {};
        }
        GetBuildZuoweiCount(buildId) {
            let builds = this.GetBuildZuoweis();
            return buildId in builds ? builds[buildId] : 1;
        }
        SetBuilZuowei(buildId, count) {
            let builds = this.GetBuildZuoweis();
            builds[buildId] = count;
            this.user[StorageUtils.buildzws] = builds;
            this.SaveUser();
        }
        GetBuildCatSerArrs() {
            return StorageUtils.buildCatSer in this.user ? this.user[StorageUtils.buildCatSer] : {};
        }
        GetBuildCatSerArr(buildId) {
            let builds = this.GetBuildCatSerArrs();
            return buildId in builds ? builds[buildId] : [];
        }
        SetBuilCatSerArr(buildId, arr) {
            let builds = this.GetBuildCatSerArrs();
            builds[buildId] = arr;
            this.user[StorageUtils.buildCatSer] = builds;
            this.SaveUser();
        }
        GetLandLv() {
            let lv = this.getUserInfoData(StorageUtils.landLv, false, "", 1);
            return lv;
        }
        SetLandLv(value) {
            this.setUserInfoData(value, false, StorageUtils.landLv, false);
        }
        GetMatou() {
            let obj = this.getUserInfoData(StorageUtils.matou, false, "", {});
            return obj;
        }
        SetMatou(value) {
            this.setUserInfoData(value, false, StorageUtils.matou, false);
        }
        SetCatName(name, catId) {
            let cat = this.GetCat(catId);
            cat[StorageUtils.cat_Name] = name;
            this.SaveUser();
        }
        GetCatName(catId) {
            return "";
        }
        SetChuruCount(value, catId) {
            let cat = this.GetCat(catId);
            cat[StorageUtils.cat_ChuruCount] = value;
            this.SaveUser();
        }
        GetChuruCount(catId) {
            let cat = this.GetCat(catId);
            let cnt = cat && StorageUtils.cat_ChuruCount in cat ? cat[StorageUtils.cat_ChuruCount] : "0";
            if (!cnt)
                cnt = 0;
            return typeof cnt === "string" ? parseInt(cnt) : cnt;
        }
        SetPoseList(value, catId) {
            let cat = this.GetCat(catId);
            cat[StorageUtils.cat_PoseList] = value;
            this.SaveUser();
        }
        GetPoseList(catId) {
            let cat = this.GetCat(catId);
            return cat && StorageUtils.cat_PoseList in cat ? cat[StorageUtils.cat_PoseList] : "";
        }
        addPoseAd(count = 1, catId, poseId) {
            let cat = this.GetCat(catId);
            let obj = cat[StorageUtils.cat_PoseAdList];
            if (obj == null) {
                obj = {};
                cat[StorageUtils.cat_PoseAdList] = obj;
            }
            let cnt = obj[poseId] || 0;
            cnt += count;
            obj[poseId] = cnt;
            this.SaveUser();
        }
        getPoseAdCount(catId, poseId) {
            let cat = this.GetCat(catId);
            let obj = cat && cat[StorageUtils.cat_PoseAdList];
            let cnt = obj ? obj[poseId] : 0;
            return cnt;
        }
        SetClothList(value, catId) {
            let cat = this.GetCat(catId);
            cat[StorageUtils.cat_ClothList] = value;
            this.SaveUser();
        }
        GetClothList(catId) {
            let cat = this.GetCat(catId);
            return cat && StorageUtils.cat_ClothList in cat ? cat[StorageUtils.cat_ClothList] : "";
        }
        GetQinmiAwardList(catId) {
            let cat = this.GetCat(catId);
            return cat && StorageUtils.cat_QinmiAwardList in cat ? cat[StorageUtils.cat_QinmiAwardList] : [];
        }
        SetQinmiAwardList(list, catId) {
            let cat = this.GetCat(catId);
            cat[StorageUtils.cat_QinmiAwardList] = list;
            this.SaveUser();
        }
        SetSpecialCatFecal(value) {
            this.setUserInfoData(value, true, StorageUtils.cat_fecal_special, false, "");
        }
        GetSpecialCatFecal() {
            return this.getUserInfoData(StorageUtils.cat_fecal_special, false, "");
        }
        SetSpecialCatFood(value) {
            this.setUserInfoData(value, true, StorageUtils.cat_food_specail, false, "");
        }
        GetSpecialCatFood() {
            return this.getUserInfoData(StorageUtils.cat_food_specail, false, "");
        }
        SetCatFecalTime(time, catId) {
            let cat = this.GetCat(catId);
            cat[StorageUtils.cat_fecal] = time;
            this.SaveUser();
        }
        GetCatLv(catId) {
            let cat = this.GetCat(catId);
            return cat && StorageUtils.cat_Lv in cat ? cat[StorageUtils.cat_Lv] : 0;
        }
        SetCatLv(lv, catId) {
            let cat = this.GetCat(catId);
            cat[StorageUtils.cat_Lv] = lv;
            this.SaveUser();
        }
        GetCatLoveLv(catId) {
            let cat = this.GetCat(catId);
            return cat && StorageUtils.cat_LoveLv in cat ? cat[StorageUtils.cat_LoveLv] : 1;
        }
        SetCatLoveLv(lv, catId) {
            let cat = this.GetCat(catId);
            cat[StorageUtils.cat_LoveLv] = lv;
            this.SaveUser();
        }
        GetCat(catId, defaultVal = null) {
            let cat = this.getUserInfoData(StorageUtils.cats, false, catId, defaultVal);
            if (cat == null) {
                cat = {};
                this.setUserInfoData(cat, false, StorageUtils.cats, false, catId, false);
            }
            return cat;
        }
        GetCats() {
            let cat = this.getUserInfoData(StorageUtils.cats, false, "", null);
            return cat;
        }
        AddItem(tempId, count = 1) {
            return this.setUserInfoData(count, true, StorageUtils.items, false, tempId);
        }
        RemoveItem(tempId, count = 1) {
            let cnt = this.setUserInfoData(-count, true, StorageUtils.items, false, tempId);
            if (cnt <= 0) {
                let data = this.getUserInfoData(StorageUtils.items, false);
                if (data) {
                    delete data[tempId];
                    this.SaveUser();
                }
            }
        }
        HasItem(tempId) {
            let count = this.getUserInfoData(StorageUtils.items, false, tempId, null);
            return count > 0;
        }
        GetItems() {
            let skins = [];
            let data = this.getUserInfoData(StorageUtils.items, false, "", null);
            if (data) {
                let tempId;
                for (tempId in data) {
                    let cnt = data[tempId];
                    if (cnt > 0) {
                        skins.push({ "tempId": tempId, "count": cnt });
                    }
                    else {
                        delete data[tempId];
                    }
                }
            }
            return skins;
        }
        GetSkinBaike(skinId) {
            let cnt = this.getUserInfoData(StorageUtils.skin_baike, false, skinId);
            return cnt;
        }
        AddSkin(skinId, count = 1) {
            this.setUserInfoData(count, true, StorageUtils.skin_baike, false, skinId);
            return this.setUserInfoData(count, true, StorageUtils.skin, false, skinId);
        }
        RemoveSkin(skinId, count = 1) {
            let cnt = this.setUserInfoData(-count, true, StorageUtils.skin, false, skinId);
            if (cnt <= 0) {
                let data = this.getUserInfoData(StorageUtils.skin, false);
                if (data) {
                    delete data[skinId];
                    this.SaveUser();
                }
            }
        }
        HasSkin(skinId) {
            let count = this.getUserInfoData(StorageUtils.skin, false, skinId, null);
            return count > 0;
        }
        GetSkins() {
            let skins = [];
            let data = this.getUserInfoData(StorageUtils.skin, false, "", {});
            if (data) {
                let skinId;
                for (skinId in data) {
                    skins.push({ "skinId": skinId, "count": data[skinId] });
                }
            }
            return skins;
        }
        addFish(time) {
            this.setUserInfoData(time, false, StorageUtils.add_fish, false);
        }
        GetAddFishTime() {
            let time = this.getUserInfoData(StorageUtils.add_fish, false);
            return time;
        }
        updateMainTask(taskId, pro, state = 0) {
            let data = this.getMainTask();
            data["taskId"] = taskId;
            data["pro"] = pro;
            data["state"] = state;
            this.setUserInfoData(data, false, StorageUtils.mainTask, false);
        }
        getMainTask() {
            let obj = this.getUserInfoData(StorageUtils.mainTask, false, "", null);
            if (obj == null) {
                obj = { "taskId": 1, "pro": 0, "state": 0 };
            }
            return obj;
        }
        InitDayTaskList(list) {
            let obj = this.getDayTask();
            if (obj == null) {
                obj = {};
            }
            if (obj["award"] == null) {
                obj["award"] = false;
            }
            if (obj["list"] == null) {
                obj["list"] = list;
            }
            if (obj[StorageUtils.freeLockDrawCount] == null) {
                obj[StorageUtils.freeLockDrawCount] = 0;
            }
            this.setUserInfoData(obj, false, StorageUtils.dayTask, true);
        }
        getDayTask() {
            let obj = this.getUserInfoData(StorageUtils.dayTask, true, "", null);
            return obj;
        }
        SaveDayTask(taskId, pro, state) {
            let obj = this.getDayTask();
            if (obj) {
                let list = obj.list;
                for (let i = 0; i < list.length; i++) {
                    let item = list[i];
                    if (item.taskId == taskId) {
                        item.pro = pro;
                        item.state = state;
                        break;
                    }
                }
                this.setUserInfoData(obj, false, StorageUtils.dayTask, true);
                return true;
            }
            return false;
        }
        SaveAllDayTaskComp(award) {
            let obj = this.getDayTask();
            if (obj) {
                obj.award = award;
                this.setUserInfoData(obj, false, StorageUtils.dayTask, true);
                return true;
            }
            return false;
        }
        get freeLockDrawCount() {
            let obj = this.getDayTask();
            let count = 0;
            if (obj != null) {
                if (obj[StorageUtils.freeLockDrawCount] == null) {
                    obj[StorageUtils.freeLockDrawCount] = 0;
                }
                count = obj[StorageUtils.freeLockDrawCount];
            }
            return count;
        }
        set freeLockDrawCount(val) {
            let obj = this.getDayTask();
            if (obj == null) {
                obj = {};
            }
            obj[StorageUtils.freeLockDrawCount] = val;
            this.setUserInfoData(obj, false, StorageUtils.dayTask, true);
        }
        GetCommodityAdCount(id) {
            return this.getUserInfoData(StorageUtils.commodityAdCount, true, id, 0);
        }
        SetCommodityAdCount(id, val) {
            this.setUserInfoData(val, false, StorageUtils.commodityAdCount, true, id);
        }
        LastGetGiftBoxTime(id) {
            return this.getUserInfoData(StorageUtils.commodityAdTime, true, id, 0);
        }
        SetLastGetGiftBoxTime(id, time) {
            this.setUserInfoData(time, false, StorageUtils.commodityAdTime, true, id);
        }
        GetShopLockDrawProp(id) {
            return this.getUserInfoData(StorageUtils.shopLockDrawProp, false, id, 0);
        }
        SetShopLockDrawProp(id, val) {
            this.setUserInfoData(val, false, StorageUtils.shopLockDrawProp, false, id, true);
        }
        GetShopLockMaxDrawProp(id) {
            return this.getUserInfoData(StorageUtils.shopLockMaxDrawProp, false, id, 0);
        }
        SetShopLockMaxDrawProp(id, val) {
            this.setUserInfoData(val, false, StorageUtils.shopLockMaxDrawProp, false, id, true);
        }
        GetDoubleGoldTime() {
            let time = this.getUserInfoData(StorageUtils.doubleGoldTime, false, "", 0);
            let offestTime = Math.floor((StorageUtils.getCurTimer() - time) / 1000);
            return offestTime;
        }
        SetDoubleGoldTime(time) {
            this.setUserInfoData(time, false, StorageUtils.doubleGoldTime, false, "", true);
        }
        GetGiftPKGAdCount(id, isDay = true) {
            return this.getUserInfoData(StorageUtils.giftPKGAdCount, isDay, id, 0);
        }
        SetGiftPKGAdCount(id, val, isDay = true) {
            this.setUserInfoData(val, false, StorageUtils.giftPKGAdCount, isDay, id);
        }
        GetGiftPKGBuyCount(id, isDay = true) {
            return this.getUserInfoData(StorageUtils.giftPKGBuyCount, isDay, id, 0);
        }
        SetGiftPKGBuyCount(id, val, isDay = true) {
            this.setUserInfoData(val, false, StorageUtils.giftPKGBuyCount, isDay, id);
        }
        LastGetGiftPKGTime(id) {
            return this.getUserInfoData(StorageUtils.giftPKGAdTime, true, id, 0);
        }
        SetLastGetGiftPKGTime(id, time) {
            this.setUserInfoData(time, false, StorageUtils.giftPKGAdTime, true, id);
        }
        LastShowGiftPKGTime(id) {
            return this.getUserInfoData(StorageUtils.giftPKGShowAdTime, true, id, 0);
        }
        SetLastShowGiftPKGTime(id, time) {
            this.setUserInfoData(time, false, StorageUtils.giftPKGShowAdTime, true, id);
        }
        GetGrowthUpGiftPack(id) {
            return this.getUserInfoData(StorageUtils.growthUpGiftPack, false, id, false);
        }
        SetGrowthUpGiftPack(id, bol) {
            this.setUserInfoData(bol, false, StorageUtils.growthUpGiftPack, false, id, true);
        }
        set isDouyinFollow(value) {
            let data = value ? 1 : 0;
            this.setUserInfoData(data, false, StorageUtils.DouyinFollow, false);
        }
        get isDouyinFollow() {
            let val = this.getUserInfoData(StorageUtils.DouyinFollow, false);
            return val == 1;
        }
        set isAgreeHWAd(value) {
            let data = value ? 1 : 0;
            this.setUserInfoData(data, false, StorageUtils.AgreeHWAd, false);
        }
        get isAgreeHWAd() {
            let val = this.getUserInfoData(StorageUtils.AgreeHWAd, false);
            return val == 1;
        }
        GetWarOrder() {
            return this.getUserInfoData(StorageUtils.warOrder, false, "");
        }
        SaveWarOrder(obj) {
            if (!obj)
                return;
            this.setUserInfoData(obj, false, StorageUtils.warOrder, false, "", true);
        }
        InitActTaskList(list) {
            let obj = this.getActTask();
            if (obj == null) {
                obj = {};
            }
            if (obj["list"] == null) {
                obj["list"] = list;
            }
            this.setUserInfoData(obj, false, StorageUtils.activity_task, false);
        }
        getActTask() {
            let obj = this.getUserInfoData(StorageUtils.activity_task, false, "", null);
            return obj;
        }
        SaveActTask(taskId, pro, state) {
            let obj = this.getActTask();
            if (obj) {
                let list = obj.list;
                for (let i = 0; i < list.length; i++) {
                    let item = list[i];
                    if (item.taskId == taskId) {
                        item.pro = pro;
                        item.state = state;
                        break;
                    }
                }
                this.setUserInfoData(obj, false, StorageUtils.activity_task, false);
                return true;
            }
            return false;
        }
        ClearActTask() {
            this.setUserInfoData(null, false, StorageUtils.activity_task, false);
        }
        get isShuaXing() {
            return this.getUserInfoData(StorageUtils.shuaxingTime, true, "", false);
        }
        set isShuaXing(val) {
            this.setUserInfoData(val, false, StorageUtils.shuaxingTime, true, "", true);
        }
        getBottleTask() {
            let obj = this.getUserInfoData(StorageUtils.bottleTask, false, "", null);
            if (obj == null) {
                obj = { "taskId": 0, "pro": 0, "state": 0 };
            }
            return obj;
        }
        updateBottleTask(taskId, pro, state = 0) {
            let data = this.getBottleTask();
            data["taskId"] = taskId;
            data["pro"] = pro;
            data["state"] = state;
            this.setUserInfoData(data, false, StorageUtils.bottleTask, false, "", true);
        }
        get isThrowBottle() {
            let obj = this.getUserInfoData(StorageUtils.isThrowBottle, false, "", false);
            return obj;
        }
        set isThrowBottle(bol) {
            if (bol == this.isThrowBottle)
                return;
            this.setUserInfoData(bol, false, StorageUtils.isThrowBottle, false, "", true);
        }
        get bottleState() {
            let obj = this.getUserInfoData(StorageUtils.bottleState, false, "", 0);
            return obj;
        }
        set bottleState(val) {
            if (val == this.bottleState)
                return;
            this.setUserInfoData(val, false, StorageUtils.bottleState, false, "", true);
        }
        get befallCatCount() {
            let obj = this.getUserInfoData(StorageUtils.befallCatCount, true, "", 0);
            return obj;
        }
        set befallCatCount(val) {
            if (val == this.befallCatCount)
                return;
            this.setUserInfoData(val, false, StorageUtils.befallCatCount, true, "", true);
        }
        get canyuyue() {
            return this.getUserInfoData(StorageUtils.canyuyue, false, "", 0);
        }
        set canyuyue(val) {
            if (val == this.canyuyue)
                return;
            this.setUserInfoData(val, false, StorageUtils.canyuyue, false, "", true);
        }
        GetLiHeState(id) {
            return this.getUserInfoData(StorageUtils.liHeState, false, id, 0);
        }
        SetLiHeState(id, val) {
            if (val == this.GetLiHeState(id))
                return;
            this.setUserInfoData(val, false, StorageUtils.liHeState, false, id, true);
        }
        get isAddGame() {
            return this.getUserInfoData(StorageUtils.isAddGame, false, "", false);
        }
        set isAddGame(bol) {
            this.setUserInfoData(bol, false, StorageUtils.isAddGame, false, "", true);
        }
        SetGuideCompelte(guideId) {
            this.setUserInfoData(guideId, false, StorageUtils.guide, false, "");
        }
        GetGuide() {
            let guideId = this.getUserInfoData(StorageUtils.guide, false, "", 0);
            return guideId;
        }
        SetGuideFunc(guideId) {
            let guides = this.GetFuncGuides();
            if (guides.indexOf(guideId) == -1) {
                guides.push(guideId);
            }
            this.setUserInfoData(guides, false, StorageUtils.guide_func, false, "");
            return guides;
        }
        GetFuncGuides() {
            let guideId = this.getUserInfoData(StorageUtils.guide_func, false, "", []);
            return guideId;
        }
        setCloseNBannerCount(value = 1, isAdd = true) {
            this.setUserInfoData(value, isAdd, StorageUtils.closeNBannerCount);
        }
        getCloseNBannerCount() {
            return this.getUserInfoData(StorageUtils.closeNBannerCount);
        }
        setNInsertCount(value = 1, isAdd = true) {
            this.setUserInfoData(value, isAdd, StorageUtils.NInsertCount);
        }
        getNInsertCount() {
            return this.getUserInfoData(StorageUtils.NInsertCount);
        }
        setShareCount(value = 1, isAdd = true) {
            this.setUserInfoData(value, isAdd, StorageUtils.shareCount);
        }
        getShareCount() {
            return this.getUserInfoData(StorageUtils.shareCount);
        }
        setUserInfoData(value, isAdd, key, isDay = true, key2 = "", isSave = true) {
            let count = this.getUserInfoData(key, isDay, key2);
            let data;
            if (isDay) {
                data = this._dayData;
            }
            else {
                data = this._foreverData;
            }
            if (typeof key2 === "number" || key2 != "") {
                let obj = data[key] || {};
                data[key] = obj;
                data = obj;
                key = key2;
            }
            if (typeof value === "number") {
                if (isAdd) {
                    count += value;
                }
                else {
                    count = value;
                }
                data[key] = count;
                if (isSave)
                    this.SaveUser();
                return count;
            }
            else {
                data[key] = value;
                if (isSave)
                    this.SaveUser();
                return value;
            }
        }
        getUserInfoData(key, isDay = true, key2 = "", defaultVal = 0) {
            let data = isDay ? this._dayData : this._foreverData;
            if (typeof key2 === "number" || key2 != "") {
                let o = data[key];
                if (!o) {
                    o = {};
                    data[key] = o;
                }
                data = data[key];
                key = key2;
            }
            if (data == null || data == 0) {
                return 0;
            }
            if (typeof data === "object" && !(key in data)) {
                if (typeof defaultVal != "number" && defaultVal == null) {
                }
                else {
                    data[key] = defaultVal;
                }
            }
            return data[key];
        }
        get user() {
            if (this._user == null) {
                this._user = {};
            }
            return this._user;
        }
        clear() {
            this._user = {};
            this.SaveUser();
        }
        SaveUser() {
            this.isNeedSave = true;
        }
    }
    StorageUtils.username = "username";
    StorageUtils.userinfo = "userinfo";
    StorageUtils.loginTime = "loginTime";
    StorageUtils.todayFirstTime = "todayFirstTime";
    StorageUtils.closeNBannerCount = "closeNBannerCount";
    StorageUtils.NInsertCount = "NInsertCount";
    StorageUtils.dayTask = "dayTask";
    StorageUtils.commodityAdTime = "commodityAdTime";
    StorageUtils.commodityAdCount = "commodityAdCount";
    StorageUtils.nipViewAdCont = "nipViewAdCont";
    StorageUtils.canGetDima = "canGetDima";
    StorageUtils.befallCatCount = "befallCatCount";
    StorageUtils.canyuyue = "canyuyue";
    StorageUtils.shareCount = "shareCount";
    StorageUtils.AgreeHWAd = "AgreeHWAd";
    StorageUtils.DouyinFollow = "DouyinFollow";
    StorageUtils.foreverRecord = "foreverRecord";
    StorageUtils.gold = "gold";
    StorageUtils.diamond = "diamond";
    StorageUtils.onlineTime = "onlineTime";
    StorageUtils.offlineTime = "offlineTime";
    StorageUtils.music = "music";
    StorageUtils.sound = "sound";
    StorageUtils.shake = "shake";
    StorageUtils.desktop = "desktop";
    StorageUtils.firstLogin = "firstLogin";
    StorageUtils.buildvs = "buildvs";
    StorageUtils.buildzws = "buildzws";
    StorageUtils.buildCatSer = "BuildCatSer";
    StorageUtils.landLv = "landLv";
    StorageUtils.matou = "matou";
    StorageUtils.cats = "cats";
    StorageUtils.cat_Name = "cat_Name";
    StorageUtils.cat_Lv = "cat_Lv";
    StorageUtils.cat_LoveLv = "cat_LoveLv";
    StorageUtils.cat_ChuruCount = "cat_ChuruCount";
    StorageUtils.cat_PoseList = "cat_PoseList";
    StorageUtils.cat_PoseAdList = "cat_PoseAdList";
    StorageUtils.cat_ClothList = "cat_ClothList";
    StorageUtils.cat_QinmiAwardList = "cat_QinmiAwardList";
    StorageUtils.cat_fecal = "cat_fecal";
    StorageUtils.cat_fecal_special = "cat_fecal_special";
    StorageUtils.cat_food_specail = "cat_food_specail";
    StorageUtils.items = "item";
    StorageUtils.skin = "skin";
    StorageUtils.skin_baike = "skin_baike";
    StorageUtils.add_fish = "add_fish";
    StorageUtils.signInfo = "signInfo";
    StorageUtils.signDays = "signDays";
    StorageUtils.signPrevTime = "signPrevTime";
    StorageUtils.mainTask = "mainTask";
    StorageUtils.guide = "guide";
    StorageUtils.guide_func = "guide_func";
    StorageUtils.freeLockDrawCount = "freeLockDrawCount";
    StorageUtils.shopLockDrawProp = "shopLockDrawProp";
    StorageUtils.shopLockMaxDrawProp = "shopLockMaxDrawProp";
    StorageUtils.doubleGoldTime = "doubleGold";
    StorageUtils.loginDayNum = "loginDayNum";
    StorageUtils.giftPKGAdTime = "giftPKGAdTime";
    StorageUtils.giftPKGAdCount = "giftPKGAdCount";
    StorageUtils.giftPKGBuyCount = "giftPKGBuyCount";
    StorageUtils.giftPKGShowAdTime = "giftPKGShowAdTime";
    StorageUtils.lixianAdGet = "lixianAdGet";
    StorageUtils.lixianGoldAward = "lixianGoldAward";
    StorageUtils.growthUpGiftPack = "growthUpGiftPack";
    StorageUtils.warOrder = "warOrder";
    StorageUtils.activity_task = "activity_task";
    StorageUtils.shuaxingTime = "shuaxingTime";
    StorageUtils.bottleTask = "bottleTask";
    StorageUtils.isThrowBottle = "isThrowBottle";
    StorageUtils.bottleState = "bottleState";
    StorageUtils.liHeState = "liHeState";
    StorageUtils.qq_subscribe = "subscribe";
    StorageUtils.add_desktop = "add_desktop";
    StorageUtils.addToFavorites = "addToFavorites";
    StorageUtils.isAddGame = "isAddGame";

    class ObjectPool {
        constructor(key, cls, objFactory = null) {
            this.gcTime = Number.MAX_VALUE;
            this.objs = null;
            this.cls = null;
            this.key = null;
            this.key = key;
            this.cls = cls;
            this.objs = new Array();
            this._objFactory = objFactory;
        }
        IsCanGc() {
            return this.objs.length > 0 && Laya.timer.currTimer >= this.gcTime;
        }
        Dispose() {
            this.objs.length = 0;
            this.gcTime = Number.MAX_VALUE;
        }
        getObj() {
            if (this.objs.length > 0) {
                let obj = this.objs.pop();
                if (this._objFactory == null && obj instanceof Laya.Script3D) {
                    obj.enabled = true;
                }
                return obj;
            }
            else if (this.cls != null) {
                return new this.cls();
            }
            else if (this._objFactory != null) {
                return this._objFactory();
            }
            else {
                return null;
            }
        }
        gcObj(obj) {
            if (obj == null) {
                return;
            }
            if (Logger.isShowLog && this.objs.indexOf(obj) != -1) {
                Logger.warning(this, "重复回收对象");
            }
            obj.Reset();
            this.objs.push(obj);
            if (this._objFactory == null && obj instanceof Laya.Script3D) {
                obj.enabled = false;
            }
            this.gcTime = Laya.timer.currTimer + Resource.CACHE_OBJECT_TIME;
        }
    }

    class ObjectPools {
        constructor() {
            this.count = 0;
            if (ObjectPools._Inst != null) {
                throw new Error("singleton!");
            }
            this._classPool = new Dictionary();
            Laya.timer.loop(Resource.CACHE_OBJECT_TIME, this, this.CheckCache);
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new ObjectPools();
            return this._Inst;
        }
        CheckCache() {
            let _classPool = this._classPool;
            let ppol = null;
            for (let key in _classPool) {
                ppol = _classPool[key];
                if (ppol && ppol.IsCanGc()) {
                    ppol.Dispose();
                }
            }
        }
        registPoolForClass(key, cls) {
            if (key == null) {
                Logger.error(this, key + "key is null!");
                return;
            }
            if (key in this._classPool) {
                Logger.error(this, key + "key has register!");
                return;
            }
            var pool = new ObjectPool(key, cls);
            this._classPool[key] = pool;
            return pool;
        }
        unRegistPoolForClass(key, cls) {
            if (key == null) {
                Logger.error(this, key + "key is null!");
                return;
            }
            var pool = this._classPool[key];
            if (pool != null) {
                delete this._classPool[key];
            }
            return pool;
        }
        getObj(key, cls = null) {
            var pool = this._classPool[key];
            if (pool == null) {
                pool = this.registPoolForClass(key, cls);
            }
            return pool.getObj();
        }
        gcObj(key, obj) {
            var pool = this._classPool[key];
            if (pool == null) {
                Logger.error(this, "unregister Class");
            }
            pool.gcObj(obj);
        }
    }
    ObjectPools._Inst = null;

    class AssetCache {
        constructor() {
        }
        static Init() {
            Laya.timer.loop(AssetCache.LOOP_TIME, AssetCache, AssetCache.CheckCache);
        }
        static CheckCache() {
            let resDic = AssetCache.assetDic;
            var iRes = null;
            for (let key in resDic) {
                iRes = resDic[key];
                if (iRes && iRes.IsCanGc()) {
                    iRes.Destroy();
                    delete resDic[key];
                }
            }
            Laya.Resource.destroyUnusedResources();
        }
        static AddResource(url, resource) {
            AssetCache.assetDic[url] = resource;
        }
        static GetResource(url) {
            return AssetCache.assetDic[url];
        }
        static LoadIcon(loader, url, handler = null) {
            if (!loader || !url)
                return;
            url = decodeURIComponent(url);
            LoadQueue.Inst.loadTexture(url, Laya.Handler.create(null, function () {
                loader.url = url;
                if (handler != null)
                    handler.run();
            }));
        }
    }
    AssetCache.assetDic = new Dictionary();
    AssetCache.LOOP_TIME = 1 * 30 * 1000;

    class LoadLevel {
        constructor() {
        }
    }
    LoadLevel.UI_MODEL = 0;
    LoadLevel.DEFAULT = 1;
    LoadLevel.SCENE = 2;
    LoadLevel.SOUND = 3;
    LoadLevel.PROLOADING = 4;

    class LoadTask {
        constructor() {
            this.handlers = new Array();
            this.level = 0;
            this.state = LoadTask.STATE_DEFAULT;
            this.isRecover = true;
            this.rate = 0;
        }
        run() {
            this.state = LoadTask.STATE_RUN;
        }
        onComplete(iRes, isCache = true) {
            if (iRes)
                Logger.log(this, "加载完成：    " + this.url);
            else
                Logger.log(this, "加载失败：    " + this.url);
            if (isCache && iRes != null && iRes instanceof Resource) {
                iRes.SetUrl(this.url);
                AssetCache.AddResource(this.url, iRes);
            }
            this.state = LoadTask.STATE_COMPLETE;
            var leng = this.handlers.length;
            var func;
            for (var i = 0; i < leng; i++) {
                func = this.handlers[i];
                if (func) {
                    iRes != null ? func.runWith(iRes) : func.run();
                }
            }
            this.handlers.length = 0;
            this.rate = 1;
        }
        onUpdate() {
            if (this.rate < 0.95) {
                this.rate += 0.01;
            }
        }
        onLoadProgress(rate) {
            if (rate > 1)
                rate = 1;
            this.rate = rate;
        }
        GetSubName() {
            return PlatformMgr.Inst.GetSubName(this.url);
        }
        GC() {
        }
        Reset() {
            this.handlers.length = 0;
            this.url = null;
            this.level = 0;
            this.state = LoadTask.STATE_DEFAULT;
            this.isRecover = true;
            this.rate = 0;
        }
    }
    LoadTask.STATE_DEFAULT = 0;
    LoadTask.STATE_RUN = 1;
    LoadTask.STATE_COMPLETE = 2;

    class AnimationClipResource extends Resource {
        constructor() {
            super();
        }
        GC() {
            ObjectPools.Inst.gcObj(AnimationClipResource.KEY, this);
        }
    }
    AnimationClipResource.KEY = "AnimationClipResource";

    class AnimationClipLoadTask extends LoadTask {
        constructor() {
            super();
        }
        run() {
            super.run();
            let subName = this.GetSubName();
            if (subName != null) {
                LoadQueue.Inst.loadPageSub(subName, Laya.Handler.create(this, this.StartLoad));
            }
            else {
                this.StartLoad(true);
            }
        }
        StartLoad(isSuccess) {
            Laya.loader.load(this.url, Laya.Handler.create(this, this.onLoadComplete), Laya.Handler.create(this, this.onLoadProgress, null, false), Laya.Loader.ANIMATIONCLIP, this.level);
        }
        onLoadComplete(ani) {
            let iRes = ObjectPools.Inst.getObj(AnimationClipResource.KEY, AnimationClipResource);
            if (!this.isRecover) {
                if (ani)
                    ani.lock = true;
            }
            iRes.Decode(ani);
            iRes.isRecover = this.isRecover;
            super.onComplete(iRes);
        }
        GC() {
            ObjectPools.Inst.gcObj(AnimationClipLoadTask.KEY, this);
        }
    }
    AnimationClipLoadTask.KEY = "AnimationClipLoadTask";

    class ByteLoadTask extends LoadTask {
        constructor() {
            super();
        }
        run() {
            super.run();
            Laya.loader.load(this.url, Laya.Handler.create(this, this.onLoadComplete), Laya.Handler.create(this, this.onLoadProgress, null, false), Laya.Loader.BUFFER, this.level);
        }
        onLoadComplete(buffer) {
            let bytes = new Laya.Byte(buffer);
            super.onComplete(bytes);
        }
        GC() {
            ObjectPools.Inst.gcObj(ByteLoadTask.KEY, this);
        }
    }
    ByteLoadTask.KEY = "ByteLoadTask";

    class GroupSheetResource extends Resource {
        constructor() {
            super();
            this.urls = null;
            this.sourceMap = {};
        }
        setUrls(urls) {
            this.urls = urls;
        }
        setRes(url, source) {
            this.sourceMap[url] = source;
        }
        getRes(resName) {
            let url = fgui.UIPackage.getItemURL(this.pkgName, resName);
            let res = this.sourceMap[url];
            if (res == null) {
                let pkg = fgui.UIPackage.getByName(this.pkgName);
                let item = pkg && pkg.getItemByName(resName);
                let sprite = item && pkg["_sprites"][item.id];
                let atlasItem = sprite && sprite.atlas;
                if (atlasItem && atlasItem.texture) {
                    atlasItem.load();
                    res = fgui.UIPackage.getItemAssetByURL(url);
                    this.setRes(url, res);
                }
            }
            return res;
        }
        loadResName(resName, handler) {
            let url = fgui.UIPackage.getItemURL(this.pkgName, resName);
            let texture = this.getRes(url);
            if (texture) {
                handler && handler.runWith(url);
            }
            else {
                PanelRegister.loadUrl(url, Laya.Handler.create(this, (handler, url) => {
                    let texture = fgui.UIPackage.getItemAssetByURL(url);
                    this.setRes(url, texture);
                    handler && handler.runWith(url);
                }, [handler]));
            }
        }
        Reset() {
            super.Reset();
            this.urls = null;
            this.pkgName = "";
            this.sourceMap = {};
        }
        GC() {
            ObjectPools.Inst.gcObj(GroupSheetResource.KEY, this);
        }
    }
    GroupSheetResource.KEY = "GroupSheetResource";

    class GroupSheetLoadTask extends LoadTask {
        constructor() {
            super();
            this.loadCount = 3;
        }
        run() {
            super.run();
            this.loadCount = 3;
            let subName = this.GetSubName();
            if (subName != null) {
                LoadQueue.Inst.loadPageSub(subName, Laya.Handler.create(this, this.StartLoad));
            }
            else {
                this.StartLoad(true);
            }
        }
        StartLoad(isSuccess) {
            let resKey = this.url;
            let url = resKey + "." + fgui.UIConfig.packageFileExtension;
            fgui.AssetProxy.inst.load(url, Laya.Handler.create(this, this.descCompleteHandler, [resKey]), null, Laya.Loader.BUFFER);
        }
        descCompleteHandler(resKey, asset) {
            if (asset == null) {
                this.loadCount--;
                if (this.loadCount > 0) {
                    this.StartLoad(true);
                    return;
                }
                else {
                    throw new Error("descCompleteHandler error resKey:" + resKey);
                }
            }
            let pkg = new fgui.UIPackage();
            pkg["_resKey"] = resKey;
            pkg["loadPackage"](new fgui.ByteBuffer(asset), resKey);
            let cnt = pkg["_items"].length;
            let urls = [];
            for (var i = 0; i < cnt; i++) {
                var pi = pkg["_items"][i];
                if (pi.type == fgui.PackageItemType.Atlas)
                    urls.push({ url: pi.file, type: Laya.Loader.IMAGE });
                else if (pi.type == fgui.PackageItemType.Sound)
                    urls.push({ url: pi.file, type: Laya.Loader.SOUND });
            }
            pkg["_customId"] = resKey;
            fgui.UIPackage["_instById"][pkg.id] = pkg;
            fgui.UIPackage["_instByName"][pkg.name] = pkg;
            this.onLoadComplete(urls);
        }
        onLoadComplete(urls) {
            var resource = new GroupSheetResource();
            resource.pkgName = this.pkgName;
            resource.setUrls(urls);
            super.onComplete(resource);
        }
        GC() {
            ObjectPools.Inst.gcObj(GroupSheetLoadTask.KEY, this);
        }
    }
    GroupSheetLoadTask.KEY = "GroupSheetLoadTask";

    class NativeAdResource extends Resource {
        constructor() {
            super();
            this.bornTime = 0;
            this.reportShow = false;
            this.reportClick = false;
        }
        GC() {
            ObjectPools.Inst.gcObj(NativeAdResource.KEY, this);
        }
        Reset() {
            if (Laya.Browser.onHWMiniGame) {
                this.nativeAd && this.nativeAd.destroy && this.nativeAd.destroy();
                Logger.debug(this, "=====>销毁原生广告 this.nativeId：" + this.nativeId);
            }
            this.nativeAd = null;
            this.adData = null;
            this.nativeId = null;
            this.reportShow = false;
            this.reportClick = false;
            this.bornTime = 0;
        }
    }
    NativeAdResource.KEY = "NativeAdResource";

    class NativeAdLoadTask extends LoadTask {
        constructor() {
            super();
            this.nativeAd = null;
            this.count = 0;
        }
        run() {
            super.run();
            this.count = 0;
            let self = this;
            if (Laya.Browser.onQGMiniGame) {
                self.nativeAd = qg.createNativeAd({ "adUnitId": this.nativeId });
            }
            else if (Laya.Browser.onVVMiniGame) {
                self.nativeAd = qg.createNativeAd({ "posId": this.nativeId });
            }
            else if (Laya.Browser.onHWMiniGame) {
                self.nativeAd = this.qg ? this.qg.createNativeAd({
                    "adUnitId": this.nativeId,
                    "success": (code) => {
                        Logger.debug(self, "loadNativeAd loadNativeAd : success");
                    },
                    "fail": (data, code) => {
                        Logger.debug(self, "loadNativeAd loadNativeAd fail: " + data + "," + code);
                        self.onLoadComplete(null);
                    },
                    "complete": () => {
                        Logger.debug(self, "loadNativeAd loadNativeAd : complete");
                    }
                }) : hbs.createNativeAd({ "adUnitId": this.nativeId });
            }
            else {
                self.nativeAd = qg.createNativeAd({ "posId": this.nativeId });
            }
            function onError(err) {
                self.nativeAd.offError(onError);
                self.count++;
                Logger.debug(self, "=====>加载原生广告错误 count: " + self.count + " error: " + JSON.stringify(err));
                self.loadTimeOut();
            }
            self.nativeAd.onError(onError);
            function onLoad(res) {
                self.nativeAd.offLoad(onLoad);
                Laya.timer.clear(self, self.loadTimeOut);
                self.nativeAd && self.nativeAd.offLoad && self.nativeAd.offLoad(onLoad);
                Logger.debug(self, "=====>原生广告加载成功  nativeId: " + self.nativeId + "   " + JSON.stringify(res.adList));
                let adList = res.adList;
                for (const adData of adList) {
                    if ((adData.iconUrlList != null && adData.iconUrlList.length >= 1) ||
                        (adData.imgUrlList != null && adData.imgUrlList.length >= 1)) {
                        let res = AssetCache.assetDic[self.url];
                        if (res instanceof NativeAdResource) {
                            res.Destroy();
                            delete AssetCache.assetDic[self.url];
                        }
                        let adRes = ObjectPools.Inst.getObj(NativeAdResource.KEY, NativeAdResource);
                        adRes.nativeId = self.nativeId;
                        adRes.nativeAd = self.nativeAd;
                        adRes.adData = adData;
                        adRes.bornTime = Laya.timer.currTimer;
                        adRes.SetUrl(self.url);
                        self.onLoadComplete(adRes);
                        break;
                    }
                }
            }
            self.nativeAd.onLoad(onLoad);
            self.nativeAd.load();
            Logger.debug(self, "===============>原生广告加载  nativeId: " + self.nativeId);
            Laya.timer.once(10000, this, this.loadTimeOut);
        }
        loadTimeOut() {
            let self = this;
            Logger.debug(self, "=====>加载原生广告超时 self.nativeId: " + self.nativeId);
            self.onLoadComplete(null);
        }
        onLoadComplete(adRes) {
            super.onComplete(adRes);
            this.count = 0;
            Laya.timer.clear(this, this.loadTimeOut);
        }
        get qg() {
            return window["qg"];
        }
    }
    NativeAdLoadTask.KEY = "NativeAdLoadTask";

    class ParticleLoadTask extends LoadTask {
        constructor() {
            super();
        }
        run() {
            super.run();
            let subName = this.GetSubName();
            if (subName != null) {
                LoadQueue.Inst.loadPageSub(subName, Laya.Handler.create(this, this.StartLoad));
            }
            else {
                this.StartLoad(true);
            }
        }
        StartLoad(isSuccess) {
            Laya.loader.load(this.url, Laya.Handler.create(this, this.onLoadComplete), Laya.Handler.create(this, this.onLoadProgress, null, false), Laya.Loader.HIERARCHY, this.level);
        }
        onLoadComplete(sprite) {
            sprite = sprite && sprite.clone();
            super.onComplete(sprite);
        }
        GC() {
            ObjectPools.Inst.gcObj(ParticleLoadTask.KEY, this);
        }
    }
    ParticleLoadTask.KEY = "ParticleLoadTask";

    class Scene3DLoadTask extends LoadTask {
        constructor() {
            super();
        }
        run() {
            super.run();
            let subName = this.GetSubName();
            if (subName != null) {
                LoadQueue.Inst.loadPageSub(subName, Laya.Handler.create(this, this.StartLoad));
            }
            else {
                this.StartLoad(true);
            }
        }
        StartLoad(isSuccess) {
            Laya.loader.load(this.url, Laya.Handler.create(this, this.onLoadComplete), Laya.Handler.create(this, this.onLoadProgress, null, false), Laya.Loader.HIERARCHY, this.level);
        }
        onLoadComplete(scene) {
            super.onComplete(scene);
        }
        GC() {
            ObjectPools.Inst.gcObj(Scene3DLoadTask.KEY, this);
        }
    }
    Scene3DLoadTask.KEY = "Scene3DLoadTask";

    class SoundLoadTask extends LoadTask {
        constructor() {
            super();
        }
        run() {
            super.run();
            let subName = this.GetSubName();
            if (subName != null) {
                LoadQueue.Inst.loadPageSub(subName, Laya.Handler.create(this, this.StartLoad));
            }
            else {
                this.StartLoad(true);
            }
        }
        StartLoad(isSuccess) {
            Laya.loader.load(this.url, Laya.Handler.create(this, this.onLoadComplete), Laya.Handler.create(this, this.onLoadProgress, null, false), Laya.Loader.SOUND, this.level);
        }
        onLoadComplete(sound) {
            super.onComplete(sound);
        }
        GC() {
            ObjectPools.Inst.gcObj(SoundLoadTask.KEY, this);
        }
    }
    SoundLoadTask.KEY = "SoundLoadTask";

    class SpineResource extends Resource {
        constructor() {
            super();
        }
        Destroy() {
            if (this.data)
                this.data.destroy();
            super.Destroy();
        }
        GC() {
            ObjectPools.Inst.gcObj(SpineResource.KEY, this);
        }
    }
    SpineResource.KEY = "SpineResource";

    class SpineLoadTask extends LoadTask {
        constructor() {
            super();
        }
        run() {
            super.run();
            let templet = new Laya.Templet();
            templet.loadAni(this.url);
            templet.once(Laya.Event.COMPLETE, this, this.onLoadSpineTemplet, [templet]);
        }
        onLoadSpineTemplet(templet) {
            let res = ObjectPools.Inst.getObj(SpineResource.KEY, SpineResource);
            res.isRecover = this.isRecover;
            res.Decode(templet);
            this.onComplete(res, true);
        }
        GC() {
            ObjectPools.Inst.gcObj(SpineLoadTask.KEY, this);
        }
    }
    SpineLoadTask.KEY = "SpineLoadTask";

    class Sprite3DLoadTask extends LoadTask {
        constructor() {
            super();
        }
        run() {
            super.run();
            let subName = this.GetSubName();
            if (subName != null) {
                LoadQueue.Inst.loadPageSub(subName, Laya.Handler.create(this, this.StartLoad));
            }
            else {
                this.StartLoad(true);
            }
        }
        StartLoad(isSuccess) {
            Laya.loader.load(this.url, Laya.Handler.create(this, this.onLoadComplete), Laya.Handler.create(this, this.onLoadProgress, null, false), Laya.Loader.HIERARCHY, this.level);
        }
        onLoadComplete(sp) {
            sp = sp && sp.clone();
            super.onComplete(sp);
        }
        GC() {
            ObjectPools.Inst.gcObj(Sprite3DLoadTask.KEY, this);
        }
    }
    Sprite3DLoadTask.KEY = "Sprite3DLoadTask";

    class SubResource extends Resource {
        constructor() {
            super();
        }
        GC() {
            ObjectPools.Inst.gcObj(SubResource.KEY, this);
        }
    }
    SubResource.KEY = "SubResource";

    class SubLoadTask extends LoadTask {
        constructor() {
            super();
            this.count = 0;
        }
        run() {
            super.run();
            let self = this;
            let platform = null;
            if (PlatformMgr.Inst.isWX) {
                platform = window["wx"];
            }
            else if (Laya.Browser.onQQMiniGame) {
                platform = window["qq"];
            }
            else if (Laya.Browser.onBDMiniGame) {
                platform = window["swan"];
            }
            else if (Laya.Browser.onQGMiniGame || Laya.Browser.onVVMiniGame) {
                platform = window["qg"];
            }
            if (platform != null) {
                this.loadSub(platform);
            }
            else {
                self.onLoadComplete(true);
            }
        }
        loadSub(platform) {
            let self = this;
            console.log("加载分包: " + this.url);
            let LoadSubpackageTask = platform["loadSubpackage"]({
                name: self.url,
                success: function (res) {
                    console.log("加载分包成功！");
                    self.onLoadComplete(true);
                },
                fail: function (res) {
                    console.log("加载分包失败！");
                    self.count++;
                    if (self.count > 3) {
                        self.onLoadComplete(true);
                    }
                    else {
                        self.loadSub(platform);
                    }
                },
                complete: function () {
                }
            });
            if (LoadSubpackageTask && LoadSubpackageTask.onProgressUpdate != null) {
                LoadSubpackageTask.onProgressUpdate(function (res) {
                    self.onLoadProgress(res.progress);
                });
            }
        }
        onLoadComplete(isSuccess) {
            let iRes = ObjectPools.Inst.getObj(SubResource.KEY, SubResource);
            iRes.isRecover = this.isRecover;
            iRes.Decode(isSuccess);
            super.onComplete(iRes);
            this.count = 0;
        }
        GC() {
            ObjectPools.Inst.gcObj(SubLoadTask.KEY, this);
        }
        Reset() {
            super.Reset();
            this.count = 0;
        }
    }
    SubLoadTask.KEY = "SubLoadTask";

    class TextureLoadTask extends LoadTask {
        constructor() {
            super();
        }
        run() {
            super.run();
            let subName = this.GetSubName();
            if (subName != null) {
                LoadQueue.Inst.loadPageSub(subName, Laya.Handler.create(this, this.StartLoad));
            }
            else {
                this.StartLoad(true);
            }
        }
        StartLoad(isSuccess) {
            Laya.loader.load(this.url, Laya.Handler.create(this, this.onLoadComplete), Laya.Handler.create(this, this.onLoadProgress, null, false), Laya.Loader.TEXTURE2D, this.level);
        }
        onLoadComplete(texture) {
            super.onComplete(texture);
        }
        GC() {
            ObjectPools.Inst.gcObj(TextureLoadTask.KEY, this);
        }
    }
    TextureLoadTask.KEY = "TextureLoadTask";

    class UIResource extends Resource {
        constructor() {
            super();
        }
        GC() {
            ObjectPools.Inst.gcObj(UIResource.KEY, this);
        }
        Destroy() {
            let index = this.url.lastIndexOf("/");
            if (index != -1) {
                let packageName = this.url.substring(index + 1);
                if (fgui.UIPackage.getByName(packageName))
                    fgui.UIPackage.removePackage(packageName);
            }
            super.Destroy();
        }
    }
    UIResource.KEY = "UIResource";

    class UILoadTask extends LoadTask {
        constructor() {
            super();
            this.urls = null;
        }
        run() {
            super.run();
            let subName = this.GetSubName();
            if (subName != null) {
                LoadQueue.Inst.loadPageSub(subName, Laya.Handler.create(this, this.StartLoad));
            }
            else {
                this.StartLoad(null);
            }
        }
        StartLoad(iRes) {
            let self = this;
            let resKey = this.url;
            this.pkgName = resKey;
            let url = resKey + "." + fgui.UIConfig.packageFileExtension;
            var descCompleteHandler = Laya.Handler.create(this, function (asset) {
                if (asset == null) {
                    console.error("[UILoadTask]=====>StartLoad error url:" + url);
                    return;
                }
                let pkg = new fgui.UIPackage();
                pkg["_resKey"] = resKey;
                pkg["loadPackage"](new fgui.ByteBuffer(asset), resKey);
                let cnt = pkg["_items"].length;
                this.urls = [];
                for (var i = 0; i < cnt; i++) {
                    var pi = pkg["_items"][i];
                    if (pi.type == fgui.PackageItemType.Atlas)
                        self.urls.push({ url: pi.file, type: Laya.Loader.IMAGE });
                    else if (pi.type == fgui.PackageItemType.Sound)
                        self.urls.push({ url: pi.file, type: Laya.Loader.SOUND });
                }
                if (self.urls.length > 0) {
                    Laya.loader.create(self.urls, Laya.Handler.create(this, this.onLoadComplete, [this.url]), Laya.Handler.create(this, this.onLoadProgress, null, false), null, null, this.level);
                }
                else
                    this.onLoadComplete(this.url, true);
            });
            fgui.AssetProxy.inst.load(url, descCompleteHandler, null, Laya.Loader.BUFFER);
        }
        onLoadComplete(url, isSuccess) {
            if (url == this.url) {
                if (isSuccess) {
                    if (!fgui.UIPackage.getByName(this.pkgName)) {
                        fgui.UIPackage.addPackage(this.url);
                    }
                    let url = this.url + "." + fgui.UIConfig.packageFileExtension;
                    let assets = [];
                    assets.push({ "url": url, "asset": Laya.loader.getRes(url) });
                    if (this.urls != null) {
                        let o;
                        for (o of this.urls) {
                            if (o)
                                assets.push({ "url": o.url, "asset": Laya.loader.getRes(o.url) });
                        }
                    }
                    let iRes = ObjectPools.Inst.getObj(UIResource.KEY, UIResource);
                    iRes.isRecover = this.isRecover;
                    iRes.Decode(isSuccess);
                    super.onComplete(iRes);
                }
            }
        }
        GC() {
            ObjectPools.Inst.gcObj(UILoadTask.KEY, this);
        }
    }
    UILoadTask.KEY = "UILoadTask";

    class LoadQueue {
        constructor() {
            this.currentActions = new Array();
            this._curByteLoad = 0;
            this._byteLoaded = 0;
            this.byteTotal = 0;
            Laya.loader.retryNum = 3;
            AssetCache.Init();
        }
        static get Inst() {
            if (this._Inst == null) {
                this._Inst = new LoadQueue();
            }
            return this._Inst;
        }
        loadPageSub(url, handler = null, isRecover = false, level = LoadLevel.DEFAULT) {
            if (this.checkCache(url, handler)) {
                return true;
            }
            if (this.checkQueue(url, handler)) {
                return false;
            }
            var loadTask = ObjectPools.Inst.getObj(SubLoadTask.KEY, SubLoadTask);
            loadTask.handlers.push(handler);
            loadTask.url = url;
            loadTask.level = level;
            loadTask.isRecover = isRecover;
            this.addTask(loadTask);
            return true;
        }
        loadSound(url, handler = null, level = LoadLevel.SOUND) {
            if (this.checkCache(url, handler)) {
                return true;
            }
            if (this.checkQueue(url, handler)) {
                return false;
            }
            var loadTask = ObjectPools.Inst.getObj(SoundLoadTask.KEY, SoundLoadTask);
            loadTask.handlers.push(handler);
            loadTask.url = url;
            loadTask.level = level;
            this.addTask(loadTask);
        }
        loadTexture(url, handler = null, level = LoadLevel.DEFAULT) {
            if (this.checkCache(url, handler)) {
                return true;
            }
            if (this.checkQueue(url, handler)) {
                return false;
            }
            var loadTask = ObjectPools.Inst.getObj(TextureLoadTask.KEY, TextureLoadTask);
            loadTask.handlers.push(handler);
            loadTask.url = url;
            loadTask.level = level;
            this.addTask(loadTask);
        }
        loadFguiIcon(pkgName, iconName, path = UrlUtils.UI, handler = null, isRecover = false, level = LoadLevel.DEFAULT) {
            this.loadFguiPkg(pkgName, path, Laya.Handler.create(this, (pkgName, iconName, handler, res) => {
                res.loadResName(iconName, handler);
            }, [pkgName, iconName, handler]), isRecover, level);
        }
        loadFguiPkg(pkgName, path = UrlUtils.UI, handler = null, isRecover = false, level = LoadLevel.DEFAULT) {
            if (fgui.UIPackage.getByName(pkgName)) {
                let res = AssetCache.assetDic[path + pkgName];
                handler && handler.runWith(res);
                return;
            }
            var loadTask = ObjectPools.Inst.getObj(GroupSheetLoadTask.KEY, GroupSheetLoadTask);
            loadTask.handlers.push(handler);
            loadTask.pkgName = pkgName;
            loadTask.url = path + pkgName;
            loadTask.level = level;
            loadTask.isRecover = isRecover;
            this.addTask(loadTask);
        }
        loadUI(url, handler = null, isRecover = true, level = LoadLevel.UI_MODEL) {
            if (this.checkCache(url, handler)) {
                return true;
            }
            if (this.checkQueue(url, handler)) {
                return false;
            }
            var loadTask = ObjectPools.Inst.getObj(UILoadTask.KEY, UILoadTask);
            loadTask.handlers.push(handler);
            loadTask.url = url;
            loadTask.isRecover = isRecover;
            loadTask.level = level;
            this.addTask(loadTask);
        }
        loadSpine(url, handler = null, isRecover = false, level = LoadLevel.UI_MODEL) {
            if (this.checkCache(url, handler)) {
                return true;
            }
            if (this.checkQueue(url, handler)) {
                return false;
            }
            var loadTask = ObjectPools.Inst.getObj(SpineLoadTask.KEY, SpineLoadTask);
            loadTask.handlers.push(handler);
            loadTask.url = url;
            loadTask.level = level;
            loadTask.isRecover = isRecover;
            this.addTask(loadTask);
        }
        loadParticle(url, handler = null, level = LoadLevel.DEFAULT) {
            if (this.checkCache(url, handler)) {
                return true;
            }
            if (this.checkQueue(url, handler)) {
                return false;
            }
            var loadTask = ObjectPools.Inst.getObj(ParticleLoadTask.KEY, ParticleLoadTask);
            loadTask.handlers.push(handler);
            loadTask.url = url;
            loadTask.level = level;
            this.addTask(loadTask);
        }
        loadSprite3D(url, handler = null, level = LoadLevel.DEFAULT) {
            if (this.checkCache(url, handler)) {
                return true;
            }
            if (this.checkQueue(url, handler)) {
                return false;
            }
            var loadTask = ObjectPools.Inst.getObj(Sprite3DLoadTask.KEY, Sprite3DLoadTask);
            loadTask.handlers.push(handler);
            loadTask.url = url;
            loadTask.level = level;
            this.addTask(loadTask);
        }
        loadScene3D(url, handler = null, level = LoadLevel.SCENE) {
            if (this.checkCache(url, handler)) {
                return true;
            }
            if (this.checkQueue(url, handler)) {
                return false;
            }
            var loadTask = ObjectPools.Inst.getObj(Scene3DLoadTask.KEY, Scene3DLoadTask);
            loadTask.handlers.push(handler);
            loadTask.url = url;
            loadTask.level = level;
            this.addTask(loadTask);
        }
        loadBin(url, handler = null, level = LoadLevel.DEFAULT) {
            if (this.checkCache(url, handler)) {
                return true;
            }
            if (this.checkQueue(url, handler)) {
                return false;
            }
            var loadTask = ObjectPools.Inst.getObj(ByteLoadTask.KEY, ByteLoadTask);
            loadTask.handlers.push(handler);
            loadTask.url = url;
            loadTask.level = level;
            this.addTask(loadTask);
        }
        loadAnimationClip(url, handler = null, isRecover = true, level = LoadLevel.DEFAULT) {
            if (this.checkCache(url, handler)) {
                return true;
            }
            if (this.checkQueue(url, handler)) {
                return false;
            }
            var loadTask = ObjectPools.Inst.getObj(AnimationClipLoadTask.KEY, AnimationClipLoadTask);
            loadTask.handlers.push(handler);
            loadTask.url = url;
            loadTask.level = level;
            loadTask.isRecover = isRecover;
            this.addTask(loadTask);
        }
        loadNativeAd(nativeId, handler = null, ignoreCache = false, level = LoadLevel.DEFAULT) {
            let url = "native_" + nativeId;
            if (this.checkCache(url, handler) && !ignoreCache) {
                return true;
            }
            if (this.checkQueue(url, handler)) {
                return false;
            }
            var loadTask = ObjectPools.Inst.getObj(NativeAdLoadTask.KEY, NativeAdLoadTask);
            loadTask.handlers.push(handler);
            loadTask.nativeId = nativeId;
            loadTask.url = url;
            loadTask.level = level;
            this.addTask(loadTask);
        }
        checkCache(url, handler) {
            var cache = AssetCache.assetDic[url];
            if (cache != null) {
                this.byteTotal++;
                this.byteLoaded++;
                if (handler != null) {
                    handler.runWith(cache);
                }
                return true;
            }
            else {
                return false;
            }
        }
        checkQueue(url, handler) {
            var task;
            var leng = this.currentActions.length;
            for (var i = 0; i < leng; i++) {
                task = this.currentActions[i];
                if (task.url == url) {
                    if (handler != null) {
                        task.handlers.push(handler);
                    }
                    return true;
                }
            }
            return false;
        }
        addTask(value) {
            this.byteTotal++;
            Logger.log(this, "加载数量：    " + this.byteTotal + "" + value.url);
            var leng = this.currentActions.length;
            if (leng == 0) {
                Laya.timer.frameLoop(1, this, this.advanceTime);
            }
            this.currentActions.push(value);
            value.run();
        }
        advanceTime() {
            if (this.isEmpty) {
                Laya.timer.clear(this, this.advanceTime);
                return;
            }
            var self = this;
            var task;
            var leng = self.currentActions.length;
            for (var i = 0; i < leng; i++) {
                task = self.currentActions[i];
                task.onUpdate();
                if (task.state == LoadTask.STATE_COMPLETE) {
                    self.currentActions.splice(i, 1);
                    leng--;
                    i--;
                    self.byteLoaded++;
                    task.GC();
                }
            }
        }
        GetRate() {
            var rate = 0;
            var self = this;
            var task;
            self.curByteLoad = self.byteLoaded;
            var leng = self.currentActions.length;
            for (var i = 0; i < leng; i++) {
                task = self.currentActions[i];
                self.curByteLoad += task.rate;
            }
            if (self.byteTotal == 0)
                return 0;
            return self.curByteLoad / self.byteTotal;
        }
        get isEmpty() {
            return this.currentActions.length == 0;
        }
        clearRate() {
            this.curByteLoad = 0;
            this.byteLoaded = 0;
            this.byteTotal = this.currentActions.length;
        }
        set curByteLoad(value) {
            this._curByteLoad = value;
        }
        get curByteLoad() {
            return this._curByteLoad;
        }
        set byteLoaded(value) {
            this._byteLoaded = value;
        }
        get byteLoaded() {
            return this._byteLoaded;
        }
    }

    class SoundMgr {
        constructor() {
            this._isMusicPlay = true;
            this._isSoundPlay = true;
            this._bgUrl = null;
            this.index = 0;
            Laya.SoundManager.autoReleaseSound = false;
            this.Reset();
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new SoundMgr();
            return this._Inst;
        }
        playSound(url, loops = 1, complete = null, soundClass = null, startTime = 0) {
            if (!this._isSoundPlay)
                return;
            Laya.SoundManager.playSound(url, loops, complete, soundClass, startTime);
        }
        playMusic(url, loops = 0, complete = null, startTime = 0) {
            if (this._bgUrl == url)
                return;
            this.playBackMusic(url, loops, complete, startTime);
        }
        playBackMusic(url, loops = 0, complete = null, startTime = 0) {
            this._bgUrl = url;
            if (!this._isMusicPlay)
                return;
            if (Laya.Browser.onVVMiniGame) {
                if (this.innerAudioContext != null)
                    this.innerAudioContext.stop();
                this.innerAudioContext = qg.createInnerAudioContext();
                this.innerAudioContext.src = url;
                this.innerAudioContext.loop = loops == 0;
                this.innerAudioContext.volume = 1;
                this.innerAudioContext.play();
            }
            else {
                LoadQueue.Inst.loadSound(url, Laya.Handler.create(null, function () {
                    Laya.SoundManager.playMusic(url, loops, complete, startTime);
                }));
            }
        }
        set isMusicPlay(value) {
            this._isMusicPlay = value;
            if (!value) {
                Laya.SoundManager.stopMusic();
                if (this.innerAudioContext != null) {
                    this.innerAudioContext.stop();
                }
            }
            else {
                if (this._bgUrl != null) {
                    this.playBackMusic(this._bgUrl);
                }
            }
        }
        set isSoundPlay(value) {
            this._isSoundPlay = value;
        }
        stop() {
            this.isMusicPlay = false;
            this.isSoundPlay = false;
        }
        Reset() {
            this.isMusicPlay = StorageUtils.Inst.GetMusic() == 1;
            this.isSoundPlay = StorageUtils.Inst.GetSound() == 1;
        }
    }
    SoundMgr._Inst = null;

    class EffectUtil {
        constructor() { }
        static listEffect(list, time = 200, delayTime = 100, extTime = 0) {
            let item = null;
            for (let i = 0; i < list.numChildren; i++) {
                item = list.getChildAt(i);
                item.alpha = 0;
                Laya.Tween.clearTween(item);
                Laya.Tween.to(item, { "alpha": 1 }, time, null, null, extTime + i * delayTime);
            }
        }
        static openWindowEffect(window, complete = null, thisObj = null, params = null, isMust = false) {
            fgui.GTween.kill(window);
            window.scaleX = window.scaleY = 0.3;
            window.alpha = 1;
            let w = window.viewWidth;
            let h = window.viewHeight;
            window.x = (w - window.scaleX * w) * 0.5;
            window.y = (h - window.scaleY * h) * 0.5;
            let _self = null;
            fgui.GTween.to2(0.3, 0.3, 1.05, 1.05, 0.3)
                .setTarget(window, _self)
                .onUpdate(onUpdate, _self)
                .onComplete(function onComplete() {
                fgui.GTween.to2(1.05, 1.05, 0.98, 0.98, 0.3).setTarget(window, _self).onUpdate(onUpdate, _self).onComplete(function onComplete1() {
                    fgui.GTween.to2(0.98, 0.98, 1, 1, 0.2).setTarget(window, _self).onUpdate(onUpdate, _self).onComplete(function onComplete2() {
                        fgui.GTween.kill(window);
                        if (complete != null) {
                            complete.apply(thisObj, params);
                        }
                    }, _self);
                }, _self);
            }, _self);
            function onUpdate(tw) {
                window.scaleX = tw.value.x;
                window.scaleY = tw.value.y;
                window.x = (w - window.scaleX * w) * 0.5;
                window.y = (h - window.scaleY * h) * 0.5;
            }
        }
        static closeWindowEffect(window, complete = null, thisObj = null, params = null, isMust = false) {
            if (window) {
                if (!isMust && window["isWindowEffect"]) {
                    return;
                }
                else if (isMust) {
                    fgui.GTween.kill(window);
                }
                window.alpha = 1;
                window.scaleX = window.scaleY = 1;
                let w = window.width;
                let h = window.height;
                fgui.GTween.to3(1, 1, 1, 0.8, 0.8, 0, 0.3).setTarget(window, this)
                    .onUpdate(function onUpdate(tw) {
                    window.scaleX = tw.value.x;
                    window.scaleY = tw.value.y;
                    window.alpha = tw.value.z;
                    window.x = (Laya.Browser.width - window.scaleX * w) * 0.5;
                    window.y = (Laya.Browser.height - window.scaleY * h) * 0.5;
                }, this)
                    .onComplete(function onComplete() {
                    fgui.GTween.kill(window);
                    if (complete != null) {
                        complete.apply(thisObj, params);
                    }
                }, this);
            }
        }
        static showListEffect(list, gapTime = 0.3, delatTime = 0, complete = null, easeType = fgui.EaseType.SineOut) {
            let self = this;
            if (list) {
                let len = list.length;
                for (let i = 0; i < len; i++) {
                    let item = list[i];
                    item.alpha = 0;
                    item.setScale(0, 0);
                    showEffect(i, len - 1);
                }
            }
            else {
                complete && complete.run();
            }
            function showEffect(j, maxIndex) {
                let item = list[j];
                fgui.GTween.to(0, 1, gapTime).setTarget(item).setEase(easeType).setDelay(delatTime * j)
                    .onUpdate((tw) => {
                    tw.target.alpha = tw.value.x;
                    tw.target.setScale(tw.value.x, tw.value.x);
                }, self)
                    .onComplete((tw) => {
                    if (j == maxIndex) {
                        complete && complete.run();
                    }
                }, self);
            }
        }
    }

    class ObjectUtils {
        constructor() {
        }
        static findChild(sp, name, ...args) {
            if (sp == null) {
                return null;
            }
            if (sp.name == name)
                return sp;
            else
                return this._findChild(sp._children, name, args);
        }
        static _findChild(spArr, name, args) {
            var arr = [];
            for (var i = 0; i < spArr.length; i++) {
                var child = spArr[i];
                if (!child.active)
                    continue;
                if (child.name == name || args.indexOf(child.name) != -1) {
                    return child;
                }
                else if (child.numChildren) {
                    arr = arr.concat(child._children);
                }
            }
            if (!arr.length)
                return null;
            return this._findChild(arr, name, args);
        }
        static instanceOfIComponet(obj) {
            return obj.discriminator == "I-AM-IComponent";
        }
        static getParentOf(cls, self = null) {
            let parent = self ? self.parent : null;
            let p = null;
            if (parent != null) {
                if (parent instanceof cls) {
                    return parent;
                }
                else {
                    return this.getParentOf(cls, parent);
                }
            }
            return null;
        }
        static globalIsIn(target, gp = null) {
            if (gp == null) {
                gp = new Laya.Point(Laya.stage.mouseX, Laya.stage.mouseY);
            }
            if (target == null || target.parent == null) {
                return false;
            }
            let targetGP = target.parent.localToGlobal(target.x, target.y);
            if (target.pivotAsAnchor) {
                let hw = target.width * 0.5;
                let hh = target.height * 0.5;
                if (gp.x >= targetGP.x - hw && gp.x <= (targetGP.x + hw) && gp.y >= targetGP.y - hh && gp.y <= (targetGP.y + hh)) {
                    return true;
                }
            }
            else {
                if (gp.x >= targetGP.x && gp.x <= (targetGP.x + target.width) && gp.y >= targetGP.y && gp.y <= (targetGP.y + target.height)) {
                    return true;
                }
            }
            return false;
        }
        static globalIsInRange(gp = null, range = null, target = null) {
            gp = gp || new Laya.Point(Laya.stage.mouseX, Laya.stage.mouseY);
            if (target && range == null) {
                let targetGP = target.localToGlobal(0, 0);
                range = range || this.range;
                if (target.pivotAsAnchor) {
                    let hw = target.width * 0.5;
                    let hh = target.height * 0.5;
                    range.x = targetGP.x - hw;
                    range.y = targetGP.y - hh;
                    range.width = target.width;
                    range.height = target.height;
                }
                else {
                    range.x = targetGP.x;
                    range.y = targetGP.y;
                    range.width = target.width;
                    range.height = target.height;
                }
            }
            return range && range.contains(gp.x, gp.y);
        }
        static getScreenTexture() {
            try {
                window["Config"].preserveDrawingBuffer = true;
                let _texture = null;
                let gl = Laya.WebGLContext.mainContext;
                let pixels = new Uint8Array(gl.drawingBufferWidth * gl.drawingBufferHeight * 4);
                gl.readPixels(0, 0, gl.drawingBufferWidth, gl.drawingBufferHeight, gl.RGBA, gl.UNSIGNED_BYTE, pixels);
                let w = gl.drawingBufferWidth;
                let h = gl.drawingBufferHeight;
                let texture2d = new Laya.Texture2D(w, h, Laya.TextureFormat.R8G8B8A8, false, false);
                texture2d.setPixels(pixels);
                _texture = new Laya.Texture(texture2d);
                window["Config"].preserveDrawingBuffer = false;
                return _texture;
            }
            catch (error) {
            }
            return null;
        }
    }
    ObjectUtils.range = new Laya.Rectangle();

    class EventObj {
        constructor(type, listener, thisObject, target) {
            this.type = type;
            this.listener = listener;
            this.thisObject = thisObject;
            this.target = target;
        }
        static create(type, listener, thisObject, target) {
            let eo = ObjectPools.Inst.getObj(EventObj.KEY, EventObj);
            eo.type = type;
            eo.listener = listener;
            eo.thisObject = thisObject;
            eo.target = target;
            return eo;
        }
        static recover(eo) {
            ObjectPools.Inst.gcObj(EventObj.KEY, eo);
        }
        addListen() {
            if (!this.type) {
                return;
            }
            if (this.target) {
                this.target.off(this.type, this.thisObject, this.listener);
                this.target.on(this.type, this.thisObject, this.listener);
            }
            else {
                GameDispatcher.Inst.off(this.type, this.thisObject, this.listener);
                GameDispatcher.Inst.on(this.type, this.thisObject, this.listener);
            }
        }
        removeListen() {
            if (!this.type) {
                return;
            }
            if (this.target) {
                this.target.off(this.type, this.thisObject, this.listener);
            }
            else {
                GameDispatcher.Inst.off(this.type, this.thisObject, this.listener);
            }
        }
        Reset() {
            this.removeListen();
            this.type = null;
            this.listener = null;
            this.thisObject = null;
            this.target = null;
        }
        GC() {
            EventObj.recover(this);
        }
    }
    EventObj.KEY = "ObjectPool.EventObj";

    class EventPool {
        constructor() {
            this.pool = null;
            this.pool = new Array();
        }
        AddListenerInPool(type, listener, thisObject, target) {
            this.addListener(type, listener, thisObject, target);
        }
        RemoveListenerFromPool(type, listener, thisObject, target) {
            let eObj = this.getEventObj(type, listener, target, thisObject);
            if (eObj) {
                eObj.removeListen();
            }
        }
        ClearListenerFromPool() {
            this.removeAllListener();
        }
        addListener(type, listener, thisObject, target) {
            let eObj = this.getEventObj(type, listener, target, thisObject);
            if (eObj == null) {
                eObj = EventObj.create(type, listener, thisObject, target);
                this.pool.push(eObj);
            }
            eObj.addListen();
        }
        removeAllListener() {
            let len = this.pool.length;
            let eObj;
            for (let i = 0; i < len; i++) {
                eObj = this.pool[i];
                eObj.removeListen();
            }
        }
        getEventObj(type, listener, target, thisObject) {
            let eObj = null;
            for (eObj of this.pool) {
                if (eObj && eObj.type == type && eObj.listener == listener) {
                    if (target && eObj.target != target) {
                        return null;
                    }
                    else if (thisObject && eObj.thisObject != thisObject) {
                        return null;
                    }
                    else {
                        return eObj;
                    }
                }
            }
            return null;
        }
        hasEventListener(type, listener, target, thisObject) {
            let obj = this.getEventObj(type, listener, target, thisObject);
            return obj != null;
        }
        Reset() {
            if (this.pool != null) {
                let eObj;
                for (eObj of this.pool) {
                    if (eObj) {
                        eObj.GC();
                    }
                }
                this.pool.length = 0;
            }
        }
    }

    class TComponent extends fgui.GComponent {
        constructor() {
            super();
            this.discriminator = "I-AM-IComponent";
            this.m_eventPool = null;
            this.m_param = null;
            this.m_eventPool = new EventPool();
        }
        constructFromXML(xml) {
            super.constructFromXML(xml);
            this.InitFairyGui();
            this.InitUI();
        }
        InitFairyGui() {
        }
        InitUI() {
        }
        InitData(param) {
        }
        get Data() {
            return this.m_param;
        }
        AddListener() {
            TComponent.CheckComponent(this, "AddListener");
        }
        RemoveListener() {
            TComponent.CheckComponent(this, "RemoveListener");
        }
        AddGameListener(type, listener, thisObject, target) {
            this.m_eventPool.AddListenerInPool(type, listener, thisObject, target);
        }
        RemoveGameListener(type, listener, thisObject, target) {
            this.m_eventPool.RemoveListenerFromPool(type, listener, thisObject, target);
        }
        Reset() {
            TComponent.CheckComponent(this, "Reset");
            if (this.m_eventPool != null) {
                this.m_eventPool.Reset();
            }
        }
        static CheckComponent(parent, fex, param = null) {
            if (parent == null || !fex)
                return;
            let child;
            let numChildren = parent.numChildren;
            for (let i = 0; i < numChildren; i++) {
                child = parent.getChildAt(i);
                if (child instanceof fgui.GList) {
                    TComponent.CheckEGlistCompoent(child, fex, param);
                }
                if (TComponent.instanceOfIComponet(child)) {
                    param != null ? child[fex](param) : child[fex]();
                }
                else if (child.numChildren > 0) {
                    TComponent.CheckComponent(child, fex, param);
                }
            }
        }
        static CheckEGlistCompoent(child, fex, param = null) {
            let eglist = child[EGList.EGLIST];
            if (eglist != null) {
                param != null ? eglist[fex](param) : eglist[fex]();
            }
        }
        static instanceOfIComponet(obj) {
            return obj.discriminator == "I-AM-IComponent";
        }
        Destroy() {
            TComponent.CheckComponent(this, "Destroy");
            if (this.m_eventPool) {
                this.m_eventPool.Reset();
                this.m_eventPool = null;
            }
            this.m_param = null;
        }
    }

    class EGList extends Laya.EventDispatcher {
        constructor(list, thisObject = null) {
            super();
            this.discriminator = "I-AM-IComponent";
            this.list = null;
            this._data = null;
            this._thisObject = null;
            this._itemRenderer = null;
            this.isAutoSize = false;
            this.isAutoScroll = false;
            this.autoHScrollSpeed = 1;
            this.autoVScrollSpeed = 1;
            this.autoVScrollInitY = 0;
            this._preSelectedIndex = -1;
            this.stageMouseDown = false;
            this.doScrollEffect = false;
            this.list = list;
            this._thisObject = thisObject;
            this._data = [];
            if (this.list != null) {
                this._thisObject = thisObject || this;
                this.list.itemRenderer = new Laya.Handler(this, this.listItemRender, null, false);
                if (this.list.scrollPane) {
                    this.list.setVirtual();
                }
            }
        }
        listItemRender(index, item) {
            if (this.Data) {
                item.InitData(this.Data[index]);
            }
        }
        get isSingle() {
            return this.list.selectionMode == fgui.ListSelectionMode.Single;
        }
        set selectionMode(val) {
            this.list.selectionMode = val;
        }
        get selectionMode() {
            return this.list.selectionMode;
        }
        clickItem(target) {
            if (this.isSingle && this._preSelectedIndex != this.list.selectedIndex) {
                this.selectItem(target);
            }
            else if (!this.isSingle) {
                this.selectItem(target);
            }
            this._preSelectedIndex = this.list.selectedIndex;
        }
        selectItem(item) {
            this.event(fgui.Events.CLICK_ITEM, item);
        }
        set itemUrl(value) {
            this.list.defaultItem = value;
        }
        get itemUrl() {
            return this.list.defaultItem;
        }
        set numItems(value) {
            this.list.numItems = value;
        }
        get numItems() {
            return this.list.numItems;
        }
        set data(value) {
            this.list.data = value;
        }
        get data() {
            return this.list.data;
        }
        set Data(value) {
            TComponent.CheckComponent(this.list, "Reset");
            this._data = value || [];
            this.list.numItems = this._data.length;
            this.setAutoSize();
        }
        get Data() {
            return this._data;
        }
        getSelection(result) {
            return this.list.getSelection(result);
        }
        setArray(arr, doScrollEffect = false, scale = 0.5) {
            this.doScrollEffect = doScrollEffect;
            if (doScrollEffect) {
                this.list.setVirtualAndLoop();
            }
            this.Data = arr;
            if (this.isSingle) {
                this.scrollToTop();
                this._preSelectedIndex = -1;
                this.selectedIndex = 0;
            }
            if (doScrollEffect) {
                this.list.off(fgui.Events.SCROLL, this, this.doSpecialEffect);
                this.list.on(fgui.Events.SCROLL, this, this.doSpecialEffect, [scale]);
                this.doSpecialEffect(scale);
            }
        }
        doSpecialEffect(scale = 0.5) {
            var midX = this.list.scrollPane.posX + this.list.viewWidth / 2;
            var cnt = this.list.numChildren;
            for (var i = 0; i < cnt; i++) {
                var obj = this.list.getChildAt(i);
                var dist = Math.abs(midX - obj.x - obj.width / 2);
                if (dist > obj.width)
                    obj.setScale(1, 1);
                else {
                    var ss = 1 + (1 - dist / obj.width) * scale;
                    obj.setScale(ss, ss);
                }
            }
            this._preSelectedIndex = -1;
            let index = (this.list.getFirstChildInView() + 1) % this.list.numItems;
            this.selectedIndex = index;
            this.event(fgui.Events.SCROLL);
        }
        setPosX(value, ani) {
            if (this.list && this.list.scrollPane) {
                this.list.scrollPane.setPosX(value, ani);
            }
        }
        canelSelect() {
            this._preSelectedIndex = -1;
            this.list.clearSelection();
        }
        setAutoSize() {
            if (this.isAutoSize) {
                let w = 0;
                let h = 0;
                let item = null;
                if (this.isHScroll) {
                    for (let i = 0; i < this.list.numChildren; i++) {
                        item = this.list.getChildAt(i);
                        if (item != null) {
                            w += item.width;
                        }
                    }
                    w += this._data.length > 1 ? (this._data.length - 1) * this.list.columnGap : 0;
                    this.list.setSize(w, this.list.height);
                }
                else if (this.isVScroll) {
                    for (let i = 0; i < this.list.numChildren; i++) {
                        item = this.list.getChildAt(i);
                        if (item != null) {
                            h += item.height;
                        }
                    }
                    h += this._data.length > 1 ? (this._data.length - 1) * this.list.lineGap : 0;
                    this.list.setSize(this.list.width, h);
                }
            }
            if (this.isAutoScroll) {
                if (this.isHScroll) {
                    let gap = this.contentWidth - this.list.viewWidth;
                    if (gap > 0) {
                        Laya.timer.frameLoop(1, this, this.loopHScroll);
                    }
                }
                else if (this.isVScroll) {
                    let gap = this.contentHeight - this.list.viewHeight;
                    if (gap > 0) {
                        Laya.timer.frameLoop(1, this, this.loopVScroll);
                    }
                }
            }
        }
        loopVScroll() {
            if (ObjectUtils.globalIsIn(this.list) && this.stageMouseDown) {
                return;
            }
            if (this.isScrollTween) {
                return;
            }
            this.list.scrollPane.posY += this.autoVScrollSpeed;
        }
        loopHScroll() {
            if (ObjectUtils.globalIsIn(this.list) && this.stageMouseDown) {
                return;
            }
            if (this.isScrollTween) {
                return;
            }
            this.list.scrollPane.posX += this.autoHScrollSpeed;
        }
        setSize(w, h, ignorePivot) {
            this.list.setSize(w, h, ignorePivot);
        }
        getAllWidth(singleW = 60, isVirtual = true) {
            if (this.isHScroll) {
                return this.list.height;
            }
            else if (isVirtual) {
                let w = 0;
                let item = null;
                for (let i = 0; i < this.list.numChildren; i++) {
                    item = this.list.getChildAt(i);
                    if (item != null) {
                        w += item.width;
                    }
                }
                w += this._data.length > 1 ? (this._data.length - 1) * this.list.columnGap : 0;
                return w;
            }
            else {
                let item = this.list.getChildAt(0);
                let sw = item ? item.width : singleW;
                let w = sw * this._data.length;
                w += this._data.length > 1 ? (this._data.length - 1) * this.list.columnGap : 0;
                return w;
            }
        }
        getAllHeight(singleH = 60, isVirtual = true) {
            if (this.isHScroll) {
                return this.list.height;
            }
            else if (isVirtual) {
                let h = 0;
                let item = null;
                for (let i = 0; i < this.list.numChildren; i++) {
                    item = this.list.getChildAt(i);
                    if (item != null) {
                        h += item.height;
                    }
                }
                h += this._data.length > 1 ? (this._data.length - 1) * this.list.lineGap : 0;
                return h;
            }
            else {
                let item = this.list.getChildAt(0);
                let sh = item ? item.height : singleH;
                let h = sh * this._data.length;
                h += this._data.length > 1 ? (this._data.length - 1) * this.list.lineGap : 0;
                return h;
            }
        }
        mouseDown() {
            this.stageMouseDown = true;
        }
        mouseUp() {
            this.stageMouseDown = false;
        }
        listScrollEnd() {
            this.mouseUp();
            this.event(fgui.Events.SCROLL_END);
        }
        get displayObject() {
            return this.list;
        }
        InitUI() {
        }
        InitData(data) {
            this._data = data;
        }
        get numChildren() {
            return this.list && this.list.numChildren;
        }
        getChildAt(index) {
            return this.list && this.list.getChildAt(index);
        }
        get items() {
            let arr = [];
            for (let i = 0; i < this.list.numChildren; i++) {
                arr.push(this.list.getChildAt(i));
            }
            return arr;
        }
        set visible(value) {
            this.list.visible = value;
        }
        get visible() {
            return this.list.visible;
        }
        get scrollPane() {
            return this.list.scrollPane;
        }
        killTween() {
            return this.scrollPane && this.scrollPane["killTween"]();
        }
        get isScrollTween() {
            return this.scrollPane && this.scrollPane["_tweening"] != 0;
        }
        get viewWidth() {
            return this.list.viewWidth;
        }
        get viewHeight() {
            return this.list.viewHeight;
        }
        setScale(sx, sy) {
            this.list && this.list.setScale(sx, sy);
        }
        setVirtual() {
            this.list.setVirtual();
        }
        setVirtualAndLoop() {
            this.list["_virtual"] = false;
            this.list.setVirtualAndLoop();
        }
        get scrollType() {
            return this.list && this.list.scrollPane && this.list.scrollPane["_scrollType"];
        }
        get isVScroll() {
            return this.list.layout == fgui.ListLayoutType.SingleColumn || this.scrollType == fgui.ScrollType.Vertical;
        }
        get isHScroll() {
            return this.list.layout == fgui.ListLayoutType.SingleRow || this.scrollType == fgui.ScrollType.Horizontal;
        }
        set isTouchScroll(value) {
            if (this.list && this.list.scrollPane) {
                this.list.scrollPane.touchEffect = value;
            }
        }
        get isTouchScroll() {
            return this.list && this.list.scrollPane && this.list.scrollPane.touchEffect;
        }
        get isBoth() {
            return this.scrollType == fgui.ScrollType.Both;
        }
        get contentWidth() {
            return this.list.scrollPane ? this.list.scrollPane.contentWidth : this.list.width;
        }
        get contentHeight() {
            return this.list.scrollPane ? this.list.scrollPane.contentHeight : this.list.height;
        }
        get width() {
            return this.list ? this.list.width : 0;
        }
        get height() {
            return this.list ? this.list.height : 0;
        }
        set height(val) {
            if (this.list) {
                this.list.height = val;
            }
        }
        set x(value) {
            this.list.x = value;
        }
        get x() {
            return this.list.x;
        }
        set y(value) {
            this.list.y = value;
        }
        get y() {
            return this.list.y;
        }
        setInitSelectedIndex() {
            this.list.selectedIndex = 0;
            this._preSelectedIndex = -1;
        }
        set listSelectedIndex(value) {
            this.list.selectedIndex = value;
        }
        set selectedIndex(value) {
            this.list.selectedIndex = value;
            let item = null;
            for (let i = 0; i < this.list.numChildren; i++) {
                item = this.list.getChildAt(i);
                if (this._data && item.Data == this._data[value]) {
                    this.clickItem(item);
                    break;
                }
            }
            this._preSelectedIndex = value;
        }
        get selectedIndex() {
            if (this.list.selectedIndex == -1) {
                return 0;
            }
            return this.list.selectedIndex;
        }
        get selectData() {
            if (this._preSelectedIndex == -1) {
                return null;
            }
            return this._data && this._data[this._preSelectedIndex];
        }
        toPrev() {
            if (this.list.scrollPane) {
                this.list.scrollPane.scrollLeft(1, true);
            }
        }
        toNext() {
            if (this.list.scrollPane) {
                this.list.scrollPane.scrollRight(1, true);
            }
        }
        scrollToView(index, ani = false, setFirst = false) {
            this.list.scrollToView(index, ani, setFirst);
        }
        set scrollEnabled(value) {
            if (this.list.scrollPane) {
                this.list.scrollPane.touchEffect = value;
            }
        }
        get scrollEnabled() {
            return this.list.scrollPane && this.list.scrollPane.touchEffect;
        }
        scrollLeft(ratio, ani) {
            if (this.list.scrollPane != null) {
                this.list.scrollPane.scrollLeft(ratio, ani);
            }
        }
        scrollRight(ratio, ani) {
            if (this.list.scrollPane != null) {
                this.list.scrollPane.scrollRight(ratio, ani);
            }
        }
        scrollToTop(ani) {
            if (this.list.scrollPane != null) {
                this.list.scrollPane.scrollTop(ani);
            }
        }
        scrollToBottom(ani) {
            if (this.list.scrollPane != null) {
                this.list.scrollPane.scrollBottom(ani);
            }
        }
        scrollUp(step = 1) {
            this.list.scrollPane.scrollStep = step;
            if (this.list.scrollPane != null) {
                this.list.scrollPane.scrollUp();
            }
        }
        scrollDown(step = 1) {
            this.list.scrollPane.scrollStep = step;
            if (this.list.scrollPane != null) {
                this.list.scrollPane.scrollDown();
            }
        }
        set lineCount(value) {
            this.list.lineCount = value;
        }
        get lineCount() {
            return this.list.lineCount;
        }
        set columnCount(value) {
            this.list.columnCount = value;
        }
        get columnCount() {
            return this.list.columnCount;
        }
        set lineGap(value) {
            this.list.lineGap = value;
        }
        get lineGap() {
            return this.list.lineGap;
        }
        set columnGap(value) {
            this.list.columnGap = value;
        }
        get columnGap() {
            return this.list.columnGap;
        }
        localToGlobal(ax, ay) {
            return this.list.localToGlobal(ax, ay);
        }
        AddListener() {
            Laya.timer.clear(this, this.loopVScroll);
            Laya.timer.clear(this, this.loopHScroll);
            fgui.GTween.kill(this);
            this.list.on(fgui.Events.CLICK_ITEM, this, this.clickItem);
            this.list.on(fgui.Events.SCROLL_END, this, this.listScrollEnd);
            if (this.isAutoScroll) {
                Laya.stage.on(Laya.Event.MOUSE_DOWN, this, this.mouseDown);
                Laya.stage.on(Laya.Event.MOUSE_UP, this, this.mouseUp);
            }
        }
        RemoveListener() {
            this.list.off(fgui.Events.CLICK_ITEM, this, this.clickItem);
            this.list.off(fgui.Events.SCROLL_END, this, this.listScrollEnd);
            this.list.off(fgui.Events.SCROLL, this, this.doSpecialEffect);
            Laya.stage.off(Laya.Event.MOUSE_DOWN, this, this.mouseDown);
            Laya.stage.off(Laya.Event.MOUSE_UP, this, this.mouseUp);
        }
        Reset() {
            fgui.GTween.kill(this);
            Laya.timer.clear(this, this.loopVScroll);
            Laya.timer.clear(this, this.loopHScroll);
            this._data = null;
            this._preSelectedIndex = -1;
        }
        Destroy() {
        }
    }
    EGList.EGLIST = "EGLIST";

    class FairyUtils {
        constructor() { }
        static setVar(parent, thisObject, btns = null) {
            if (!parent)
                return;
            if (parent != null && thisObject != null) {
                for (let i = 0; i < parent.numChildren; i++) {
                    let disObj;
                    disObj = parent.getChildAt(i);
                    if (disObj == null || disObj.name == "icon" || disObj.name == "title") {
                        continue;
                    }
                    thisObject[disObj.name] = disObj;
                    if (disObj instanceof fgui.GTextField && disObj.font) {
                    }
                    else if (disObj.name && disObj.name.indexOf("eglist_") == 0 && disObj instanceof fgui.GList) {
                        let eglist = new EGList(disObj, thisObject);
                        thisObject[disObj.name] = eglist;
                        disObj[EGList.EGLIST] = eglist;
                    }
                    if (btns != null && disObj instanceof fgui.GButton) {
                        btns.push(disObj);
                    }
                    if (btns != null && disObj instanceof fgui.GComponent)
                        this.setVar(disObj, null, btns);
                }
            }
        }
        static targetIsAllVisible(target) {
            return target && target.visible && target.internalVisible && target.internalVisible2 && target.internalVisible3;
        }
    }

    class View extends TComponent {
        constructor() {
            super();
            this.cls = null;
            this.gcTime = 0;
        }
        Init(param) {
            this.m_param = param;
            this.gcTime = Number.MAX_VALUE;
        }
        onResize() {
        }
        get Data() {
            return this.m_param;
        }
        Reset() {
            super.Reset();
            this.gcTime = Laya.timer.currTimer + Resource.CACHE_UI_TIME;
        }
        IsCanGc() {
            return Laya.timer.currTimer >= this.gcTime;
        }
        Destroy() {
            super.Destroy();
            this.cls = null;
        }
    }

    class ViewStruct {
        constructor() {
            this.layerType = LayerType.TYPE_WINDOW;
            this.layout = LayerType.LAYOUT_CENTER;
            this.isShowMaskBg = false;
            this.mashBgAlpha = 0.7;
            this.isShowImgBg = false;
            this.maskName = "sp_mask";
            this.isExclusion = true;
            this.isEffect = false;
            this.isCtrlTimerScale = false;
        }
    }

    class UIBaseWindow extends View {
        constructor(pkgName = "", resName = "", ui_res = UrlUtils.UI) {
            super();
            this._pkgName = "";
            this._resName = "";
            this._view = null;
            this.m_mask = null;
            this.m_imgBg = null;
            this.btn_close = null;
            this.viewStruct = null;
            this._btns = null;
            this._isLoaded = false;
            this._iRes = null;
            this.isReset = false;
            this.ui_res = null;
            this._isShowbanner = false;
            this._bannerData = null;
            this.name = pkgName;
            this._pkgName = pkgName;
            this._resName = resName;
            this.ui_res = ui_res;
            this._btns = new Array();
            this.SetViewStruct();
        }
        LoadAtlas() {
            this.visible = false;
            this.isReset = false;
            let url = this.ui_res + this._pkgName;
            if (this.isLoaded) {
                this.onDelay();
            }
            else {
                if (!fgui.UIPackage.getByName(this._pkgName))
                    this.showLoadingLeaf();
                LoadQueue.Inst.loadUI(url, Laya.Handler.create(this, this.onLoaded));
            }
        }
        showLoadingLeaf() {
            UISystem.showLoadingLeaf();
        }
        hideLoadingLeaf() {
            UISystem.hideLoadingLeaf();
        }
        onLoaded(iRes) {
            this.hideLoadingLeaf();
            if (iRes == null || this.isReset)
                return;
            this._iRes = iRes;
            let isSuccess = iRes.GetRes();
            if (!isSuccess)
                return;
            this.loadComplete();
        }
        loadComplete() {
            let self = this;
            self._isLoaded = true;
            let packageName = this.ui_res + self._pkgName;
            if (fgui.UIPackage.getByName(packageName) == null) {
                fgui.UIPackage.addPackage(packageName);
            }
            self.registerClasses();
            self._view = fgui.UIPackage.createObject(self._pkgName, self._resName).asCom;
            self.setSize(self._view.viewWidth, self._view.viewHeight);
            self.addChild(self._view);
            self._view.name = self._resName;
            FairyUtils.setVar(self._view, self, self._btns);
            self.InitUI();
            self.onDelay();
        }
        registerClasses() {
        }
        onDelay() {
            let self = this;
            LayerType.SetWindowLayer(self);
            self.visible = true;
            self.viewStruct.isShowMaskBg ? self.showMask() : self.hideMask();
            self.viewStruct.isShowImgBg ? self.showImgBg() : self.hideImgBg();
            self.onResize();
            self.AddListener();
            self.InitData(self.m_param);
            if (self.viewStruct.isEffect) {
                EffectUtil.openWindowEffect(self._view, this.onEffectEnd, this);
            }
            this.OpenWindow();
        }
        InitData(param) {
            this.m_param = param;
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.fx_UI_btn_Open));
            GameDispatcher.Inst.event(UIGameEvent.ON_SHOW, this.panelName);
        }
        onEffectEnd() {
        }
        OpenWindow() {
            if (this.viewStruct.isCtrlTimerScale) {
                GlobalConfig.isPlay = false;
            }
        }
        CloseWindow() {
            if (this.viewStruct.isCtrlTimerScale) {
                GlobalConfig.isPlay = true;
            }
        }
        get isLoaded() {
            return this._isLoaded;
        }
        Registered() {
        }
        InitUI() {
        }
        AddListener() {
            super.AddListener();
            for (let btn of this._btns) {
                btn.onClick(this, this.onBtnClick, [btn]);
            }
        }
        RemoveListener() {
            super.RemoveListener();
            for (let btn of this._btns) {
                btn.offClick(this, this.onBtnClick);
            }
        }
        onBtnClick(btn) {
            if (btn == this.btn_close) {
                this.onBtnCloseClick();
            }
            else {
                SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.Click_OK));
            }
        }
        onBtnCloseClick() {
            this.close();
        }
        close() {
            UISystem.Inst.RemoveWindowClass(this.cls);
        }
        SetViewStruct() {
            if (this.viewStruct == null) {
                this.viewStruct = new ViewStruct();
            }
        }
        get panelName() {
            return this._resName;
        }
        GetViewStruct() {
            return this.viewStruct;
        }
        onResize() {
            LayerType.SetWindowLayout(this);
            if (this.m_mask != null) {
                this.m_mask.viewWidth = GlobalConfig.viewWidth;
                this.m_mask.viewHeight = GlobalConfig.viewHeight;
                let img = this.m_mask.getChild("img").asImage;
                img.width = GlobalConfig.viewWidth;
                img.height = GlobalConfig.viewHeight;
                this.m_mask.x = -this.x;
                this.m_mask.y = -this.y;
            }
            if (this.m_imgBg != null) {
                this.m_imgBg.width = GlobalConfig.viewWidth;
                this.m_imgBg.height = GlobalConfig.viewHeight;
                this.m_imgBg.x = -this.x;
                this.m_imgBg.y = -this.y;
            }
        }
        showMask() {
            if (this.m_mask == null) {
                this.m_mask = fgui.UIPackage.createObject("common", this.viewStruct.maskName).asCom;
                this.addChildAt(this.m_mask, 0);
                this.m_mask.touchable = true;
                this.m_mask.alpha = this.viewStruct.mashBgAlpha;
            }
        }
        hideMask() {
            if (this.m_mask != null) {
                if (this.m_mask.parent != null)
                    this.m_mask.parent.removeChild(this.m_mask);
                this.m_mask = null;
            }
        }
        showImgBg() {
            if (this.m_imgBg == null) {
                let bg = fgui.UIPackage.createObject("common", "bg");
                if (bg) {
                    this.m_imgBg = bg.asImage;
                    this.addChildAt(this.m_imgBg, 0);
                }
                else {
                    console.error("====>bg is null panelName:" + this.panelName + " bg: " + bg);
                }
            }
            GameDispatcher.Inst.event(UIGameEvent.UI_SHOW_IMGBG);
        }
        hideImgBg() {
            if (this.m_imgBg != null) {
                if (this.m_imgBg.parent != null)
                    this.m_imgBg.parent.removeChild(this.m_mask);
                this.m_imgBg = null;
            }
        }
        set isThrough(value) {
            this._view.opaque = !value;
        }
        get isThrough() {
            return !this._view.opaque;
        }
        setSize(wv, hv, ignorePivot) {
            super.setSize(wv, hv, ignorePivot);
            if (this._view)
                this._view.setSize(wv, hv, ignorePivot);
        }
        Reset() {
            this.isReset = true;
            super.Reset();
            this.CloseWindow();
            this.hideBanner();
            if (this.viewStruct.isEffect) {
                fgui.GTween.kill(this._view);
            }
            if (this.viewStruct.isShowImgBg) {
                GameDispatcher.Inst.event(UIGameEvent.UI_HIDE_IMGBG);
            }
            this.m_param = null;
            GameDispatcher.Inst.event(UIGameEvent.ON_HIDE, this.panelName);
            Laya.timer.clearAll(this);
        }
        Destroy() {
            super.Destroy();
            this.hideMask();
            this.hideImgBg();
            if (this._iRes != null) {
                this._iRes.Dispose();
                this._iRes = null;
            }
            if (this._view) {
                this._view.dispose();
                this._view = null;
            }
            this._isLoaded = false;
            this._resName = "";
        }
        showBanner(data = null) {
            this._bannerData = data;
            this._isShowbanner = true;
            GameDispatcher.Inst.event(UIGameEvent.SHOW_BANNER, [this, this._bannerData]);
        }
        get isShowBanner() {
            return this._isShowbanner;
        }
        reshowBanner() {
            this.showBanner(this._bannerData);
        }
        hideBanner() {
            this._isShowbanner = false;
            this._bannerData = null;
            GameDispatcher.Inst.event(UIGameEvent.HIDE_BANNER, this);
        }
    }

    class AdType {
    }
    AdType.TYPE_Banner = 1;
    AdType.TYPE_RewardedVideo = 2;
    AdType.TYPE_Insert = 3;
    AdType.TYPE_NativeIcon = 4;
    AdType.TYPE_NativeBaner = 5;
    AdType.TYPE_Box = 6;
    AdType.TYPE_Block = 7;
    AdType.TYPE_NativeInsert = 8;
    AdType.ad_1 = 1;
    AdType.ad_2 = 2;
    AdType.ad_3 = 3;
    AdType.ad_4 = 4;
    AdType.ad_5 = 5;
    AdType.ad_6 = 6;
    AdType.ad_7 = 7;
    AdType.ad_8 = 8;
    AdType.ad_9 = 9;
    AdType.ad_10 = 10;
    AdType.ad_11 = 11;
    AdType.ad_12 = 12;
    AdType.ad_13 = 13;
    AdType.ad_14 = 14;
    AdType.ad_15 = 15;
    AdType.ad_16 = 16;
    AdType.ad_17 = 17;
    AdType.ad_18 = 18;
    AdType.ad_19 = 19;
    AdType.ad_20 = 20;
    AdType.ad_21 = 21;
    AdType.ad_22 = 22;
    AdType.ad_23 = 23;
    AdType.ad_24 = 24;
    AdType.ad_25 = 25;
    AdType.ad_26 = 26;
    AdType.ad_27 = 27;
    AdType.ad_28 = 28;
    AdType.ad_29 = 29;
    AdType.ad_30 = 30;
    AdType.ad_31 = 31;
    AdType.ad_32 = 32;

    class ConfigValue {
        constructor() {
        }
        parseItem(byteArray) {
            this.key = byteArray.readUTFString();
            this.value = byteArray.readUTFString();
        }
        static getItemByKey(id) {
            return ConfigValue.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return ConfigValue.itemsDic[key1 + "," + key2];
        }
    }
    ConfigValue.items = new Array();
    ConfigValue.itemsDic = new Dictionary();

    class ConfigLib {
        constructor() {
        }
        static ReadArray(byte) {
            var length = byte.readUint8();
            if (length == 0) {
                return [];
            }
            var isHasNext = byte.readByte() == 1;
            var splitArr = new Array(length);
            for (var i = 0; i < length; i++) {
                if (isHasNext) {
                    splitArr[i] = this.ReadArray(byte);
                }
                else {
                    splitArr[i] = byte.readUTFString();
                }
            }
            return splitArr;
        }
    }

    class catResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint16();
            this.name = byteArray.readUTFString();
            this.desc = byteArray.readUTFString();
            this.icon = byteArray.readUTFString();
            this.res = byteArray.readUTFString();
            this.disposition = byteArray.readUTFString();
            this.character = byteArray.readUTFString();
            this.quality = byteArray.readUint8();
            this.item = byteArray.readUint16();
            this.itemNum = byteArray.readUint8();
            this.unlockType = ConfigLib.ReadArray(byteArray);
            this.loveId = byteArray.readUint8();
            this.miaoId = byteArray.readUint8();
            this.actId = ConfigLib.ReadArray(byteArray);
            this.acunlockType = ConfigLib.ReadArray(byteArray);
            this.acunlockNum = ConfigLib.ReadArray(byteArray);
            this.catball = byteArray.readUTFString();
            this.effcatball = byteArray.readUTFString();
            this.specialball = byteArray.readUTFString();
            this.specialballEff = byteArray.readUTFString();
        }
        static getItemByKey(id) {
            return catResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return catResource.itemsDic[key1 + "," + key2];
        }
    }
    catResource.items = new Array();
    catResource.itemsDic = new Dictionary();

    class itemResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint16();
            this.name = byteArray.readUTFString();
            this.type = byteArray.readUint8();
            this.icon = byteArray.readUTFString();
            this.res = ConfigLib.ReadArray(byteArray);
            this.quality = byteArray.readUint8();
            this.desc = byteArray.readUTFString();
        }
        static getItemByKey(id) {
            return itemResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return itemResource.itemsDic[key1 + "," + key2];
        }
    }
    itemResource.items = new Array();
    itemResource.itemsDic = new Dictionary();

    class ArrayUtil {
        constructor() {
        }
        static RemoveAt(arr, index) {
            let obj = arr.pop();
            if (index < arr.length) {
                arr[index] = obj;
            }
        }
        static shuffle(arr) {
            let i = arr.length;
            if (i > 1) {
                while (i) {
                    let j = Math.floor(Math.random() * i--);
                    [arr[j], arr[i]] = [arr[i], arr[j]];
                }
            }
            return arr;
        }
        static GetRandom(arr, isSort = false) {
            arr = arr.slice();
            let result = [];
            while (arr.length > 0) {
                let index = MathUtil.makeRandom(0, arr.length);
                if (isSort) {
                    if (result.indexOf(arr[index]) == -1) {
                        result.push(arr[index]);
                    }
                }
                else {
                    result.push(arr[index]);
                }
                ArrayUtil.RemoveAt(arr, index);
            }
            return result;
        }
        static sortOn(arr, props) {
            return arr.sort(function (a, b) {
                let arr = [a, b];
                arr = arr.concat(props);
                return ArrayUtil.sortByProps.apply(null, arr);
            });
        }
        static sortByProps(item1, item2, ...params) {
            let props = [];
            for (let _i = 2; _i < arguments.length; _i++) {
                props[_i - 2] = arguments[_i];
            }
            let cps = [];
            let asc = true;
            if (props.length < 1) {
                for (let p in item1) {
                    if (item1[p] > item2[p]) {
                        cps.push(1);
                        break;
                    }
                    else if (item1[p] === item2[p]) {
                        cps.push(0);
                    }
                    else {
                        cps.push(-1);
                        break;
                    }
                }
            }
            else {
                for (let i = 0; i < props.length; i++) {
                    let prop = props[i];
                    for (let o in prop) {
                        asc = prop[o] === ArrayUtil.NUMERIC;
                        if (item1[o] > item2[o]) {
                            cps.push(asc ? 1 : -1);
                            break;
                        }
                        else if (item1[o] === item2[o]) {
                            cps.push(0);
                        }
                        else {
                            cps.push(asc ? -1 : 1);
                            break;
                        }
                    }
                }
            }
            for (let j = 0; j < cps.length; j++) {
                if (cps[j] === 1 || cps[j] === -1) {
                    return cps[j];
                }
            }
            return 0;
        }
    }
    ArrayUtil.DESCENDING = "desc";
    ArrayUtil.NUMERIC = "asc";

    class ItemVO {
        constructor() {
            this._itemId = 0;
            this.itemRes = null;
            this.count = 0;
        }
        set itemId(value) {
            this._itemId = value;
            this.itemRes = itemResource.getItemByKey(value);
        }
        get itemId() {
            return this._itemId;
        }
        getCount() {
            if (this.count instanceof BigNumber) {
                return this.count.toNumber();
            }
            else if (typeof this.count === "string") {
                return parseInt(this.count);
            }
            else if (typeof this.count === "number") {
                return this.count;
            }
        }
        get catRes() {
            return ItemModel.Inst.getCatRes(this._itemId);
        }
        get itemUrl() {
            return ItemVO.GetItemUrl(this.itemRes);
        }
        get catHeadUrl() {
            let catRes = this.catRes;
            return catRes ? PanelRegister.getUrl("icon_cat_head", this.catRes.icon) : "";
        }
        get isToy() {
            return this.catRes != null;
        }
        static GetItemUrl(res) {
            let icon = res instanceof itemResource ? res.icon : res;
            if (res instanceof itemResource) {
                if (res.type == ItemModel.TYPE_BUILD) {
                    return UrlUtils.GetIconPartUrl(icon);
                }
            }
            return UrlUtils.GetIconItemsUrl(icon);
        }
    }

    class ItemModel {
        constructor() {
            this.catResMap = {};
            this.m_itemArrs = new Array();
            this.m_items = new Dictionary();
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new ItemModel();
            return this._Inst;
        }
        init() {
            let catRes = null;
            for (let i = 0; i < catResource.items.length; i++) {
                catRes = catResource.items[i];
                this.catResMap[catRes.item] = catRes;
            }
            let items = StorageUtils.Inst.GetItems();
            let obj;
            let itemVO;
            for (let i = 0; i < items.length; i++) {
                obj = items[i];
                itemVO = new ItemVO();
                itemVO.itemId = obj.tempId;
                itemVO.count = obj.count;
                this.m_itemArrs.push(itemVO);
                this.m_items[itemVO.itemId] = itemVO;
            }
            this.m_itemArrs = this.m_itemArrs.sort((a, b) => {
                return a.itemId - b.itemId;
            });
        }
        getCatRes(itemId) {
            return this.catResMap[itemId];
        }
        itemIsCat(itemId) {
            if (itemId instanceof itemResource) {
                return this.getCatRes(itemId.id) != null;
            }
            return this.getCatRes(itemId) != null;
        }
        AddItemVO(itemVO) {
            if (!itemVO)
                return itemVO;
            this.m_items[itemVO.itemId] = itemVO;
            this.m_itemArrs.push(itemVO);
            return itemVO;
        }
        RemoveItemVO(itemVO) {
            if (itemVO) {
                let index = this.m_itemArrs.indexOf(itemVO);
                ArrayUtil.RemoveAt(this.m_itemArrs, index);
                delete this.m_items[itemVO.itemId];
                return itemVO;
            }
            return null;
        }
        GetItemVOById(itemId) {
            return this.m_items[itemId];
        }
        GetAllItems() {
            return this.m_itemArrs;
        }
        GetAllToyItems() {
            let list = [];
            let vo;
            for (let i = 0; i < this.m_itemArrs.length; i++) {
                vo = this.m_itemArrs[i];
                if (vo && vo.isToy) {
                    list.push(vo);
                }
            }
            return list;
        }
        GetItemCount(itemId) {
            let vo = this.GetItemVOById(itemId);
            return vo ? vo.getCount() : 0;
        }
    }
    ItemModel.ITEM_GOLD = 1;
    ItemModel.ITEM_GEM = 2;
    ItemModel.ITEM_CHURU = 11;
    ItemModel.ITEM_NIP = 12;
    ItemModel.ITEM_SHALOU = 13;
    ItemModel.TYPE_MONEY = 1;
    ItemModel.TYPE_ITEM = 2;
    ItemModel.TYPE_BUILD = 3;
    ItemModel.TYPE_SKIN = 4;
    ItemModel.TYPE_ACTIVITY = 5;
    ItemModel.QUA_N = 1;
    ItemModel.QUA_R = 2;
    ItemModel.QUA_S = 3;
    ItemModel._Inst = null;

    class AwardVO {
        constructor(tempId, data = null) {
            this.type = AwardVO.TYPE_ITEM;
            this.tempId = 0;
            this.data = null;
            this.rate = 1;
            this.tempId = tempId;
            this.setData(data);
        }
        setData(data) {
            this.data = data;
            this.rate = 1;
            if (typeof data === "object") {
                if (data != null && data.hasOwnProperty("rate")) {
                    this.rate = data.rate || 1;
                }
                if (data != null && data.hasOwnProperty("tempId")) {
                    this.tempId = data.tempId;
                }
                if (data != null && data.hasOwnProperty("type")) {
                    this.type = data.type;
                }
            }
        }
        get res() {
            return null;
        }
        get itemRes() {
            return this.IsItem ? itemResource.getItemByKey(this.tempId) : null;
        }
        get catRes() {
            if (this.IsItem) {
                return ItemModel.Inst.getCatRes(this.tempId);
            }
            return this.IsCat ? catResource.getItemByKey(this.tempId) : null;
        }
        get itemUrl() {
            return ItemVO.GetItemUrl(this.itemRes);
        }
        get catHeadUrl() {
            let catRes = this.catRes;
            return catRes ? PanelRegister.getUrl("icon_cat_head", this.catRes.icon) : "";
        }
        get IsNormal() {
            return this.type == AwardVO.TYPE_NORMAL;
        }
        get IsItem() {
            return this.type == AwardVO.TYPE_ITEM;
        }
        get IsCat() {
            return this.type == AwardVO.TYPE_CAT;
        }
        getCoinUrl(rate = 1, useSmall = false) {
            if (this.isGold) {
                return PanelRegister.getUrl("common", "Gold");
            }
            else if (this.isGem) {
                return PanelRegister.getUrl("common", "Gem");
            }
            return "";
        }
        getValueString(sign = "") {
            let cnt = this.count;
            if (!(cnt instanceof BigNumber)) {
                cnt = new BigNumber(cnt);
            }
            cnt = cnt.multipliedBy(this.rate);
            return sign + MathUtil.goldFormat(cnt);
        }
        get count() {
            if (this.data && this.data.hasOwnProperty("count")) {
                return this.data.count;
            }
            return this.res && this.res.count;
        }
        getCount() {
            if (this.count instanceof BigNumber) {
                return this.count.toNumber();
            }
            else if (typeof this.count === "string") {
                return parseInt(this.count);
            }
            else if (typeof this.count === "number") {
                return this.count;
            }
        }
        get IsMoney() {
            return this.itemRes && this.itemRes.type == ItemModel.TYPE_MONEY;
        }
        get isGold() {
            return this.itemRes && this.itemRes.id == ItemModel.ITEM_GOLD;
        }
        get isGem() {
            return this.itemRes && this.itemRes.id == ItemModel.ITEM_GEM;
        }
        get isTableAgain() {
            return this.res && this.res.type == 2;
        }
        get from() {
            if (this.data && this.data.hasOwnProperty("from")) {
                return this.data.from;
            }
            return this.res.from;
        }
        get fromName() {
            return AwardVO.FROM_NAMES[this.from];
        }
        setRate(value = 1) {
            this.rate = value;
        }
        get moreCoin() {
            return this.res.more_coin;
        }
        get isFromTrunTable() {
            return this.from == AwardVO.Turntable;
        }
        get isSevenLogin() {
            return this.from == AwardVO.SevenLogin;
        }
        clone() {
            let vo = new AwardVO(this.tempId, null);
            vo.type = this.type;
            return vo;
        }
    }
    AwardVO.TYPE_NORMAL = 0;
    AwardVO.TYPE_ITEM = 1;
    AwardVO.TYPE_CAT = 2;
    AwardVO.Turntable = 1;
    AwardVO.SevenLogin = 2;
    AwardVO.DesktopAddCoin = 10001;
    AwardVO.SubscribeAddCoin = 10002;
    AwardVO.FROM_NAMES = ["",
        "转盘", "七日登陆奖励", "", "", ""];

    class AdModel {
        constructor() {
            this.pMap = {};
            this.wxIsShowMuiltGrid = false;
            this.gotGrothCount = 0;
            this.clickChuruCount = 0;
            this.clickChuruTime = 0;
            this.bannerChuruTime = 0;
            this.closeBannerTime = 0;
            this.boxLastCloseTime = 0;
            this.boxShowIntervalTime = 120000;
            this.boxCloseBtnDelayShowTime = 5000;
            this._showBoxViewJilv = 0.2;
            this.buildUpLevelCount = 0;
            this._nativeBannerUpdateTime = 40000;
        }
        static get Inst() {
            return this.inst = this.inst || new AdModel();
        }
        init() {
        }
        getNativeAd(nativeId) {
            let adRes = null;
            if (typeof nativeId === "string") {
                let url = "native_" + nativeId;
                adRes = AssetCache.GetResource(url);
            }
            else {
                adRes = this.GetNativeAdRes(nativeId);
            }
            return adRes;
        }
        destroyNative(nativeId) {
            let adRes = this.getNativeAd(nativeId);
            if (adRes) {
                delete AssetCache.assetDic[adRes.url];
                adRes.Destroy();
            }
        }
        hasReportShow(nativeId) {
            let adRes = this.getNativeAd(nativeId);
            return adRes instanceof NativeAdResource && adRes.reportShow == true;
        }
        reportAdShow(nativeAd) {
            let adRes = this.GetNativeAdRes(nativeAd);
            if (adRes) {
                adRes.reportShow = true;
            }
        }
        reportNativeClick(nativeAd) {
            let adRes = this.GetNativeAdRes(nativeAd);
            if (adRes) {
                adRes.reportClick = true;
            }
        }
        GetNativeAdRes(nativeAd) {
            let key;
            for (key in AssetCache.assetDic) {
                let res = AssetCache.assetDic[key];
                if (res instanceof NativeAdResource && res.nativeAd == nativeAd) {
                    return res;
                }
            }
            return null;
        }
        get showBoxViewJilv() {
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                this._showBoxViewJilv = 1;
            }
            return this._showBoxViewJilv;
        }
        get boxAward() {
            if (!this._boxAward) {
                let cv = ConfigValue.getItemByKey("openBoxAward");
                let awardStr = cv && cv.value;
                let id = 1;
                let num = 100;
                if (awardStr) {
                    let arr = awardStr.split(",");
                    if (arr[0])
                        id = parseInt(arr[0]);
                    if (arr[1])
                        num = parseInt(arr[1]);
                }
                this._boxAward = new AwardVO(id, { "count": num });
            }
            return this._boxAward;
        }
        get nativeBannerUpdateTime() {
            if (Laya.Browser.onQGMiniGame) {
                let p4 = this.getPolicy(AdModel.p4, "number");
                let time = p4;
                if (isNaN(time) || time < 0) {
                    time = 0;
                }
                this._nativeBannerUpdateTime = time * 1000;
            }
            else {
                this._nativeBannerUpdateTime = 0;
            }
            return this._nativeBannerUpdateTime;
        }
        addPolicy(data, platfrom = "") {
            platfrom = platfrom || this.GetPlatformName();
            this.pMap[platfrom] = data;
        }
        getPolicy(key = "p1", type = "string") {
            let platfrom = this.GetPlatformName();
            let data = this.pMap[platfrom];
            let value = typeof data === "object" ? data[key] : data;
            if (type == "number" && typeof value === "string") {
                if (value.indexOf(".") != -1) {
                    value = parseFloat(value);
                }
                else {
                    value = parseInt(value);
                }
            }
            return value;
        }
        GetPlatformName() {
            return PlatformMgr.Inst.GetPlatformName();
        }
    }
    AdModel.inst = null;
    AdModel.nativeId_oppo = "";
    AdModel.p1 = "p1";
    AdModel.p2 = "p2";
    AdModel.p3 = "p3";
    AdModel.p4 = "p4";
    AdModel.p5 = "p5";
    AdModel.p6 = "p6";
    AdModel.p7 = "p7";
    AdModel.p8 = "p8";
    AdModel.yes = "yes";
    AdModel.no = "no";

    class GuideResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint8();
            this.guide = byteArray.readUint8();
            this.page = byteArray.readUint8();
            this.name = byteArray.readUTFString();
            this.step = byteArray.readUint8();
            this.type = byteArray.readUint8();
            this.text = byteArray.readUTFString();
            this.point = ConfigLib.ReadArray(byteArray);
            this.stayTime = byteArray.readUint16();
            this.showGuideAni = byteArray.readUint8();
            this.isForce = byteArray.readUint8();
            this.clickSize = ConfigLib.ReadArray(byteArray);
            this.offseXY = ConfigLib.ReadArray(byteArray);
            this.panel = ConfigLib.ReadArray(byteArray);
            this.catId = byteArray.readUint16();
            this.buildId = byteArray.readUint8();
            this.propID = byteArray.readUint16();
            this.propnum = byteArray.readUint8();
            this.level = byteArray.readUint8();
            this.task = byteArray.readUint16();
        }
        static getItemByKey(id) {
            return GuideResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return GuideResource.itemsDic[key1 + "," + key2];
        }
    }
    GuideResource.items = new Array();
    GuideResource.itemsDic = new Dictionary();

    class GuideVO {
        constructor(tempId) {
            this.tempId = 0;
            this.guideRes = null;
            this.setTemp(tempId);
        }
        setTemp(id) {
            this.tempId = id;
            this.guideRes = GuideResource.getItemByKey(id);
        }
        get isMain() {
            return this.guideRes && this.guideRes.guide == GuideVO.GUIDE_ORDER;
        }
        get isFunc() {
            return this.guideRes && this.guideRes.guide == GuideVO.GUIDE_FUNC;
        }
        get isSign() {
            return this.guideRes && this.guideRes.task == GuideVO.TASK_9999;
        }
        get page() {
            return this.guideRes && this.guideRes.page;
        }
        get step() {
            return this.guideRes && this.guideRes.step;
        }
        get isForce() {
            return this.guideRes && this.guideRes.isForce == 1;
        }
        get isShowCircle() {
            return this.guideRes && this.guideRes.showGuideAni > 0;
        }
        next() {
            this.setTemp(this.tempId + 1);
        }
        get prevGuideRes() {
            return GuideResource.getItemByKey(this.tempId - 1);
        }
        get nextGuideRes() {
            return GuideResource.getItemByKey(this.tempId + 1);
        }
        get nextIsSamePage() {
            return this.nextGuideRes && this.page == this.nextGuideRes.page;
        }
        get isStart() {
            return this.guideRes && this.guideRes.type == GuideVO.TYPE_START;
        }
        get isTips() {
            return this.guideRes && this.guideRes.type == GuideVO.TYPE_TIPS;
        }
        get isClickButton() {
            return this.guideRes && this.guideRes.type == GuideVO.TYPE_CLICK_BUTTON;
        }
        get isClickCat() {
            return this.guideRes && this.guideRes.type == GuideVO.TYPE_CLICK_CAT;
        }
        get isClickBuild() {
            return this.guideRes && this.guideRes.type == GuideVO.TYPE_CLICK_BUILD;
        }
        get isPlay() {
            return this.guideRes && this.guideRes.type == GuideVO.TYPE_CLICK_PLAY;
        }
        get isLeaveGift() {
            return this.guideRes && this.guideRes.type == GuideVO.TYPE_LEAVE_GIFT;
        }
        get isClickGift() {
            return this.guideRes && this.guideRes.id == 17;
        }
        get isLevelBuild() {
            return this.guideRes && this.guideRes.type == GuideVO.TYPE_BUILD_LEVEL || (this.guideRes.buildId > 0 && this.guideRes.level > 0);
        }
        get panel() {
            return this.guideRes ? this.guideRes.panel.join("|") : "";
        }
        isStep(page, step) {
            return this.guideRes && this.guideRes.page == page && this.guideRes.step == step;
        }
        isLessThan(page, step) {
            if (this.guideRes) {
                if (this.guideRes.page < page) {
                    return true;
                }
                else if (this.guideRes.page == page && this.guideRes.step < step) {
                    return true;
                }
            }
            return false;
        }
    }
    GuideVO.GUIDE_ORDER = 1;
    GuideVO.GUIDE_FUNC = 2;
    GuideVO.TYPE_START = 1;
    GuideVO.TYPE_TIPS = 2;
    GuideVO.TYPE_CLICK_BUTTON = 3;
    GuideVO.TYPE_CLICK_CAT = 4;
    GuideVO.TYPE_CLICK_BUILD = 5;
    GuideVO.TYPE_CLICK_PLAY = 6;
    GuideVO.TYPE_LEAVE_GIFT = 7;
    GuideVO.TYPE_BUILD_LEVEL = 8;
    GuideVO.TASK_9999 = 9999;

    class GuideModel {
        constructor() {
            this.isOpenGuide = true;
            this.showGuideBg = false;
            this.guideMap = {};
            this.funcGuides = [];
            this.hasDoFuncGuides = [];
            this.taskFuncGuideMap = {};
            this.guideVO = null;
            this.funGuideVO = null;
            this.signGuideVO = null;
            this.levelGiftGuideRes = null;
            this.endWeiguanDiaoyuchang = false;
        }
        static get Inst() {
            return this.inst = this.inst || new GuideModel();
        }
        init() {
            if (!this.isOpenGuide) {
                return;
            }
            let res;
            for (let i = 0; i < GuideResource.items.length; i++) {
                res = GuideResource.items[i];
                this.guideMap[`${res.page}_${res.step}`] = res;
                this.guideMap[res.id] = res;
                if (res.guide == GuideVO.GUIDE_FUNC) {
                    this.funcGuides.push(res);
                    let taskGuideArr = this.taskFuncGuideMap[res.task];
                    if (taskGuideArr == null) {
                        taskGuideArr = [];
                        this.taskFuncGuideMap[res.task] = taskGuideArr;
                    }
                    taskGuideArr.push(res);
                }
                if (res.type == 7) {
                    this.levelGiftGuideRes = res;
                }
            }
            this.hasDoFuncGuides = StorageUtils.Inst.GetFuncGuides();
            this.getSignGuideVo();
            let guideId = StorageUtils.Inst.GetGuide() + 1;
            res = this.GetRes(guideId);
            if (res) {
                if (res.page == 1) {
                    guideId = 1;
                }
                this.guideVO = new GuideVO(guideId);
                GlobalConfig.isOpenAutoTouch = this.guideVO.page > 10;
            }
            else {
                this.guideVO = null;
                GlobalConfig.isOpenAutoTouch = true;
            }
            if (this.guideVO.isFunc) {
                this.guideVO = null;
            }
        }
        doNext() {
            if (this.guideVO) {
                this.guideVO.next();
            }
            if (this.guideVO.guideRes == null || this.guideVO.isFunc) {
                this.guideVO = null;
            }
            if (this.guideVO) {
                GlobalConfig.isOpenAutoTouch = this.guideVO.guideRes.page > 10;
            }
            else {
                GlobalConfig.isOpenAutoTouch = true;
            }
        }
        NeedGuideCat() {
            return this.guideVO && this.guideVO.tempId <= 46;
        }
        GetRes(id) {
            return this.guideMap[id];
        }
        GetResBy(page, step) {
            return this.guideMap[`${page}_${step}`];
        }
        HasGuide() {
            return this.guideVO != null;
        }
        IsNewer() {
            return this.guideVO && this.guideVO.tempId == 1;
        }
        getFunGuides(taskId) {
            return this.taskFuncGuideMap[taskId];
        }
        skipMainGuide(taskId) {
            if (taskId > 0) {
                if (this.guideVO && this.guideVO.guideRes.task > 0 && this.guideVO.guideRes.task < taskId) {
                    while (this.guideVO != null) {
                        this.guideVO.next();
                        if (this.guideVO.guideRes == null) {
                            this.guideVO = null;
                            break;
                        }
                        if (this.guideVO.guideRes.task >= taskId || this.guideVO.guideRes.task == 0) {
                            if (!this.guideVO.isMain) {
                                this.guideVO = null;
                            }
                            break;
                        }
                    }
                }
            }
        }
        setFuncGuide(taskId) {
            if (this.guideVO != null) {
                return;
            }
            let guideRes;
            let funGuideArr = null;
            if (this.funGuideVO != null && this.funGuideVO.guideRes) {
                funGuideArr = this.getFunGuides(this.funGuideVO.guideRes.task);
                if (funGuideArr) {
                    for (let i = 0; i < funGuideArr.length; i++) {
                        guideRes = funGuideArr[i];
                        if (guideRes && this.HasGuideFunc(guideRes.id)) {
                            this.completeFuncGuide(guideRes.id);
                        }
                    }
                }
            }
            else if (this.funGuideVO == null) {
                this.funGuideVO = new GuideVO(0);
            }
            this.funGuideVO.setTemp(0);
            funGuideArr = this.getFunGuides(taskId);
            if (funGuideArr && funGuideArr.length > 0) {
                for (let i = 0; i < funGuideArr.length; i++) {
                    guideRes = funGuideArr[i];
                    if (guideRes && !this.HasGuideFunc(guideRes.id)) {
                        this.funGuideVO.setTemp(guideRes.id);
                        break;
                    }
                }
            }
        }
        doNextFuncGuide() {
            if (this.funGuideVO && this.funGuideVO.tempId > 0) {
                let oldTaskId = this.funGuideVO.guideRes ? this.funGuideVO.guideRes.task : 0;
                this.funGuideVO.next();
                if (this.funGuideVO.guideRes == null || this.funGuideVO.guideRes.task != oldTaskId) {
                    this.funGuideVO.setTemp(0);
                }
            }
        }
        HasGuideFunc(guideId) {
            return this.hasDoFuncGuides.indexOf(guideId) != -1;
        }
        completeFuncGuide(guideId) {
            this.hasDoFuncGuides = StorageUtils.Inst.SetGuideFunc(guideId);
        }
        getSignGuideVo() {
            let guideResList = this.getFunGuides(GuideVO.TASK_9999);
            let len = guideResList ? guideResList.length : 0;
            let res;
            for (let i = 0; i < len; i++) {
                res = guideResList[i];
                if (res && !this.HasGuideFunc(res.id)) {
                    this.signGuideVO = new GuideVO(res.id);
                    break;
                }
            }
            return this.signGuideVO;
        }
        doNextSignGuide() {
            if (this.signGuideVO) {
                this.signGuideVO.next();
                if (!this.signGuideVO.isSign) {
                    this.signGuideVO = null;
                }
            }
        }
        IsMainGuideAll() {
            return this.guideVO == null;
        }
        IsCanShowBtns() {
            return this.guideVO == null || this.guideVO.tempId >= GuideModel.GUID_SHOWBTNS_ID;
        }
        CheckGuideAll() {
            for (const guidRes of this.funcGuides) {
                if (guidRes && this.hasDoFuncGuides.indexOf(guidRes.id) == -1) {
                    return false;
                }
            }
            return this.guideVO == null;
        }
    }
    GuideModel.inst = null;
    GuideModel.GUID_GETGIFT_START_ID = 13;
    GuideModel.GUID_GETGIFT_ID = 14;
    GuideModel.GUID_GETGIFT_END_ID = 15;
    GuideModel.GUID_CREATE_MAOWU_ID = 34;
    GuideModel.GUID_SHOWBTNS_ID = 48;
    GuideModel.GUID_CLICK_MATOU_ID = 50;
    GuideModel.GUID_CLICK_MZBS_ID = 56;
    GuideModel.GUID_INVITE_CAT2_ID = 72;

    class taskResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint8();
            this.viceType = byteArray.readUint8();
            this.cost = byteArray.readUint8();
            this.desc = byteArray.readUTFString();
            this.item = byteArray.readUint8();
            this.num = byteArray.readUint8();
            this.weight = byteArray.readUint8();
        }
        static getItemByKey(id) {
            return taskResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return taskResource.itemsDic[key1 + "," + key2];
        }
    }
    taskResource.items = new Array();
    taskResource.itemsDic = new Dictionary();

    class GlobalJugger {
        constructor() {
        }
        static start() {
            Laya.timer.frameLoop(1, this, this.entter_frame);
            GlobalJugger.timeMS = Laya.timer.currTimer;
        }
        static stop() {
            Laya.timer.clear(this, this.entter_frame);
            GlobalJugger.isCheck = true;
        }
        static entter_frame() {
            if (!GlobalConfig.isActive) {
                GlobalJugger.fps = GlobalJugger.checkFps60;
                GlobalJugger.checkLastTime = 0;
                return;
            }
            GlobalJugger.delta = Laya.timer.delta;
            this.CheckFps();
            if (GlobalJugger.delta > GlobalJugger.fps) {
                GlobalJugger.delta = GlobalJugger.fps;
            }
            GlobalJugger.timeMS += GlobalJugger.delta;
            GlobalJugger.time = GlobalJugger.timeMS * 0.001;
            for (let ianimatable of GlobalJugger.animatinArr) {
                ianimatable.advanceTime();
            }
        }
        static CheckFps() {
            if (GlobalJugger.delta >= GlobalJugger.checkFps20) {
                if (GlobalJugger.fps == GlobalJugger.checkFps20) {
                    GlobalJugger.checkLastTime = 0;
                    return;
                }
                GlobalJugger.checkLastTime += GlobalJugger.delta;
                if (GlobalJugger.checkLastTime > GlobalJugger.checkFpsTime) {
                    GlobalJugger.fps = GlobalJugger.checkFps20;
                    GlobalJugger.checkLastTime = 0;
                    GameDispatcher.Inst.event(UIGameEvent.UPDATE_FPS, GlobalJugger.checkFps20);
                }
                return;
            }
            if (GlobalJugger.delta >= GlobalJugger.checkFps30) {
                if (GlobalJugger.fps == GlobalJugger.checkFps30) {
                    GlobalJugger.checkLastTime = 0;
                    return;
                }
                GlobalJugger.checkLastTime += GlobalJugger.delta;
                if (GlobalJugger.checkLastTime > GlobalJugger.checkFpsTime) {
                    GlobalJugger.fps = GlobalJugger.checkFps30;
                    GlobalJugger.checkLastTime = 0;
                    GameDispatcher.Inst.event(UIGameEvent.UPDATE_FPS, GlobalJugger.checkFps30);
                }
                return;
            }
            if (GlobalJugger.delta >= GlobalJugger.checkFps45) {
                if (GlobalJugger.fps == GlobalJugger.checkFps45) {
                    GlobalJugger.checkLastTime = 0;
                    return;
                }
                GlobalJugger.checkLastTime += GlobalJugger.delta;
                if (GlobalJugger.checkLastTime > GlobalJugger.checkFpsTime) {
                    GlobalJugger.fps = GlobalJugger.checkFps45;
                    GlobalJugger.checkLastTime = 0;
                    GameDispatcher.Inst.event(UIGameEvent.UPDATE_FPS, GlobalJugger.checkFps45);
                }
                return;
            }
            if (GlobalJugger.delta >= GlobalJugger.checkFps60) {
                if (GlobalJugger.fps == GlobalJugger.checkFps60) {
                    GlobalJugger.checkLastTime = 0;
                    return;
                }
                GlobalJugger.checkLastTime += GlobalJugger.delta;
                if (GlobalJugger.checkLastTime > GlobalJugger.checkFpsTime) {
                    GlobalJugger.fps = GlobalJugger.checkFps60;
                    GlobalJugger.checkLastTime = 0;
                    GameDispatcher.Inst.event(UIGameEvent.UPDATE_FPS, GlobalJugger.checkFps60);
                }
                return;
            }
        }
        static add(value) {
            if (this.animatinArr.indexOf(value) == -1) {
                if (this.animatinArr.length == 0)
                    this.start();
                this.animatinArr.push(value);
            }
        }
        static remove(value) {
            var index = this.animatinArr.indexOf(value);
            if (index != -1) {
                ArrayUtil.RemoveAt(this.animatinArr, index);
                if (this.animatinArr.length == 0)
                    this.stop();
            }
        }
    }
    GlobalJugger.timeMS = 0;
    GlobalJugger.time = 0;
    GlobalJugger.delta = 0;
    GlobalJugger.animatinArr = new Array();
    GlobalJugger.fps = 1 / 60 * 1000;
    GlobalJugger.checkLastTime = 0;
    GlobalJugger.checkFpsTime = 10000;
    GlobalJugger.checkFps20 = 1 / 20 * 1000;
    GlobalJugger.checkFps30 = 1 / 30 * 1000;
    GlobalJugger.checkFps45 = 1 / 45 * 1000;
    GlobalJugger.checkFps60 = 1 / 60 * 1000;
    GlobalJugger.isCheck = true;

    class ActionNoQueue {
        constructor(maxCount = 0) {
            this.maxCount = 0;
            this.actions = new Array();
            this.maxCount = maxCount;
        }
        advanceTime() {
            if (this.actions.length == 0) {
                GlobalJugger.remove(this);
                return;
            }
            let length = this.maxCount == 0 || this.actions.length < this.maxCount ? this.actions.length : this.maxCount;
            for (let i = 0; i < length; i++) {
                let curAction = this.actions[i];
                if (curAction.IsComplete()) {
                    curAction.OnComplete();
                    curAction.GC();
                    ArrayUtil.RemoveAt(this.actions, i);
                    i--;
                    length--;
                }
                else {
                    curAction.advanceTime();
                }
            }
        }
        ClearAll() {
            for (let action of this.actions) {
                action.GC();
            }
            this.actions.length = 0;
        }
        AddAction(action) {
            if (action == null)
                return;
            this.actions.push(action);
            if (this.actions.length == 1) {
                GlobalJugger.add(this);
            }
            action.Run();
        }
        RemoveAction(action) {
            var index = this.actions.indexOf(action);
            if (index != -1) {
                this.actions.splice(index, 1);
                if (this.actions.length == 0)
                    Laya.timer.clear(this, this.advanceTime);
            }
        }
        IsEmpty() {
            return this.actions.length == 0;
        }
    }

    class BaseAction {
        constructor() {
            this.cls = null;
            this.complete = null;
            this.m_isComplete = false;
        }
        ;
        Run() {
        }
        advanceTime() {
        }
        IsComplete() {
            return this.m_isComplete;
        }
        OnComplete() {
            if (this.complete != null)
                this.complete.run();
        }
        Reset() {
            this.cls = null;
            this.complete = null;
            this.m_isComplete = false;
        }
        GC() {
        }
    }

    class EffectAction extends BaseAction {
        constructor() {
            super();
            this.resName = null;
            this.parent = null;
            this.from = null;
            this.to = null;
            this.rot = null;
            this.isDestroy = true;
            this.time = 3000;
            this.scale = 1;
            this.isLocalToStage = true;
            this.loadSourceComplete = null;
        }
        get type() {
            return 3;
        }
        Run() {
            super.Run();
            EffectMgr.Inst.Play(this.parent, this.resName, this.from, this.to, this.rot, this.loadSourceComplete, this.isDestroy, this.time, this.scale, this.isLocalToStage);
            this.m_isComplete = true;
        }
        Reset() {
            super.Reset();
            this.resName = null;
            this.parent = null;
            this.from = null;
            this.to = null;
            this.isDestroy = true;
            this.time = 3000;
            this.scale = 1;
            this.isLocalToStage = true;
        }
        GC() {
            ObjectPools.Inst.gcObj(EffectAction.KEY, this);
        }
    }
    EffectAction.KEY = "EffectAction";

    class FlashMatAction extends BaseAction {
        constructor() {
            super();
            this.target = null;
            this.loop = 3;
            this.oldColor = new Laya.Vector4();
            this.albedoColor = new Laya.Vector4();
            this._oldAlbedoIntensity = 1;
            this._albedoIntensity = 1;
        }
        Run() {
            super.Run();
            this.albedoColor.setValue(1, 0.9853, 0.4669, 1);
            if (this.target instanceof Laya.SkinnedMeshSprite3D) {
                this.mat = this.target.skinnedMeshRenderer.sharedMaterial && this.target.skinnedMeshRenderer.sharedMaterial.clone();
                this.mat ? this.mat.albedoColor.cloneTo(this.oldColor) : null;
                this._oldAlbedoIntensity = this.mat ? this.mat.albedoIntensity : 1;
                this.startTween();
            }
            else if (this.target instanceof Laya.MeshSprite3D) {
                this.mat = this.target.meshRenderer.material && this.target.meshRenderer.material.clone();
                this.mat ? this.mat.albedoColor.cloneTo(this.oldColor) : null;
                this._oldAlbedoIntensity = this.mat ? this.mat.albedoIntensity : 1;
                this.startTween();
            }
            else {
                this.m_isComplete = true;
            }
            if (this.mat == null) {
                console.log(`--------${this.target.name} 无材质-----无法闪烁!!!`);
                this.m_isComplete = true;
            }
        }
        startTween() {
            fgui.GTween.to4(this.oldColor.x, this.oldColor.y, this.oldColor.z, 1, this.albedoColor.x, this.albedoColor.y, this.albedoColor.z, 4, 0.6)
                .setTarget(this.target).setRepeat(this.loop, true)
                .onUpdate((tw) => {
                this.albedoColor.x = tw.value.x;
                this.albedoColor.y = tw.value.y;
                this.albedoColor.z = tw.value.z;
                this._albedoIntensity = tw.value.w;
                this.updateColor();
            }, this)
                .onComplete(() => {
                this.endTween();
            }, this);
        }
        endTween() {
            fgui.GTween.to3(this.albedoColor.x, this.albedoColor.y, this.albedoColor.z, this.oldColor.x, this.oldColor.y, this.oldColor.z, 0.3)
                .setTarget(this.target)
                .onUpdate((tw) => {
                this.albedoColor.x = tw.value.x;
                this.albedoColor.y = tw.value.y;
                this.albedoColor.z = tw.value.z;
                this.updateColor();
            }, this)
                .onComplete(() => {
                this.m_isComplete = true;
            }, this);
        }
        updateColor() {
            if (this.mat && this.mat._shaderValues == null) {
                return;
            }
            if (this.target instanceof Laya.SkinnedMeshSprite3D) {
                if (this.target.skinnedMeshRenderer && this.mat) {
                    this.mat.albedoColor = this.albedoColor;
                    this.mat.albedoIntensity = this._albedoIntensity;
                    this.target.skinnedMeshRenderer.material = this.mat;
                }
            }
            else if (this.target instanceof Laya.MeshSprite3D) {
                if (this.target.meshRenderer && this.mat) {
                    this.mat.albedoColor = this.albedoColor;
                    this.mat.albedoIntensity = this._albedoIntensity;
                    this.target.meshRenderer.material = this.mat;
                }
            }
        }
        Reset() {
            super.Reset();
            if (this.mat && this.mat._shaderValues) {
                this.mat.albedoColor = this.oldColor;
                this.mat.albedoIntensity = this._oldAlbedoIntensity;
                if (this.target instanceof Laya.SkinnedMeshSprite3D) {
                    if (this.target.skinnedMeshRenderer)
                        this.target.skinnedMeshRenderer.material = this.mat;
                }
                else if (this.target instanceof Laya.MeshSprite3D) {
                    if (this.target.meshRenderer) {
                        this.target.meshRenderer.material = this.mat;
                    }
                }
            }
            this.target = null;
            this.mat = null;
            this.loop = 3;
        }
        GC() {
            ObjectPools.Inst.gcObj(FlashMatAction.KEY, this);
        }
    }
    FlashMatAction.KEY = "FlashMatAction";

    class islandResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint8();
            this.name = byteArray.readUTFString();
            this.icon = byteArray.readUTFString();
            this.nodeName = byteArray.readUTFString();
            this.maxLvl = byteArray.readUint16();
            this.moveLlv = byteArray.readUint16();
            this.houseMaxLvl = byteArray.readUint8();
            this.boatNum = byteArray.readUint8();
            this.cameraTweenStart = ConfigLib.ReadArray(byteArray);
            this.board = ConfigLib.ReadArray(byteArray);
            this.desc = byteArray.readUTFString();
        }
        static getItemByKey(id) {
            return islandResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return islandResource.itemsDic[key1 + "," + key2];
        }
    }
    islandResource.items = new Array();
    islandResource.itemsDic = new Dictionary();

    class ItemCtrl {
        constructor() {
        }
        static get Inst() {
            return this.inst = this.inst || new ItemCtrl();
        }
        init() {
        }
        AddItemVO(itemId, count = 1) {
            let itemVO = ItemModel.Inst.GetItemVOById(itemId);
            if (itemVO == null) {
                itemVO = new ItemVO();
                itemVO.itemId = itemId;
                if (!itemVO.itemRes)
                    return;
                ItemModel.Inst.AddItemVO(itemVO);
            }
            if (typeof itemVO.count === "number") {
                itemVO.count += count;
            }
            StorageUtils.Inst.AddItem(itemId, count);
            if (itemVO.itemRes && itemVO.itemRes.type == ItemModel.TYPE_BUILD) {
                GameDispatcher.Inst.event(UIGameEvent.ADD_BUILD_PARTS, itemVO);
            }
            GameDispatcher.Inst.event(UIGameEvent.ADD_ITEM, itemVO);
            GameDispatcher.Inst.event(UIGameEvent.SHOW_BAGBTN_RED, true);
            return itemVO;
        }
        RemoveItemVO(itemId, count = 1) {
            let itemVO = ItemModel.Inst.GetItemVOById(itemId);
            if (itemVO != null) {
                if (typeof itemVO.count === "number") {
                    itemVO.count -= count;
                }
                if (itemVO.count <= 0) {
                    itemVO.count = 0;
                    ItemModel.Inst.RemoveItemVO(itemVO);
                }
                StorageUtils.Inst.RemoveItem(itemVO.itemId, count);
                GameDispatcher.Inst.event(UIGameEvent.REMOVE_ITEM, itemVO);
            }
            return itemVO;
        }
    }
    ItemCtrl.inst = null;

    class BaseScript extends Laya.Script3D {
        constructor() { super(); }
        onEnable() {
        }
        onDisable() {
        }
        SetVO(vo) {
            this.vo = vo;
        }
        Reset() {
        }
        GC() {
        }
        Dispose() {
        }
    }

    class BuildScript extends BaseScript {
        constructor() { super(); }
        onAwake() {
            super.onAwake();
        }
        onUpdate() {
            super.onUpdate();
        }
        get buildVO() {
            return this.vo;
        }
    }

    class bulidEffResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint8();
            this.resName = byteArray.readUint8();
            this.position = ConfigLib.ReadArray(byteArray);
        }
        static getItemByKey(id) {
            return bulidEffResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return bulidEffResource.itemsDic[key1 + "," + key2];
        }
    }
    bulidEffResource.items = new Array();
    bulidEffResource.itemsDic = new Dictionary();

    class bulidResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint16();
            this.name = byteArray.readUTFString();
            this.icon = byteArray.readUTFString();
            this.resId = byteArray.readUTFString();
            this.actId = byteArray.readUTFString();
            this.type = byteArray.readUint8();
            this.desc = byteArray.readUTFString();
            this.Belong = byteArray.readUint8();
            this.openId = byteArray.readUint16();
            this.openOrder = byteArray.readUint8();
            this.itemId1 = byteArray.readUint16();
            this.itemId2 = byteArray.readUint16();
            this.servePrice1 = byteArray.readUint8();
            this.servePrice2 = byteArray.readUint16();
            this.servePrice3 = byteArray.readUint16();
            this.servePrice4 = byteArray.readUint16();
            this.positionNum = byteArray.readUint8();
            this.positionPrice1 = byteArray.readUint8();
            this.positionPrice2 = byteArray.readUint16();
            this.time = byteArray.readUint8();
            this.score = byteArray.readUint8();
            this.baseScore = byteArray.readUint16();
            this.unlockGold = byteArray.readUTFString();
            this.getPara = byteArray.readFloat64();
            this.costPara = byteArray.readFloat64();
            this.position = ConfigLib.ReadArray(byteArray);
            this.iconOffsetY = byteArray.readFloat64();
            this.selectCameraPos = ConfigLib.ReadArray(byteArray);
            this.previewCameraPos = ConfigLib.ReadArray(byteArray);
            this.selectCameraParRot = ConfigLib.ReadArray(byteArray);
            this.selectCameraRot = ConfigLib.ReadArray(byteArray);
            this.upgradeEff = byteArray.readUint8();
            this.advancedEff = byteArray.readUint8();
            this.eventIds = ConfigLib.ReadArray(byteArray);
            this.createEventIds = ConfigLib.ReadArray(byteArray);
        }
        static getItemByKey(id) {
            return bulidResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return bulidResource.itemsDic[key1 + "," + key2];
        }
    }
    bulidResource.items = new Array();
    bulidResource.itemsDic = new Dictionary();

    class bulidSeatResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.buildId = byteArray.readUTFString();
            this.lv = byteArray.readUint8();
            this.setunlock = byteArray.readUint16();
            this.addParts = ConfigLib.ReadArray(byteArray);
            this.partEffectPos = ConfigLib.ReadArray(byteArray);
        }
        static getItemByKey(id) {
            return bulidSeatResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return bulidSeatResource.itemsDic[key1 + "," + key2];
        }
    }
    bulidSeatResource.items = new Array();
    bulidSeatResource.itemsDic = new Dictionary();

    class bulidupResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.buildId = byteArray.readUTFString();
            this.stage = byteArray.readUint8();
            this.icon = byteArray.readUTFString();
            this.partName = byteArray.readUTFString();
            this.showParts = ConfigLib.ReadArray(byteArray);
            this.addParts = ConfigLib.ReadArray(byteArray);
            this.unlockLvl = byteArray.readUint16();
            this.addon = byteArray.readUint32();
            this.addonlvlUp = byteArray.readUint16();
            this.levelAni = byteArray.readUTFString();
        }
        static getItemByKey(id) {
            return bulidupResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return bulidupResource.itemsDic[key1 + "," + key2];
        }
    }
    bulidupResource.items = new Array();
    bulidupResource.itemsDic = new Dictionary();

    class MaowuUpResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.stage = byteArray.readUint8();
            this.icon = byteArray.readUTFString();
            this.partName = byteArray.readUTFString();
            this.showParts = ConfigLib.ReadArray(byteArray);
            this.addParts = ConfigLib.ReadArray(byteArray);
            this.unlockLvl = byteArray.readUint8();
            this.levelAni = byteArray.readUTFString();
            this.cost = byteArray.readFloat64();
            this.houseMaxLvl = byteArray.readUint8();
        }
        static getItemByKey(id) {
            return MaowuUpResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return MaowuUpResource.itemsDic[key1 + "," + key2];
        }
    }
    MaowuUpResource.items = new Array();
    MaowuUpResource.itemsDic = new Dictionary();

    class AnimatorUtil {
        static play(ani, name, handler = null, speed = 1, normalizedTime = Number.NEGATIVE_INFINITY) {
            if (ani == null) {
                handler && handler.runWith(false);
                return;
            }
            if (!this.hasAction(ani, name)) {
                Logger.log(this, "[playAction]--------------->播放动作失败，没有该动作:" + name);
                handler && handler.runWith(false);
                return;
            }
            Laya.timer.clear(this, this.playLoop);
            Laya.timer.clear(this, this.playAction);
            ani.enabled = true;
            ani.speed = speed;
            ani.play(name, 0, normalizedTime);
            Laya.timer.frameLoop(1, this, this.playLoop, [ani, name, handler]);
        }
        static playLoop(ani, action, handler) {
            if (ani && ani.getCurrentAnimatorPlayState().normalizedTime >= 1) {
                Laya.timer.clear(this, this.playLoop);
                if (handler != null) {
                    handler.runWith(true);
                }
            }
        }
        static getAnimationTime(ani, action) {
            let aniState = ani.getControllerLayer(0).getAnimatorState(action);
            let time = aniState && aniState.clip ? aniState.clip.duration() * 1000 : 0;
            return time;
        }
        static playAction(ani, action, handler = null, time = -1, delay = 100, ignoreSame = true, speed = 1) {
            let self = this;
            Laya.timer.clear(this, this.playAction);
            if (action == "") {
                handler && handler.runWith(false);
                return;
            }
            this.play(ani, action, Laya.Handler.create(self, function complete(action, handler, time, delay, ignoreSame, speed) {
                Laya.timer.clear(self, self.playLoop);
                if (time < 0) {
                    if (delay > 0) {
                        Laya.timer.once(delay, self, self.playAction, [action, handler, time, delay, ignoreSame, speed], true);
                    }
                    else {
                        self.playAction(ani, action, handler, time, delay, ignoreSame);
                    }
                    return;
                }
                time--;
                if (time > 0) {
                    self.playAction(ani, action, handler, time, delay, ignoreSame, speed);
                }
                else {
                    if (handler != null) {
                        handler.run();
                    }
                }
            }, [action, handler, time, delay, ignoreSame, speed], true), speed);
        }
        static hasAction(ani, name) {
            let aniState = ani && ani.getControllerLayer(0) ? ani.getControllerLayer(0).getAnimatorState(name) : null;
            return aniState != null;
        }
    }

    class CollisionType {
    }
    CollisionType.TYPE_DEFAULT = Laya.Physics3DUtils.COLLISIONFILTERGROUP_DEFAULTFILTER;
    CollisionType.TYPE_MAP = Laya.Physics3DUtils.COLLISIONFILTERGROUP_CUSTOMFILTER1;
    CollisionType.TYPE_BUILD = Laya.Physics3DUtils.COLLISIONFILTERGROUP_CUSTOMFILTER2;
    CollisionType.TYPE_CAT = Laya.Physics3DUtils.COLLISIONFILTERGROUP_CUSTOMFILTER3;
    CollisionType.TYPE_RAY_CLICK = Laya.Physics3DUtils.COLLISIONFILTERGROUP_CUSTOMFILTER4;
    CollisionType.TYPE_ENTITY = Laya.Physics3DUtils.COLLISIONFILTERGROUP_CUSTOMFILTER5;

    class Entity {
        constructor(displayObject) {
            this._displayObject = null;
            this._vo = null;
            this._active = true;
            this._displayObject = displayObject;
        }
        SetVO(vo) {
            this._vo = vo;
        }
        onAwake() {
            this.SetCollider(this._displayObject);
        }
        SetCollider(owner) {
            let phy = MathUtil.FindPhysicsCollider(owner);
            if (phy) {
                phy.isTrigger = true;
                phy.collisionGroup = this.GetCollisonGroup();
                phy.canCollideWith = this.GetCanCollideWith();
            }
        }
        GetCollisonGroup() {
            return CollisionType.TYPE_DEFAULT;
        }
        GetCanCollideWith() {
            return CollisionType.TYPE_DEFAULT;
        }
        set active(value) {
            this._active = value;
        }
        get active() {
            return this._active;
        }
        get transform() {
            return this._displayObject ? this._displayObject.transform : null;
        }
        get owner() {
            return this._displayObject;
        }
        dispose() {
            this._vo = null;
            if (this._displayObject && !this._displayObject.destroyed) {
                this._displayObject.destroy();
            }
            this._displayObject = null;
        }
    }

    class Build extends Entity {
        constructor(displayObject) {
            super(displayObject);
            this._vo = null;
            this.buildScript = null;
            this.base = null;
            this.zuoweiNode = null;
            this.teShuPeiJianNode = null;
            this.showParts = null;
            this.addParts = null;
            this._addNodes = [];
            this._addNum = 0;
            this.onAwake();
        }
        SetVO(vo) {
            super.SetVO(vo);
            this.buildScript.SetVO(vo);
            this.onChangeParts();
            this.onChangeZuoweiParts();
            this.onChangeTeShuPeiJianParts();
        }
        onAwake() {
            super.onAwake();
            this.showParts = new Dictionary();
            this.addParts = new Dictionary();
            this.zuoweiNode = new Laya.Sprite3D();
            this.owner.addChild(this.zuoweiNode);
            this.teShuPeiJianNode = new Laya.Sprite3D();
            this.owner.addChild(this.teShuPeiJianNode);
            this.buildScript = this._displayObject.addComponent(BuildScript);
            this.clickAni = this.owner.getComponent(Laya.Animator);
            if (this.clickAni) {
                this.clickAni.enabled = false;
            }
            this.base = this.owner.getChildByName("base");
            if (this.base) {
                this.createAni = this.base.getComponent(Laya.Animator);
                if (this.createAni) {
                    this.createAni.enabled = false;
                }
            }
        }
        PlayCreateAni(handler = null) {
            if (this.createAni) {
                this.createAni.enabled = true;
                let action = this.buildScript.buildVO.createAniName;
                AnimatorUtil.play(this.createAni, action, Laya.Handler.create(this, (isSuccess) => {
                    this.createAni.enabled = false;
                    let pos = this._vo.buildPos.clone();
                    pos.y += 10;
                    EffectMgr.Inst.playConfigEffect(EffectMgr.Eff_sheshi_yanhua, this._displayObject.scene, null, pos);
                    if (handler) {
                        handler.run();
                        handler = null;
                    }
                }), 1);
            }
            else {
                let pos = this._vo.buildPos.clone();
                pos.y += 10;
                EffectMgr.Inst.playConfigEffect(EffectMgr.Eff_sheshi_yanhua, this._displayObject.scene, null, pos);
                if (handler) {
                    handler.run();
                    handler = null;
                }
            }
        }
        BuildStageUpAni(handler = null) {
            if (this.createAni) {
                this.createAni.enabled = true;
                let action = this.buildScript.buildVO.stageUpAniName;
                this.createAni.getControllerLayer(0).getAnimatorState(action).clip.islooping = false;
                AnimatorUtil.play(this.createAni, action, Laya.Handler.create(this, (isSuccess) => {
                    this.createAni.enabled = false;
                    let pos = this._vo.buildPos.clone();
                    pos.y += 10;
                    EffectMgr.Inst.playConfigEffect(EffectMgr.Eff_sheshi_yanhua, this._displayObject.scene, null, pos);
                    if (handler) {
                        handler.run();
                        handler = null;
                    }
                }), 1);
            }
        }
        PlayClickAni() {
            if (this.clickAni) {
                this.clickAni.enabled = true;
                let pos = this._vo.upgradeEffPos;
                EffectMgr.Inst.playConfigEffect(EffectMgr.Eff_sheshi_smoke, this._displayObject.scene, null, pos);
                let action = this.buildScript.buildVO.buildRes.icon + "_shengji";
                AnimatorUtil.play(this.clickAni, action, Laya.Handler.create(this, () => {
                    this.clickAni.enabled = false;
                }), 1);
            }
        }
        PlayStageUp() {
            Laya.timer.clear(this, this.PlayStageUpAni);
            let pos = this._vo.advancedEffPos;
            EffectMgr.Inst.playConfigEffect(EffectMgr.Eff_shengji_glow2, this._displayObject.scene, null, pos);
            this.onChangeParts(true);
        }
        onChangeParts(isStageUp = false) {
            Laya.timer.clear(this, this.checkLoadAll);
            this._addNodes.length = 0;
            this._addNum = 0;
            let partNode = MathUtil.FindSprite3DByName(this._displayObject, this._vo.partName);
            if (partNode != null) {
                let showParts = this._vo.showParts;
                for (let i = 0; i < partNode.numChildren; i++) {
                    const node = partNode.getChildAt(i);
                    let isShow = showParts.indexOf(node.name) == -1;
                    node.active = isShow;
                }
            }
            let node = MathUtil.FindSprite3DByName(this._displayObject, "node");
            if (node != null) {
                let dic = new Dictionary();
                for (let i = 0; i < node.numChildren; i++) {
                    const part = node.getChildAt(i);
                    dic[part.name] = part;
                }
                let addParts = this._vo.addParts;
                for (const partName of addParts) {
                    let part = dic[partName];
                    if (part != null) {
                        delete dic[partName];
                    }
                    else {
                        LoadQueue.Inst.loadSprite3D(UrlUtils.GetBuildUrl(partName), Laya.Handler.create(this, this.onPartComplete, [node]));
                        if (isStageUp) {
                            this._addNum++;
                            if (this._addNum == 1) {
                                Laya.timer.loop(1, this, this.checkLoadAll);
                            }
                        }
                    }
                }
                for (const partName in dic) {
                    let part = dic[partName];
                    if (part) {
                        delete dic[partName];
                        if (!part.destroyed) {
                            part.destroy();
                        }
                    }
                }
            }
        }
        onPartComplete(node, sp) {
            if (node && sp) {
                node.addChild(sp);
                if (this._addNodes.indexOf(sp) == -1) {
                    this._addNodes.push(sp);
                }
            }
        }
        checkLoadAll() {
            if (this._addNodes.length == this._addNum) {
                Laya.timer.clear(this, this.checkLoadAll);
                this.FlashParts();
            }
        }
        FlashParts() {
            let part;
            for (let i = 0; i < this._addNodes.length; i++) {
                part = this._addNodes[i];
                EffectMgr.Inst.playMatEffect(part);
            }
            Laya.timer.once(2000, this, this.PlayStageUpAni);
        }
        PlayStageUpAni() {
            let pos = this._vo.buildPos.clone();
            pos.y -= 2;
            EffectMgr.Inst.playConfigEffect(EffectMgr.Eff_shengji_glow1, this._displayObject.scene, null, pos);
            EffectMgr.Inst.playConfigEffect(EffectMgr.Eff_shengji_glow2, this._displayObject.scene, null, pos);
        }
        onChangeZuoweiParts(isAdd = false) {
            this._addNum = 0;
            let node = this.zuoweiNode;
            if (node != null) {
                let dic = new Dictionary();
                for (let i = 0; i < node.numChildren; i++) {
                    const part = node.getChildAt(i);
                    dic[part.name] = part;
                }
                let addParts = this._vo.zuoweiAddParts;
                for (const partName of addParts) {
                    let part = dic[partName];
                    if (part != null) {
                        delete dic[partName];
                    }
                    else {
                        LoadQueue.Inst.loadSprite3D(UrlUtils.GetBuildUrl(partName), Laya.Handler.create(this, this.onPartComplete2, [node, isAdd]));
                    }
                }
                for (const partName in dic) {
                    let part = dic[partName];
                    if (part) {
                        delete dic[partName];
                        if (!part.destroyed) {
                            part.destroy();
                        }
                    }
                }
            }
        }
        onPartComplete2(node, isAdd, sp) {
            node.addChild(sp);
            if (isAdd) {
                let partEffectPos = this.buildVO.buildSeatRes.partEffectPos;
                EffectMgr.Inst.playConfigEffect(EffectMgr.Eff_sheshi_smoke, sp, null, new Laya.Vector3(-parseFloat(partEffectPos[0]), parseFloat(partEffectPos[1]), parseFloat(partEffectPos[2])));
            }
        }
        onChangeTeShuPeiJianParts(isAdd = false) {
            this._addNum = 0;
            let node = this.teShuPeiJianNode;
            if (node != null) {
                let dic = new Dictionary();
                for (let i = 0; i < node.numChildren; i++) {
                    const part = node.getChildAt(i);
                    dic[part.name] = part;
                }
                let addParts = this._vo.teShuPeiJianAddParts;
                for (const partName of addParts) {
                    let part = dic[partName];
                    if (part != null) {
                        delete dic[partName];
                    }
                    else {
                        LoadQueue.Inst.loadSprite3D(UrlUtils.GetBuildUrl(partName), Laya.Handler.create(this, this.onPartComplete3, [node, isAdd]));
                    }
                }
                for (const partName in dic) {
                    let part = dic[partName];
                    if (part) {
                        delete dic[partName];
                        if (!part.destroyed) {
                            part.destroy();
                        }
                    }
                }
            }
        }
        onPartComplete3(node, isAdd, sp) {
            if (sp) {
                node && node.addChild(sp);
            }
        }
        get buildVO() {
            return this._vo;
        }
        GetCollisonGroup() {
            return CollisionType.TYPE_BUILD;
        }
        GetCanCollideWith() {
            return CollisionType.TYPE_RAY_CLICK;
        }
        dispose() {
            super.dispose();
            if (this.buildScript) {
                this.buildScript.destroy();
            }
            this.buildScript = null;
            Laya.timer.clear(this, this.checkLoadAll);
            Laya.timer.clear(this, this.PlayStageUpAni);
        }
    }

    class BuildMatou extends Build {
    }

    class StringUtil {
        static formatToTime(time, formart = "00:00:00") {
            time = Number(time.toFixed(0));
            let second = time % 60;
            let minute = ((time - second) / 60) % 60;
            let hour = (time - 60 * minute - second) / 3600;
            let strSecond = second < 10 ? "0" + second.toString() : second.toString();
            let strMinute = minute < 10 ? "0" + minute.toString() : minute.toString();
            let strHour = hour < 10 ? "0" + hour.toString() : hour.toString();
            if (formart == "hms") {
                return strHour + "h " + strMinute + "m " + strSecond + "s";
            }
            else if (formart == "00:00") {
                return strMinute + ":" + strSecond;
            }
            else if (formart == "hms2") {
                let day = Math.floor(hour / 24);
                hour = Math.ceil(hour % 24);
                return (day > 0 ? day + "天" : "") + (hour > 0 ? hour + "小时" : "") + (minute > 0 ? minute + "分钟" : "") + (second > 0 ? second + "秒" : "");
            }
            else if (formart == "天00:00:00") {
                let day = Math.floor(hour / 24);
                hour = Math.ceil(hour % 24);
                let hourStr = hour < 10 ? "0" + hour.toString() : hour.toString();
                return day + "天" + hourStr + ":" + strMinute + ":" + strSecond;
            }
            return strHour + ":" + strMinute + ":" + strSecond;
        }
    }

    class AddFishModel {
        constructor() {
            this.addFishTime = 0;
            this.endTime = 0;
            this.HighCatfood = 0;
            this.HighCatfoodAdvertisement = 0;
            this.HighCatfoodmultiple = 0;
        }
        static get Inst() {
            return this.inst = this.inst || new AddFishModel();
        }
        init() {
            AddFishModel.FISH_TIME = parseFloat(ConfigValue.getItemByKey("addFish").value) * 3600000;
            this.HighCatfood = parseInt(ConfigValue.getItemByKey("HighCatfood").value);
            this.HighCatfoodAdvertisement = parseInt(ConfigValue.getItemByKey("HighCatfoodAdvertisement").value);
            this.HighCatfoodmultiple = parseInt(ConfigValue.getItemByKey("HighCatfoodmultiple").value);
            this.addFishTime = StorageUtils.Inst.GetAddFishTime();
            this.endTime = this.addFishTime + AddFishModel.FISH_TIME;
            AddFishCtrl.Inst.init();
        }
        AddFish() {
            this.addFishTime = Laya.timer.currTimer;
            this.endTime = this.addFishTime + AddFishModel.FISH_TIME;
            StorageUtils.Inst.addFish(this.addFishTime);
        }
        HasFish() {
            return this.addFishTime > 0 && this.GetSurplusTime() > 0;
        }
        GetSurplusTime() {
            return this.endTime - Laya.timer.currTimer;
        }
    }
    AddFishModel.inst = null;
    AddFishModel.FISH_TIME = 0;

    class baseResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.lvl = byteArray.readUint8();
            this.catMax = byteArray.readUint8();
            this.cost = byteArray.readFloat64();
            this.partName = byteArray.readUTFString();
            this.showParts = ConfigLib.ReadArray(byteArray);
            this.addParts = ConfigLib.ReadArray(byteArray);
            this.levelAni = byteArray.readUTFString();
        }
        static getItemByKey(id) {
            return baseResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return baseResource.itemsDic[key1 + "," + key2];
        }
    }
    baseResource.items = new Array();
    baseResource.itemsDic = new Dictionary();

    class MatouModel {
        constructor() {
            this._obj = {};
            this.boatADTime = 14400;
            this.boatNTime = 0;
            this.boatRTime = 0;
            this.boatSTime = 0;
            this.boatCd = 0;
            this.boatADDay = 0;
            this.maxInviteSCount = 0;
            this.curUpCat = null;
            this._obj = StorageUtils.Inst.GetMatou();
            this.boatADTime = parseFloat(ConfigValue.getItemByKey("boatADTime").value);
            this.boatNTime = parseFloat(ConfigValue.getItemByKey("boatNTime").value);
            this.boatRTime = parseFloat(ConfigValue.getItemByKey("boatRTime").value);
            this.boatSTime = parseFloat(ConfigValue.getItemByKey("boatSTime").value);
            this.boatCd = parseFloat(ConfigValue.getItemByKey("boatCd").value);
            this.boatADDay = parseFloat(ConfigValue.getItemByKey("boatADDay").value);
            if (StorageUtils.Inst.IsClearEveryDay)
                this.ClearEveryDay();
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new MatouModel();
            return this._Inst;
        }
        SaveUser() {
            StorageUtils.Inst.SetMatou(this._obj);
        }
        SetAdCount(value) {
            this._obj[MatouModel.adCount] = value;
            this.SaveUser();
        }
        GetAdCount() {
            return MatouModel.adCount in this._obj ? this._obj[MatouModel.adCount] : 0;
        }
        SetAdTime(value) {
            this._obj[MatouModel.adTime] = value;
            this.SaveUser();
        }
        GetAdTime() {
            return MatouModel.adTime in this._obj ? this._obj[MatouModel.adTime] : 0;
        }
        get inviteQuaSCount() {
            return MatouModel.inviteSCount in this._obj ? this._obj[MatouModel.inviteSCount] : 0;
        }
        set inviteQuaSCount(val) {
            this._obj[MatouModel.inviteSCount] = val;
            this.SaveUser();
        }
        SetMatouList(index, value) {
            let matouList = this.GetMatouList();
            matouList[index] = value;
            this._obj[MatouModel.matouList] = matouList;
            this.SaveUser();
        }
        GetMatouList() {
            return MatouModel.matouList in this._obj ? this._obj[MatouModel.matouList] : {};
        }
        ;
        GetMatouByIndex(index) {
            let matouList = this.GetMatouList();
            return index in matouList ? matouList[index] : null;
        }
        GetTimeByQuality(quality) {
            if (quality == 1)
                return this.boatNTime;
            else if (quality == 2)
                return this.boatRTime;
            else if (quality == 3)
                return this.boatSTime;
        }
        ClearEveryDay() {
            this.SetAdTime(0);
            this.SetAdCount(0);
            this.SaveUser();
        }
    }
    MatouModel._Inst = null;
    MatouModel.adCount = "adCount";
    MatouModel.adTime = "adTime";
    MatouModel.matouList = "matouList";
    MatouModel.inviteSCount = "inviteSCount";

    class I18nResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUTFString();
            this.msg = byteArray.readUTFString();
        }
        static getItemByKey(id) {
            return I18nResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return I18nResource.itemsDic[key1 + "," + key2];
        }
    }
    I18nResource.items = new Array();
    I18nResource.itemsDic = new Dictionary();

    class UITipView extends UIBaseWindow {
        constructor() {
            super("common", "UITipView");
            this.n1 = null;
            this.txt = null;
            this.handler = null;
            this.touchable = false;
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = 5;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
        }
        InitData(param) {
            let self = this;
            super.InitData(param);
            this.setPivot(0.5, 0.5);
            this.y = (Laya.stage.height - this.height) * 0.5;
            if (typeof param === "string") {
                this.txt.text = param;
                this.handler = null;
                this.n1.width = this.txt.textWidth + 30;
                this.viewWidth = this.n1.width;
            }
            else if (typeof param === "object") {
                this.txt.text = param.text;
                this.handler = param.handler;
                this.n1.width = this.txt.textWidth + 30;
                this.viewWidth = this.n1.width;
                if (param.hasOwnProperty("y")) {
                    this.y = param.y;
                }
                if (param.hasOwnProperty("offsetY")) {
                    this.y += param.offsetY;
                }
                if (param.hasOwnProperty("isTween") && param.isTween == false) {
                    this.alpha = 1;
                    let delay = param.hasOwnProperty("delay") ? param.delay : 1;
                    if (delay >= 0) {
                        fgui.GTween.to(1, 0, 0.5).setTarget(self, "alpha").setDelay(delay).onComplete(function () {
                            UISystem.Inst.RemoveWindowView(self);
                            if (self.handler != null) {
                                self.handler.run();
                            }
                        }, null);
                    }
                    return;
                }
            }
            this.alpha = 0;
            Laya.timer.frameOnce(2, null, function () {
                fgui.GTween.to(0, 1, 0.5).setTarget(self, "alpha").onComplete(function () {
                    fgui.GTween.to(1, 0, 0.5).setTarget(self, "alpha").setDelay(1).onComplete(function () {
                        UISystem.Inst.RemoveWindowView(self);
                        if (self.handler != null) {
                            self.handler.run();
                        }
                    }, null);
                }, null);
            });
        }
        Reset() {
            super.Reset();
        }
    }

    class I18nUtils {
        constructor() {
        }
        static ShowI18nError(errorCode) {
            this.ShowI18nById(errorCode);
        }
        static ShowI18nById(id) {
            var codeRes = I18nResource.getItemByKey(id);
            if (codeRes == null) {
                I18nUtils.ShowMsg("I18nResource找不到		" + id);
                return;
            }
            I18nUtils.ShowMsg(codeRes.msg);
        }
        static ShowMsg(value) {
            UISystem.Inst.CreateWindowView(UITipView, value);
        }
        static getLang(code, ...args) {
            let res = I18nResource.getItemByKey(code);
            let str = res ? res.msg : code;
            if (args.length > 0) {
                for (let i = 0, n = args.length; i < n; i++) {
                    str = str.replace("{" + i + "}", args[i]);
                }
            }
            return str;
        }
    }

    class shopResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint8();
            this.type = byteArray.readUint8();
            this.itemId = byteArray.readUint8();
            this.num = byteArray.readUint16();
            this.price = byteArray.readUint16();
            this.limit = byteArray.readUint16();
            this.freeNum = byteArray.readUint8();
        }
        static getItemByKey(id) {
            return shopResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return shopResource.itemsDic[key1 + "," + key2];
        }
    }
    shopResource.items = new Array();
    shopResource.itemsDic = new Dictionary();

    class EventType {
    }
    EventType.kaishijiazai = "kaishijiazai";
    EventType.jiazaiwancheng = "jiazaiwancheng";
    EventType.maoqiu = "maoqiu";
    EventType.momao = "momao";
    EventType.jianzhujiesuo = "jianzhujiesuo";
    EventType.maomijiesuo = "maomijiesuo";
    EventType.xishouyindao = "xishouyindao";
    EventType.fenxiang = "fenxiang";
    EventType.shop = "shop";
    EventType.piaoliupingyaoqing = "piaoliupingyaoqing";
    EventType.shipinguanggao = "shipinguanggao";
    EventType.bannerguangggao = "bannerguangggao";

    class skinResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint16();
            this.itemId = byteArray.readUint16();
            this.suitId = byteArray.readUint8();
            this.name = byteArray.readUTFString();
            this.award_type = byteArray.readUint8();
            this.resId = ConfigLib.ReadArray(byteArray);
            this.icon = byteArray.readUint8();
            this.partNodeName = ConfigLib.ReadArray(byteArray);
            this.quality = byteArray.readUint8();
            this.addon = byteArray.readUint16();
            this.request = byteArray.readUint8();
        }
        static getItemByKey(id) {
            return skinResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return skinResource.itemsDic[key1 + "," + key2];
        }
    }
    skinResource.items = new Array();
    skinResource.itemsDic = new Dictionary();

    class suitResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.suitId = byteArray.readUint8();
            this.name = byteArray.readUTFString();
            this.skinIds = ConfigLib.ReadArray(byteArray);
            this.award_type = byteArray.readUint8();
            this.designation = ConfigLib.ReadArray(byteArray);
        }
        static getItemByKey(id) {
            return suitResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return suitResource.itemsDic[key1 + "," + key2];
        }
    }
    suitResource.items = new Array();
    suitResource.itemsDic = new Dictionary();

    class SkinVO {
        constructor() {
            this.tempId = 0;
            this.isWear = false;
            this.count = 0;
            this.owner = null;
        }
        static GetSkinUrl(res) {
            let skinRes = res instanceof skinResource ? res : skinResource.getItemByKey(res);
            let itemRes = itemResource.getItemByKey(skinRes.itemId);
            if (itemRes == null) {
                console.error(`没有对应的皮肤【${skinRes.name}】 skinId:${skinRes.itemId}`);
            }
            return itemRes ? PanelRegister.getUrl("icon_items", itemRes.icon) : "";
        }
        static GetPartName(part) {
            return SkinVO.PART_NAMES[part];
        }
        get skinRes() {
            return skinResource.getItemByKey(this.tempId);
        }
        get itemRes() {
            return itemResource.getItemByKey(this.skinRes.itemId);
        }
        get quaIsN() {
            return this.itemRes.quality == ItemModel.QUA_N;
        }
        get quaIsR() {
            return this.itemRes.quality == ItemModel.QUA_R;
        }
        get quaIsS() {
            return this.itemRes.quality == ItemModel.QUA_S;
        }
        get iconUrl() {
            return ItemVO.GetItemUrl(this.itemRes);
        }
        get part() {
            return this.skinRes ? this.skinRes.icon : 0;
        }
        get isSuit() {
            return this.skinRes && this.skinRes.suitId > 0;
        }
        get hasOwned() {
            return this.count > 0;
        }
        decode(data) {
            if (data) {
                this.tempId = typeof data.tempId === "string" ? parseInt(data.tempId) : data.tempId;
                this.isWear = data.isWear;
                this.count = data.count;
            }
        }
        toJson() {
            return { "tempId": this.tempId, "isWear": this.isWear, "count": this.count };
        }
    }
    SkinVO.PART_BACK = 1;
    SkinVO.PART_HAND = 2;
    SkinVO.PART_GLASS = 3;
    SkinVO.PART_HAT = 4;
    SkinVO.PART_FACE = 5;
    SkinVO.PART_NACKLACE = 6;
    SkinVO.PART_NAMES = ["", "背部", "手链", "眼镜", "帽子", "脸蛋", "项链"];

    class SuitVO {
        constructor(suitId) {
            this.suitId = 0;
            this.skinIds = [];
            this._skinVos = [];
            this.suitRes = null;
            this.owner = null;
            this.suitId = suitId;
            this.suitRes = suitResource.getItemByKey(suitId);
            let skinVO = null;
            for (let i = 0; i < this.suitRes.skinIds.length; i++) {
                let skinId = parseInt(this.suitRes.skinIds[i]);
                this.skinIds.push(skinId);
                skinVO = SkinModel.Inst.GetSkinVo(skinId);
                this._skinVos.push(skinVO);
            }
        }
        get skinVos() {
            return this._skinVos;
        }
        get skinOwnedCount() {
            let count = 0;
            let skinId = 0;
            for (let i = 0; i < this.skinIds.length; i++) {
                skinId = this.skinIds[i];
                if (skinId > 0 && StorageUtils.Inst.HasSkin(skinId)) {
                    count++;
                }
            }
            return count;
        }
    }

    class SkinModel {
        constructor() {
            this.suitVoMap = {};
            this.suitVos = [];
            this.ownedSkinMap = {};
        }
        static get Inst() {
            return this.inst = this.inst || new SkinModel();
        }
        static GetClothGoldAddRate(skinVos) {
            let num = 0;
            if (skinVos != null) {
                let skinVO;
                for (skinVO of skinVos) {
                    if (skinVO.quaIsN) {
                        num += 400;
                    }
                    else if (skinVO.quaIsR) {
                        num += 800;
                    }
                    else if (skinVO.quaIsS) {
                        num += 1500;
                    }
                }
            }
            return num;
        }
        init() {
            let skins = StorageUtils.Inst.GetSkins();
            let obj;
            let skinVo;
            for (let i = 0; i < skins.length; i++) {
                obj = skins[i];
                skinVo = new SkinVO();
                skinVo.tempId = obj.skinId;
                skinVo.count = obj.count;
                this.ownedSkinMap[skinVo.tempId] = skinVo;
            }
            let suitVo = null;
            let suitRes;
            for (let i = 0; i < suitResource.items.length; i++) {
                suitRes = suitResource.items[i];
                if (suitRes.suitId > 0) {
                    suitVo = new SuitVO(suitRes.suitId);
                    this.suitVos.push(suitVo);
                    this.suitVoMap[suitRes.suitId] = suitVo;
                }
            }
        }
        sortVos(skinVos) {
            skinVos = skinVos.sort((a, b) => {
                return a.tempId - b.tempId;
            });
            return skinVos;
        }
        getSuitVo(suitId) {
            return this.suitVoMap[suitId];
        }
        GetAllSuitVos() {
            return this.suitVos;
        }
        GetAllSuitVosByAwardType(type = 0) {
            if (type == 0) {
                return this.suitVos;
            }
            let arr = [];
            let suitVo;
            for (let i = 0; i < this.suitVos.length; i++) {
                suitVo = this.suitVos[i];
                if (suitVo && suitVo.suitRes.award_type == type) {
                    arr.push(suitVo);
                }
            }
            return arr;
        }
        GetAllSkinVos(part = 0) {
            let arr = [];
            let skinVo = null;
            for (let key in this.ownedSkinMap) {
                skinVo = this.GetSkinVo(parseInt(key));
                if (skinVo && !skinVo.isSuit && skinVo.hasOwned) {
                    if (part == 0) {
                        arr.push(skinVo);
                    }
                    else if (skinVo.part == part) {
                        arr.push(skinVo);
                    }
                }
            }
            arr = this.sortVos(arr);
            return arr;
        }
        AddSkin(skin, count = 1) {
            let tempId = skin instanceof SkinVO ? skin.tempId : skin;
            let skinVo = this.GetSkinVo(tempId);
            let cnt = StorageUtils.Inst.AddSkin(tempId, count);
            skinVo.count = cnt;
            GameDispatcher.Inst.event(UIGameEvent.ADD_SKIN, skinVo);
            GameDispatcher.Inst.event(UIGameEvent.SHOW_BAGBTN_RED, true);
            return skinVo;
        }
        RemoveSkin(skinId) {
            let skinVo = this.GetSkinVo(skinId);
            if (skinVo) {
                skinVo.count--;
                if (skinVo.count < 0) {
                    skinVo.count = 0;
                }
                StorageUtils.Inst.RemoveSkin(skinId);
                GameDispatcher.Inst.event(UIGameEvent.REMOVE_SKIN, skinVo);
            }
            return skinVo;
        }
        GetSkinVo(tempId) {
            let skinVO = this.ownedSkinMap[tempId];
            if (skinVO == null) {
                skinVO = new SkinVO();
                skinVO.tempId = tempId;
                this.ownedSkinMap[skinVO.tempId] = skinVO;
            }
            return skinVO;
        }
    }
    SkinModel.inst = null;

    class ShopVO {
        constructor(id) {
            this.id = 0;
            this.adCount = -1;
            this._adTime = -1;
            this.id = id;
            this._shopRes = shopResource.getItemByKey(this.id);
            this.type = this._shopRes.type;
        }
        get shopRes() {
            return this._shopRes = this._shopRes || shopResource.getItemByKey(this.id);
        }
        get isLookAd() {
            return this.shopRes.price == 0;
        }
        get maxCount() {
            return this.shopRes.limit;
        }
        get curCount() {
            if (this.adCount == -1) {
                this.adCount = StorageUtils.Inst.GetCommodityAdCount(this.id);
            }
            return this.adCount;
        }
        set curCount(val) {
            if (this.curCount == this.maxCount)
                return;
            this.adCount = val;
            if (this.adCount > this.maxCount) {
                this.adCount = this.maxCount;
            }
            StorageUtils.Inst.SetCommodityAdCount(this.id, this.adCount);
        }
        get freeNum() {
            return this.shopRes.freeNum;
        }
        get adTime() {
            if (this._adTime == -1) {
                this._adTime = StorageUtils.Inst.LastGetGiftBoxTime(this.id);
            }
            return this._adTime;
        }
        set adTime(val) {
            if (!this.isLookAd)
                return;
            this._adTime = val;
            StorageUtils.Inst.SetLastGetGiftBoxTime(this.id, val);
        }
        UpdateAdTime() {
            this.adTime = Laya.timer.currTimer;
        }
        get canLookAd() {
            return this.curCount < this.maxCount;
        }
        get price() {
            return this.shopRes.price;
        }
        get isCd() {
            if (!this.isLookAd) {
                return false;
            }
            return Laya.timer.currTimer - this.adTime < ShopModel.Inst.adTimeCD;
        }
        GetAward() {
            this.curCount++;
            if (!this.canLookAd) {
                GameDispatcher.Inst.event(UIGameEvent.SHOPBUYUPPERLIMIT, this);
            }
            let id = this.shopRes.itemId;
            let num = this.shopRes.num;
            let itemType = itemResource.getItemByKey(id).type;
            if (id == 1) {
                MoneyModel.gold.add(num);
            }
            else if (id == 2) {
                MoneyModel.diamond.add(num);
            }
            else if (id == 13) {
                let goldNum = BuildModel.Inst.GetAllBuildGoldRate(num * 60 * 60, false);
                MoneyModel.gold.add(goldNum);
            }
            else if (itemType == ItemModel.TYPE_ITEM || itemType == ItemModel.TYPE_BUILD) {
                ItemCtrl.Inst.AddItemVO(id, num);
            }
            else if (itemType == ItemModel.TYPE_SKIN) {
                this.AddSkinAward(id, num);
            }
            let buyType = "";
            let shopName = "";
            if (this.isLookAd) {
                if (this.curCount >= this.freeNum) {
                    buyType = "视频";
                }
                else {
                    buyType = "免费";
                }
            }
            else {
                buyType = this.price + "钻石";
            }
            if (this.isGoldTimeShouyi) {
                shopName = this.shopRes.num + "小时金币";
            }
            else if (this.isMaobohe) {
                shopName = this.shopRes.num + "个猫薄荷";
            }
            else if (this.isGetQiulu) {
                shopName = "免费啾噜";
            }
            else if (this.isGetDiamond) {
                shopName = "免费钻石";
            }
            PlatformMgr.Inst.aldSendEvent(EventType.shop, shopName + "-" + buyType);
        }
        AddSkinAward(itemId, num) {
            let skinResArr = skinResource.items;
            for (let i = 0; i < skinResArr.length; i++) {
                let res = skinResArr[i];
                if (res.itemId == itemId) {
                    SkinModel.Inst.AddSkin(res.id, num);
                    break;
                }
            }
        }
        get isGetDiamond() {
            return this.type == ShopVO.GETDIAMOND;
        }
        get isGoldTimeShouyi() {
            return this.type == ShopVO.GOLDTIMESHOUYI;
        }
        get isGetQiulu() {
            return this.type == ShopVO.GETQIULU;
        }
        get isMaobohe() {
            return this.type == ShopVO.BUYMAOBOHE;
        }
    }
    ShopVO.GETDIAMOND = 1;
    ShopVO.GOLDTIMESHOUYI = 2;
    ShopVO.GETQIULU = 3;
    ShopVO.BUYMAOBOHE = 4;

    class ShopModel {
        constructor() {
            this.shopVos = [];
            this.shopMap = {};
            this.adTimeInterval = 0;
        }
        static get Inst() {
            return this.inst = this.inst || new ShopModel();
        }
        init() {
            let vo = null;
            let res;
            for (let i = 0; i < shopResource.items.length; i++) {
                res = shopResource.items[i];
                vo = new ShopVO(res.id);
                this.shopVos.push(vo);
                this.shopMap[res.id] = vo;
            }
            this.adTimeInterval = parseInt(ConfigValue.getItemByKey("gifttime").value) * 1000;
        }
        GetShopVoByType(type) {
            let shopVoArr = [];
            for (let i = 0; i < this.shopVos.length; i++) {
                let vo = this.shopVos[i];
                if (vo.type == type) {
                    shopVoArr.push(vo);
                    continue;
                }
            }
            return shopVoArr;
        }
        GetShopVos() {
            return this.shopVos;
        }
        get adTimeCD() {
            return this.adTimeInterval;
        }
    }
    ShopModel.inst = null;

    class catActResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint8();
            this.actName = byteArray.readUTFString();
            this.trigger = byteArray.readUTFString();
        }
        static getItemByKey(id) {
            return catActResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return catActResource.itemsDic[key1 + "," + key2];
        }
    }
    catActResource.items = new Array();
    catActResource.itemsDic = new Dictionary();

    class CatPoseVO {
        constructor(tempId) {
            this.tempId = 0;
            this.parent = null;
            this.isUnlock = false;
            this.conditionVo = null;
            this.tempId = tempId;
        }
        get canUnlock() {
            if (this.conditionVo) {
                if (this.conditionVo.isCatLevel) {
                    return this.parent.level >= this.conditionVo.needCount;
                }
                else {
                    return this.conditionVo.IsFull;
                }
            }
            return false;
        }
        get actRes() {
            return catActResource.getItemByKey(this.tempId);
        }
        get iconUrl() {
            return PanelRegister.getUrl("icon_pose", this.actRes.trigger);
        }
        get iconUrl2() {
            return PanelRegister.getUrl("icon_pose_rect", this.actRes.trigger);
        }
        get iconUrl3() {
            return PanelRegister.getUrl("icon_pose_cirle", this.actRes.trigger);
        }
    }

    class changeResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint16();
            this.sale = byteArray.readUint8();
            this.buy = byteArray.readUint16();
        }
        static getItemByKey(id) {
            return changeResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return changeResource.itemsDic[key1 + "," + key2];
        }
    }
    changeResource.items = new Array();
    changeResource.itemsDic = new Dictionary();

    class ExchangeVO {
        constructor() {
            this.isBuy = true;
            this.tempId = 0;
            this.count = 0;
        }
        get changeRes() {
            return changeResource.getItemByKey(this.tempId);
        }
        get itemRes() {
            return itemResource.getItemByKey(this.tempId);
        }
        get itemUrl() {
            return ItemVO.GetItemUrl(this.itemRes);
        }
        get catRes() {
            return ItemModel.Inst.getCatRes(this.itemRes.id);
        }
        getCount() {
            return this.count;
        }
    }

    class clothingdrawResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.ID = byteArray.readUint16();
            this.item = byteArray.readUint16();
            this.num = byteArray.readUint16();
            this.weight = byteArray.readUint16();
        }
        static getItemByKey(id) {
            return clothingdrawResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return clothingdrawResource.itemsDic[key1 + "," + key2];
        }
    }
    clothingdrawResource.items = new Array();
    clothingdrawResource.itemsDic = new Dictionary();

    class propdrawResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.ID = byteArray.readUint16();
            this.propid = byteArray.readUint16();
            this.num = byteArray.readUint8();
            this.weight = byteArray.readUint16();
            this.integral = byteArray.readUint8();
        }
        static getItemByKey(id) {
            return propdrawResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return propdrawResource.itemsDic[key1 + "," + key2];
        }
    }
    propdrawResource.items = new Array();
    propdrawResource.itemsDic = new Dictionary();

    class ShopLockDrawModel {
        constructor() {
            this.daojuVos = [];
            this.daojuMap = {};
            this.pifuVos = [];
            this.pifuMap = {};
            this.integralVos = [];
            this.allVos = [];
            this.allMap = {};
            this._daoJuProp = null;
            this._fushiProp = null;
            this.maxProp = 2;
            this.offestProp = 10;
            this.lockDrawfreeDima = 0;
            this.lockDrawOneDima_daoju = 0;
            this.lockDrawTenDima_daoju = 0;
            this.lockDrawOneDima_pifu = 0;
            this.lockDrawTenDima_pifu = 0;
        }
        static get Inst() {
            return this.inst = this.inst || new ShopLockDrawModel();
        }
        init() {
            this._freeLockDrawCount = StorageUtils.Inst.freeLockDrawCount;
            this._maxdaojuProp = StorageUtils.Inst.GetShopLockMaxDrawProp(ShopLockDrawVO.TYPE_DAOJU);
            this._maxfushiProp = StorageUtils.Inst.GetShopLockMaxDrawProp(ShopLockDrawVO.TYPE_PIFU);
            let vodaoju = null;
            let resdaoju;
            for (let i = 0; i < propdrawResource.items.length; i++) {
                resdaoju = propdrawResource.items[i];
                vodaoju = new ShopLockDrawVO(resdaoju.ID, resdaoju, ShopLockDrawVO.TYPE_DAOJU);
                if (vodaoju.isDaoJu) {
                    this.daojuVos.push(vodaoju);
                    this.daojuMap[resdaoju.ID] = vodaoju;
                }
                if (resdaoju.integral == 1) {
                    this.integralVos.push(vodaoju);
                }
                this.allVos.push(vodaoju);
                this.allMap[resdaoju.ID] = vodaoju;
            }
            let voPifu = null;
            let respifu;
            for (let i = 0; i < clothingdrawResource.items.length; i++) {
                respifu = clothingdrawResource.items[i];
                voPifu = new ShopLockDrawVO(respifu.ID, respifu, ShopLockDrawVO.TYPE_PIFU);
                if (voPifu.isPiFu) {
                    this.pifuVos.push(voPifu);
                    this.pifuMap[respifu.ID] = voPifu;
                }
                this.allVos.push(voPifu);
                this.allMap[respifu.ID] = voPifu;
            }
            this.lockDrawfreeDima = parseFloat(ConfigValue.getItemByKey("itemChestSpecNum").value);
            this.lockDrawOneDima_daoju = parseFloat(ConfigValue.getItemByKey("itemChestOneNum").value);
            this.lockDrawTenDima_daoju = parseFloat(ConfigValue.getItemByKey("itemChestTenNum").value);
            this.lockDrawOneDima_pifu = parseFloat(ConfigValue.getItemByKey("skinChestOneNum").value);
            this.lockDrawTenDima_pifu = parseFloat(ConfigValue.getItemByKey("skinChestTenNum").value);
        }
        get allDaojuVos() {
            return this.daojuVos;
        }
        get allPiFuVos() {
            return this.pifuVos;
        }
        get canFreeLockDraw() {
            return this.freeLockDrawCount > 0;
        }
        get freeLockDrawCount() {
            return StorageUtils.Inst.freeLockDrawCount;
        }
        set freeLockDrawCount(val) {
            if (this.freeLockDrawCount == val)
                return;
            this._freeLockDrawCount = val;
            StorageUtils.Inst.freeLockDrawCount = val;
        }
        get daoJuProp() {
            this._daoJuProp = this._daoJuProp || StorageUtils.Inst.GetShopLockDrawProp(ShopLockDrawVO.TYPE_DAOJU);
            return this._daoJuProp;
        }
        set daoJuProp(val) {
            this._daoJuProp = val;
            StorageUtils.Inst.SetShopLockDrawProp(ShopLockDrawVO.TYPE_DAOJU, this._daoJuProp);
        }
        get maxdaojuProp() {
            let count = this._maxdaojuProp;
            return (count + 1) * this.offestProp;
        }
        get fushiProp() {
            this._fushiProp = this._fushiProp || StorageUtils.Inst.GetShopLockDrawProp(ShopLockDrawVO.TYPE_PIFU);
            return this._fushiProp;
        }
        set fushiProp(val) {
            this._fushiProp = val;
            StorageUtils.Inst.SetShopLockDrawProp(ShopLockDrawVO.TYPE_PIFU, this._fushiProp);
        }
        get maxfushiProp() {
            let count = this._maxfushiProp;
            return (count + 1) * this.offestProp;
        }
        GetRandomAward(type = ShopLockDrawVO.TYPE_DAOJU, num = 1) {
            let arr = [];
            let vos;
            if (type == ShopLockDrawVO.TYPE_DAOJU) {
                vos = this.allDaojuVos;
            }
            else if (type == ShopLockDrawVO.TYPE_PIFU) {
                vos = this.allPiFuVos;
            }
            else {
                vos = this.allVos;
            }
            if (vos.length > 0) {
                for (let i = 0; i < num; i++) {
                    arr.push(MathUtil.GetRandomItemByWeight(vos, 1)[0]);
                }
            }
            return arr;
        }
        GetRandomCourier() {
            let catVos = CatModel.Inst.GetAllCatVOs();
            let index = MathUtil.makeRandom(0, catVos.length - 1);
            return catVos[index];
        }
        CanGetJiFenAward(type) {
            if (type == ShopLockDrawVO.TYPE_DAOJU) {
                return this.daoJuProp >= this.maxdaojuProp;
            }
            else if (type == ShopLockDrawVO.TYPE_PIFU) {
                return this.fushiProp >= this.maxfushiProp;
            }
            return false;
        }
        GetJiFenAward(type) {
            if (!this.CanGetJiFenAward(type))
                return;
            let integralVos = this.integralVos;
            let count;
            if (type == ShopLockDrawVO.TYPE_DAOJU) {
                this.daoJuProp -= this.maxdaojuProp;
                count = this._maxdaojuProp;
            }
            else if (type == ShopLockDrawVO.TYPE_PIFU) {
                this.fushiProp -= this.maxfushiProp;
                count = this._maxfushiProp;
            }
            if (count < this.maxProp) {
                count++;
                StorageUtils.Inst.SetShopLockMaxDrawProp(type, count);
                if (type == ShopLockDrawVO.TYPE_DAOJU) {
                    this._maxdaojuProp = count;
                }
                else if (type == ShopLockDrawVO.TYPE_PIFU) {
                    this._maxfushiProp = count;
                }
            }
            let award = MathUtil.GetRandomItemByWeight(integralVos, 1);
            if (award.length > 0) {
                award[0].GetAward();
                return award[0];
            }
        }
    }
    ShopLockDrawModel.inst = null;
    ShopLockDrawModel.IDLEANI = "event_Box";
    ShopLockDrawModel.FASTMAILANI = "event_come";
    ShopLockDrawModel.boxRes = "box2";
    ShopLockDrawModel.EXPRESS_BAGID = 87;
    ShopLockDrawModel.EXPRESS_HATID = 88;

    class ShopLockDrawVO {
        constructor(id, res, _type) {
            this.id = id;
            this._randomRes = res;
            this.type = _type;
            this.weight = this._randomRes.weight;
        }
        GetAward() {
            if (this.isDaoJu) {
                ShopLockDrawModel.Inst.daoJuProp++;
            }
            else if (this.isPiFu) {
                ShopLockDrawModel.Inst.fushiProp++;
            }
            let itemId = this.awardId;
            let num = this.num;
            let itemType = this.awardRes.type;
            if (itemType == ItemModel.TYPE_MONEY) {
                if (itemId == 1) {
                    MoneyModel.gold.add(num);
                }
                else if (itemId == 2) {
                    MoneyModel.diamond.add(num);
                }
            }
            else if (itemId == 13) {
                let goldNum = BuildModel.Inst.GetAllBuildGoldRate(num * 60 * 60, false);
                MoneyModel.gold.add(goldNum);
            }
            else if (itemType == ItemModel.TYPE_ITEM || itemType == ItemModel.TYPE_BUILD) {
                ItemCtrl.Inst.AddItemVO(itemId, num);
            }
            else if (itemType == ItemModel.TYPE_SKIN) {
                this.AddSkinAward(itemId, num);
            }
        }
        AddSkinAward(itemId, num) {
            let skinResArr = skinResource.items;
            for (let i = 0; i < skinResArr.length; i++) {
                let res = skinResArr[i];
                if (res.itemId == itemId) {
                    SkinModel.Inst.AddSkin(res.id, num);
                    break;
                }
            }
        }
        get num() {
            return this.randomRes.num;
        }
        get awardId() {
            if (this.randomRes.item != null) {
                return this.randomRes.item;
            }
            return this.randomRes.propid;
        }
        get awardIconUrl() {
            let url;
            let icon = this.awardRes.icon;
            let type = this.awardRes.type;
            if (type == ItemModel.TYPE_BUILD) {
                url = UrlUtils.GetIconPartUrl(icon);
            }
            else {
                url = PanelRegister.getUrl("icon_items", icon);
            }
            return url;
        }
        get quality() {
            if (this.awardRes.quality == ItemModel.QUA_N) {
                return "N";
            }
            else if (this.awardRes.quality == ItemModel.QUA_R) {
                return "R";
            }
            else if (this.awardRes.quality == ItemModel.QUA_S) {
                return "S";
            }
            else {
                return "";
            }
        }
        get awardRes() {
            return itemResource.getItemByKey(this.awardId);
        }
        get randomRes() {
            return this._randomRes;
        }
        get isDaoJu() {
            return this.type == ShopLockDrawVO.TYPE_DAOJU;
        }
        get isPiFu() {
            return this.type == ShopLockDrawVO.TYPE_PIFU;
        }
        get isJiFenAward() {
            return true;
        }
    }
    ShopLockDrawVO.TYPE_DAOJU = 1;
    ShopLockDrawVO.TYPE_PIFU = 2;
    ShopLockDrawVO.TYPE_COUNTAWARD = 3;

    class TListItem extends TComponent {
        constructor() {
            super();
            this._btns = null;
        }
        constructFromXML(xml) {
            super.constructFromXML(xml);
            this.AddListener();
        }
        InitFairyGui() {
            this._btns = new Array();
            FairyUtils.setVar(this, this, this._btns);
        }
        InitData(param) {
            this.m_param = param;
        }
    }

    class GiftResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint8();
            this.name = byteArray.readUTFString();
            this.itemId = byteArray.readUint8();
            this.type = byteArray.readUint8();
            this.quality = byteArray.readUint8();
            this.num = byteArray.readUint8();
            this.min_rate = byteArray.readUint8();
            this.max_rate = byteArray.readUint16();
            this.weight = byteArray.readUint16();
        }
        static getItemByKey(id) {
            return GiftResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return GiftResource.itemsDic[key1 + "," + key2];
        }
    }
    GiftResource.items = new Array();
    GiftResource.itemsDic = new Dictionary();

    class BaseVO {
        constructor() {
            this.state = BaseVO.STATE_NORMAL;
            this.m_position = null;
            this.curPos = null;
            this.resName = null;
            this.name = null;
            this.type = 0;
            this.r = 0;
            this.g = 0;
            this.b = 0;
            this.a = 0;
            this.position = null;
            this.rotationEuler = null;
            this.scale = null;
            this.objId = 0;
            this.IsCanCheck = true;
            this.isCanRemove = false;
            this.taskId = 0;
            this.isDeath = false;
            this.attackerType = 0;
            this.isCanHit = true;
            this.m_position = new Laya.Vector3();
            this.position = new Laya.Vector3();
            this.rotationEuler = new Laya.Vector3();
            this.scale = new Laya.Vector3();
            BaseVO.COUNT++;
            this.objId = BaseVO.COUNT;
        }
        Parse(bytes) {
            this.resName = BaseVO.readUtf8String(bytes);
            this.name = BaseVO.readUtf8String(bytes);
            this.position.setValue(-bytes.readFloat32(), bytes.readFloat32(), bytes.readFloat32());
            this.rotationEuler.setValue(-bytes.readFloat32(), -bytes.readFloat32(), bytes.readFloat32());
            this.scale.setValue(bytes.readFloat32(), bytes.readFloat32(), bytes.readFloat32());
        }
        GetPostion() {
            if (this.curPos == null)
                return this.m_position;
            else
                return this.curPos;
        }
        SetPostion(value) {
            this.curPos = value;
        }
        ParseArr(value) {
            BaseVO.COUNT++;
            this.objId = BaseVO.COUNT;
        }
        static readUtf8String(bytes) {
            let len = bytes.readUint16();
            return bytes.readUTFBytes(len);
        }
        get scriptCls() {
            return null;
        }
        CheckIsDeath() {
            return this.isDeath;
        }
    }
    BaseVO.COUNT = 0;
    BaseVO.MAP = 1;
    BaseVO.ENTITY = 2;
    BaseVO.CAT = 3;
    BaseVO.LIGHT = 4;
    BaseVO.BEZIER = 5;
    BaseVO.ENEMY = 6;
    BaseVO.DOOR = 7;
    BaseVO.GUIDE = 8;
    BaseVO.BUILD = 9;
    BaseVO.TREE = 10;
    BaseVO.PATH_POINT = 18;
    BaseVO.SIGN = 19;
    BaseVO.NAVIGATE = 20;
    BaseVO.ITEM = 8;
    BaseVO.STATE_NORMAL = 0;
    BaseVO.STATE_ADD = 1;
    BaseVO.STATE_REMOVE = 2;

    class GiftVO extends BaseVO {
        constructor(giftId) {
            super();
            this.catId = 0;
            this.giftId = 0;
            this.itemId = 0;
            this.count = 0;
            this.rate = 1;
            this.giftId = giftId;
            this.giftRes = GiftResource.getItemByKey(this.giftId);
            if (this.giftRes) {
                this.itemId = this.giftRes.itemId;
            }
        }
        GetGold() {
            return BuildModel.Inst.GetAllBuildGoldRate(this.rate, false);
        }
        IsGold() {
            if (this.giftRes) {
                return this.giftRes.type == GiftVO.GOLD;
            }
            return this.itemRes && this.itemRes.id == ItemModel.ITEM_GOLD;
        }
        IsGem() {
            return this.itemRes && this.itemRes.id == ItemModel.ITEM_GEM;
        }
        IsChuru() {
            if (this.giftRes) {
                return this.giftRes.type == GiftVO.CHURU;
            }
            return this.itemRes && this.itemRes.id == ItemModel.ITEM_CHURU;
        }
        IsCatItem() {
            if (this.giftRes) {
                return this.giftRes.type == GiftVO.CAT_ITEM;
            }
            return ItemModel.Inst.itemIsCat(this.itemId);
        }
        get itemRes() {
            return itemResource.getItemByKey(this.itemId);
        }
    }
    GiftVO.GOLD = 1;
    GiftVO.CHURU = 2;
    GiftVO.CAT_ITEM = 3;

    class CatGiftModel {
        constructor() {
            this.leaveGiftVO = null;
            this.giftMap = {};
        }
        static get Inst() {
            return this.inst = this.inst || new CatGiftModel();
        }
        init() {
        }
        AddGift(gift) {
            if (!gift)
                return;
            this.giftMap[gift.giftVO.objId] = gift;
        }
        RemoveGift(gift) {
            if (gift && gift.giftVO) {
                delete this.giftMap[gift.giftVO.objId];
            }
        }
        GetGift(objId) {
            return this.giftMap[objId];
        }
        GetGiftByCatObjId(catObjId) {
            let gift;
            for (const key in this.giftMap) {
                gift = this.giftMap[key];
                if (gift && gift.giftVO.catId == catObjId) {
                    return gift;
                }
            }
            return null;
        }
        makeGiftVO(quality = 1) {
            let list = this.GetGiftResList(quality);
            let res = MathUtil.GetRandomItem(list, 10000);
            let giftVO = new GiftVO(res.id);
            let random = MathUtil.makeRandom(giftVO.giftRes.min_rate, giftVO.giftRes.max_rate);
            if (giftVO.IsGold()) {
                giftVO.rate = random;
                giftVO.count = giftVO.GetGold();
            }
            else if (giftVO.IsChuru()) {
                giftVO.rate = random;
                giftVO.count = giftVO.giftRes.num * random;
            }
            else if (giftVO.IsCatItem()) {
                let itemRes = this.getRandomItem(quality);
                giftVO.itemId = itemRes.id;
                giftVO.count = giftVO.giftRes.num;
            }
            console.log("-----》生成礼物数据: " + res.id + " giftVO.itemId: " + giftVO.itemId + " giftVO.count: " + giftVO.count);
            return giftVO;
        }
        getRandomItem(quality) {
            let items = [];
            let itemRes = null;
            for (let i = 0; i < itemResource.items.length; i++) {
                itemRes = itemResource.items[i];
                if (itemRes.quality == quality && ItemModel.Inst.itemIsCat(itemRes)) {
                    items.push(itemRes);
                }
            }
            let random = MathUtil.makeRandom(0, items.length - 1);
            return items[random];
        }
        makeGiftBy(itemId, count, catId) {
            let giftVO = new GiftVO(0);
            giftVO.itemId = itemId;
            giftVO.count = count;
            giftVO.catId = catId;
            giftVO.rate = 1;
            return giftVO;
        }
        GetGiftResList(quality = 1) {
            let list = [];
            let res;
            for (let i = 0; i < GiftResource.items.length; i++) {
                res = GiftResource.items[i];
                if (res.quality == quality) {
                    list.push(res);
                }
            }
            return list;
        }
    }
    CatGiftModel.inst = null;

    class ActionQueue {
        constructor() {
            this.actions = new Array();
            this.curAction = null;
            this.handler = null;
        }
        advanceTime() {
            if (this.actions.length == 0) {
                GlobalJugger.remove(this);
                if (this.handler != null)
                    this.handler.run();
                return;
            }
            if (this.curAction == null) {
                this.curAction = this.actions[0];
                this.curAction.Run();
                return;
            }
            if (this.curAction.IsComplete()) {
                let curAction = this.actions.shift();
                curAction.OnComplete();
                curAction.GC();
                this.curAction = null;
            }
            else {
                this.curAction.advanceTime();
            }
        }
        ClearAll() {
            for (let action of this.actions) {
                action.GC();
            }
            this.actions.length = 0;
            this.handler = null;
            this.curAction = null;
        }
        AddAction(action) {
            if (action == null)
                return;
            this.actions.push(action);
            if (this.actions.length == 1) {
                GlobalJugger.add(this);
            }
        }
        RemoveAction(action) {
            var index = this.actions.indexOf(action);
            if (index != -1) {
                this.actions.splice(index, 1);
                if (this.actions.length == 0)
                    Laya.timer.clear(this, this.advanceTime);
            }
        }
        RemoveActionByCls(cls) {
            for (let i = 0; i < this.actions.length; i++) {
                let action = this.actions[i];
                if (action instanceof cls) {
                    this.actions.splice(i, 1);
                }
            }
        }
        IsEmpty() {
            return this.actions.length == 0;
        }
    }

    class LoadSprite3dAction extends BaseAction {
        constructor() {
            super();
            this.url = null;
            this.handler = null;
            this.level = LoadLevel.DEFAULT;
        }
        Run() {
            super.Run();
            LoadQueue;
            LoadQueue.Inst.loadSprite3D(this.url, Laya.Handler.create(this, this.onLoadComplete), this.level);
        }
        onLoadComplete(sp) {
            if (this.handler) {
                this.handler.runWith(sp);
                this.handler = null;
            }
            this.m_isComplete = true;
        }
        Reset() {
            super.Reset();
            this.url = null;
            this.handler = null;
        }
        GC() {
            ObjectPools.Inst.gcObj(LoadSprite3dAction.KEY, this);
        }
    }
    LoadSprite3dAction.KEY = "LoadSprite3dAction";

    class NumberUtil {
        constructor() {
        }
        static toInt(value, wi = 1) {
            return value - value % wi;
        }
        static toLocaleString(value) {
            value = NumberUtil.toInt(value);
            let m = NumberUtil.toInt(value / 1000000);
            let k = NumberUtil.toInt(value % 1000000 / 1000);
            let s = value % 1000;
            let result = "";
            if (m > 0) {
                if (k == 0) {
                    result += m + "M";
                }
                else if (k < 10) {
                    result += m + ".00" + k + "M";
                }
                else if (k < 100) {
                    result += m + ".0" + k + "M";
                }
                else if (k > 0) {
                    result += m + "." + k + "M";
                }
            }
            else {
                if (k > 0) {
                    if (s == 0) {
                        result += k + "K";
                    }
                    else if (s < 10) {
                        result += k + ".00" + s + "K";
                    }
                    else if (s < 100) {
                        result += k + ".0" + s + "K";
                    }
                    else if (s > 0) {
                        result += k + "." + s + "K";
                    }
                }
                else
                    result += s;
            }
            return result;
        }
    }

    class BaseSceneTree {
        constructor(cls = null, isCheckRemove = false) {
            this.trees = null;
            this.infos = new Array();
            this.m_rect = null;
            this.index = 0;
            this.frameMax = 30;
            this.isCheckRemove = false;
            this.isCheckTrue = false;
            this.isHasHave = false;
            this.cls = null;
            this.count = "";
            this.countIndex = 0;
            this.cls = cls;
            this.isCheckRemove = isCheckRemove;
        }
        Init(cls, rect, count, frameMax) {
            this.m_rect = rect;
            this.frameMax = frameMax;
            this.trees = null;
            this.count = this.count + "_" + count;
            count--;
            if (count <= 0)
                return;
            if (cls == null)
                cls = this.cls;
            this.trees = new Array(BaseSceneTree.TREE_COUNT);
            let ww = rect.width / BaseSceneTree.TREE_COUNT * 2;
            let hh = rect.height / BaseSceneTree.TREE_COUNT * 2;
            for (let i = 0; i < this.trees.length; i++) {
                let tree = new cls();
                tree.Init(cls, new Laya.Rectangle(rect.x + ww * (i % 2), rect.y + hh * NumberUtil.toInt(i / 2), ww, hh), count, this.frameMax);
                this.trees[i] = tree;
                tree.countIndex = i;
            }
        }
        static CheckRange(x, y, rect) {
            if (x >= rect.x && x < rect.right &&
                y >= rect.y && y < rect.bottom) {
                return true;
            }
            else
                return false;
        }
        static CheckRangeRect(rect1, rect2) {
            if (rect1.x > rect2.right || rect1.right < rect2.x || rect1.y > rect2.bottom || rect1.bottom < rect2.y) {
                return false;
            }
            else {
                return true;
            }
        }
        IsHasRang() {
            return true;
        }
        IsCanCreate() {
            return this.trees == null;
        }
        AddInfo(value, isAddToScene = false, handler = null) {
            if (this.trees == null) {
                if (BaseSceneTree.CheckRange(value.position.x, value.position.z, this.m_rect)) {
                    this.infos.push(value);
                    value.state = BaseVO.STATE_NORMAL;
                    if (isAddToScene)
                        this.AddEntity(value, handler);
                }
            }
            else {
                for (let tree of this.trees) {
                    tree.AddInfo(value, isAddToScene, handler);
                }
            }
        }
        RemoveInfo(value) {
            if (this.trees == null) {
                let i = this.infos.indexOf(value);
                if (i != -1) {
                    this.RemoveByIndex(value, i);
                    this.RemoveEntity(value, true);
                }
            }
            else {
                for (let tree of this.trees) {
                    tree.RemoveInfo(value);
                }
            }
        }
        RemoveByIndex(value, i) {
            if (this.index < i)
                this.index--;
            ArrayUtil.RemoveAt(this.infos, i);
        }
        IsCheckComplete() {
            return this.trees == null && this.index >= this.frameMax;
        }
        advanceTime() {
            if (!this.IsHasRang()) {
                if (this.isCheckTrue) {
                    this.index = 0;
                    return;
                }
                if (!this.IsCanCreate()) {
                    this.isCheckTrue = true;
                    for (let tree of this.trees) {
                        tree.advanceTime();
                        if (!tree.isCheckTrue)
                            this.isCheckTrue = false;
                    }
                    return;
                }
                else {
                    this.CheckRangOut();
                }
                return;
            }
            else {
                this.isCheckTrue = false;
                if (!this.IsCanCreate()) {
                    for (let tree of this.trees) {
                        tree.advanceTime();
                    }
                    return;
                }
                else {
                    this.CheckRangIn();
                }
            }
        }
        CheckInfo(info) {
            this.CheckEntity(info) ? this.AddEntity(info) : this.RemoveEntity(info, false);
        }
        CheckRangIn() {
            for (let i = 0; i < this.frameMax; i++) {
                if (this.infos.length == 0 ||
                    this.index >= this.infos.length) {
                    this.index = 0;
                    return;
                }
                let info = this.infos[this.index];
                if (info == null) {
                    this.index++;
                    continue;
                }
                if (this.IsCanRemove(info)) {
                    this.CheckRemove(info);
                    this.RemoveByIndex(info, this.index);
                    this.RemoveEntity(info, true);
                    continue;
                }
                if (info.IsCanCheck)
                    this.CheckInfo(info);
                this.index++;
            }
        }
        CheckRangOut() {
            if (this.index == 0) {
                this.isCheckTrue = false;
                this.isHasHave = false;
            }
            for (let i = 0; i < this.frameMax; i++) {
                if (this.infos.length == 0 ||
                    this.index >= this.infos.length) {
                    this.index = 0;
                    if (!this.isHasHave)
                        this.isCheckTrue = true;
                    return;
                }
                let info = this.infos[this.index];
                if (info == null) {
                    this.index++;
                    continue;
                }
                if (this.IsCanRemove(info)) {
                    this.CheckRemove(info);
                    this.RemoveByIndex(info, this.index);
                    this.RemoveEntity(info, true);
                    continue;
                }
                if (info.IsCanCheck)
                    this.RemoveEntity(info, false);
                else
                    this.isHasHave = true;
                this.index++;
            }
        }
        IsCanRemove(value) {
            return value.isCanRemove && value.CheckIsDeath();
        }
        CheckRemove(value) {
        }
        CheckEntity(value) {
            return false;
        }
        AddEntity(value, handler = null) {
        }
        RemoveEntity(value, isDispose) {
        }
        RemoveEntitys() {
            for (let info of this.infos) {
                this.RemoveEntity(info, true);
            }
            this.infos.length = 0;
        }
        Clear() {
            if (this.trees == null) {
                this.RemoveEntitys();
                this.index = 0;
            }
            else {
                for (let tree of this.trees) {
                    tree.Clear();
                }
            }
        }
    }
    BaseSceneTree.TREE_COUNT = 4;

    class BuildSceneTree extends BaseSceneTree {
        constructor(cls = null, isCheckRemove = false) {
            super(cls, isCheckRemove);
        }
        CheckEntity(value) {
            return value && value.IsUnLock();
        }
        AddEntity(value, handler = null) {
            if (value.state != BaseVO.STATE_NORMAL) {
                return;
            }
            let entity = EntityModel.Inst.GetBuild(value.objId);
            if (entity != null) {
                if (!entity.active) {
                    value.state = BaseVO.STATE_ADD;
                    entity.active = true;
                }
                return;
            }
            value.state = BaseVO.STATE_ADD;
            this.addScript(value, handler);
        }
        RemoveEntity(value, isDispose) {
            if (value.state != BaseVO.STATE_ADD) {
                return;
            }
            value.state = BaseVO.STATE_NORMAL;
            let entity = EntityModel.Inst.GetBuild(value.objId);
            if (entity != null && entity.active) {
                entity.active = false;
            }
            if (isDispose) {
                entity.dispose();
                EntityModel.Inst.RemoveBuild(value.objId);
            }
        }
        addScript(vo, handler = null) {
            let lf = this;
            let resName = vo.resName;
            let loadAction = ObjectPools.Inst.getObj(LoadSprite3dAction.KEY, LoadSprite3dAction);
            loadAction.url = UrlUtils.GetBuildUrl(resName);
            loadAction.handler = Laya.Handler.create(lf, lf.addbase, [vo, handler]);
            BuildSceneTree.actions.AddAction(loadAction);
        }
        addbase(vo, handler, entity) {
            if (vo == null || vo.state != BaseVO.STATE_ADD)
                return;
            if (entity == null)
                return;
            SceneMgr.Inst.AddMap(vo.resName, entity);
            entity.name = vo.name;
            let rotationEuler = entity.transform.rotationEuler;
            vo.rotationEuler.cloneTo(rotationEuler);
            entity.transform.rotationEuler = rotationEuler;
            let position = entity.transform.position;
            vo.position.cloneTo(position);
            entity.transform.position = position;
            let scale = entity.transform.getWorldLossyScale();
            vo.scale.cloneTo(scale);
            entity.transform.setWorldLossyScale(scale);
            let build = EntityModel.Inst.GetBuildClsByType(vo.buildRes.type, entity);
            build.SetVO(vo);
            EntityModel.Inst.AddBuild(vo.objId, build);
            if (handler != null)
                handler.runWith(build);
        }
        RemoveEntitys() {
            super.RemoveEntitys();
            BuildSceneTree.actions.ClearAll();
        }
    }
    BuildSceneTree.actions = new ActionQueue();

    class MapNetVO {
        constructor() {
            this.endPos = null;
            this.nextMap = null;
        }
        GC() {
            ObjectPools.Inst.gcObj(MapNetVO.KEY, this);
        }
        Reset() {
            this.endPos = null;
            this.nextMap = null;
        }
    }
    MapNetVO.KEY = "MapNetVO";

    class PathPointModel {
        constructor() {
            this.m_pathArrs = new Array();
            this.m_paths = new Dictionary();
        }
        AddPathPoint(pathPointVO) {
            this.m_paths[pathPointVO.objId] = pathPointVO;
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new PathPointModel();
            return this._Inst;
        }
        UnlockPathPoint(landLv) {
            this.m_pathArrs.length = 0;
            for (const key in this.m_paths) {
                let pathPointVO = this.m_paths[key];
                if (pathPointVO.landId <= landLv) {
                    this.m_pathArrs.push(pathPointVO);
                    pathPointVO.addPaths();
                }
            }
        }
        GetPathVOById(pathId) {
            return this.m_paths[pathId];
        }
        GetAllPaths() {
            return this.m_pathArrs;
        }
        GetRandomPathVO() {
            let index = MathUtil.makeRandom(0, this.m_pathArrs.length);
            return this.m_pathArrs[index];
        }
        GetShortDistance(pos) {
            let dis = Number.MAX_VALUE;
            let onPathVO = null;
            for (const pathVO of this.m_pathArrs) {
                let curDis = pathVO.GetDistanceSquared(pos);
                if (curDis < dis) {
                    dis = curDis;
                    onPathVO = pathVO;
                }
            }
            return onPathVO;
        }
        FindPath2(sPos, toPos, sPathVO, toPahtVO, onPathPointVO, pathArr) {
            if (sPathVO == null)
                sPathVO = PathPointModel.Inst.GetShortDistance(sPos);
            if (toPahtVO == null)
                toPahtVO = PathPointModel.Inst.GetShortDistance(toPos);
            this.GetNearPath(toPos, sPathVO, toPahtVO, onPathPointVO, pathArr);
            return pathArr;
        }
        GetNearPath(toPos, curPathPointVO, endPathPointVO, onPathPointVO, outArr) {
            if (curPathPointVO == null)
                return;
            if (onPathPointVO != null) {
                outArr.push(curPathPointVO);
            }
            if (curPathPointVO == endPathPointVO)
                return;
            let nextPathPointVO = curPathPointVO.GetNearPathPointVO(toPos, onPathPointVO);
            if (nextPathPointVO != null && nextPathPointVO != curPathPointVO) {
                this.GetNearPath(toPos, nextPathPointVO, endPathPointVO, curPathPointVO, outArr);
            }
        }
        CheckNearLink(curPath, onPath, pos) {
            let dis1 = Laya.Vector3.distance(onPath.position, pos);
            let dis2 = Laya.Vector3.distance(onPath.position, curPath.position) + Laya.Vector3.distance(curPath.position, pos);
            return dis1 < dis2;
        }
        FindPath(toPos, sPathPointVO, pathPointVOs = null) {
            if (pathPointVOs == null)
                pathPointVOs = [];
            let tPathPointVO = this.GetShortDistance(toPos);
            let mapArr = this.getMapArrById(tPathPointVO, sPathPointVO, pathPointVOs);
            let mapNetVO = null;
            let length = mapArr.length;
            for (let i = 0; i < length; i++) {
                let vo = new MapNetVO();
                vo.nextMap = mapNetVO;
                if (mapNetVO == null)
                    vo.endPos = toPos;
                else
                    vo.endPos = mapArr[i].position;
                mapNetVO = vo;
            }
            return mapNetVO;
        }
        getMapArrById(toPathPointVO, currentPathPointVO, pathPointVOs) {
            pathPointVOs.length = 0;
            if (currentPathPointVO != toPathPointVO)
                this.parseMap(toPathPointVO, currentPathPointVO.nextPaths, pathPointVOs, [], false);
            pathPointVOs.push(currentPathPointVO);
            return pathPointVOs;
        }
        check(tomap, toPathPointVO) {
            for (const pathVO of tomap) {
                if (pathVO.pathPointVO == toPathPointVO)
                    return true;
            }
            return false;
        }
        parseMap(toPathPointVO, tomap, includeMapIdArr, sArr, sort) {
            if (sArr == null)
                sArr = [];
            if (this.check(tomap, toPathPointVO)) {
                includeMapIdArr.push(toPathPointVO);
                return true;
            }
            let pathVOs = tomap.slice();
            let leng = pathVOs.length;
            let pathPointVO = null;
            for (let i = 0; i < leng; i++) {
                let pathVO = sort ? pathVOs.shift() : pathVOs.pop();
                pathPointVO = pathVO.pathPointVO;
                if (pathPointVO != null) {
                    if (sArr.indexOf(pathPointVO) == -1) {
                        sArr.push(pathPointVO);
                        if (pathPointVO == toPathPointVO) {
                            includeMapIdArr.push(pathPointVO);
                            return true;
                        }
                        else {
                            let value = this.parseMap(toPathPointVO, pathPointVO.nextPaths, includeMapIdArr, sArr, sort);
                            if (value) {
                                includeMapIdArr.push(pathPointVO);
                                return value;
                            }
                        }
                    }
                }
            }
            return false;
        }
    }
    PathPointModel._Inst = null;

    class EntityBuffScript extends BaseScript {
        constructor() {
            super();
            this.transform = null;
        }
        onAwake() {
            this.transform = this.owner.transform;
        }
        onUpdate() {
            super.onUpdate();
            let lf = this;
        }
        IsDeath() {
            return false;
        }
        Reset() {
        }
        GC() {
            this.enabled = false;
            if (this.owner != null)
                this.owner.removeSelf();
        }
        Dispose() {
            if (this.owner && !this.owner.destroyed) {
                this.owner.destroy();
            }
        }
    }

    class EntityBaseScript extends EntityBuffScript {
        constructor() {
            super();
            this.animator = null;
            this.hit = null;
            this.curWeapon = null;
            this.entity = null;
            this.fllowTarget = null;
            this._atk = 0;
            this._def = 0;
            this._speed = 0;
            this._maxHp = 0;
            this._curHp = 0;
            this.dataVO = null;
            this.isDeath = false;
        }
        onAwake() {
            super.onAwake();
            this.entity = this.owner.getChildByName("kitten");
            if (this.entity != null) {
                this.animator = this.entity.getComponent(Laya.Animator);
            }
            else {
                this.animator = this.owner.getComponent(Laya.Animator);
            }
            this.SetInitPostion();
        }
        get localRotationEulerY() {
            return this.GetEntityTransform().localRotationEulerY;
        }
        set localRotationEulerY(value) {
            this.GetEntityTransform().localRotationEulerY = value;
        }
        lookAt(target, up, isLocal) {
            this.GetEntityTransform().lookAt(target, up, isLocal);
            this.GetEntityTransform().localRotationEulerY += 180;
        }
        Init() {
            this.isDeath = false;
            this.SetActiveCollider(true);
        }
        SetActiveCollider(value) {
            if (!this.owner)
                return;
            let phy = this.owner.getComponent(Laya.PhysicsCollider);
            if (phy) {
                phy.enabled = value;
                if (value) {
                    phy.collisionGroup = this.GetCollisonGroup();
                    phy.canCollideWith = this.GetCanCollideWith();
                }
            }
        }
        GetCollisonGroup() {
            return CollisionType.TYPE_DEFAULT;
        }
        GetCanCollideWith() {
            return CollisionType.TYPE_DEFAULT;
        }
        GetBullet() {
            return null;
        }
        SetInit(dataVO) {
            this.dataVO = dataVO;
            this.SetInitPostion();
        }
        SetInitPostion() {
            if (this.dataVO == null || this.transform == null)
                return;
            this.dataVO.SetPostion(this.transform.position);
        }
        SetStateMachine(value, data) {
        }
        getStateMachine() {
            return null;
        }
        CrossFadeState(actionName, transitionDuration, handler = null, speed = 1) {
            if (this.animator == null) {
                if (handler) {
                    Laya.timer.once(500, null, function () {
                        handler.run();
                    });
                }
                return;
            }
            this.animator.speed = speed;
            this.animator.crossFade(actionName, transitionDuration, 0, 0);
            if (handler == null)
                return;
            Laya.timer.frameLoop(1, this, this.playComplete, [handler, actionName]);
        }
        PlayState(actionName, handler = null, speed = 1) {
            if (this.animator == null) {
                if (handler) {
                    Laya.timer.once(500, null, function () {
                        handler.run();
                    });
                }
                return;
            }
            this.animator.speed = speed;
            this.animator.play(actionName, 0, 0);
            if (handler == null)
                return;
            Laya.timer.frameLoop(1, this, this.playComplete, [handler, actionName]);
        }
        playComplete(handler, actionName) {
            if (this.animator == null)
                return;
            let playState = this.animator.getControllerLayer().getCurrentPlayState();
            if (playState != null && playState.animatorState.name == actionName && playState.normalizedTime >= 1) {
                Laya.timer.clear(this, this.playComplete);
                if (handler != null)
                    handler.run();
            }
        }
        GetAnimator() {
            return this.animator;
        }
        GetTransform() {
            return this.transform;
        }
        GetEntityTransform() {
            if (this.entity != null)
                return this.entity.transform;
            return this.transform;
        }
        GetDataVO() {
            return this.dataVO;
        }
        SetDeath() {
            this.isDeath = true;
            this.SetActiveCollider(false);
        }
        IsDeath() {
            return this.isDeath;
        }
        Reset() {
            super.Reset();
            Laya.timer.clear(this, this.playComplete);
            if (this.dataVO != null) {
                this.dataVO.SetPostion(null);
                this.dataVO = null;
            }
            this._atk = 0;
            this._maxHp = 0;
            this._curHp = 0;
            this._def = 0;
            this._speed = 0;
            this.fllowTarget = null;
        }
        GC() {
            super.GC();
        }
        Dispose() {
            super.Dispose();
        }
    }

    class CatScript extends EntityBaseScript {
        constructor() {
            super();
        }
        onAwake() {
            super.onAwake();
        }
        get catVO() {
            return this.vo;
        }
    }

    class CatSkinSctipt extends Laya.Script3D {
        constructor() {
            super();
            this.skins = null;
        }
        onAwake() {
        }
        ChangeSkin(parts) {
            if (this.skins == null) {
                this.skins = new Dictionary();
            }
            let partDic = this.CheckHasPart(parts);
            for (const key in this.skins) {
                if (!(key in partDic)) {
                    let part = this.skins[key];
                    if (part && part instanceof Laya.Sprite3D && !part.destroyed) {
                        part.destroy();
                    }
                    delete this.skins[key];
                }
            }
            for (const partId of parts) {
                let skinRes = skinResource.getItemByKey(partId);
                if (skinRes) {
                    for (let i = 0; i < skinRes.resId.length; i++) {
                        let nodeKey = skinRes.partNodeName[i] + skinRes.resId[i];
                        if (!(nodeKey in this.skins)) {
                            this.skins[nodeKey] = UrlUtils.GetCatUrl(skinRes.resId[i]);
                            this.CheckPartNode(skinRes.partNodeName[i], skinRes.resId[i]);
                        }
                    }
                }
                else {
                    console.error("皮肤配置表没找到数据：   " + partId);
                }
            }
        }
        CheckHasPart(parts) {
            let partDic = new Dictionary();
            for (const partId of parts) {
                let skinRes = skinResource.getItemByKey(partId);
                if (skinRes) {
                    for (let i = 0; i < skinRes.resId.length; i++) {
                        partDic[skinRes.partNodeName[i] + skinRes.resId[i]] = UrlUtils.GetCatUrl(skinRes.resId[i]);
                    }
                }
            }
            return partDic;
        }
        CheckPartNode(partNodeName, resName) {
            if (!partNodeName || !resName) {
                console.error("有部位或乖点名称为空：   partNodeName：" + partNodeName + "      resName:" + resName);
                return;
            }
            let key = partNodeName + resName;
            let part = this.skins[key];
            if (part != null && part instanceof Laya.Sprite3D) {
                if (part.name == resName)
                    return;
                if (!part.destroyed)
                    part.destroy();
            }
            let url = UrlUtils.GetCatUrl(resName);
            LoadQueue.Inst.loadSprite3D(url, Laya.Handler.create(this, this.onPartComplete, [url, partNodeName, resName], false));
        }
        onPartComplete(url, spartNodeName, resName, entity) {
            let part = this.skins && this.skins[spartNodeName + resName];
            if (!part)
                return;
            if (part instanceof Laya.Sprite3D)
                return;
            part = entity;
            let parentNode = MathUtil.FindSprite3DByName(this.owner, spartNodeName);
            if (parentNode != null) {
                parentNode.addChild(part);
                this.skins[spartNodeName + resName] = part;
                GameDispatcher.Inst.event(UIGameEvent.CATADDSKIN, [resName, part]);
            }
            else {
                console.error("预制体" + this.owner.name + "没找到挂点：" + spartNodeName);
            }
        }
        GetPart(key) {
            return this.skins[key];
        }
        onDestroy() {
            for (const key in this.skins) {
                let part = this.skins[key];
                if (part && part instanceof Laya.Sprite3D && !part.destroyed) {
                    part.destroy();
                }
            }
            this.skins = null;
            super.onDestroy();
        }
    }

    class EntityAnimator extends Entity {
        constructor(displayObject) {
            super(displayObject);
            this.animator = null;
            this.animatorCtr = null;
            this.stateMachine = null;
        }
        onAwake() {
            super.onAwake();
            this.InitAnimator();
            this.InitStateMachine();
        }
        InitAnimator() {
            this.animator = this.owner && this.owner.getComponent(Laya.Animator);
            if (this.animator)
                this.animatorCtr = this.animator.getControllerLayer(0);
        }
        InitStateMachine() {
        }
        SetStateMachine(value, data) {
            if (this.stateMachine)
                this.stateMachine.SetCurState(value, data);
        }
        getStateMachine() {
            if (this.stateMachine == null)
                return null;
            return this.stateMachine;
        }
        CrossFadeState(actionName, handler = null, transitionDuration = 0.1, speed = 1) {
            if (this.animator == null) {
                if (handler) {
                    Laya.timer.once(500, null, function () {
                        handler.run();
                    });
                }
                return;
            }
            let animationCip = this.animatorCtr.getAnimatorState(actionName);
            if (animationCip == null) {
                let aniState = new Laya.AnimatorState();
                LoadQueue.Inst.loadAnimationClip(UrlUtils.GetCatAnimationUrl(actionName), Laya.Handler.create(this, this.OnResComplete, [actionName, handler, transitionDuration, speed, aniState]), false);
            }
            else {
                this.OnResComplete(actionName, handler, transitionDuration, speed, null, null);
            }
        }
        OnResComplete(actionName, handler, transitionDuration, speed, animatorState, iRes) {
            if (animatorState != null && iRes != null && this.animatorCtr.getAnimatorState(actionName) == null) {
                let animationClip = iRes.GetRes();
                if (!animationClip || !animationClip["_nodes"]) {
                    console.log(actionName);
                }
                animatorState.name = actionName;
                animatorState.speed = speed;
                animatorState.clipStart = 0;
                animatorState.clip = animationClip;
                this.animatorCtr.addState(animatorState);
            }
            this.animator.speed = speed;
            this.animator.crossFade(actionName, transitionDuration, 0, 0);
            if (handler == null)
                return;
            Laya.timer.frameLoop(1, this, this.playComplete, [handler, actionName]);
        }
        PlayState(actionName, handler = null, speed = 1) {
            if (this.animator == null) {
                if (handler) {
                    Laya.timer.once(500, null, function () {
                        handler.run();
                    });
                }
                return;
            }
            let animationCip = this.animatorCtr.getAnimatorState(actionName);
            if (animationCip == null) {
                let aniState = new Laya.AnimatorState();
                LoadQueue.Inst.loadAnimationClip(UrlUtils.GetCatAnimationUrl(actionName), Laya.Handler.create(this, this.OnResComplete2, [actionName, handler, speed, aniState]), false);
            }
            else {
                this.OnResComplete2(actionName, handler, speed, null, null);
            }
        }
        OnResComplete2(actionName, handler, speed, animatorState, iRes) {
            if (animatorState != null && iRes != null && this.animatorCtr.getAnimatorState(actionName) == null) {
                let animationClip = iRes.GetRes();
                animatorState.name = actionName;
                animatorState.speed = speed;
                animatorState.clipStart = 0;
                animatorState.clip = animationClip;
                this.animatorCtr.addState(animatorState);
            }
            this.animator.speed = speed;
            this.animator.play(actionName, 0, 0);
            if (handler == null)
                return;
            Laya.timer.frameLoop(1, this, this.playComplete, [handler, actionName]);
        }
        playComplete(handler, actionName) {
            if (this.animator == null)
                return;
            let playState = this.animator.getControllerLayer().getCurrentPlayState();
            if (playState != null && playState.animatorState.name == actionName && playState.normalizedTime >= 1) {
                Laya.timer.clear(this, this.playComplete);
                if (handler != null)
                    handler.run();
            }
        }
    }

    class FiniteStateMachine {
        constructor() {
            this._currentState = null;
        }
        GetCurState() {
            return this._currentState;
        }
        SetCurState(value, data) {
            if (this._currentState == value)
                return;
            if (this._currentState == null && value != null) {
                GlobalJugger.add(this);
            }
            else if (this._currentState != null && value == null) {
                GlobalJugger.remove(this);
            }
            var state;
            if (this._currentState) {
                state = this._currentState;
                this._currentState = null;
                state.exit();
                state.SetCurStateMachine(null);
            }
            else if (value) {
            }
            this._currentState = value;
            if (this._currentState) {
                this._currentState.SetCurStateMachine(this);
                this._currentState.enter(data);
            }
        }
        advanceTime() {
            if (this._currentState) {
                this._currentState.advanceTime(Laya.timer.delta);
            }
        }
        forceStop() {
            if (this._currentState)
                this._currentState.forceStop();
            this.SetCurState(null);
        }
    }

    class CatEventAction extends BaseAction {
        constructor() {
            super();
            this.cat = null;
            this.actionRes = null;
        }
        Run() {
            this.PlayAni();
        }
        PlayAni() {
            if (this.actionRes.actionName.length > 0) {
                this.cat.CrossFadeState(this.actionRes.actionName[0]);
            }
        }
        Reset() {
            super.Reset();
            this.actionRes = null;
            this.cat = null;
        }
    }

    class CatAddOrRemoveEventAction extends CatEventAction {
        constructor() {
            super();
        }
        Run() {
            super.Run();
            let delayTime = parseFloat(this.actionRes.param[2][0]);
            if (delayTime > 0) {
                Laya.timer.once(delayTime, this, this.OnDelayTime, [this.actionRes, this.cat], false);
            }
            else {
                this.OnDelayTime(this.actionRes, this.cat);
            }
            this.onPlayComplete();
        }
        OnDelayTime(actionRes, cat) {
            let isAdd = actionRes.param[1][0] == "true";
            let gongjus = actionRes.param[0];
            let len = gongjus.length * 0.5;
            if (isAdd) {
                for (let i = 0; i < len; i++) {
                    this.AddGongju(gongjus[i * 2], gongjus[i * 2 + 1], cat);
                }
            }
            else {
                for (let i = 0; i < len; i++) {
                    this.RemoveGongju(gongjus[i * 2], gongjus[i * 2 + 1], cat);
                }
            }
        }
        PlayAni() {
        }
        onPlayComplete() {
            this.m_isComplete = true;
        }
        AddGongju(resName, nodeName, cat, handler = null) {
            let url = UrlUtils.GetEntityLhUrl(resName);
            LoadQueue.Inst.loadSprite3D(url, Laya.Handler.create(this, this.onLoadComplete, [resName, nodeName, cat, handler]));
        }
        onLoadComplete(resName, nodeName, cat, handler, sp) {
            if (cat == null)
                return;
            let node = cat.owner;
            if (nodeName != null && nodeName != "null") {
                node = MathUtil.FindSprite3DByName(cat.owner, nodeName);
            }
            if (node == null)
                return;
            sp.name = resName;
            node.addChild(sp);
            if (handler != null)
                handler.runWith(sp);
        }
        RemoveGongju(resName, nodeName, cat) {
            let node = cat.owner;
            if (nodeName != null && nodeName != "null") {
                node = MathUtil.FindSprite3DByName(cat.owner, nodeName);
            }
            if (node == null)
                return;
            let sp = MathUtil.FindSprite3DByName(node, resName);
            if (sp == null || sp.destroyed)
                return;
            sp.destroy();
        }
        Reset() {
            super.Reset();
        }
        GC() {
            ObjectPools.Inst.gcObj(CatAddOrRemoveEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    CatAddOrRemoveEventAction.KEY = "CatAddOrRemoveEventAction";

    class CatCameraRotationEventAction extends CatEventAction {
        constructor() {
            super();
        }
        Run() {
            super.Run();
            let rx = parseFloat(this.actionRes.param[0]);
            let ry = parseFloat(this.actionRes.param[1]);
            let rz = parseFloat(this.actionRes.param[2]);
            let time = parseFloat(this.actionRes.param[3]);
            let transform = SceneMgr.Inst.GetCameraBuildScript().transform;
            Laya.Tween.to(transform, {
                "localRotationEuler": new Laya.Vector3(transform.localRotationEulerX + rx, transform.localRotationEulerY - ry, transform.localRotationEulerZ + rz)
            }, time);
            this.m_isComplete = true;
        }
        PlayAni() {
        }
        Reset() {
            super.Reset();
        }
        GC() {
            ObjectPools.Inst.gcObj(CatCameraRotationEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    CatCameraRotationEventAction.KEY = "CatCameraRotationEventAction";

    class CatCameraTweenEventAction extends CatEventAction {
        constructor() {
            super();
        }
        Run() {
            super.Run();
            if (SceneMgr.Inst.GetCameraBuildScript().IsSelected()) {
                this.onTweenComplete();
                return;
            }
            let position = new Laya.Vector3(-parseFloat(this.actionRes.param[0]), parseFloat(this.actionRes.param[1]), parseFloat(this.actionRes.param[2]));
            let roteteEuler = new Laya.Vector3(-parseFloat(this.actionRes.param[3]), 180 - parseFloat(this.actionRes.param[4]), parseFloat(this.actionRes.param[5]));
            let fov = parseFloat(this.actionRes.param[6]);
            let time = parseFloat(this.actionRes.param[7]);
            let isFinish = this.actionRes.param[8] == "true";
            let isShow = this.actionRes.param[9] == "true";
            if (isShow) {
                GameDispatcher.Inst.event(UIGameEvent.WHITE_TO_MAIN);
            }
            else {
                GameDispatcher.Inst.event(UIGameEvent.BLACK_TO_WHITE);
            }
            let cameraScript = SceneMgr.Inst.GetCameraBuildScript();
            cameraScript.ResetCameraParent();
            cameraScript.CameraTweenTo(isFinish ? null : Laya.Handler.create(this, this.onTweenComplete), roteteEuler, position, fov, time);
            if (isFinish) {
                this.onTweenComplete();
            }
        }
        onTweenComplete() {
            this.m_isComplete = true;
        }
        Reset() {
            super.Reset();
        }
        GC() {
            ObjectPools.Inst.gcObj(CatCameraTweenEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    CatCameraTweenEventAction.KEY = "CatCameraTweenEventAction";

    class AnimatorScript extends Laya.Script3D {
        constructor() {
            super();
            this.animator = null;
            this.animatorCtr = null;
        }
        onAwake() {
            super.onAwake();
            this.animator = this.owner.getComponent(Laya.Animator);
            if (this.animator)
                this.animatorCtr = this.animator.getControllerLayer(0);
        }
        CrossFadeState(actionName, handler = null, transitionDuration = 0.1, speed = 1) {
            if (this.animator == null) {
                if (handler) {
                    Laya.timer.once(500, null, function () {
                        handler.run();
                    });
                }
                return;
            }
            let animationCip = this.animatorCtr.getAnimatorState(actionName);
            if (animationCip == null) {
                let aniState = new Laya.AnimatorState();
                LoadQueue.Inst.loadAnimationClip(UrlUtils.GetCatAnimationUrl(actionName), Laya.Handler.create(this, this.OnResComplete, [actionName, handler, transitionDuration, speed, aniState]), false);
            }
            else {
                this.OnResComplete(actionName, handler, transitionDuration, speed, null, null);
            }
        }
        OnResComplete(actionName, handler, transitionDuration, speed, animatorState, iRes) {
            if (animatorState != null && iRes != null && this.animatorCtr.getAnimatorState(actionName) == null) {
                let animationClip = iRes.GetRes();
                if (!animationClip || !animationClip["_nodes"]) {
                    console.log(actionName);
                }
                animatorState.name = actionName;
                animatorState.speed = speed;
                animatorState.clipStart = 0;
                animatorState.clip = animationClip;
                this.animatorCtr.addState(animatorState);
            }
            this.animator.speed = speed;
            this.animator.crossFade(actionName, transitionDuration, 0, 0);
            if (handler == null)
                return;
            Laya.timer.frameLoop(1, this, this.playComplete, [handler, actionName]);
        }
        PlayState(actionName, handler = null, speed = 1) {
            if (this.animator == null) {
                if (handler) {
                    Laya.timer.once(500, null, function () {
                        handler.run();
                    });
                }
                return;
            }
            let animationCip = this.animatorCtr.getAnimatorState(actionName);
            if (animationCip == null) {
                let aniState = new Laya.AnimatorState();
                LoadQueue.Inst.loadAnimationClip(UrlUtils.GetCatAnimationUrl(actionName), Laya.Handler.create(this, this.OnResComplete2, [actionName, handler, speed, aniState]), false);
            }
            else {
                this.OnResComplete2(actionName, handler, speed, null, null);
            }
        }
        OnResComplete2(actionName, handler, speed, animatorState, iRes) {
            if (animatorState != null && iRes != null && this.animatorCtr.getAnimatorState(actionName) == null) {
                let animationClip = iRes.GetRes();
                animatorState.name = actionName;
                animatorState.speed = speed;
                animatorState.clipStart = 0;
                animatorState.clip = animationClip;
                this.animatorCtr.addState(animatorState);
            }
            this.animator.speed = speed;
            this.animator.play(actionName, 0, 0);
            if (handler == null)
                return;
            Laya.timer.frameLoop(1, this, this.playComplete, [handler, actionName]);
        }
        playComplete(handler, actionName) {
            if (this.animator == null)
                return;
            let playState = this.animator.getControllerLayer().getCurrentPlayState();
            if (playState != null && playState.animatorState.name == actionName && playState.normalizedTime >= 1) {
                Laya.timer.clear(this, this.playComplete);
                if (handler != null)
                    handler.run();
            }
        }
    }

    class ChuanScript extends AnimatorScript {
        constructor() {
            super();
            this.fllowTarget = null;
            this.transform = null;
            this.index = 0;
            this.cat = null;
        }
        onAwake() {
            super.onAwake();
            this.transform = this.owner.transform;
        }
        onLateUpdate() {
            if (this.fllowTarget == null)
                return;
            let position = this.fllowTarget.position;
            this.transform.position.cloneTo(position);
            this.fllowTarget.position = position;
        }
        onDestroy() {
            this.cat = null;
            this.index = 0;
        }
    }

    class CatChuanEventAction extends CatEventAction {
        constructor() {
            super();
            this.chuanScripte = null;
            this.Eff_shuiwen = null;
        }
        Run() {
            super.Run();
            let isCom = this.actionRes.param[0] == "true";
            if (isCom) {
                LoadQueue.Inst.loadSprite3D(UrlUtils.GetBuildUrl("cat2_boat"), Laya.Handler.create(this, this.onLoadComplete));
            }
            else {
                this.chuanScripte = EntityModel.Inst.GetChuan(this.cat);
                this.PlayChuanAni(CatChuanEventAction.AnimationArr_GO[this.chuanScripte.index], false);
            }
        }
        onLoadComplete(sp) {
            SceneMgr.Inst.AddMap(sp.name, sp);
            let chuan = sp.addComponent(ChuanScript);
            chuan.cat = this.cat;
            EntityModel.Inst.AddChuan(chuan);
            this.chuanScripte = chuan;
            SceneMgr.Inst.GetCameraScript().PlayState(CatChuanEventAction.AnimationArr_ACTIONNAME[this.chuanScripte.index], Laya.Handler.create(this, this.onCamereAniComplete, [this.chuanScripte.index]));
            this.PlayChuanAni(CatChuanEventAction.AnimationArr_Com[this.chuanScripte.index], true);
        }
        PlayAni() {
            this.cat.CrossFadeState(this.actionRes.actionName[0], null, 0);
        }
        PlayChuanAni(aniName, isUp) {
            this.chuanScripte.fllowTarget = this.cat.transform;
            this.chuanScripte.PlayState(aniName, Laya.Handler.create(this, this.onMoveComplete, [isUp]));
            let localRotationEuler = this.cat.transform.localRotationEuler;
            localRotationEuler.setValue(0, parseFloat(this.actionRes.param[1]), 0);
            this.cat.transform.localRotationEuler = localRotationEuler;
            this.AddEfect(isUp);
        }
        AddEfect(isUp) {
            LoadQueue.Inst.loadParticle(UrlUtils.GetEffectUrl("Eff_shuiwen"), Laya.Handler.create(this, this.onLoadEffectComplete, [isUp]));
        }
        onLoadEffectComplete(isUp, eff) {
            if (this.chuanScripte == null || eff == null)
                return;
            this.chuanScripte && this.chuanScripte.owner && this.chuanScripte.owner.addChild(eff);
            eff.transform.localRotationEuler = new Laya.Vector3(0, isUp ? 0 : 180, 0);
            eff.transform.localPosition = new Laya.Vector3(0, 0.3, 0);
        }
        onCamereAniComplete(index) {
        }
        onMoveComplete(isUp) {
            this.m_isComplete = true;
            GameDispatcher.Inst.event(UIGameEvent.CAT_COMPLETE_ONLAND, this.cat);
            if (isUp && !SceneMgr.Inst.GetCameraBuildScript().IsSelected()) {
                GameDispatcher.Inst.event(UIGameEvent.ISHIDE_ICON, false);
            }
        }
        Reset() {
            super.Reset();
            if (this.chuanScripte) {
                this.chuanScripte.cat = null;
                this.chuanScripte.fllowTarget = null;
                this.chuanScripte = null;
            }
            if (this.Eff_shuiwen && !this.Eff_shuiwen.destroyed) {
                this.Eff_shuiwen.destroy();
            }
            this.Eff_shuiwen = null;
        }
        GC() {
            ObjectPools.Inst.gcObj(CatChuanEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    CatChuanEventAction.KEY = "CatChuanEventAction";
    CatChuanEventAction.AnimationArr_Com = ["boat", "boat02", "boat03", "boat04"];
    CatChuanEventAction.AnimationArr_GO = ["go_01", "boat_go02", "boat_go03", "boat_go04"];
    CatChuanEventAction.AnimationArr_ACTIONNAME = ["matou_yaoqing", "matou_yaoqing02", "matou_yaoqing03", "matou_yaoqing"];

    class CatResetPositionRotationEventAction extends CatEventAction {
        constructor() {
            super();
        }
        Run() {
            super.Run();
            let position = this.cat.transform.position;
            position.setValue(-parseFloat(this.actionRes.param[0]), parseFloat(this.actionRes.param[1]), parseFloat(this.actionRes.param[2]));
            this.cat.transform.position = position;
            let rotationEuler = this.cat.transform.rotationEuler;
            rotationEuler.setValue(-parseFloat(this.actionRes.param[3]), 180 - parseFloat(this.actionRes.param[4]) + 180, parseFloat(this.actionRes.param[5]));
            this.cat.transform.rotationEuler = rotationEuler;
            this.onPlayComplete();
        }
        PlayAni() {
        }
        onPlayComplete() {
            this.m_isComplete = true;
        }
        Reset() {
            super.Reset();
        }
        GC() {
            ObjectPools.Inst.gcObj(CatResetPositionRotationEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    CatResetPositionRotationEventAction.KEY = "CatResetPositionRotationEventAction";

    class CatFishingEventAction extends CatEventAction {
        constructor() {
            super();
            this.gongjus = new Array();
        }
        Run() {
            super.Run();
            this.PlayAni1();
        }
        PlayAni() {
        }
        PlayAni1() {
            this.loadDaoju("cat2_equipRef_rod", "Bip001 L Hand");
            this.loadDaoju("cat2_equipRef_bucket", null);
            Laya.timer.once(5000, this, this.onDelayFish);
            this.cat.CrossFadeState(this.actionRes.actionName[0], Laya.Handler.create(this, this.PlayAni3));
        }
        PlayAni3() {
            if (this.cat == null)
                return;
            Laya.timer.once(1500, this, this.onPlayComplete);
            this.cat && this.cat.PlayState(this.actionRes.actionName[0]);
        }
        onPlayComplete() {
            this.m_isComplete = true;
        }
        onDelayFish() {
            this.loadDaoju("cat2_equipRef_fishAnim", null, Laya.Handler.create(this, this.onFishComplete));
        }
        loadDaoju(resName, nodeName, handler = null) {
            let url = UrlUtils.GetEntityLhUrl(resName);
            LoadQueue.Inst.loadSprite3D(url, Laya.Handler.create(this, this.onLoadComplete, [nodeName, handler]));
        }
        onLoadComplete(nodeName, handler, sp) {
            if (this.cat == null)
                return;
            let node = this.cat.owner;
            if (nodeName != null) {
                node = MathUtil.FindSprite3DByName(this.cat.owner, nodeName);
            }
            if (node == null)
                return;
            node.addChild(sp);
            this.gongjus.push(sp);
            if (handler != null)
                handler.runWith(sp);
        }
        onFishComplete(sp) {
            if (!sp)
                return;
            let animatorS = sp.addComponent(AnimatorScript);
            animatorS.PlayState("fishing_yu", Laya.Handler.create(this, this.onPlayFishComplete, [sp]));
        }
        onPlayFishComplete(sp) {
            if (sp && !sp.destroyed) {
                sp.destroy();
            }
        }
        Reset() {
            super.Reset();
            for (const sp of this.gongjus) {
                if (sp && !sp.destroyed) {
                    sp.destroy();
                }
            }
            this.gongjus.length = 0;
        }
        GC() {
            ObjectPools.Inst.gcObj(CatFishingEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    CatFishingEventAction.KEY = "CatFishingEventAction";

    class CatFollowTargetEventAction extends CatEventAction {
        constructor() {
            super();
        }
        Run() {
            super.Run();
            let isAdd = this.actionRes.param[0] == "true";
            if (isAdd) {
                SceneMgr.Inst.GetCameraBuildScript().FollowCat(this.cat);
            }
            else {
                SceneMgr.Inst.GetCameraBuildScript().OverFollowCat();
            }
            this.m_isComplete = true;
        }
        PlayAni() {
        }
        Reset() {
            super.Reset();
        }
        GC() {
            ObjectPools.Inst.gcObj(CatFollowTargetEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    CatFollowTargetEventAction.KEY = "CatFollowTargetEventAction";

    class Line {
        constructor(start, end) {
            this._StartPoint = null;
            this._EndPoint = null;
            this.StartPoint = start;
            this.EndPoint = end;
        }
        set StartPoint(value) {
            this._StartPoint = value;
        }
        get StartPoint() {
            return this._StartPoint;
        }
        set EndPoint(value) {
            this._EndPoint = value;
        }
        get EndPoint() {
            return this._EndPoint;
        }
        isMe(point) {
            if (this.StartPoint == point || this.EndPoint == point) {
                return true;
            }
            else {
                return false;
            }
        }
        GetPoint(t) {
            var point = new Laya.Vector3();
            if (t < 0) {
                t = 0;
            }
            else if (t > 1) {
                t = 1;
            }
            point.x = this.EndPoint.x - this.StartPoint.x;
            point.y = this.EndPoint.y - this.StartPoint.y;
            point.z = this.EndPoint.z - this.StartPoint.z;
            point.x *= t;
            point.y *= t;
            point.z *= t;
            point.x += this.StartPoint.x;
            point.y += this.StartPoint.y;
            point.z += this.StartPoint.z;
            return point;
        }
    }

    class Bezier {
        constructor() {
        }
        SetPoints(points) {
            if (points.length < 2) {
                throw (new Error("实例化贝塞尔曲线至少需要2个点"));
            }
            else {
                this.m_Points = new Array();
                this.createdLine = new Array();
                this.CreateLine(points);
            }
        }
        AddPoint(point) {
            this.m_Points.push(point);
            this.CreateLine(this.m_Points);
        }
        AddPointAt(index, point) {
            if (index >= 0 && index < this.m_Points.length) {
                this.m_Points.splice(index, 0, point);
                this.CreateLine(this.m_Points);
            }
            else {
                throw (new Error("索引超出范围"));
            }
        }
        RemovePoint(point) {
            if (this.m_Points.length > 2) {
                for (var i = 0; i < this.m_Points.length; i++) {
                    if (this.m_Points[i] == point) {
                        this.m_Points.splice(i, 1);
                        this.CreateLine(this.m_Points);
                    }
                    else {
                        continue;
                    }
                }
            }
            else {
                new Error("当前曲线锚点数量已经最低，不能移除锚点");
            }
        }
        RemovePointAt(index) {
            if (this.m_Points.length > 2) {
                this.m_Points.splice(index, 1);
                this.CreateLine(this.m_Points);
            }
            else {
                new Error("当前曲线锚点数量已经最低，不能移除锚点");
            }
        }
        UpdatePoint(ListIndex, point) {
            if (ListIndex < 0) {
                throw (new Error("坐标索引参数错误（取值必须大于0）"));
            }
            else if (ListIndex >= this.m_Points.length) {
                throw (new Error("坐标索引参数错误（取值必须x小于曲线顶点的个数）"));
            }
            else {
                this.m_Points[ListIndex] = point;
                this.CreateLine(this.m_Points);
            }
        }
        GetPoint(t) {
            var point = new Laya.Vector3();
            if (t < 0) {
                t = 0;
            }
            else if (t > 1) {
                t = 1;
            }
            var bufListLine = this.createdLine;
            if (bufListLine == null) {
                throw (new Error("曲线锚点为空"));
            }
            while (bufListLine.length > 1) {
                bufListLine = this.CaculateResoultLine(bufListLine, t);
            }
            if (bufListLine.length == 1) {
                point = bufListLine[0].GetPoint(t);
            }
            else {
                throw (new Error("Program Error : Current Line Count is:   " + bufListLine.length));
            }
            return point;
        }
        CaculateResoultLine(Lines, t) {
            let ListLine = new Array();
            for (var i = 0; i < Lines.length - 1; i++) {
                var j = i + 1;
                let bufLine = new Line(Lines[i].GetPoint(t), Lines[j].GetPoint(t));
                ListLine.push(bufLine);
            }
            return ListLine;
        }
        CreateLine(points) {
            this.createdLine = new Array();
            this.m_Points = points;
            for (var i = 0; i < points.length; i++) {
                let j = i + 1;
                if (j >= points.length) {
                    break;
                }
                else {
                    let curLine = new Line(points[i], points[j]);
                    this.createdLine.push(curLine);
                }
            }
        }
    }

    class BezierScript extends Laya.Script3D {
        constructor() {
            super();
            this.transform = null;
            this.bezierCurve = null;
            this.moveTime = 0;
            this.startTime = 0;
            this.randomAngle = null;
            this.handler = null;
            this.isRemove = false;
        }
        onAwake() {
            this.bezierCurve = new Bezier();
            this.transform = this.owner.transform;
        }
        Play(points, moveTime, handler = null, isRemove = true) {
            this.moveTime = moveTime;
            this.bezierCurve.SetPoints(points);
            this.startTime = Laya.timer.currTimer * 0.001;
            this.randomAngle = new Laya.Vector3(0, Math.random() * 6 - 3, Math.random() * 6 - 3);
            this.handler = handler;
            this.isRemove = isRemove;
        }
        onUpdate() {
            if (this.bezierCurve == null)
                return;
            let curTime = Laya.timer.currTimer * 0.001;
            let passTime = curTime - this.startTime;
            let rate = passTime / this.moveTime;
            if (rate <= 1) {
                this.transform.position = this.bezierCurve.GetPoint(rate);
            }
            else {
                if (this.handler) {
                    this.handler.run();
                    this.handler = null;
                }
                if (this.isRemove)
                    this.destroy();
            }
        }
        Reset() {
            if (this.handler) {
                this.handler.recover();
                this.handler = null;
            }
        }
    }

    class CatJumpTargetEventAction extends CatEventAction {
        constructor() {
            super();
            this.targetPos = null;
        }
        Run() {
            super.Run();
            this.targetPos = new Laya.Vector3(-parseFloat(this.actionRes.param[0]), parseFloat(this.actionRes.param[1]), parseFloat(this.actionRes.param[2]));
            this.FindPath();
        }
        PlayAni() {
            this.PlayAniIndex(0);
        }
        PlayAniIndex(index) {
            if (this.actionRes == null)
                return;
            if (index >= this.actionRes.actionName.length) {
                this.onMoveComplete();
                return;
            }
            this.cat.CrossFadeState(this.actionRes.actionName[index], Laya.Handler.create(this, this.PlayAniIndex, [++index]), 0, parseFloat(this.actionRes.param[5]));
        }
        FindPath() {
            let sPos = this.cat.transform.position.clone();
            let ePos = this.targetPos.clone();
            let topPos = sPos.y > ePos.y ? sPos : ePos;
            let dir = new Laya.Vector3();
            Laya.Vector3.subtract(ePos, sPos, dir);
            Laya.Vector3.scale(dir, 0.5, dir);
            let cPos = new Laya.Vector3(sPos.x + dir.x, topPos.y + 0.5, sPos.z + dir.z);
            let posArr = [sPos, cPos, ePos];
            let lookPos = ePos.clone();
            lookPos.y = sPos.y;
            this.cat.transform.lookAt(lookPos, Laya.Vector3["_Up"], false);
            this.cat.transform.localRotationEulerY += 180;
            let delyaTime = parseFloat(this.actionRes.param[3]);
            if (delyaTime > 0) {
                Laya.timer.once(delyaTime, this, this.OnDelayJump, [posArr]);
            }
            else {
                this.OnDelayJump(posArr);
            }
        }
        OnDelayJump(posArr) {
            let bezir = this.cat.owner.addComponent(BezierScript);
            bezir.Play(posArr, parseFloat(this.actionRes.param[4]) * 0.001, null);
        }
        onMoveComplete() {
            this.m_isComplete = true;
        }
        Reset() {
            super.Reset();
            Laya.timer.clear(this, this.OnDelayJump);
        }
        GC() {
            ObjectPools.Inst.gcObj(CatJumpTargetEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    CatJumpTargetEventAction.KEY = "CatJumpTargetEventAction";

    class MoveProper {
        constructor() {
            this.startPos = new Laya.Vector3();
            this.endPos = new Laya.Vector3();
            this.moveX = 0;
            this.moveY = 0;
            this.moveZ = 0;
            this.taskNeedTime = 0;
            this.taskStartTime = 0;
        }
        init(startPos, endPos, speed) {
            startPos.cloneTo(this.startPos);
            endPos.cloneTo(this.endPos);
            this.SetStartTime(Laya.timer.currTimer);
            this.moveX = this.endPos.x - this.startPos.x;
            this.moveY = this.endPos.y - this.startPos.y;
            this.moveZ = this.endPos.z - this.startPos.z;
            this.taskNeedTime = this.distance / speed * 1000;
        }
        get distance() {
            return Laya.Vector3.distance(this.startPos, this.endPos);
        }
        get Rate() {
            if (this.taskNeedTime == 0)
                return 1;
            let _rate = (Laya.timer.currTimer - this.taskStartTime) / this.taskNeedTime;
            if (_rate > 1)
                _rate = 1;
            return _rate;
        }
        SetStartTime(time) {
            this.taskStartTime = time;
        }
        get curX() {
            return this.startPos.x + this.Rate * this.moveX;
        }
        get curY() {
            return this.startPos.y + this.Rate * this.moveY;
        }
        get curZ() {
            return this.startPos.z + this.Rate * this.moveZ;
        }
        clear() {
            this.startPos.setValue(0, 0, 0);
            this.endPos.setValue(0, 0, 0);
            this.moveX = 0;
            this.moveY = 0;
            this.moveZ = 0;
            this.taskNeedTime = 0;
            this.taskStartTime = 0;
        }
    }

    class CatMoveEventAction extends CatEventAction {
        constructor() {
            super();
            this.moveProper = null;
            this.isMove = false;
            this._startTime = 0;
            this.moveProper = new MoveProper();
        }
        Run() {
            super.Run();
            this.isMove = false;
        }
        MoveTo(toPos, speed = 2) {
            if (toPos == null) {
                return;
            }
            this._startTime = Laya.timer.currTimer;
            this.isMove = true;
            let catPos = this.cat.transform.position;
            this.moveProper.init(catPos, toPos, speed);
            this.cat.transform.lookAt(toPos, new Laya.Vector3(0, 1, 0), false);
            let localRotationEuler = this.cat.transform.localRotationEuler;
            localRotationEuler.y += 180;
            localRotationEuler.x = 360 - this.cat.transform.localRotationEulerX;
            this.cat.transform.localRotationEuler = localRotationEuler;
        }
        advanceTime() {
            if (!this.isMove || this.cat == null)
                return;
            let position = this.cat.transform.position;
            position.setValue(this.moveProper.curX, this.moveProper.curY, this.moveProper.curZ);
            this.cat.transform.position = position;
            if (this.moveProper.Rate >= 1) {
                this.moveProper.clear();
                this.isMove = false;
                this.onMoveComplete();
            }
            else {
                if (Laya.timer.currTimer - this._startTime > 5000 && MathUtil.random(3000)) {
                    CatFecalBallCtrl.Inst.CheckFecal(this.cat);
                }
            }
        }
        onMoveComplete() {
        }
        Reset() {
            super.Reset();
            this.isMove = false;
            this._startTime = 0;
        }
    }

    class CatLineMoveEventAction extends CatMoveEventAction {
        constructor() {
            super();
        }
        Run() {
            super.Run();
            this.FindPath();
        }
        FindPath() {
            if (this.cat.curPathVO) {
                this.cat.curPathVO.catId = 0;
                this.cat.curPathVO = null;
            }
            let speed = this.actionRes.param.length > 3 ? parseFloat(this.actionRes.param[3]) : 2;
            this.MoveTo(new Laya.Vector3(-parseFloat(this.actionRes.param[0]), parseFloat(this.actionRes.param[1]), parseFloat(this.actionRes.param[2])), speed);
        }
        onMoveComplete() {
            this.m_isComplete = true;
        }
        Reset() {
            super.Reset();
        }
        GC() {
            ObjectPools.Inst.gcObj(CatLineMoveEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    CatLineMoveEventAction.KEY = "CatLineMoveEventAction";

    class PathNavigateModel {
        constructor() {
            this.m_pathArrs = new Array();
            this.m_paths = new Dictionary();
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new PathNavigateModel();
            return this._Inst;
        }
        AddPathNavigate(pathNavigateVO) {
            this.m_paths[pathNavigateVO.objId] = pathNavigateVO;
        }
        GetPathVOById(pathId) {
            return this.m_paths[pathId];
        }
        GetAllPaths() {
            return this.m_pathArrs;
        }
        UnlockPathPoint() {
            this.m_pathArrs.length = 0;
            for (const key in this.m_paths) {
                let pathPointVO = this.m_paths[key];
                this.m_pathArrs.push(pathPointVO);
                pathPointVO.addPaths();
            }
        }
        GetShortDistance(toPos) {
            let dis = Number.MAX_VALUE;
            let onPathVO = null;
            for (const pathVO of this.m_pathArrs) {
                let curDis = pathVO.GetDistanceSquared(toPos);
                if (curDis < dis) {
                    dis = curDis;
                    onPathVO = pathVO;
                }
            }
            return onPathVO;
        }
        FindPath2(sPos, toPos) {
            let curPathVO = this.GetShortDistance(sPos);
            let endPathVO = this.GetShortDistance(toPos);
            let mapArr = [];
            this.GetNearPath(toPos, curPathVO, endPathVO, null, mapArr);
            let pathArr = [];
            let length = mapArr.length;
            if (length > 0) {
                PathPointModel.Inst.FindPath2(sPos, mapArr[0].GetNearPath().position, null, mapArr[0].GetNearPath(), null, pathArr);
                let nextIndex = 0;
                let sPath = null;
                let ePath = null;
                length--;
                for (let i = 0; i < length; i++) {
                    nextIndex = i + 1;
                    sPath = mapArr[i].GetNearPath();
                    ePath = mapArr[nextIndex].GetNearPath();
                    PathPointModel.Inst.FindPath2(sPath.position, ePath.position, sPath, ePath, this.GetOnPath(sPath, pathArr), pathArr);
                }
            }
            let ePath = endPathVO.GetNearPath();
            PathPointModel.Inst.FindPath2(ePath.position, toPos, ePath, null, this.GetOnPath(ePath, pathArr), pathArr);
            this.CheckHasPath(0, pathArr);
            let mapNetVO = ObjectPools.Inst.getObj(MapNetVO.KEY, MapNetVO);
            mapNetVO.endPos = toPos;
            length = pathArr.length;
            let endIndex = length - 1;
            for (let i = endIndex; i >= 0; i--) {
                if (length >= 3) {
                    if (i == 0) {
                        if (PathPointModel.Inst.CheckNearLink(pathArr[i], pathArr[i + 1], sPos))
                            continue;
                    }
                }
                let vo = ObjectPools.Inst.getObj(MapNetVO.KEY, MapNetVO);
                vo.nextMap = mapNetVO;
                vo.endPos = pathArr[i].position;
                mapNetVO = vo;
            }
            return mapNetVO;
        }
        GetNearPath(toPos, curPathPointVO, endPathPointVO, onPathPointVO, outArr) {
            if (curPathPointVO == null)
                return;
            outArr.push(curPathPointVO);
            if (curPathPointVO == endPathPointVO)
                return;
            let nextPathPointVO = curPathPointVO.GetNearPathPointVO2(toPos, onPathPointVO);
            if (nextPathPointVO != null && nextPathPointVO != curPathPointVO) {
                this.GetNearPath(toPos, nextPathPointVO, endPathPointVO, curPathPointVO, outArr);
            }
        }
        GetOnPath(curPathVO, pathArr) {
            let pathVO = null;
            for (let i = pathArr.length - 1; i >= 0; i--) {
                pathVO = pathArr[i];
                if (pathVO != curPathVO)
                    return pathVO;
            }
            return null;
        }
        CheckHasPath(index, pathArr) {
            let length = pathArr.length;
            if (index >= length)
                return;
            let onIndex = index;
            let curIndex = index;
            let curPathVO = pathArr[index];
            for (let i = index + 2; i < length; i++) {
                if (curPathVO.CheckHasPath(pathArr[i].objId)) {
                    curIndex = i - 1;
                }
            }
            let lastIndex = curIndex - onIndex;
            if (lastIndex > 0)
                pathArr.splice(onIndex + 1, lastIndex);
            index++;
            this.CheckHasPath(index, pathArr);
        }
    }
    PathNavigateModel._Inst = null;

    class CatMoveTargetEventAction extends CatMoveEventAction {
        constructor() {
            super();
            this.mapNextVO = null;
        }
        Run() {
            super.Run();
            if (this.cat.curPathVO) {
                this.cat.curPathVO.catId = 0;
                this.cat.curPathVO = null;
            }
            let toPos = new Laya.Vector3(-parseFloat(this.actionRes.param[0]), parseFloat(this.actionRes.param[1]), parseFloat(this.actionRes.param[2]));
            this.mapNextVO = PathNavigateModel.Inst.FindPath2(this.cat.transform.position, toPos);
            this.FindPath();
        }
        FindPath() {
            if (this.mapNextVO == null) {
                this.m_isComplete = true;
                return;
            }
            this.MoveTo(this.mapNextVO.endPos, 8);
        }
        onMoveComplete() {
            if (this.mapNextVO != null) {
                let onMapNextVO = this.mapNextVO;
                this.mapNextVO = this.mapNextVO.nextMap;
                onMapNextVO.GC();
            }
            this.FindPath();
        }
        Reset() {
            super.Reset();
            if (this.mapNextVO) {
                this.mapNextVO.GC();
                this.mapNextVO = null;
            }
        }
        GC() {
            ObjectPools.Inst.gcObj(CatMoveTargetEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    CatMoveTargetEventAction.KEY = "CatMoveTargetEventAction";

    class CatPlayAniEventAction extends CatEventAction {
        constructor() {
            super();
        }
        Run() {
            super.Run();
        }
        PlayAni() {
            let isCross = this.actionRes.param[0] == "true";
            let time = parseFloat(this.actionRes.param[1]);
            let actionName = this.actionRes.actionName.length > 1 ? this.actionRes.actionName[MathUtil.makeRandom(0, this.actionRes.actionName.length)] : this.actionRes.actionName[0];
            if (time > 0) {
                Laya.timer.once(time, this, this.onPlayComplete);
                if (isCross)
                    this.cat.CrossFadeState(actionName);
                else
                    this.cat.PlayState(actionName);
            }
            else {
                if (isCross)
                    this.cat.CrossFadeState(actionName, Laya.Handler.create(this, this.onPlayComplete));
                else
                    this.cat.PlayState(actionName, Laya.Handler.create(this, this.onPlayComplete));
            }
        }
        onPlayComplete() {
            if (this.cat == null)
                return;
            this.m_isComplete = true;
        }
        Reset() {
            super.Reset();
            Laya.timer.clearAll(this);
        }
        GC() {
            ObjectPools.Inst.gcObj(CatPlayAniEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    CatPlayAniEventAction.KEY = "CatPlayAniEventAction";

    class CatRoationEventAction extends CatEventAction {
        constructor() {
            super();
        }
        Run() {
            super.Run();
            let rotationEuler = this.cat.transform.rotationEuler;
            rotationEuler.setValue(-parseFloat(this.actionRes.param[0]), 180 - parseFloat(this.actionRes.param[1]) + 180, parseFloat(this.actionRes.param[2]));
            this.cat.transform.rotationEuler = rotationEuler;
            this.onPlayComplete();
        }
        PlayAni() {
        }
        onPlayComplete() {
            this.m_isComplete = true;
        }
        Reset() {
            super.Reset();
        }
        GC() {
            ObjectPools.Inst.gcObj(CatRoationEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    CatRoationEventAction.KEY = "CatRoationEventAction";

    class CatWalkEventAction extends CatMoveEventAction {
        constructor() {
            super();
            this.count = 0;
            this.toPathVO = null;
        }
        Run() {
            super.Run();
            this.count = parseInt(this.actionRes.param[0]);
            if (this.count == 0)
                this.count = 1;
            this.FindPath();
        }
        FindPath() {
            let curPathVO = this.cat.curPathVO;
            if (curPathVO == null) {
                curPathVO = PathPointModel.Inst.GetShortDistance(this.cat.transform.position);
                this.MoveToPath(curPathVO);
            }
            else {
                this.MoveToRandomPath();
            }
        }
        MoveToRandomPath() {
            if (this.cat.curPathVO == null) {
                this.m_isComplete = true;
                return;
            }
            if (this.count <= 0) {
                this.m_isComplete = true;
                return;
            }
            this.count--;
            let curPathVO = this.cat.curPathVO;
            let toNextPahtVO = curPathVO.GetRandomNextPoint();
            if (toNextPahtVO == null) {
                this.m_isComplete = true;
                return;
            }
            curPathVO.catId = 0;
            this.MoveToPath(toNextPahtVO);
        }
        MoveToPath(toPathVO) {
            this.toPathVO = toPathVO;
            if (toPathVO) {
                toPathVO.catId = this.cat.catVO.objId;
                this.MoveTo(toPathVO.position);
            }
        }
        onMoveComplete() {
            this.cat.curPathVO = this.toPathVO;
            this.MoveToRandomPath();
        }
        advanceTime() {
            super.advanceTime();
        }
        Reset() {
            super.Reset();
            this.toPathVO = null;
        }
        GC() {
            ObjectPools.Inst.gcObj(CatWalkEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    CatWalkEventAction.KEY = "CatWalkEventAction";

    class ActionResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint16();
            this.eventId = byteArray.readUint8();
            this.type = byteArray.readUint8();
            this.actionName = ConfigLib.ReadArray(byteArray);
            this.param = ConfigLib.ReadArray(byteArray);
        }
        static getItemByKey(id) {
            return ActionResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return ActionResource.itemsDic[key1 + "," + key2];
        }
    }
    ActionResource.items = new Array();
    ActionResource.itemsDic = new Dictionary();

    class CatEventResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.eventId = byteArray.readUint8();
            this.type = byteArray.readUint8();
            this.weight = byteArray.readUint8();
        }
        static getItemByKey(id) {
            return CatEventResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return CatEventResource.itemsDic[key1 + "," + key2];
        }
    }
    CatEventResource.items = new Array();
    CatEventResource.itemsDic = new Dictionary();

    class CatBlackOrWhiteEventAction extends CatEventAction {
        constructor() {
            super();
        }
        Run() {
            super.Run();
            let isWhiteToBlack = this.actionRes.param[0] == "true";
            let time = parseFloat(this.actionRes.param[1]);
            if (isWhiteToBlack) {
                GameDispatcher.Inst.event(UIGameEvent.WHITE_TO_BLACK);
            }
            else {
                GameDispatcher.Inst.event(UIGameEvent.SHOW_MAIN_BTNS, false);
            }
            Laya.timer.once(time, this, this.onPlayComplete);
        }
        PlayAni() {
        }
        onPlayComplete() {
            this.m_isComplete = true;
        }
        Reset() {
            super.Reset();
            Laya.timer.clearAll(this);
        }
        GC() {
            ObjectPools.Inst.gcObj(CatBlackOrWhiteEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    CatBlackOrWhiteEventAction.KEY = "CatBlackOrWhiteEventAction";

    class TaskType {
    }
    TaskType.TASKTYPE_UNLOCK_BUILD = 0;
    TaskType.TASKTYPE_UP_BUILD = 1;
    TaskType.TASKTYPE_INVITE_CAT = 2;
    TaskType.TASKTYPE_GIVEQIULU = 3;
    TaskType.TASKTYPE_UP_CAT = 300000;
    TaskType.TASKTYPE_GET_CAT = 4;
    TaskType.TASKTYPE_MO_CAT = 5;
    TaskType.TASKTYPE_GET_MAOQIU = 6;
    TaskType.TASKTYPE_WEIMAO = 7;
    TaskType.TASKTYPE_LOCKDRAW = 8;
    TaskType.TASKTYPE_DRESSUP = 9;
    TaskType.TASKTYPE_PAIZHAO = 10;
    TaskType.TASKTYPE_DAYTASKCOMP = 11;
    TaskType.TASKTYPE_GIVEDRESS = 12;
    TaskType.TASKTYPE_CATBAIZS = 13;
    TaskType.TASKTYPE_BUILDLEVEL = 14;
    TaskType.TASKTYPE_CATLEVEL = 15;
    TaskType.TASKTYPE_BUYDAOJU = 16;
    TaskType.TASKTYPE_BUILDSCORE = 17;
    TaskType.TASKTYPE_ADDSERVER = 18;
    TaskType.TASKTYPE_UNLOCKISLAND = 19;
    TaskType.TASKTYPE_ADDMATOUNUM = 20;
    TaskType.TASKTYPE_ADDCATHOME = 21;

    class BuildPlayAniamtorScript extends AnimatorScript {
        constructor() {
            super();
            this.animators = null;
        }
        onAwake() {
            super.onAwake();
        }
        PlayAni(animators) {
            this.animators = animators;
            this.PlayArr(0);
        }
        PlayArr(index) {
            if (this.owner == null || this.owner.destroyed)
                return;
            let aniNameIndex = 2 * index;
            let aniTimeIndex = aniNameIndex + 1;
            if (aniNameIndex >= this.animators.length || aniTimeIndex > this.animators.length) {
                this.onOver();
                return;
            }
            index++;
            this.PlayState(this.animators[aniNameIndex]);
            Laya.timer.once(parseInt(this.animators[aniTimeIndex]), this, this.PlayArr, [index]);
        }
        onOver() {
            this.destroy();
        }
    }

    class CatEventCtrl {
        constructor() {
            this.checktime = 1000;
            this.endTime = 0;
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new CatEventCtrl();
            return this._Inst;
        }
        Init() {
            this.UpdateCheckTime();
            Laya.timer.loop(1000, this, this.OnCheckTime);
            GameDispatcher.Inst.on(UIGameEvent.ADD_FISH, this, this.OnAddFish);
        }
        UpdateCheckTime() {
            let lastTime = 0;
            if (this.endTime != 0)
                lastTime = (Laya.timer.currTimer - this.endTime) % this.checktime;
            let catCount = CatModel.Inst.GetCatCount();
            let config = null;
            if (catCount <= 5) {
                config = ConfigValue.getItemByKey("checktimeone");
            }
            else if (catCount <= 10) {
                config = ConfigValue.getItemByKey("checktimetwo");
            }
            else {
                config = ConfigValue.getItemByKey("checktimethree");
            }
            if (config) {
                this.checktime = parseInt(config.value) * 1000;
            }
            if (GlobalConfig.CheckCatEventTIme > 0) {
                this.checktime = GlobalConfig.CheckCatEventTIme;
            }
            this.endTime = Laya.timer.currTimer - lastTime + this.checktime;
        }
        OnCheckTime() {
            if (Laya.timer.currTimer >= this.endTime) {
                let lastTime = (Laya.timer.currTimer - this.endTime) % this.checktime;
                this.endTime = Laya.timer.currTimer - lastTime + this.checktime;
                this.CheckAllEvent();
            }
        }
        CheckAllEvent() {
            let builds = BuildModel.Inst.GetBuildVOs();
            let kongBuilds = [];
            for (const buildVO of builds) {
                if (buildVO.HasKongZuowei()) {
                    kongBuilds.push(buildVO);
                }
            }
            if (kongBuilds.length == 0)
                return;
            let buildVO = kongBuilds[MathUtil.makeRandom(0, kongBuilds.length)];
            if (buildVO == null)
                return;
            this.CheckBuildEvent(buildVO);
        }
        CheckBuildEvent(buildVO) {
            let cats = EntityModel.Inst.GetCats();
            let kongCats = [];
            for (const key in cats) {
                let cat = cats[key];
                if (cat != null && cat.IsCanPlayBuild()) {
                    kongCats.push(cat);
                }
            }
            let cat = kongCats[MathUtil.makeRandom(0, kongCats.length)];
            if (cat == null)
                return;
            cat.SetBuildState(buildVO);
        }
        OnAddFish() {
            if (!AddFishModel.Inst.HasFish())
                return;
            let buildVO = BuildModel.Inst.maowuBuildVO;
            for (let i = 0; i < 3; i++) {
                this.CheckBuildEvent(buildVO);
            }
        }
    }
    CatEventCtrl._Inst = null;

    class BuildSignScript extends BaseScript {
        constructor() { super(); }
        onAwake() {
            super.onAwake();
        }
        get buildVO() {
            return this.vo;
        }
        onReset() {
            super.onReset();
        }
        onDestroy() {
            super.onDestroy();
        }
    }

    class BuildSign extends Entity {
        constructor(displayObject) {
            super(displayObject);
            this.bs = null;
            this.onAwake();
        }
        onAwake() {
            super.onAwake();
            this.bs = this._displayObject.getComponent(BuildSignScript) || this._displayObject.addComponent(BuildSignScript);
        }
        SetCollider(owner) {
        }
        SetVO(vo) {
            super.SetVO(vo);
            let buildVO = BuildModel.Inst.GetBuildVOBy(this.signVO.buildId);
            this.bs.SetVO(buildVO);
        }
        get signVO() {
            return this._vo;
        }
        dispose() {
            super.dispose();
        }
    }

    class SignModel {
        constructor() {
            this.signPool = {};
        }
        static get Inst() {
            return this.inst = this.inst || new SignModel();
        }
        init() {
        }
        AddSingVO(singVo) {
            this.signPool[singVo.buildId] = singVo;
        }
        GetSignVO(buildId) {
            return this.signPool[buildId];
        }
    }
    SignModel.inst = null;

    class CameraModel {
        constructor() {
        }
        static get Inst() {
            return this.inst = this.inst || new CameraModel();
        }
        init() {
            CameraModel.FOV_NORMAL = parseInt(ConfigValue.getItemByKey("FOV_NORMAL").value);
        }
    }
    CameraModel.inst = null;
    CameraModel.FOV_NORMAL = 44;

    class CameraCtrl {
        constructor() {
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new CameraCtrl();
            return this._Inst;
        }
        playBanjiaCameraAni(landLevel = 2, completeHandler = null) {
            let name = "Land2_build_camera";
            if (LandModel.Inst.level >= 3) {
                name = "land3_build_camera";
            }
            let camera = SceneMgr.Inst.GetCamera();
            camera.fieldOfView = CameraModel.FOV_NORMAL;
            SceneMgr.Inst.GetCameraBuildScript().ResetCameraParent();
            let oldPos = camera.transform.position.clone();
            let oldRot = camera.transform.rotationEuler.clone();
            let landRes = islandResource.getItemByKey(landLevel);
            let cameraTweenStart = landRes.cameraTweenStart;
            let rotationEuler = camera.transform.rotationEuler;
            rotationEuler.setValue(-parseFloat(cameraTweenStart[0]), parseFloat(cameraTweenStart[1]), parseFloat(cameraTweenStart[2]));
            camera.transform.rotationEuler = rotationEuler;
            let position = camera.transform.position;
            position.setValue(-parseFloat(cameraTweenStart[3]), 180 - parseFloat(cameraTweenStart[4]), parseFloat(cameraTweenStart[5]));
            camera.transform.position = position;
            SceneMgr.Inst.GetCameraScript().PlayState(name, Laya.Handler.create(this, (oldPos, oldRot, completeHandler) => {
                let camera = SceneMgr.Inst.GetCamera();
                camera.transform.position = oldPos;
                camera.transform.rotationEuler = oldRot;
                completeHandler && completeHandler.run();
            }, [oldPos, oldRot, completeHandler]));
        }
        OverSelected() {
            SceneMgr.Inst.GetCameraBuildScript().OverBuildSelected();
            SceneMgr.Inst.GetCameraBuildScript().OverSelected();
        }
        MoveBuild(buildVO) {
            if (buildVO == null)
                return;
            SceneMgr.Inst.GetCameraBuildScript().MoveBuild(buildVO);
        }
        MoveCat(catVO) {
            if (catVO == null)
                return;
            SceneMgr.Inst.GetCameraBuildScript().PanningFollowCat(catVO);
        }
        OverFollowCat() {
            SceneMgr.Inst.GetCameraBuildScript().OverFollowCat();
        }
        cameraIsTween() {
            return SceneMgr.Inst.GetCameraBuildScript().cameraIsTween();
        }
        MoveToBorn(complete = null) {
            let camera = SceneMgr.Inst.GetCamera();
            camera.transform.position = new Laya.Vector3(25.47, 73.68, -48.15);
            camera.transform.rotationEuler = new Laya.Vector3(-47.027, 180 - 36.397, 0);
            camera.fieldOfView = 90;
            Laya.Tween.to(camera, { fieldOfView: CameraModel.FOV_NORMAL }, 3000, Laya.Ease.sineOut, Laya.Handler.create(this, () => {
                complete && complete.run();
                GameDispatcher.Inst.event(UIGameEvent.SCENE_RESIZE);
                GameDispatcher.Inst.event(UIGameEvent.ISHIDE_ICON, false);
            }));
        }
    }
    CameraCtrl._Inst = null;

    class LandCtrl {
        constructor() {
            this.buildSignMap = {};
        }
        static get Inst() {
            return this.inst = this.inst || new LandCtrl();
        }
        init() {
            let lv = LandModel.Inst.level;
            let scene = SceneMgr.Inst.GetScene();
            let parent = scene.getChildAt(0);
            let landNode = null;
            for (let i = 1; i <= 3; i++) {
                landNode = parent.getChildByName("Land0" + i);
                landNode.active = i <= lv;
            }
        }
        moveHouse() {
            GameDispatcher.Inst.event(UIGameEvent.ISHIDE_ICON, true);
            let vo = BuildModel.Inst.moveHouseBuildVO;
            BuildCtrl.Inst.RemoveBuild(vo);
            LandModel.Inst.moveHouse();
            GameDispatcher.Inst.event(UIGameEvent.START_MOVE_HOUSE);
            let camera = SceneMgr.Inst.GetCamera();
            let name = "Land2_build_camera";
            if (LandModel.Inst.level >= 3) {
                name = "land3_build_camera";
            }
            let scene = SceneMgr.Inst.GetScene();
            let parent = scene.getChildAt(0);
            let landNode = parent.getChildByName("Land0" + LandModel.Inst.level);
            if (landNode) {
                landNode.active = true;
            }
            CameraCtrl.Inst.playBanjiaCameraAni(LandModel.Inst.level, Laya.Handler.create(this, this.endMoveHouse));
            SceneMgr.Inst.UpdateLand(LandModel.Inst.level);
        }
        endMoveHouse(oldPos, oldRot) {
            BuildCtrl.Inst.CheckCanUnlockBUild();
            SceneMgr.Inst.GetCameraBuildScript().OverSelected();
            GameDispatcher.Inst.event(UIGameEvent.END_MOVE_HOUSE);
            PathPointModel.Inst.UnlockPathPoint(LandModel.Inst.level);
        }
        AddSign(buildVo) {
            let buildId = buildVo instanceof BuildVO ? buildVo.buildRes.id : buildVo;
            if (buildId <= 0 || this.GetBuildSign(buildId) != null) {
                return;
            }
            let signVo = SignModel.Inst.GetSignVO(buildId);
            if (signVo) {
                let buildVO = BuildModel.Inst.GetBuildVOBy(signVo.buildId);
                if (buildVO && buildVO.isJingYuBanjia) {
                    return;
                }
                LoadQueue.Inst.loadSprite3D(UrlUtils.GetBuildUrl("cat2_unlock"), Laya.Handler.create(this, (signVo, sp) => {
                    sp.transform.position = signVo.position;
                    sp.transform.rotationEuler = signVo.rotationEuler;
                    sp.name = "cat2_unlock_" + signVo.buildId;
                    SceneMgr.Inst.AddMap("cat2_unlock", sp);
                    let buildSign = new BuildSign(sp);
                    buildSign.SetVO(signVo);
                    this.buildSignMap[signVo.buildId] = buildSign;
                }, [signVo]));
            }
            else {
                Logger.log(this, `场景中没有导出建筑[${buildId}]的指示牌数据...`);
            }
        }
        RemoveSign(buildId) {
            let buildSign = this.GetBuildSign(buildId);
            if (buildSign) {
                SceneMgr.Inst.RemoveMap("cat2_unlock", buildSign.owner);
                buildSign.dispose();
            }
            delete this.buildSignMap[buildId];
        }
        GetBuildSign(buildId) {
            return this.buildSignMap[buildId];
        }
    }
    LandCtrl.inst = null;

    class BuildCtrl {
        constructor() {
            this.unlockHandlerMap = {};
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new BuildCtrl();
            return this._Inst;
        }
        init() {
            GameDispatcher.Inst.on(UIGameEvent.BUILD_UNLOCK, this, this.UnlockBuild);
            GameDispatcher.Inst.on(UIGameEvent.BUILD_ADD, this, this.addBuild);
            GameDispatcher.Inst.on(UIGameEvent.UPDATE_CURRENCY_DATA, this, this.onMoneyUpdate);
        }
        onMoneyUpdate() {
            this.CheckCanLevelUpBuild();
        }
        CheckCanLevelUpBuild() {
            if (GuideModel.Inst.guideVO != null) {
                return;
            }
            let lessCostBuildVO = null;
            let buildVos = BuildModel.Inst.GetBuildVOs();
            let buildVo;
            let prevCost;
            for (let i = 0; i < buildVos.length; i++) {
                buildVo = buildVos[i];
                if (!buildVo || !buildVo.IsUnLock() || buildVo.IsMaxLv() || buildVo.isJingYuBanjia || buildVo.isMatou || buildVo.isMaowo) {
                    continue;
                }
                let costGold = buildVo.GetLvUpCostGold();
                if (MoneyModel.gold.isGreaterThanOrEqualTo(costGold)) {
                    if (prevCost == null || costGold.isLessThan(prevCost)) {
                        lessCostBuildVO = buildVo;
                    }
                }
                prevCost = costGold;
            }
            GameDispatcher.Inst.event(UIGameEvent.BUILD_LEVEL_LESS, lessCostBuildVO);
        }
        CheckAllBuild() {
            this.CheckCanUnlockBUild();
            this.CheckMatou();
        }
        RemoveBuild(vo) {
            MapQuadTree.Inst.RemoveVO(vo);
            BuildModel.Inst.RemoveBuildVO(vo);
            GameDispatcher.Inst.event(UIGameEvent.BUILD_REMOE, [vo]);
        }
        CheckCanUnlockBUild() {
            GameDispatcher.Inst.event(UIGameEvent.BUILD_CANUNLOCK_ICON, [null, false]);
            let buildVOs = BuildModel.Inst.GetBuildVOs();
            for (const buildVO of buildVOs) {
                if (!buildVO)
                    continue;
                if (buildVO.IsCanUnlock()) {
                    LandCtrl.Inst.AddSign(buildVO);
                    GameDispatcher.Inst.event(UIGameEvent.BUILD_CANUNLOCK_ICON, [buildVO, true]);
                }
                else if (!buildVO.IsUnLock() && buildVO.land <= LandModel.Inst.level) {
                    LandCtrl.Inst.AddSign(buildVO);
                }
                else {
                    LandCtrl.Inst.RemoveSign(buildVO.buildRes.id);
                }
            }
            if (LandModel.Inst.level >= 2) {
                LandCtrl.Inst.AddSign(301);
            }
        }
        CheckMatou() {
            let isCd = true;
            let isYaoqing = false;
            let nowTime = (new Date()).getTime();
            let matouNum = LandModel.Inst.boatNum;
            for (let i = 0; i < matouNum; i++) {
                let matou = MatouModel.Inst.GetMatouByIndex(i);
                if (matou == null) {
                    isCd = false;
                }
                else {
                    let endTime = matou["endTime"];
                    if (endTime > 0 && nowTime < endTime) {
                    }
                    else {
                        isCd = false;
                        isYaoqing = true;
                    }
                }
            }
            if (isYaoqing) {
                GameDispatcher.Inst.event(UIGameEvent.BUILD_ADD_OR_REMOVE_ICON, [BuildModel.Inst.matouBuildVO, true, "matou_shangdao"]);
                return;
            }
            if (isCd) {
                GameDispatcher.Inst.event(UIGameEvent.BUILD_ADD_OR_REMOVE_ICON, [BuildModel.Inst.matouBuildVO, true, "matou_guanggao"]);
                return;
            }
            GameDispatcher.Inst.event(UIGameEvent.BUILD_ADD_OR_REMOVE_ICON, [BuildModel.Inst.matouBuildVO, true, "matou_xunzhao"]);
        }
        addBuild(buildVO) {
            if (buildVO) {
                let handler = this.unlockHandlerMap[buildVO.buildRes.id];
                handler && handler.run();
            }
        }
        UnlockBuild(buildVO) {
            if (buildVO) {
                this.unlockHandlerMap[buildVO.buildRes.id] = Laya.Handler.create(this, this.playUnlockAni, [buildVO]);
            }
        }
        playUnlockAni(buildVO) {
            if (!buildVO)
                return;
            delete this.unlockHandlerMap[buildVO.buildRes.id];
            let build = EntityModel.Inst.GetBuild(buildVO.objId);
            if (build && buildVO.level == 1) {
                build.PlayCreateAni(Laya.Handler.create(this, this.onUnlockPlayComplete, [buildVO]));
                CatCtrl.Inst.SetCatWg(buildVO, buildVO.isDiaoyuchan ? Laya.Handler.create(this, this.onPlayDiaoyuWeiguanComplete, [buildVO]) : null);
            }
        }
        onUnlockPlayComplete(buildVO) {
            PlatformMgr.Inst.aldSendEvent(EventType.jianzhujiesuo, buildVO.buildRes.id + "_" + buildVO.buildRes.name);
            if (buildVO.isCatHouse)
                return;
            CatEventCtrl.Inst.CheckBuildEvent(buildVO);
        }
        onPlayDiaoyuWeiguanComplete(buildVO) {
            CatEventCtrl.Inst.CheckBuildEvent(buildVO);
            GuideModel.Inst.endWeiguanDiaoyuchang = true;
        }
        BuildLevelUp(buildVO, add = 1, isAd = false, handler = null) {
            if (buildVO == null)
                return;
            if (buildVO.IsMaxLv())
                return;
            if (!isAd) {
                let costGold = buildVO.GetLvUpCostGold();
                if (MoneyModel.gold.isLessThan(costGold)) {
                    return I18nUtils.ShowI18nById("jbbz");
                }
                MoneyModel.gold.cost(costGold);
                this.levelUpBuild(buildVO, add, null, true);
                GameDispatcher.Inst.event(UIGameEvent.UPDATE_CURRENCY_DATA);
            }
            else {
                PlatformMgr.Inst.showAd(AdType.ad_31, Laya.Handler.create(this, this.levelUpBuild, [buildVO, add, handler]));
            }
        }
        BuildLevelUp2(buildVO, add = 1) {
            if (buildVO == null)
                return add;
            if (buildVO.IsMaxLv()) {
                return add;
            }
            let costGold = buildVO.GetLvUpCostGold();
            if (MoneyModel.gold.isLessThan(costGold)) {
                return add;
            }
            MoneyModel.gold.cost(costGold);
            this.levelUpBuild(buildVO, 1, null, true);
            add--;
            if (add > 0) {
                add = this.BuildLevelUp2(buildVO, add);
            }
            GameDispatcher.Inst.event(UIGameEvent.UPDATE_CURRENCY_DATA);
            return add;
        }
        levelUpBuild(buildVO, add = 1, handler = null, isSuccess) {
            if (isSuccess) {
                if (!buildVO)
                    return;
                buildVO.level += add;
                if (buildVO.level > buildVO.curLandMaxLevel) {
                    buildVO.level = buildVO.curLandMaxLevel;
                }
                if (buildVO.isMatou) {
                    GameDispatcher.Inst.event(UIGameEvent.ONMATOU_UPLV);
                }
                else if (buildVO.isMaowo) {
                    GameDispatcher.Inst.event(UIGameEvent.ONMAOWU_UPLV);
                }
            }
            handler && handler.runWith(isSuccess);
        }
        MaoWuStageUp() {
            if (BuildModel.Inst.isMaowuMaxLevel)
                return;
            let vo = BuildModel.Inst.maowuBuildVO;
            let build = EntityModel.Inst.GetBuild(vo.objId);
            if (build) {
                let costGold = BuildModel.Inst.nextMaowuUpRes.cost;
                if (MoneyModel.gold.isLessThan(costGold)) {
                    return I18nUtils.ShowI18nById("jbbz");
                }
                GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, TaskType.TASKTYPE_ADDCATHOME);
                BuildModel.Inst.maowuBuildVO.stage++;
                build.PlayStageUp();
                build.BuildStageUpAni();
                GameDispatcher.Inst.event(UIGameEvent.ONMAOWU_UPSTAGE);
            }
        }
        BuildZuoweiLvUp(buildVO) {
            if (buildVO == null) {
                return;
            }
            buildVO.zwCount += 1;
            let build = EntityModel.Inst.GetBuild(buildVO.objId);
            if (build) {
                build.onChangeZuoweiParts(true);
            }
        }
        PlayBuildAnimator(animators, buildId, nodeName) {
            let buildVO = BuildModel.Inst.GetBuildVOBy(buildId);
            if (buildVO == null)
                return;
            let build = EntityModel.Inst.GetBuild(buildVO.objId);
            if (build == null)
                return;
            let node = MathUtil.FindSprite3DByName(build.owner, nodeName);
            if (node == null)
                return;
            let animator = node.addComponent(BuildPlayAniamtorScript);
            animator.PlayAni(animators);
        }
        BuildAddTeShuPeiJian(itemVo) {
            if (!itemVo || itemVo.itemRes.type != ItemModel.TYPE_BUILD)
                return;
            let buildVO;
            let buildVos = BuildModel.Inst.GetBuildVOs();
            let itemId = itemVo.itemRes.id;
            for (let i = 0; i < buildVos.length; i++) {
                const buildvo = buildVos[i];
                if (buildvo.buildRes.itemId1 == itemId || buildvo.buildRes.itemId2 == itemId) {
                    buildVO = buildvo;
                    break;
                }
            }
            if (!buildVO)
                return;
            let build = EntityModel.Inst.GetBuild(buildVO.objId);
            buildVO.UpdateTeShuPeiJian();
            if (build) {
                build.onChangeTeShuPeiJianParts(true);
            }
        }
    }
    BuildCtrl._Inst = null;

    class CatPlayBuildAnimatorEventAction extends CatEventAction {
        constructor() {
            super();
        }
        Run() {
            super.Run();
            let animators = this.actionRes.param[0];
            let buildId = parseInt(this.actionRes.param[1]);
            let nodeName = this.actionRes.param[2];
            BuildCtrl.Inst.PlayBuildAnimator(animators, buildId, nodeName);
            this.m_isComplete = true;
        }
        PlayAni() {
        }
        Reset() {
            super.Reset();
        }
        GC() {
            ObjectPools.Inst.gcObj(CatPlayBuildAnimatorEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    CatPlayBuildAnimatorEventAction.KEY = "CatPlayBuildAnimatorEventAction";

    class BuildAddTargetScript extends AnimatorScript {
        constructor() {
            super();
            this.node = null;
            this.fllowTarget = null;
        }
        onAwake() {
            super.onAwake();
        }
        onUpdate() {
            if (this.node == null || this.fllowTarget == null)
                return;
            let node = this.node;
            let position = this.fllowTarget.transform.position;
            node.transform.position.cloneTo(position);
            this.fllowTarget.transform.position = position;
            let rotationEuler = this.fllowTarget.transform.rotationEuler;
            node.transform.rotationEuler.cloneTo(rotationEuler);
            this.fllowTarget.transform.rotationEuler = rotationEuler;
        }
        AddNode(target, time) {
            this.fllowTarget = target;
            let onPosition = target.transform.position.clone();
            let onRotationEuler = target.transform.rotationEuler.clone();
            let onScale = target.transform.getWorldLossyScale().clone();
            let node = new Laya.Sprite3D();
            this.owner.addChild(node);
            let position = node.transform.position;
            onPosition.cloneTo(position);
            node.transform.position = position;
            let rotationEuler = node.transform.rotationEuler;
            onRotationEuler.cloneTo(rotationEuler);
            node.transform.rotationEuler = rotationEuler;
            let scale = node.transform.getWorldLossyScale();
            onScale.cloneTo(scale);
            node.transform.setWorldLossyScale(scale);
            this.node = node;
            Laya.timer.once(time, this, this.onOver, [onPosition, onRotationEuler, onScale]);
        }
        onOver(pos, rot, sc) {
            if (this.node != null) {
                if (!this.node.destroyed)
                    this.node.destroy();
                this.node = null;
            }
            this.fllowTarget = null;
            this.destroy();
        }
    }

    class CatAddBuildNodeEventAction extends CatEventAction {
        constructor() {
            super();
        }
        Run() {
            super.Run();
            let buildId = parseInt(this.actionRes.param[0]);
            let nodeName = this.actionRes.param[1];
            let time = parseInt(this.actionRes.param[2]);
            this.m_isComplete = true;
            let buildVO = BuildModel.Inst.GetBuildVOBy(buildId);
            if (buildVO == null)
                return;
            let build = EntityModel.Inst.GetBuild(buildVO.objId);
            if (build == null)
                return;
            let node = MathUtil.FindSprite3DByName(build.owner, nodeName);
            if (node == null)
                return;
            let addTargetScripte = node.addComponent(BuildAddTargetScript);
            addTargetScripte.AddNode(this.cat.owner, time);
        }
        PlayAni() {
        }
        Reset() {
            super.Reset();
        }
        GC() {
            ObjectPools.Inst.gcObj(CatAddBuildNodeEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    CatAddBuildNodeEventAction.KEY = "CatAddBuildNodeEventAction";

    class QiziScript extends Laya.Script3D {
        constructor() { super(); }
        onAwake() {
            super.onAwake();
        }
        SetHead(resName) {
            LoadQueue.Inst.loadTexture(UrlUtils.GetCatHead(resName), Laya.Handler.create(this, this.onComplete));
        }
        onComplete(tex) {
            if (tex == null)
                return;
            let abyssinian = MathUtil.FindSprite3DByName(this.owner, "abyssinian");
            if (abyssinian == null)
                return;
            let sharedMaterial = abyssinian.skinnedMeshRenderer.sharedMaterial;
            if (sharedMaterial) {
                sharedMaterial.albedoTexture = tex;
            }
        }
        onDestroy() {
        }
    }

    class SceneAddOrRemoveEventAction extends CatEventAction {
        constructor() {
            super();
        }
        Run() {
            super.Run();
            let actions = this.actionRes.param;
            for (let i = 0; i < actions.length; i++) {
                this.ParseParms(actions[i], this.cat.catVO.resName);
            }
            this.m_isComplete = true;
        }
        ParseParms(params, catName) {
            let delayTime = parseInt(params[1]);
            if (delayTime > 0) {
                Laya.timer.once(delayTime, this, this.OnDelayTime, [params, catName], false);
            }
            else {
                this.OnDelayTime(params, catName);
            }
        }
        OnDelayTime(params, catName) {
            let isAdd = params[0] == "true";
            let resName = params[2];
            if (isAdd) {
                this.AddGongju(resName, params, catName);
            }
            else {
                this.RemoveGongju(resName);
            }
        }
        PlayAni() {
        }
        AddGongju(resName, params, catName, handler = null) {
            let resType = params.length > 9 ? params[9] : 1;
            let url = "";
            if (resType == 1) {
                url = UrlUtils.GetEntityLhUrl(resName);
                LoadQueue.Inst.loadSprite3D(url, Laya.Handler.create(this, this.onLoadComplete, [resName, params, catName, handler]));
            }
            else if (resType == 2) {
                url = UrlUtils.GetEffectUrl(resName);
                LoadQueue.Inst.loadParticle(url, Laya.Handler.create(this, this.onLoadComplete, [resName, params, catName, handler]));
            }
        }
        onLoadComplete(resName, params, catName, handler, sp) {
            if (sp == null)
                return;
            sp.name = resName;
            SceneMgr.Inst.AddEvent(sp.name, sp);
            let positon = sp.transform.position;
            positon.setValue(-parseFloat(params[3]), parseFloat(params[4]), parseFloat(params[5]));
            sp.transform.position = positon;
            let rotationEuler = sp.transform.rotationEuler;
            rotationEuler.setValue(-parseFloat(params[6]), parseFloat(params[7]), parseFloat(params[8]));
            sp.transform.rotationEuler = rotationEuler;
            if (sp.name == "qizi") {
                let sc = sp.addComponent(QiziScript);
                sc.SetHead(catName);
            }
            if (handler != null)
                handler.runWith(sp);
        }
        RemoveGongju(resName) {
            let node = MathUtil.FindSprite3DByName(SceneMgr.Inst.GetEvent(resName), resName);
            if (node == null || node.destroyed)
                return;
            node.destroy();
        }
        Reset() {
            super.Reset();
        }
        GC() {
            ObjectPools.Inst.gcObj(SceneAddOrRemoveEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    SceneAddOrRemoveEventAction.KEY = "SceneAddOrRemoveEventAction";

    class CatPlayYueyueAniEventAction extends CatEventAction {
        constructor() {
            super();
        }
        Run() {
            super.Run();
        }
        PlayAni() {
            let isCross = this.actionRes.param[0] == "true";
            let time = parseFloat(this.actionRes.param[1]);
            let actionName = this.actionRes.actionName.length > 1 ? this.actionRes.actionName[MathUtil.makeRandom(0, this.actionRes.actionName.length)] : this.actionRes.actionName[0];
            if (time > 0) {
                Laya.timer.once(time, this, this.onPlayComplete);
                if (isCross)
                    this.cat.CrossFadeState(actionName);
                else
                    this.cat.PlayState(actionName);
            }
            else {
                if (isCross)
                    this.cat.CrossFadeState(actionName, Laya.Handler.create(this, this.onPlayComplete));
                else
                    this.cat.PlayState(actionName, Laya.Handler.create(this, this.onPlayComplete));
            }
            let position = this.cat.transform.position.clone();
            position.y = 4;
            EffectMgr.Inst.playConfigEffect(EffectMgr.Eff_shengji_flower, this.cat.owner.parent, null, position);
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.event_happy));
        }
        onPlayComplete() {
            if (this.cat == null)
                return;
            this.m_isComplete = true;
        }
        Reset() {
            super.Reset();
            Laya.timer.clearAll(this);
        }
        GC() {
            ObjectPools.Inst.gcObj(CatPlayYueyueAniEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    CatPlayYueyueAniEventAction.KEY = "CatPlayYueyueAniEventAction";

    class CatPlaySceneAnimatorEventAction extends CatEventAction {
        constructor() {
            super();
            this.resName = null;
        }
        Run() {
            super.Run();
            this.resName = this.actionRes.param[1];
        }
        PlayStart(sp) {
            let nodeName = this.actionRes.param[2];
            let node = MathUtil.FindSprite3DByName(sp, nodeName);
            if (node == null)
                return;
            let animators = this.actionRes.param[0];
            let animator = node.addComponent(BuildPlayAniamtorScript);
            animator.PlayAni(animators);
            this.m_isComplete = true;
        }
        PlayAni() {
        }
        advanceTime() {
            let sp = SceneMgr.Inst.GetEvent(this.resName);
            if (sp == null)
                return;
            this.PlayStart(sp);
        }
        Reset() {
            super.Reset();
            this.resName = null;
        }
        GC() {
            ObjectPools.Inst.gcObj(CatPlaySceneAnimatorEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    CatPlaySceneAnimatorEventAction.KEY = "CatPlaySceneAnimatorEventAction";

    class CatBuildNodeMoverEventAction extends CatEventAction {
        constructor() {
            super();
        }
        Run() {
            super.Run();
            let longTime = 0;
            let actionArr = this.actionRes.param;
            for (let i = 0; i < actionArr.length; i++) {
                let actions = actionArr[i];
                let buildId = parseInt(actions[0]);
                let nodeName = actions[1];
                let time = parseFloat(actions[2]);
                if (time > longTime) {
                    longTime = time;
                }
                let buildVO = BuildModel.Inst.GetBuildVOBy(buildId);
                if (buildVO == null)
                    continue;
                let build = EntityModel.Inst.GetBuild(buildVO.objId);
                if (build == null)
                    continue;
                let node = MathUtil.FindSprite3DByName(build.owner, nodeName);
                if (node == null)
                    continue;
                let sx = -parseFloat(actions[3]);
                let sy = parseFloat(actions[4]);
                let sz = parseFloat(actions[5]);
                let srx = parseFloat(actions[6]);
                let sry = -parseFloat(actions[7]);
                let srz = parseFloat(actions[8]);
                let ex = -parseFloat(actions[9]);
                let ey = parseFloat(actions[10]);
                let ez = parseFloat(actions[11]);
                let erx = parseFloat(actions[12]);
                let ery = -parseFloat(actions[13]);
                let erz = parseFloat(actions[14]);
                let localPosition = node.transform.localPosition;
                localPosition.setValue(sx, sy, sz);
                node.transform.localPosition = localPosition;
                let localRotationEuler = node.transform.localRotationEuler;
                localRotationEuler.setValue(srx, sry, srz);
                node.transform.localRotationEuler = localRotationEuler;
                Laya.Tween.to(node.transform, { "localPositionX": ex, "localPositionY": ey, "localPositionZ": ez, "localRotationEulerX": erx, "localRotationEulerY": ery, "localRotationEulerZ": erz }, time);
            }
            Laya.timer.once(longTime, this, this.onActionComplete, null, false);
        }
        PlayAni() {
        }
        onActionComplete() {
            this.m_isComplete = true;
        }
        Reset() {
            super.Reset();
            Laya.timer.clear(this, this.onActionComplete);
        }
        GC() {
            ObjectPools.Inst.gcObj(CatBuildNodeMoverEventAction.KEY, this);
        }
        Dispose() {
        }
    }
    CatBuildNodeMoverEventAction.KEY = "CatBuildNodeMoverEventAction";

    class CatEventModel {
        constructor() {
            this.m_actions = new Dictionary();
            this.m_events = new Dictionary();
            for (const actionRes of ActionResource.items) {
                let actionArr = this.m_actions[actionRes.eventId];
                if (actionArr == null) {
                    actionArr = [];
                    this.m_actions[actionRes.eventId] = actionArr;
                }
                actionArr.push(actionRes);
            }
            for (const catEventRes of CatEventResource.items) {
                let eventArr = this.m_events[catEventRes.type];
                if (eventArr == null) {
                    eventArr = [];
                    this.m_events[catEventRes.type] = eventArr;
                }
                eventArr.push(catEventRes);
            }
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new CatEventModel();
            return this._Inst;
        }
        GetRandomEventActions(type) {
            let eventArrs = this.m_events[type];
            let jv = Math.random();
            let onJv = 0;
            let max = 0;
            let catEventRes = null;
            for (let i = 0; i < eventArrs.length; i++) {
                catEventRes = eventArrs[i];
                max += catEventRes.weight;
            }
            for (let i = 0; i < eventArrs.length; i++) {
                catEventRes = eventArrs[i];
                let curJv = onJv + catEventRes.weight / max;
                if (jv > onJv && jv <= curJv) {
                    return this.GetActions(catEventRes);
                }
                onJv = curJv;
            }
            return [];
        }
        GetEventActionsById(eventId) {
            let catEventRes = CatEventResource.getItemByKey(eventId);
            return this.GetActions(catEventRes);
        }
        GetActions(catEventRes) {
            let actionArr = this.m_actions[catEventRes.eventId];
            let actions = [];
            for (const actionRes of actionArr) {
                actions.push(this.GetAction(actionRes));
            }
            return actions;
        }
        GetAction(actionRes) {
            let action = null;
            switch (actionRes.type) {
                case CatEventModel.ACTION_WALK:
                    action = ObjectPools.Inst.getObj(CatWalkEventAction.KEY, CatWalkEventAction);
                    break;
                case CatEventModel.ACTION_MOVE_TO_TARGET:
                    action = ObjectPools.Inst.getObj(CatMoveTargetEventAction.KEY, CatMoveTargetEventAction);
                    break;
                case CatEventModel.ACTION_PLAY_ANIMATION:
                    action = ObjectPools.Inst.getObj(CatPlayAniEventAction.KEY, CatPlayAniEventAction);
                    break;
                case CatEventModel.ACTION_CHENG_CHUAN:
                    action = ObjectPools.Inst.getObj(CatChuanEventAction.KEY, CatChuanEventAction);
                    break;
                case CatEventModel.ACTION_JUMP_TARGET:
                    action = ObjectPools.Inst.getObj(CatJumpTargetEventAction.KEY, CatJumpTargetEventAction);
                    break;
                case CatEventModel.ACTION_LINE_MOVE:
                    action = ObjectPools.Inst.getObj(CatLineMoveEventAction.KEY, CatLineMoveEventAction);
                    break;
                case CatEventModel.ACTION_CAMERA_TWEEN:
                    action = ObjectPools.Inst.getObj(CatCameraTweenEventAction.KEY, CatCameraTweenEventAction);
                    break;
                case CatEventModel.ACTION_ROTATION:
                    action = ObjectPools.Inst.getObj(CatRoationEventAction.KEY, CatRoationEventAction);
                    break;
                case CatEventModel.ACTION_FISHING:
                    action = ObjectPools.Inst.getObj(CatFishingEventAction.KEY, CatFishingEventAction);
                    break;
                case CatEventModel.ACTION_ADD_OR_REMOVE:
                    action = ObjectPools.Inst.getObj(CatAddOrRemoveEventAction.KEY, CatAddOrRemoveEventAction);
                    break;
                case CatEventModel.ACTION_CAMERA_FOLLOW_TARGET:
                    action = ObjectPools.Inst.getObj(CatFollowTargetEventAction.KEY, CatFollowTargetEventAction);
                    break;
                case CatEventModel.ACTION_CAMERA_ROTATION:
                    action = ObjectPools.Inst.getObj(CatCameraRotationEventAction.KEY, CatCameraRotationEventAction);
                    break;
                case CatEventModel.ACTION_RESET_POSITION_ROTATION:
                    action = ObjectPools.Inst.getObj(CatResetPositionRotationEventAction.KEY, CatResetPositionRotationEventAction);
                    break;
                case CatEventModel.ACTION_BLACK_OR_WHITE:
                    action = ObjectPools.Inst.getObj(CatBlackOrWhiteEventAction.KEY, CatBlackOrWhiteEventAction);
                    break;
                case CatEventModel.ACTION_PLAY_BUILD_ANIMATOR:
                    action = ObjectPools.Inst.getObj(CatPlayBuildAnimatorEventAction.KEY, CatPlayBuildAnimatorEventAction);
                    break;
                case CatEventModel.ACTION_ADD_BUILD:
                    action = ObjectPools.Inst.getObj(CatAddBuildNodeEventAction.KEY, CatAddBuildNodeEventAction);
                    break;
                case CatEventModel.ACTION_ADD_TO_SCENE:
                    action = ObjectPools.Inst.getObj(SceneAddOrRemoveEventAction.KEY, SceneAddOrRemoveEventAction);
                    break;
                case CatEventModel.ACTION_PLAY_YUYUE:
                    action = ObjectPools.Inst.getObj(CatPlayYueyueAniEventAction.KEY, CatPlayYueyueAniEventAction);
                    break;
                case CatEventModel.ACTION_PLAY_ADD_SCENE_NODE_ANIMATOR:
                    action = ObjectPools.Inst.getObj(CatPlaySceneAnimatorEventAction.KEY, CatPlaySceneAnimatorEventAction);
                    break;
                case CatEventModel.ACTION_BUILD_NODE_MOVE:
                    action = ObjectPools.Inst.getObj(CatBuildNodeMoverEventAction.KEY, CatBuildNodeMoverEventAction);
                    break;
                default:
                    break;
            }
            if (action)
                action.actionRes = actionRes;
            return action;
        }
    }
    CatEventModel._Inst = null;
    CatEventModel.TYPE_RANDOM = 1;
    CatEventModel.TYPE_NORMAL = 0;
    CatEventModel.ACTION_WALK = 1;
    CatEventModel.ACTION_MOVE_TO_TARGET = 2;
    CatEventModel.ACTION_PLAY_ANIMATION = 3;
    CatEventModel.ACTION_CHENG_CHUAN = 4;
    CatEventModel.ACTION_JUMP_TARGET = 5;
    CatEventModel.ACTION_LINE_MOVE = 6;
    CatEventModel.ACTION_CAMERA_TWEEN = 7;
    CatEventModel.ACTION_ROTATION = 8;
    CatEventModel.ACTION_FISHING = 9;
    CatEventModel.ACTION_ADD_OR_REMOVE = 10;
    CatEventModel.ACTION_CAMERA_FOLLOW_TARGET = 11;
    CatEventModel.ACTION_CAMERA_ROTATION = 12;
    CatEventModel.ACTION_RESET_POSITION_ROTATION = 13;
    CatEventModel.ACTION_BLACK_OR_WHITE = 14;
    CatEventModel.ACTION_PLAY_BUILD_ANIMATOR = 15;
    CatEventModel.ACTION_ADD_BUILD = 16;
    CatEventModel.ACTION_ADD_TO_SCENE = 17;
    CatEventModel.ACTION_PLAY_YUYUE = 18;
    CatEventModel.ACTION_PLAY_ADD_SCENE_NODE_ANIMATOR = 19;
    CatEventModel.ACTION_BUILD_NODE_MOVE = 20;

    class CatStrokeVO {
        constructor() {
            this.state = CatStrokeVO.CAT_STATE_NORMAL;
            this._strokeTime = 0;
            this._refuseTime = 0;
        }
        OnChangeState(state, param) {
            if (this.state == state)
                return false;
            this.state = state;
            switch (state) {
                case CatStrokeVO.CAT_STATE_NORMAL:
                    break;
                case CatStrokeVO.CAT_STATE_BUSY_STROKE:
                    break;
                case CatStrokeVO.CAT_STATE_READY:
                    break;
                case CatStrokeVO.CAT_STATE_REFUSE_STROKE:
                    this._refuseTime = Laya.timer.currTimer;
                    break;
                case CatStrokeVO.CAT_STATE_STROKING:
                    break;
                case CatStrokeVO.CAT_STATE_STROKED:
                    this._strokeTime = Laya.timer.currTimer;
                    break;
                default:
                    break;
            }
            return true;
        }
        IsOverTime() {
            return this.strokeTime >= CatModel.Inst.touchTime;
        }
        IsRefuseOver() {
            return this.refuseTime >= CatModel.Inst.touchTime;
        }
        get strokeTime() {
            return Laya.timer.currTimer - this._strokeTime;
        }
        get refuseTime() {
            return Laya.timer.currTimer - this._refuseTime;
        }
        IsNormal() {
            return this.state == CatStrokeVO.CAT_STATE_NORMAL;
        }
        IsStroking() {
            return this.state == CatStrokeVO.CAT_STATE_STROKING;
        }
        IsBusy() {
            return this.state == CatStrokeVO.CAT_STATE_BUSY_STROKE;
        }
        IsReady() {
            return this.state == CatStrokeVO.CAT_STATE_READY;
        }
        IsStroked() {
            return this.state == CatStrokeVO.CAT_STATE_STROKED;
        }
        IsRefuse() {
            return this.state == CatStrokeVO.CAT_STATE_REFUSE_STROKE;
        }
        IsStartStroke() {
            return this.state == CatStrokeVO.CAT_READY_STROKED;
        }
        IsAnger() {
            return this.state == CatStrokeVO.CAT_START_ANGER;
        }
    }
    CatStrokeVO.CAT_STATE_NORMAL = 0;
    CatStrokeVO.CAT_STATE_BUSY_STROKE = 1;
    CatStrokeVO.CAT_STATE_READY = 2;
    CatStrokeVO.CAT_STATE_REFUSE_STROKE = 3;
    CatStrokeVO.CAT_STATE_STROKING = 4;
    CatStrokeVO.CAT_STATE_STROKED = 5;
    CatStrokeVO.CAT_READY_STROKED = 6;
    CatStrokeVO.CAT_START_ANGER = 7;

    class BaseState {
        constructor() {
            this.data = null;
        }
        enter(data) {
            this.data = data;
        }
        forceStop() {
        }
        exit() {
            this.data = null;
        }
        advanceTime(time) {
        }
        GetCurStateMachine() {
            return this.curStateMachine;
        }
        SetCurStateMachine(value) {
            this.curStateMachine = value;
        }
    }

    class CatBaseState extends BaseState {
        constructor() { super(); }
        enter(data) {
            super.enter(data);
            this.StartPlayState();
        }
        PlayState(name, handler = null, speed = 1) {
            if (this.cat)
                this.cat.PlayState(name, handler, speed);
        }
        CrossFadeState(name, handler = null, transitionDuration = 0.1, speed = 1) {
            if (this.cat)
                this.cat.CrossFadeState(name, handler, transitionDuration, speed);
        }
        StartPlayState() {
            this.CrossFadeState(this.GetStateName());
        }
        GetStateName() {
            return null;
        }
        exit() {
        }
        Pause() {
        }
        Resume() {
        }
        get cat() {
            if (!this.GetCurStateMachine()) {
                console.error("状态对象已移除");
                return null;
            }
            return this.GetCurStateMachine().cat;
        }
    }

    class CatEventState extends CatBaseState {
        constructor() {
            super();
            this.actions = new ActionQueue();
        }
        enter(data) {
            super.enter(data);
        }
        StartPlayState() {
        }
        OnActionComplete() {
        }
        exit() {
            super.exit();
            this.actions.ClearAll();
        }
    }

    class CatBuildEventState extends CatEventState {
        constructor(eventIds) {
            super();
            this.eventIds = null;
            this.eventIds = eventIds;
        }
        enter(data) {
            super.enter(data);
            this.cat.OnChangeShow(true);
            if (this.cat.catVO.catStrokeVo.IsOverTime() && this.cat.catVO.catStrokeVo.IsOverTime())
                CatCtrl.Inst.OnChangeState(this.cat.catVO, CatStrokeVO.CAT_STATE_BUSY_STROKE);
            this.ParseEvent();
        }
        ParseEvent() {
            let buildVO = this.data;
            let index = buildVO.curZuowei.AddZuowei(this.cat.catVO);
            if (index == null) {
                return;
            }
            this.InitEvents(parseInt(this.eventIds[index]));
        }
        InitEvents(eventId) {
            let actions = CatEventModel.Inst.GetEventActionsById(eventId);
            this.actions.handler = Laya.Handler.create(this, this.OnActionComplete);
            for (const action of actions) {
                let actionName = action.actionRes.actionName;
                for (const resName of actionName) {
                    LoadQueue.Inst.loadAnimationClip(UrlUtils.GetCatAnimationUrl(resName), null, false, LoadLevel.UI_MODEL);
                }
                action.cat = this.cat;
                this.actions.AddAction(action);
            }
        }
        StartPlayState() {
        }
        OnActionComplete() {
            this.cat.PlayState("event_happy", Laya.Handler.create(this, this.PlayYuyue));
            let position = this.cat.transform.position.clone();
            position.y = 4;
            EffectMgr.Inst.playConfigEffect(EffectMgr.Eff_shengji_flower, this.cat.owner.parent, null, position);
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.event_happy));
            let buildVO = this.data;
            CatCtrl.Inst.CatPlayBuildReward(this.cat, buildVO);
            if (buildVO.IsHasServer(this.cat.catVO.catRes.id))
                ItemCtrl.Inst.AddItemVO(ItemModel.ITEM_CHURU);
        }
        PlayYuyue() {
            if (this.cat == null)
                return;
            let buildVO = this.data;
            buildVO.curZuowei.RemoveZuowei(this.cat.catVO);
            this.cat.OnUpdateShowOrHide();
        }
        exit() {
            super.exit();
        }
    }

    class CatXiuxianEventState extends CatEventState {
        constructor() {
            super();
        }
        enter(data) {
            super.enter(data);
            if (this.cat) {
                this.cat.catVO.isXiuXian = true;
            }
            this.Start();
        }
        Start() {
            let actions = CatEventModel.Inst.GetRandomEventActions(CatEventModel.TYPE_RANDOM);
            this.actions.handler = Laya.Handler.create(this, this.OnActionComplete);
            for (const action of actions) {
                let actionName = action.actionRes.actionName;
                for (const resName of actionName) {
                    LoadQueue.Inst.loadAnimationClip(UrlUtils.GetCatAnimationUrl(resName), null, false, LoadLevel.UI_MODEL);
                }
                action.cat = this.cat;
                this.actions.AddAction(action);
            }
            if (this.cat && this.cat.catVO && this.cat.catVO.catStrokeVo.IsOverTime()) {
                CatCtrl.Inst.SetCatNormalState(this.cat);
            }
        }
        StartPlayState() {
        }
        advanceTime(time) {
            super.advanceTime(time);
            if (this.cat == null || this.cat.catVO == null)
                return;
            if (!this.cat.catVO.catStrokeVo.IsNormal())
                return;
            if (!this.cat.catVO.IsOverShowIcon())
                return;
            CatCtrl.Inst.SetCanLumao(this.cat, false);
        }
        OnActionComplete() {
            if (this.cat.catVO.IsVisitor && this.cat.IsVisitorEnd()) {
                let stateMachine = this.cat.getStateMachine();
                if (stateMachine)
                    stateMachine.SetCurState(stateMachine.matouDownEvent);
                return;
            }
            this.Start();
        }
        exit() {
            this.cat.catVO.isXiuXian = false;
            if (!this.cat.catVO.IsVisitor)
                GameDispatcher.Inst.event(UIGameEvent.CAT_ADD_OR_REMOVE_ICON, [this.cat, false]);
            super.exit();
        }
    }

    class CatCreateEventState extends CatEventState {
        constructor() {
            super();
        }
        enter(data) {
            super.enter(data);
            this.cat.OnChangeShow(true);
            this.InitEvents();
        }
        InitEvents() {
            GameDispatcher.Inst.event(UIGameEvent.ISHIDE_ICON, true);
            let actions = CatEventModel.Inst.GetEventActionsById(30);
            this.actions.handler = Laya.Handler.create(this, this.OnActionComplete);
            for (const action of actions) {
                if (action) {
                    let actionName = action.actionRes.actionName;
                    for (const resName of actionName) {
                        LoadQueue.Inst.loadAnimationClip(UrlUtils.GetCatAnimationUrl(resName), null, false, LoadLevel.UI_MODEL);
                    }
                    action.cat = this.cat;
                    this.actions.AddAction(action);
                }
            }
        }
        PlayInvitMusic() {
            Laya.timer.once(4000, this, () => {
                SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.fx_congratulation_invite));
            });
        }
        StartPlayState() {
            this.PlayInvitMusic();
        }
        OnActionComplete() {
            GameDispatcher.Inst.event(UIGameEvent.ISHIDE_ICON, false);
            this.cat.OnUpdateShowOrHide();
        }
        exit() {
            super.exit();
        }
    }

    class CatDiaoyuEventState extends CatBuildEventState {
        enter(data) {
            super.enter(data);
            LoadQueue.Inst.loadSprite3D(UrlUtils.GetEntityLhUrl("cat2_equipRef_bucket"));
            LoadQueue.Inst.loadSprite3D(UrlUtils.GetEntityLhUrl("cat2_equipRef_bucket"));
            LoadQueue.Inst.loadSprite3D(UrlUtils.GetEntityLhUrl("cat2_equipRef_fishAnim"));
        }
    }

    class CatGuideEventState extends CatEventState {
        constructor() {
            super();
            this.eventId = 0;
            this.handler = null;
        }
        enter(data) {
            super.enter(data);
            this.eventId = data[0];
            this.handler = data[1];
            this.InitEvents();
        }
        InitEvents() {
            GameDispatcher.Inst.event(UIGameEvent.ISHIDE_ICON, true);
            let actions = CatEventModel.Inst.GetEventActionsById(this.eventId);
            this.actions.handler = Laya.Handler.create(this, this.OnActionComplete);
            for (const action of actions) {
                let actionName = action.actionRes.actionName;
                for (const resName of actionName) {
                    LoadQueue.Inst.loadAnimationClip(UrlUtils.GetCatAnimationUrl(resName), null, false, LoadLevel.UI_MODEL);
                }
                action.cat = this.cat;
                this.actions.AddAction(action);
            }
        }
        StartPlayState() {
        }
        OnActionComplete() {
            GameDispatcher.Inst.event(UIGameEvent.ISHIDE_ICON, false);
            this.GetCurStateMachine().forceStop();
        }
        exit() {
            if (this.handler) {
                this.handler.runWith(this.cat);
                this.handler = null;
            }
            super.exit();
            this.eventId = null;
        }
    }

    class CatLumaoBaseState extends BaseState {
        constructor() {
            super();
        }
        enter(data) {
            super.enter(data);
        }
        get cat() {
            return this.data;
        }
        ClickCat() {
        }
        exit() {
            super.exit();
        }
    }

    class CatLumaoJinxingzongState1 extends CatLumaoBaseState {
        constructor() {
            super();
            this.isCanTouch = false;
        }
        enter(data) {
            super.enter(data);
            this.isCanTouch = true;
            this.cat.PlayState("rub00_start", Laya.Handler.create(this, this.OnReadyComplete));
            GameDispatcher.Inst.on(UIGameEvent.CAT_STROKING_DOWN_OR_UP, this, this.OnCatStrokingDownOrUp);
            GameDispatcher.Inst.on(UIGameEvent.CAT_STROKING_COMPLETE, this, this.OnCatStrokingComplete);
            let urls = [MusicType.cat_interaction_ready11, MusicType.cat_interaction_ready21, MusicType.cat_interaction_ready31];
            let url = urls[MathUtil.makeRandom(0, urls.length - 1)];
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(url));
        }
        OnReadyComplete() {
            if (this.cat == null)
                return;
            CatCtrl.Inst.OnChangeState(this.cat.catVO, CatStrokeVO.CAT_STATE_STROKING, 3);
        }
        OnCatStrokingDownOrUp(isDown) {
            if (!this.isCanTouch)
                return;
            if (isDown) {
                this.cat.PlayState("rub00_loop02", Laya.Handler.create(this, this.OnReadyComplete1));
            }
        }
        OnReadyComplete1() {
            if (this.cat == null)
                return;
            this.cat.PlayState("rub00_loop");
        }
        OnCatStrokingComplete(catVO) {
            if (!this.isCanTouch)
                return;
            if (this.cat == null)
                return;
            if (this.cat.catVO != catVO)
                return;
            this.isCanTouch = false;
            this.cat.PlayState("rub00_end", Laya.Handler.create(this, this.OnCatStrokingPlayComplete));
        }
        OnCatStrokingPlayComplete() {
            if (this.cat == null)
                return;
            this.cat.PlayState("event_happy", Laya.Handler.create(this, this.OnCatStrokingPlayGaoxing));
            let position = this.cat.transform.position.clone();
            position.y = 4;
            EffectMgr.Inst.playConfigEffect(EffectMgr.Eff_shengji_flower, this.cat.owner.parent, null, position);
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.event_happy));
        }
        OnCatStrokingPlayGaoxing() {
            if (this.cat == null)
                return;
            this.cat.OverLumao();
        }
        exit() {
            super.exit();
            GameDispatcher.Inst.off(UIGameEvent.CAT_STROKING_DOWN_OR_UP, this, this.OnCatStrokingDownOrUp);
            GameDispatcher.Inst.off(UIGameEvent.CAT_STROKING_COMPLETE, this, this.OnCatStrokingComplete);
        }
    }

    class CatLumaoJinxingzongState2 extends CatLumaoBaseState {
        constructor() {
            super();
            this.isCanTouch = false;
        }
        enter(data) {
            super.enter(data);
            this.isCanTouch = true;
            this.cat.PlayState("rub02_start", Laya.Handler.create(this, this.OnReadyComplete));
            GameDispatcher.Inst.on(UIGameEvent.CAT_STROKING_DOWN_OR_UP, this, this.OnCatStrokingDownOrUp);
            GameDispatcher.Inst.on(UIGameEvent.CAT_STROKING_COMPLETE, this, this.OnCatStrokingComplete);
            let urls = [MusicType.cat_interaction_ready12, MusicType.cat_interaction_ready22, MusicType.cat_interaction_ready32];
            let url = urls[MathUtil.makeRandom(0, urls.length - 1)];
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(url));
        }
        OnReadyComplete() {
            if (this.cat == null)
                return;
            CatCtrl.Inst.OnChangeState(this.cat.catVO, CatStrokeVO.CAT_STATE_STROKING, 2);
        }
        OnCatStrokingDownOrUp(isDown) {
            if (!this.isCanTouch)
                return;
            if (isDown) {
                this.cat.PlayState("rub02_loop02", Laya.Handler.create(this, this.OnReadyComplete1));
            }
        }
        OnReadyComplete1() {
            if (this.cat == null)
                return;
            this.cat.PlayState("rub02_loop");
        }
        OnCatStrokingComplete(catVO) {
            if (!this.isCanTouch)
                return;
            if (this.cat == null)
                return;
            if (this.cat.catVO != catVO)
                return;
            this.isCanTouch = false;
            this.cat.PlayState("rub02_end", Laya.Handler.create(this, this.OnCatStrokingPlayComplete));
        }
        OnCatStrokingPlayComplete() {
            if (this.cat == null)
                return;
            this.cat.PlayState("event_happy", Laya.Handler.create(this, this.OnCatStrokingPlayGaoxing));
            let position = this.cat.transform.position.clone();
            position.y = 4;
            EffectMgr.Inst.playConfigEffect(EffectMgr.Eff_shengji_flower, this.cat.owner.parent, null, position);
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.event_happy));
        }
        OnCatStrokingPlayGaoxing() {
            if (this.cat == null)
                return;
            this.cat.OverLumao();
        }
        exit() {
            super.exit();
            GameDispatcher.Inst.off(UIGameEvent.CAT_STROKING_DOWN_OR_UP, this, this.OnCatStrokingDownOrUp);
            GameDispatcher.Inst.off(UIGameEvent.CAT_STROKING_COMPLETE, this, this.OnCatStrokingComplete);
        }
    }

    class CatLumaoJinxingzongState3 extends CatLumaoBaseState {
        constructor() {
            super();
            this.isCanTouch = false;
        }
        enter(data) {
            super.enter(data);
            this.isCanTouch = true;
            GameDispatcher.Inst.on(UIGameEvent.CAT_STROKING_DOWN_OR_UP, this, this.OnCatStrokingDownOrUp);
            GameDispatcher.Inst.on(UIGameEvent.CAT_STROKING_COMPLETE, this, this.OnCatStrokingComplete);
            CatCtrl.Inst.OnChangeState(this.cat.catVO, CatStrokeVO.CAT_STATE_STROKING, 1);
            let urls = [MusicType.cat_interaction_ready13, MusicType.cat_interaction_ready23, MusicType.cat_interaction_ready33];
            let url = urls[MathUtil.makeRandom(0, urls.length - 1)];
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(url));
        }
        OnCatStrokingDownOrUp(isDown) {
            if (!this.isCanTouch)
                return;
            if (isDown) {
                this.cat.PlayState("rub01_loop02", Laya.Handler.create(this, this.OnReadyComplete1));
            }
        }
        OnReadyComplete1() {
            if (this.cat == null)
                return;
            this.cat.PlayState("rub01_loop");
        }
        OnCatStrokingComplete(catVO) {
            if (!this.isCanTouch)
                return;
            if (this.cat == null)
                return;
            if (this.cat.catVO != catVO)
                return;
            this.isCanTouch = false;
            this.cat.PlayState("event_happy", Laya.Handler.create(this, this.OnCatStrokingPlayGaoxing));
            let position = this.cat.transform.position.clone();
            position.y = 4;
            EffectMgr.Inst.playConfigEffect(EffectMgr.Eff_shengji_flower, this.cat.owner.parent, null, position);
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.event_happy));
        }
        OnCatStrokingPlayGaoxing() {
            if (this.cat == null)
                return;
            this.cat.OverLumao();
        }
        exit() {
            super.exit();
            GameDispatcher.Inst.off(UIGameEvent.CAT_STROKING_DOWN_OR_UP, this, this.OnCatStrokingDownOrUp);
            GameDispatcher.Inst.off(UIGameEvent.CAT_STROKING_COMPLETE, this, this.OnCatStrokingComplete);
        }
    }

    class CatLumaoJnxingzhongState extends CatLumaoBaseState {
        constructor() {
            super();
            this.stateMachine = new FiniteStateMachine();
            this.states = [new CatLumaoJinxingzongState1(), new CatLumaoJinxingzongState2(), new CatLumaoJinxingzongState3()];
        }
        enter(data) {
            super.enter(data);
            let state = this.states[MathUtil.makeRandom(0, this.states.length)];
            this.stateMachine.SetCurState(state, this.cat);
            SceneMgr.Inst.GetCameraBuildScript().ForwardCat(this.cat.catVO, 0.04598784, 1.47, -6.786611, 6.334, 0, 0, 50);
        }
        exit() {
            super.exit();
            this.stateMachine.forceStop();
        }
    }

    class CatLumaoReadyState extends CatLumaoBaseState {
        constructor() {
            super();
            this.index = 0;
        }
        enter(data) {
            super.enter(data);
            this.cat.PlayState("interaction_ready", Laya.Handler.create(this, this.onReadComplete));
            CatCtrl.Inst.OnChangeState(this.cat.catVO, CatStrokeVO.CAT_STATE_READY);
            SceneMgr.Inst.GetCameraBuildScript().ForwardCat(this.cat.catVO, 0.04598784, 1.47, -6.786611, 6.334, 0, 0, 50);
        }
        onReadComplete() {
            this.cat && this.cat.PlayState("idle_wait");
        }
        ClickCat() {
            if (this.IsJujueEnd())
                return;
            this.index++;
            this.cat.PlayState("interaction_refuse0" + this.index, Laya.Handler.create(this, this.PlayerComplete));
            CatCtrl.Inst.OnChangeState(this.cat.catVO, CatStrokeVO.CAT_STATE_REFUSE_STROKE, this.index);
        }
        PlayerComplete() {
            if (this.cat == null)
                return;
            if (this.IsJujueEnd()) {
                this.GotoXiuxian();
                return;
            }
        }
        GotoXiuxian() {
            CatCtrl.Inst.OnChangeState(this.cat.catVO, CatStrokeVO.CAT_START_ANGER);
            let stateMachine = this.cat.getStateMachine();
            if (stateMachine)
                stateMachine.SetCurState(stateMachine.xiuxianEvent);
        }
        IsJujueEnd() {
            return this.index >= 3;
        }
        exit() {
            super.exit();
            this.index = 0;
        }
    }

    class CatLumaoEventState extends CatEventState {
        constructor() {
            super();
            this.stateMachine = new FiniteStateMachine();
            this.state_ready = new CatLumaoReadyState();
            this.state_lumaozhong = new CatLumaoJnxingzhongState();
            this.isCanLumao = true;
        }
        enter(data) {
            super.enter(data);
            this.stateMachine.SetCurState(this.state_ready, data);
            if (GuideModel.Inst.funGuideVO && GuideModel.Inst.funGuideVO.page == 15) {
                this.isCanLumao = true;
            }
            else if (GuideModel.Inst.funGuideVO && GuideModel.Inst.funGuideVO.page == 22 && GuideModel.Inst.funGuideVO.step == 4) {
                this.isCanLumao = true;
            }
            else if (!this.cat.catVO.isForciblylumao) {
                this.isCanLumao = MathUtil.random(CatModel.Inst.catStrokeSuccessCoe);
            }
            else {
                this.isCanLumao = true;
            }
        }
        StartPlayState() {
        }
        ClickCat() {
            if (this.cat.catVO.catStrokeVo.IsStroking())
                return;
            let curState = this.stateMachine.GetCurState();
            if (curState == this.state_ready && this.isCanLumao) {
                CatCtrl.Inst.OnChangeState(this.cat.catVO, CatStrokeVO.CAT_READY_STROKED);
                this.stateMachine.SetCurState(this.state_lumaozhong, this.data);
                return;
            }
            curState.ClickCat();
        }
        exit() {
            super.exit();
            this.stateMachine.forceStop();
        }
    }

    class CatMatouDownEventState extends CatEventState {
        constructor() {
            super();
        }
        enter(data) {
            super.enter(data);
            let chaunScript = EntityModel.Inst.GetKongDownChuan();
            if (chaunScript) {
                chaunScript.cat = this.cat;
                let actions = CatEventModel.Inst.GetEventActionsById(CatMatouDownEventState.MATOU_DOWN_EVENT[chaunScript.index]);
                this.actions.handler = Laya.Handler.create(this, this.OnActionComplete2, [chaunScript]);
                for (const action of actions) {
                    let actionName = action.actionRes.actionName;
                    for (const resName of actionName) {
                        LoadQueue.Inst.loadAnimationClip(UrlUtils.GetCatAnimationUrl(resName), null, false, LoadLevel.UI_MODEL);
                    }
                    action.cat = this.cat;
                    this.actions.AddAction(action);
                }
            }
            GameDispatcher.Inst.event(UIGameEvent.CAT_ADD_OR_REMOVE_ICON, [this.cat, true, 2]);
        }
        StartPlayState() {
        }
        OnActionComplete2(chaunScript) {
            if (chaunScript) {
                if (chaunScript.owner && !chaunScript.owner.destroyed) {
                    chaunScript.owner.destroy();
                }
            }
            if (this.cat) {
                GameDispatcher.Inst.event(UIGameEvent.CAT_ADD_OR_REMOVE_ICON, [this.cat, false]);
                EntityModel.Inst.RemoveChuan(chaunScript);
                CatCtrl.Inst.RemoveCat(this.cat.catVO);
                if (CatModel.Inst.selectCatVO != null) {
                    if (CatModel.Inst.selectCatVO.IsVisitor) {
                        GameDispatcher.Inst.event(UIGameEvent.CAT_MATOUDOWN, CatModel.Inst.selectCatVO);
                        SceneMgr.Inst.GetCameraBuildScript().OverSelected();
                    }
                }
            }
        }
        exit() {
            super.exit();
        }
    }
    CatMatouDownEventState.MATOU_DOWN_EVENT = [4, 6, 8, 10];

    class AwardMgr {
        constructor() {
            this.awardVO = new AwardVO(0);
            this.itemVO = new ItemVO();
        }
        static get Inst() {
            return this.inst = this.inst || new AwardMgr();
        }
        GetAward(itemId, type = AwardVO.TYPE_ITEM, count = 1, showView = true) {
            this.awardVO.setData({ "tempId": itemId, "type": type, "count": count });
            if (itemId == ItemModel.ITEM_GOLD) {
                MoneyModel.gold.add(count);
            }
            else if (itemId == ItemModel.ITEM_GEM) {
                MoneyModel.diamond.add(count);
            }
            else {
                if (count instanceof BigNumber) {
                    ItemCtrl.Inst.AddItemVO(itemId, count.toNumber());
                }
                else if (typeof count === "number") {
                    ItemCtrl.Inst.AddItemVO(itemId, count);
                }
            }
            if (showView) {
                UISystem.Inst.CreateWindowView(UIAwardView, this.awardVO);
            }
        }
        award(data) {
            let itemRes;
            if (data instanceof GiftVO) {
                itemRes = itemResource.getItemByKey(data.itemId);
                if (data.IsGold()) {
                    MoneyModel.gold.add(data.count);
                    data = new AwardVO(ItemModel.ITEM_GOLD, { "count": data.count });
                    data.type = AwardVO.TYPE_ITEM;
                }
                else if (data.IsGem()) {
                    MoneyModel.diamond.add(data.count);
                    let cnt = data.count instanceof BigNumber ? data.count.toNumber() : data.count;
                    data = new AwardVO(itemRes.id, { "count": cnt });
                    data.type = AwardVO.TYPE_ITEM;
                }
                else {
                    this.itemVO.itemId = itemRes.id;
                    this.itemVO.count = data.count;
                    if (typeof data.count === "number") {
                        ItemCtrl.Inst.AddItemVO(itemRes.id, data.count);
                    }
                    data = this.itemVO;
                }
            }
            else {
                itemRes = itemResource.getItemByKey(data.itemId);
                let count = data.count;
                this.itemVO.itemId = itemRes.id;
                this.itemVO.count = count;
                if (itemRes.id == ItemModel.ITEM_GOLD) {
                    MoneyModel.gold.add(count);
                }
                else if (itemRes.id == ItemModel.ITEM_GEM) {
                    MoneyModel.diamond.add(count);
                }
                else if (itemRes) {
                    ItemCtrl.Inst.AddItemVO(itemRes.id, count);
                }
                data = this.itemVO;
            }
            UISystem.Inst.CreateWindowView(UIAwardView, data);
        }
    }
    AwardMgr.inst = null;

    class EffecResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint8();
            this.resName = byteArray.readUTFString();
        }
        static getItemByKey(id) {
            return EffecResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return EffecResource.itemsDic[key1 + "," + key2];
        }
    }
    EffecResource.items = new Array();
    EffecResource.itemsDic = new Dictionary();

    class CatGiftScript extends EntityBaseScript {
        constructor() { super(); }
        onAwake() {
            super.onAwake();
        }
        onClick() {
            CatGiftCtrl.Inst.AwardGift(this.giftVO);
        }
        PlayEffect() {
            let res = EffecResource.getItemByKey(EffectMgr.Eff_sheshi_glow);
            if (res) {
                let url = UrlUtils.GetEffectUrl(res.resName);
                LoadQueue.Inst.loadParticle(url, Laya.Handler.create(this, this.LoadParticle));
            }
        }
        LoadParticle(sp) {
            if (sp) {
                sp.transform.localPosition = new Laya.Vector3(0, 0, 0);
                this.target && this.target.addChild(sp);
            }
        }
        get giftVO() {
            return this.vo;
        }
        get target() {
            return this.owner;
        }
        onReset() {
            super.onReset();
        }
    }

    class CatGift extends EntityAnimator {
        constructor(displayObject) {
            super(displayObject);
            this.gs = null;
            this._bornTime = 0;
            this.onAwake();
        }
        onAwake() {
            this.InitPhy();
            this.gs = this.owner && (this.owner.getComponent(CatGiftScript) || this.owner.addComponent(CatGiftScript));
            super.onAwake();
        }
        SetVO(vo) {
            super.SetVO(vo);
            this.gs.SetVO(vo);
            this._bornTime = Laya.timer.currTimer;
            this.InitPos();
            this.PlayEffect();
        }
        InitPhy() {
        }
        InitPos() {
            let cat = EntityModel.Inst.GetCat(this.giftVO.catId);
            if (cat) {
                let pos = cat.transform.position.clone();
                this._displayObject.transform.position = pos;
            }
        }
        PlayEffect() {
            if (this.gs)
                this.gs.PlayEffect();
        }
        get giftVO() {
            return this._vo;
        }
        GetCollisonGroup() {
            return CollisionType.TYPE_ENTITY;
        }
        GetCanCollideWith() {
            return CollisionType.TYPE_RAY_CLICK;
        }
        dispose() {
            if (this.gs && !this.gs.destroyed) {
                this.gs.destroy();
                this.gs = null;
            }
            super.dispose();
        }
    }

    class CatGiftCtrl {
        constructor() {
            this.addMap = {};
        }
        static get Inst() {
            return this.inst = this.inst || new CatGiftCtrl();
        }
        init() {
        }
        AddGift(catVo) {
            if (catVo && !this.addMap.hasOwnProperty(catVo.objId)) {
                this.addMap[catVo.objId] = Laya.timer.currTimer;
                LoadQueue.Inst.loadSprite3D(UrlUtils.GetBuildUrl("cat2_WorldDrop_01"), Laya.Handler.create(this, this.addbase, [catVo]));
            }
        }
        addbase(vo, entity) {
            if (!entity || vo == null)
                return;
            delete this.addMap[vo.objId];
            if (vo instanceof CatVO) {
                let giftVo = CatGiftModel.Inst.makeGiftVO(vo.catRes.quality);
                giftVo.catId = vo.objId;
                let gift = new CatGift(entity);
                SceneMgr.Inst.AddGift(giftVo.objId + "", entity);
                gift.SetVO(giftVo);
                CatGiftModel.Inst.AddGift(gift);
            }
        }
        AddItemGift(itemId, count, catObjId, pos) {
            let giftVo = CatGiftModel.Inst.makeGiftBy(itemId, count, catObjId);
            LoadQueue.Inst.loadSprite3D(UrlUtils.GetBuildUrl("cat2_WorldDrop_01"), Laya.Handler.create(this, (giftVo, pos, entity) => {
                if (entity) {
                    let gift = new CatGift(entity);
                    SceneMgr.Inst.AddGift(giftVo.objId + "", entity);
                    gift.SetVO(giftVo);
                    entity.transform.position = pos;
                    CatGiftModel.Inst.AddGift(gift);
                }
            }, [giftVo, pos]));
            return giftVo;
        }
        AwardGift(giftVO) {
            if (GuideModel.Inst.guideVO && GuideModel.Inst.guideVO.guideRes.id <= GuideModel.GUID_GETGIFT_ID) {
                return;
            }
            AwardMgr.Inst.award(giftVO);
            let gift = CatGiftModel.Inst.GetGift(giftVO.objId);
            GameDispatcher.Inst.event(UIGameEvent.AWARD_CAT_GIFT, giftVO);
            this.RemoveGift(gift);
        }
        RemoveGift(gift) {
            if (gift) {
                CatGiftModel.Inst.RemoveGift(gift);
                gift.dispose();
            }
        }
    }
    CatGiftCtrl.inst = null;

    class CatMatouUpEventState extends CatEventState {
        constructor() {
            super();
        }
        enter(data) {
            super.enter(data);
            MatouModel.Inst.curUpCat = this.cat;
            GameDispatcher.Inst.event(UIGameEvent.CAT_START_ONLAND);
            GameDispatcher.Inst.event(UIGameEvent.CAT_ADD_OR_REMOVE_ICON, [this.cat, true, 1]);
            GameDispatcher.Inst.on(UIGameEvent.CAT_SKIP_ONLAND, this, this.onCatSkipOnland);
            this.InitEvents();
        }
        InitEvents() {
            GameDispatcher.Inst.event(UIGameEvent.ISHIDE_ICON, true);
            let index = EntityModel.Inst.GetKongIndex();
            let eventId = CatMatouUpEventState.MATOU_UP_EVENT[index];
            let actions = CatEventModel.Inst.GetEventActionsById(eventId);
            this.actions.handler = Laya.Handler.create(this, this.OnActionComplete);
            for (const action of actions) {
                let actionName = action.actionRes.actionName;
                for (const resName of actionName) {
                    LoadQueue.Inst.loadAnimationClip(UrlUtils.GetCatAnimationUrl(resName), null, false, LoadLevel.UI_MODEL);
                }
                action.cat = this.cat;
                this.actions.AddAction(action);
            }
        }
        StartPlayState() {
        }
        onCatSkipOnland() {
            this.actions.RemoveActionByCls(CatCameraTweenEventAction);
            SceneMgr.Inst.GetCameraScript().StopAni();
            let cameraScript = SceneMgr.Inst.GetCameraBuildScript();
            cameraScript.ResetCameraParent();
            cameraScript.CameraTweenTo(null, new Laya.Vector3(-47.027, 180 - 36.397, 0), new Laya.Vector3(20.9, 73.68, -74.9), 21, 600);
        }
        OnActionComplete() {
            CatGiftCtrl.Inst.AddGift(this.cat.catVO);
            let stateMachine = this.GetCurStateMachine();
            stateMachine.SetCurState(stateMachine.xiuxianEvent);
            if (!SceneMgr.Inst.GetCameraBuildScript().IsSelected()) {
                GameDispatcher.Inst.event(UIGameEvent.ISHIDE_ICON, false);
            }
        }
        exit() {
            GameDispatcher.Inst.off(UIGameEvent.CAT_SKIP_ONLAND, this, this.onCatSkipOnland);
            super.exit();
        }
    }
    CatMatouUpEventState.MATOU_UP_EVENT = [3, 5, 7, 9];

    class CatWgEventState extends CatEventState {
        constructor() {
            super();
            this.handler = null;
        }
        enter(data) {
            super.enter(data);
            if (this.cat.catVO.catStrokeVo.IsOverTime() && this.cat.catVO.catStrokeVo.IsOverTime())
                CatCtrl.Inst.OnChangeState(this.cat.catVO, CatStrokeVO.CAT_STATE_BUSY_STROKE);
            this.InitEvents(data[0]);
            this.handler = data[1];
        }
        InitEvents(eventId) {
            let actions = CatEventModel.Inst.GetEventActionsById(eventId);
            this.actions.handler = Laya.Handler.create(this, this.OnActionComplete);
            for (const action of actions) {
                let actionName = action.actionRes.actionName;
                for (const resName of actionName) {
                    LoadQueue.Inst.loadAnimationClip(UrlUtils.GetCatAnimationUrl(resName), null, false, LoadLevel.UI_MODEL);
                }
                action.cat = this.cat;
                this.actions.AddAction(action);
            }
        }
        StartPlayState() {
        }
        OnActionComplete() {
            if (this.cat == null)
                return;
            let stateMachine = this.GetCurStateMachine();
            stateMachine.SetCurState(stateMachine.xiuxianEvent);
            if (this.handler) {
                this.handler.run();
                this.handler = null;
            }
        }
        exit() {
            super.exit();
        }
    }

    class CatFiniteStateMachine extends FiniteStateMachine {
        constructor() {
            super();
            this._cat = null;
            this.guildEvent = new CatGuideEventState();
            this.createEvent = new CatCreateEventState();
            this.wgEvent = new CatWgEventState();
            this.xiuxianEvent = new CatXiuxianEventState();
            this.lumaoEvent = new CatLumaoEventState();
            this.matouUpEvent = new CatMatouUpEventState();
            this.matouDownEvent = new CatMatouDownEventState();
            this.build_states = null;
            this.IsPlay = true;
            this.build_states = new Dictionary();
            for (const buildRes of bulidResource.items) {
                if (buildRes.eventIds.length <= 0)
                    continue;
                if (buildRes.id == 101) {
                    this.resiger(buildRes.id, new CatDiaoyuEventState(buildRes.eventIds));
                }
                else {
                    this.resiger(buildRes.id, new CatBuildEventState(buildRes.eventIds));
                }
            }
        }
        resiger(buildId, state) {
            this.build_states[buildId] = state;
        }
        SetBuildState(buildVO) {
            let state = this.build_states[buildVO.buildRes.id];
            if (state == null)
                return false;
            this.SetCurState(state, buildVO);
            return true;
        }
        advanceTime() {
            if (this.IsPlay)
                super.advanceTime();
        }
        SetCurState(value, data) {
            this.IsPlay = true;
            super.SetCurState(value, data);
        }
        Pause() {
            this.IsPlay = false;
            this.GetCurState().Pause();
        }
        Resume() {
            this.IsPlay = true;
            this.GetCurState().Resume();
        }
        getIsPlay() {
            return this.IsPlay;
        }
        forceStop() {
            super.forceStop();
        }
        set cat(v) {
            this._cat = v;
        }
        get cat() {
            return this._cat;
        }
    }

    class Cat extends EntityAnimator {
        constructor(displayObject) {
            super(displayObject);
            this._vo = null;
            this.catScript = null;
            this.catSkinScript = null;
            this.curPathVO = null;
            this.visitorTime = 0;
            this.onAwake();
        }
        SetVO(vo) {
            super.SetVO(vo);
            this.catScript.SetVO(vo);
            let catVO = vo;
            this.ChangeSkin(catVO.wornSkinIds);
            this.InitPos();
            this.InitState();
            if (!this.catVO.IsVisitor && !this.catVO.IsGuide)
                this.OnUpdateShowOrHide();
        }
        InitState() {
            let stateMachine = this.stateMachine;
            let catVO = this._vo;
            if (!catVO.IsVisitor) {
                this.SetStateMachine(stateMachine.xiuxianEvent);
            }
            else {
                this.visitorTime = Laya.timer.currTimer + CatModel.Inst.boatStayTime * 1000;
                if (catVO.IsGuide)
                    return;
                this.SetStateMachine(stateMachine.matouUpEvent);
            }
        }
        onAwake() {
            this.InitPhy();
            this.catScript = this._displayObject.addComponent(CatScript);
            this.catSkinScript = this._displayObject.addComponent(CatSkinSctipt);
            super.onAwake();
        }
        InitStateMachine() {
            let stateMachine = new CatFiniteStateMachine();
            this.stateMachine = stateMachine;
            stateMachine.cat = this;
        }
        InitPhy() {
            let phy = this._displayObject.addComponent(Laya.PhysicsCollider);
            var planeShape = new Laya.BoxColliderShape(0.3, 0.3, 0.3);
            phy.colliderShape = planeShape;
            phy.isTrigger = true;
        }
        InitPos() {
            let pathVO = PathPointModel.Inst.GetRandomPathVO();
            this.curPathVO = pathVO;
            if (pathVO) {
                pathVO.catId = this.catVO.objId;
                let position = this._displayObject.transform.position;
                this._displayObject.transform.position = pathVO.GetPos(position);
            }
        }
        get catVO() {
            return this._vo;
        }
        IsVisitorEnd() {
            return Laya.timer.currTimer > this.visitorTime;
        }
        ChangeSkin(parts) {
            if (this.catSkinScript != null) {
                this.catSkinScript.ChangeSkin(parts);
            }
        }
        Show() {
            if (this.catVO.isShow)
                return;
            this.catVO.isShow = true;
            this.OnUpdateShowOrHide();
        }
        Hide() {
            if (!this.catVO.isShow)
                return;
            this.catVO.isShow = false;
            if (this.stateMachine.GetCurState() != this.getStateMachine().xiuxianEvent)
                return;
            this.OnUpdateShowOrHide();
        }
        OnUpdateSelectedCat() {
            if (this.catVO.IsGuide || this.catVO.IsVisitor)
                return;
            if (this.stateMachine.GetCurState() != this.getStateMachine().xiuxianEvent)
                return;
            this.OnUpdateShowOrHide();
        }
        OnUpdateShowOrHide() {
            let stateMachine = this.stateMachine;
            let isShow = this.catVO.isForciblyShow ? true : this.catVO.isShow;
            this.OnChangeShow(isShow);
            if (isShow) {
                this.stateMachine.SetCurState(stateMachine.xiuxianEvent);
            }
            else {
                this.stateMachine.forceStop();
            }
        }
        OnChangeShow(isShow) {
            this.owner.active = isShow;
        }
        ClickCat() {
            let stateMachine = this.getStateMachine();
            if (stateMachine.GetCurState() == stateMachine.xiuxianEvent) {
                stateMachine.SetCurState(stateMachine.lumaoEvent, this);
                let urls = [MusicType.cat_sit_yawn1, MusicType.cat_sit_yawn2, MusicType.cat_sit_yawn3];
                let url = urls[MathUtil.makeRandom(0, urls.length - 1)];
                SoundMgr.Inst.playSound(UrlUtils.GetMusic(url));
                if (this.catVO.isForciblylumao && this.catVO.isMainViewSelect)
                    stateMachine.lumaoEvent.ClickCat();
                return;
            }
            else if (stateMachine.GetCurState() == stateMachine.lumaoEvent) {
                stateMachine.lumaoEvent.ClickCat();
            }
        }
        StopLumao() {
            let stateMachine = this.getStateMachine();
            if (stateMachine.GetCurState() != stateMachine.lumaoEvent) {
                return;
            }
            stateMachine.SetCurState(stateMachine.xiuxianEvent);
        }
        OverLumao() {
            this.StopLumao();
            if (!this.catVO.IsVisitor)
                GameDispatcher.Inst.event(UIGameEvent.CAT_ADD_OR_REMOVE_ICON, [this, false]);
            SceneMgr.Inst.GetCameraBuildScript().SelectedCat(this.catVO, true);
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.event_happy));
        }
        IsCanPlayBuild() {
            if (!this.catVO.isShow)
                return false;
            let stateMachine = this.stateMachine;
            if (stateMachine == null)
                return false;
            if (stateMachine.GetCurState() == null)
                return true;
            return stateMachine.GetCurState() == stateMachine.xiuxianEvent;
        }
        SetBuildState(buildVO) {
            let stateMachine = this.stateMachine;
            if (stateMachine == null)
                return false;
            return stateMachine.SetBuildState(buildVO);
        }
        getStateMachine() {
            if (this.stateMachine == null)
                return null;
            return this.stateMachine;
        }
        GetCollisonGroup() {
            return CollisionType.TYPE_CAT;
        }
        GetCanCollideWith() {
            return CollisionType.TYPE_RAY_CLICK;
        }
        dispose() {
            let stateMachine = this.stateMachine;
            if (stateMachine == null)
                return;
            stateMachine.forceStop();
            stateMachine.cat = null;
            GameDispatcher.Inst.event(UIGameEvent.CAT_ADD_OR_REMOVE_ICON, [this, false]);
            EntityModel.Inst.RemoveCat(this.catVO.objId);
            super.dispose();
            this.curPathVO = null;
            if (this.catScript) {
                this.catScript.destroy();
            }
            this.catScript = null;
        }
    }

    class CatSceneTree extends BaseSceneTree {
        constructor(cls = null, isCheckRemove = false) {
            super(cls, isCheckRemove);
        }
        CheckEntity(value) {
            return true;
        }
        AddEntity(value, handler = null) {
            if (value.state != BaseVO.STATE_NORMAL) {
                return;
            }
            let cat = EntityModel.Inst.GetCat(value.objId);
            if (cat != null) {
                if (!cat.active) {
                    value.state = BaseVO.STATE_ADD;
                    cat.active = true;
                }
                return;
            }
            value.state = BaseVO.STATE_ADD;
            this.addScript(value, handler);
        }
        RemoveEntity(value, isDispose) {
            if (value.state != BaseVO.STATE_ADD) {
                return;
            }
            value.state = BaseVO.STATE_NORMAL;
            let cat = EntityModel.Inst.GetCat(value.objId);
            if (cat == null)
                return;
            if (isDispose) {
                cat.dispose();
            }
            else {
                if (cat.active) {
                    cat.active = false;
                }
            }
        }
        addScript(vo, handler = null) {
            let lf = this;
            let resName = vo.resName;
            let loadAction = ObjectPools.Inst.getObj(LoadSprite3dAction.KEY, LoadSprite3dAction);
            loadAction.url = UrlUtils.GetCatUrl(resName);
            loadAction.handler = Laya.Handler.create(lf, lf.addbase, [vo, handler]);
            CatSceneTree.actions.AddAction(loadAction);
        }
        addbase(vo, handler, entity) {
            if (vo == null || vo.state != BaseVO.STATE_ADD)
                return;
            if (entity == null)
                return;
            SceneMgr.Inst.AddMap(vo.resName, entity);
            entity.name = vo.resName;
            let cat = new Cat(entity);
            if (vo instanceof CatVO) {
                cat.SetVO(vo);
                EntityModel.Inst.AddCat(vo.objId, cat);
            }
            if (handler != null)
                handler.runWith(cat);
        }
        RemoveEntitys() {
            super.RemoveEntitys();
            CatSceneTree.actions.ClearAll();
        }
    }
    CatSceneTree.actions = new ActionQueue();

    class MapSceneTree extends BaseSceneTree {
        constructor(cls = null, isCheckRemove = false) {
            super(cls, isCheckRemove);
            this.dic = new Dictionary();
        }
        CheckEntity(value) {
            return true;
        }
        AddEntity(value, handler = null) {
            if (value.state != BaseVO.STATE_NORMAL) {
                return;
            }
            let entity = this.dic[value.objId];
            if (entity != null) {
                if (!entity.active) {
                    value.state = BaseVO.STATE_ADD;
                    entity.active = true;
                }
                return;
            }
            value.state = BaseVO.STATE_ADD;
            this.addScript(value, handler);
        }
        RemoveEntity(value, isDispose) {
            if (value.state != BaseVO.STATE_ADD) {
                return;
            }
            value.state = BaseVO.STATE_NORMAL;
            let entity = this.dic[value.objId];
            if (entity != null && entity.active) {
                entity.active = false;
            }
            if (isDispose) {
                if (!entity.destroyed)
                    entity.destroy();
                delete this.dic[value.objId];
            }
        }
        addScript(vo, handler = null) {
            let lf = this;
            let resName = vo.resName;
            LoadQueue.Inst.loadSprite3D(UrlUtils.GetMapUrl(resName), Laya.Handler.create(lf, lf.addbase, [vo, handler]));
        }
        addbase(vo, handler, entity) {
            if (vo == null || vo.state != BaseVO.STATE_ADD)
                return;
            if (entity == null)
                return;
            SceneMgr.Inst.AddMap(vo.resName, entity);
            entity.name = vo.name;
            var cls = vo.scriptCls;
            if (cls != null) {
                let baseScript = entity.addComponent(cls);
                if (baseScript != null) {
                    baseScript.SetVO(vo);
                }
            }
            let rotationEuler = entity.transform.rotationEuler;
            vo.rotationEuler.cloneTo(rotationEuler);
            entity.transform.rotationEuler = rotationEuler;
            let position = entity.transform.position;
            vo.position.cloneTo(position);
            entity.transform.position = position;
            let scale = entity.transform.getWorldLossyScale();
            vo.scale.cloneTo(scale);
            entity.transform.setWorldLossyScale(scale);
            this.dic[vo.objId] = entity;
            if (handler != null)
                handler.runWith(entity);
            this.SetCollider(entity);
        }
        SetCollider(owner) {
            let phy = owner.getComponent(Laya.PhysicsCollider);
            if (phy) {
                phy.collisionGroup = this.GetCollisonGroup();
                phy.canCollideWith = this.GetCanCollideWith();
            }
        }
        GetCollisonGroup() {
            return CollisionType.TYPE_MAP;
        }
        GetCanCollideWith() {
            return CollisionType.TYPE_CAT;
        }
    }

    class TreeSceneTree extends BaseSceneTree {
        constructor(cls = null, isCheckRemove = false) {
            super(cls, isCheckRemove);
            this.dic = new Dictionary();
        }
        CheckEntity(value) {
            return value.landLv <= LandModel.Inst.level;
        }
        AddEntity(value, handler = null) {
            if (value.state != BaseVO.STATE_NORMAL) {
                return;
            }
            let entity = this.dic[value.objId];
            if (entity != null) {
                if (!entity.active) {
                    value.state = BaseVO.STATE_ADD;
                    entity.active = true;
                }
                return;
            }
            value.state = BaseVO.STATE_ADD;
            this.addScript(value, handler);
        }
        RemoveEntity(value, isDispose) {
            if (value.state != BaseVO.STATE_ADD) {
                return;
            }
            value.state = BaseVO.STATE_NORMAL;
            let entity = this.dic[value.objId];
            if (entity != null && entity.active) {
                entity.active = false;
            }
            if (isDispose) {
                if (!entity.destroyed)
                    entity.destroy();
                delete this.dic[value.objId];
            }
        }
        addScript(vo, handler = null) {
            let lf = this;
            let resName = vo.resName;
            let loadAction = ObjectPools.Inst.getObj(LoadSprite3dAction.KEY, LoadSprite3dAction);
            loadAction.url = UrlUtils.GetEntityLhUrl(resName);
            loadAction.handler = Laya.Handler.create(lf, lf.addbase, [vo, handler]);
            TreeSceneTree.actions.AddAction(loadAction);
        }
        addbase(vo, handler, entity) {
            if (vo == null || vo.state != BaseVO.STATE_ADD)
                return;
            if (entity == null)
                return;
            let mat = MathUtil.GetMaterial(entity);
            if (mat) {
                SceneMgr.Inst.AddTree(mat.name, entity);
            }
            entity.name = vo.name;
            var cls = vo.scriptCls;
            if (cls != null) {
                let baseScript = entity.addComponent(cls);
                if (baseScript != null) {
                    baseScript.SetVO(vo);
                }
            }
            let rotationEuler = entity.transform.rotationEuler;
            vo.rotationEuler.cloneTo(rotationEuler);
            entity.transform.rotationEuler = rotationEuler;
            let position = entity.transform.position;
            vo.position.cloneTo(position);
            entity.transform.position = position;
            let scale = entity.transform.getWorldLossyScale();
            vo.scale.cloneTo(scale);
            entity.transform.setWorldLossyScale(scale);
            this.dic[vo.objId] = entity;
            if (handler != null)
                handler.runWith(entity);
        }
        RemoveEntitys() {
            super.RemoveEntitys();
            this.dic = new Dictionary();
            TreeSceneTree.actions.ClearAll();
        }
    }
    TreeSceneTree.actions = new ActionQueue();

    class MapQuadTree {
        constructor() {
            this.trees = new Array();
            this.mapTree = new MapSceneTree(MapSceneTree);
            this.catTree = new CatSceneTree(CatSceneTree);
            this.buildTree = new BuildSceneTree(BuildSceneTree);
            this.treeTree = new TreeSceneTree(TreeSceneTree);
            this.Registers();
        }
        SetMapInit() {
            this.OnResize();
        }
        Start() {
            GlobalJugger.add(this);
        }
        Registers() {
            this.trees.push(this.mapTree);
            this.trees.push(this.treeTree);
            this.trees.push(this.buildTree);
            this.trees.push(this.catTree);
        }
        OnResize() {
            let left = -500;
            let top = -500;
            let width = 1000;
            let height = 1000;
            let count = 1;
            let mapMaxNum = 1;
            this.mapTree.Init(null, new Laya.Rectangle(left, top, width, height), count, mapMaxNum);
            this.treeTree.Init(null, new Laya.Rectangle(left, top, width, height), count, mapMaxNum);
            this.catTree.Init(null, new Laya.Rectangle(left, top, width, height), count, mapMaxNum);
            this.buildTree.Init(null, new Laya.Rectangle(left, top, width, height), count, mapMaxNum);
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new MapQuadTree();
            return this._Inst;
        }
        advanceTime() {
            for (let tree of this.trees) {
                tree.advanceTime();
            }
        }
        Clear() {
            for (let tree of this.trees) {
                tree.Clear();
            }
            GlobalJugger.remove(this);
        }
        AddVO(vo, isAddToScene = false, handler = null) {
            switch (vo.type) {
                case BaseVO.MAP:
                    this.mapTree.AddInfo(vo, isAddToScene, handler);
                    break;
                case BaseVO.CAT:
                    this.catTree.AddInfo(vo, isAddToScene, handler);
                    break;
                case BaseVO.BUILD:
                    this.buildTree.AddInfo(vo, isAddToScene, handler);
                    break;
                case BaseVO.TREE:
                    this.treeTree.AddInfo(vo, isAddToScene, handler);
                    break;
                default:
                    break;
            }
        }
        RemoveVO(vo) {
            switch (vo.type) {
                case BaseVO.MAP:
                    this.mapTree.RemoveInfo(vo);
                    break;
                case BaseVO.CAT:
                    this.catTree.RemoveInfo(vo);
                    break;
                case BaseVO.BUILD:
                    this.buildTree.RemoveInfo(vo);
                case BaseVO.TREE:
                    this.treeTree.RemoveInfo(vo);
                default:
                    break;
            }
        }
    }

    class PlatformModel {
        constructor() {
            this.platfrom = "";
            this.windowWidth = 0;
            this.windowHeight = 0;
            this.scaleX = 1;
            this.scaleY = 1;
            this.rightUpRect = null;
            this.bannerRect = null;
            this.bannerSizeW = 0;
            this.bannerSizeH = 100;
            this.Bannergold = 10000;
        }
        static get Inst() {
            return this.inst = this.inst || new PlatformModel();
        }
        init() {
            this.Bannergold = parseInt(ConfigValue.getItemByKey("Bannergold").value);
        }
        setWindowSize(windowWidth, windowHeight) {
            this.windowWidth = windowWidth;
            this.windowHeight = windowHeight;
            console.log("=====>Laya.stage.width: " + Laya.stage.width + " Laya.stage.height: " + Laya.stage.height);
            console.log("=====>Laya.stage.clientScaleX : " + Laya.stage.clientScaleX + "," + Laya.stage.clientScaleY);
            console.log("=====>windowWidth: " + windowWidth + " windowHeight: " + windowHeight);
            this.scaleX = this.windowWidth / Laya.stage.width;
            this.scaleY = this.windowHeight / Laya.stage.height;
            console.log("=====>this.scaleX: " + this.scaleX + " this.scaleY: " + this.scaleY);
        }
        setRect(x, y, width, height) {
            this.rightUpRect = this.rightUpRect || new Laya.Rectangle();
            this.rightUpRect.x = Math.ceil(x / this.scaleX);
            this.rightUpRect.y = Math.ceil(y / this.scaleY);
            this.rightUpRect.width = Math.ceil(width / this.scaleX);
            this.rightUpRect.height = Math.ceil(height / this.scaleY);
            console.log("右上角坐标: " + this.rightUpRect.x + "," + this.rightUpRect.y + "," + this.rightUpRect.width + "," + this.rightUpRect.height);
        }
        setBannerRect(x, y, width, height) {
            if (this.bannerRect == null) {
                this.bannerRect = new Laya.Rectangle();
                this.bannerRect.width = Math.ceil(width / this.scaleX);
                this.bannerRect.height = Math.ceil(height / this.scaleY);
                this.bannerSizeW = width || 320;
                this.bannerSizeH = height || 111;
            }
            if (Laya.Browser.onHWMiniGame) {
                this.bannerRect.width = Math.ceil(width);
                this.bannerRect.height = Math.ceil(height);
                this.bannerRect.x = Math.ceil(x);
                this.bannerRect.y = Math.ceil(y);
            }
            else {
                this.bannerRect.x = Math.ceil(x / this.scaleX);
                this.bannerRect.y = Math.ceil(y / this.scaleY);
            }
        }
    }
    PlatformModel.inst = null;
    PlatformModel.wx_muilt_grid = "adunit-c5e0bfcea3f05af2";

    class UINatiEmbedView extends UIBaseWindow {
        constructor() {
            super("ad", "UINatiEmbedView");
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_AD;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = false;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
        }
        AddListener() {
            super.AddListener();
            this.ad_icon.onClick(this, this.onBtnOkClick);
        }
        InitData(data) {
            super.InitData(data);
            this.ad_icon.visible = true;
            this.btn_cha.visible = true;
            let str = MathUtil.goldFormat(PlatformModel.Inst.Bannergold);
            this.btn_coin.title = str;
            AwardMgr.Inst.GetAward(ItemModel.ITEM_GOLD, AwardVO.TYPE_ITEM, PlatformModel.Inst.Bannergold, false);
            let adObj = data[0];
            let showHandler = data[1];
            this.clickHandler = data[2];
            this.adRes = data[3];
            if (showHandler != null)
                showHandler.run();
            let lf = this;
            let url = adObj["icon"];
            let iconArr = adObj["imgUrlList"];
            if (iconArr != null && iconArr.length > 0)
                url = iconArr[0];
            else {
                iconArr = adObj["logoUrlList"];
                if (iconArr != null && iconArr.length > 0)
                    url = iconArr[0];
            }
            if (!url)
                return;
            this.visible = false;
            this.ad_icon && AssetCache.LoadIcon(this.ad_icon, url, Laya.Handler.create(this, this.onLoadComplete1, [url]));
        }
        onLoadComplete1() {
            this.visible = true;
        }
        onBtnOkClick() {
            if (this.clickHandler != null) {
                this.clickHandler.run();
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_cha) {
                this.onBtnChaClick();
            }
        }
        onBtnChaClick() {
            this.ad_icon.visible = false;
            this.btn_cha.visible = false;
        }
        onBtnCloseClick() {
            let from = this.btn_coin.localToGlobal(0, 0);
            EffectMgr.Inst.playGoldEffect(PlatformModel.Inst.Bannergold, from);
            if (this.adRes != null)
                PlatformMgr.Inst.hideAd(this.adRes.id);
            super.onBtnCloseClick();
        }
        Reset() {
            super.Reset();
            this.adRes = null;
            if (this.clickHandler) {
                this.clickHandler.recover();
            }
            this.clickHandler = null;
        }
    }

    class AdCtrl {
        constructor() {
            this.wxIsLevelUp5 = false;
            this.wxIsShowEmbedAd = false;
            this.giveChuruShowAd = true;
        }
        static get Inst() {
            return this.inst = this.inst || new AdCtrl();
        }
        init() {
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame || Laya.Browser.onTTMiniGame) {
                this.wxIsLevelUp5 = false;
                this.wxIsShowEmbedAd = false;
            }
            else {
                this.wxIsLevelUp5 = true;
                this.wxIsShowEmbedAd = true;
            }
        }
        loadNative(nativeId, isDestroy = false) {
            if (isDestroy) {
            }
            Laya.timer.clear(this, this.loadNativeError);
            LoadQueue.Inst.loadNativeAd(nativeId, Laya.Handler.create(this, this.cacheNewNativeAd, [nativeId]), true);
        }
        cacheNewNativeAd(nativeId) {
            let url = "native_" + nativeId;
            let adRes = AssetCache.GetResource(url);
            if (adRes) {
                GameDispatcher.Inst.event(UIGameEvent.CACHE_NATIVE_AD, nativeId);
            }
            else {
            }
        }
        loadNativeError(nativeId) {
            Logger.debug(this, "=====>加载原生广告失败&重新加载 nativeId: " + nativeId);
            this.loadNative(nativeId);
        }
        reportNativeClick(nativeAd) {
            let self = this;
            AdModel.Inst.reportNativeClick(nativeAd);
            Laya.timer.once(600, self, self.destroyNative, [nativeAd, true]);
        }
        destroyNative(nativeId, isReLoad = false) {
            let adRes = AdModel.Inst.getNativeAd(nativeId);
            if (adRes instanceof NativeAdResource) {
                if (Laya.Browser.onHWMiniGame) {
                }
                else {
                    if (adRes.bornTime > 0 && (Laya.timer.currTimer - adRes.bornTime) < 20000) {
                        Logger.debug(this, "=====>原生广告[" + nativeId + "]销毁失败  创建时间太短!");
                        return;
                    }
                }
                Logger.debug(this, "=====>销毁原生广告[" + nativeId + "]成功");
                AdModel.Inst.destroyNative(nativeId);
                if (isReLoad) {
                    if (Laya.Browser.onQGMiniGame) {
                        Laya.timer.clear(this, this.reloadNativeAd);
                        Laya.timer.once(1000, this, this.reloadNativeAd);
                    }
                    else if (Laya.Browser.onHWMiniGame) {
                    }
                }
            }
        }
        reloadNativeAd() {
            LoadQueue.Inst.loadNativeAd(AdModel.nativeId_oppo);
        }
        addGrothCount(count = 1) {
            if (PlatformMgr.Inst.isWX) {
                if (AdModel.Inst.getPolicy(AdModel.p3) != AdModel.yes) {
                    count = 0;
                }
            }
            AdModel.Inst.gotGrothCount += count;
            GameDispatcher.Inst.event(UIGameEvent.UPDATE_GROTH_COUNT);
        }
        clearGrothCount(isSuccess) {
            if (isSuccess) {
                AdModel.Inst.gotGrothCount = 0;
                GameDispatcher.Inst.event(UIGameEvent.UPDATE_GROTH_COUNT);
            }
            else {
                I18nUtils.ShowMsg("观看视频未完成，领取奖励失败");
            }
        }
        addClickChurCount(count = 1) {
            if (AdModel.Inst.bannerChuruTime > 0 && (Laya.timer.currTimer - AdModel.Inst.bannerChuruTime) <= 30000) {
                AdModel.Inst.clickChuruCount = 0;
                return;
            }
            if (AdModel.Inst.clickChuruTime > 0 && (Laya.timer.currTimer - AdModel.Inst.clickChuruTime) > 200) {
                AdModel.Inst.clickChuruCount = 0;
            }
            AdModel.Inst.clickChuruCount += count;
            AdModel.Inst.clickChuruTime = Laya.timer.currTimer;
        }
        clearChuruCount() {
            AdModel.Inst.clickChuruCount = 0;
        }
        get canShowBoxBtn() {
            let bol = this.canShowBoxView && Laya.timer.currTimer - AdModel.Inst.boxLastCloseTime >= AdModel.Inst.boxShowIntervalTime;
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                return AdModel.Inst.getPolicy(AdModel.p5) == AdModel.yes && bol;
            }
            else {
                bol = false;
            }
            return bol;
        }
        get canShowBoxView() {
            return GuideModel.Inst.guideVO == null;
        }
        get PickupFecalBallShowBox() {
            let bol = this.canShowBoxView && Math.random() < AdModel.Inst.showBoxViewJilv;
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                return AdModel.Inst.getPolicy(AdModel.p6) == AdModel.yes && bol;
            }
            else {
                bol = false;
            }
            return bol;
        }
        SetBoxViewCloseTime() {
            AdModel.Inst.boxLastCloseTime = Laya.timer.currTimer;
        }
        DelayShowInsertAd() {
            Laya.timer.once(500, this, () => {
                PlatformMgr.Inst.showAd(AdType.ad_3);
            });
        }
        ShowNatiEmbed(view) {
            let isShow = false;
            if (Laya.Browser.onQGMiniGame && GuideModel.Inst.IsMainGuideAll()) {
                let p2 = AdModel.Inst.getPolicy(AdModel.p2);
                let delayTime = 0;
                if (p2 != null) {
                    p2 = p2 && p2.replace(/\，/g, ",");
                    let arr = p2.split(",");
                    let jv = arr[0] && parseInt(arr[0]) / 100;
                    isShow = Math.random() < jv;
                    delayTime = arr[1] ? parseInt(arr[1]) : 0;
                }
                isShow && Laya.timer.once(delayTime * 1000, this, this.ShowNatiEmbedAdView);
            }
            if (!isShow) {
                view.reshowBanner();
            }
        }
        ShowNatiEmbedAdView() {
            PlatformMgr.Inst.showAd(AdType.ad_32);
        }
        ClearShowNatiEmbedTimer() {
            Laya.timer.clear(this, this.ShowNatiEmbedAdView);
            UISystem.Inst.RemoveWindowClass(UINatiEmbedView);
        }
        SetBannerCloseBtnScale(btnClose) {
            if (Laya.Browser.onQGMiniGame) {
                let p1 = AdModel.Inst.getPolicy(AdModel.p1);
                let scale = parseFloat(p1);
                if (!isNaN(scale) && scale >= 0) {
                    btnClose.setScale(scale, scale);
                }
            }
        }
        AddDesktop(handler) {
            if (Laya.Browser.onQGMiniGame) {
                let complete = Laya.Handler.create(this, (isCreat) => {
                    if (!isCreat) {
                        PlatformMgr.Inst.installShortcut(handler);
                    }
                    else {
                        handler.runWith(isCreat);
                    }
                });
                PlatformMgr.Inst.hasShortcutInstalled(complete);
            }
        }
        DelayShowSkinViewBanner(showAdHander, hideAdHandler = null) {
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                this.DelayShowAndHideBannerByPolicy(AdModel.p8, showAdHander, hideAdHandler);
            }
        }
        DelayShowCatInviteBanner(showAdHander, hideAdHandler = null) {
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                this.DelayShowAndHideBannerByPolicy(AdModel.p7, showAdHander, hideAdHandler);
            }
        }
        DelayShowAndHideBannerByPolicy(policyKey, showAdHander, hideAdHandler = null) {
            let p = AdModel.Inst.getPolicy(policyKey);
            p = p && p.replace(/\，/g, ",");
            let arr = p && p.split(",");
            if (arr) {
                if (arr[0] && arr[0] == AdModel.yes) {
                    let time = arr[1] && parseInt(arr[1]);
                    if (isNaN(time) || time < 0) {
                        time = 0;
                    }
                    this.DelayShowAndHideBanner(time * 1000, showAdHander, hideAdHandler);
                }
                else {
                    hideAdHandler && hideAdHandler.run();
                }
            }
        }
        DelayShowAndHideBanner(time, showAdHander, hideAdHandler, hideDelayTime = 2000) {
            Laya.timer.once(time, this, () => {
                showAdHander && showAdHander.run();
                Laya.timer.once(hideDelayTime, this, () => {
                    hideAdHandler && hideAdHandler.run();
                }, null, false);
            }, null, false);
        }
        ClearBuildUpLevelCount() {
            AdModel.Inst.buildUpLevelCount = 0;
        }
        AddBuildUpLevelCount(count = 1) {
            AdModel.Inst.buildUpLevelCount += count;
        }
        get BuildCanUpFiveLevel() {
            return AdModel.Inst.buildUpLevelCount >= 10;
        }
    }
    AdCtrl.inst = null;

    class UIAlertView extends UIBaseWindow {
        constructor() {
            super("common", "UIAlertView");
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = LayerType.LAYOUT_CENTER;
            this.viewStruct.isShowMaskBg = true;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
            this.c2 = this._view.getController("c2");
            this.comp_cost = this.btn_ok2["_titleObject"];
        }
        InitData(data) {
            super.InitData(data);
            this.btn_close.visible = true;
            this.img_title.visible = false;
            this.txt_title.visible = true;
            this.txt_title.text = "提示";
            this.c2.selectedIndex = 0;
            if (typeof data === "object") {
                if (data.hasOwnProperty("cost")) {
                    this.c2.selectedIndex = 1;
                    this.comp_cost.InitData(data.cost);
                }
                if (data.hasOwnProperty("title")) {
                    if (data.title.indexOf("ui://") != -1) {
                        this.img_title.visible = true;
                        this.img_title.url = data.title;
                    }
                    else {
                        this.txt_title.visible = true;
                        this.txt_title.text = data.title;
                    }
                }
                if (data.hasOwnProperty("text")) {
                    this.txt_content.text = data.text.replace("\\n", "\n");
                }
                if (data.hasOwnProperty("flag")) {
                    this.setFlag(data.flag);
                }
                else {
                    this.setFlag(UIAlertView.OK);
                }
            }
            else if (typeof data === "string") {
                this.img_title.visible = false;
                this.txt_title.visible = true;
                this.txt_title.text = "提示";
                this.btn_ok.text = "确定";
                this.setFlag(UIAlertView.OK);
            }
            if (this.c1.selectedIndex == 1) {
                this.btn_ok.y = this.btn_ok2.y = this.btn_no.y;
                this.btn_ok.x = this.btn_ok2.x = (this.btn_no.x + 231);
            }
            else {
                this.btn_ok.y = this.btn_ok2.y = this.btn_no.y;
                this.btn_ok.x = this.btn_ok2.x = (this.width / 2 - this.btn_ok2.width / 2);
            }
            this.showBanner();
        }
        setFlag(value) {
            this.c1.selectedIndex = value;
        }
        onBtnCloseClick() {
            let closeHandler = this.Data && this.Data["closeHandler"];
            if (closeHandler != null) {
                closeHandler.run();
                closeHandler = null;
            }
            super.onBtnCloseClick();
        }
        onBtnClick(btn) {
            let self = this;
            super.onBtnClick(btn);
            if (btn == this.btn_ok) {
                let handler = this.Data && this.Data["okHandler"];
                if (handler != null) {
                    handler.run();
                    handler = null;
                }
                SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.Click_OK));
                this.close();
            }
            else if (btn == this.btn_ok2) {
                if (this.comp_cost.Data instanceof MoneyVo) {
                    let count;
                    if (this.comp_cost.Data.isDiamond) {
                        count = StorageUtils.Inst.GetDiamond();
                    }
                    else if (this.comp_cost.Data.isGold) {
                        count = MoneyModel.gold.num;
                    }
                    if (!this.comp_cost.Data.isLessThan(count)) {
                        UISystem.Inst.CreateWindowView(UITipView, `${this.comp_cost.Data.name}不足。`);
                        return;
                    }
                    if (this.comp_cost.Data.isDiamond) {
                        MoneyModel.diamond.cost(this.comp_cost.Data.num);
                    }
                    else if (this.comp_cost.Data.isGold) {
                        MoneyModel.gold.cost(this.comp_cost.Data.num);
                    }
                    SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.Click_OK));
                }
                let handler = this.Data && this.Data["okHandler"];
                if (handler != null) {
                    handler.run();
                    handler = null;
                }
                this.close();
            }
            else if (btn == this.btn_no) {
                let handler = this.Data && this.Data["cancelHandler"];
                if (handler != null) {
                    handler.run();
                    handler = null;
                }
                this.close();
            }
        }
        Reset() {
            super.Reset();
            fgui.GTween.kill(this);
        }
    }
    UIAlertView.OK = 0;
    UIAlertView.OK_AND_NO = 1;
    UIAlertView.NULL = 2;

    class Bezier2D {
        constructor(obj, time, delayTime, points, ease = Laya.Ease.linearIn, handler = null) {
            this.obj = null;
            this.passtime = 0;
            this.delayTime = 0;
            this.resultPos = null;
            this.obj = obj;
            this.delayTime = delayTime;
            this.time = time;
            this.points = points;
            this.resultPos = new Laya.Point();
            this.ease = ease;
            this.handler = handler;
        }
        start() {
            Laya.timer.frameLoop(1, this, this.fly);
            this.passtime = 0;
        }
        fly() {
            this.passtime += Laya.timer.delta;
            if (this.passtime <= this.delayTime)
                return;
            if ((this.passtime - this.delayTime) > this.time) {
                Laya.timer.clear(this, this.fly);
                this.obj.removeFromParent();
                this.handler && this.handler.run();
                return;
            }
            if (this.obj != null && this.points.length > 1) {
                let ratio = this.ease(this.passtime - this.delayTime, 0, 1, this.time);
                MathUtil.GetBezierPoint(this.points, ratio, this.resultPos);
                this.obj.x = this.resultPos.x;
                this.obj.y = this.resultPos.y;
            }
        }
    }

    class UIFlyCoinItem extends fgui.GComponent {
        constructor() {
            super();
        }
        constructFromXML(xml) {
            super.constructFromXML(xml);
            FairyUtils.setVar(this, this);
            this.c1 = this.getController("c1");
            this.c1.selectedIndex = 0;
        }
        static Create() {
            return PanelRegister.createGObject("main", "UIFlyCoinItem");
        }
        Recover() {
            this.Reset();
            PanelRegister.recover(this);
        }
        SetGold(val) {
            this.txt_gold.text = MathUtil.goldFormat(val);
        }
        FlyToTarget(targetPos) {
            let pointArr = [];
            let pointNum = MathUtil.makeRandom(1, 4);
            pointArr.push(new Laya.Point(this.x, this.y));
            for (let i = 0; i < pointNum; i++) {
                let x = MathUtil.makeRandom(0, Laya.stage.width);
                let y = MathUtil.makeRandom(0, Math.min(Laya.stage.height, this.y + 200));
                pointArr.push(new Laya.Point(x, y));
            }
            pointArr.push(targetPos);
            let bezier = new Bezier2D(this, 2000, 0, pointArr, Laya.Ease.sineIn);
            bezier.start();
        }
        Showy(time = 1000, offestY = 80) {
            Laya.Tween.to(this, { alpha: 0.5 }, Math.max(time - 700, 300), Laya.Ease.linearNone, null, Math.max(time - 300, 0));
            Laya.Tween.to(this, { y: this.y - offestY }, time, Laya.Ease.sineOut, Laya.Handler.create(this, () => {
                this.Recover();
            }));
        }
        SetType(type = 0) {
            this.c1.selectedIndex = type;
        }
        Reset() {
            Laya.Tween.clearAll(this);
            this.alpha = 1;
            this.scaleX = 1;
            this.scaleY = 1;
        }
    }
    UIFlyCoinItem.TYPE_BEZIER = 0;
    UIFlyCoinItem.TYPE_NORMAL = 1;
    UIFlyCoinItem.ITEM_TYPE_GOLD = 0;
    UIFlyCoinItem.ITEM_TYPE_CHURU = 1;
    UIFlyCoinItem.ITEM_TYPE_FISH = 2;

    class UICatInfoView extends UIBaseWindow {
        constructor() {
            super("cat", "UICatInfoView");
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_TOP;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            if (data instanceof CatVO) {
                PanelRegister.loadUrl(data.HeadIconUrl, Laya.Handler.create(this, (url) => {
                    this.img_icon.icon = url;
                }));
                this.txt_name.text = data.catRes.name;
                this.txt_desc.text = data.catRes.desc;
            }
            this.showBanner();
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
        }
        onResize() {
            super.onResize();
        }
        Reset() {
            super.Reset();
        }
    }

    class TBaseButton extends fgui.GButton {
        constructor() {
            super();
            this.discriminator = "I-AM-IComponent";
        }
        constructFromXML(xml) {
            super.constructFromXML(xml);
            FairyUtils.setVar(this, this);
            this.InitUI();
        }
        InitUI() {
        }
        InitData(param) {
        }
        get Data() {
            return null;
        }
        AddListener() { }
        RemoveListener() {
        }
        Reset() {
        }
        Destroy() {
        }
    }

    class Icon extends TBaseButton {
        constructor() {
            super();
        }
        InitUI() {
            this.touchable = false;
            if (this._iconObject instanceof fgui.GLoader) {
                this._iconObject.icon = "";
            }
        }
        InitData(data) {
            super.InitData(data);
            if (data instanceof Laya.Texture) {
            }
            else if (data) {
                this.setIcon(data);
            }
            else {
                this.Reset();
            }
        }
        set url(value) {
            if (!value) {
                this.Reset();
            }
            else {
                let pkgItem = fgui.UIPackage.getItemByURL(value);
                let url = pkgItem.owner.name + "|" + pkgItem.name;
                this.setIcon(url);
            }
        }
        get url() {
            return this._iconUrl;
        }
        setIcon(url, callback = null) {
            this._iconUrl = url;
            if (!url) {
                callback && callback.run();
                this.Reset();
            }
            else if (url.indexOf("|") != -1) {
                let arr = url.split("|");
                this.setSheet(arr[0], arr[1]);
            }
            else {
                this.icon = url;
            }
        }
        setSheet(pkgName, resName, path = UrlUtils.UI) {
            LoadQueue.Inst.loadFguiPkg(pkgName, path, Laya.Handler.create(this, this.onloadSheetComplete, [pkgName, resName]));
        }
        onloadSheetComplete(pkgName, resName, value) {
            if (!value.getRes(resName)) {
                value.loadResName(resName, Laya.Handler.create(this, (pkgName, resName) => {
                    this.icon = PanelRegister.getUrl(pkgName, resName);
                }, [pkgName, resName]));
            }
            else {
                this.icon = PanelRegister.getUrl(pkgName, resName);
            }
        }
        set texture(value) {
            if (this._iconObject instanceof fgui.GLoader) {
                this._iconObject["onExternalLoadSuccess"](value);
            }
        }
        get texture() {
            if (this._iconObject instanceof fgui.GLoader) {
                return this._iconObject.content.texture;
            }
            return null;
        }
        Reset() {
            super.Reset();
            this.icon = "";
            this._iconUrl = "";
        }
    }

    class TListButtonItem extends TBaseButton {
        constructor() {
            super();
            this.m_eventPool = null;
            this.m_param = null;
            this._btns = null;
            this.m_eventPool = new EventPool();
        }
        constructFromXML(xml) {
            super.constructFromXML(xml);
            this._btns = new Array();
            FairyUtils.setVar(this, this, this._btns);
            this.InitUI();
            this.AddListener();
        }
        InitUI() {
        }
        InitData(param) {
            this.m_param = param;
        }
        get Data() {
            return this.m_param;
        }
        setIcon(url) {
            if (this._iconObject instanceof Icon) {
                this._iconObject.url = url;
            }
            else {
                PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                    this.icon = url;
                }));
            }
        }
        AddListener() {
            TComponent.CheckComponent(this, "AddListener");
        }
        RemoveListener() {
            TComponent.CheckComponent(this, "RemoveListener");
        }
        AddGameListener(type, listener, thisObject, target) {
            this.m_eventPool.AddListenerInPool(type, listener, thisObject, target);
        }
        RemoveGameListener(type, listener, thisObject, target) {
            this.m_eventPool.RemoveListenerFromPool(type, listener, thisObject, target);
        }
        Reset() {
            TComponent.CheckComponent(this, "Reset");
            if (this.m_eventPool != null) {
                this.m_eventPool.Reset();
            }
        }
        Destroy() {
            TComponent.CheckComponent(this, "Destroy");
            if (this.m_eventPool) {
                this.m_eventPool = null;
            }
            this.m_param = null;
        }
    }

    class UISkinGoodItem extends TListButtonItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
            this.c3 = this.getController("c3");
            this.c4 = this.getController("c4");
            this.c5 = this.getController("c5");
            this.c6 = this.getController("c6");
        }
        InitData(data) {
            super.InitData(data);
            if (data instanceof SkinVO) {
                this.selected = data.isWear;
                this.setIcon(data.iconUrl);
                this.txt_name.text = data.itemRes.name;
                this.c1.selectedIndex = data.itemRes.quality;
                this.c2.selectedIndex = 1;
                this.c3.selectedIndex = 0;
            }
        }
        set part(value) {
            this.c4.selectedIndex = value;
        }
        get part() {
            return this.c4.selectedIndex;
        }
        set isLock(value) {
            this.c5.selectedIndex = value ? 1 : 0;
        }
        get isLock() {
            return this.c5.selectedIndex == 1;
        }
        set isSuit(value) {
            this.c6.selectedIndex = value ? 1 : 0;
        }
        get isSuit() {
            return this.c6.selectedIndex == 1;
        }
        get skinVo() {
            return this.Data;
        }
        Reset() {
            super.Reset();
        }
    }

    class UISkinPartItem extends TListButtonItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
        }
        InitData(data) {
            super.InitData(data);
            if (data instanceof SkinVO) {
                this.part = 0;
                this.setIcon(data.iconUrl);
            }
            else {
                this.setIcon("");
            }
        }
        set part(value) {
            this.c1.selectedIndex = value;
        }
        get part() {
            return this.c1.selectedIndex;
        }
        Reset() {
            super.Reset();
        }
    }

    class UISkinSuitItem extends TListItem {
        constructor() {
            super();
            this.isCatBag = true;
        }
        InitUI() {
            super.InitUI();
        }
        InitData(data) {
            super.InitData(data);
            if (data instanceof SuitVO) {
                this.txt_name.text = data.suitRes.name;
                let skinVos = data.skinVos;
                if (this.isCatBag) {
                    this.txt_count.text = this.catVo.GetSkinSuitOwnedCount(data.suitId) + "/" + data.skinIds.length;
                }
                else {
                    this.txt_count.text = data.skinOwnedCount + "/" + data.skinIds.length;
                }
                this.eglist_items.Data = skinVos;
                this.updateParts();
            }
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onClickBagItem, this, this.eglist_items);
        }
        get catVo() {
            return this.parent && this.parent.data;
        }
        updateParts() {
            let item = null;
            for (let i = 0; i < this.eglist_items.numChildren; i++) {
                item = this.eglist_items.getChildAt(i);
                if (item && item.Data instanceof SkinVO) {
                    if (this.isCatBag) {
                        if (this.catVo.HasSkin(item.Data.skinRes.id)) {
                            item.part = 0;
                        }
                        else {
                            item.part = item.Data.part;
                        }
                    }
                    else {
                        if (item.Data.hasOwned) {
                            item.part = 0;
                        }
                        else {
                            item.part = item.Data.part;
                        }
                    }
                    if (this.isInSkinView) {
                        item.selected = this.catVo.HasWear(item.Data.tempId);
                    }
                    else {
                        item.selected = false;
                    }
                }
            }
        }
        onClickBagItem(item) {
            if (item.part == 0) {
                GameDispatcher.Inst.event(UIGameEvent.CLICK_SUIT_ITEM, item);
            }
        }
        get isInSkinView() {
            if (this.parent && this.parent.parent && this.parent.parent.name == "UISkinView") {
                return true;
            }
            return false;
        }
        TakeOffCloth(skinId) {
            let item;
            for (let i = 0; i < this.eglist_items.numChildren; i++) {
                item = this.eglist_items.getChildAt(i);
                if (item instanceof UISkinGoodItem && item.skinVo.tempId == skinId) {
                    item.selected = false;
                    return true;
                }
            }
            return false;
        }
        Reset() {
            super.Reset();
            this.RemoveGameListener(fgui.Events.CLICK_ITEM, this.onClickBagItem, this, this.eglist_items);
        }
    }

    class UISkinGoodItem2 extends TListButtonItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
            this.c3 = this.getController("c3");
            this.c4 = this.getController("c4");
            this.c5 = this.getController("c5");
            this.c6 = this.getController("c6");
        }
        InitData(data) {
            super.InitData(data);
            if (data instanceof SkinVO) {
                this.selected = false;
                this.setIcon(data.iconUrl);
                this.txt_name.text = data.itemRes.name;
                this.c1.selectedIndex = data.itemRes.quality;
                this.c2.selectedIndex = 1;
                this.c3.selectedIndex = 0;
            }
        }
        set part(value) {
            this.c4.selectedIndex = value;
        }
        get part() {
            return this.c4.selectedIndex;
        }
        set isLock(value) {
            this.c5.selectedIndex = value ? 1 : 0;
        }
        get isLock() {
            return this.c5.selectedIndex == 1;
        }
        set isSuit(value) {
            this.c6.selectedIndex = value ? 1 : 0;
        }
        get isSuit() {
            return this.c6.selectedIndex == 1;
        }
        get skinVo() {
            return this.Data;
        }
        Reset() {
            super.Reset();
        }
    }

    class UISkinChangeAlert extends UIBaseWindow {
        constructor() {
            super("skin", "UISkinChangeAlert");
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_TOP;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowMaskBg = false;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            let items = data[0];
            this._catVo = data[1];
            this.c1.selectedIndex = data[2];
            this.eglist_items.Data = items;
            let gapRate = SkinModel.GetClothGoldAddRate(items);
            if (this.c1.selectedIndex == 0) {
                this.txt_desc.text = `是否赠送${this._catVo.catName}下列衣服？`;
                this.txt_from.text = "0%";
                this.txt_to.text = CatModel.getRate(gapRate);
            }
            else {
                this.txt_desc.text = `是否取回${this._catVo.catName}身上的下列衣服？`;
                this.txt_from.text = CatModel.getRate(gapRate);
                this.txt_to.text = "0%";
            }
            this.showBanner();
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_ok) {
                let skinVo;
                if (this.c1.selectedIndex == 0) {
                    for (let i = 0; i < this.eglist_items.Data.length; i++) {
                        skinVo = this.eglist_items.Data[i];
                        this.catVo.addSkin(skinVo.tempId);
                        SkinModel.Inst.RemoveSkin(skinVo.tempId);
                    }
                    GameDispatcher.Inst.event(UIGameEvent.GIVE_SKINS, [this.eglist_items.Data]);
                }
                else {
                    for (let i = 0; i < this.eglist_items.Data.length; i++) {
                        skinVo = this.eglist_items.Data[i];
                        this.catVo.takeOffCloth(skinVo.tempId);
                        this.catVo.removeCloth(skinVo.tempId);
                        SkinModel.Inst.AddSkin(skinVo);
                    }
                    GameDispatcher.Inst.event(UIGameEvent.GET_BACK_SKINS, [this.eglist_items.Data]);
                }
                this.close();
            }
            else if (btn == this.btn_no) {
                this.close();
            }
        }
        get catVo() {
            return this._catVo;
        }
        Reset() {
            super.Reset();
        }
    }

    class UISkinBagView extends UIBaseWindow {
        constructor() {
            super("skin", "UISkinBagView");
            this.tabSelectIndex = 0;
            this.tab1SelectIndex = 0;
            this.skinVos = [];
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("skin", "UISkinGoodItem2", UISkinGoodItem2);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
            this.eglist_suits.setVirtual();
            this.eglist_items.setVirtual();
            this.tab.selectionMode = fgui.ListSelectionMode.Single;
            this.tab1.selectionMode = fgui.ListSelectionMode.Single;
            this.c1 = this._view.getController("c1");
            this.c2 = this._view.getController("c2");
            this.p1 = this._view.getController("p1");
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                this.p1.selectedPage = "wx";
            }
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onTabClick, this, this.tab);
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onTab2Click, this, this.tab1);
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onClickBagItem, this, this.eglist_items);
            this.AddGameListener(UIGameEvent.CLICK_SUIT_ITEM, this.onClickSuitItem, this);
            this.AddGameListener(UIGameEvent.GIVE_SKINS, this.onGiveSKins, this);
            this.AddGameListener(UIGameEvent.GET_BACK_SKINS, this.onGetBackSkins, this);
        }
        InitData(data) {
            super.InitData(data);
            this.eglist_suits.data = this.eglist_items.data = this.catVo;
            this.tab.selectedIndex = 0;
            this.tab1.selectedIndex = 0;
            this.updateList();
            this.showBanner();
        }
        onTabClick(btn) {
            if (this.tabSelectIndex == this.tab.selectedIndex)
                return;
            this.tabSelectIndex = this.tab.selectedIndex;
            this.updateList();
        }
        onTab2Click(btn) {
            if (this.tab1SelectIndex == this.tab1.selectedIndex)
                return;
            this.tab1SelectIndex = this.tab1.selectedIndex;
            this.updateList();
        }
        updateList() {
            if (this.tab.selectedIndex == 0) {
                if (this.tab1.selectedIndex == 7) {
                    this.eglist_suits.itemUrl = PanelRegister.getUrl("common", "UIBagSuitItem");
                    this.eglist_suits.Data = SkinModel.Inst.GetAllSuitVos();
                }
                else {
                    this.eglist_items.itemUrl = PanelRegister.getUrl("common", "UIBagItem");
                    this.eglist_items.Data = SkinModel.Inst.GetAllSkinVos(this.tab1.selectedIndex);
                }
            }
            else if (this.tab.selectedIndex == 1) {
                if (this.tab1.selectedIndex == 7) {
                    this.eglist_suits.itemUrl = PanelRegister.getUrl("skin", "UISkinSuitItem");
                    this.eglist_suits.Data = SkinModel.Inst.GetAllSuitVos();
                }
                else {
                    this.eglist_items.itemUrl = PanelRegister.getUrl("skin", "UISkinGoodItem2");
                    this.eglist_items.Data = this.catVo.GetAllSkinVos(this.tab1.selectedIndex);
                }
            }
            this.eglist_items.scrollToTop();
            this.eglist_suits.scrollToTop();
            UISkinBagView.selectedBackVos.length = 0;
            UISkinBagView.selectedGiveVos.length = 0;
            this.UpdataBtnOkShow();
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_ok) {
                this.skinVos = this.getSelectedVos();
                UISystem.Inst.CreateWindowView(UISkinChangeAlert, [this.skinVos, this.catVo, this.tab.selectedIndex]);
            }
        }
        onClickBagItem(item) {
            if (this.tab.selectedIndex == 0 && item.Data instanceof SkinVO && this.catVo.HasSkin(item.Data.tempId)) {
                UISystem.Inst.CreateWindowView(UITipView, `已有${item.Data.itemRes.name}`);
                return;
            }
            item.selected = !item.selected;
            if (this.tab.selectedIndex == 0) {
                let index = UISkinBagView.selectedGiveVos.indexOf(item.Data);
                if (item.selected && index == -1) {
                    UISkinBagView.selectedGiveVos.push(item.Data);
                }
                else if (!item.selected && index != -1) {
                    UISkinBagView.selectedGiveVos.splice(index, 1);
                }
            }
            else {
                let index = UISkinBagView.selectedBackVos.indexOf(item.Data);
                if (item.selected && index == -1) {
                    UISkinBagView.selectedBackVos.push(item.Data);
                }
                else if (!item.selected && index != -1) {
                    UISkinBagView.selectedBackVos.splice(index, 1);
                }
            }
            this.UpdataBtnOkShow();
        }
        onClickSuitItem(item) {
            if (item.part == 0) {
                item.selected = !item.selected;
                if (this.tab.selectedIndex == 0) {
                    let index = UISkinBagView.selectedGiveVos.indexOf(item.Data);
                    if (item.selected && index == -1) {
                        UISkinBagView.selectedGiveVos.push(item.Data);
                    }
                    else if (!item.selected && index != -1) {
                        UISkinBagView.selectedGiveVos.splice(index, 1);
                    }
                }
                else {
                    let index = UISkinBagView.selectedBackVos.indexOf(item.Data);
                    if (item.selected && index == -1) {
                        UISkinBagView.selectedBackVos.push(item.Data);
                    }
                    else if (!item.selected && index != -1) {
                        UISkinBagView.selectedBackVos.splice(index, 1);
                    }
                }
            }
            this.UpdataBtnOkShow();
        }
        UpdataBtnOkShow() {
            let skinVos = this.getSelectedVos();
            if (!skinVos || skinVos.length == 0) {
                this.btn_ok.enabled = false;
            }
            else {
                this.btn_ok.enabled = true;
            }
        }
        onGiveSKins(skinVos) {
            this.updateList();
            UISkinBagView.selectedBackVos.length = 0;
            UISkinBagView.selectedGiveVos.length = 0;
        }
        onGetBackSkins(skinVos) {
            this.updateList();
            UISkinBagView.selectedBackVos.length = 0;
            UISkinBagView.selectedGiveVos.length = 0;
        }
        getSelectedVos() {
            if (this.tab.selectedIndex == 0) {
                return UISkinBagView.selectedGiveVos;
            }
            else {
                return UISkinBagView.selectedBackVos;
            }
        }
        get catVo() {
            return this.Data;
        }
        Reset() {
            super.Reset();
            this.skinVos = null;
            UISkinBagView.selectedBackVos.length = 0;
            UISkinBagView.selectedGiveVos.length = 0;
        }
    }
    UISkinBagView.selectedGiveVos = [];
    UISkinBagView.selectedBackVos = [];

    class UISkinView extends UIBaseWindow {
        constructor() {
            super("skin", "UISkinView");
            this.isOtherViewEnter = true;
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("skin", "UISkinPartItem", UISkinPartItem);
            PanelRegister.registerClass("skin", "UISkinGoodItem", UISkinGoodItem);
            PanelRegister.registerClass("skin", "UISkinSuitItem", UISkinSuitItem);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowMaskBg = false;
            this.viewStruct.isEffect = false;
        }
        InitUI() {
            super.InitUI();
            this.eglist_items.setVirtual();
            this.eglist_suit.setVirtual();
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onTabClick, this, this.tab);
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onClickSkinItem, this, this.eglist_items);
            this.AddGameListener(UIGameEvent.CLICK_SUIT_ITEM, this.onClickSkinItem, this);
            this.AddGameListener(UIGameEvent.CAT_TAKEOFF_CLOTH, this.onCatTakeOffCloth, this);
            this.AddGameListener(UIGameEvent.GIVE_SKINS, this.onGiveSKins, this);
            this.AddGameListener(UIGameEvent.GET_BACK_SKINS, this.onGetBackSkins, this);
        }
        InitData(data) {
            super.InitData(data);
            this.txt_name.text = this.catVo.catName;
            this.updateWearSkinList();
            this.updateCoinAdd();
            this.tab.selectedIndex = 0;
            this.onTabClick();
            this.CheckBtnRed();
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                this.hideBanner();
                if (this.isOtherViewEnter) {
                    this.isOtherViewEnter = false;
                    let handler = Laya.Handler.create(this, this.showBanner);
                    let hideBanner = Laya.Handler.create(this, this.hideBanner);
                    AdCtrl.Inst.DelayShowSkinViewBanner(handler, hideBanner);
                }
            }
            else {
                this.hideBanner();
            }
        }
        updateWearSkinList() {
            let item;
            for (let i = 1; i <= 6; i++) {
                item = this.getPartItem(i);
                item.part = i;
            }
            let skinVO;
            let wearClothList = this.catVo.getWearClothList();
            for (let i = 0; i < wearClothList.length; i++) {
                skinVO = wearClothList[i];
                if (skinVO) {
                    item = this.getPartItem(skinVO.part);
                    item.InitData(skinVO);
                }
            }
        }
        updateCoinAdd() {
            let total = 0;
            let cnt_n = 0;
            let cnt_r = 0;
            let cnt_s = 0;
            let skinVO;
            let ownedClothList = this.catVo.skinVoList;
            for (let i = 0; i < ownedClothList.length; i++) {
                skinVO = ownedClothList[i];
                if (skinVO) {
                    if (skinVO.quaIsN) {
                        cnt_n++;
                        total += 400;
                    }
                    else if (skinVO.quaIsR) {
                        cnt_r++;
                        total += 800;
                    }
                    else if (skinVO.quaIsS) {
                        cnt_s++;
                        total += 1500;
                    }
                }
            }
            this.txt_total.text = CatModel.getRate(total);
            this.txt_n_cnt.text = "" + cnt_n;
            this.txt_r_cnt.text = "" + cnt_r;
            this.txt_s_cnt.text = "" + cnt_s;
        }
        getPartItem(part) {
            return this["part_" + part];
        }
        onTabClick() {
            this.updateList();
        }
        onClickSkinItem(item) {
            if (UISystem.Inst.HasView(UISkinBagView)) {
                return;
            }
            if (item instanceof UISkinGoodItem) {
                let partItem = this.getPartItem(item.skinVo.part);
                if (!this.catVo.HasWear(item.skinVo.tempId)) {
                    this.catVo.wearCloth(item.skinVo.tempId);
                    item.selected = true;
                    partItem.InitData(item.skinVo);
                    partItem.part = 0;
                }
                else {
                    this.catVo.takeOffCloth(item.skinVo.tempId);
                    item.selected = false;
                    partItem.part = item.skinVo.part;
                }
            }
        }
        onCatTakeOffCloth(catVo, skinId) {
            if (this.catVo == catVo) {
                let item = null;
                if (this.tab.selectedIndex == 7) {
                    for (let i = 0; i < this.eglist_suit.numChildren; i++) {
                        item = this.eglist_suit.getChildAt(i);
                        if (item instanceof UISkinSuitItem && item.TakeOffCloth(skinId)) {
                            break;
                        }
                    }
                }
                else {
                    for (let i = 0; i < this.eglist_items.numChildren; i++) {
                        item = this.eglist_items.getChildAt(i);
                        if (item && item.skinVo.tempId == skinId) {
                            item.selected = false;
                            break;
                        }
                    }
                }
                let res = skinResource.getItemByKey(skinId);
                let partItem = res && this.getPartItem(res.icon);
                if (partItem) {
                    partItem.part = res.icon;
                }
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            let catVo = null;
            if (btn == this.btn_left) {
                catVo = CatModel.Inst.getPrevCatVo();
                if (catVo != this.catVo) {
                    this.InitData(catVo);
                    SceneMgr.Inst.GetCameraBuildScript().SelectedCat(catVo);
                }
            }
            else if (btn == this.btn_right) {
                catVo = CatModel.Inst.getNextCatVo();
                if (catVo != this.catVo) {
                    this.InitData(catVo);
                    SceneMgr.Inst.GetCameraBuildScript().SelectedCat(catVo);
                }
            }
            else if (btn == this.btn_catInfo) {
                UISystem.Inst.CreateWindowView(UICatInfoView, this.catVo);
            }
            else if (btn == this.btn_baseInfo) {
                UISystem.Inst.CreateWindowView(UICatDetailsView, this.catVo);
            }
            else if (btn == this.btn_give) {
                UISystem.Inst.CreateWindowView(UISkinBagView, this.catVo);
            }
        }
        onGiveSKins(skinVos) {
            for (let i = 0; i < skinVos.length; i++) {
                let skinVo = skinVos[i];
                let partItem = this.getPartItem(skinVo.part);
                this.catVo.wearCloth(skinVo.tempId);
                partItem.InitData(skinVo);
                partItem.part = 0;
            }
            this.updateList();
            this.CheckBtnRed();
        }
        onGetBackSkins(skinVos) {
            this.updateList();
        }
        updateList() {
            this.eglist_suit.data = this.eglist_items.data = this.catVo;
            if (this.tab.selectedIndex == 7) {
                this.eglist_suit.Data = SkinModel.Inst.GetAllSuitVos();
            }
            else {
                this.eglist_items.Data = this.catVo.GetAllSkinVos(this.tab.selectedIndex);
            }
            this.eglist_items.scrollToTop();
            this.eglist_suit.scrollToTop();
            this.updateCoinAdd();
        }
        CheckBtnRed() {
            let c1 = this.btn_give.getController("c1");
            if (c1) {
                let skinVos = SkinModel.Inst.GetAllSkinVos();
                let showRed = 0;
                if (skinVos) {
                    for (let i = 0; i < skinVos.length; i++) {
                        let skinVo = skinVos[i];
                        if (!skinVo.owner) {
                            showRed = 1;
                            break;
                        }
                    }
                }
                c1.selectedIndex = showRed;
            }
        }
        get catVo() {
            return this.Data;
        }
        onBtnCloseClick() {
            SceneMgr.Inst.GetCameraBuildScript().OverSelected();
            CatCtrl.Inst.StopLumao(this.catVo);
            super.onBtnCloseClick();
        }
        Reset() {
            super.Reset();
            this.isOtherViewEnter = true;
        }
    }

    class catlangResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint8();
            this.name = byteArray.readUint8();
            this.icon = byteArray.readUTFString();
        }
        static getItemByKey(id) {
            return catlangResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return catlangResource.itemsDic[key1 + "," + key2];
        }
    }
    catlangResource.items = new Array();
    catlangResource.itemsDic = new Dictionary();

    class CatLangModel {
        constructor() {
            this.langTypeMap = {};
        }
        static get Inst() {
            return this.inst = this.inst || new CatLangModel();
        }
        init() {
            let len = catlangResource.items.length;
            let item = null;
            for (let i = 0; i < len; i++) {
                item = catlangResource.items[i];
                let arr = this.langTypeMap[item.name];
                if (arr == null) {
                    arr = [];
                    this.langTypeMap[item.name] = arr;
                }
                arr.push(item);
            }
        }
        getCatLang(type, exceptTypes = null) {
            if (type == 0) {
                let items = null;
                if (exceptTypes && exceptTypes.length > 0) {
                    items = [];
                    let item = null;
                    for (let i = 0; i < catlangResource.items.length; i++) {
                        item = catlangResource.items[i];
                        if (exceptTypes.indexOf(item.name) == -1) {
                            items.push(item);
                        }
                    }
                }
                else {
                    items = catlangResource.items;
                }
                return items;
            }
            return this.langTypeMap[type];
        }
    }
    CatLangModel.inst = null;

    class CatTipComp extends TListItem {
        constructor() {
            super();
            this.index = 0;
            this.items = null;
            this.excepts = [];
        }
        InitUI() {
            super.InitUI();
        }
        InitData(data) {
            super.InitData(data);
            if (typeof data === "number") {
                let index = data > 0 ? data - 1 : 0;
                this.setItems([catlangResource.items[index]]);
                this.showTip();
            }
            else if (typeof data === "object") {
                let type = data.type;
                let exceptTypes = data.hasOwnProperty("exceptTypes") ? data.exceptTypes : [];
                let items = CatLangModel.Inst.getCatLang(type, exceptTypes);
                this.setItems(items);
                this.reshow();
            }
            else {
                this.setItems(catlangResource.items);
                this.reshow();
            }
        }
        setItems(items) {
            this.items = items;
            if (this.items) {
                this.excepts.length = 0;
                let item;
                if (this.items) {
                    for (let i = 0; i < this.items.length; i++) {
                        item = this.items[i];
                        this.excepts.push(i);
                    }
                }
                this.excepts = ArrayUtil.shuffle(this.excepts);
            }
        }
        showTip() {
            this.index = this.excepts.length == 1 ? 0 : this.excepts.pop();
            let item = this.items && this.items[this.index];
            if (item) {
                this.txt_tips.text = item.icon;
                fgui.GTween.to(0, 1, 0.6).setTarget(this).setEase(fgui.EaseType.ElasticInOut)
                    .onUpdate((tw) => {
                    this.setScale(tw.value.x, tw.value.x);
                    this.alpha = tw.value.x;
                }, this);
            }
            if (this.excepts.length <= 0) {
                this.setItems(this.items);
            }
        }
        reshow() {
            this.showTip();
            if (this.items == null || this.items.length <= 1) {
            }
            else {
                Laya.timer.once(10000, this, this.reshow);
            }
        }
        Reset() {
            super.Reset();
            this.txt_tips.text = "";
            fgui.GTween.kill(this);
            Laya.timer.clearAll(this);
            this.excepts.length = 0;
            this.items = null;
        }
    }

    class UICatIntimacyCostItem extends TListItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController('c1');
        }
        InitData(data) {
            super.InitData(data);
        }
        setItem(tempId, count = 1) {
            let itemRes = itemResource.getItemByKey(tempId);
            let url = itemRes ? ItemVO.GetItemUrl(itemRes) : "";
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                this.img_icon.url = url;
            }));
            this.txt_num.text = count + "";
        }
        set isLock(value) {
            this.c1.selectedIndex = value ? 1 : 0;
        }
        get isLock() {
            return this.c1.selectedIndex == 1;
        }
        Reset() {
            super.Reset();
        }
    }

    class UICatPostureItem extends TListButtonItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
        }
        InitData(data) {
            super.InitData(data);
            if (data instanceof CatPoseVO) {
                PanelRegister.loadUrl(data.iconUrl2, Laya.Handler.create(this, (url) => {
                    this.icon = url;
                }));
                if (data.isUnlock) {
                    this.c1.selectedIndex = 0;
                    this.touchable = false;
                }
                else {
                    if (data.canUnlock) {
                        if (data.conditionVo.isNeedGem) {
                            this.c1.selectedIndex = 2;
                        }
                        else {
                            this.c1.selectedIndex = 1;
                        }
                        this.touchable = true;
                    }
                    else {
                        this.c1.selectedIndex = 2;
                    }
                    if (data.conditionVo) {
                        if (data.conditionVo.isCatLevel) {
                            this.c2.selectedIndex = 0;
                            this.txt_lv.text = data.conditionVo.needCount + "级";
                        }
                        else if (data.conditionVo.isNeedGem) {
                            this.c2.selectedIndex = 1;
                            this.comp_cost.setMoney(MoneyVo.DIAMOND, data.conditionVo.needCount);
                            this.touchable = true;
                        }
                        else if (data.conditionVo.isViewAd) {
                            this.c2.selectedIndex = 2;
                            this.btn_ad.title = data.conditionVo.count + "/" + data.conditionVo.needCount;
                            this.touchable = true;
                        }
                    }
                }
            }
        }
        onClickAd() {
            let complete = Laya.Handler.create(this, (isSuccess) => {
                if (isSuccess) {
                    this.poseVo.conditionVo.count += 1;
                    StorageUtils.Inst.addPoseAd(1, this.poseVo.parent.catRes.id, this.poseVo.actRes.id);
                    this.btn_ad.title = this.poseVo.conditionVo.count + "/" + this.poseVo.conditionVo.needCount;
                    if (this.poseVo.canUnlock) {
                        this.poseVo.parent.unlockPose(this.poseVo.tempId);
                        this.unlock();
                    }
                }
            });
            PlatformMgr.Inst.showAd(AdType.ad_9, complete);
        }
        get conditionVo() {
            return this.poseVo && this.poseVo.conditionVo;
        }
        get isNeedGem() {
            return this.Data instanceof CatPoseVO && this.Data.conditionVo ? this.Data.conditionVo.isNeedGem : false;
        }
        get isAd() {
            return this.Data instanceof CatPoseVO && this.Data.conditionVo ? this.Data.conditionVo.isViewAd : false;
        }
        get isCatLevel() {
            return this.Data instanceof CatPoseVO && this.Data.conditionVo ? this.Data.conditionVo.isCatLevel : false;
        }
        get canLock() {
            return this.Data instanceof CatPoseVO && this.Data.conditionVo ? this.Data.conditionVo.IsFull : false;
        }
        get cost() {
            return this.comp_cost.Data;
        }
        unlock() {
            this.c1.selectedIndex = 0;
            this.touchable = false;
        }
        get poseVo() {
            return this.Data;
        }
        Reset() {
            if (this.m_eventPool != null) {
                this.m_eventPool.Reset();
            }
            this.m_param = null;
            this.icon = "";
            this.btn_ad.offClick(this, this.onClickAd);
        }
    }

    class UICatPostureItem2 extends TListButtonItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
        }
        InitData(data) {
            super.InitData(data);
            if (data instanceof CatPoseVO) {
                PanelRegister.loadUrl(data.iconUrl3, Laya.Handler.create(this, (url) => {
                    this.icon = url;
                }));
            }
        }
        get poseVo() {
            return this.Data;
        }
        Reset() {
            super.Reset();
        }
    }

    class UICatStateItem extends TListItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
        }
        InitData(data) {
            super.InitData(data);
            Laya.timer.clear(this, this.updateCoolTime);
            if (data instanceof CatVO) {
                if (data.catStrokeVo.IsBusy() || data.catStrokeVo.IsStroking() || data.catStrokeVo.IsStroked()) {
                    this.c1.selectedIndex = 1;
                    this.txt_title.text = `现在正在忙`;
                }
                else {
                    this.c1.selectedIndex = 0;
                }
            }
            else if (data instanceof CatStrokeVO) {
                if (data.IsBusy()) {
                    this.c1.selectedIndex = 1;
                    this.txt_title.text = `现在正在忙`;
                }
                else if (data.IsReady()) {
                    this.c1.selectedIndex = 2;
                    this.txt_title.text = `现在正在忙`;
                }
                else if (data.IsRefuse()) {
                    this.c1.selectedIndex = 3;
                    Laya.timer.loop(1000, this, this.updateCoolTime);
                }
                else if (data.IsAnger()) {
                    this.c1.selectedIndex = 4;
                    this.txt_title.text = `......`;
                }
                else if (data.IsStroked()) {
                    this.c1.selectedIndex = 5;
                    this.updateCoolTime();
                    Laya.timer.loop(1000, this, this.updateCoolTime);
                }
                else {
                    this.c1.selectedIndex = 0;
                    this.txt_title.text = `......`;
                }
            }
        }
        updateCoolTime() {
            if (this.Data instanceof CatStrokeVO) {
                if (this.Data.IsStroked()) {
                    let time = CatModel.Inst.touchTime - this.Data.strokeTime;
                    if (time >= 0) {
                        time = StringUtil.formatToTime(Math.floor(time * 0.001), "00:00");
                        this.txt_title.text = `刚刚撸过\n${time}`;
                    }
                }
                else if (this.Data.IsRefuse()) {
                    let time = CatModel.Inst.touchTime - this.Data.refuseTime;
                    if (time >= 0) {
                        time = StringUtil.formatToTime(Math.floor(time * 0.001), "00:00");
                        this.txt_title.text = `正在生气中\n${time}`;
                    }
                }
            }
        }
        Reset() {
            super.Reset();
            this.txt_title.text = "";
            Laya.timer.clear(this, this.updateCoolTime);
        }
    }

    class UICatChangeNameView extends UIBaseWindow {
        constructor() {
            super("cat", "UICatChangeNameView");
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
        }
        InitUI() {
            super.InitUI();
            this.btn_ok.title = "108.2万";
        }
        AddListener() {
            super.AddListener();
            this.btn_ok.enabled = MoneyModel.gold.cost(1082000, false, true);
        }
        InitData(data) {
            super.InitData(data);
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_ok) {
                if (this.txt_name.text == "") {
                    UISystem.Inst.CreateWindowView(UITipView, "名字不能为空!");
                }
                else {
                    if (this.Data instanceof CatVO) {
                        CatCtrl.Inst.ChangeName(this.Data, this.txt_name.text);
                    }
                    else {
                        let handler = this.Data && this.Data["changeHandler"];
                        handler && handler.runWith(this.txt_name.text);
                    }
                    this.close();
                    UISystem.Inst.CreateWindowView(UITipView, "修改名字成功");
                }
            }
        }
        Reset() {
            super.Reset();
        }
    }

    class ConditionVO {
        constructor(type, tempId, needCount) {
            this.type = ConditionVO.ITEM;
            this.tempId = 0;
            this.needCount = 0;
            this.count = 0;
            this.type = type;
            this.needCount = needCount;
            this.tempId = tempId;
        }
        get IsFull() {
            if (this.isNeedGold) {
                return MoneyModel.gold.isGreaterThanOrEqualTo(this.needCount);
            }
            else if (this.isNeedGem) {
                return StorageUtils.Inst.GetDiamond() >= this.needCount;
            }
            else if (this.isViewAd) {
                return this.count >= this.needCount;
            }
            return this.count >= this.needCount;
        }
        get IconUrl() {
            if (this.IsItem) {
                let itemRes = itemResource.getItemByKey(this.tempId);
                return ItemVO.GetItemUrl(itemRes);
            }
            else if (this.isRoom) {
                return PanelRegister.getUrl("cat", "house");
            }
            else if (this.isScore) {
                if (this.tempId == 1) {
                    return PanelRegister.getUrl("common", "score_relax");
                }
                else if (this.tempId == 2) {
                    return PanelRegister.getUrl("common", "score_food");
                }
            }
            else if (this.isBuildLevel) {
                let buildRes = bulidResource.getItemByKey(this.tempId);
                return buildRes ? UrlUtils.GetIconBuildUrl(buildRes.icon) : "";
            }
            return "";
        }
        get itemRes() {
            return this.IsItem ? itemResource.getItemByKey(this.tempId) : null;
        }
        get isNeedGold() {
            return this.itemRes && this.itemRes.id == ItemModel.ITEM_GOLD;
        }
        get isNeedGem() {
            return this.itemRes && this.itemRes.id == ItemModel.ITEM_GEM;
        }
        get Desc() {
            if (this.IsItem) {
                return "需收集喵星人喜欢的道具";
            }
            else if (this.isRoom) {
                return "岛内需有剩余的猫屋";
            }
            else if (this.isScore) {
                if (this.tempId == 1) {
                    return `岛内的休闲分数需高于${this.needCount}分。`;
                }
                else if (this.tempId == 2) {
                    return `岛内的餐饮分数需高于${this.needCount}分。`;
                }
            }
            else if (this.isBuildLevel) {
                let buildRes = bulidResource.getItemByKey(this.tempId);
                return `岛内的${buildRes.name}等级需高于${this.needCount}级。`;
            }
            return "";
        }
        get IsItem() {
            return this.type == ConditionVO.ITEM;
        }
        get isRoom() {
            return this.type == ConditionVO.ROOM;
        }
        get isScore() {
            return this.type == ConditionVO.SCORE;
        }
        get isBuildLevel() {
            return this.type == ConditionVO.BUILD_LEVEL;
        }
        get isCatLevel() {
            return this.type == ConditionVO.CAT_LEVEL;
        }
        get isViewAd() {
            return this.type == ConditionVO.AD_VIEW;
        }
    }
    ConditionVO.ITEM = 1;
    ConditionVO.ROOM = 2;
    ConditionVO.SCORE = 3;
    ConditionVO.BUILD_LEVEL = 4;
    ConditionVO.CAT_LEVEL = 5;
    ConditionVO.AD_VIEW = 6;

    class InviteView extends TComponent {
        constructor() {
            super();
            this.conditions = [];
        }
        InitFairyGui() {
            FairyUtils.setVar(this, this);
        }
        InitUI() {
            super.InitUI();
            this.eglist_items.isAutoSize = true;
        }
        InitData(data) {
            super.InitData(data);
            this.conditions.length = 0;
            let vo = null;
            if (data instanceof CatVO) {
                if (data.catRes.item > 0) {
                    vo = new ConditionVO(ConditionVO.ITEM, data.catRes.item, data.catRes.itemNum);
                    let itemVO = ItemModel.Inst.GetItemVOById(data.catRes.item);
                    vo.count = itemVO ? itemVO.getCount() : 0;
                    this.conditions.push(vo);
                }
                vo = new ConditionVO(ConditionVO.ROOM, 0, 1);
                vo.count = CatModel.Inst.GetLastMaowuCount();
                this.conditions.push(vo);
                let type = parseInt(data.catRes.unlockType[0]);
                let viceType = parseInt(data.catRes.unlockType[1]);
                let num = parseInt(data.catRes.unlockType[2]);
                if (type > 0) {
                    if (type == 1) {
                        vo = new ConditionVO(ConditionVO.SCORE, viceType, num);
                        vo.count = data.GetOtherScore();
                    }
                    else {
                        vo = new ConditionVO(ConditionVO.BUILD_LEVEL, viceType, num);
                        let buildVO = BuildModel.Inst.GetBuildVOBy(viceType);
                        vo.count = buildVO ? buildVO.level : 0;
                    }
                    this.conditions.push(vo);
                }
                this.eglist_items.Data = this.conditions;
            }
        }
        Reset() {
            super.Reset();
            this.conditions.length = 0;
        }
    }

    class VideoButton extends TListButtonItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            if (Laya.Browser.onHWMiniGame && PlatformMgr.Inst.isReview) {
                this.c1.selectedPage = "noad";
            }
            else if (PlatformMgr.Inst.isWX && PlatformMgr.Inst.isReview) {
                this.c1.selectedPage = "share";
            }
            else {
                this.c1.selectedPage = "ad";
            }
        }
        InitData(data) {
            super.InitData(data);
        }
        set isShowAd(value) {
            if (this.icon2) {
                this.icon2.url = this.icon;
            }
            this.c1.selectedPage = value ? "ad" : "noad";
        }
        SetAd(val) {
            this.c1.selectedIndex = val;
        }
        get isUpFiveLevel() {
            return this.c1.selectedPage == "5ji";
        }
        get isShowAd() {
            return this.c1.selectedPage == "ad";
        }
        Reset() {
            super.Reset();
        }
    }

    class BuildMaowuUpStageItem extends TListItem {
        constructor() {
            super();
            this.c1 = null;
            this.btn_lv = null;
            this.txt_addCatCount = null;
            this.buildVO = null;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
        }
        InitData(data) {
            super.InitData(data);
        }
        AddListener() {
            this.btn_lv.onClick(this, this.OnBtnLvClick);
        }
        OnBtnLvClick() {
            if (!BuildModel.Inst.canKuoJian)
                return;
            let costGold = BuildModel.Inst.nextMaowuUpRes ? BuildModel.Inst.nextMaowuUpRes.cost : 0;
            if (this.btn_lv.isShowAd) {
                PlatformMgr.Inst.showAd(AdType.ad_31, Laya.Handler.create(this, this.onAdShow, [costGold]));
            }
            else {
                BuildCtrl.Inst.MaoWuStageUp();
            }
        }
        onAdShow(award, isSuccess) {
            if (isSuccess) {
                AwardMgr.Inst.GetAward(ItemModel.ITEM_GOLD, AwardVO.TYPE_ITEM, award);
                this.UpdateBuildVO(this.buildVO);
            }
        }
        UpdateBuildVO(buildVO) {
            let index = 0;
            if (BuildModel.Inst.isMaowuMaxLevel) {
                index = 0;
            }
            else {
                if (BuildModel.Inst.canKuoJian) {
                    index = 2;
                }
                else {
                    index = 1;
                }
            }
            this.c1.selectedIndex = index;
            if (BuildModel.Inst.canKuoJian) {
                this.btn_lv.enabled = true;
                let maowuUpRes = BuildModel.Inst.nextMaowuUpRes;
                if (maowuUpRes) {
                    this.txt_addCatCount.text = "+" + (maowuUpRes.houseMaxLvl - BuildModel.Inst.maowuUpRes.houseMaxLvl) + "";
                    this.btn_lv.getChild("txt_count").text = MathUtil.goldFormat(maowuUpRes.cost);
                    this.btn_lv.isShowAd = MoneyModel.gold.isLessThan(maowuUpRes.cost);
                }
            }
            else {
                this.btn_lv.enabled = false;
            }
        }
        Reset() {
            super.Reset();
        }
        dispose() {
            this.Reset();
            super.dispose();
            this.buildVO = null;
        }
    }

    class UICatHeadItem extends TListButtonItem {
        constructor() {
            super();
            this.img_icon = null;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
            this.c3 = this.getController("c3");
            this.c4 = this.getController("c4");
            this.c5 = this.getController("c5");
            this.group_pro = this.getChild("group_pro").asGroup;
        }
        InitData(data) {
            super.InitData(data);
            if (data instanceof CatVO) {
                this.UpdateData();
            }
        }
        UpdateData() {
            let url = PanelRegister.getUrl("icon_cat_head", this.catVO.catRes.icon);
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                this.img_icon.url = url;
            }, [url]));
            this.txt_name.text = this.catVO.catRes.name;
            this.c1.selectedIndex = this.catVO.catRes.quality;
            if (this.catVO.IsVisitor) {
                this.c2.selectedIndex = 0;
                this.c3.selectedIndex = 5;
            }
            else if (this.catVO.IsHas()) {
                this.txt_lvl.visible = true;
                this.c2.selectedIndex = this.catVO.loveLv;
                this.c3.selectedIndex = 4;
                this.c4.selectedIndex = 1;
                this.txt_lvl.text = this.catVO.level + "j";
            }
            else {
                this.c2.selectedIndex = 0;
                if (this.catVO.CheckItemCount()) {
                    if (this.catVO.CanInvite()) {
                        this.c3.selectedIndex = 3;
                    }
                    else {
                        this.c3.selectedIndex = 2;
                    }
                }
                else {
                    this.c3.selectedIndex = 1;
                }
                this.c4.selectedIndex = 0;
                let itemVO = ItemModel.Inst.GetItemVOById(this.catVO.catRes.item);
                if (itemVO) {
                    this.group_pro.visible = itemVO.count > 0;
                    this.txt_rate.text = itemVO.count + "/" + this.catVO.catRes.itemNum;
                }
                else {
                    this.group_pro.visible = false;
                }
            }
        }
        showBlack(bol) {
            this.img_lock.visible = bol;
        }
        CanInvite() {
            return this.c3.selectedIndex == 3;
        }
        CanCheckCondition() {
            return this.c3.selectedIndex == 2;
        }
        set isHideName(value) {
            this.c4.selectedIndex = value ? 1 : 0;
        }
        get isHideName() {
            return this.c4.selectedIndex == 1;
        }
        get isSelect() {
            return this.c5.selectedIndex == 1;
        }
        set isSelect(bo) {
            this.c5.selectedIndex = bo ? 1 : 0;
        }
        get catVO() {
            return this.Data;
        }
        Reset() {
            super.Reset();
            if (this.img_lock)
                this.img_lock.visible = false;
        }
    }

    class UICatSelectView extends UIBaseWindow {
        constructor() {
            super("cat", "UICatSelectView");
            this.selectCatVo = null;
            this.buildVo = null;
            this.serverIndex = 0;
            this.preSelectItem = null;
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("cat", "UICatHeadItem", UICatHeadItem);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isEffect = true;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
            this.txt_gold = this.btn_sure.getChild("txt_gold").asTextField;
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.OnClickItem, this, this.eglist_cat);
        }
        InitData(data) {
            super.InitData(data);
            this.buildVo = data[0];
            this.serverIndex = data[1];
            let buildVo = this.buildVo;
            if (buildVo && buildVo.catServerArr) {
                this.buyPrice = buildVo.GetServerCostGold(this.serverIndex);
            }
            this.UpdateData();
            this.showBanner();
        }
        UpdateData() {
            let catArr = [];
            let cats = CatModel.Inst.GetHasCatVOs();
            let buildVo = this.buildVo;
            let serArr = [];
            if (buildVo && buildVo.catServerArr) {
                serArr = buildVo.catServerArr;
            }
            for (let i = 0; i < cats.length; i++) {
                if (serArr.indexOf(cats[i].catRes.id) == -1) {
                    catArr.push(cats[i]);
                }
            }
            this.eglist_cat.Data = catArr;
            this.eglist_cat.scrollToTop();
            this.eglist_cat.selectedIndex = 0;
            this.txt_gold.text = MathUtil.goldFormat(this.buyPrice);
            this.txt_tips.text = "请在喵星人定制服务中添加喵星人！\n添加为定制服务对象的喵星人使用设施后，可获得一个啾噜礼物。";
            if (MoneyModel.gold.isLessThan(this.buyPrice)) {
                this.btn_sure.enabled = false;
            }
            else {
                this.btn_sure.enabled = true;
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_sure) {
                this.OnBtnSureClick();
            }
            else if (btn == this.btn_cancel) {
                this.close();
            }
        }
        OnBtnSureClick() {
            let buildVo = this.buildVo;
            if (this.selectCatVo && buildVo && buildVo.catServerArr) {
                let costGold = this.buyPrice;
                if (MoneyModel.gold.isLessThan(costGold)) {
                    return I18nUtils.ShowI18nById("jbbz");
                }
                MoneyModel.gold.cost(costGold);
                buildVo.AddServerAt(this.selectCatVo.catRes.id, this.serverIndex);
                this.close();
            }
        }
        OnClickItem(item) {
            if (this.preSelectItem == item)
                return;
            this.selectCatVo = item.catVO;
            item.isSelect = true;
            if (this.preSelectItem) {
                this.preSelectItem.isSelect = false;
            }
            this.preSelectItem = item;
        }
        Reset() {
            super.Reset();
            this.selectCatVo = null;
            this.buildVo = null;
            this.serverIndex = 0;
            this.buyPrice = null;
            if (this.preSelectItem) {
                this.preSelectItem.isSelect = false;
            }
            this.preSelectItem = null;
        }
    }

    class ServerCatItem extends fgui.GComponent {
        constructor() {
            super();
            this.buildVo = null;
            this.index = 0;
        }
        constructFromXML(xml) {
            super.constructFromXML(xml);
            FairyUtils.setVar(this, this);
            this.ctr = this.getController("ctr");
            this.btn_add.onClick(this, this.OnBtnAddCatClick);
            this.img_head.onClick(this, this.ChangeCatServer);
        }
        OnBtnAddCatClick() {
            if (this.buildVo) {
                let serverArr = this.buildVo.catServerArr;
                if (serverArr.length == 4)
                    return;
                UISystem.Inst.CreateWindowView(UICatSelectView, [this.buildVo, this.index]);
            }
        }
        ChangeCatServer() {
            if (this.buildVo) {
                let serverArr = this.buildVo.catServerArr;
                if (serverArr.length < 4)
                    return;
                UISystem.Inst.CreateWindowView(UICatSelectView, [this.buildVo, this.index]);
            }
        }
        UpdateCatVO(buildVo, id = null) {
            this.index = id == null ? 0 : id;
            this.buildVo = buildVo;
            let catIdArr = buildVo.catServerArr;
            let catVo = id == null ? null : CatModel.Inst.GetCatVOById(catIdArr[id]);
            if (catVo) {
                this.ctr.selectedIndex = 1;
                let url = PanelRegister.getUrl("icon_cat_head", catVo.catRes.icon);
                PanelRegister.loadUrl(url, Laya.Handler.create(this, (headUrl) => {
                    this.img_head.url = headUrl;
                }));
            }
            else {
                this.ctr.selectedIndex = 0;
            }
        }
    }

    class UIBuildBanjiaLvUpItem extends TListItem {
        constructor() {
            super();
            this.txt_count = null;
            this.btn_lv = null;
            this.buildVO = null;
        }
        InitUI() {
            super.InitUI();
            this.txt_count = this.btn_lv.getChild("txt_count").asTextField;
        }
        InitData(data) {
            super.InitData(data);
        }
        AddListener() {
            this.btn_lv.onClick(this, this.onBtnLvClick);
            this.btn_lv.on(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            this.btn_lv.on(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            this.btn_lv.on(Laya.Event.MOUSE_OUT, this, this.onMouseUp);
        }
        onMouseDown(e) {
            if (GlobalConfig.isOpenAutoTouch) {
                Laya.timer.loop(GlobalConfig.autoTouchTimeGap, this, this.loopClick);
            }
        }
        onMouseUp(e) {
            Laya.timer.clear(this, this.loopClick);
        }
        loopClick() {
            if (this.btn_lv.enabled && this.btn_lv.visible && ObjectUtils.globalIsIn(this.btn_lv)) {
                this.onBtnLvClick(null);
            }
        }
        onBtnLvClick(event) {
            if (!this.buildVO)
                return;
            if (this.btn_lv.isShowAd) {
                PlatformMgr.Inst.showAd(AdType.ad_31, Laya.Handler.create(this, this.onAdShow, [this.buildVO.GetLvUpCostGold()]));
            }
            else if (this.btn_lv.isUpFiveLevel) {
                PlatformMgr.Inst.showAd(AdType.ad_31, Laya.Handler.create(this, this.onAdUpFiveLevel));
            }
            else {
                BuildCtrl.Inst.BuildLevelUp(this.buildVO);
                AdCtrl.Inst.AddBuildUpLevelCount();
            }
        }
        onAdUpFiveLevel(isSuccess) {
            if (isSuccess) {
                let count = BuildCtrl.Inst.BuildLevelUp2(this.buildVO, 5);
                I18nUtils.ShowMsg(this.buildVO.buildRes.name + "连升了" + (5 - count) + "级");
                AdCtrl.Inst.ClearBuildUpLevelCount();
                this.UpdateBuildVO(this.buildVO);
            }
        }
        onAdShow(award, isSuccess) {
            if (isSuccess) {
                AwardMgr.Inst.GetAward(ItemModel.ITEM_GOLD, AwardVO.TYPE_ITEM, award);
                this.UpdateBuildVO(this.buildVO);
            }
        }
        UpdateBuildVO(buildVO) {
            if (buildVO != this.buildVO) {
                AdCtrl.Inst.ClearBuildUpLevelCount();
            }
            this.buildVO = buildVO;
            if (buildVO) {
                this.txt_count.text = MathUtil.goldFormat(buildVO.GetLvUpCostGold());
                let ctr = this.getController("ctr");
                ctr.selectedIndex = buildVO.IsMaxLv() ? 1 : 0;
                let isOpenFastUpLevel = true;
                isOpenFastUpLevel = AdCtrl.Inst.wxIsLevelUp5;
                let costGold = buildVO.GetLvUpCostGold();
                if (isOpenFastUpLevel) {
                    this.btn_lv.isShowAd = MoneyModel.gold.isLessThan(costGold);
                    if (!this.btn_lv.isShowAd) {
                        if (AdCtrl.Inst.BuildCanUpFiveLevel) {
                            this.btn_lv.SetAd(2);
                        }
                    }
                }
                else {
                    this.btn_lv.isShowAd = false;
                }
            }
        }
        Reset() {
            super.Reset();
            AdCtrl.Inst.ClearBuildUpLevelCount();
            this.btn_lv.offClick(this, this.onBtnLvClick);
            this.btn_lv.off(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            this.btn_lv.off(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            this.btn_lv.off(Laya.Event.MOUSE_OUT, this, this.onMouseUp);
            Laya.timer.clear(this, this.loopClick);
        }
        dispose() {
            this.Reset();
            super.dispose();
            this.buildVO = null;
        }
    }

    class UIBuildLvUpItem extends TListItem {
        constructor() {
            super();
            this.txt_addGold = null;
            this.txt_sheshi = null;
            this.txt_count = null;
            this.btn_lv = null;
            this.sp_type2 = null;
            this.buildVO = null;
        }
        InitUI() {
            super.InitUI();
            this.txt_count = this.btn_lv.getChild("txt_count").asTextField;
        }
        InitData(data) {
            super.InitData(data);
            this.btn_lv.onClick(this, this.onBtnLvClick);
            this.btn_lv.on(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            this.btn_lv.on(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            this.btn_lv.on(Laya.Event.MOUSE_OUT, this, this.onMouseUp);
        }
        onMouseDown(e) {
            if (GlobalConfig.isOpenAutoTouch) {
                Laya.timer.loop(GlobalConfig.autoTouchTimeGap, this, this.loopClick);
            }
        }
        onMouseUp(e) {
            Laya.timer.clear(this, this.loopClick);
        }
        loopClick() {
            if (this.btn_lv.enabled && this.btn_lv.visible && ObjectUtils.globalIsIn(this.btn_lv)) {
                this.onBtnLvClick(null);
            }
        }
        onBtnLvClick(event) {
            if (this.buildVO == null || this.buildVO.IsMaxLv())
                return;
            if (this.btn_lv.isShowAd) {
                PlatformMgr.Inst.showAd(AdType.ad_31, Laya.Handler.create(this, this.onAdShow, [this.buildVO.GetLvUpCostGold()]));
            }
            else if (this.btn_lv.isUpFiveLevel) {
                PlatformMgr.Inst.showAd(AdType.ad_31, Laya.Handler.create(this, this.onAdUpFiveLevel));
            }
            else {
                BuildCtrl.Inst.BuildLevelUp(this.buildVO);
                AdCtrl.Inst.AddBuildUpLevelCount();
            }
        }
        onAdUpFiveLevel(isSuccess) {
            if (isSuccess) {
                let count = BuildCtrl.Inst.BuildLevelUp2(this.buildVO, 5);
                I18nUtils.ShowMsg(this.buildVO.buildRes.name + "连升了" + (5 - count) + "级");
                AdCtrl.Inst.ClearBuildUpLevelCount();
                this.UpdateBuildVO(this.buildVO);
            }
        }
        onAdShow(award, isSuccess) {
            if (isSuccess) {
                AwardMgr.Inst.GetAward(ItemModel.ITEM_GOLD, AwardVO.TYPE_ITEM, award);
                this.UpdateBuildVO(this.buildVO);
            }
        }
        UpdateBuildVO(buildVO) {
            if (buildVO != this.buildVO) {
                AdCtrl.Inst.ClearBuildUpLevelCount();
            }
            this.buildVO = buildVO;
            if (!buildVO)
                return;
            let addGold = BigNumber.minus(buildVO.GetAddGold(buildVO.level + 1), buildVO.GetAddGold(buildVO.level));
            this.txt_addGold.text = "+" + MathUtil.goldFormat(addGold);
            let addScore = buildVO.GetScore(buildVO.level + 1) - buildVO.GetScore(buildVO.level);
            this.txt_sheshi.text = "+" + addScore;
            this.txt_count.text = MathUtil.goldFormat(buildVO.GetLvUpCostGold());
            this.sp_type2.SetIndex(buildVO.buildRes.type);
            let ctr = this.getController("ctr");
            ctr.selectedIndex = buildVO.IsMaxLv() ? 1 : 0;
            let costGold = buildVO.GetLvUpCostGold();
            let isOpenFastUpLevel = AdCtrl.Inst.wxIsLevelUp5;
            if (GuideModel.Inst.guideVO != null)
                isOpenFastUpLevel = false;
            if (isOpenFastUpLevel) {
                this.btn_lv.isShowAd = MoneyModel.gold.isLessThan(costGold);
                if (!this.btn_lv.isShowAd) {
                    if (AdCtrl.Inst.BuildCanUpFiveLevel) {
                        this.btn_lv.SetAd(2);
                    }
                }
            }
            else {
                this.btn_lv.isShowAd = false;
            }
        }
        Reset() {
            super.Reset();
            AdCtrl.Inst.ClearBuildUpLevelCount();
            this.btn_lv.offClick(this, this.onBtnLvClick);
            Laya.stage.off(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            Laya.stage.off(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            Laya.timer.clear(this, this.loopClick);
            this.btn_lv.off(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            this.btn_lv.off(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            this.btn_lv.off(Laya.Event.MOUSE_OUT, this, this.onMouseUp);
        }
        dispose() {
            this.Reset();
            super.dispose();
            this.buildVO = null;
        }
    }

    class UIBuildMaowuLvUpItem extends TListItem {
        constructor() {
            super();
            this.txt_addCatCount = null;
            this.txt_count = null;
            this.btn_lv = null;
            this.buildVO = null;
        }
        InitUI() {
            super.InitUI();
            this.txt_count = this.btn_lv.getChild("txt_count").asTextField;
        }
        AddListener() {
            super.AddListener();
            this.btn_lv.onClick(this, this.onBtnLvClick);
            this.btn_lv.on(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            this.btn_lv.on(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            this.btn_lv.on(Laya.Event.MOUSE_OUT, this, this.onMouseUp);
        }
        RemoveListener() {
            super.RemoveListener();
            this.btn_lv.offClick(this, this.onBtnLvClick);
            this.btn_lv.off(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            this.btn_lv.off(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            this.btn_lv.off(Laya.Event.MOUSE_OUT, this, this.onMouseUp);
        }
        InitData(data) {
            super.InitData(data);
        }
        onMouseDown(e) {
            if (GlobalConfig.isOpenAutoTouch) {
                Laya.timer.loop(GlobalConfig.autoTouchTimeGap, this, this.loopClick);
            }
        }
        onMouseUp(e) {
            Laya.timer.clear(this, this.loopClick);
        }
        loopClick() {
            if (this.btn_lv.enabled && this.btn_lv.visible && ObjectUtils.globalIsIn(this.btn_lv)) {
                this.onBtnLvClick(null);
            }
        }
        onBtnLvClick(event) {
            if (!this.buildVO)
                return;
            let nextBaseRes = baseResource.getItemByKey(this.buildVO.level + 1);
            if (nextBaseRes == null)
                return;
            if (this.btn_lv.isShowAd) {
                PlatformMgr.Inst.showAd(AdType.ad_31, Laya.Handler.create(this, this.onAdShow, [nextBaseRes.cost]));
            }
            else {
                BuildCtrl.Inst.BuildLevelUp(this.buildVO);
            }
        }
        onAdShow(award, isSuccess) {
            if (isSuccess) {
                AwardMgr.Inst.GetAward(ItemModel.ITEM_GOLD, AwardVO.TYPE_ITEM, award);
                this.UpdateBuildVO(this.buildVO);
            }
        }
        UpdateBuildVO(buildVO) {
            this.buildVO = buildVO;
            if (!buildVO)
                return;
            let ctr = this.getController("ctr");
            let curBaseRes = baseResource.getItemByKey(buildVO.level);
            let nextBaseRes = baseResource.getItemByKey(buildVO.level + 1);
            if (buildVO.IsMaxLv() || nextBaseRes == null) {
                ctr.selectedIndex = 1;
            }
            else {
                ctr.selectedIndex = 0;
                let addCatCount = nextBaseRes.catMax - curBaseRes.catMax;
                this.txt_addCatCount.text = "+" + addCatCount;
                this.txt_count.text = MathUtil.goldFormat(nextBaseRes.cost);
            }
            if (nextBaseRes != null) {
                this.btn_lv.enabled = true;
                let costGold = nextBaseRes.cost;
                this.btn_lv.isShowAd = MoneyModel.gold.isLessThan(costGold);
            }
            else {
                this.btn_lv.enabled = false;
            }
        }
        Reset() {
            super.Reset();
            this.btn_lv.offClick(this, this.onBtnLvClick);
            this.btn_lv.off(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            this.btn_lv.off(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            this.btn_lv.off(Laya.Event.MOUSE_OUT, this, this.onMouseUp);
            Laya.timer.clear(this, this.loopClick);
        }
        dispose() {
            this.Reset();
            super.dispose();
            this.buildVO = null;
        }
    }

    class UIBuildMaowuView extends fgui.GComponent {
        constructor() {
            super();
            this.txt_des = null;
            this.txt_lv = null;
            this.txt_syCount = null;
            this.txt_ssCount = null;
            this.txt_zwCount = null;
            this.img_1 = null;
            this.sp_type1 = null;
            this.sp_lvup = null;
            this.sp_kuojiantishi = null;
        }
        constructFromXML(xml) {
            super.constructFromXML(xml);
            FairyUtils.setVar(this, this);
        }
        UpdateBuildVO(buildVO) {
            if (!buildVO)
                return;
            this.sp_lvup.UpdateBuildVO(buildVO);
            this.sp_kuojiantishi.UpdateBuildVO(buildVO);
            this.txt_des.text = buildVO.buildRes.desc;
            this.txt_lv.text = (buildVO.level - 1) + "级";
            let curBaseRes = baseResource.getItemByKey(buildVO.level);
            let catCount = CatModel.Inst.GetCatCount();
            this.txt_syCount.text = curBaseRes.catMax + "";
            this.txt_ssCount.text = catCount + "";
            this.txt_zwCount.text = (curBaseRes.catMax - catCount) + "";
            let url = UrlUtils.GetIconBuildUrl(buildVO.buildRes.icon);
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                if (this.img_1) {
                    this.img_1.url = url;
                }
            }));
            this.sp_type1.SetIndex(buildVO.buildRes.type);
        }
    }

    class TimeUtil {
        constructor() { }
        static formatToHMS(time, flex = ":") {
            time = NumberUtil.toInt(time);
            var h = (time - time % 3600) / 3600 % 24;
            var m = ((time % 3600) - (time % 60)) / 60;
            var s = time % 60;
            var mStr = "";
            if (h < 10)
                mStr += "0";
            mStr += h;
            mStr += flex;
            if (m < 10)
                mStr += "0";
            mStr += m;
            mStr += flex;
            if (s < 10)
                mStr += "0";
            mStr += s;
            return mStr;
        }
        static formatToMS(time, flex = ":") {
            var h = (time - time % 3600) / 3600 % 24;
            var m = ((time % 3600) - (time % 60)) / 60;
            var s = Math.floor(time % 60);
            var mStr = "";
            if (h > 0) {
                m += h * 60;
            }
            if (m < 10)
                mStr += "0";
            mStr += m;
            mStr += flex;
            if (s < 10)
                mStr += "0";
            mStr += s;
            return mStr;
        }
    }

    class UIBuildMatouItem extends TListItem {
        constructor() {
            super();
            this.img_icon = null;
            this.btn_findCat = null;
            this.btn_suoduanshijian = null;
            this.btn_qingshangdao = null;
            this.sp_yaoqingTime = null;
            this.txt_cdtime = null;
            this.sp_quality = null;
            this.m_index = 0;
        }
        constructFromXML(xml) {
            super.constructFromXML(xml);
            this.sp_yaoqingTime.min = 0;
            this.sp_yaoqingTime.max = 100;
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(Laya.Event.CLICK, this.onBtnFindCatClick, this, this.btn_findCat);
            this.AddGameListener(Laya.Event.CLICK, this.onBtnSuoduanshijianClick, this, this.btn_suoduanshijian);
            this.AddGameListener(Laya.Event.CLICK, this.onBtnQingshangdaoClick, this, this.btn_qingshangdao);
            this.AddGameListener(Laya.Event.CLICK, this.onBtnUpLvClick, this, this.btn_lv);
            this.AddGameListener(UIGameEvent.UPDATE_CURRENCY_DATA, this.onUpdateGold, this);
            this.AddGameListener(UIGameEvent.ONMATOU_UPLV, this.UpdateData, this);
        }
        RemoveListener() {
            super.RemoveListener();
        }
        onBtnFindCatClick() {
            let matou = MatouModel.Inst.GetMatouByIndex(this.m_index);
            if (matou == null)
                matou = {};
            let catVO = CatModel.Inst.GetRandomCatVO();
            if (catVO == null)
                return;
            let cdTime = MatouModel.Inst.GetTimeByQuality(catVO.catRes.quality) * 1000;
            matou["endTime"] = TimeModel.Inst.LocalTime + cdTime;
            matou["catId"] = catVO.catRes.id;
            MatouModel.Inst.SetMatouList(this.m_index, matou);
            this.UpdateData();
            GameDispatcher.Inst.event(UIGameEvent.MATOU_JIAN_CD_TIME);
            BuildCtrl.Inst.CheckMatou();
            LoadQueue.Inst.loadSprite3D(UrlUtils.GetCatUrl(catVO.resName));
            GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, TaskType.TASKTYPE_GET_CAT);
        }
        onBtnSuoduanshijianClick() {
            let nowTime = (new Date()).getTime();
            let endTime = MatouModel.Inst.GetAdTime();
            if (nowTime < endTime) {
                I18nUtils.ShowI18nById("matou_ad_cd");
                return;
            }
            let adCount = MatouModel.Inst.GetAdCount();
            if (adCount >= MatouModel.Inst.boatADDay) {
                I18nUtils.ShowI18nById("matou_no_ad_count");
                return;
            }
            PlatformMgr.Inst.showAd(AdType.ad_23, Laya.Handler.create(this, this.onAdComplete));
        }
        onAdComplete(isSuccess) {
            if (!isSuccess)
                return;
            let boatCd = MatouModel.Inst.boatCd * 1000;
            MatouModel.Inst.SetAdTime((new Date()).getTime() + boatCd);
            let adCount = MatouModel.Inst.GetAdCount();
            adCount++;
            MatouModel.Inst.SetAdCount(adCount);
            let matou = MatouModel.Inst.GetMatouByIndex(this.m_index);
            if (matou == null)
                return;
            let boatADTime = MatouModel.Inst.boatADTime;
            matou["endTime"] -= boatADTime * 1000;
            MatouModel.Inst.SetMatouList(this.m_index, matou);
            this.UpdateData();
            GameDispatcher.Inst.event(UIGameEvent.MATOU_JIAN_CD_TIME);
            BuildCtrl.Inst.CheckMatou();
        }
        onBtnQingshangdaoClick() {
            let matou = MatouModel.Inst.GetMatouByIndex(this.m_index);
            if (matou == null)
                return;
            if (!EntityModel.Inst.IsHasKongChuan()) {
                I18nUtils.ShowI18nById("matou_no_port");
                return;
            }
            let newCatVO = CatModel.Inst.AddVisitorCatVO(matou["catId"]);
            MatouModel.Inst.SetMatouList(this.m_index, null);
            this.UpdateData();
            GameDispatcher.Inst.event(UIGameEvent.MATOU_JIAN_CD_TIME);
            BuildCtrl.Inst.CheckMatou();
            SceneMgr.Inst.GetCameraBuildScript().ResetSelectBuild();
            UISystem.Inst.RemoveWindowClass(UIBuildView);
            MapQuadTree.Inst.AddVO(newCatVO, true);
            GameDispatcher.Inst.event(UIGameEvent.DAYTASK_COMP_PROP, TaskVO.YAOQINGSHANGAN);
        }
        onBtnUpLvClick() {
            if (this.btn_lv.isShowAd) {
                let vo = BuildModel.Inst.matouBuildVO;
                if (vo) {
                    let costGold = vo.GetLvUpCostGold(this.m_index);
                    PlatformMgr.Inst.showAd(AdType.ad_31, Laya.Handler.create(this, this.onAdShow, [costGold]));
                }
            }
            else {
                BuildCtrl.Inst.BuildLevelUp(BuildModel.Inst.matouBuildVO);
            }
        }
        onAdShow(award, isSuccess) {
            if (isSuccess) {
                AwardMgr.Inst.GetAward(ItemModel.ITEM_GOLD, AwardVO.TYPE_ITEM, award);
                this.onUpdateGold();
            }
        }
        onUpdateGold() {
            let vo = BuildModel.Inst.matouBuildVO;
            let costGold = vo.GetLvUpCostGold(this.m_index);
            let isShowAd = MoneyModel.gold.isLessThan(costGold);
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame || Laya.Browser.onTTMiniGame) {
                isShowAd = false;
            }
            this.btn_lv.isShowAd = isShowAd;
        }
        OnCameraComplete(newCatVO) {
        }
        SetIndex(index) {
            this.m_index = index;
            this.UpdateData();
        }
        UpdateData() {
            let ctr = this.getController("ctr");
            Laya.Tween.clearAll(this.sp_yaoqingTime);
            let qualityCtr = this.sp_quality.getController("ctr");
            let matouNum = LandModel.Inst.boatNum;
            let vo = BuildModel.Inst.matouBuildVO;
            if (this.m_index >= matouNum) {
                ctr.selectedIndex = 3;
                qualityCtr.selectedIndex = 0;
            }
            else if (this.m_index == vo.level) {
                ctr.selectedIndex = 4;
                let vo = BuildModel.Inst.matouBuildVO;
                let costGold = vo.GetLvUpCostGold(this.m_index);
                this.btn_lv.getChild("txt_count").text = MathUtil.goldFormat(costGold);
                this.onUpdateGold();
            }
            else if (this.m_index > vo.level) {
                ctr.selectedIndex = 5;
            }
            else {
                this.img_icon.url = "ui://build/marinaLv" + (this.m_index + 1);
                let matou = MatouModel.Inst.GetMatouByIndex(this.m_index);
                if (matou == null) {
                    ctr.selectedIndex = 0;
                    qualityCtr.selectedIndex = 0;
                }
                else {
                    let endTime = matou["endTime"];
                    let nowTime = (new Date()).getTime();
                    if (endTime > 0 && nowTime < endTime) {
                        let lastTime = endTime - nowTime;
                        ctr.selectedIndex = 1;
                        let catId = matou["catId"];
                        let catVO = CatModel.Inst.GetCatVOById(catId);
                        if (catVO == null)
                            return;
                        let cdTime = MatouModel.Inst.GetTimeByQuality(catVO.catRes.quality) * 1000;
                        let rate = 1 - (lastTime / cdTime);
                        this.sp_yaoqingTime.value = Math.ceil(rate * 100);
                        Laya.Tween.to(this.sp_yaoqingTime, { "value": 100 }, lastTime, null, Laya.Handler.create(this, this.UpdateData), 0, true).update = Laya.Handler.create(this, this.onUpdate, [cdTime], false);
                        qualityCtr.selectedIndex = catVO.catRes.quality;
                    }
                    else {
                        ctr.selectedIndex = 2;
                        qualityCtr.selectedIndex = 0;
                    }
                }
            }
        }
        onUpdate(cdTime) {
            let time = (1 - this.sp_yaoqingTime.value / 100) * cdTime;
            let str = TimeUtil.formatToHMS(time * 0.001);
            if (str == this.txt_cdtime.text)
                return;
            this.txt_cdtime.text = str;
        }
        Reset() {
            super.Reset();
            Laya.Tween.clearAll(this.sp_yaoqingTime);
        }
    }

    class UIBuildMatouView extends fgui.GComponent {
        constructor() {
            super();
            this.txt_des = null;
            this.txt_cdtime = null;
            this.txt_adCount = null;
            this.img_timeDi = null;
            this.img_1 = null;
            this.sp_list = null;
        }
        constructFromXML(xml) {
            super.constructFromXML(xml);
            FairyUtils.setVar(this, this);
            this.sp_list.itemRenderer = Laya.Handler.create(this, this.RenderListItem, null, false);
            this.sp_list.setVirtual();
        }
        RenderListItem(index, item) {
            item.SetIndex(index);
        }
        UpdateBuildVO(buildVO) {
            this.txt_des.text = buildVO.buildRes.desc;
            let url = UrlUtils.GetIconBuildUrl(buildVO.buildRes.icon);
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                if (this.img_1) {
                    this.img_1.url = url;
                }
            }));
            this.sp_list.numItems = 4;
            this.UpdateCdTime();
        }
        UpdateCdTime() {
            Laya.timer.clearAll(this);
            Laya.timer.loop(1000, this, this.UpdateMatouAd);
            this.UpdateMatouAd();
        }
        UpdateMatouAd() {
            let nowTime = TimeModel.Inst.LocalTime;
            let endTime = MatouModel.Inst.GetAdTime();
            let time = (endTime - nowTime) * 0.001;
            if (time > 0) {
                this.txt_cdtime.visible = true;
                this.img_timeDi.visible = true;
                this.txt_cdtime.text = TimeUtil.formatToMS(time);
            }
            else {
                this.txt_cdtime.visible = false;
                this.img_timeDi.visible = false;
                Laya.timer.clearAll(this);
            }
            this.txt_adCount.text = MatouModel.Inst.GetAdCount() + "/" + MatouModel.Inst.boatADDay;
        }
        Reset() {
            for (let i = 0; i < this.sp_list.numChildren; i++) {
                let item = this.sp_list.getChildAt(i);
                item.Reset();
            }
            Laya.timer.clearAll(this);
        }
    }

    class UIBuildMoveHouseView extends TListItem {
        constructor() {
            super();
            this.txt_des = null;
            this.txt_lv = null;
            this.txt_addRateAll = null;
            this.img_1 = null;
            this.img_2 = null;
            this.img_rategold = null;
            this.img_ratedi = null;
            this.sp_exp = null;
            this.sp_lvup = null;
            this.buildVo = null;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
        }
        InitData(data) {
            super.InitData(data);
        }
        UpdateBuildVO(buildVO) {
            if (!buildVO)
                return;
            this.buildVo = buildVO;
            this.sp_lvup.UpdateBuildVO(buildVO);
            this.txt_des.text = buildVO.buildRes.desc;
            this.txt_lv.text = buildVO.level + "级";
            let url = UrlUtils.GetIconBuildUrl(buildVO.buildRes.icon);
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                if (this.img_1) {
                    this.img_1.url = url;
                }
            }));
            this.sp_exp.min = 0;
            this.sp_exp.max = buildVO.GetNeedUpMaxLv();
            this.sp_exp.tweenValue(buildVO.GetCurUpUnlockLv(), 0.2);
            let nextBuildUpRes = buildVO.GetNextStage();
            this.c1.selectedIndex = nextBuildUpRes ? 0 : 1;
            if (this.c1.selectedIndex == 0) {
                this.txt_addRateAll.text = nextBuildUpRes.unlockLvl + "级";
                let url = UrlUtils.GetIconBuildUrl(nextBuildUpRes.icon);
                PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                    if (url)
                        this.img_2.url = url;
                }));
            }
        }
    }

    class UIBuildPeijianItem extends fgui.GComponent {
        constructor() {
            super();
            this.img_1 = null;
            this.txt_addRate = null;
            this.ctr = null;
        }
        constructFromXML(xml) {
            super.constructFromXML(xml);
            FairyUtils.setVar(this, this);
            this.ctr = this.getController("ctr");
        }
        SetItemId(itemId, showLevel = false) {
            let itemRes = itemResource.getItemByKey(itemId);
            let itemVO = ItemModel.Inst.GetItemVOById(itemId);
            let url = itemRes.icon;
            let showAddition = 0;
            if (itemVO != null) {
                showAddition = 1;
            }
            else {
                url += "_2";
                showAddition = 0;
            }
            this.SetLevel(showLevel);
            this.SetIcon(url);
            this.SetAddition(showAddition, BuildModel.Inst.addonItem);
        }
        SetLevel(showLevel = false, level = 0) {
            this.txt_level.visible = showLevel;
            this.txt_level.text = "lv" + level;
        }
        SetIcon(icon) {
            let url = UrlUtils.GetIconPartUrl(icon);
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                if (this.img_1) {
                    this.img_1.url = url;
                }
            }));
        }
        SetAddition(showAddition = 0, addition = BuildModel.Inst.addonItem) {
            this.ctr.selectedIndex = showAddition;
            this.txt_addRate.text = "+" + addition * 100 + "%";
        }
        dispose() {
            super.dispose();
            this.img_1 = null;
            this.ctr = null;
        }
    }

    class UIBuildZuoweiUpItem extends TListItem {
        constructor() {
            super();
            this.txt_zwCount = null;
            this.txt_count = null;
            this.btn_lv = null;
            this.buildVO = null;
        }
        InitUI() {
            super.InitUI();
            this.txt_count = this.btn_lv.getChild("txt_count").asTextField;
        }
        InitData(data) {
            super.InitData(data);
            this.btn_lv.onClick(this, this.onBtnLvClick);
            this.btn_lv.on(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            this.btn_lv.on(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            this.btn_lv.on(Laya.Event.MOUSE_OUT, this, this.onMouseUp);
        }
        onMouseDown(e) {
            if (GlobalConfig.isOpenAutoTouch) {
                Laya.timer.loop(GlobalConfig.autoTouchTimeGap, this, this.loopClick);
            }
        }
        onMouseUp(e) {
            Laya.timer.clear(this, this.loopClick);
        }
        loopClick() {
            if (this.btn_lv.enabled && this.btn_lv.visible && ObjectUtils.globalIsIn(this.btn_lv)) {
                this.onBtnLvClick(null);
            }
        }
        onBtnLvClick(event) {
            if (!this.buildVO)
                return;
            if (!this.buildVO.IsShowZuowei())
                return;
            let costGold = this.buildVO.GetZuoweiCostGold();
            if (this.btn_lv.isShowAd) {
                PlatformMgr.Inst.showAd(AdType.ad_31, Laya.Handler.create(this, this.onAdShow, [costGold]));
            }
            else {
                BuildCtrl.Inst.BuildZuoweiLvUp(this.buildVO);
                MoneyModel.gold.cost(costGold);
            }
        }
        onAdShow(award, isSuccess) {
            if (isSuccess) {
                AwardMgr.Inst.GetAward(ItemModel.ITEM_GOLD, AwardVO.TYPE_ITEM, award);
                this.UpdateBuildVO(this.buildVO);
            }
        }
        UpdateBuildVO(buildVO) {
            this.buildVO = buildVO;
            this.txt_zwCount.text = "+1";
            this.txt_count.text = MathUtil.goldFormat(buildVO.GetZuoweiCostGold());
            let costGold = this.buildVO.GetZuoweiCostGold();
            this.btn_lv.isShowAd = MoneyModel.gold.isLessThan(costGold);
        }
        Reset() {
            super.Reset();
            this.btn_lv.offClick(this, this.onBtnLvClick);
            this.btn_lv.off(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            this.btn_lv.off(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            this.btn_lv.off(Laya.Event.MOUSE_OUT, this, this.onMouseUp);
            Laya.timer.clear(this, this.loopClick);
        }
        dispose() {
            this.Reset();
            super.dispose();
            this.buildVO = null;
        }
    }

    class UIExplainView extends UIBaseWindow {
        constructor() {
            super("build", "UIExplainView");
            this.c1 = null;
            this.buildVo = null;
            this.index = 0;
            this.explain0 = "-抽取喵星人道具时，可获得特殊道具。\n-每获得一个特殊道具，便可提升一级。\n-等级提升后，可以从设施处获得额外金币奖励。";
            this.explain1 = "-添加为定制服务对象的喵星人，每次使用设施即可获得1个啾噜礼物。\n-请将希望快速提升等级的喵星人添加为定制服务对象。\n-添加为定制服务对象的喵星人数量达到4只时，可以免费进行更换。";
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layout = 8;
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isEffect = true;
            this.viewStruct.isExclusion = false;
        }
        registerClasses() {
            super.registerClasses();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
        }
        InitData(param) {
            this.m_param = param;
            this.index = param[0];
            this.buildVo = param[1];
            this.UpdateData();
        }
        AddListener() {
            super.AddListener();
        }
        UpdateData() {
            let index = this.index;
            this.c1.selectedIndex = index;
            if (index == 0) {
                this.txt_explain.text = this.explain0;
                this.UpdatePeijianItem();
            }
            else if (index == 1) {
                this.txt_explain.text = this.explain1;
            }
        }
        UpdatePeijianItem() {
            if (!this.buildVo)
                return;
            let item = this.explainItem0;
            let peijianItem0 = item.getChild("peijianItem0");
            let peijianItem1 = item.getChild("peijianItem1");
            let peijianItem2 = item.getChild("peijianItem2");
            let itemId = this.buildVo.buildRes.itemId2;
            let itemRes = itemResource.getItemByKey(itemId);
            peijianItem0.SetIcon(itemRes.icon + "_2");
            peijianItem1.SetIcon(itemRes.icon);
            peijianItem2.SetIcon(itemRes.icon);
            peijianItem0.SetLevel();
            peijianItem1.SetLevel(true, 1);
            peijianItem2.SetLevel(true, 2);
            peijianItem0.SetAddition();
            peijianItem1.SetAddition(1, BuildModel.Inst.addonItem);
            peijianItem2.SetAddition(1, BuildModel.Inst.addonItem * 2);
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
        }
        RemoveListener() {
            super.RemoveListener();
        }
        onResize() {
            super.onResize();
        }
        Reset() {
            super.Reset();
            this.buildVo = null;
        }
    }

    class UICatServiceItem extends fgui.GComponent {
        constructor() {
            super();
            this.buildVo = null;
        }
        constructFromXML(xml) {
            super.constructFromXML(xml);
            FairyUtils.setVar(this, this);
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
            this.btn_tips.onClick(this, this.OnBtnTipsClick);
            GameDispatcher.Inst.off(UIGameEvent.BUILD_UPDATE_SERVER, this, this.UpdateBuildVO);
            GameDispatcher.Inst.on(UIGameEvent.BUILD_UPDATE_SERVER, this, this.UpdateBuildVO);
        }
        OnBtnTipsClick() {
            UISystem.Inst.CreateWindowView(UIExplainView, [1, this.buildVo]);
        }
        UpdateBuildVO(buildVo) {
            if (!buildVo)
                return;
            let isUnlock = LandModel.Inst.IsUnlockServer();
            if (isUnlock) {
                this.c1.selectedIndex = 1;
                this.buildVo = buildVo;
                let catIdArr = buildVo.catServerArr;
                let serCatArr = [this.btn_head0, this.btn_head1, this.btn_head2, this.btn_head3];
                let catCount = serCatArr.length;
                for (let i = 0; i < catCount; i++) {
                    let serCat = serCatArr[i];
                    if (i <= catIdArr.length) {
                        serCat.UpdateCatVO(buildVo, i);
                    }
                    else {
                        serCat.UpdateCatVO(buildVo);
                    }
                }
                this.c2.selectedIndex = catIdArr.length;
            }
            else {
                this.c1.selectedIndex = 0;
            }
        }
    }

    class UISheshiView extends fgui.GComponent {
        constructor() {
            super();
            this.txt_des = null;
            this.txt_lv = null;
            this.txt_addRateAll = null;
            this.txt_syCount = null;
            this.txt_ssCount = null;
            this.txt_zwCount = null;
            this.txt_addRate = null;
            this.img_1 = null;
            this.img_2 = null;
            this.img_rategold = null;
            this.img_ratedi = null;
            this.sp_exp = null;
            this.btn_peijian = null;
            this.sp_buildType = null;
            this.sp_lvup = null;
            this.sp_zuowei = null;
            this.sp_type1 = null;
            this.sp_peijian1 = null;
            this.sp_peijian2 = null;
            this.btn_dingzhi = null;
            this.sp_serverItem = null;
            this.buildVo = null;
        }
        constructFromXML(xml) {
            super.constructFromXML(xml);
            FairyUtils.setVar(this, this);
            this.btn_peijian.onClick(this, this.OnBtnPeijianClick, [0]);
        }
        InitUI() {
        }
        OnBtnPeijianClick(id) {
            UISystem.Inst.CreateWindowView(UIExplainView, [id, this.buildVo]);
        }
        UpdateBuildVO(buildVO) {
            if (!buildVO)
                return;
            this.buildVo = buildVO;
            this.sp_lvup.UpdateBuildVO(buildVO);
            this.sp_serverItem.UpdateBuildVO(buildVO);
            if (buildVO.IsShowZuowei()) {
                this.sp_zuowei.visible = true;
                this.sp_zuowei.UpdateBuildVO(buildVO);
            }
            else
                this.sp_zuowei.visible = false;
            this.img_rategold.visible = false;
            this.txt_addRateAll.visible = false;
            this.img_2.visible = false;
            this.sp_exp.visible = false;
            this.img_ratedi.visible = false;
            this.txt_des.text = buildVO.buildRes.desc;
            this.txt_lv.text = buildVO.level + "级";
            this.txt_ssCount.text = buildVO.GetCurLvScore() + "";
            this.txt_syCount.text = MathUtil.goldFormat(buildVO.GetAllAddGold()) + "/" + buildVO.GetTime() + "秒";
            this.txt_addRate.text = "喵星人金币嘉奖" + Math.floor(CatModel.Inst.GetAllRate() * 0.01) + "%";
            this.txt_zwCount.text = buildVO.zwCount + "";
            let upRate = buildVO.GetNextUpRate();
            if (upRate > 0) {
                this.img_rategold.visible = true;
                this.txt_addRateAll.visible = true;
                this.img_ratedi.visible = true;
                this.txt_addRateAll.text = "+" + Math.floor(upRate / 100) + "%";
            }
            let needmaxLv = buildVO.GetNeedUpMaxLv();
            if (needmaxLv) {
                this.sp_exp.visible = true;
                this.sp_exp.min = 0;
                this.sp_exp.max = buildVO.GetNeedUpMaxLv();
                this.sp_exp.tweenValue(buildVO.GetCurUpUnlockLv(), 0.2);
            }
            let nextBuildUpRes = buildVO.GetNextStage();
            if (nextBuildUpRes) {
                this.img_2.visible = true;
                let url = UrlUtils.GetIconBuildUrl(nextBuildUpRes.icon);
                PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                    this.img_2.url = url;
                }));
            }
            let url = UrlUtils.GetIconBuildUrl(buildVO.buildRes.icon);
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                if (this.img_1) {
                    this.img_1.url = url;
                }
            }));
            this.sp_type1.SetIndex(buildVO.buildRes.type);
            this.sp_peijian1.SetItemId(buildVO.buildRes.itemId1);
            this.sp_peijian2.SetItemId(buildVO.buildRes.itemId2);
            let ctr = this.sp_buildType.getController("ctr");
            if (ctr)
                ctr.selectedIndex = buildVO.buildRes.type;
        }
    }

    class UIBuildView extends UIBaseWindow {
        constructor() {
            super("build", "UIBuildView");
            this.btn_left = null;
            this.btn_right = null;
            this.txt_name = null;
            this.txt_name2 = null;
            this.sp_sheshi = null;
            this.sp_maowu = null;
            this.sp_matou = null;
            this.sp_banjia = null;
            this.com_cameraRotate = null;
            this.ctr = null;
            this.buildVO = null;
            this.btn_close2 = null;
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.isEffect = true;
        }
        registerClasses() {
            super.registerClasses();
            fgui.UIObjectFactory.setPackageItemExtension(fgui.UIPackage.getItemURL("build", "UISheshiView"), UISheshiView);
            fgui.UIObjectFactory.setPackageItemExtension(fgui.UIPackage.getItemURL("build", "UIBuildLvUpItem"), UIBuildLvUpItem);
            fgui.UIObjectFactory.setPackageItemExtension(fgui.UIPackage.getItemURL("build", "UIBuildZuoweiUpItem"), UIBuildZuoweiUpItem);
            fgui.UIObjectFactory.setPackageItemExtension(fgui.UIPackage.getItemURL("build", "UIBuildPeijianItem"), UIBuildPeijianItem);
            fgui.UIObjectFactory.setPackageItemExtension(fgui.UIPackage.getItemURL("build", "UIBuildMaowuView"), UIBuildMaowuView);
            fgui.UIObjectFactory.setPackageItemExtension(fgui.UIPackage.getItemURL("build", "UIBuildMaowuLvUpItem"), UIBuildMaowuLvUpItem);
            fgui.UIObjectFactory.setPackageItemExtension(fgui.UIPackage.getItemURL("build", "UIBuildMatouView"), UIBuildMatouView);
            fgui.UIObjectFactory.setPackageItemExtension(fgui.UIPackage.getItemURL("build", "UIBuildMatouItem"), UIBuildMatouItem);
            fgui.UIObjectFactory.setPackageItemExtension(fgui.UIPackage.getItemURL("build", "UICatServiceItem"), UICatServiceItem);
            fgui.UIObjectFactory.setPackageItemExtension(fgui.UIPackage.getItemURL("build", "ServerCatItem"), ServerCatItem);
            fgui.UIObjectFactory.setPackageItemExtension(fgui.UIPackage.getItemURL("build", "UIBuildMoveHouseView"), UIBuildMoveHouseView);
            fgui.UIObjectFactory.setPackageItemExtension(fgui.UIPackage.getItemURL("build", "UIBuildBanjiaLvUpItem"), UIBuildBanjiaLvUpItem);
            fgui.UIObjectFactory.setPackageItemExtension(fgui.UIPackage.getItemURL("build", "BuildMaowuUpStageItem"), BuildMaowuUpStageItem);
            PanelRegister.registerClass("build", "btn_lv", VideoButton);
        }
        InitUI() {
            super.InitUI();
            this.ctr = this._view.getController("ctr");
        }
        AddListener() {
            super.AddListener();
            GameDispatcher.Inst.on(UIGameEvent.UPDATE_CURRENCY_DATA, this, this.UpdateCurrency);
            GameDispatcher.Inst.on(UIGameEvent.MATOU_JIAN_CD_TIME, this, this.onMatouJianCdTime);
            GameDispatcher.Inst.on(UIGameEvent.ONMAOWU_UPLV, this, this.OnMaowuUpdate);
            GameDispatcher.Inst.on(UIGameEvent.ONMAOWU_UPSTAGE, this, this.OnMaowuUpdate);
        }
        RemoveListener() {
            super.RemoveListener();
            GameDispatcher.Inst.off(UIGameEvent.UPDATE_CURRENCY_DATA, this, this.UpdateCurrency);
            GameDispatcher.Inst.off(UIGameEvent.MATOU_JIAN_CD_TIME, this, this.onMatouJianCdTime);
            GameDispatcher.Inst.off(UIGameEvent.ONMAOWU_UPLV, this, this.OnMaowuUpdate);
            GameDispatcher.Inst.off(UIGameEvent.ONMAOWU_UPSTAGE, this, this.OnMaowuUpdate);
        }
        InitData(param) {
            super.InitData(param);
            TComponent.CheckComponent(this._view, "InitData");
            if (param instanceof BuildVO) {
                this.UpdateBuildVO(param);
                SceneMgr.Inst.GetCameraBuildScript().SelectedBuild(param);
            }
            if (GuideModel.Inst.guideVO != null) {
                this.btn_right.visible = this.btn_left.visible = false;
            }
            else {
                this.btn_right.visible = this.btn_left.visible = true;
            }
            this.hideBanner();
        }
        OnMaowuUpdate() {
            this.sp_maowu.UpdateBuildVO(BuildModel.Inst.maowuBuildVO);
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_close2) {
                this.onBtnCloseClick();
            }
            else if (btn == this.btn_left) {
                this.onBtnLeftClick();
            }
            else if (btn == this.btn_right) {
                this.onBtnRightClick();
            }
        }
        onBtnLeftClick() {
            let buildVO = BuildModel.Inst.GetOnBuildVO(this.buildVO);
            SceneMgr.Inst.GetCameraBuildScript().SelectedBuild(buildVO, false);
            this.UpdateBuildVO(buildVO);
        }
        onBtnRightClick() {
            let buildVO = BuildModel.Inst.GetNextBuildVO(this.buildVO);
            SceneMgr.Inst.GetCameraBuildScript().SelectedBuild(buildVO, false);
            this.UpdateBuildVO(buildVO);
        }
        UpdateBuildVO(buildVO, isInit = true) {
            this.buildVO = buildVO;
            this.ctr.selectedIndex = buildVO.buildRes.type;
            if (buildVO.buildRes.type == BuildVO.TYPE_MAOWO) {
                this.sp_maowu.UpdateBuildVO(buildVO);
            }
            else if (buildVO.buildRes.type == BuildVO.TYPE_MATOU) {
                if (isInit)
                    this.sp_matou.UpdateBuildVO(buildVO);
            }
            else if (buildVO.buildRes.type == BuildVO.TYPE_BANJIASHESHI) {
                this.sp_banjia.UpdateBuildVO(buildVO);
            }
            else {
                this.sp_sheshi.UpdateBuildVO(buildVO);
            }
            this.txt_name.text = buildVO.buildRes.name;
            this.txt_name2.text = buildVO.buildRes.name;
        }
        GetBuildVO() {
            return this.buildVO;
        }
        UpdateCurrency() {
            this.UpdateBuildVO(this.buildVO, false);
        }
        onMatouJianCdTime() {
            if (this.buildVO.buildRes.type == BuildVO.TYPE_MATOU) {
                this.sp_matou.UpdateCdTime();
            }
        }
        onResize() {
            super.onResize();
        }
        onBtnCloseClick() {
            if (GuideModel.Inst.isOpenGuide && GuideModel.Inst.guideVO &&
                (GuideModel.Inst.guideVO.guideRes.page == 6)) {
                let panel = GuideModel.Inst.guideVO.guideRes.panel;
                if (panel && panel[panel.length - 1] == "btn_close") {
                }
                else {
                    return;
                }
            }
            super.onBtnCloseClick();
            SceneMgr.Inst.GetCameraBuildScript().OverBuildSelected();
            GameDispatcher.Inst.event(UIGameEvent.ON_HIDE, this.panelName);
        }
        Reset() {
            super.Reset();
            this.sp_matou.Reset();
            this.buildVO = null;
            GameDispatcher.Inst.event(UIGameEvent.HIDE_BUILD_PROBAR);
        }
    }

    class UICatView extends UIBaseWindow {
        constructor() {
            super("cat", "UICatView");
            this.btn_queding = null;
            this.eglist_cat = null;
            this.cats = null;
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = true;
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("cat", "UICatHeadItem", UICatHeadItem);
        }
        InitUI() {
            super.InitUI();
            this.eglist_cat.setVirtual();
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.UPDATE_CURRENCY_DATA, this.UpdateCurrency, this);
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onTabClick, this, this.tab);
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onClickCatItem, this, this.eglist_cat);
            this.AddGameListener(UIGameEvent.REMOVE_CAT, this.onRemoveCat, this);
            this.AddGameListener(UIGameEvent.ON_SHOW, this.onShow, this);
            this.AddGameListener(UIGameEvent.ON_HIDE, this.onHide, this);
        }
        InitData(param) {
            super.InitData(param);
            this.tab.selectedIndex = 0;
            this.onChanged();
            if (PlatformMgr.Inst.isWX) {
                if (AdModel.Inst.wxIsShowMuiltGrid) {
                    this.hideBanner();
                    this.showMgridAd();
                }
                else {
                    this.showBanner();
                }
            }
            else if (Laya.Browser.onHWMiniGame) {
                this.hideBanner();
                PlatformMgr.Inst.showAd(AdType.ad_3);
            }
            else if (Laya.Browser.onQQMiniGame) {
                this.hideBanner();
                PlatformMgr.Inst.showAd(AdType.ad_3);
            }
            else if (PlatformMgr.Inst.isMZ) {
                this.hideBanner();
                PlatformMgr.Inst.showAd(AdType.ad_3);
            }
            else if ((Laya.Browser.onVVMiniGame || Laya.Browser.onKGMiniGame) && GuideModel.Inst.guideVO == null) {
                this.hideBanner();
                AdCtrl.Inst.DelayShowInsertAd();
            }
            else if (Laya.Browser.onQGMiniGame) {
                this.hideBanner();
                AdCtrl.Inst.ShowNatiEmbed(this);
            }
            else {
                this.showBanner();
            }
        }
        showMgridAd() {
            this.ad_mgrid.InitData({ "adId": AdType.ad_28 });
        }
        onShow(panelName) {
            if (panelName == "UICatInviteAlert") {
                if (AdModel.Inst.wxIsShowMuiltGrid && PlatformMgr.Inst.isWX) {
                    this.ad_mgrid.Reset();
                }
                else {
                    this.hideBanner();
                }
            }
        }
        onHide(panelName) {
            if (panelName == "UICatInviteAlert") {
                if (AdModel.Inst.wxIsShowMuiltGrid && PlatformMgr.Inst.isWX) {
                    this.hideBanner();
                    this.showMgridAd();
                }
                else {
                    this.showBanner();
                }
            }
        }
        onTabClick() {
            this.onChanged();
        }
        onRemoveCat(catVo) {
            if (catVo && catVo.IsVisitor) {
                if (this.tab.selectedIndex == 0) {
                    this.onChanged();
                }
            }
        }
        onChanged() {
            if (this.tab.selectedIndex == 0) {
                this.cats = CatModel.Inst.GetAllCatVOs(3);
                this.img_red.visible = CatModel.Inst.CheckCanInvite();
            }
            else {
                this.cats = CatModel.Inst.GetAllCatVOs();
                this.img_red.visible = false;
            }
            this.eglist_cat.Data = this.cats;
            this.eglist_cat.scrollToTop();
            let item;
            for (let i = 0; i < this.eglist_cat.numChildren; i++) {
                item = this.eglist_cat.getChildAt(i);
                if (item && (item.CanInvite() || item.CanCheckCondition())) {
                    item.showBlack(this.tab.selectedIndex == 0);
                }
            }
        }
        onClickCatItem(item) {
            if (item.catVO == null)
                return;
            if (item.catVO.IsHas() && !item.catVO.IsVisitor) {
                UISystem.Inst.CreateWindowView(UICatDetailsView, item.catVO);
            }
            else if (item.CanInvite()) {
                UISystem.Inst.CreateWindowView(UICatInviteAlert, item.catVO);
            }
            else {
                if (item.catVO.IsVisitor) {
                    this.close();
                    SceneMgr.Inst.GetCameraBuildScript().SelectedCat(item.catVO);
                }
                UISystem.Inst.CreateWindowView(UICatInviteAlert, item.catVO);
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_queding) {
                this.close();
            }
        }
        UpdateCurrency() {
        }
        onResize() {
            super.onResize();
        }
        Reset() {
            super.Reset();
            this.cats = null;
        }
    }

    class UICatInviteItem extends TListItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            if (data instanceof ConditionVO) {
                this.cb.selected = data.IsFull;
                this.probar.max = data.needCount;
                this.probar.value = data.count;
                PanelRegister.loadUrl(data.IconUrl, Laya.Handler.create(this, (url) => {
                    this.img_icon.url = data.IconUrl;
                }));
                this.txt_desc.text = data.Desc;
            }
            this.probar.onClick(this, this.onClickProbar);
            this.img_icon.onClick(this, this.onClickProbar);
        }
        onClickProbar() {
            if (this.Data instanceof ConditionVO && !this.cb.selected) {
                if (this.Data.isRoom) {
                    if (BuildModel.Inst.maowuBuildVO && BuildModel.Inst.maowuBuildVO.IsUnLock()) {
                        UISystem.Inst.CreateWindowView(UIBuildView, BuildModel.Inst.maowuBuildVO);
                        GameDispatcher.Inst.event(UIGameEvent.CLICK_BUILD, BuildModel.Inst.maowuBuildVO);
                        UISystem.Inst.RemoveWindowClass(UICatView);
                        UISystem.Inst.RemoveWindowClass(UICatInviteAlert);
                    }
                }
                else if (this.Data.isBuildLevel) {
                    let buildVo = BuildModel.Inst.GetBuildVOBy(this.Data.tempId);
                    if (buildVo && !this.Data.IsFull && buildVo.IsUnLock()) {
                        UISystem.Inst.CreateWindowView(UIBuildView, buildVo);
                        GameDispatcher.Inst.event(UIGameEvent.CLICK_BUILD, buildVo);
                        UISystem.Inst.RemoveWindowClass(UICatView);
                        UISystem.Inst.RemoveWindowClass(UICatInviteAlert);
                    }
                }
            }
        }
        Reset() {
            super.Reset();
            this.probar.offClick(this, this.onClickProbar);
            this.img_icon.offClick(this, this.onClickProbar);
        }
    }

    class UICatInviteAlert extends UIBaseWindow {
        constructor() {
            super("cat", "UICatInviteAlert");
            this.isMainEnter = false;
            this.touchId = NaN;
            this.downP = new Laya.Point();
            this.moveP = new Laya.Point();
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("cat", "InviteView", InviteView);
            PanelRegister.registerClass("cat", "CatTipComp", CatTipComp);
            PanelRegister.registerClass("cat", "UICatInviteItem", UICatInviteItem);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowMaskBg = false;
            this.viewStruct.isEffect = false;
        }
        InitUI() {
            super.InitUI();
            this.group_main = this._view.getChild("group_main").asGroup;
            this.c1 = this._view.getController("c1");
            this.c2 = this._view.getController("c2");
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.CAT_STATE_STROKE_CHANGE, this.onCatStrokeChange, this);
            this.AddGameListener(UIGameEvent.CAT_MATOUDOWN, this.onCatMaTouDown, this);
        }
        RemoveListener() {
            for (let btn of this._btns) {
                btn.offClick(this, this.onBtnClick);
            }
        }
        InitData(data) {
            let isMainEnter = false;
            if (data instanceof Array) {
                isMainEnter = data[1];
                data = data[0];
            }
            this.isMainEnter = isMainEnter;
            super.InitData(data);
            this.visible = this.touchable = true;
            this.hideBanner();
            if (data instanceof CatVO) {
                this.txt_name.text = (data.IsVisitor ? "游客" : "") + data.catName;
                PanelRegister.loadUrl(data.HeadIconUrl, Laya.Handler.create(this, (url) => {
                    this.img_icon.icon = url;
                }));
                this.inviteView.InitData(data);
                this.btn_invite.enabled = data.CheckItemCount() && (!data.NeedOtherConndition() || data.CheckOtherCondition());
                this.btn_left.visible = this.btn_right.visible = data.IsVisitor;
                if (data.IsVisitor || data.IsGuide) {
                    SceneMgr.Inst.GetCameraBuildScript().SelectedCat(data);
                }
                let catVo = CatModel.Inst.GetCatVOById(data.catRes.id);
                if (catVo != null) {
                    if (catVo.IsGuide && GuideModel.Inst.guideVO && GuideModel.Inst.guideVO.page < 5) {
                        this.c1.selectedIndex = 2;
                    }
                    else if (catVo.IsHas()) {
                        this.c1.selectedIndex = 1;
                    }
                    else {
                        this.c1.selectedIndex = 0;
                    }
                }
                else {
                    this.c1.selectedIndex = 0;
                }
                if (this.c1.selectedIndex == 2) {
                    this.tipComp.InitData({ "type": 4 });
                }
                else if (GuideModel.Inst.guideVO && GuideModel.Inst.guideVO.page == 1) {
                    this.tipComp.InitData({ "type": 9 });
                }
                else {
                    this.tipComp.InitData({ "type": 5 });
                }
                this.onCatStrokeChange(data);
                if (UISystem.Inst.HasView(UICatView)) {
                    this.cameraScript.touchable = false;
                    if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                        let handler = Laya.Handler.create(this, this.showBanner);
                        let hideHandler = Laya.Handler.create(this, this.hideBanner);
                        AdCtrl.Inst.DelayShowCatInviteBanner(handler, hideHandler);
                    }
                }
                else {
                    this.cameraScript.touchable = true;
                }
                Laya.stage.off(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
                if (data.isForciblylumao && isMainEnter) {
                    CatCtrl.Inst.ClickCat(data);
                }
            }
            this.group_main.y = Laya.stage.height - this.group_main.height;
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            let catVo = null;
            if (btn == this.btn_catInfo) {
                if (GuideModel.Inst.guideVO != null) {
                    return;
                }
                UISystem.Inst.CreateWindowView(UICatInfoView, this.Data);
            }
            else if (btn == this.btn_invite) {
                if (!this.catVo.CheckMaowuCount()) {
                    I18nUtils.ShowI18nById("maowu_no_new");
                    return;
                }
                SceneMgr.Inst.GetCameraBuildScript().ClearSelectedCat();
                SceneMgr.Inst.GetCameraBuildScript().ResetCameraParent();
                CatCtrl.Inst.InviteCat(this.catVo);
                this.close();
                UISystem.Inst.RemoveWindowClass(UICatView);
            }
            else if (btn == this.btn_left) {
                catVo = CatModel.Inst.getPrevCatVo();
                if (catVo != this.catVo) {
                    CatCtrl.Inst.StopLumao(this.catVo);
                    if (!catVo.IsHas()) {
                        this.InitData(catVo);
                    }
                    else {
                        this.close();
                        UISystem.Inst.CreateWindowView(UICatDetailsView, catVo);
                    }
                }
            }
            else if (btn == this.btn_right) {
                catVo = CatModel.Inst.getNextCatVo();
                if (catVo != this.catVo) {
                    CatCtrl.Inst.StopLumao(this.catVo);
                    if (!catVo.IsHas()) {
                        this.InitData(catVo);
                    }
                    else {
                        this.close();
                        UISystem.Inst.CreateWindowView(UICatDetailsView, catVo);
                    }
                }
            }
        }
        onCatMaTouDown(catVo) {
            if (catVo instanceof CatVO) {
                if (catVo.IsVisitor) {
                    this.close();
                }
            }
        }
        onCatStrokeChange(catVO, param = 0) {
            if (catVO == null || catVO.objId != this.catVo.objId) {
                return;
            }
            this.cameraScript.touchable = true;
            this.img_hand.visible = false;
            this.stopStoke();
            this.c2.selectedIndex = catVO.catStrokeVo.state;
            if (catVO.catStrokeVo.IsRefuse()) {
                if (param >= 3) {
                    this.close();
                    SceneMgr.Inst.GetCameraBuildScript().OverSelected();
                    return;
                }
            }
            else if (this.catVo.catStrokeVo.IsStartStroke()) {
                this.cameraScript.touchable = false;
                this.pro_mo.visible = true;
            }
            else if (this.catVo.catStrokeVo.IsAnger()) {
                this.cameraScript.touchable = false;
            }
            else if (catVO.catStrokeVo.IsReady()) {
                this.tweenDown();
                Laya.timer.once(800, this, () => {
                    let svec = this.cameraScript.GetCatScreenPos();
                    if (svec) {
                        this.img_hand.setXY(svec.x + this.img_hand.width * 0.5, svec.y + this.img_hand.height * 0.5 - 100);
                        this.img_hand.visible = true;
                    }
                });
            }
            else if (catVO.catStrokeVo.IsStroking()) {
                this.tweenDown();
                this.img_hand.visible = false;
                this.img_hand2.visible = false;
                if (param == 1 && this.isMainEnter) {
                    this.img_kuang.visible = false;
                    Laya.timer.once(1000, this, this.startStroke, [param]);
                }
                else {
                    this.startStroke(param);
                }
                this.cameraScript.touchable = false;
            }
            else {
                this.tweenUp();
            }
            this.updateCatState();
        }
        updateCatState() {
            let stateData = [];
            if (this.catVo.catStrokeVo.IsStroked()) {
                stateData.push(this.catVo);
            }
            stateData.push(this.catVo.catStrokeVo);
        }
        readyToStroke() {
            this.img_hand.visible = false;
        }
        startStroke(part) {
            this.img_kuang.visible = true;
            this.pro_mo.visible = true;
            this.img_hand2.visible = false;
            this.pro_mo.value = 0;
            let svec = this.cameraScript.GetCatScreenPos();
            if (svec) {
                let x = svec.x;
                let y = svec.y;
                if (part == 1) {
                    y = y - 200;
                }
                else if (part == 2) {
                    y = y - 50;
                }
                else if (part == 3) {
                    x = x + 70;
                    y = y - 50;
                }
                this.img_kuang.setXY(x, y);
                this.showTip("摸摸喵星人吧。", false);
                Laya.stage.on(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            }
        }
        onMouseDown(e) {
            this.hideTip();
            if (isNaN(this.touchId)) {
                this.touchId = e.touchId;
                this.downP.setTo(Laya.stage.mouseX, Laya.stage.mouseY);
                let circle = this.img_kuang.width * 0.5;
                GameDispatcher.Inst.event(UIGameEvent.CAT_STROKING_DOWN_OR_UP, true);
                this.img_hand2.setXY(this.downP.x, this.downP.y);
                this.img_hand2.visible = true;
                Laya.stage.on(Laya.Event.MOUSE_MOVE, this, this.onMouseMove);
                Laya.stage.on(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            }
        }
        onMouseMove(e) {
            if (this.touchId == e.touchId) {
                this.moveP.setTo(Laya.stage.mouseX, Laya.stage.mouseY);
                this.img_hand2.setXY(this.moveP.x, this.moveP.y);
                this.pro_mo.tweenValue(this.pro_mo.value + 0.5, 0.3);
                if (this.pro_mo.value >= this.pro_mo.max) {
                    this.img_kuang.visible = false;
                    this.pro_mo.visible = false;
                    this.onMouseUp(null);
                    CatCtrl.Inst.OverLumao(this.catVo);
                    this.tweenUp();
                }
            }
        }
        onMouseUp(e) {
            if (this.pro_mo.visible) {
                this.showTip("摸摸喵星人吧。", false);
            }
            this.img_hand2.visible = false;
            Laya.stage.off(Laya.Event.MOUSE_MOVE, this, this.onMouseMove);
            Laya.stage.off(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            this.touchId = NaN;
            GameDispatcher.Inst.event(UIGameEvent.CAT_STROKING_DOWN_OR_UP, false);
        }
        showTip(txt, isTweenHide = false) {
            this.hideTip();
            this.txt_tip.text = txt;
            this.group_tip.visible = true;
            fgui.GTween.to(0, 1, 0.3).setTarget(this.group_tip, "alpha").setEase(fgui.EaseType.Linear);
            if (isTweenHide) {
                fgui.GTween.to(this.group_tip.alpha, 0, 0.3).setTarget(this.group_tip, "alpha").setEase(fgui.EaseType.Linear).setDelay(1)
                    .onComplete(() => {
                    this.group_tip.visible = false;
                }, this);
            }
        }
        hideTip() {
            this.txt_tip.text = "";
            this.group_tip.visible = false;
            fgui.GTween.kill(this.group_tip, true);
        }
        stopStoke() {
            Laya.stage.off(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
        }
        tweenDown() {
            let toY = Laya.stage.height - this.group_main.height * 0.8;
            fgui.GTween.to(this.group_main.y, toY, 0.6).setTarget(this.group_main).setEase(fgui.EaseType.Linear)
                .onUpdate((tw) => {
                this.group_main.y = tw.value.x;
            }, this);
        }
        tweenUp() {
            this.hideTip();
            let toY = Laya.stage.height - this.group_main.height;
            fgui.GTween.to(this.group_main.y, toY, 0.6).setTarget(this.group_main).setEase(fgui.EaseType.Linear)
                .onUpdate((tw) => {
                this.group_main.y = tw.value.x;
            }, this);
        }
        get catVo() {
            return this.Data;
        }
        onBtnCloseClick() {
            if (GuideModel.Inst.isOpenGuide && GuideModel.Inst.guideVO &&
                (GuideModel.Inst.guideVO.guideRes.page == 1 ||
                    GuideModel.Inst.guideVO.guideRes.page == 3 ||
                    GuideModel.Inst.guideVO.guideRes.page == 4 ||
                    GuideModel.Inst.guideVO.guideRes.page == 7 ||
                    GuideModel.Inst.guideVO.guideRes.page == 14)) {
                let panel = GuideModel.Inst.guideVO.guideRes.panel;
                let panelName = panel[0];
                if (panelName == this.panelName) {
                    if (panel && panel[panel.length - 1] == "btn_close") {
                    }
                    else {
                        return;
                    }
                }
                else if (GuideModel.Inst.guideVO && GuideModel.Inst.guideVO.page == 4 && GuideModel.Inst.guideVO.isLessThan(4, 3)) {
                    return;
                }
                else if (GuideModel.Inst.guideVO.guideRes.page == 1 ||
                    GuideModel.Inst.guideVO.guideRes.page == 3 ||
                    GuideModel.Inst.guideVO.guideRes.page == 7) {
                    return;
                }
            }
            SceneMgr.Inst.GetCameraBuildScript().OverSelected();
            super.onBtnCloseClick();
        }
        Reset() {
            if (this.catVo.IsVisitor || this.catVo.IsGuide) {
                CatCtrl.Inst.StopLumao(this.catVo);
            }
            super.Reset();
            this.cameraScript.touchable = false;
            this.onMouseUp(null);
            this.hideTip();
        }
    }

    class catloveResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint8();
            this.quality = byteArray.readUint8();
            this.lvl = byteArray.readUint8();
            this.lvlName = byteArray.readUTFString();
            this.lvlIcon = byteArray.readUTFString();
            this.needLvl = byteArray.readUint8();
            this.award = byteArray.readUint16();
            this.itemNum = byteArray.readUint8();
            this.addon = byteArray.readUint16();
        }
        static getItemByKey(id) {
            return catloveResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return catloveResource.itemsDic[key1 + "," + key2];
        }
    }
    catloveResource.items = new Array();
    catloveResource.itemsDic = new Dictionary();

    class UICatQinmiItem extends TListItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.btn_diamond.getController("c1");
        }
        AddListener() {
            super.AddListener();
            this.btn_diamond.onClick(this, this.onClickDiamond);
        }
        RemoveListener() {
            super.RemoveListener();
            this.btn_diamond.offClick(this, this.onClickDiamond);
        }
        InitData(data) {
            super.InitData(data);
            let lv2 = data.lv2;
            this.catVo = data.catVo;
            let res = catloveResource.getItemBy2Keys(this.catVo.catRes.quality, lv2);
            if (res) {
                this.txt_title.text = res.lvlName;
                this.img_icon.url = PanelRegister.getUrl("cat", "qinmi" + res.lvl);
                if (res.lvl == 1) {
                    this.txt_lvl.text = "Lv.1~19";
                }
                else if (res.lvl == 2) {
                    this.txt_lvl.text = "Lv.20~29";
                }
                else if (res.lvl == 3) {
                    this.txt_lvl.text = "Lv.30~39";
                }
                else if (res.lvl == 4) {
                    this.txt_lvl.text = "Lv.40~49";
                }
                else if (res.lvl == 5) {
                    this.txt_lvl.text = "Lv.50~";
                }
                this.btn_diamond.title = "x" + res.award;
                this.btn_diamond.visible = res.award > 0;
                if (this.catVo.hasAwardQinmi(lv2)) {
                    this.c1.selectedIndex = 2;
                    this.btn_diamond.touchable = false;
                }
                else if (this.catVo.canAwardQinmi(lv2)) {
                    this.c1.selectedIndex = 1;
                    this.btn_diamond.touchable = true;
                }
                else {
                    this.c1.selectedIndex = 0;
                    this.btn_diamond.touchable = false;
                }
            }
            this.res = res;
        }
        onClickDiamond() {
            if (this.catVo) {
                this.catVo.awardQinmiAward(this.Data.lv2);
            }
            AwardMgr.Inst.award({ "itemId": ItemModel.ITEM_GEM, "count": this.res.award });
            this.InitData(this.Data);
        }
        Reset() {
            super.Reset();
            this.res = null;
        }
    }

    class UICatQinmiView extends UIBaseWindow {
        constructor() {
            super("cat", "UICatQinmiView");
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("cat", "UICatQinmiItem", UICatQinmiItem);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            if (data instanceof CatVO) {
                let arr = [];
                for (let i = 1; i <= 5; i++) {
                    arr.push({ "lv2": i, "catVo": data });
                }
                this.eglist_items.Data = arr;
            }
            this.showBanner();
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
        }
        Reset() {
            super.Reset();
        }
    }

    class UICatDetailsView extends UIBaseWindow {
        constructor() {
            super("cat", "UICatDetailsView");
            this.isMainEnter = false;
            this.flyItemPos = new Laya.Vector3();
            this.touchId = NaN;
            this.downP = new Laya.Point();
            this.moveP = new Laya.Point();
            this.radius = 10;
            this.isInCircle = false;
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("cat", "CatTipComp", CatTipComp);
            PanelRegister.registerClass("cat", "UICatIntimacyCostItem", UICatIntimacyCostItem);
            PanelRegister.registerClass("cat", "UICatPostureItem", UICatPostureItem);
            PanelRegister.registerClass("cat", "UICatPostureItem2", UICatPostureItem2);
            PanelRegister.registerClass("cat", "UICatStateItem", UICatStateItem);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowMaskBg = false;
            this.viewStruct.isEffect = false;
        }
        InitUI() {
            super.InitUI();
            this.comp_cost1.isLock = false;
            this.c1 = this._view.getController("c1");
            this.c2 = this._view.getController("c2");
            let scaleY = 1;
            if (Laya.Browser.onQQMiniGame) {
                scaleY = PlatformModel.Inst.scaleY / Laya.stage.clientScaleY;
            }
            let rightUpRect = PlatformModel.Inst.rightUpRect;
            if (rightUpRect) {
                this.pro_mo.y = rightUpRect.y * scaleY + rightUpRect.height * scaleY + 150;
            }
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onPoseItemClick, this, this.eglist_posture);
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onPoseItemClick2, this, this.eglist_zishi);
            this.AddGameListener(UIGameEvent.CAT_STATE_STROKE_CHANGE, this.onCatStrokeChange, this);
            this.AddGameListener(UIGameEvent.CAT_UPDATE_QINMI, this.onCatQinmiUpdate, this);
            this.AddGameListener(UIGameEvent.CAT_CHANGE_NAME, this.onCatChangeName, this);
            this.AddGameListener(UIGameEvent.UNLOCK_POSE, this.onUnlockPose, this);
            this.AddGameListener(UIGameEvent.ADD_ITEM, this.onAddItem, this);
            this.AddGameListener(Laya.Event.MOUSE_DOWN, this.onStageMouseDown, this, Laya.stage);
            this.AddGameListener(Laya.Event.MOUSE_UP, this.onStageMouseUp, this, Laya.stage);
        }
        RemoveListener() {
            for (let btn of this._btns) {
                btn.offClick(this, this.onBtnClick);
            }
        }
        InitData(data) {
            let isMainEnter = false;
            if (data instanceof Array) {
                isMainEnter = data[1];
                data = data[0];
            }
            this.isMainEnter = isMainEnter;
            super.InitData(data);
            SceneMgr.Inst.GetCameraBuildScript().SelectedCat(data);
            if (data instanceof CatVO) {
                data.isMainViewSelect = this.isMainEnter;
                this.txt_name.text = data.catName || data.catRes.name;
                PanelRegister.loadUrl(data.HeadIconUrl, Laya.Handler.create(this, (url) => {
                    this.img_icon.icon = url;
                }));
                this.tipComp.InitData({ "type": 4 });
                this.img_char.url = PanelRegister.getUrl("cat", "character_" + data.catRes.character);
                this.onCatStrokeChange(data);
                this.UpdateCoinAdd();
                this.eglist_posture.Data = data.poseVoList;
                this.eglist_zishi.Data = data.getUnlockPoseVoList();
                this.UpdateQinmi();
                this.cameraScript.touchable = true;
                Laya.stage.off(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
                if (data.isForciblylumao && isMainEnter) {
                    CatCtrl.Inst.ClickCat(data);
                }
            }
            this.group_main.y = Laya.stage.height - this.group_main.height;
            this.hideBanner();
        }
        UpdateCoinAdd() {
            this.txt_coinPer.text = CatModel.getRate(this.catVo.totalGoldAddRate);
            this.comp_jinbi_qinmi.text = CatModel.getRate(this.catVo.loveGoldAddRate);
            this.comp_jinbi_clothes.text = CatModel.getRate(this.catVo.clothGoldAddRate);
        }
        onCatQinmiUpdate() {
            this.UpdateQinmi();
            this.UpdateCoinAdd();
        }
        onAddItem(itemVO) {
            if (itemVO && (itemVO.itemId == ItemModel.ITEM_CHURU || itemVO.itemId == ItemModel.ITEM_GOLD)) {
                this.onCatQinmiUpdate();
            }
        }
        UpdateQinmi() {
            let data = this.catVo;
            this.eglist_posture.Data = data.poseVoList;
            this.pro_qimi.max = data.getLoveItemLimit();
            this.pro_qimi.value = data.ownedChuruCount;
            let itemRes1 = itemResource.getItemByKey(ItemModel.ITEM_CHURU);
            let count1 = ItemModel.Inst.GetItemCount(ItemModel.ITEM_CHURU);
            this.comp_cost1.setItem(ItemModel.ITEM_CHURU, count1);
            let itemRes2 = itemResource.getItemByKey(data.catRes.item);
            let count2 = ItemModel.Inst.GetItemCount(data.catRes.item);
            this.comp_cost2.setItem(data.catRes.item, count2);
            this.group_qmTo.visible = data.canLevelLove;
            this.btn_gift.visible = this.group_qmTo.visible;
            this.btn_give.visible = !this.btn_gift.visible;
            this.btn_give.enabled = count1 > 0 && !this.catVo.isLevelLimit;
            this.btn_gift.enabled = count2 > 0 && !this.catVo.isLevelLimit;
            if (Laya.Browser.onTTMiniGame) {
                this.btn_give.isShowAd = false;
            }
            else {
                if (count1 <= 0) {
                    if (AdCtrl.Inst.giveChuruShowAd) {
                        this.btn_give.enabled = true;
                        this.btn_give.isShowAd = true;
                    }
                    else {
                        this.btn_give.isShowAd = false;
                    }
                }
                else {
                    this.btn_give.isShowAd = false;
                }
            }
            this.comp_cost1.isLock = this.group_qmTo.visible;
            this.comp_cost2.isLock = !this.comp_cost1.isLock;
            if (!this.group_qmTo.visible) {
                let url = ItemVO.GetItemUrl(itemRes1);
                PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                    this.pro_item.icon = url;
                }));
                this.pro_item.title = this.pro_qimi.value + "/" + this.pro_qimi.max;
            }
            else {
                this.pro_qimi.max = data.nextCatLoveRes.itemNum;
                this.pro_qimi.value = count2;
                let url = ItemVO.GetItemUrl(itemRes2);
                PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                    this.pro_item.icon = url;
                }));
                if (data.nextCatLoveRes) {
                    this.pro_item.visible = true;
                    this.pro_item.title = "" + data.nextCatLoveRes.itemNum;
                }
                else {
                    this.pro_item.visible = false;
                }
            }
            this.txt_qinmi_state.text = this.btn_qimi_from.text = data.getLoveLvDesc();
            this.txt_qinmi_icon.url = this.btn_qimi_from.icon = PanelRegister.getUrl("cat", data.catLoveRes.lvlIcon);
            this.txt_qinmi_lv.text = `等级${data.level}`;
            this.btn_qimiAdd.text = "0.5%";
            this.btn_qimiAdd2.text = CatModel.getRate(data.loveGoldAddRate);
            if (data.loveLv < 5 && data.nextCatLoveRes) {
                this.group_qmTo.visible = true;
                this.btn_qimi_to.text = data.getLoveLvDesc(data.nextLoveLv);
                this.btn_qimi_to.icon = PanelRegister.getUrl("cat", data.nextCatLoveRes.lvlIcon);
            }
            else {
                this.group_qmTo.visible = false;
            }
            if (this.catVo.isLevelLimit) {
                this.pro_qimi.value = 0;
                this.txt_level_limit.visible = true;
            }
            else {
                this.txt_level_limit.visible = false;
            }
            this.pro_item.visible = !this.txt_level_limit.visible;
        }
        onUnlockPose(posId, catId) {
            this.eglist_posture.Data = this.catVo.poseVoList;
            this.eglist_zishi.Data = this.catVo.getUnlockPoseVoList();
            UISystem.Inst.CreateWindowView(UIAwardView, catActResource.getItemByKey(posId));
        }
        UpdateCat() {
            this.InitData(this.catVo);
        }
        onPoseItemClick(item) {
            if (item) {
                if (item.poseVo.isUnlock)
                    return;
                if (item.isNeedGem) {
                    UISystem.Inst.CreateWindowView(UIAlertView, {
                        "cost": item.cost, "flag": UIAlertView.OK_AND_NO, "title": "确认购买姿势", "text": "是否支付钻石学习姿势?", "okHandler": Laya.Handler.create(this, () => {
                            this.unlockPose(item);
                        })
                    });
                }
                else if (item.isAd) {
                    item.onClickAd();
                }
                else if (item.isCatLevel) {
                    if (item.conditionVo) {
                        item.conditionVo.count = item.poseVo.parent.level;
                    }
                    if (item.canLock) {
                        this.unlockPose(item);
                    }
                }
            }
        }
        unlockPose(item) {
            this.catVo.unlockPose(item.poseVo.tempId);
            item.unlock();
            this.eglist_zishi.Data = this.catVo.getUnlockPoseVoList();
        }
        onPoseItemClick2(item) {
            if (item) {
                CatCtrl.Inst.ChangePose(this.catVo, item.poseVo.tempId);
                GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [TaskType.TASKTYPE_CATBAIZS, item.poseVo.tempId]);
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            let catVo = null;
            if (btn == this.btn_left) {
                catVo = CatModel.Inst.getPrevCatVo();
                if (catVo != this.catVo) {
                    CatCtrl.Inst.StopLumao(this.catVo);
                    if (catVo.IsHas()) {
                        this.InitData(catVo);
                    }
                    else {
                        this.close();
                        UISystem.Inst.CreateWindowView(UICatInviteAlert, catVo);
                    }
                }
            }
            else if (btn == this.btn_right) {
                catVo = CatModel.Inst.getNextCatVo();
                if (catVo != this.catVo) {
                    CatCtrl.Inst.StopLumao(this.catVo);
                    if (catVo.IsHas()) {
                        this.InitData(catVo);
                    }
                    else {
                        this.close();
                        UISystem.Inst.CreateWindowView(UICatInviteAlert, catVo);
                    }
                }
            }
            else if (btn == this.btn_give) {
                this.giveChuru();
            }
            else if (btn == this.btn_gift) {
                this.giveGood();
            }
            else if (btn == this.btn_catInfo) {
                UISystem.Inst.CreateWindowView(UICatInfoView, this.catVo);
            }
            else if (btn == this.btn_changeName) {
                UISystem.Inst.CreateWindowView(UIAlertView, {
                    "flag": UIAlertView.OK_AND_NO, "text": "想给喵星人换个名字吗?",
                    "okHandler": Laya.Handler.create(this, this.openChangeNameView)
                });
            }
            else if (btn == this.btn_help_qinmi) {
                UISystem.Inst.CreateWindowView(UICatQinmiView, this.catVo);
            }
            else if (btn == this.btn_clothes) {
                UISystem.Inst.CreateWindowView(UISkinView, this.catVo);
            }
        }
        onStageMouseDown(e) {
            if (GlobalConfig.isOpenAutoTouch) {
                Laya.timer.loop(GlobalConfig.autoTouchTimeGap, this, this.loopClick);
            }
        }
        onStageMouseUp(e) {
            Laya.timer.clear(this, this.loopClick);
        }
        loopClick() {
            if (this.btn_give.visible && this.btn_give.enabled && ObjectUtils.globalIsIn(this.btn_give)) {
                this.giveChuru();
            }
            else if (this.btn_gift.visible && this.btn_gift.enabled && ObjectUtils.globalIsIn(this.btn_gift)) {
                this.giveGood();
            }
        }
        giveChuru() {
            if (this.btn_give.isShowAd) {
                PlatformMgr.Inst.showAd(AdType.ad_26, Laya.Handler.create(this, this.onGiveAd));
                return;
            }
            let itemVO = ItemModel.Inst.GetItemVOById(ItemModel.ITEM_CHURU);
            if (itemVO && itemVO.count > 0) {
                let oldLevel = this.catVo.level;
                ItemCtrl.Inst.RemoveItemVO(ItemModel.ITEM_CHURU, 1);
                this.catVo.ownedChuruCount += 1;
                this.onCatQinmiUpdate();
                GameDispatcher.Inst.event(UIGameEvent.DAYTASK_COMP_PROP, TaskVO.SONGLI);
                GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [TaskType.TASKTYPE_GIVEQIULU, this.catVo.catRes.id]);
                let cat = EntityModel.Inst.GetCat(this.catVo.objId);
                if (!cat)
                    return;
                let camera = SceneMgr.Inst.GetCamera();
                cat && cat.transform.position.cloneTo(this.flyItemPos);
                this.flyItemPos.y += 2;
                let pos = MathUtil.GetPointFrom3DWorld(camera, this.flyItemPos);
                EffectMgr.Inst.FlyGold(1, pos, UIFlyCoinItem.TYPE_NORMAL, UIFlyCoinItem.ITEM_TYPE_CHURU);
                let headSp = cat && ObjectUtils.findChild(cat.owner, "Bip001 Head");
                headSp && headSp.transform.position.cloneTo(this.flyItemPos);
                if (oldLevel == this.catVo.level) {
                    this.flyItemPos.x += Math.random() * 0.5 - 0.25;
                    this.flyItemPos.y += 0.1;
                    this.flyItemPos.z += Math.random() * 0.5 - 0.25;
                    EffectMgr.Inst.playConfigEffect(EffectMgr.Eff_shengji_aixin, cat.owner, null, this.flyItemPos, null, null, 4000, true, true);
                }
                else {
                    EffectMgr.Inst.playConfigEffect(EffectMgr.Eff_shengji_jiantou, cat.owner, null, this.flyItemPos, null, null, 4000, true, false);
                }
                if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                    let p4 = AdModel.Inst.getPolicy("p4");
                    if (p4 == "yes") {
                        AdCtrl.Inst.addClickChurCount();
                        if (AdModel.Inst.clickChuruCount >= 5) {
                            this.showBanner();
                            AdCtrl.Inst.clearChuruCount();
                            AdModel.Inst.bannerChuruTime = Laya.timer.currTimer;
                            Laya.timer.once(2000, this, this.hideBanner);
                        }
                    }
                }
            }
        }
        onGiveAd(isSuccess) {
            if (isSuccess) {
                AwardMgr.Inst.award({ "itemId": ItemModel.ITEM_CHURU, "count": 40 });
            }
        }
        giveGood() {
            let nextCatLoveRes = this.catVo.nextCatLoveRes;
            let itemVO = ItemModel.Inst.GetItemVOById(this.catVo.catRes.item);
            if (itemVO) {
                if (itemVO.count >= nextCatLoveRes.itemNum) {
                    ItemCtrl.Inst.RemoveItemVO(this.catVo.catRes.item, nextCatLoveRes.itemNum);
                    this.catVo.loveLv++;
                    this.UpdateCat();
                }
                else {
                    let txt = `${itemVO.itemRes.name}道具不足。(${itemVO.count}/${nextCatLoveRes.itemNum})`;
                    this.showTip(txt, true);
                }
            }
        }
        openChangeNameView() {
            UISystem.Inst.CreateWindowView(UICatChangeNameView, this.catVo);
        }
        onCatChangeName(catVo) {
            if (this.catVo == catVo) {
                this.txt_name.text = catVo.catName;
            }
        }
        onCatStrokeChange(catVO, param = 0) {
            if (catVO == null || this.catVo.objId != catVO.objId) {
                return;
            }
            this.cameraScript.touchable = true;
            let prev = this.c2.selectedIndex;
            this.c2.selectedIndex = catVO.catStrokeVo.state;
            if (prev == 2) {
                if (GuideModel.Inst.funGuideVO && GuideModel.Inst.funGuideVO.isStep(22, 8)) {
                    GuideModel.Inst.completeFuncGuide(GuideModel.Inst.funGuideVO.tempId);
                }
            }
            this.askMc.visible = false;
            this.stopStoke();
            this.img_hand.visible = false;
            Logger.log(this, "====>onCatStrokeChange state: " + catVO.catStrokeVo.state);
            if (catVO.catStrokeVo.IsRefuse()) {
                this.tipComp.InitData({ "type": 7 });
                if (param >= 3) {
                    this.close();
                    SceneMgr.Inst.GetCameraBuildScript().OverSelected();
                    return;
                }
            }
            else if (catVO.catStrokeVo.IsReady()) {
                this.tweenDown();
                Laya.timer.once(800, this, this.showHand);
                Laya.timer.once(2800, this, this.hideAsk);
                this.tipComp.InitData({ "type": 6 });
            }
            else if (this.catVo.catStrokeVo.IsStartStroke()) {
                this.cameraScript.touchable = false;
                this.pro_mo.visible = true;
                this.tipComp.InitData({ "type": 2 });
            }
            else if (this.catVo.catStrokeVo.IsAnger()) {
                this.cameraScript.touchable = false;
                this.tipComp.InitData({ "type": 7 });
            }
            else if (catVO.catStrokeVo.IsStroking()) {
                this.tweenDown();
                this.img_hand.visible = false;
                this.img_hand2.visible = false;
                Laya.timer.clear(this, this.showHand);
                fgui.GTween.kill(this.askMc);
                if (param == 1 && this.isMainEnter) {
                    this.img_kuang.visible = false;
                    Laya.timer.once(1000, this, this.startStroke, [param]);
                }
                else {
                    this.startStroke(param);
                }
                this.cameraScript.touchable = false;
                this.tipComp.InitData({ "type": 2 });
            }
            else {
                this.tweenUp();
                this.tipComp.InitData({ "type": 4 });
            }
            this.updateCatState();
        }
        showHand() {
            this.img_hand.visible = true;
            this.askMc.visible = true;
            this.updateHandXY();
            fgui.GTween.to(0, 1, 0.3).setTarget(this.askMc, "alpha");
        }
        updateHandXY() {
            let svec = this.GetCatInScreePos();
            if (svec) {
                this.img_hand.setXY(svec.x + this.img_hand.width * 0.5, svec.y + this.img_hand.height * 0.5 - 100);
                if (this.askMc.visible) {
                    this.askMc.setXY(this.img_hand.x + 20, this.img_hand.y - 300);
                }
            }
        }
        hideAsk() {
            if (this.askMc.visible) {
                fgui.GTween.to(this.askMc.alpha, 0, 0.3).setTarget(this.askMc, "alpha").onComplete(() => {
                    this.askMc.visible = false;
                }, this);
            }
        }
        updateCatState() {
            let stateData = [];
            if (this.catVo.catStrokeVo.IsStroked()) {
                stateData.push(this.catVo);
            }
            stateData.push(this.catVo.catStrokeVo);
            this.eglist_state.Data = stateData;
        }
        readyToStroke() {
            this.img_hand.visible = false;
        }
        GetCatInScreePos() {
            let camera = SceneMgr.Inst.GetCamera();
            let cat = EntityModel.Inst.GetCat(this.catVo.objId);
            if (!cat)
                return;
            return MathUtil.GetPointFrom3DWorld(camera, cat.transform.position);
        }
        startStroke(part) {
            this.img_kuang.visible = true;
            this.pro_mo.visible = true;
            this.img_hand2.visible = false;
            this.pro_mo.value = 0;
            let svec = this.GetCatInScreePos();
            if (svec) {
                let x = svec.x;
                let y = svec.y;
                if (part == 1) {
                    y = y - 200;
                }
                else if (part == 2) {
                    y = y - 50;
                }
                else if (part == 3) {
                    x = x + 70;
                    y = y - 50;
                }
                this.img_kuang.setXY(x, y);
                this.showTip("摸摸喵星人吧。", false);
                Laya.stage.on(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            }
        }
        onMouseDown(e) {
            this.hideTip();
            if (isNaN(this.touchId)) {
                this.askMc.visible = false;
                this.touchId = e.touchId;
                this.downP.setTo(Laya.stage.mouseX, Laya.stage.mouseY);
                this.radius = this.img_kuang.width * 0.5;
                this.isInCircle = ObjectUtils.globalIsIn(this.img_kuang);
                GameDispatcher.Inst.event(UIGameEvent.CAT_STROKING_DOWN_OR_UP, true);
                this.img_hand2.setXY(this.downP.x, this.downP.y);
                this.img_hand2.visible = true;
                Laya.stage.on(Laya.Event.MOUSE_MOVE, this, this.onMouseMove);
                Laya.stage.on(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            }
        }
        onMouseMove(e) {
            if (this.touchId == e.touchId) {
                this.moveP.setTo(Laya.stage.mouseX, Laya.stage.mouseY);
                this.img_hand2.setXY(this.moveP.x, this.moveP.y);
                if (!this.isInCircle) {
                    return;
                }
                this.pro_mo.tweenValue(this.pro_mo.value + 1.2, 0.3);
                if (this.pro_mo.value >= this.pro_mo.max) {
                    this.img_kuang.visible = false;
                    this.pro_mo.visible = false;
                    this.onMouseUp(null);
                    Laya.stage.off(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
                    CatCtrl.Inst.OverLumao(this.catVo);
                    this.tweenUp();
                    GameDispatcher.Inst.event(UIGameEvent.DAYTASK_COMP_PROP, TaskVO.MOMAO);
                    GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, TaskType.TASKTYPE_MO_CAT);
                }
            }
        }
        onMouseUp(e) {
            if (this.pro_mo.visible) {
                this.showTip("摸摸喵星人吧。", false);
            }
            this.img_hand2.visible = false;
            Laya.stage.off(Laya.Event.MOUSE_MOVE, this, this.onMouseMove);
            Laya.stage.off(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            this.touchId = NaN;
            GameDispatcher.Inst.event(UIGameEvent.CAT_STROKING_DOWN_OR_UP, false);
        }
        showTip(txt, isTweenHide = false) {
            this.hideTip();
            this.txt_tip.text = txt;
            this.group_tip.visible = true;
            fgui.GTween.to(0, 1, 0.3).setTarget(this.group_tip, "alpha").setEase(fgui.EaseType.Linear);
            if (isTweenHide) {
                fgui.GTween.to(this.group_tip.alpha, 0, 0.3).setTarget(this.group_tip, "alpha").setEase(fgui.EaseType.Linear).setDelay(1)
                    .onComplete(() => {
                    this.group_tip.visible = false;
                }, this);
            }
        }
        hideTip() {
            this.txt_tip.text = "";
            this.group_tip.visible = false;
            fgui.GTween.kill(this.group_tip, true);
        }
        stopStoke() {
            Laya.stage.off(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
        }
        tweenDown() {
            let toY = Laya.stage.height - this.group_main.height * 0.7;
            fgui.GTween.to(this.group_main.y, toY, 0.6).setTarget(this.group_main).setEase(fgui.EaseType.Linear)
                .onUpdate((tw) => {
                this.group_main.y = tw.value.x;
            }, this);
        }
        tweenUp() {
            this.hideTip();
            let toY = Laya.stage.height - this.group_main.height;
            fgui.GTween.to(this.group_main.y, toY, 0.6).setTarget(this.group_main).setEase(fgui.EaseType.Linear)
                .onUpdate((tw) => {
                this.group_main.y = tw.value.x;
            }, this);
        }
        get catVo() {
            return this.Data;
        }
        onBtnCloseClick() {
            super.onBtnCloseClick();
            SceneMgr.Inst.GetCameraBuildScript().OverSelected();
        }
        onResize() {
            super.onResize();
        }
        Reset() {
            CatCtrl.Inst.StopLumao(this.catVo);
            super.Reset();
            this.stopStoke();
            this.visible = true;
            fgui.GTween.kill(this.group_main);
            Laya.timer.clearAll(this);
            this.pro_mo.value = 0;
            Laya.stage.off(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            this.onMouseUp(null);
            fgui.GTween.kill(this.askMc);
            Laya.timer.clear(this, this.hideBanner);
            Laya.timer.clear(this, this.showHand);
            Laya.timer.clear(this, this.hideAsk);
            Laya.timer.clear(this, this.startStroke);
            this.hideTip();
        }
    }

    class CVModel {
        constructor() {
            this.cvMap = {};
        }
        static get Inst() {
            return this.inst;
        }
        GetValue(key, type = CVModel.valType_string, split = ",") {
            if (this.cvMap[key] != null)
                return this.cvMap[key];
            let cv = ConfigValue.getItemByKey(key);
            let val = cv && cv.value;
            let retVal;
            if (val == null)
                return;
            if (type == CVModel.valType_number && typeof val === "string") {
                retVal = val.indexOf(".") == -1 ? parseInt(val) : parseFloat(val);
            }
            else if (type == CVModel.valType_array) {
                retVal = val.split(split);
            }
            else
                retVal = val;
            this.cvMap[key] = retVal;
            return retVal;
        }
    }
    CVModel.inst = new CVModel();
    CVModel.valType_string = "valType_string";
    CVModel.valType_number = "valType_number";
    CVModel.valType_array = "valType_array";
    CVModel.leaveTime = "leaveTime";
    CVModel.goldtime = "goldtime";
    CVModel.goldnumb = "goldnumb";
    CVModel.bActId = "bActId";
    CVModel.cActId = "cActId";

    class actResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint16();
            this.name = byteArray.readUTFString();
            this.group = byteArray.readUint16();
            this.part = byteArray.readUint8();
            this.loop = byteArray.readUint8();
        }
        static getItemByKey(id) {
            return actResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return actResource.itemsDic[key1 + "," + key2];
        }
    }
    actResource.items = new Array();
    actResource.itemsDic = new Dictionary();

    class UICatBefallView extends UIBaseWindow {
        constructor() {
            super("befallCat", "UICatBefallView");
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            this.catVo = CatModel.Inst.GetRandomBefallCatVO();
            CatModel.Inst.befallIntervalTime = 0;
            CatModel.Inst.befallCatCount++;
            this.spine_tips.play("miaoxingren");
            this.spine_get.play("lijihuoqu");
            let spineNameArr = ["mxrjl_1", "mxrjl_2", "mxrjl_3", "mxrjl_4", "mxrjl_5"];
            let randomName = spineNameArr[MathUtil.makeRandom(0, spineNameArr.length)];
            this.btn_spine.play(randomName);
            this.sp_cat.LoadCat(this.catVo.catRes.res, Laya.Handler.create(this, () => {
                let ranAniName = this.GetRandomAniName();
                ranAniName && this.sp_cat.PlayAni(ranAniName);
            }), this._view);
            this.txt_catName.text = this.catVo.catRes.name;
            this.txt_jiacheng.text = "邀请后，岛屿总收益增加" + CatModel.getRate(this.catVo.totalGoldAddRate);
            let c1 = this.icon_nrs.getController("c1");
            c1.selectedIndex = Math.min(this.catVo.catRes.quality, c1.pageCount - 1);
            this.showBanner();
        }
        GetRandomAniName() {
            let ret = null;
            let cActIdStr = CVModel.Inst.GetValue(CVModel.cActId, CVModel.valType_array);
            if (cActIdStr) {
                let cActIds = cActIdStr;
                let ranId = parseInt(cActIds[MathUtil.makeRandom(0, cActIds.length)]);
                let act = actResource.getItemByKey(ranId);
                ret = act && act.name;
            }
            return ret;
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.spine_get) {
                this.OnBtnYaoQingClick();
            }
            else if (btn == this.btn_back) {
                this.OnBtnBackClick();
            }
        }
        OnBtnYaoQingClick() {
            if (!this.catVo)
                return;
            if (CatModel.Inst.GetLastMaowuCount() <= 0) {
                let catHouseVo = BuildModel.Inst.maowuBuildVO;
                if (catHouseVo && catHouseVo.IsUnLock()) {
                    UISystem.Inst.CreateWindowView(UIBuildView, catHouseVo);
                    GameDispatcher.Inst.event(UIGameEvent.CLICK_BUILD, catHouseVo);
                }
                this.close();
                return I18nUtils.ShowI18nById("maowu_no_new");
            }
            PlatformMgr.Inst.showAd(AdType.ad_24, Laya.Handler.create(this, this.GetAward));
        }
        GetAward(isSuccess = true) {
            if (!isSuccess)
                return;
            let catRes = this.catVo.catRes;
            if (CatModel.Inst.GetLastMaowuCount() > 0) {
                let catVo = this.catVo;
                if (catVo.IsHas()) {
                    ItemCtrl.Inst.AddItemVO(catRes.item, catRes.itemNum);
                    UIAwardView.awardVO.setData({ "tempId": catVo.catRes.item, "count": catVo.catRes.itemNum });
                    UISystem.Inst.CreateWindowView(UIAwardView, UIAwardView.awardVO);
                }
                else {
                    CatCtrl.Inst.AddCat(catVo);
                }
            }
            else {
                I18nUtils.ShowI18nById("maowu_no_new");
            }
            this.close();
        }
        OnBtnBackClick() {
            let handler = Laya.Handler.create(this, this.OnBtnYaoQingClick);
            UISystem.Inst.CreateWindowView(UICatBefallAlert, handler);
        }
        onResize() {
            super.onResize();
            this._view.setSize(Laya.stage.width, Laya.stage.height);
        }
        Reset() {
            super.Reset();
        }
    }

    class UICatBefallAlert extends UIBaseWindow {
        constructor() {
            super("befallCat", "UICatBefallAlert");
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = false;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                if (AdCtrl.Inst.wxIsShowEmbedAd) {
                    this.hideBanner();
                    Laya.timer.once(1000, this, this.resizeBanner);
                }
                else {
                    this.showBanner();
                }
            }
            else if (Laya.Browser.onQGMiniGame) {
                this.hideBanner();
                AdCtrl.Inst.ShowNatiEmbed(this);
            }
            else {
                this.showBanner();
            }
        }
        resizeBanner() {
            if (GuideModel.Inst.IsMainGuideAll()) {
                let bottom = this.bg.y + this.bg.height;
                UISystem.Inst.CreateWindowView(UIWXBannerView, { "bottom": bottom });
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_no) {
                UISystem.Inst.RemoveWindowClass(UICatBefallView);
                this.close();
            }
        }
        onBtnCloseClick() {
            if (this.Data instanceof Laya.Handler) {
                this.Data.run();
            }
            super.onBtnCloseClick();
        }
        onResize() {
            super.onResize();
            this._view.setSize(Laya.stage.width, Laya.stage.height);
        }
        Reset() {
            super.Reset();
            AdCtrl.Inst.ClearShowNatiEmbedTimer();
        }
    }

    class UIWXBannerView extends UIBaseWindow {
        constructor() {
            super("ad", "UIWXBannerView");
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_AD;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = false;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            let str = MathUtil.goldFormat(PlatformModel.Inst.Bannergold);
            this.btn_coin.title = str;
            AwardMgr.Inst.GetAward(ItemModel.ITEM_GOLD, AwardVO.TYPE_ITEM, PlatformModel.Inst.Bannergold, false);
            let bottom = data.bottom;
            this.group_main.y = bottom - this.group_main.height;
            this.resizeBanner();
        }
        resizeBanner() {
            let bottom = this.bg.y + this.bg.height - 10;
            let data = { "bottomY": bottom };
            this.showBanner(data);
            GameDispatcher.Inst.event(UIGameEvent.RESIZE_BANNER, data);
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
        }
        onBtnCloseClick() {
            super.onBtnCloseClick();
            let from = this.btn_coin.localToGlobal(0, 0);
            EffectMgr.Inst.playGoldEffect(PlatformModel.Inst.Bannergold, from);
        }
        Reset() {
            super.Reset();
            UISystem.Inst.RemoveWindowClass(UICatBefallView);
            UISystem.Inst.RemoveWindowClass(UICatBefallAlert);
        }
    }

    class UIBuildCreateView extends UIBaseWindow {
        constructor() {
            super("build", "UIBuildCreateView");
            this.buildVo = null;
            this.surplusTime = NaN;
            this.complete = null;
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = true;
        }
        registerClasses() {
            super.registerClasses();
        }
        InitUI() {
            super.InitUI();
            this.txt_coinNum = this.btn_create.getChild("txt_coinNum").asTextField;
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.UPDATE_CURRENCY_DATA, this.OnUpdateCurrency, this);
            this.AddGameListener(UIGameEvent.ON_HIDE, this.onHide, this);
        }
        RemoveListener() {
            super.RemoveListener();
        }
        InitData(param) {
            this.m_param = param;
            if (Array.isArray(param)) {
                this.buildVo = param[0];
                this.complete = param[1];
            }
            else {
                this.buildVo = param;
            }
            if (this.buildVo && !this.buildVo.PreIsUnLock()) {
                this.btn_create.enabled = false;
            }
            else {
                this.btn_create.enabled = true;
            }
            this.UpdateData();
            this.OnUpdateCurrency();
            Laya.timer.loop(1000, this, this.UpdateTime);
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                if (AdCtrl.Inst.wxIsShowEmbedAd) {
                    this.hideBanner();
                    Laya.timer.once(1000, this, this.resizeBanner);
                }
                else {
                    this.showBanner();
                }
            }
            else if (Laya.Browser.onQGMiniGame) {
                if (GuideModel.Inst.IsMainGuideAll()) {
                    this.hideBanner();
                    AdCtrl.Inst.ShowNatiEmbed(this);
                }
            }
            else if (Laya.Browser.onTTMiniGame) {
                this.hideBanner();
            }
            else {
                this.showBanner();
            }
        }
        resizeBanner() {
            if (GuideModel.Inst.IsMainGuideAll()) {
                let bottom = this.bg.y + this.bg.height;
                UISystem.Inst.CreateWindowView(UIWXBannerView, { "bottom": bottom });
            }
        }
        onHide(panelName) {
            if (panelName == "UITipView" || panelName == "UIGuideView") {
            }
            else {
                this.showBanner();
            }
        }
        UpdateTime() {
            if (isNaN(this.surplusTime) || this.surplusTime <= 0) {
                this.txt_time.visible = false;
            }
            else {
                this.surplusTime--;
                this.txt_time.visible = true;
                this.txt_time.text = `剩余时间: ${StringUtil.formatToTime(this.surplusTime, "天00:00:00")}`;
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_create) {
                this.OnBtnCreateClick();
            }
            else if (btn == this.btn_adCreat) {
                this.OnBtnAdClick();
            }
        }
        OnBtnCreateClick() {
            if (this.buildVo instanceof BuildVO) {
                if (this.buildVo.buildRes.id == 1) {
                    if (GuideModel.Inst.guideVO && GuideModel.Inst.guideVO.guideRes.id < GuideModel.GUID_CREATE_MAOWU_ID) {
                        UISystem.Inst.CreateWindowView(UITipView, "请先完成钓鱼场升级引导");
                        return;
                    }
                }
                let buildRes = this.buildVo.buildRes;
                let costGold = parseFloat(buildRes.unlockGold);
                if (MoneyModel.gold.isLessThan(costGold)) {
                    return I18nUtils.ShowI18nById("jbbz");
                }
                MoneyModel.gold.add(-costGold);
                this.CreatBuild(true);
            }
        }
        OnBtnAdClick() {
            if (this.buildVo.buildRes.id == 1) {
                if (GuideModel.Inst.guideVO && GuideModel.Inst.guideVO.guideRes.id < GuideModel.GUID_CREATE_MAOWU_ID) {
                    UISystem.Inst.CreateWindowView(UITipView, "请先完成钓鱼场升级引导");
                    return;
                }
            }
            PlatformMgr.Inst.showAd(AdType.ad_25, Laya.Handler.create(this, this.CreatBuild));
        }
        CreatBuild(isSuccess = true) {
            if (!isSuccess)
                return;
            if (!this.buildVo)
                return;
            this.buildVo.level = 1;
            BuildCtrl.Inst.CheckCanUnlockBUild();
            this.complete && this.complete.runWith(this.buildVo);
            GameDispatcher.Inst.event(UIGameEvent.BUILD_UNLOCK, this.buildVo);
            this.close();
        }
        OnUpdateCurrency() {
            if (!this.buildVo)
                return;
            let buildRes = this.buildVo.buildRes;
            let costGold = parseFloat(buildRes.unlockGold);
            if (!this.buildVo.PreIsUnLock()) {
                this.btn_create.enabled = false;
            }
            else {
                if (MoneyModel.gold.isLessThan(costGold)) {
                    this.btn_create.enabled = false;
                }
                else {
                    this.btn_create.enabled = true;
                }
            }
        }
        UpdateData() {
            if (this.buildVo instanceof BuildVO) {
                let buildRes = this.buildVo.buildRes;
                let url = UrlUtils.GetIconBuildUrl(buildRes.icon);
                PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                    this.sp_build.url = url;
                }));
                this.txt_buildName.text = buildRes.name;
                this.txt_tips.text = buildRes.desc;
                let gold = buildRes.unlockGold == "" ? 0 : parseFloat(buildRes.unlockGold);
                this.txt_coinNum.text = MathUtil.goldFormat(gold);
                this.surplusTime = this.GetSurplusTime(gold);
                this.UpdateTime();
            }
        }
        GetSurplusTime(costGold) {
            let rate = BuildModel.Inst.GetAllBuildGoldRate(1, false);
            let gold = MoneyModel.gold.num.minus(costGold).multipliedBy(-1);
            if (gold.isLessThan(0) || rate.isZero())
                return 0;
            return gold.div(rate).integerValue().toNumber();
        }
        onResize() {
            super.onResize();
        }
        Reset() {
            super.Reset();
            this.buildVo = null;
            this.complete = null;
            this.surplusTime = NaN;
            Laya.timer.clear(this, this.UpdateTime);
            AdCtrl.Inst.ClearShowNatiEmbedTimer();
        }
    }

    class UILoadingLeafView extends fgui.GComponent {
        constructor() {
            super();
        }
        constructFromXML(xml) {
            super.constructFromXML(xml);
            FairyUtils.setVar(this, this);
        }
        InitData(data) {
            this.Reset();
            this.onResize();
            this.spine_loadingcircle.play("loadingcircle");
        }
        onBtnClick(btn) {
        }
        onResize() {
            this.setSize(Laya.stage.width, Laya.stage.height);
        }
        Reset() {
            Laya.timer.clearAll(this);
            this.spine_loadingcircle.Reset();
        }
    }

    class UISceneBaseIcon extends TComponent {
        constructor() {
            super(...arguments);
            this.outPos = new Laya.Vector4();
            this.inPos = new Laya.Vector3();
            this.initScale = 0;
        }
        constructFromXML(xml) {
            super.constructFromXML(xml);
            this.AddListener();
            this.initScale = this.scaleX;
        }
        InitFairyGui() {
            FairyUtils.setVar(this, this);
        }
        onResize() {
            this.scaleX = this.scaleY = this.initScale * GlobalConfig.scene_scale;
        }
        Destroy() {
            super.Destroy();
            this.dispose();
        }
    }

    class UISceneCanUnlockIcon extends UISceneBaseIcon {
        constructor() {
            super();
            this.img_icon = null;
            this.txt_buyPrice = null;
            this.out = new Laya.Vector3();
            this.initDepth = NaN;
        }
        static Create() {
            return fgui.UIPackage.createObject("main", "UISceneCanUnlockIcon");
        }
        InitUI() {
        }
        InitData(param) {
            this.m_param = param;
            let buildVO = this.m_param;
            let url = UrlUtils.GetIconBuildUrl(buildVO.buildRes.icon);
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                this.img_icon.url = url;
            }));
            let bigNum = buildVO.NeedGold();
            this.txt_buyPrice.text = MathUtil.goldFormat(bigNum);
            this.onResize();
        }
        get buildVO() {
            return this.m_param;
        }
        AddListener() {
            super.AddListener();
            this.onClick(this, this.OnClickIcon);
        }
        OnClickIcon() {
            let guideVO = GuideModel.Inst.guideVO;
            if (this.buildVO.buildRes.id == 102 && guideVO && guideVO.isForce && guideVO.guideRes.id < GuideModel.GUID_CLICK_MZBS_ID) {
                return;
            }
            else if (this.buildVO.isMaowo) {
                if (GuideModel.Inst.guideVO && GuideModel.Inst.guideVO.isLessThan(8, 1)) {
                    UISystem.Inst.CreateWindowView(UITipView, "请先完成其他引导");
                    return;
                }
            }
            UISystem.Inst.CreateWindowView(UIBuildCreateView, [this.Data, null]);
        }
        RemoveListener() {
            super.RemoveListener();
            this.offClick(this, this.OnClickIcon);
        }
        onResize() {
            super.onResize();
            let buildVO = this.m_param;
            if (buildVO == null)
                return;
            let position = buildVO.buildRes.position;
            this.inPos.setValue(-parseFloat(position[0]), parseFloat(position[1]), parseFloat(position[2]));
            let camera = SceneMgr.Inst.GetCamera();
            camera.transform.getForward(this.out);
            let zz1 = new Laya.Vector3(-this.out.x, -this.out.y, -this.out.z);
            let curZ = MathUtil.ProjectDistance(this.inPos, camera.transform.position, zz1);
            let scale = GlobalConfig.scene_scale;
            if (scale != 2)
                scale = 1 / (2 - scale);
            if (isNaN(this.initDepth)) {
                this.initDepth = -75;
            }
            if (curZ)
                scale *= (this.initDepth / curZ);
            scale = fgui.ToolSet.clamp(scale, 0.1, 2);
            this.scaleX = this.scaleY = scale;
            SceneMgr.Inst.Get3DTo2D(this.inPos, this.outPos);
            this.x = this.outPos.x;
            this.y = this.outPos.y + buildVO.buildRes.iconOffsetY * scale;
        }
    }

    class UISceneCatIcon extends UISceneBaseIcon {
        constructor() {
            super(...arguments);
            this.ctr = null;
            this.cat = null;
        }
        static Create() {
            return fgui.UIPackage.createObject("main", "UISceneCatIcon");
        }
        InitFairyGui() {
            FairyUtils.setVar(this, this);
        }
        InitUI() {
            this.mc.playing = false;
            this.ctr = this.getController("ctr");
        }
        InitData(param) {
            this.cat = param[0];
            this.onResize();
            this.mc.playing = true;
            let index = param[1];
            this.ctr.selectedIndex = index;
            if (this.spine_mo instanceof TSpineButton) {
                if (index == 0) {
                    this.spine_mo.play("moyimo");
                }
                else {
                    this.spine_mo.Reset();
                }
            }
            else {
                throw new Error("this.spine_mo 不是TSpineButton");
            }
        }
        AddListener() {
            super.AddListener();
            Laya.timer.frameLoop(1, this, this.onResize);
            this.onClick(this, this.OnClickMoCat);
            this.spine_mo.onClick(this, this.OnClickMoCat);
        }
        OnClickMoCat() {
            if (!this.cat)
                return;
            if (this.ctr.selectedIndex == 0) {
                CatCtrl.Inst.EnterLuMaoOnMainView(this.cat.catVO);
            }
        }
        RemoveListener() {
            super.RemoveListener();
            Laya.timer.clear(this, this.onResize);
            this.mc.offClick(this, this.OnClickMoCat);
            this.spine_mo.offClick(this, this.OnClickMoCat);
        }
        onResize() {
            super.onResize();
            if (this.cat == null)
                return;
            let position = this.cat.transform.position;
            position.cloneTo(this.inPos);
            this.inPos.y += 2;
            SceneMgr.Inst.Get3DTo2D(this.inPos, this.outPos);
            this.x = this.outPos.x;
            this.y = this.outPos.y;
        }
        Reset() {
            super.Reset();
            this.mc.playing = false;
        }
    }

    class UISceneCatPenIconItem extends UISceneBaseIcon {
        constructor() {
            super(...arguments);
            this.initCatBasinDis = NaN;
        }
        static Create() {
            return fgui.UIPackage.createObject("main", "UISceneCatPenIconItem");
        }
        InitFairyGui() {
            FairyUtils.setVar(this, this);
        }
        InitUI() {
            super.InitUI();
            this.touchable = true;
        }
        InitData(param) {
            this.m_param = param;
            this.catPen = param;
            Laya.timer.once(1000, this, this.onResize);
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(Laya.Event.CLICK, this.clickIcon, this, this);
        }
        RemoveListener() {
            super.RemoveListener();
        }
        clickIcon(e) {
            e.stopPropagation();
            if (CatHouse.Inst) {
                CatHouse.Inst.AddFish();
            }
        }
        onResize() {
            super.onResize();
            if (BuildModel.Inst.maowuBuildVO == null) {
                return;
            }
            let scale = GlobalConfig.scene_scale - 0.1;
            let catBasinPos = new Laya.Vector3(-18.74, 1.73, 0.45);
            let camera = SceneMgr.Inst.GetCamera();
            camera.transform.getForward(this.inPos);
            let zz1 = new Laya.Vector3(-this.inPos.x, -this.inPos.y, -this.inPos.z);
            let curZ = MathUtil.ProjectDistance(catBasinPos, camera.transform.position, zz1);
            if (isNaN(this.initCatBasinDis)) {
                this.initCatBasinDis = curZ;
            }
            scale *= (this.initCatBasinDis / curZ);
            scale = Math.max(0, scale);
            this.setScale(scale, scale);
            catBasinPos.cloneTo(this.inPos);
            this.inPos.y += 4;
            let v2 = MathUtil.GetPointFrom3DWorld(camera, this.inPos);
            this.setXY(v2.x - 5, v2.y);
        }
    }

    class UISceneIconItem extends UISceneBaseIcon {
        static Create() {
            return fgui.UIPackage.createObject("main", "UISceneIconItem");
        }
        InitFairyGui() {
            FairyUtils.setVar(this, this);
        }
        InitUI() {
            this.touchable = false;
            this.c1 = this.getController("c1");
        }
        InitData(param) {
            this.m_param = param[0];
            this.c1.selectedPage = param[1];
            this.onResize();
        }
        AddListener() {
            super.AddListener();
        }
        RemoveListener() {
            super.RemoveListener();
        }
        onResize() {
            super.onResize();
            let buildVO = this.m_param;
            if (buildVO == null)
                return;
            let position = buildVO.buildRes.position;
            this.inPos.setValue(-parseFloat(position[0]), parseFloat(position[1]) + 4, parseFloat(position[2]));
            SceneMgr.Inst.Get3DTo2D(this.inPos, this.outPos);
            this.x = this.outPos.x;
            this.y = this.outPos.y;
        }
    }

    class UISceneLandAlertView extends UIBaseWindow {
        constructor() {
            super("main", "UISceneLandAlertView");
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_CENTER;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isEffect = true;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            this.UpdateData();
        }
        UpdateData() {
            let landUrl = LandModel.Inst.nextIcon || LandModel.Inst.icon;
            let name = LandModel.Inst.nextName || LandModel.Inst.name;
            let url = UrlUtils.GetIconBuildUrl(landUrl);
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                this.sp_land.url = url;
            }));
            this.txt_landName.text = name;
            this.txt_landTips.text = LandModel.Inst.NextLandRes.desc;
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_sure) {
                this.OnBtnSureClick();
            }
        }
        OnBtnSureClick() {
            this.close();
        }
        Reset() {
            super.Reset();
        }
    }

    class UISceneLandPromptIcon extends UISceneBaseIcon {
        constructor() {
            super();
            this.img_icon = null;
            this.out = new Laya.Vector3();
            this.initDepth = NaN;
        }
        static Create() {
            return fgui.UIPackage.createObject("main", "UISceneLandPromptIcon");
        }
        InitUI() {
        }
        InitData(param) {
            this.m_param = param;
            let isLandRes = this.m_param;
            let url = UrlUtils.GetIconBuildUrl(isLandRes.icon);
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                this.img_icon.url = url;
            }));
            this.onResize();
        }
        get buildVO() {
            return this.m_param;
        }
        AddListener() {
            super.AddListener();
            this.onClick(this, this.OnClickIcon);
        }
        OnClickIcon() {
            UISystem.Inst.CreateWindowView(UISceneLandAlertView);
        }
        RemoveListener() {
            super.RemoveListener();
            this.offClick(this, this.OnClickIcon);
        }
        onResize() {
            super.onResize();
            let isLandRes = this.m_param;
            if (isLandRes == null)
                return;
            let position = isLandRes.board;
            this.inPos.setValue(-parseFloat(position[0]), parseFloat(position[1]), parseFloat(position[2]));
            let camera = SceneMgr.Inst.GetCamera();
            camera.transform.getForward(this.out);
            let zz1 = new Laya.Vector3(-this.out.x, -this.out.y, -this.out.z);
            let curZ = MathUtil.ProjectDistance(this.inPos, camera.transform.position, zz1);
            let scale = GlobalConfig.scene_scale;
            if (scale != 2)
                scale = 1 / (2 - scale);
            if (isNaN(this.initDepth)) {
                this.initDepth = -75;
            }
            if (curZ)
                scale *= (this.initDepth / curZ);
            scale = fgui.ToolSet.clamp(scale, 0.1, 2);
            this.scaleX = this.scaleY = scale;
            SceneMgr.Inst.Get3DTo2D(this.inPos, this.outPos);
            this.x = this.outPos.x;
            this.y = this.outPos.y - 50 * scale;
        }
    }

    class UISceneIconView extends TComponent {
        constructor() {
            super(...arguments);
            this.buildDic = new Dictionary();
            this.catDic = new Dictionary();
        }
        InitUI() {
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.BUILD_ADD_OR_REMOVE_ICON, this.onBuildAddOrRremoveIcon, this);
            this.AddGameListener(UIGameEvent.BUILD_CANUNLOCK_ICON, this.onBuildCanUnlockIcon, this);
            this.AddGameListener(UIGameEvent.CAT_ADD_OR_REMOVE_ICON, this.onCatAddOrRremoveIcon, this);
            this.AddGameListener(UIGameEvent.SCENE_RESIZE, this.OnSceneResize, this);
            this.AddGameListener(UIGameEvent.ISHIDE_ICON, this.IsHideIcon, this);
            this.AddGameListener(UIGameEvent.UPDATE_FISH_STATE, this.onUpdateFishState, this);
            this.AddGameListener(UIGameEvent.LAND_PROMPT_ICON, this.onLandPromptIcon, this);
            if (GuideModel.Inst.isOpenGuide) {
                this.AddGameListener(UIGameEvent.GUIDE_SHOW, this.onShowGuide, this);
            }
        }
        RemoveListener() {
            super.RemoveListener();
        }
        onShowGuide(guideVO) {
            if (UISceneIconView.canUnlockBuild && !UISceneIconView.canUnlockBuild.visible && guideVO && guideVO.page > 1) {
                UISceneIconView.canUnlockBuild.visible = !GuideModel.Inst.HasGuide() || GuideModel.Inst.guideVO.guideRes.page > 1;
            }
        }
        onLandPromptIcon(isLand, isAdd) {
            if (isAdd) {
                if (UISceneIconView.isLandPrompt == null) {
                    UISceneIconView.isLandPrompt = UISceneLandPromptIcon.Create();
                    this.addChildAt(UISceneIconView.isLandPrompt, 0);
                }
                UISceneIconView.isLandPrompt.InitData(isLand);
            }
            else {
                if (UISceneIconView.isLandPrompt != null) {
                    UISceneIconView.isLandPrompt.RemoveListener();
                    UISceneIconView.isLandPrompt.Reset();
                    UISceneIconView.isLandPrompt.Destroy();
                    UISceneIconView.isLandPrompt = null;
                }
            }
        }
        onBuildCanUnlockIcon(buildVO, isAdd) {
            if (isAdd) {
                if (UISceneIconView.canUnlockBuild == null) {
                    UISceneIconView.canUnlockBuild = UISceneCanUnlockIcon.Create();
                    this.addChildAt(UISceneIconView.canUnlockBuild, 0);
                }
                if (GuideModel.Inst.isOpenGuide) {
                    UISceneIconView.canUnlockBuild.visible = !GuideModel.Inst.HasGuide() || (GuideModel.Inst.guideVO && GuideModel.Inst.guideVO.guideRes.page > 1);
                }
                UISceneIconView.canUnlockBuild.InitData(buildVO);
            }
            else {
                if (UISceneIconView.canUnlockBuild != null) {
                    UISceneIconView.canUnlockBuild.RemoveListener();
                    UISceneIconView.canUnlockBuild.Reset();
                    UISceneIconView.canUnlockBuild.Destroy();
                    UISceneIconView.canUnlockBuild = null;
                }
            }
        }
        onUpdateFishState(isShow) {
            if (!isShow) {
                if (UISceneIconView.FishIcon == null) {
                    UISceneIconView.FishIcon = UISceneCatPenIconItem.Create();
                    this.addChildAt(UISceneIconView.FishIcon, 0);
                }
                UISceneIconView.FishIcon.InitData(null);
            }
            else {
                if (UISceneIconView.FishIcon != null) {
                    UISceneIconView.FishIcon.RemoveListener();
                    UISceneIconView.FishIcon.Reset();
                    UISceneIconView.FishIcon.Destroy();
                    UISceneIconView.FishIcon = null;
                }
            }
        }
        onBuildAddOrRremoveIcon(buildVO, isAdd, bsName) {
            if (!buildVO)
                return;
            let build = this.buildDic[buildVO.buildRes.id];
            if (isAdd) {
                if (GuideModel.Inst.IsNewer()) {
                    return;
                }
                if (build == null) {
                    build = UISceneIconItem.Create();
                    this.addChild(build);
                    this.buildDic[buildVO.buildRes.id] = build;
                }
                build.InitData([buildVO, bsName]);
            }
            else {
                if (build != null) {
                    build.RemoveListener();
                    build.Reset();
                    build.Destroy();
                    delete this.buildDic[buildVO.buildRes.id];
                    build = null;
                }
            }
        }
        onCatAddOrRremoveIcon(cat, isAdd, index = 0) {
            if (cat == null || cat.catVO == null) {
                return;
            }
            let catIcon = this.catDic[cat.catVO.objId];
            if (isAdd) {
                if (catIcon == null) {
                    catIcon = UISceneCatIcon.Create();
                    this.addChild(catIcon);
                    this.catDic[cat.catVO.objId] = catIcon;
                }
                catIcon.InitData([cat, index]);
            }
            else {
                if (catIcon != null) {
                    catIcon.RemoveListener();
                    catIcon.Reset();
                    catIcon.Destroy();
                    delete this.catDic[cat.catVO.objId];
                    catIcon = null;
                }
            }
        }
        IsHideIcon(isHide = false) {
            this.visible = !isHide;
            if (!isHide) {
                this.OnSceneResize();
            }
        }
        OnSceneResize() {
            for (let i = 0; i < this.numChildren; i++) {
                const item = this.getChildAt(i);
                item.onResize();
            }
        }
    }
    UISceneIconView.canUnlockBuild = null;
    UISceneIconView.isLandPrompt = null;
    UISceneIconView.FishIcon = null;

    class UIGiveExpressView extends UIBaseWindow {
        constructor() {
            super("storeCont", "UIGiveExpressView");
            this.sp_catModel = null;
            this.c1 = null;
            this.group_bg = null;
            this.complete = null;
            this.catVo = null;
            this.oldGroupX = 0;
            this.startTime = 0;
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
            this.oldGroupX = this.group_bg.x;
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.CATADDSKIN, this.OnAddSkinPart, this);
        }
        InitData(data) {
            super.InitData(data);
            this.catVo = data[0];
            this.complete = data[1];
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.gachaEmbient_Intro));
            if (this.catVo instanceof CatVO) {
                let complete = Laya.Handler.create(this, (sp) => {
                    this.startTime = Laya.timer.currTimer;
                    this.PlayAni();
                    this.sp_catModel.AddCatPart([ShopLockDrawModel.EXPRESS_BAGID, ShopLockDrawModel.EXPRESS_HATID]);
                    let parentNode = MathUtil.FindSprite3DByName(sp, "Bip001 Spine");
                    EffectMgr.Inst.playConfigEffect(EffectMgr.Eff_smoke, parentNode, null, null, null, null, 4000, true, false);
                });
                this.sp_catModel.LoadCat(this.catVo.catRes.res, complete);
            }
            this.hideBanner();
        }
        OnAddSkinPart(resName, part) {
            let skinRes = skinResource.getItemByKey(ShopLockDrawModel.EXPRESS_BAGID);
            let resId = skinRes.resId;
            let name;
            for (let i = 0; i < resId.length; i++) {
                name = resId[i];
                if (name == resName) {
                    if (part) {
                        let aniScript = part.addComponent(AnimatorScript);
                        aniScript.PlayState("cat2_delivery_bag_01", null, 0);
                        Laya.timer.once(Math.max(1800 - (Laya.timer.currTimer - this.startTime), 0), this, () => {
                            if (part && part.transform) {
                                let partPos = part.transform.position.clone();
                                let scale = part.transform.getWorldLossyScale();
                                let rotationE = part.transform.rotationEuler.clone();
                                part.removeSelf();
                                this.sp_catModel.AddSprite3d(part);
                                part.transform.position = partPos;
                                part.transform.setWorldLossyScale(scale);
                                part.transform.rotationEuler = rotationE;
                                aniScript.PlayState("cat2_delivery_bag_01", null, 1);
                            }
                        });
                    }
                    continue;
                }
            }
        }
        PlayAni() {
            Laya.timer.once(500, this, () => {
                Laya.Tween.to(this.group_bg, { x: 0 }, 500, Laya.Ease.sineOut);
            });
            let complete = Laya.Handler.create(this, () => {
                this.complete && this.complete.run();
                this.close();
            });
            this.sp_catModel.PlayAni(ShopLockDrawModel.FASTMAILANI, false, complete);
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
        }
        Reset() {
            super.Reset();
            this.c1.selectedIndex = 0;
            this.complete = null;
            this.catVo = null;
            this.group_bg.x = this.oldGroupX;
            Laya.timer.clearAll(this);
            Laya.Tween.clearAll(this.group_bg);
        }
    }

    class ShopContListItem extends TListItem {
        constructor() {
            super();
            this.sp_item = null;
            this.txt_num = null;
            this.txt_desc = null;
        }
        InitUI() {
            super.InitUI();
        }
        InitData(data) {
            super.InitData(data);
            this.UpdateData();
        }
        UpdateData() {
            let vo = this.Data;
            if (vo instanceof ShopLockDrawVO) {
                this.sp_item.icon = vo.awardIconUrl;
                this.txt_num.text = "x" + vo.num;
                this.txt_desc.text = vo.awardRes.name;
                this.sp_item.ShowNRS(1, vo.quality);
            }
            else if (vo instanceof SkinVO) {
                PanelRegister.loadUrl(vo.iconUrl, Laya.Handler.create(this, (url) => {
                    this.sp_item.icon = url;
                }));
                this.txt_num.text = "x" + 1;
                this.txt_desc.text = vo.skinRes.name;
                let quaStr = "";
                let qua = vo.skinRes.quality;
                if (qua == 1) {
                    quaStr = "N";
                }
                else if (qua == 2) {
                    quaStr = "R";
                }
                else if (qua == 3) {
                    quaStr = "S";
                }
                this.sp_item.ShowNRS(1, quaStr);
            }
            else if (vo instanceof AwardVO) {
                let qua = 0;
                let name = "";
                let num = 1;
                let iconUrl = "";
                if (vo.IsItem) {
                    qua = vo.itemRes.quality;
                    let icon = vo.itemRes.icon;
                    let type = vo.itemRes.type;
                    if (type == ItemModel.TYPE_BUILD) {
                        iconUrl = UrlUtils.GetIconPartUrl(icon);
                    }
                    else {
                        iconUrl = UrlUtils.GetIconItemsUrl(icon);
                    }
                    name = vo.itemRes.name;
                }
                else if (vo.IsCat) {
                    qua = vo.catRes.quality;
                    iconUrl = vo.catHeadUrl;
                    name = vo.catRes.name;
                }
                num = vo.count;
                PanelRegister.loadUrl(iconUrl, Laya.Handler.create(this, (url) => {
                    this.sp_item.icon = iconUrl;
                }));
                this.txt_num.text = "x" + num;
                this.txt_desc.text = name;
                let quaStr = "";
                if (qua == 1) {
                    quaStr = "N";
                }
                else if (qua == 2) {
                    quaStr = "R";
                }
                else if (qua == 3) {
                    quaStr = "S";
                }
                this.sp_item.ShowNRS(1, quaStr);
            }
        }
        Reset() {
            super.Reset();
        }
    }

    class ShopDaoJuContListItem extends TListItem {
        constructor() {
            super();
            this.c1 = null;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
        }
        InitData(data) {
            super.InitData(data);
            if (data >= this.c1.pageCount)
                return;
            this.c1.selectedIndex = data;
        }
        Reset() {
            super.Reset();
        }
    }

    class ShopLockAwardBtn extends TListButtonItem {
        constructor() {
            super();
            this.c1 = null;
            this.txt_desc = null;
            this.txt_dimNum = null;
            this.txt_count = null;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            this.SetCanLock(1);
            this.SetLockAwardCount("1");
        }
        InitData(data) {
            super.InitData(data);
        }
        SetCanLock(index = 0) {
            this.c1.selectedIndex = index;
        }
        SetDiamond(val) {
            this.txt_dimNum.text = val;
        }
        SetLockAwardCount(val) {
            this.txt_count.text = val + "次!";
        }
        Reset() {
            super.Reset();
        }
    }

    class UIWaresIconItem extends TListButtonItem {
        constructor() {
            super();
            this.c1 = null;
            this.c2 = null;
            this.sp_nrs0 = null;
            this.sp_nrs1 = null;
            this.txt_num = null;
            this.txt_num2 = null;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
        }
        InitData(data) {
            super.InitData(data);
        }
        ShowNRS(index = 0, nrs = "N") {
            this.c1.selectedIndex = index;
            if (nrs == "" || nrs == null) {
                this.c1.selectedIndex = 0;
                return;
            }
            if (index == 1) {
                this.sp_nrs0.getController("c1").selectedPage = nrs;
            }
            else if (index == 2) {
                this.sp_nrs1.getController("c1").selectedPage = nrs;
            }
        }
        ShowNum(index = 0, str = "") {
            this.c2.selectedIndex = index;
            if (index == 1) {
                this.txt_num.text = str;
            }
            else if (index == 2) {
                this.txt_num2.text = str;
            }
        }
        Reset() {
            super.Reset();
        }
    }

    class UIStoreContentView extends UIBaseWindow {
        constructor() {
            super("storeCont", "UIStoreContentView");
            this.c1 = null;
            this.sp_catModel = null;
            this.sp_getprop = null;
            this.txt_desc = null;
            this.txt_courierName = null;
            this.txt_tip = null;
            this.btn_lockAward = null;
            this.btn_lockAwardOne = null;
            this.btn_lockAwardTen = null;
            this.btn_jifenAward = null;
            this.eglist_daojus = null;
            this.eglist_cloth = null;
            this.btn_close2 = null;
            this.txt_tzdesc = null;
            this.btn_close3 = null;
            this.eglist_tz = null;
            this.btn_buy = null;
            this.txt_shouyidesc = null;
            this.vos = null;
            this.type = 0;
            this.catCourier = null;
            this.skinVos = null;
            this.awardVos = null;
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("storeCont", "ShopLockAwardBtn", ShopLockAwardBtn);
            PanelRegister.registerClass("storeCont", "ShopContListItem", ShopContListItem);
            PanelRegister.registerClass("storeCont", "ShopContListItem2", ShopContListItem);
            PanelRegister.registerClass("storeCont", "ShopDaoJuContListItem", ShopDaoJuContListItem);
            PanelRegister.registerClass("storeCont", "UIWaresIconItem", UIWaresIconItem);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
            this.btn_lockAwardTen.SetLockAwardCount("10+1");
            this.btn_lockAwardTen.SetCanLock(2);
            this.spine_light = this.btn_jifenAward.getChild("spine_light");
            this.eglist_daojus.setVirtual();
            this.eglist_cloth.setVirtual();
            let scaleY = 1;
            if (Laya.Browser.onQQMiniGame) {
                scaleY = PlatformModel.Inst.scaleY / Laya.stage.clientScaleY;
            }
            if (PlatformModel.Inst.rightUpRect) {
                this.group_top.y = PlatformModel.Inst.rightUpRect.y * scaleY + PlatformModel.Inst.rightUpRect.height * scaleY - 30;
            }
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            this.catCourier = ShopLockDrawModel.Inst.GetRandomCourier();
            let vos = null;
            if (data instanceof Array) {
                vos = data[1];
                data = data[0];
            }
            this.c1.selectedIndex = data;
            if (data == 0) {
                this.vos = ShopLockDrawModel.Inst.allDaojuVos;
                this.type = ShopLockDrawVO.TYPE_DAOJU;
            }
            else if (data == 1) {
                this.vos = ShopLockDrawModel.Inst.allPiFuVos;
                this.type = ShopLockDrawVO.TYPE_PIFU;
            }
            else if (data == 2) {
                this.skinVos = vos;
            }
            else if (data == 3) {
                this.awardVos = vos;
            }
            this.InitCatModel();
            this.UpdateData();
            this.hideBanner();
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_lockAward) {
                if (ShopLockDrawModel.Inst.canFreeLockDraw) {
                    let buyok = MoneyModel.diamond.cost(ShopLockDrawModel.Inst.lockDrawfreeDima, true, false);
                    if (buyok) {
                        ShopLockDrawModel.Inst.freeLockDrawCount--;
                        this.LockDraw(1);
                        this.AldSendEvent(ShopLockDrawModel.Inst.lockDrawfreeDima + "钻石");
                    }
                }
            }
            else if (btn == this.btn_lockAwardOne) {
                let dimaNum = 0;
                if (this.type == ShopLockDrawVO.TYPE_DAOJU) {
                    dimaNum = ShopLockDrawModel.Inst.lockDrawOneDima_daoju;
                }
                else if (this.type == ShopLockDrawVO.TYPE_PIFU) {
                    dimaNum = ShopLockDrawModel.Inst.lockDrawOneDima_pifu;
                }
                let buyok = MoneyModel.diamond.cost(dimaNum, true, false);
                if (buyok) {
                    this.LockDraw(1);
                    this.AldSendEvent(dimaNum + "钻石");
                }
            }
            else if (btn == this.btn_lockAwardTen) {
                let dimaNum = 0;
                if (this.type == ShopLockDrawVO.TYPE_DAOJU) {
                    dimaNum = ShopLockDrawModel.Inst.lockDrawTenDima_daoju;
                }
                else if (this.type == ShopLockDrawVO.TYPE_PIFU) {
                    dimaNum = ShopLockDrawModel.Inst.lockDrawTenDima_pifu;
                }
                let buyok = MoneyModel.diamond.cost(dimaNum, true, false);
                if (buyok) {
                    this.LockDraw(11);
                    this.AldSendEvent(dimaNum + "钻石");
                }
            }
            else if (btn == this.btn_close2 || btn == this.btn_close3) {
                this.onBtnCloseClick();
            }
            else if (btn == this.btn_jifenAward) {
                this.OnClickJiFenAward();
            }
            else if (btn == this.btn_buy) {
                this.OnClickBuyBtn();
            }
        }
        AldSendEvent(buyType) {
            let shopName = "";
            if (this.type == ShopLockDrawVO.TYPE_DAOJU) {
                shopName = "喵星人道具";
            }
            else if (this.type == ShopLockDrawVO.TYPE_PIFU) {
                shopName = "服饰";
            }
            PlatformMgr.Inst.aldSendEvent(EventType.shop, shopName + "-" + buyType);
        }
        InitCatModel() {
            let complete = Laya.Handler.create(this, (sp) => {
                this.sp_catModel.PlayAni(ShopLockDrawModel.IDLEANI, true, Laya.Handler.create(this, () => {
                    this.sp_catModel.PlayAni("idle_wait", true);
                }));
                let type = this.c1.selectedIndex;
                if (type == 0 || type == 1) {
                    if (type == 1) {
                        let suitvos = SkinModel.Inst.GetAllSuitVos();
                        let ran = MathUtil.makeRandom(0, suitvos.length - 1);
                        let suit = suitvos[ran];
                        this.sp_catModel.AddCatPart(suit.skinIds);
                        if (this.catCourier) {
                            this.txt_tip.text = "服装赞助";
                            this.txt_courierName.text = suit.suitRes.name;
                        }
                    }
                    else {
                        this.sp_catModel.AddCatPart([ShopLockDrawModel.EXPRESS_HATID]);
                        if (this.catCourier) {
                            this.txt_tip.text = "快递负责人";
                            this.txt_courierName.text = this.catCourier.catRes.name;
                        }
                    }
                }
                else if (type == 2 && this.skinVos) {
                    let parts = [];
                    for (let i = 0; i < this.skinVos.length; i++) {
                        parts.push(this.skinVos[i].tempId);
                    }
                    this.sp_catModel.AddCatPart(parts);
                }
                else if (type == 3 && this.awardVos) {
                }
            });
            this.sp_catModel.LoadCat(this.catCourier.catRes.res, complete);
            let url = UrlUtils.GetBuildUrl(ShopLockDrawModel.boxRes);
            let point = new Laya.Point(this.sp_catModel.x + 200, this.sp_catModel.y);
            this.sp_catModel.AddModel(url, point, 10);
            let boxUrl = UrlUtils.GetCatUrl("cat2_delivery_bag_01");
            LoadQueue.Inst.loadSprite3D(boxUrl);
        }
        UpdateData() {
            this.UpdateLockData();
            this.UpdateTaoZuang();
            this.UpdateTeShu();
        }
        UpdateLockData() {
            if (!this.vos)
                return;
            this.UpdateLockDrawBtn();
            this.btn_lockAward.SetDiamond(ShopLockDrawModel.Inst.lockDrawfreeDima + "");
            let dimaOneNum = 0;
            let dimaTenNum = 0;
            if (this.type == ShopLockDrawVO.TYPE_DAOJU) {
                this.eglist_daojus.Data = [0, 7, 1, 2, 3, 4, 5, 6];
                dimaOneNum = ShopLockDrawModel.Inst.lockDrawOneDima_daoju;
                dimaTenNum = ShopLockDrawModel.Inst.lockDrawTenDima_daoju;
            }
            else if (this.type == ShopLockDrawVO.TYPE_PIFU) {
                dimaOneNum = ShopLockDrawModel.Inst.lockDrawOneDima_pifu;
                dimaTenNum = ShopLockDrawModel.Inst.lockDrawTenDima_pifu;
            }
            this.btn_lockAwardOne.SetDiamond(dimaOneNum + "");
            this.btn_lockAwardTen.SetDiamond(dimaTenNum + "");
            this.UpdateProp();
        }
        UpdateProp() {
            if (this.type == ShopLockDrawVO.TYPE_DAOJU) {
                let maxDaoJuProp = ShopLockDrawModel.Inst.maxdaojuProp;
                let curDaoJuProp = ShopLockDrawModel.Inst.daoJuProp;
                this.sp_getprop.max = maxDaoJuProp;
                this.sp_getprop.min = 0;
                this.sp_getprop.value = Math.min(curDaoJuProp, maxDaoJuProp);
            }
            else if (this.type == ShopLockDrawVO.TYPE_PIFU) {
                let maxfushiProp = ShopLockDrawModel.Inst.maxfushiProp;
                let curfushiProp = ShopLockDrawModel.Inst.fushiProp;
                this.sp_getprop.max = maxfushiProp;
                this.sp_getprop.min = 0;
                this.sp_getprop.value = Math.min(curfushiProp, maxfushiProp);
            }
            if (ShopLockDrawModel.Inst.CanGetJiFenAward(this.type)) {
                this.spine_light.play("chengzhanglight");
            }
            else {
                this.spine_light.Reset();
            }
            GameDispatcher.Inst.event(UIGameEvent.SHOW_LIBAO_RED);
        }
        UpdateLockDrawBtn() {
            if (ShopLockDrawModel.Inst.canFreeLockDraw) {
                this.btn_lockAward.SetLockAwardCount("1");
                this.btn_lockAward.SetCanLock(1);
                this.btn_lockAward.touchable = true;
            }
            else {
                this.btn_lockAward.SetCanLock(0);
                this.btn_lockAward.touchable = false;
            }
        }
        LockDraw(count = 1) {
            let arr = ShopLockDrawModel.Inst.GetRandomAward(this.type, count);
            for (let i = 0; i < arr.length; i++) {
                arr[i].GetAward();
            }
            let handler = Laya.Handler.create(this, () => {
                UISystem.Inst.CreateWindowView(UIAwardView, arr);
                GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, TaskType.TASKTYPE_LOCKDRAW);
                this.UpdateLockDrawBtn();
                this.UpdateProp();
            });
            UISystem.Inst.CreateWindowView(UIGiveExpressView, [this.catCourier, handler]);
        }
        OnClickJiFenAward() {
            if (!ShopLockDrawModel.Inst.CanGetJiFenAward(this.type)) {
                return;
            }
            let vo = ShopLockDrawModel.Inst.GetJiFenAward(this.type);
            UISystem.Inst.CreateWindowView(UIAwardView, vo);
            this.UpdateProp();
        }
        UpdateTaoZuang() {
            if (!this.skinVos)
                return;
            this.eglist_tz.Data = this.skinVos;
        }
        OnClickBuyBtn() {
        }
        UpdateTeShu() {
            if (!this.awardVos)
                return;
            this.eglist_tz.Data = this.awardVos;
        }
        Reset() {
            super.Reset();
            this.vos = null;
            this.skinVos = null;
            this.awardVos = null;
            this.eglist_tz.Data = [];
        }
    }

    class BaseTaskVO {
        constructor(tempId, type = 0) {
            this.tempId = 0;
            this.state = 0;
            this.pro = 0;
            this.type = 0;
            this.tempId = tempId;
            this.type = type;
        }
        setup(data) {
            this.pro = data.pro;
            this.state = data.state;
        }
        get curPro() {
            return this.pro;
        }
        set curPro(val) {
            this.pro = val;
        }
        get maxCount() {
            return 1;
        }
        get taskRes() {
            return taskResource.getItemByKey(this.tempId);
        }
        get isMain() {
            return this.type == 1;
        }
        get isDay() {
            return this.type == 0;
        }
        get isNormal() {
            return this.state == BaseTaskVO.STATE_NORMAL;
        }
        get canAward() {
            return this.state == BaseTaskVO.STATE_CAN_AWARD;
        }
        get isCompleted() {
            return this.state == BaseTaskVO.STATE_COMPLETED;
        }
        GetAward() {
            this.state = BaseTaskVO.STATE_COMPLETED;
        }
    }
    BaseTaskVO.STATE_NORMAL = 0;
    BaseTaskVO.STATE_CAN_AWARD = 1;
    BaseTaskVO.STATE_COMPLETED = 2;

    class UITaskAwardItem extends TListButtonItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
            this.c3 = this.getController("c3");
        }
        InitData(data) {
            super.InitData(data);
        }
        SetState(state) {
            if (this.n1) {
                this.n1.visible = true;
            }
            this.btn_adGet.Reset();
            if (state == BaseTaskVO.STATE_NORMAL) {
                this.c1.selectedIndex = 0;
            }
            else if (state == BaseTaskVO.STATE_CAN_AWARD) {
                this.c1.selectedIndex = 1;
                if (this.isAd) {
                    this.btn_adGet.visible = true;
                    this.btn_adGet.play("chengzhanglight");
                    if (this.n1) {
                        this.n1.visible = false;
                    }
                }
            }
            else if (state == BaseTaskVO.STATE_COMPLETED) {
                this.c1.selectedIndex = 2;
                if (this.isAd) {
                    this.btn_adGet.visible = false;
                }
            }
        }
        get canAward() {
            return this.c1.selectedIndex == 1;
        }
        SetNum(val) {
            this.txt_num.text = val;
            this.title = val;
        }
        get isYellow() {
            return this.c2.selectedIndex == 1;
        }
        set isYellow(bol) {
            this.c2.selectedIndex = bol ? 1 : 0;
        }
        get isAd() {
            return this.c3.selectedIndex == 1;
        }
        set isAd(bol) {
            this.c3.selectedIndex = bol ? 1 : 0;
        }
        Reset() {
            super.Reset();
            this.icon = "";
            this.txt_num.text = "";
            this.title = "";
        }
    }

    class UIGuideView extends UIBaseWindow {
        constructor() {
            super("guide", "UIGuideView");
            this.showGuideTime = 0;
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_DIVERSION;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowMaskBg = false;
            this.viewStruct.isEffect = false;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
            if (GuideModel.Inst.showGuideBg) {
                this.bg.getChild("n0").alpha = 0.5;
            }
            this.msk = this.bg.getChild("msk").asGraph;
            this.c1 = this._view.getController("c1");
            this.t0 = this._view.getTransition("t0");
            this.spine_circle.Reset();
            let scaleY = 1;
            if (Laya.Browser.onQQMiniGame) {
                scaleY = PlatformModel.Inst.scaleY / Laya.stage.clientScaleY;
            }
            let rightUpRect = PlatformModel.Inst.rightUpRect;
            if (rightUpRect) {
                this.group_guide_tip.y = rightUpRect.y * scaleY + rightUpRect.height * scaleY + 20;
            }
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(Laya.Event.CLICK, this.onClickStage, this, Laya.stage);
            this.AddGameListener(UIGameEvent.GUIDE_SHOW, this.showGuide, this);
            this.AddGameListener(UIGameEvent.GUIDE_COMPELTE, this.onGuideComplete, this);
            this.AddGameListener(UIGameEvent.GUIDE_COMPELTE_ALL, this.onGuideAllComplete, this);
            this.AddGameListener(UIGameEvent.CLICK_CAT, this.onClickCat, this);
            this.AddGameListener(UIGameEvent.CLICK_BUILD, this.onClickBuild, this);
            this.AddGameListener(UIGameEvent.AWARD_CAT_GIFT, this.onAwardCatGift, this);
            this.AddGameListener(UIGameEvent.BUILD_LEVEL_UP, this.onBuildLevelUp, this);
            this.AddGameListener(UIGameEvent.ADD_GUIDE_CAT, this.onAddGuideCat, this);
        }
        InitData(data) {
            super.InitData(data);
            this.c1.selectedIndex = 0;
            if (data instanceof GuideVO) {
                this.showGuide(data);
                this.onAddGuideCat();
            }
        }
        onAddGuideCat(cat = null) {
            let catId = GuideModel.Inst.GetRes(1).catId || 10001;
            let catVo = CatModel.Inst.GetCatVOById(catId);
            if (this.guideVO && this.guideVO.guideRes &&
                this.guideVO.guideRes.id >= GuideModel.GUID_GETGIFT_START_ID && this.guideVO.guideRes.id <= GuideModel.GUID_GETGIFT_END_ID) {
                if (cat == null) {
                    cat = EntityModel.Inst.GetCat(catVo.objId);
                }
                if (cat) {
                    let guideRes = GuideModel.Inst.levelGiftGuideRes;
                    let pos = new Laya.Vector3(1.25, 1.269, -25.15);
                    CatGiftCtrl.Inst.AddItemGift(guideRes.propID, guideRes.propnum, cat.catVO.objId, pos);
                    if (this.guideVO.isLeaveGift) {
                        GuideCtrl.Inst.compelteGuide(this.guideVO);
                        return;
                    }
                }
            }
            if (this.guideVO && this.guideVO.isClickCat) {
                CameraCtrl.Inst.MoveCat(this.catVo || catVo);
            }
        }
        onGuideComplete(guidVO) {
            if (!guidVO)
                return;
            if (guidVO.isClickCat) {
                GameDispatcher.Inst.event(UIGameEvent.CAT_ADD_OR_REMOVE_ICON, [this.cat, false, 3]);
            }
        }
        onGuideAllComplete() {
            if (GuideModel.Inst.CheckGuideAll()) {
                this.close();
                UISystem.Inst.CreateWindowView(UITipView, "已完成所有引导");
            }
        }
        showGuide(guideVO) {
            if (!guideVO)
                return;
            if (guideVO.tempId == 0 || guideVO.guideRes == null)
                return;
            if (guideVO.guideRes.task == 53 && LandModel.Inst.level == 1) {
                return;
            }
            this._guideVO = guideVO;
            this.catVo = CatModel.Inst.GetGuideCatVo();
            this.showGuideTime = Laya.timer.currTimer;
            if (guideVO.isLeaveGift && CatGiftModel.Inst.leaveGiftVO != null) {
                this.compelteGuide();
                return;
            }
            if (guideVO.guideRes.task > 0 && TaskModel.Inst.GetTaskIsComplete(guideVO.guideRes.task)) {
                this.compelteGuide();
                return;
            }
            this.hideGuiding();
            this.bg.visible = this.bg.touchable = guideVO.isForce;
            if (guideVO.guideRes.text) {
                let y = parseInt(guideVO.guideRes.point[1]);
                y = isNaN(y) ? 0 : y;
                this.showGuideTip(guideVO.guideRes.text, guideVO.guideRes.stayTime, y);
            }
            else {
                this.group_guide_tip.visible = false;
            }
            Laya.timer.clear(this, this.checkTips);
            if (guideVO.isTips) {
                Laya.timer.frameLoop(1, this, this.checkTips, [guideVO]);
                return;
            }
            if (guideVO.isClickButton) {
                let panel = guideVO.guideRes.panel;
                let btn = panel ? panel[panel.length - 1] : "";
                let view = UISystem.Inst.GetWindowByName(panel[0]);
                if (view == null && btn == "btn_close") {
                    this.compelteGuide();
                    return;
                }
                let buildVo = guideVO.guideRes.buildId > 0 && BuildModel.Inst.GetBuildVOBy(guideVO.guideRes.buildId);
                if (panel.join("|") == "UIBuildCreateView|btn_create" && buildVo && buildVo.IsUnLock()) {
                    this.compelteGuide();
                    return;
                }
            }
            if (guideVO.guideRes.id == GuideModel.GUID_INVITE_CAT2_ID) {
                let catVO = CatModel.Inst.GetCatVOById(guideVO.guideRes.catId);
                if (catVO && catVO.IsHas()) {
                    this.compelteGuide();
                    return;
                }
            }
            if (guideVO.isShowCircle) {
                this.showCircle(guideVO.guideRes);
                this.updateXY(guideVO);
                Laya.timer.clear(this, this.updateXY);
                Laya.timer.frameLoop(1, this, this.updateXY, [guideVO]);
            }
            else {
                Laya.timer.clear(this, this.updateXY);
            }
            if (guideVO.isPlay) {
                SceneMgr.Inst.GetCameraBuildScript().OverSelected();
                let stayTime = guideVO.guideRes.stayTime > 0 ? guideVO.guideRes.stayTime : 500;
                Laya.timer.once(stayTime, this, this.compelteGuide);
            }
            else if (guideVO.isLevelBuild) {
                Laya.timer.frameLoop(1, this, this.checkBuildLevel);
            }
            else if (guideVO.isClickCat) {
                if (UISystem.Inst.HasView(UICatInviteAlert) || UISystem.Inst.HasView(UICatDetailsView)) {
                    if (guideVO.guideRes.catId != 2) {
                        this.compelteGuide();
                    }
                }
                let catVo = CatModel.Inst.GetGuideCatVo();
                CameraCtrl.Inst.MoveCat(this.catVo || catVo);
                this.dontTouchabe();
            }
            if (guideVO.page == 15 && guideVO.step == 1) {
                CameraCtrl.Inst.MoveCat(this.catVo);
            }
            if (guideVO.isClickBuild || guideVO.isLevelBuild) {
                let buildVo = BuildModel.Inst.GetBuildVOBy(guideVO.guideRes.buildId);
                CameraCtrl.Inst.MoveBuild(buildVo);
                this.dontTouchabe();
            }
        }
        dontTouchabe() {
            this.touchable = false;
            this.isThrough = false;
            fgui.GTween.delayedCall(1).setTarget(this).onComplete(() => {
                if (!this.touchable) {
                    this.touchable = true;
                    this.isThrough = true;
                }
            }, this);
        }
        checkBuildLevel() {
            if (this.guideVO && this.guideVO.isLevelBuild) {
                let buildVO = BuildModel.Inst.GetBuildVOBy(this.guideVO.guideRes.buildId);
                if (buildVO && buildVO.level >= this.guideVO.guideRes.level) {
                    this.compelteGuide();
                }
            }
            else {
                Laya.timer.clear(this, this.checkBuildLevel);
            }
        }
        onBuildLevelUp(buildVo) {
            if (this.guideVO && this.guideVO.guideRes && this.guideVO.guideRes.level > 0 &&
                buildVo.buildRes.id == this.guideVO.guideRes.buildId &&
                buildVo.level >= this.guideVO.guideRes.level) {
                this.compelteGuide();
            }
        }
        showCircle(guideRes) {
            this.c1.selectedIndex = 2;
            this.spine_circle.Reset();
            if (guideRes.showGuideAni == 1) {
                this.spine_circle.play("finger");
            }
            else if (guideRes.showGuideAni == 2) {
                this.spine_circle.play("circle");
            }
            else if (guideRes.showGuideAni == 3) {
                this.spine_circle.play("touch");
            }
        }
        checkTips(guidVO) {
            if (UISystem.Inst.HasView(UIAwardView)) {
                this.hideGuide();
                return;
            }
            if (guidVO.isTips) {
                this.c1.selectedIndex = 1;
                this.group_guide_tip.visible = true;
            }
        }
        updateXY(guideVO) {
            if (!guideVO || guideVO.guideRes == null)
                return;
            if (guideVO.guideRes.page == 24 && LandModel.Inst.level <= 1) {
                this.hideGuide();
                return;
            }
            if (guideVO.isClickCat) {
                let catVO = CatModel.Inst.GetCatVOById(guideVO.guideRes.catId);
                this.cat = catVO && EntityModel.Inst.GetCat(catVO.objId);
                if (this.cat == null && guideVO.guideRes.catId == 1) {
                    if (GuideModel.Inst.canStrokeCatVo == null) {
                        let level = 0;
                        if (guideVO.guideRes.page == 22) {
                            level = 10;
                        }
                        GuideModel.Inst.canStrokeCatVo = CatModel.Inst.GetOneCanStrokeCat(level);
                    }
                    this.catVo = GuideModel.Inst.canStrokeCatVo || this.catVo;
                    this.cat = GuideModel.Inst.canStrokeCatVo && EntityModel.Inst.GetCat(GuideModel.Inst.canStrokeCatVo.objId);
                }
            }
            if (UISystem.Inst.HasView(UIAwardView) || UISystem.Inst.HasView(UIGiveExpressView)) {
                this.hideGuide();
                return;
            }
            let gapTime = Laya.timer.currTimer - this.showGuideTime;
            if (gapTime < 800 && (guideVO.isClickBuild || guideVO.isClickCat) && !guideVO.isClickGift) {
                this.hideGuide();
                this.touchable = false;
                this.isThrough = false;
                return;
            }
            if (!this.touchable) {
                this.touchable = true;
                this.isThrough = true;
            }
            if (guideVO.isShowCircle) {
                this.c1.selectedIndex = 2;
            }
            if (guideVO.isClickButton) {
                let panel = guideVO.guideRes.panel;
                let ps = panel.join("|");
                let btn = this.getButton(panel);
                if (btn == null) {
                    this.hideGuide();
                    return;
                }
                if (ps == "UIBuildView|sp_maowu|sp_lvup|btn_lv" && BuildModel.Inst.maowuBuildVO
                    && BuildModel.Inst.maowuBuildVO.IsMaxLv()) {
                    this.compelteGuide();
                    return;
                }
                else if (ps == "UIShoppingView|sp_shoppanel|sp_manghe1") {
                    if (UISystem.Inst.HasView(UIStoreContentView)) {
                        this.hideGuide();
                        return;
                    }
                    let gp = btn.localToGlobal(btn.width * 0.5, btn.height * 0.5);
                    let rect = this.getButton(["UIShoppingView", "sp_shoppanel"]);
                    if (!ObjectUtils.globalIsInRange(gp, null, rect)) {
                        this.hideGuide();
                        return;
                    }
                }
                else if (ps == "UICatDetailsView|eglist_posture@0") {
                    let view = UISystem.Inst.GetWindowView(UICatDetailsView);
                    let catVo = view && view.catVo;
                    if (catVo && catVo.level < this.guideVO.guideRes.level) {
                        this.hideGuide();
                        return;
                    }
                }
                else if (ps == "UICatDetailsView|eglist_zishi@0") {
                    if (btn.parent == null || !btn.parent.internalVisible) {
                        this.hideGuide();
                        return;
                    }
                }
                if (panel[0] == "UIBuildView") {
                    let p2 = panel[1];
                    let buildView = UISystem.Inst.GetWindowView(UIBuildView);
                    let buildVO = buildView ? buildView.GetBuildVO() : null;
                    if (buildVO == null) {
                        this.hideGuide();
                        return;
                    }
                    else if (p2 == "sp_maowu" && !buildVO.isMaowo) {
                        this.hideGuide();
                        return;
                    }
                    else if (p2 == "sp_sheshi" && !buildVO.isSheshi) {
                        this.hideGuide();
                        return;
                    }
                    else if (p2 == "sp_matou" && !buildVO.isMatou) {
                        this.hideGuide();
                        return;
                    }
                    else if (p2 == "sp_banjia" && !buildVO.isJingYuBanjia) {
                        this.hideGuide();
                        return;
                    }
                }
                else if (panel[0] == "UISkinView") {
                    if (ps == "UISkinView|eglist_items@0" && UISystem.Inst.HasView(UISkinBagView)) {
                        this.hideGuide();
                        return;
                    }
                }
                let isAllVisible = btn.alpha == 1 && btn.visible && btn.internalVisible && btn.internalVisible2 && btn.internalVisible3;
                if (btn instanceof fgui.GButton && (!isAllVisible || !btn.touchable)) {
                    this.hideGuide();
                    return;
                }
                if (btn instanceof UITaskAwardItem && btn.name == "item_main" && !btn.canAward) {
                    this.hideGuide();
                    return;
                }
                if (panel[0] == "UIMainView") {
                    if (this.guideVO.isForce) {
                        UISystem.Inst.removeAllFromMain([LayerType.TYPE_AD, LayerType.TYPE_DIVERSION]);
                        CameraCtrl.Inst.OverSelected();
                    }
                    else if (UISystem.Inst.hasAnyViewOnMain([this])) {
                        this.hideGuide();
                        return;
                    }
                }
                if (btn && isAllVisible) {
                    let gp = btn.localToGlobal(0, 0);
                    if (!btn.pivotAsAnchor) {
                        gp.x += btn.width * 0.5;
                        gp.y += btn.height * 0.5;
                    }
                    this.updateCircleXY(guideVO, gp.x, gp.y, btn);
                }
                else {
                    this.hideGuide();
                }
            }
            else if (guideVO.isClickBuild) {
                let mainView = UISystem.Inst.GetWindowView(UIMainView);
                if (mainView && !mainView.IsCanTouch()) {
                    return;
                }
                if (this.guideVO.isForce) {
                    let buildView = UISystem.Inst.GetWindowView(UIBuildView);
                    let buildVO = buildView && buildView.GetBuildVO();
                    if (buildVO && buildVO.buildRes.id == this.guideVO.guideRes.buildId) {
                        this.compelteGuide();
                        return;
                    }
                    else {
                        UISystem.Inst.removeAllFromMain([LayerType.TYPE_AD, LayerType.TYPE_DIVERSION]);
                        CameraCtrl.Inst.OverSelected();
                    }
                }
                else if (UISystem.Inst.hasAnyViewOnMain([this])) {
                    let topView = UISystem.Inst.getTopWindow([UIGuideView]);
                    if (topView instanceof UIBuildCreateView && topView.buildVo && topView.buildVo.buildRes &&
                        topView.buildVo.buildRes.id == this.guideVO.guideRes.buildId) {
                        this.compelteGuide();
                        return;
                    }
                    this.hideGuide();
                    return;
                }
                if (guideVO.guideRes.propID > 0) {
                    this.bg.touchable = false;
                    let catId = GuideModel.Inst.GetRes(1).catId || 10001;
                    let catVo = CatModel.Inst.GetCatVOById(catId);
                    let catGift = catVo && CatGiftModel.Inst.GetGiftByCatObjId(catVo.objId);
                    if (catGift) {
                        let v2 = MathUtil.GetPointFrom3DWorld(SceneMgr.Inst.GetCamera(), catGift.owner.transform.position);
                        this.updateCircleXY(guideVO, v2.x, v2.y);
                    }
                    return;
                }
                let buildVo = BuildModel.Inst.GetBuildVOBy(guideVO.guideRes.buildId);
                if (buildVo) {
                    if (!buildVo.IsUnLock()) {
                        if (UISceneIconView.canUnlockBuild && UISceneIconView.canUnlockBuild.buildVO == buildVo) {
                            this.updateCircleXY(guideVO, UISceneIconView.canUnlockBuild.x, UISceneIconView.canUnlockBuild.y - 150);
                        }
                    }
                    else {
                        let build = EntityModel.Inst.GetBuild(buildVo.objId);
                        if (build) {
                            let pos = build.buildVO.buildPos;
                            let v2 = MathUtil.GetPointFrom3DWorld(SceneMgr.Inst.GetCamera(), pos);
                            this.updateCircleXY(guideVO, v2.x, v2.y);
                        }
                    }
                }
                else {
                    this.hideGuide();
                }
            }
            else if (guideVO.isClickCat) {
                if (guideVO.guideRes.catId == 2) {
                    if (UISystem.Inst.HasView(UICatDetailsView)) {
                    }
                    else {
                        this.hideGuide();
                        return;
                    }
                }
                else if (UISystem.Inst.hasAnyViewOnMain([UIGuideView, UITipView, UILoadingLeafView])) {
                    this.hideGuide();
                    return;
                }
                if (this.guideVO.isForce) {
                    UISystem.Inst.removeAllFromMain([LayerType.TYPE_AD, LayerType.TYPE_DIVERSION]);
                    CameraCtrl.Inst.OverSelected();
                }
                else if (UISystem.Inst.hasAnyViewOnMain([this])) {
                    if (guideVO.guideRes.catId == 2) {
                    }
                    else {
                        this.hideGuide();
                        return;
                    }
                }
                if (guideVO.page == 3 && !GuideModel.Inst.endWeiguanDiaoyuchang) {
                    this.hideGuide();
                    return;
                }
                this.bg.touchable = false;
                let catVO = CatModel.Inst.GetCatVOById(guideVO.guideRes.catId);
                this.cat = catVO && EntityModel.Inst.GetCat(catVO.objId);
                if (this.cat == null && guideVO.guideRes.catId == 1) {
                    if (GuideModel.Inst.canStrokeCatVo == null) {
                        let level = 0;
                        if (guideVO.guideRes.page == 22) {
                            level = 10;
                        }
                        GuideModel.Inst.canStrokeCatVo = CatModel.Inst.GetOneCanStrokeCat(level);
                    }
                    this.catVo = GuideModel.Inst.canStrokeCatVo || this.catVo;
                    this.cat = GuideModel.Inst.canStrokeCatVo && EntityModel.Inst.GetCat(GuideModel.Inst.canStrokeCatVo.objId);
                }
                else if (guideVO.guideRes.catId == 2) {
                    let catVo = CatModel.Inst.selectCatVO || GuideModel.Inst.canStrokeCatVo;
                    if (catVo == null) {
                        if (guideVO.guideRes.page == 22) {
                            catVo = CatModel.Inst.GetCatVOByLevel(10) || CatModel.Inst.GetGuideCatVo();
                        }
                        else {
                            catVo = CatModel.Inst.GetGuideCatVo();
                        }
                    }
                    if (catVo) {
                        this.cat = EntityModel.Inst.GetCat(catVo.objId);
                        if (CatModel.Inst.selectCatVO == null) {
                        }
                    }
                }
                if (this.cat) {
                    GameDispatcher.Inst.event(UIGameEvent.CAT_ADD_OR_REMOVE_ICON, [this.cat, true, 3]);
                    let v2 = MathUtil.GetPointFrom3DWorld(SceneMgr.Inst.GetCamera(), this.cat.owner.transform.position);
                    this.updateCircleXY(guideVO, v2.x, v2.y);
                }
                else {
                    this.hideGuide();
                }
            }
        }
        updateCircleXY(guideVO, x, y, btn = null) {
            if (guideVO.guideRes.text) {
                let y = parseInt(guideVO.guideRes.point[1]);
                y = isNaN(y) ? 0 : y;
                this.showGuideTip(guideVO.guideRes.text, guideVO.guideRes.stayTime, y);
            }
            else {
                this.group_guide_tip.visible = false;
            }
            x = Math.ceil(x);
            y = Math.ceil(y);
            this.bg.visible = this.bg.touchable = guideVO.isForce;
            if (guideVO.isClickCat) {
                this.bg.touchable = false;
            }
            let offsetX = parseInt(guideVO.guideRes.offseXY[0]) || 0;
            let offsetY = parseInt(guideVO.guideRes.offseXY[1]) || 0;
            if (!this.spine_circle.isShow) {
                this.showCircle(guideVO.guideRes);
            }
            let px = x + offsetX;
            let py = y + offsetY;
            px = px < 0 ? 0 : (px > 1000 ? 1000 : px);
            py = py < 0 ? 0 : (py > 1800 ? 1800 : py);
            this.spine_circle.setXY(px, py);
            this.img_circle.setXY(this.spine_circle.x, this.spine_circle.y);
            if (btn instanceof fgui.GObject) {
                this.img_circle.setSize(btn.width, btn.height);
            }
            else {
                let clickSize = guideVO.guideRes.clickSize;
                this.img_circle.setSize((parseInt(clickSize[0]) || 80), (parseInt(clickSize[1]) || 60));
            }
            this.msk.setSize(this.img_circle.width, this.img_circle.height);
            this.msk.setXY(this.spine_circle.x, this.spine_circle.y);
        }
        hideGuide() {
            if (this.guideVO && this.guideVO.guideRes && this.guideVO.isClickButton && this.guideVO.guideRes.text && this.spine_circle.visible) {
                this.group_guide_tip.visible = true;
            }
            else {
                this.group_guide_tip.visible = false;
            }
            this.bg.touchable = this.bg.visible = false;
            this.spine_circle.Reset();
        }
        showGuideTip(text, stayTime = 0, y = 0) {
            this.c1.selectedIndex = this.guideVO.isTips ? 1 : 2;
            this.txt_gtips.text = text;
            if (y > 0) {
                this.group_guide_tip.y = y;
            }
            if (!this.group_guide_tip.visible) {
                this.group_guide_tip.visible = true;
                this.t0.play(Laya.Handler.create(this, () => {
                    if (stayTime > 0) {
                        Laya.timer.once(stayTime, this, this.endTipGuide);
                    }
                }));
            }
        }
        endTipGuide() {
            this.hideGuideTip();
            this.compelteGuide();
        }
        hideGuideTip() {
            Laya.timer.clear(this, this.hideGuideTip);
            this.t0.playReverse(Laya.Handler.create(this, () => {
                this.group_guide_tip.visible = false;
                this.c1.selectedIndex = 0;
            }));
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
        }
        getButton(panel) {
            let panelName = panel[0];
            let btnName = panel[1];
            let index;
            let window = UISystem.Inst.GetWindowByName(panelName);
            let btn;
            for (let i = 1; i < panel.length; i++) {
                index = -1;
                btnName = panel[i];
                if (btnName.indexOf("@") != -1) {
                    let arr = btnName.split("@");
                    btnName = arr[0];
                    index = parseInt(arr[1]);
                }
                if (i == 1) {
                    btn = window ? window[btnName] : null;
                }
                else {
                    btn = btn && btn.getChild(btnName);
                }
                if (index != -1) {
                    btn = btn && index < btn.numChildren && btn.getChildAt(index);
                }
                if (!btn) {
                    break;
                }
            }
            if (this.guideVO && this.guideVO.isFunc && btn instanceof UICatHeadItem && btn.catVO && btn.catVO.IsVisitor) {
                let list = btn.parent;
                let index2 = 0;
                if (list instanceof fgui.GList) {
                    for (let i = 0; i < list.numChildren; i++) {
                        btn = list.getChildAt(i);
                        if (btn instanceof UICatHeadItem && btn.catVO && btn.catVO.IsHas()) {
                            if (index == index2) {
                                break;
                            }
                            index2++;
                        }
                    }
                }
            }
            return btn;
        }
        onClickCat(catVO) {
            if (this.guideVO && catVO && this.guideVO.isClickCat && this.guideVO.guideRes.catId == catVO.catRes.id) {
                this.compelteGuide();
            }
        }
        onClickBuild(buildVO) {
            if (this.guideVO && this.guideVO.isClickBuild && this.guideVO.guideRes.buildId == buildVO.buildRes.id) {
                this.compelteGuide();
            }
        }
        onAwardCatGift(giftVo) {
            if (this.guideVO && this.guideVO.guideRes && this.guideVO.guideRes.type == GuideVO.TYPE_CLICK_BUILD && this.guideVO.guideRes.propID == giftVo.itemId) {
                this.compelteGuide();
            }
        }
        onClickStage(e) {
            if (Laya.timer.currTimer - this.showGuideTime < 200) {
                return;
            }
            let isInCircle = this.img_circle.visible && ObjectUtils.globalIsIn(this.img_circle);
            if (this.guideVO) {
                if (this.guideVO.isLeaveGift) {
                    return;
                }
                if (this.guideVO.isShowCircle && !this.spine_circle.visible) {
                    return;
                }
                if (this.guideVO.isClickButton && this.guideVO.guideRes.buildId > 0 && this.guideVO.guideRes.level > 0) {
                    let buildVO = BuildModel.Inst.GetBuildVOBy(this.guideVO.guideRes.buildId);
                    if (buildVO && buildVO.level < this.guideVO.guideRes.level) {
                        return;
                    }
                }
                if (!this.guideVO.isForce) {
                    if (this.guideVO.isTips) {
                        if (this.group_guide_tip.visible) {
                            this.compelteGuide();
                            return;
                        }
                    }
                    if (this.guideVO.isShowCircle && !isInCircle) {
                        return;
                    }
                    else {
                        this.compelteGuide();
                    }
                    return;
                }
                if (this.guideVO.isClickButton) {
                    if (!isInCircle && this.guideVO.isForce) {
                        this.showGuiding();
                        return;
                    }
                    let panel = this.guideVO.guideRes.panel;
                    let btn = this.getButton(panel);
                    if (btn) {
                        if (ObjectUtils.globalIsIn(btn)) {
                            this.compelteGuide();
                        }
                        else {
                        }
                    }
                    else {
                        this.compelteGuide();
                    }
                }
                else if (this.guideVO.isForce) {
                    if (isInCircle) {
                        this.compelteGuide();
                    }
                    else {
                        this.showGuiding();
                    }
                }
            }
        }
        compelteGuide() {
            this.spine_circle.Reset();
            this.c1.selectedIndex = 0;
            GuideCtrl.Inst.compelteGuide(this.guideVO);
            Laya.timer.clear(this, this.checkBuildLevel);
            Laya.timer.clear(this, this.endTipGuide);
            this.hideGuiding();
        }
        get guideVO() {
            return this._guideVO;
        }
        showGuiding() {
            if (GuideModel.Inst.IsCanShowBtns()) {
                return;
            }
            this.img_guiding.visible = true;
            this.img_guiding.alpha = 0;
            fgui.GTween.to(this.img_guiding.alpha, 1, 0.3).setTarget(this.img_guiding, "alpha").onComplete(() => {
                fgui.GTween.delayedCall(1).setTarget(this.img_guiding).onComplete(() => {
                    this.hideGuiding();
                }, this);
            }, this);
        }
        hideGuiding() {
            fgui.GTween.kill(this.img_guiding);
            fgui.GTween.to(this.img_guiding.alpha, 0, 0.3).setTarget(this.img_guiding, "alpha").onComplete(() => {
                this.img_guiding.visible = false;
            }, this);
        }
        Reset() {
            super.Reset();
            Laya.timer.clear(this, this.updateXY);
            Laya.timer.clearAll(this);
            fgui.GTween.kill(this);
            fgui.GTween.kill(this.img_guiding);
            this.img_guiding.visible = false;
            this._guideVO = null;
        }
    }

    class GuideCtrl {
        constructor() {
        }
        static get Inst() {
            return this.inst = this.inst || new GuideCtrl();
        }
        init() {
            GuideModel.Inst.init();
            if (TaskModel.Inst.mainTaskVo) {
                this.startFuncGuide(TaskModel.Inst.mainTaskVo.tempId);
            }
            GameDispatcher.Inst.on(UIGameEvent.UPDATE_TASK, this, this.onUpdateTask);
        }
        onUpdateTask(type) {
            if (type == 1 && TaskModel.Inst.mainTask && TaskModel.Inst.mainTask.isNormal) {
                let mainTaskVO = TaskModel.Inst.mainTask;
                if (mainTaskVO) {
                    this.startFuncGuide(mainTaskVO.tempId);
                }
            }
        }
        startFuncGuide(taskId) {
            this.skipMainGuide(taskId);
            GuideModel.Inst.canStrokeCatVo = null;
            GuideModel.Inst.setFuncGuide(taskId);
            GameDispatcher.Inst.event(UIGameEvent.GUIDE_SHOW, GuideModel.Inst.funGuideVO);
        }
        skipMainGuide(taskId) {
            GuideModel.Inst.skipMainGuide(taskId);
            this.showMainGuide();
        }
        start() {
            let mainView = UISystem.Inst.GetWindowView(UIMainView);
            if (!GuideModel.Inst.isOpenGuide || GuideModel.Inst.CheckGuideAll()) {
                mainView.start();
                return;
            }
            if (GuideModel.Inst.IsNewer()) {
                let catId = GuideModel.Inst.guideVO.guideRes.catId || 10001;
                CatCtrl.Inst.AddGuildCat(catId, 28, Laya.Handler.create(this, this.endFirst));
                if (MoneyModel.gold.isLessThanOrEqualTo(0)) {
                    let startGemNum = ConfigValue.getItemByKey("startGemNum").value;
                    MoneyModel.gold.add(startGemNum);
                }
            }
            else {
                mainView.start();
                let catId = GuideModel.Inst.GetRes(1).catId || 10001;
                if (GuideModel.Inst.NeedGuideCat()) {
                    CatCtrl.Inst.AddGuildCat(catId, 0, Laya.Handler.create(this, this.addGuideCat));
                }
                else {
                }
            }
            if (!GuideModel.Inst.CheckGuideAll()) {
                if (GuideModel.Inst.guideVO) {
                    UISystem.Inst.CreateWindowView(UIGuideView, GuideModel.Inst.guideVO);
                }
                else {
                    UISystem.Inst.CreateWindowView(UIGuideView, GuideModel.Inst.funGuideVO);
                }
            }
            LoadQueue.Inst.loadUI(UrlUtils.UI + "task", null, false);
            LoadQueue.Inst.loadUI(UrlUtils.UI + "cat", null, false);
            LoadQueue.Inst.loadUI(UrlUtils.UI + "build", null, false);
        }
        addGuideCat(cat) {
            GameDispatcher.Inst.event(UIGameEvent.ADD_GUIDE_CAT, cat);
        }
        endFirst(cat) {
            this.compelteGuide(GuideModel.Inst.guideVO);
        }
        showMainGuide() {
            if (GuideModel.Inst.CheckGuideAll()) {
                UISystem.Inst.RemoveWindowClass(UIGuideView);
                GameDispatcher.Inst.event(UIGameEvent.GUIDE_COMPELTE_ALL);
                console.log("-----------------------所有引导结束---------------------------");
            }
            else if (GuideModel.Inst.guideVO) {
                GameDispatcher.Inst.event(UIGameEvent.GUIDE_SHOW, GuideModel.Inst.guideVO);
            }
            else {
                GameDispatcher.Inst.event(UIGameEvent.MAIN_GUIDE_COMPELTE_ALL);
            }
        }
        compelteGuide(guideVO) {
            if (!guideVO || guideVO.guideRes == null) {
                return;
            }
            PlatformMgr.Inst.aldSendEvent(EventType.xishouyindao, "ID_" + guideVO.guideRes.id);
            if (guideVO.isSign) {
                GuideModel.Inst.completeFuncGuide(guideVO.tempId);
                GameDispatcher.Inst.event(UIGameEvent.GUIDE_COMPELTE, guideVO);
                GuideModel.Inst.doNextSignGuide();
                this.showSignGuide();
                return;
            }
            if (guideVO.isMain) {
                StorageUtils.Inst.SetGuideCompelte(guideVO.tempId);
                GameDispatcher.Inst.event(UIGameEvent.GUIDE_COMPELTE, guideVO);
                GuideModel.Inst.doNext();
                this.showMainGuide();
            }
            else if (guideVO.isFunc) {
                GuideModel.Inst.completeFuncGuide(guideVO.tempId);
                GameDispatcher.Inst.event(UIGameEvent.GUIDE_COMPELTE, guideVO);
                GuideModel.Inst.doNextFuncGuide();
                if (GuideModel.Inst.CheckGuideAll()) {
                    UISystem.Inst.RemoveWindowClass(UIGuideView);
                    GameDispatcher.Inst.event(UIGameEvent.GUIDE_COMPELTE_ALL);
                    console.log("-----------------------所有引导结束---------------------------");
                }
                else if (GuideModel.Inst.funGuideVO != null) {
                    GameDispatcher.Inst.event(UIGameEvent.GUIDE_SHOW, GuideModel.Inst.funGuideVO);
                }
            }
        }
        showSignGuide() {
            if (GuideModel.Inst.CheckGuideAll()) {
                UISystem.Inst.RemoveWindowClass(UIGuideView);
                GameDispatcher.Inst.event(UIGameEvent.GUIDE_COMPELTE_ALL);
            }
            else if (GuideModel.Inst.signGuideVO) {
                GameDispatcher.Inst.event(UIGameEvent.GUIDE_SHOW, GuideModel.Inst.signGuideVO);
            }
            else if (GuideModel.Inst.guideVO) {
                GameDispatcher.Inst.event(UIGameEvent.GUIDE_SHOW, GuideModel.Inst.guideVO);
            }
            else if (GuideModel.Inst.funGuideVO) {
                GameDispatcher.Inst.event(UIGameEvent.GUIDE_SHOW, GuideModel.Inst.funGuideVO);
            }
        }
    }
    GuideCtrl.inst = null;

    class CatCtrl {
        constructor() {
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new CatCtrl();
            return this._Inst;
        }
        SetCatMaxCount(count) {
            CatModel.Inst.SetMaxCount(count);
            let num = 0;
            let cats = CatModel.Inst.GetAllCatVOs(1);
            for (let i = 0; i < cats.length; i++) {
                let catVO = cats[i];
                if (catVO.IsHas()) {
                    num++;
                    if (num <= count) {
                        let cat = EntityModel.Inst.GetCat(catVO.objId);
                        if (cat != null)
                            cat.Show();
                    }
                    else {
                        let cat = EntityModel.Inst.GetCat(catVO.objId);
                        if (cat != null) {
                            cat.Hide();
                        }
                    }
                }
            }
        }
        AddGuildCat(catId = 10001, eventId = 28, handler = null) {
            let catVO = CatModel.Inst.GetCatVOById(catId);
            let cat = EntityModel.Inst.GetCat(catVO.objId);
            if (cat != null) {
                this.onAddComplete(eventId, handler, false, cat);
                return;
            }
            if (!catVO.IsHas()) {
                catVO.IsGuide = true;
            }
            MapQuadTree.Inst.AddVO(catVO, true, Laya.Handler.create(this, this.onAddComplete, [eventId, handler, true]));
        }
        onAddComplete(eventId, handler, isXiuxian, cat) {
            if (eventId > 0)
                cat.SetStateMachine(cat.getStateMachine().guildEvent, [eventId, handler]);
            else {
                if (isXiuxian)
                    cat.SetStateMachine(cat.getStateMachine().xiuxianEvent);
                if (handler) {
                    handler.runWith(cat);
                }
            }
        }
        InviteCat(catVO) {
            if (catVO.IsVisitor) {
                this.RemoveCat(catVO, true);
                catVO = CatModel.Inst.GetCatVOById(catVO.catRes.id);
            }
            let toysNum = ItemModel.Inst.GetItemCount(catVO.catRes.item);
            if (toysNum >= catVO.catRes.itemNum) {
                ItemCtrl.Inst.RemoveItemVO(catVO.catRes.item, catVO.catRes.itemNum);
                this.AddCat(catVO, true);
            }
        }
        AddCat(catVO, isAni = true) {
            if (!catVO)
                return;
            PlatformMgr.Inst.aldSendEvent(EventType.maomijiesuo, catVO.catRes.id + "_" + catVO.catRes.name);
            if (catVO.IsGuide) {
                EntityModel.Inst.RemoveLastChuan();
            }
            catVO.isShow = !CatModel.Inst.IsHasMax();
            if (isAni && catVO.isShow) {
                UISystem.showLoadingLeaf();
            }
            catVO.level += 1;
            catVO.IsGuide = false;
            CatModel.Inst.AddCatVO();
            let cat = EntityModel.Inst.GetCat(catVO.objId);
            if (cat != null) {
                this.onAddCatComplete(isAni, cat);
            }
            else {
                MapQuadTree.Inst.AddVO(catVO, true, Laya.Handler.create(this, this.onAddCatComplete, [isAni]));
            }
            GameDispatcher.Inst.event(UIGameEvent.CAT_UNLOCK, catVO);
            CatEventCtrl.Inst.UpdateCheckTime();
        }
        onAddCatComplete(isAni, cat) {
            UISystem.hideLoadingLeaf();
            if (isAni)
                cat.SetStateMachine(cat.getStateMachine().createEvent);
        }
        RemoveCat(catVO, isRemoveChuan = false) {
            if (catVO.IsVisitor) {
                CatModel.Inst.RemoveVisitorCatVO(catVO);
                if (isRemoveChuan)
                    EntityModel.Inst.RemoveLastChuan();
            }
            MapQuadTree.Inst.RemoveVO(catVO);
            GameDispatcher.Inst.event(UIGameEvent.REMOVE_CAT, catVO);
        }
        ChangeName(catVo, name) {
            if (catVo && catVo.catName != name) {
                catVo.catName = name;
                GameDispatcher.Inst.event(UIGameEvent.CAT_CHANGE_NAME, catVo);
            }
        }
        ChangeSkin(objId, parts) {
            let cat = EntityModel.Inst.GetCat(objId);
            if (cat == null)
                return;
            cat.ChangeSkin(parts);
        }
        ClickCatOnMainView(vo) {
            if (GuideModel.Inst.guideVO && GuideModel.Inst.guideVO.isClickBuild) {
                return;
            }
            if (vo.IsVisitor || vo.IsGuide) {
                UISystem.Inst.CreateWindowView(UICatInviteAlert, vo);
            }
            else {
                UISystem.Inst.CreateWindowView(UICatDetailsView, vo);
            }
            GameDispatcher.Inst.event(UIGameEvent.CLICK_CAT, vo);
        }
        EnterLuMaoOnMainView(vo) {
            if (GuideModel.Inst.guideVO && GuideModel.Inst.guideVO.isClickBuild) {
                return;
            }
            let bol = GuideModel.Inst.CheckGuideAll();
            if (vo.IsVisitor || vo.IsGuide) {
                UISystem.Inst.CreateWindowView(UICatInviteAlert, [vo, bol]);
            }
            else {
                UISystem.Inst.CreateWindowView(UICatDetailsView, [vo, bol]);
            }
            GameDispatcher.Inst.event(UIGameEvent.CLICK_CAT, vo);
        }
        ClickCat(catVO) {
            let cat = EntityModel.Inst.GetCat(catVO.objId);
            if (cat == null)
                return;
            if (catVO.catStrokeVo.IsBusy())
                return;
            if (!catVO.catStrokeVo.IsStroked())
                cat.ClickCat();
        }
        StopLumao(catVO) {
            if (catVO == null) {
                return;
            }
            let cat = EntityModel.Inst.GetCat(catVO.objId);
            if (cat == null)
                return;
            if (catVO.catStrokeVo.IsReady() || catVO.catStrokeVo.IsRefuse() || catVO.catStrokeVo.IsStroking()) {
                this.SetCatNormalState(cat);
            }
            cat.StopLumao();
        }
        SetCanLumao(cat, value) {
            if (cat.catVO.IsVisitor)
                return;
            if (value) {
                GameDispatcher.Inst.event(UIGameEvent.CAT_ADD_OR_REMOVE_ICON, [cat, true, 0]);
                if (cat.catVO.isForciblylumao)
                    return;
                cat.catVO.showIconEndTime = Laya.timer.currTimer + CatModel.Inst.headtime;
                CatModel.Inst.AddShowIconCount();
            }
            else {
                GameDispatcher.Inst.event(UIGameEvent.CAT_ADD_OR_REMOVE_ICON, [cat, false]);
                if (!cat.catVO.isForciblylumao)
                    return;
                cat.catVO.showIconEndTime = 0;
                CatModel.Inst.RemoveShowIconCount();
            }
            cat.catVO.isForciblylumao = value;
        }
        SetCatNormalState(cat) {
            if (cat == null || cat.catVO == null)
                return;
            if (cat.catVO.IsGuide)
                return;
            if (!cat.catVO.catStrokeVo.IsOverTime())
                return;
            if (!cat.catVO.catStrokeVo.IsRefuseOver())
                return;
            CatCtrl.Inst.OnChangeState(cat.catVO, CatStrokeVO.CAT_STATE_NORMAL);
            if (CatModel.Inst.IsMaxShowIcon()) {
                if (!cat.catVO.isForciblylumao)
                    this.SetCanLumao(cat, false);
            }
            else {
                this.SetCanLumao(cat, true);
            }
        }
        OverLumao(catVO) {
            let mainView = UISystem.Inst.GetWindowView(UIMainView);
            let from = new Laya.Point(Laya.stage.width * 0.5, Laya.stage.height * 0.5);
            let to = mainView.GoldPoint;
            let total = CatModel.Inst.GetTouchGoldAward();
            MoneyModel.gold.add(total);
            EffectMgr.Inst.playGoldEffect(total, from, to);
            GameDispatcher.Inst.event(UIGameEvent.CAT_STROKING_COMPLETE, catVO);
            CatCtrl.Inst.OnChangeState(catVO, CatStrokeVO.CAT_STATE_STROKED);
            PlatformMgr.Inst.aldSendEvent(EventType.momao);
        }
        OnChangeState(catVO, state, param = null) {
            if (catVO) {
                catVO.OnChangeState(state, param);
                GameDispatcher.Inst.event(UIGameEvent.CAT_STATE_STROKE_CHANGE, [catVO, param]);
            }
        }
        ChangePose(catVO, poseId) {
            let cat = EntityModel.Inst.GetCat(catVO.objId);
            if (cat == null)
                return;
            if (cat.getStateMachine().GetCurState() != cat.getStateMachine().lumaoEvent)
                return;
            let actRes = catActResource.getItemByKey(poseId);
            if (actRes == null)
                return;
            cat.CrossFadeState(actRes.actName, Laya.Handler.create(this, this.onPlayPoseComplete, [cat]));
        }
        onPlayPoseComplete(cat) {
            cat.PlayState("idle_wait");
        }
        CatPlayBuildReward(cat, buildVO) {
            let guideVO = GuideModel.Inst.guideVO;
            if (cat.catVO.IsGuide && CatGiftModel.Inst.leaveGiftVO == null) {
                let pos = cat.owner.transform.position;
                pos = new Laya.Vector3(0.371, 1.582, -24.56);
                if (guideVO && guideVO.guideRes.buildId == buildVO.buildRes.id && guideVO.isLeaveGift) {
                    CatGiftModel.Inst.leaveGiftVO = CatGiftCtrl.Inst.AddItemGift(guideVO.guideRes.propID, guideVO.guideRes.propnum, cat.catVO.objId, pos);
                    GuideCtrl.Inst.compelteGuide(GuideModel.Inst.guideVO);
                }
                else {
                    let res = GuideModel.Inst.levelGiftGuideRes;
                    CatGiftModel.Inst.leaveGiftVO = CatGiftCtrl.Inst.AddItemGift(res.propID, res.propnum, cat.catVO.objId, pos);
                }
            }
        }
        GetNearCat(positon) {
            let cats = EntityModel.Inst.GetCats();
            let onDis = Number.MAX_VALUE;
            let onCat = null;
            for (const key in cats) {
                let cat = cats[key];
                if (cat != null && cat.IsCanPlayBuild()) {
                    let curDis = Laya.Vector3.distanceSquared(positon, cat.transform.position);
                    if (curDis < onDis) {
                        onDis = curDis;
                        onCat = cat;
                    }
                }
            }
            return onCat;
        }
        SetCatWg(buildVO, handler = null) {
            let createEventIds = buildVO.buildRes.createEventIds;
            if (createEventIds.length <= 0)
                return;
            for (let i = 0; i < createEventIds.length; i++) {
                let cat = this.GetNearCat(buildVO.buildPos);
                if (cat == null)
                    return;
                cat.SetStateMachine(cat.getStateMachine().wgEvent, [createEventIds[i], handler]);
            }
        }
    }
    CatCtrl._Inst = null;

    class catUpResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint8();
            this.lvl = byteArray.readUint8();
            this.addon = byteArray.readUint16();
            this.itemNum = byteArray.readUint8();
        }
        static getItemByKey(id) {
            return catUpResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return catUpResource.itemsDic[key1 + "," + key2];
        }
    }
    catUpResource.items = new Array();
    catUpResource.itemsDic = new Dictionary();

    class CatFecalBallModel {
        constructor() {
            this.maxCount = 10;
            this.ballGold = 0;
            this.livetime = 0;
            this.losttime = 4000;
            this.spacingtime = 0;
            this.HighHairball = 0;
            this.HighHairballAdvertisement = 0;
            this.HighHairballmultiple = 0;
            this.fecalMap = {};
            this.fecalTimeMap = {};
            this._count = 0;
            this.ballGold = parseInt(ConfigValue.getItemByKey("ballGold").value);
            this.maxCount = parseInt(ConfigValue.getItemByKey("keeptime").value);
            this.livetime = parseInt(ConfigValue.getItemByKey("livetime").value) * 1000;
            this.losttime = parseInt(ConfigValue.getItemByKey("losttime").value) * 1000;
            this.spacingtime = parseInt(ConfigValue.getItemByKey("spacingtime").value) * 1000;
            this.HighHairball = parseInt(ConfigValue.getItemByKey("HighHairball").value);
            this.HighHairballAdvertisement = parseInt(ConfigValue.getItemByKey("HighHairballAdvertisement").value);
            this.HighHairballmultiple = parseInt(ConfigValue.getItemByKey("HighHairballmultiple").value);
        }
        static get Inst() {
            return this.inst = this.inst || new CatFecalBallModel();
        }
        init() {
            let cats = StorageUtils.Inst.GetCats();
            let objId;
            let cat;
            for (objId in cats) {
                cat = cats[objId];
                if (cat) {
                    let time = cat[StorageUtils.cat_fecal];
                    if (time > 0) {
                        this.fecalTimeMap[objId] = time;
                    }
                }
            }
        }
        AddFecal(objId, entity) {
            this._count++;
            this.fecalMap[objId] = entity;
            let lstime = StorageUtils.getCurTimer();
            this.fecalTimeMap[objId] = lstime;
            StorageUtils.Inst.SetCatFecalTime(lstime, objId);
        }
        RemoveFecal(objId) {
            this._count--;
            delete this.fecalMap[objId];
        }
        getPrevTime(objId) {
            let time = this.fecalTimeMap[objId] || 0;
            return time;
        }
        GetFecal(objId) {
            return this.fecalMap[objId];
        }
        GetFecals() {
            return this.fecalMap;
        }
        GetCount() {
            return this._count;
        }
        GetFecalGoldAward() {
            let num = BuildModel.Inst.GetAllBuildGoldRate(this.ballGold, false);
            return num;
        }
        IsFull() {
            let num = this.GetCount();
            return num >= this.maxCount;
        }
    }
    CatFecalBallModel.inst = null;

    class CatVO extends BaseVO {
        constructor() {
            super();
            this._catRes = null;
            this._level = 0;
            this._loveLv = 1;
            this.catStrokeVo = null;
            this._catName = "";
            this._ownedChuruCount = 0;
            this._wornSkinIds = [];
            this.clothMap = {};
            this.skinVoList = [];
            this.poseVoList = [];
            this.qinmiLvlAwardList = [];
            this._IsVisitor = false;
            this._IsGuide = false;
            this._isXiuXian = false;
            this.isShow = true;
            this.isForciblyShow = false;
            this.showIconEndTime = 0;
            this.isForciblylumao = false;
            this.isMainViewSelect = false;
            this.type = BaseVO.CAT;
            this.catStrokeVo = new CatStrokeVO();
        }
        get isCanStroke() {
            return this.catStrokeVo.IsNormal() && this.catStrokeVo.IsOverTime();
        }
        set catRes(value) {
            this._catRes = value;
            this.resName = this._catRes.res;
            this._catName = StorageUtils.Inst.GetCatName(value.id);
            this._level = StorageUtils.Inst.GetCatLv(value.id);
            this._loveLv = StorageUtils.Inst.GetCatLoveLv(value.id);
            this._ownedChuruCount = StorageUtils.Inst.GetChuruCount(value.id);
            this.getUnlockPoseList();
            this.getSkinVoList();
            this.qinmiLvlAwardList = StorageUtils.Inst.GetQinmiAwardList(value.id);
        }
        getSkinVoList() {
            this.skinVoList.length = 0;
            this._wornSkinIds.length = 0;
            let clothList = StorageUtils.Inst.GetClothList(this.catRes.id);
            let skinVO = null;
            for (let i = 0; i < clothList.length; i++) {
                skinVO = new SkinVO();
                skinVO.decode(clothList[i]);
                this.skinVoList.push(skinVO);
                this.clothMap[skinVO.tempId] = skinVO;
                if (skinVO.isWear) {
                    this._wornSkinIds.push(skinVO.tempId);
                }
            }
        }
        getUnlockPoseList() {
            let unlockedPoseList = StorageUtils.Inst.GetPoseList(this.catRes.id);
            this.poseVoList.length = 0;
            let posVo = null;
            for (let i = 0; i < this._catRes.actId.length; i++) {
                let actId = parseInt(this._catRes.actId[i]);
                let acunlockType = parseInt(this._catRes.acunlockType[i]);
                let acunlockNum = parseInt(this._catRes.acunlockNum[i]);
                posVo = new CatPoseVO(actId);
                posVo.parent = this;
                if (acunlockType == 1) {
                    posVo.conditionVo = new ConditionVO(ConditionVO.CAT_LEVEL, 0, acunlockNum);
                }
                else if (acunlockType == 2) {
                    posVo.conditionVo = new ConditionVO(ConditionVO.AD_VIEW, 0, acunlockNum);
                    posVo.conditionVo.count = StorageUtils.Inst.getPoseAdCount(this.catRes.id, actId);
                }
                else if (acunlockType == 3) {
                    posVo.conditionVo = new ConditionVO(ConditionVO.ITEM, ItemModel.ITEM_GEM, acunlockNum);
                }
                posVo.isUnlock = unlockedPoseList.indexOf(actId) != -1;
                this.poseVoList.push(posVo);
            }
        }
        get wornSkinIds() {
            return this._wornSkinIds;
        }
        get catRes() {
            return this._catRes;
        }
        get catUpRes() {
            return catUpResource.getItemByKey(this.level);
        }
        get nextCatUpRes() {
            return catUpResource.getItemByKey(this.level + 1);
        }
        get catLoveRes() {
            return catloveResource.getItemBy2Keys(this.catRes.quality, this.loveLv);
        }
        set catName(value) {
            this._catName = value;
            StorageUtils.Inst.SetCatName(value, this.catRes.id);
        }
        get catName() {
            return this._catName || this.catRes.name;
        }
        set isXiuXian(value) {
            this._isXiuXian = value;
        }
        get isXiuXian() {
            return this._isXiuXian;
        }
        addSkin(tempId) {
            let vo = null;
            if (tempId instanceof SkinVO) {
                vo = this.getSkinVO(tempId.tempId);
                if (!vo && this.skinVoList.indexOf(tempId) == -1) {
                    this.skinVoList.push(tempId);
                    this.clothMap[tempId.tempId] = tempId;
                    this.saveClothList();
                    GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [TaskType.TASKTYPE_GIVEDRESS, tempId.tempId]);
                }
            }
            else {
                vo = this.getSkinVO(tempId);
                if (vo == null) {
                    vo = new SkinVO();
                    vo.tempId = tempId;
                    vo.count = 1;
                    this.skinVoList.push(vo);
                    this.clothMap[tempId] = vo;
                    this.saveClothList();
                    GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [TaskType.TASKTYPE_GIVEDRESS, tempId]);
                }
                else {
                }
            }
        }
        removeCloth(tempId) {
            let vo = this.getSkinVO(tempId);
            let index = this.skinVoList.indexOf(vo);
            if (vo && index != -1) {
                ArrayUtil.RemoveAt(this.skinVoList, index);
                delete this.clothMap[tempId];
                this.saveClothList();
            }
        }
        wearCloth(skinId) {
            let vo = this.getSkinVO(skinId);
            if (vo) {
                for (let i = 0; i < this._wornSkinIds.length; i++) {
                    let skinId = this._wornSkinIds[i];
                    let skinRes = skinResource.getItemByKey(skinId);
                    if (skinRes && skinRes.icon == vo.skinRes.icon) {
                        this.takeOffCloth(skinId);
                        break;
                    }
                }
                vo.isWear = true;
                this.saveClothList();
                let index = this._wornSkinIds.indexOf(vo.tempId);
                if (index == -1) {
                    this._wornSkinIds.push(vo.tempId);
                    GameDispatcher.Inst.event(UIGameEvent.DAYTASK_COMP_PROP, TaskVO.SONGPIFU);
                    GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [TaskType.TASKTYPE_DRESSUP, vo.skinRes.id]);
                }
                CatCtrl.Inst.ChangeSkin(this.objId, this.wornSkinIds);
                GameDispatcher.Inst.event(UIGameEvent.CAT_WEAR_CLOTH, [this, skinId]);
            }
        }
        takeOffCloth(skinId) {
            let vo = this.getSkinVO(skinId);
            if (vo && vo.isWear) {
                vo.isWear = false;
                this.saveClothList();
                let index = this._wornSkinIds.indexOf(vo.tempId);
                if (index != -1)
                    this._wornSkinIds.splice(index, 1);
                CatCtrl.Inst.ChangeSkin(this.objId, this.wornSkinIds);
                GameDispatcher.Inst.event(UIGameEvent.CAT_TAKEOFF_CLOTH, [this, skinId]);
            }
        }
        HasWear(skinId) {
            let vo = this.getSkinVO(skinId);
            return vo && vo.isWear;
        }
        GetSkinSuitOwnedCount(suitId) {
            let count = 0;
            let vo;
            let suitRes = suitResource.getItemByKey(suitId);
            for (let i = 0; i < suitRes.skinIds.length; i++) {
                let skinId = suitRes.skinIds[i];
                vo = this.getSkinVO(skinId);
                if (vo && vo.count > 0) {
                    count++;
                }
            }
            return count;
        }
        HasSkin(tempId) {
            let vo = this.getSkinVO(tempId);
            return vo && vo.count > 0;
        }
        GetAllSkinVos(part = 0) {
            let arr = [];
            let skinVo = null;
            for (let i = 0; i < this.skinVoList.length; i++) {
                skinVo = this.skinVoList[i];
                if (skinVo && !skinVo.isSuit) {
                    if (part == 0) {
                        arr.push(skinVo);
                    }
                    else if (skinVo.part == part) {
                        arr.push(skinVo);
                    }
                }
            }
            return arr;
        }
        GetAllSuitVos() {
            let arr = [];
            let skinVo = null;
            for (let i = 0; i < this.skinVoList.length; i++) {
                skinVo = this.skinVoList[i];
                if (skinVo && skinVo.isSuit) {
                    arr.push(skinVo);
                }
            }
            return arr;
        }
        getWearClothList() {
            let arr = [];
            let vo;
            for (let i = 0; i < this.skinVoList.length; i++) {
                vo = this.skinVoList[i];
                if (vo && vo.isWear) {
                    arr.push(vo);
                }
            }
            return arr;
        }
        getSkinVO(tempId) {
            return this.clothMap[tempId];
        }
        saveClothList() {
            let arr = [];
            let vo;
            for (let i = 0; i < this.skinVoList.length; i++) {
                vo = this.skinVoList[i];
                if (vo) {
                    arr.push(vo.toJson());
                }
            }
            StorageUtils.Inst.SetClothList(arr, this.catRes.id);
        }
        set ownedChuruCount(value) {
            this._ownedChuruCount = value;
            if (!this.canLevelLove && this._ownedChuruCount >= this.getLoveItemLimit(this.level)) {
                this.levelUp();
            }
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.giveChuru));
            StorageUtils.Inst.SetChuruCount(value, this.catRes.id);
        }
        get ownedChuruCount() {
            return this._ownedChuruCount;
        }
        levelUp() {
            this.level++;
            this.ownedChuruCount = 0;
        }
        get isLevelLimit() {
            return this.nextCatLoveRes == null;
        }
        checkLvUnlockPose() {
            let poseVo;
            for (let i = 0; i < this.poseVoList.length; i++) {
                poseVo = this.poseVoList[i];
                if (poseVo && !poseVo.isUnlock && poseVo.conditionVo && poseVo.conditionVo.isCatLevel) {
                    poseVo.conditionVo.count = this.level;
                }
            }
        }
        unlockPose(tempId) {
            let poseVo = null;
            let unlockedPoseList = [];
            let needSave = false;
            for (let i = 0; i < this.poseVoList.length; i++) {
                poseVo = this.poseVoList[i];
                if (poseVo && poseVo.tempId == tempId && !poseVo.isUnlock && poseVo.canUnlock) {
                    poseVo.isUnlock = true;
                    needSave = true;
                }
                if (poseVo.isUnlock) {
                    unlockedPoseList.push(poseVo.tempId);
                }
            }
            if (needSave) {
                StorageUtils.Inst.SetPoseList(unlockedPoseList, this.catRes.id);
                GameDispatcher.Inst.event(UIGameEvent.UNLOCK_POSE, [tempId, this.catRes.id]);
            }
        }
        getUnlockPoseVoList() {
            let arr = [];
            let poseVo = null;
            for (let i = 0; i < this.poseVoList.length; i++) {
                poseVo = this.poseVoList[i];
                if (poseVo && poseVo.isUnlock) {
                    arr.push(poseVo);
                }
            }
            return arr;
        }
        set level(value) {
            if (this._level == value)
                return;
            this._level = value;
            if (this.level > 1) {
                GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [TaskType.TASKTYPE_UP_CAT, this.catRes.id]);
                GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [TaskType.TASKTYPE_CATLEVEL, this.catRes.id]);
            }
            else if (this.level == 1) {
                GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [TaskType.TASKTYPE_INVITE_CAT, this.catRes.id]);
            }
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.catLevelup));
            StorageUtils.Inst.SetCatLv(value, this._catRes.id);
        }
        get level() {
            return this._level;
        }
        set loveLv(value) {
            if (this._loveLv == value)
                return;
            this._loveLv = value;
            StorageUtils.Inst.SetCatLoveLv(value, this._catRes.id);
            GameDispatcher.Inst.event(UIGameEvent.CAT_UPDATE_QINMI, this);
        }
        get loveLv() {
            return this._loveLv;
        }
        get canLevelLove() {
            if (this._level == 20 && this.loveLv == 1) {
                return true;
            }
            else if (this._level == 30 && this.loveLv == 2) {
                return true;
            }
            else if (this._level == 40 && this.loveLv == 3) {
                return true;
            }
            else if (this._level == 50 && this.loveLv == 4) {
                return true;
            }
            return false;
        }
        getLoveItemLimit(lv = NaN) {
            lv = isNaN(lv) ? this.level : lv;
            let lvRes = catUpResource.getItemByKey(lv);
            return lvRes ? lvRes.itemNum : 0;
        }
        getLoveLvDesc(lv = NaN) {
            lv = isNaN(lv) ? this.loveLv : lv;
            let res = catloveResource.getItemBy2Keys(this.catRes.quality, lv);
            return res ? res.lvlName : "";
        }
        get nextLoveLv() {
            return this.loveLv + 1;
        }
        get nextCatLoveRes() {
            return catloveResource.getItemBy2Keys(this.catRes.quality, this.nextLoveLv);
        }
        awardQinmiAward(lv) {
            if (this.qinmiLvlAwardList.indexOf(lv) == -1) {
                this.qinmiLvlAwardList.push(lv);
                StorageUtils.Inst.SetQinmiAwardList(this.qinmiLvlAwardList, this.catRes.id);
            }
        }
        hasAwardQinmi(lv) {
            return this.qinmiLvlAwardList.indexOf(lv) != -1;
        }
        canAwardQinmi(lv2) {
            let bol = this.loveLv >= lv2;
            return bol && !this.hasAwardQinmi(lv2);
        }
        get loveGoldAddRate() {
            let c1 = this.catLoveRes ? this.catLoveRes.addon : 0;
            let c2 = this.catUpRes ? this.catUpRes.addon : 0;
            return c1 + c2;
        }
        get clothGoldAddRate() {
            let wearCloths = this.skinVoList;
            return SkinModel.GetClothGoldAddRate(wearCloths);
        }
        get totalGoldAddRate() {
            return this.loveGoldAddRate + this.clothGoldAddRate;
        }
        IsHas() {
            return this._level > 0;
        }
        get IsVisitor() {
            return this._IsVisitor;
        }
        set IsVisitor(value) {
            this._IsVisitor = value;
        }
        get IsGuide() {
            return this._IsGuide && !this.IsHas();
        }
        set IsGuide(value) {
            this._IsGuide = value;
        }
        get quaIsN() {
            return this.catRes.quality == CatVO.QUA_N;
        }
        get quaIsR() {
            return this.catRes.quality == CatVO.QUA_R;
        }
        get quaIsS() {
            return this.catRes.quality == CatVO.QUA_S;
        }
        CheckItemCount() {
            let itemVO = ItemModel.Inst.GetItemVOById(this.catRes.item);
            return itemVO && itemVO.count >= this.catRes.itemNum;
        }
        CheckMaowuCount() {
            if (CatModel.Inst.GetLastMaowuCount() < 1)
                return false;
            return true;
        }
        NeedOtherConndition() {
            return parseInt(this.catRes.unlockType[1]) > 0;
        }
        GetOtherScore() {
            let scoreType = parseInt(this.catRes.unlockType[1]);
            let score = 0;
            if (scoreType == 1) {
                score = BuildModel.Inst.GetBuildScore(BuildVO.TYPE_XIUXIANSHESHI);
            }
            else if (scoreType == 2) {
                score = BuildModel.Inst.GetBuildScore(BuildVO.TYPE_CANYINSHESHI);
            }
            else if (scoreType == 3) {
                score = BuildModel.Inst.GetBuildScore(BuildVO.TYPE_HUANGJINGSHESHI);
            }
            else if (scoreType == 4) {
                score = BuildModel.Inst.GetBuildScore(BuildVO.TYPE_YULESHESHI);
            }
            else if (scoreType == 5) {
                score = BuildModel.Inst.GetBuildScore(BuildVO.TYPE_CANYINSHESHI);
            }
            return score;
        }
        CheckOtherCondition() {
            let type = parseInt(this.catRes.unlockType[0]);
            let viceType = parseInt(this.catRes.unlockType[1]);
            let need = parseInt(this.catRes.unlockType[2]);
            if (type == 1) {
                return this.GetOtherScore() >= need;
            }
            else if (type == 2) {
                let buildVo = BuildModel.Inst.GetBuildVOBy(viceType);
                return buildVo && buildVo.level >= need;
            }
            return false;
        }
        get isCanInvite() {
            if (this.IsHas()) {
                return 0;
            }
            let count = 0;
            if (this.CheckItemCount()) {
                count += 2;
            }
            if (this.CheckMaowuCount()) {
                count++;
            }
            if (!this.NeedOtherConndition()) {
                count++;
            }
            else if (this.NeedOtherConndition() && this.CheckOtherCondition()) {
                count++;
            }
            return count;
        }
        CanInvite() {
            if (this.IsHas()) {
                return false;
            }
            if (!this.CheckItemCount()) {
                return false;
            }
            if (!this.CheckMaowuCount()) {
                return false;
            }
            if (this.NeedOtherConndition() && !this.CheckOtherCondition()) {
                return false;
            }
            return true;
        }
        get HeadIconUrl() {
            return PanelRegister.getUrl("icon_cat_head", this.catRes.icon);
        }
        get CatIconUrl() {
            return PanelRegister.getUrl("icon_pose", this.catRes.icon);
        }
        OnChangeState(state, param) {
            return this.catStrokeVo && this.catStrokeVo.OnChangeState(state, param);
        }
        CanFecal() {
            let prevTime = CatFecalBallModel.Inst.getPrevTime(this.objId);
            let currTimer = StorageUtils.getCurTimer();
            return this.isXiuXian && (prevTime == 0 || (currTimer - prevTime) >= CatFecalBallModel.Inst.spacingtime);
        }
        IsOverShowIcon() {
            return this.showIconEndTime > 0 && Laya.timer.currTimer > this.showIconEndTime;
        }
    }
    CatVO.QUA_N = 1;
    CatVO.QUA_R = 2;
    CatVO.QUA_S = 3;

    class UIAwardItem extends TListItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
        }
        InitData(data) {
            super.InitData(data);
            if (data instanceof ExchangeVO) {
                let iconUrl;
                if (data.itemRes.type == ItemModel.TYPE_BUILD) {
                    iconUrl = PanelRegister.getUrl("icon_award_part", data.itemRes.icon);
                    this.setItemIcon(iconUrl);
                }
                else {
                    iconUrl = data.itemUrl;
                    this.setItemIcon(iconUrl);
                }
                this.txt_count.text = "x" + data.count;
                this.c1.selectedIndex = data.itemRes.quality;
                let catRes = data.catRes;
                if (catRes) {
                    this.c2.selectedIndex = 1;
                    if (this.img_cat) {
                        let url = PanelRegister.getUrl("icon_cat_head", catRes.icon);
                        this.setCatIcon(url);
                    }
                }
                else {
                    this.c2.selectedIndex = 0;
                }
            }
            else if (data instanceof AwardVO) {
                if (data.IsItem && data.itemRes.type == ItemModel.TYPE_BUILD) {
                    let partUrl = PanelRegister.getUrl("icon_award_part", data.itemRes.icon);
                    this.setItemIcon(partUrl);
                }
                else {
                    this.setItemIcon(data.itemUrl);
                }
                if (this.img_cat) {
                    this.setCatIcon(data.catHeadUrl);
                }
                this.txt_count.text = "x" + data.getValueString();
                this.c1.selectedIndex = data.itemRes.quality;
                this.c2.selectedIndex = 0;
                if (data.IsItem) {
                    if (data.catRes) {
                        this.c2.selectedIndex = 1;
                    }
                }
                else if (data.IsCat) {
                    this.c2.selectedIndex = 1;
                }
            }
            else if (data instanceof ItemVO) {
                if (data.itemRes.type == ItemModel.TYPE_BUILD) {
                    let partUrl = PanelRegister.getUrl("icon_award_part", data.itemRes.icon);
                    this.setItemIcon(partUrl);
                }
                else {
                    this.setItemIcon(data.itemUrl);
                }
                this.txt_count.text = "x" + MathUtil.goldFormat(data.count, 0);
                this.c1.selectedIndex = data.itemRes.quality;
                if (data.catRes) {
                    this.c2.selectedIndex = 1;
                    if (this.img_cat) {
                        this.setCatIcon(data.catHeadUrl);
                    }
                }
                else {
                    this.c2.selectedIndex = 0;
                }
            }
            else if (data instanceof SkinVO) {
                this.c1.selectedIndex = data.itemRes.quality;
                this.setItemIcon(data.iconUrl);
                this.txt_count.text = "";
            }
            else if (data instanceof CatVO) {
                this.c1.selectedIndex = data.catRes.quality;
                this.setItemIcon(data.HeadIconUrl);
                this.txt_count.text = "";
            }
            else if (data instanceof itemResource) {
                if (data.type == ItemModel.TYPE_BUILD) {
                    let partUrl = PanelRegister.getUrl("icon_award_part", data.icon);
                    this.setItemIcon(partUrl);
                }
                else {
                    this.setItemIcon(ItemVO.GetItemUrl(data));
                }
                this.txt_count.text = "x1";
                this.c1.selectedIndex = data.quality;
                let catRes = ItemModel.Inst.getCatRes(data.id);
                if (catRes) {
                    this.c2.selectedIndex = 1;
                    if (this.img_cat) {
                        let url = PanelRegister.getUrl("icon_cat_head", catRes.icon);
                        this.setCatIcon(url);
                    }
                }
                else {
                    this.c2.selectedIndex = 0;
                }
            }
            else if (data instanceof ShopLockDrawVO) {
                if (data.awardRes.type == ItemModel.TYPE_BUILD) {
                    let partUrl = PanelRegister.getUrl("icon_award_part", data.awardRes.icon);
                    this.setItemIcon(partUrl);
                }
                else {
                    this.setItemIcon(data.awardIconUrl);
                }
                this.txt_count.text = "x" + data.num;
                this.c1.selectedIndex = data.awardRes.quality;
                let catRes = ItemModel.Inst.getCatRes(data.awardId);
                if (catRes) {
                    this.c2.selectedIndex = 1;
                    if (this.img_cat) {
                        let url = PanelRegister.getUrl("icon_cat_head", catRes.icon);
                        this.setCatIcon(url);
                    }
                }
                else {
                    this.c2.selectedIndex = 0;
                }
            }
            else if (data instanceof CatPoseVO) {
                this.setItemIcon(data.iconUrl);
                this.txt_count.text = "x1";
                this.c1.selectedIndex = 0;
                this.c2.selectedIndex = 0;
            }
            else if (data instanceof catActResource) {
                let url = PanelRegister.getUrl("icon_pose", data.trigger);
                this.setItemIcon(url);
                this.txt_count.text = "x1";
                this.c1.selectedIndex = 0;
                this.c2.selectedIndex = 0;
            }
        }
        setItemIcon(url) {
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                this.img_icon.url = url;
            }));
        }
        setCatIcon(url) {
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                this.img_cat.icon = url;
            }));
        }
        Reset() {
            super.Reset();
        }
    }

    class UIAwardView extends UIBaseWindow {
        constructor() {
            super("award", "UIAwardView");
            this.awardArr = null;
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("award", "UIAwardItem", UIAwardItem);
            PanelRegister.registerClass("award", "UIAwardItem2", UIAwardItem);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_TOP;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowMaskBg = false;
            this.viewStruct.isEffect = false;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
            this.t0 = this._view.getTransition("t0");
            this.t0.stop();
            this.btn_close2.touchable = true;
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(Laya.Event.CLICK, this.onBtnCloseClick, this, this.btn_close2);
        }
        InitData(data) {
            UIAwardView.awardCount = data instanceof Array ? data.length : 1;
            this.btn_close.visible = UIAwardView.awardCount >= 10;
            this.btn_close2.visible = !this.btn_close.visible;
            UIAwardView.gotCount = 0;
            this.UpdateData(data);
            this.item_award.setScale(0, 0);
            this.hideBanner();
        }
        UpdateData(data) {
            super.InitData(data);
            this.group_desc.visible = false;
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.itemDrop));
            if (data instanceof Array) {
                if (!this.awardArr) {
                    this.awardArr = data.slice();
                }
                this.showSingle(data.pop());
            }
            else {
                this.showSingle(data);
            }
        }
        showSingle(data) {
            if (data instanceof ExchangeVO) {
                data = data.itemRes;
            }
            let isSuper = false;
            this.c1.selectedIndex = 0;
            this.item_award.InitData(data);
            if (data instanceof AwardVO) {
                if (data.IsItem) {
                    this.group_desc.visible = data.catRes != null;
                    if (data.catRes) {
                        this.txt_desc.text = `获得"${data.itemRes.name}"道具\n集齐可邀请"${data.catRes.name}"喵星人`;
                    }
                    isSuper = data.itemRes.quality >= 3;
                }
            }
            else if (data instanceof ItemVO) {
                this.group_desc.visible = data.catRes != null;
                this.txt_desc.text = data.itemRes.desc;
                if (data.catRes) {
                    this.txt_desc.text = `获得"${data.itemRes.name}"道具\n集齐可邀请"${data.catRes.name}"喵星人`;
                }
                isSuper = data.itemRes.quality >= 3;
            }
            else if (data instanceof itemResource) {
                let catRes = ItemModel.Inst.getCatRes(data.id);
                if (catRes) {
                    this.txt_desc.text = `获得"${data.name}"道具\n集齐可邀请"${catRes.name}"喵星人`;
                }
                isSuper = data.quality >= 3;
                this.txt_desc.text = data.desc;
                this.group_desc.visible = catRes != null && this.txt_desc.text != "";
            }
            else if (data instanceof ShopLockDrawVO) {
                this.group_desc.visible = true;
                let catRes = ItemModel.Inst.getCatRes(data.awardId);
                if (catRes) {
                    this.txt_desc.text = `获得"${data.awardRes.name}"道具\n集齐可邀请"${catRes.name}"喵星人`;
                }
                this.txt_desc.text = data.awardRes.desc;
                isSuper = data.awardRes.quality >= 3;
            }
            else if (data instanceof CatPoseVO || data instanceof catActResource) {
                this.group_desc.visible = true;
                this.txt_desc.text = "喵星人学会了新姿势!";
                isSuper = false;
            }
            if (isSuper) {
                this.spine_get_light.play("get_light2");
            }
            else {
                this.spine_get_light.play("get_light");
            }
            this.t0.play(null, 1);
            UIAwardView.gotCount++;
            if (UIAwardView.awardCount >= 10 && UIAwardView.gotCount == UIAwardView.awardCount) {
                if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                    let p1 = AdModel.Inst.getPolicy("p1");
                    if (p1 == "yes") {
                        this.btn_close2.visible = true;
                        this.showBanner();
                    }
                }
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
        }
        onBtnCloseClick() {
            let vos = this.Data instanceof Array ? this.Data.slice() : null;
            if (vos) {
                if (vos.length > 0) {
                    this.UpdateData(vos);
                    this.viewStruct.isEffect = false;
                    return;
                }
                else if (this.awardArr.length > 1 && this.c1.selectedIndex != 1) {
                    if (this.awardArr[0] instanceof ShopLockDrawVO) {
                        this.c1.selectedIndex = 1;
                        this.eglist_items.Data = this.awardArr;
                        let arr = this.eglist_items.items;
                        EffectUtil.showListEffect(arr, 0.4, 0.2);
                        this.spine_get_light.play("get_light");
                    }
                    else {
                        this.close();
                    }
                    return;
                }
                else {
                    this.awardArr = null;
                }
            }
            else {
                this.awardArr = null;
            }
            super.onBtnCloseClick();
        }
        Reset() {
            super.Reset();
            this.t0.stop();
        }
    }
    UIAwardView.awardVO = new AwardVO(0);
    UIAwardView.awardCount = 0;
    UIAwardView.gotCount = 0;

    class UIMoneyTipsView extends UIBaseWindow {
        constructor() {
            super("main", "UIMoneyTipsView");
            this.c1 = null;
            this.btn_get = null;
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = LayerType.LAYOUT_CENTER;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isEffect = true;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            this.c1.selectedIndex = data;
            this.showBanner();
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_get) {
                this.OnClickGetBtn();
            }
        }
        OnClickGetBtn() {
            if (this.c1.selectedIndex == 0) {
                let vos = ShopModel.Inst.GetShopVoByType(ShopVO.GETDIAMOND);
                if (vos.length > 0) {
                    if (!vos[0].canLookAd) {
                        return I18nUtils.ShowI18nById("gem_upper_limit");
                    }
                    else if (vos[0].isCd) {
                        return I18nUtils.ShowI18nById("matou_ad_cd");
                    }
                }
            }
            let handler = Laya.Handler.create(this, (success) => {
                if (success) {
                    let count = 0;
                    let tempId;
                    if (this.c1.selectedIndex == 0) {
                        count = parseInt(ConfigValue.getItemByKey("getgem").value);
                        tempId = MoneyVo.DIAMOND;
                        MoneyModel.diamond.add(count);
                        let shopVos = ShopModel.Inst.GetShopVoByType(ShopVO.GETDIAMOND);
                        if (shopVos.length > 0) {
                            shopVos[0].UpdateAdTime();
                            shopVos[0].curCount++;
                        }
                    }
                    else if (this.c1.selectedIndex == 1) {
                        count = parseInt(ConfigValue.getItemByKey("getgold").value);
                        tempId = MoneyVo.GOLD;
                        MoneyModel.gold.add(count);
                    }
                    UIAwardView.awardVO.setData({ "count": count, "tempId": tempId });
                    UISystem.Inst.CreateWindowView(UIAwardView, UIAwardView.awardVO);
                    this.close();
                }
            });
            PlatformMgr.Inst.showAd(AdType.ad_15, handler);
        }
        Reset() {
            super.Reset();
        }
    }

    class MoneyVo {
        constructor(type, num = 0) {
            this.type = MoneyVo.GOLD;
            this.updateHanler = null;
            this.type = type;
            if (num instanceof BigNumber) {
                this.num = num;
            }
            else {
                this.num = new BigNumber(num);
            }
        }
        setVal(num) {
            if (num instanceof BigNumber) {
                this.num = num;
            }
            else {
                this.num = new BigNumber(num);
            }
            this.updateMoney();
        }
        add(value) {
            this.num = this.num.plus(value);
            this.updateMoney();
        }
        cost(num, isTip = false, onlyTest = false) {
            if (this.num.isGreaterThanOrEqualTo(num)) {
                if (!onlyTest) {
                    this.num = this.num.minus(num);
                    this.updateMoney();
                }
                return true;
            }
            else {
                if (isTip) {
                    if (this.isDiamond) {
                        UISystem.Inst.CreateWindowView(UIMoneyTipsView, 0);
                    }
                    else if (this.isGold) {
                        UISystem.Inst.CreateWindowView(UIMoneyTipsView, 1);
                    }
                }
                return false;
            }
        }
        updateMoney() {
            this.updateHanler && this.updateHanler.runWith(this);
        }
        getValueString(sign = "") {
            let cnt = this.num;
            if (!(cnt instanceof BigNumber)) {
                cnt = new BigNumber(cnt);
            }
            return sign + MathUtil.goldFormat(cnt);
        }
        get name() {
            if (this.isDiamond) {
                return "钻石";
            }
            return "金币";
        }
        reset() {
            this.num = new BigNumber(0);
        }
        get iconUrl() {
            if (this.isDiamond) {
                return PanelRegister.getUrl("main", "Gem");
            }
            return PanelRegister.getUrl("main", "Gold");
        }
        get isDiamond() {
            return this.type == MoneyVo.DIAMOND;
        }
        get isGold() {
            return this.type == MoneyVo.GOLD;
        }
        get isAds() {
            return this.type == MoneyVo.ADS;
        }
        isGreaterThan(count) {
            return this.num.isGreaterThan(count);
        }
        isGreaterThanOrEqualTo(count) {
            return this.num.isGreaterThanOrEqualTo(count);
        }
        isLessThan(count) {
            return this.num.isLessThan(count);
        }
        isLessThanOrEqualTo(count) {
            return this.num.isLessThanOrEqualTo(count);
        }
    }
    MoneyVo.GOLD = 1;
    MoneyVo.DIAMOND = 2;
    MoneyVo.ADS = 3;
    MoneyVo.ARDER_SCORE = 4;
    MoneyVo.FOOD_SCORE = 5;
    MoneyVo.YULE_SCORE = 6;

    class MoneyModel {
        constructor() {
        }
        static get Inst() {
            return this.inst = this.inst || new MoneyModel();
        }
        init() {
            MoneyModel.gold.setVal(StorageUtils.Inst.GetGold());
            MoneyModel.diamond.setVal(StorageUtils.Inst.GetDiamond());
            MoneyModel.gold.updateHanler = Laya.Handler.create(this, this.updateMoney, null, false);
            MoneyModel.diamond.updateHanler = Laya.Handler.create(this, this.updateMoney, null, false);
        }
        updateMoney(moneyVo, isSave = true) {
            if (isSave) {
                if (moneyVo.isGold) {
                    StorageUtils.Inst.setGold(moneyVo.num);
                }
                else if (moneyVo.isDiamond) {
                    StorageUtils.Inst.setDiamond(moneyVo.num);
                }
            }
            GameDispatcher.Inst.event(UIGameEvent.UPDATE_CURRENCY_DATA, [moneyVo]);
        }
    }
    MoneyModel.inst = null;
    MoneyModel.gold = new MoneyVo(MoneyVo.GOLD);
    MoneyModel.diamond = new MoneyVo(MoneyVo.DIAMOND);

    class CatModel {
        constructor() {
            this.touchGold = 0;
            this.foodGold = 0;
            this.m_cats = new Array();
            this.m_catDic = new Dictionary();
            this.boatNChance = 0;
            this.boatRChance = 0;
            this.boatSChance = 0;
            this.boatStayTime = 0;
            this.touchTime = 0;
            this.catCount = 0;
            this.selectCatVO = null;
            this._maxCount = 0;
            this.showIconMaxCount = 5;
            this.curShowIconCount = 0;
            this.headtime = 0;
            this.catStrokeSpeed = 1.2;
            this.catStrokeSuccessCoe = 5000;
            this.ownedCatVos = null;
            this.curCatVo = null;
            this._befallIntervalTime = 0;
            this.boatNChance = parseFloat(ConfigValue.getItemByKey("boatNChance").value);
            this.boatRChance = parseFloat(ConfigValue.getItemByKey("boatRChance").value);
            this.boatSChance = parseFloat(ConfigValue.getItemByKey("boatSChance").value);
            this.boatStayTime = parseFloat(ConfigValue.getItemByKey("boatStayTime").value);
            this.touchTime = parseFloat(ConfigValue.getItemByKey("touchTime").value) * 1000;
            this.touchGold = parseInt(ConfigValue.getItemByKey("touchGold").value);
            this.foodGold = parseInt(ConfigValue.getItemByKey("foodGold").value);
            this.showIconMaxCount = parseInt(ConfigValue.getItemByKey("headnum").value);
            this.headtime = parseInt(ConfigValue.getItemByKey("headtime").value) * 1000;
            this.catStrokeSpeed = parseFloat(ConfigValue.getItemByKey("catStrokeSpeed").value);
            this._ChanceNum = parseInt(ConfigValue.getItemByKey("ChanceNum").value);
            this._AwaitTime = parseInt(ConfigValue.getItemByKey("AwaitTime").value);
            this.changeS = parseInt(ConfigValue.getItemByKey("ChanceS").value);
            this.changeR = parseInt(ConfigValue.getItemByKey("ChanceR").value);
            this.changeN = parseInt(ConfigValue.getItemByKey("ChanceN").value);
            this._befallCatCount = 0;
            let catVO = null;
            let length = catResource.items.length;
            if (this._maxCount == 0)
                this.SetMaxCount(length);
            for (let i = 0; i < length; i++) {
                catVO = new CatVO();
                catVO.catRes = catResource.items[i];
                this.m_cats.push(catVO);
                this.m_catDic[catVO.catRes.id] = catVO;
                if (catVO.IsHas()) {
                    catVO.isShow = !this.IsHasMax();
                    this.AddCatVO();
                }
            }
        }
        static getRate(num) {
            num = isNaN(num) ? 0 : num / 100;
            let str = num.toFixed(2);
            if (str.indexOf(".") != -1) {
                while (str.charAt(str.length - 1) == "0") {
                    str = str.substr(0, str.length - 1);
                    if (str.charAt(str.length - 1) == ".") {
                        str = str.replace(".", "");
                        break;
                    }
                }
            }
            return str + "%";
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new CatModel();
            return this._Inst;
        }
        AddShowIconCount() {
            this.curShowIconCount++;
        }
        RemoveShowIconCount() {
            this.curShowIconCount--;
        }
        IsMaxShowIcon() {
            return this.curShowIconCount >= this.showIconMaxCount;
        }
        SetMaxCount(count) {
            this._maxCount = count;
        }
        GetMaxCount() {
            return this._maxCount;
        }
        IsHasMax() {
            return this.catCount >= this.GetMaxCount();
        }
        AddVisitorCatVO(catId) {
            let newCatVO = new CatVO();
            newCatVO.catRes = catResource.getItemByKey(catId);
            newCatVO.IsVisitor = true;
            this.m_cats.push(newCatVO);
            this.m_catDic[this.GetVisitorCatId(catId)] = newCatVO;
            return newCatVO;
        }
        RemoveVisitorCatVO(catVO) {
            let index = this.m_cats.indexOf(catVO);
            if (index != -1) {
                ArrayUtil.RemoveAt(this.m_cats, index);
                delete this.m_catDic[this.GetVisitorCatId(catVO.catRes.id)];
            }
        }
        GetCatIdByVisitor(id) {
            return 100000 + id;
        }
        GetVisitorCatId(id) {
            return id - 100000;
        }
        GetCatVOById(id) {
            return this.m_catDic[id];
        }
        GetCatVOByLevel(level = 1) {
            let catVO = null;
            let key;
            for (key in this.m_catDic) {
                catVO = this.m_catDic[key];
                if (catVO && catVO.level >= level) {
                    return catVO;
                }
            }
            return null;
        }
        GetOneCanStrokeCat(level = 0) {
            if (this.ownedCatVos == null) {
                this.ownedCatVos = this.GetHasCatVOs();
            }
            let catVos = [];
            let catVO = null;
            if (this.ownedCatVos && this.ownedCatVos.length > 0) {
                for (let i = 0; i < this.ownedCatVos.length; i++) {
                    catVO = this.ownedCatVos[i];
                    if (level > 0 && catVO.level >= level) {
                        catVos.push(catVO);
                        continue;
                    }
                    if (catVO && catVO.isCanStroke) {
                        catVos.push(catVO);
                        continue;
                    }
                }
            }
            catVos = ArrayUtil.sortOn(catVos, [{ "level": "desc" }]);
            return catVos[0];
        }
        GetGuideCatVo() {
            let vo = this.GetCatVOById(10001);
            return vo;
        }
        HasCat(id) {
            let catVO = this.m_catDic[id];
            if (catVO == null)
                return false;
            return catVO.IsHas();
        }
        GetAllCatVOs(type = 0) {
            let cats = [];
            let cat_owned = [];
            let cat_vistor = [];
            let cat_lock = [];
            let cat_canunlock = [];
            let catVO;
            for (let i = 0; i < this.m_cats.length; i++) {
                catVO = this.m_cats[i];
                if (catVO.IsVisitor) {
                    cat_vistor.push(catVO);
                    continue;
                }
                if (catVO.IsHas()) {
                    cat_owned.push(catVO);
                }
                else {
                    cat_lock.push(catVO);
                    if (catVO.CheckItemCount()) {
                        cat_canunlock.push(catVO);
                    }
                }
            }
            if (type == 0) {
                cats = cat_owned.concat(cat_lock);
            }
            else if (type == 1) {
                cats = cat_owned;
            }
            else if (type == 2) {
                cats = cat_vistor;
            }
            else if (type == 3) {
                cats = cat_vistor.concat(cat_owned).concat(cat_canunlock);
            }
            else if (type == 4) {
                cats = cat_lock;
            }
            else if (type == 5) {
                cats = cat_vistor.concat(cat_owned);
            }
            cats = ArrayUtil.sortOn(cats, [{ "IsVisitor": "desc" }, { "level": "desc" }, { "isCanInvite": "desc" }, { "id": "asc" }]);
            return cats;
        }
        GetHasCatVOs() {
            return this.GetAllCatVOs(1);
        }
        getPrevCatVo(onlyOwned = true) {
            let index = 0;
            let catVos = onlyOwned ? this.GetAllCatVOs(5) : this.m_cats;
            if (this.curCatVo) {
                index = catVos.indexOf(this.curCatVo);
                if (index > 0) {
                    this.curCatVo = catVos[index - 1];
                }
                else if (catVos.length > 0) {
                    this.curCatVo = catVos[catVos.length - 1];
                }
            }
            else {
                this.curCatVo = catVos[index];
            }
            return this.curCatVo;
        }
        getNextCatVo(onlyOwned = true) {
            let index = 0;
            let catVos = onlyOwned ? this.GetAllCatVOs(5) : this.m_cats;
            if (this.curCatVo) {
                index = catVos.indexOf(this.curCatVo) + 1;
                if (index < catVos.length) {
                    this.curCatVo = catVos[index];
                }
                else {
                    this.curCatVo = catVos[0];
                }
            }
            else {
                this.curCatVo = catVos[index];
            }
            return this.curCatVo;
        }
        AddCatVO() {
            this.catCount++;
        }
        GetCatCount() {
            return this.catCount;
        }
        GetAllRate() {
            let num = 0;
            let catVos = this.GetHasCatVOs();
            let catVo = null;
            for (let i = 0; i < catVos.length; i++) {
                catVo = catVos[i];
                if (catVo) {
                    num += catVo.totalGoldAddRate;
                }
            }
            return num;
        }
        GetRandomCatVO() {
            let rateAll = this.boatNChance + this.boatRChance + this.boatSChance;
            let jv = MathUtil.makeRandom(0, rateAll);
            let curJv = this.boatNChance;
            let quality = 0;
            if (jv <= rateAll) {
                quality = CatVO.QUA_S;
            }
            rateAll -= this.boatSChance;
            if (jv <= rateAll) {
                quality = CatVO.QUA_R;
            }
            rateAll -= this.boatRChance;
            if (jv <= rateAll) {
                quality = CatVO.QUA_N;
            }
            if (MatouModel.Inst.inviteQuaSCount < MatouModel.Inst.maxInviteSCount) {
                quality = CatVO.QUA_S;
                MatouModel.Inst.inviteQuaSCount++;
            }
            let catArr = [];
            for (const catVO of this.m_cats) {
                if (catVO.catRes.quality == quality) {
                    catArr.push(catVO);
                }
            }
            return catArr[MathUtil.makeRandom(0, catArr.length)];
        }
        GetLastMaowuCount() {
            let buildVO = BuildModel.Inst.maowuBuildVO;
            if (buildVO == null)
                return 0;
            let curBaseRes = baseResource.getItemByKey(buildVO.level);
            if (curBaseRes == null)
                return 0;
            let catCount = this.GetCatCount();
            return curBaseRes.catMax - catCount;
        }
        GetTouchGoldAward() {
            let num;
            let all = BuildModel.Inst.GetAllBuildGoldRate(1, false);
            if (all instanceof BigNumber) {
                num = all.multipliedBy(this.touchGold);
            }
            else {
                num = new BigNumber(all);
                num = num.multipliedBy(this.touchGold);
            }
            return num;
        }
        addFoodGold() {
            let award = this.GetFoodGoldAward();
            MoneyModel.gold.add(award);
            return award;
        }
        GetFoodGoldAward() {
            let num;
            let all = BuildModel.Inst.GetAllBuildGoldRate(1, false);
            if (all instanceof BigNumber) {
                num = all.multipliedBy(this.foodGold);
            }
            else {
                num = new BigNumber(all);
                num = num.multipliedBy(this.foodGold);
            }
            return num;
        }
        GetOwnedRandomOne() {
            let catVos = this.GetHasCatVOs();
            let index = MathUtil.makeRandom(0, catVos.length);
            return catVos[index];
        }
        CheckCanInvite() {
            let catvos = this.m_cats;
            for (let i = 0; i < catvos.length; i++) {
                let vo = catvos[i];
                if (vo && !vo.IsHas() && vo.CanInvite()) {
                    return true;
                }
            }
            return false;
        }
        get befallCatCount() {
            return this._befallCatCount;
        }
        set befallCatCount(val) {
            if (this.befallCatCount == val)
                return;
            this._befallCatCount = val;
            StorageUtils.Inst.befallCatCount = val;
        }
        get maxBefallCatCount() {
            return this._ChanceNum;
        }
        get befallIntervalTime() {
            return this._befallIntervalTime;
        }
        set befallIntervalTime(val) {
            this._befallIntervalTime = val;
        }
        get maxBefallIntervalTime() {
            return this._AwaitTime;
        }
        get canBefall() {
            return this.befallCatCount < this.maxBefallCatCount;
        }
        GetRandomBefallCatVO() {
            let changeS = this.changeS;
            let changeR = this.changeR;
            let changeN = this.changeN;
            let rateAll = changeN + changeR + changeS;
            let jv = MathUtil.makeRandom(0, rateAll);
            let quality = 0;
            if (jv <= rateAll) {
                quality = CatVO.QUA_S;
            }
            rateAll -= changeS;
            if (jv <= rateAll) {
                quality = CatVO.QUA_R;
            }
            rateAll -= changeR;
            if (jv <= rateAll) {
                quality = CatVO.QUA_N;
            }
            let catArr = [];
            let catVos = this.m_cats;
            for (const catVO of catVos) {
                if (catVO.catRes.quality == quality) {
                    catArr.push(catVO);
                }
            }
            return catArr[MathUtil.makeRandom(0, catArr.length)];
        }
    }
    CatModel._Inst = null;

    class UISpecialCatFoodAlert extends UIBaseWindow {
        constructor() {
            super("CatBall", "UISpecialCatFoodAlert");
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
            this.c2 = this._view.getController("c2");
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            this.handler = data && data.handler;
            let count = StorageUtils.Inst.GetSpecialCatFood();
            if (count == 0) {
                this.c2.selectedIndex = 0;
            }
            else {
                let isAd = MathUtil.random(AddFishModel.Inst.HighCatfoodAdvertisement);
                this.c2.selectedIndex = isAd ? 1 : 0;
            }
            this.spine_maoliang.play("maoliang");
            StorageUtils.Inst.SetSpecialCatFood(1);
            this.showBanner();
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_get) {
                this.getAward();
            }
            else if (btn == this.btn_get_ad) {
                PlatformMgr.Inst.showAd(AdType.ad_11, Laya.Handler.create(this, this.getAward));
            }
        }
        getAward(isSuccess = true) {
            if (isSuccess) {
                this.handler && this.handler.run();
                this.close();
            }
        }
        Reset() {
            super.Reset();
            this.handler = null;
        }
    }

    class AddFishCtrl {
        constructor() {
        }
        static get Inst() {
            return this.inst = this.inst || new AddFishCtrl();
        }
        init() {
            if (AddFishModel.Inst.HasFish()) {
                Laya.timer.loop(1000, this, this.render);
            }
        }
        render() {
            if (!AddFishModel.Inst.HasFish()) {
                this.RemoveFish();
            }
        }
        AddFish() {
            AddFishModel.Inst.AddFish();
            GameDispatcher.Inst.event(UIGameEvent.ADD_FISH);
            if (AddFishModel.Inst.HasFish()) {
                Laya.timer.loop(1000, this, this.render);
            }
        }
        getAddFishAward(pos) {
            let isSpeacil = MathUtil.random(AddFishModel.Inst.HighCatfood);
            if (!isSpeacil) {
                this.awardSpecailFood(pos, isSpeacil);
            }
            else {
                UISystem.Inst.CreateWindowView(UISpecialCatFoodAlert, { "handler": Laya.Handler.create(this, this.awardSpecailFood, [pos, isSpeacil]) });
            }
            EffectMgr.Inst.playConfigEffect(EffectMgr.Eff_shuihua, SceneMgr.Inst.GetScene(), null, pos, null, null, 1500);
            GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, TaskType.TASKTYPE_WEIMAO);
        }
        awardSpecailFood(pos, isSpeacil) {
            let camera = SceneMgr.Inst.GetCamera();
            let total = CatModel.Inst.addFoodGold();
            if (isSpeacil) {
                total = total.multipliedBy(AddFishModel.Inst.HighCatfoodmultiple);
            }
            MoneyModel.gold.add(total);
            let v2 = MathUtil.GetPointFrom3DWorld(camera, pos);
            EffectMgr.Inst.playGoldEffect(total, v2);
        }
        RemoveFish() {
            GameDispatcher.Inst.event(UIGameEvent.REMOVE_FISH);
            Laya.timer.clear(this, this.render);
        }
    }
    AddFishCtrl.inst = null;

    class CatBasinScript extends EntityBaseScript {
        constructor() {
            super();
            this.ui = null;
            this.yu = null;
            this.yuAnis = [];
            this.numMap = {};
            this.catBasinPos = new Laya.Vector3(-18.74, 1.73, 0.45);
        }
        onAwake() {
            super.onAwake();
            if (this.effectCont == null) {
                this.effectCont = new Laya.Sprite3D();
                this.effectCont.transform.localPosition = new Laya.Vector3(7.8, 2, 3.8);
                this.effectCont.transform.localRotationEuler = new Laya.Vector3(0, -90, 0);
                this.target.addChild(this.effectCont);
            }
            this.ui = this.target.getChildByName("ui");
            this.yu = this.target.getChildByName("yu");
            this.yu.active = true;
            this.cat2_fish_01 = this.yu.getChildByName("cat2_fish_01");
            let sp;
            for (let i = 0; i < this.yu.numChildren; i++) {
                sp = this.yu.getChildAt(i);
                let ani = sp && sp.getComponent(Laya.Animator);
                if (ani) {
                    ani.enabled = false;
                    this.yuAnis.push(ani);
                }
            }
            let numSp;
            for (let i = 0; i <= 9; i++) {
                numSp = this.ui.getChildByName("0_" + i);
                numSp.active = false;
                this.numMap[`0_${i}`] = numSp;
                numSp = this.ui.getChildByName("1_" + i);
                numSp.active = false;
                this.numMap[`1_${i}`] = numSp;
            }
            this.updateCount();
            this.SetCollider(this.target);
            this.ShowFish(AddFishModel.Inst.HasFish());
            GameDispatcher.Inst.on(UIGameEvent.CAT_UNLOCK, this, this.onCatUnlock);
            GameDispatcher.Inst.on(UIGameEvent.REMOVE_FISH, this, this.ClearBasin);
        }
        onCatUnlock(catVO) {
            this.updateCount();
        }
        updateCount() {
            let catNum = CatModel.Inst.GetCatCount();
            this.showNumber(catNum);
        }
        showNumber(num) {
            let str = num.toString();
            let numSp = null;
            let char;
            for (let i = 0; i <= 9; i++) {
                char = str.charAt(0);
                numSp = this.numMap[`0_${i}`];
                numSp.active = parseInt(char) == i;
                char = str.length >= 2 ? str.charAt(1) : "";
                numSp = this.numMap[`1_${i}`];
                numSp.active = str.length >= 2 && parseInt(char) == i;
            }
        }
        AddFish() {
            if (AddFishModel.Inst.HasFish()) {
                this.startTime();
            }
            else {
                this.pos = this.effectCont.transform.position;
                this.rot = this.effectCont.transform.rotationEuler;
                EffectMgr.Inst.playConfigEffect(EffectMgr.Eff_fish, this.target.scene, null, this.pos, this.rot, Laya.Handler.create(this, () => {
                    Laya.timer.once(1500, this, this.endAddFish);
                }), 1500);
                AddFishCtrl.Inst.AddFish();
                SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.catMeal));
            }
        }
        endAddFish() {
            this.ShowFish(true);
            this.startTime();
            let pos = this.cat2_fish_01.transform.position;
            AddFishCtrl.Inst.getAddFishAward(pos);
            let camera = SceneMgr.Inst.GetCamera();
            let v3 = this.catBasinPos.clone();
            v3.y += 1.5;
            let posfish = MathUtil.GetPointFrom3DWorld(camera, v3);
            v3.y += 2;
            let targetPos = MathUtil.GetPointFrom3DWorld(camera, v3);
            let offestY = posfish.y - targetPos.y;
            let item = EffectMgr.Inst.FlyGold("", posfish, UIFlyCoinItem.TYPE_NORMAL, UIFlyCoinItem.ITEM_TYPE_FISH, 2000, offestY);
            let scale = GlobalConfig.scene_scale;
            if (scale != 2)
                scale = 1 / (2 - scale);
            item.setScale(scale, scale);
        }
        startTime() {
            this.updateTime(true, 5);
            Laya.timer.loop(1000, this, this.updateTime, null, true);
            fgui.GTween.delayedCall(5.3).setTarget(this).onComplete(() => {
                Laya.timer.clear(this, this.updateTime);
            }, this);
        }
        updateTime(tweenStart = false, stay = 0) {
            if (AddFishModel.Inst.HasFish()) {
                let time = AddFishModel.Inst.GetSurplusTime();
                let timestr = StringUtil.formatToTime(time / 1000, "00:00:00");
                let mainUI = UISystem.Inst.GetWindowView(UIMainView);
                mainUI && mainUI.showProBar(timestr, time, AddFishModel.FISH_TIME, tweenStart, stay, this);
            }
            else {
                Laya.timer.clear(this, this.updateTime);
                this.ClearBasin();
            }
        }
        ShowFish(isShow) {
            if (this.yu) {
                this.yu.active = isShow;
                let ani;
                for (let i = 0; i < this.yuAnis.length; i++) {
                    ani = this.yuAnis[i];
                    if (ani) {
                        ani.enabled = isShow;
                    }
                }
                GameDispatcher.Inst.event(UIGameEvent.UPDATE_FISH_STATE, [isShow]);
            }
        }
        ClearBasin() {
            this.ShowFish(false);
        }
        get buildVO() {
            return this.vo;
        }
        get target() {
            return this.owner;
        }
        SetCollider(owner) {
            let phy = owner.getComponent(Laya.PhysicsCollider);
            if (phy) {
                phy.collisionGroup = this.GetCollisonGroup();
                phy.canCollideWith = this.GetCanCollideWith();
            }
        }
        GetCollisonGroup() {
            return CollisionType.TYPE_BUILD;
        }
        GetCanCollideWith() {
            return CollisionType.TYPE_RAY_CLICK;
        }
        onReset() {
            super.onReset();
            Laya.timer.clear(this, this.updateTime);
            Laya.timer.clear(this, this.ShowFish);
            Laya.timer.clearAll(this);
            fgui.GTween.kill(this);
        }
    }

    class CatHouse extends Build {
        constructor(displayObject) {
            super(displayObject);
            this.sign_1 = null;
        }
        SetVO(vo) {
            super.SetVO(vo);
            this.sign_1 = this.base.getChildByName("sign_1");
            if (this.sign_1 && this.buildVO.isCatHouse) {
                let bs = this.sign_1.addComponent(CatBasinScript);
                bs.SetVO(vo);
            }
        }
        onAwake() {
            super.onAwake();
            this.sign_1 = this.base.getChildByName("sign_1");
        }
        AddFish() {
            let bs = this.sign_1.getComponent(CatBasinScript);
            if (bs) {
                bs.AddFish();
            }
        }
    }
    CatHouse.Inst = null;

    class EntityModel {
        constructor() {
            this.builds = new Dictionary();
            this.cats = new Dictionary();
            this.chuans = new Array(4);
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new EntityModel();
            return this._Inst;
        }
        AddBuild(objId, entity) {
            this.builds[objId] = entity;
            GameDispatcher.Inst.event(UIGameEvent.BUILD_ADD, entity.buildVO);
        }
        RemoveBuild(objId) {
            delete this.builds[objId];
        }
        GetBuild(objId) {
            return this.builds[objId];
        }
        AddCat(objId, entity) {
            this.cats[objId] = entity;
        }
        RemoveCat(objId) {
            delete this.cats[objId];
        }
        GetCat(objId) {
            return this.cats[objId];
        }
        GetCats() {
            return this.cats;
        }
        AddChuan(entity) {
            for (let i = 0; i < this.chuans.length; i++) {
                let chuan = this.chuans[i];
                if (chuan == null) {
                    entity.index = i;
                    this.chuans[i] = entity;
                    return;
                }
            }
        }
        GetKongIndex() {
            for (let i = 0; i < this.chuans.length; i++) {
                let chuan = this.chuans[i];
                if (chuan == null) {
                    return i;
                }
            }
        }
        RemoveChuan(entity) {
            let index = this.chuans.indexOf(entity);
            if (index != null) {
                this.chuans[index] = null;
            }
        }
        GetChuan(cat) {
            for (let i = this.chuans.length - 1; i >= 0; i--) {
                let chuan = this.chuans[i];
                if (chuan && chuan.cat == cat) {
                    return chuan;
                }
            }
            return null;
        }
        IsHasKongChuan() {
            for (let i = 0; i < this.chuans.length; i++) {
                let chuan = this.chuans[i];
                if (chuan == null) {
                    return true;
                }
            }
            return false;
        }
        GetKongDownChuan() {
            for (let i = this.chuans.length - 1; i >= 0; i--) {
                let chuan = this.chuans[i];
                if (chuan && chuan.cat == null) {
                    return chuan;
                }
            }
            return null;
        }
        RemoveLastChuan() {
            for (let i = this.chuans.length - 1; i >= 0; i--) {
                let chuan = this.chuans[i];
                if (chuan != null) {
                    chuan.owner.destroy();
                    this.chuans[i] = null;
                    return;
                }
            }
        }
        GetBuildClsByType(type, owner) {
            if (type == BuildVO.TYPE_MATOU) {
                return new BuildMatou(owner);
            }
            else if (type == BuildVO.TYPE_MAOWO) {
                CatHouse.Inst = new CatHouse(owner);
                return CatHouse.Inst;
            }
            return new Build(owner);
        }
        Clear() {
        }
    }
    EntityModel._Inst = null;

    class BuildModel {
        constructor() {
            this.m_builds = new Array();
            this.costlvlUp = 0;
            this.addonItem = 0;
            this.maowuBuildVO = null;
            this.matouBuildVO = null;
            this.moveHouseBuildVO = null;
            this.costlvlUp = parseInt(ConfigValue.getItemByKey("costlvlUp").value);
            this.addonItem = parseInt(ConfigValue.getItemByKey("addonItem").value) * 0.0001;
            this.matoulv2costGold = parseInt(ConfigValue.getItemByKey("boatUpUse2").value);
            this.matoulv3costGold = parseInt(ConfigValue.getItemByKey("boatUpUse3").value);
            this.matoulv4costGold = parseInt(ConfigValue.getItemByKey("boatUpUse4").value);
            Laya.timer.loop(1000, this, this.OnCheck);
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new BuildModel();
            return this._Inst;
        }
        AddBuildVO(value) {
            if (!value) {
                return;
            }
            if (value.isJingYuBanjia && value.buildRes.Belong < LandModel.Inst.level) {
                return;
            }
            this.m_builds.push(value);
            this.m_builds.sort(this.onSort);
            if (value.buildRes.id == 1) {
                this.maowuBuildVO = value;
            }
            else if (value.buildRes.id == 2) {
                this.matouBuildVO = value;
            }
            else if (value.isJingYuBanjia && value.buildRes.Belong == LandModel.Inst.level) {
                this.moveHouseBuildVO = value;
            }
            value.Init();
        }
        RemoveBuildVO(vo) {
            let index = this.m_builds.indexOf(vo);
            ArrayUtil.RemoveAt(this.m_builds, index);
            this.m_builds.sort(this.onSort);
        }
        onSort(a, b) {
            if (a.buildRes.openOrder < b.buildRes.openOrder)
                return -1;
            else if (a.buildRes.openOrder > b.buildRes.openOrder)
                return 1;
            else
                0;
        }
        GetPrevOderBuildVO(order) {
            let prev;
            let buildVo;
            for (let i = 0; i < this.m_builds.length; i++) {
                buildVo = this.m_builds[i];
                if (buildVo && buildVo.buildRes.openOrder == order) {
                    return prev;
                }
                prev = buildVo;
            }
            return null;
        }
        LandUp() {
            this.moveHouseBuildVO = this.GetBuildVoByType(BuildVO.TYPE_BANJIASHESHI, LandModel.Inst.level);
        }
        GetBuildVOs() {
            return this.m_builds;
        }
        GetCurBuildVos() {
            let buildVos = [];
            let vo;
            for (let i = 0; i < this.m_builds.length; i++) {
                vo = this.m_builds[i];
                if (vo.buildRes.Belong <= LandModel.Inst.level) {
                    buildVos.push(vo);
                }
            }
            buildVos = buildVos.sort(this.onSort);
            return buildVos;
        }
        GetUnlockBuilds() {
            let builds = [];
            for (const buildVO of this.m_builds) {
                if (!buildVO)
                    continue;
                if (buildVO.IsUnLock())
                    builds.push(buildVO);
            }
            return builds;
        }
        GetOnBuildVO(buildVO) {
            let builds = this.GetUnlockBuilds();
            let index = builds.indexOf(buildVO);
            index--;
            if (index < 0) {
                index = builds.length - 1;
            }
            return builds[index];
        }
        GetNextBuildVO(buildVO) {
            let builds = this.GetUnlockBuilds();
            let index = builds.indexOf(buildVO);
            index++;
            if (index >= builds.length) {
                index = 0;
            }
            return builds[index];
        }
        OnCheck() {
            for (const buildVO of this.m_builds) {
                if (!buildVO)
                    continue;
                if (buildVO.IsUnLock()) {
                    if (buildVO.CheckAddGold()) {
                        let addGold = buildVO.GetAllAddGold();
                        MoneyModel.gold.add(addGold);
                    }
                }
                else
                    return;
            }
        }
        GetBuildScore(type) {
            let score = 0;
            for (const buildVO of this.m_builds) {
                if (!buildVO)
                    continue;
                if (buildVO.buildRes.type == type) {
                    if (buildVO.IsUnLock()) {
                        score += buildVO.GetCurLvScore();
                    }
                }
            }
            return score;
        }
        GetBuildVoByType(type, landLv = 1) {
            for (const buildVO of this.m_builds) {
                if (buildVO.buildRes.type == type && buildVO.buildRes.Belong == landLv) {
                    return buildVO;
                }
            }
            return null;
        }
        GetBuildVOBy(tempId) {
            for (const buildVO of this.m_builds) {
                if (buildVO.buildRes.id == tempId) {
                    return buildVO;
                }
            }
            return null;
        }
        GetAllBuildGoldRate(time = 1, useDouble = true) {
            let double = GlobalConfig.isDouble ? 2 : 1;
            if (!useDouble) {
                double = 1;
            }
            let num = new BigNumber(0);
            for (const buildVO of this.m_builds) {
                if (!buildVO)
                    continue;
                if (buildVO.IsUnLock()) {
                    let add = buildVO.GetAddGoldRate();
                    num = num.plus(add);
                }
            }
            return num.multipliedBy(time).multipliedBy(double);
        }
        ChangeMatou(buildVO) {
            if (buildVO == null)
                return;
            let build = EntityModel.Inst.GetBuild(buildVO.objId);
            if (build == null)
                return;
            build.onChangeParts();
        }
        GetMatouLvCostGold(lv = NaN) {
            if (isNaN(lv)) {
                lv = this.matouBuildVO.level;
            }
            if (lv == 1) {
                return this.matoulv2costGold;
            }
            else if (lv == 2) {
                return this.matoulv3costGold;
            }
            else if (lv == 3) {
                return this.matoulv4costGold;
            }
            else {
                return 0;
            }
        }
        get maowuUpRes() {
            let vo = this.maowuBuildVO;
            return vo && MaowuUpResource.getItemByKey(vo.stage);
        }
        get nextMaowuUpRes() {
            let vo = this.maowuBuildVO;
            return vo && MaowuUpResource.getItemByKey(vo.stage + 1);
        }
        get canKuoJian() {
            return !!this.nextMaowuUpRes && this.nextMaowuUpRes.unlockLvl <= LandModel.Inst.level;
        }
        get isMaowuMaxLevel() {
            return this.nextMaowuUpRes == null;
        }
    }
    BuildModel._Inst = null;

    class ZuoweiVO {
        constructor(count) {
            this.catVOs = null;
            this.catVOs = new Array(count);
        }
        AddZuowei(catVO) {
            for (let i = 0; i < this.catVOs.length; i++) {
                let chuan = this.catVOs[i];
                if (chuan == null) {
                    this.catVOs[i] = catVO;
                    return i;
                }
            }
        }
        RemoveZuowei(catVO) {
            let index = this.catVOs.indexOf(catVO);
            if (index != null) {
                this.catVOs[index] = null;
            }
        }
        HasCount() {
            let count = 0;
            for (let i = 0; i < this.catVOs.length; i++) {
                let chuan = this.catVOs[i];
                if (chuan != null) {
                    count++;
                }
            }
            return count;
        }
    }

    class BuildVO extends BaseVO {
        constructor() {
            super();
            this.buildRes = null;
            this.buildUpRes = null;
            this.buildSeatRes = null;
            this.nextBuildSeatRes = null;
            this._stage = 0;
            this._level = 0;
            this._zwCount = 0;
            this._catServerArr = [];
            this.startTime = 0;
            this._teShuPeiJianArr = [];
            this._buildPos = new Laya.Vector3();
            this._upgradeEffPos = new Laya.Vector3();
            this._advancedEffPos = new Laya.Vector3();
            this.curZuowei = null;
            this.bigNumber = new BigNumber(0);
            this.addLvGold = new BigNumber(0);
        }
        Parse(bytes) {
            let buildId = bytes.readUint16();
            this.buildRes = bulidResource.getItemByKey(buildId);
            super.Parse(bytes);
            this._stage = StorageUtils.Inst.GetBuildStage(this.buildRes.id);
            this._level = StorageUtils.Inst.GetBuildLv(this.buildRes.id);
            this._zwCount = StorageUtils.Inst.GetBuildZuoweiCount(this.buildRes.id);
            this._catServerArr = StorageUtils.Inst.GetBuildCatSerArr(this.buildRes.id);
            this.buildUpRes = bulidupResource.getItemBy2Keys(this.buildRes.id, this._stage);
            if (this._zwCount > 0)
                this.buildSeatRes = bulidSeatResource.getItemBy2Keys(this.buildRes.id, this._zwCount);
            this.nextBuildSeatRes = bulidSeatResource.getItemBy2Keys(this.buildRes.id, this._zwCount + 1);
            this.startTime = Laya.timer.currTimer;
            this._buildPos.setValue(-parseFloat(this.buildRes.position[0]), parseFloat(this.buildRes.position[1]), parseFloat(this.buildRes.position[2]));
            let effRes = bulidEffResource.getItemByKey(this.buildRes.upgradeEff);
            effRes && this._upgradeEffPos.setValue(-parseFloat(effRes.position[0]), parseFloat(effRes.position[1]), parseFloat(effRes.position[2]));
            effRes = bulidEffResource.getItemByKey(this.buildRes.advancedEff);
            effRes && this._advancedEffPos.setValue(-parseFloat(effRes.position[0]), parseFloat(effRes.position[1]), parseFloat(effRes.position[2]));
            if (this.isCatHouse) {
                this._zwCount = 3;
                this.curZuowei = new ZuoweiVO(3);
            }
            else if (this.buildRes.positionNum > 0) {
                this.curZuowei = new ZuoweiVO(this.buildRes.positionNum);
            }
            if (this.isMatou) {
                if (this._level < 1) {
                    this._level = 1;
                    StorageUtils.Inst.SetBuildLv(this.buildRes.id, this._level);
                }
                if (this.stage < 1) {
                    this.stage = 1;
                }
            }
        }
        Init() {
            let items = bulidupResource.items;
            let arr = [];
            for (let i = 0; i < items.length; i++) {
                const item = items[i];
                if (parseInt(item.buildId) == this.buildRes.id) {
                    arr.push(item);
                }
            }
            if (arr.length > 0 && this._stage > arr[arr.length - 1].stage) {
                this._stage = arr[arr.length - 1].stage;
                this.buildUpRes = bulidupResource.getItemBy2Keys(this.buildRes.id, this._stage);
                StorageUtils.Inst.SetBuildStage(this.buildRes.id, this._stage);
            }
            if (this.level > this.curLandMaxLevel) {
                this.level = this.curLandMaxLevel;
            }
            this.UpdateTeShuPeiJian();
        }
        UpdateTeShuPeiJian() {
            let item1 = ItemModel.Inst.GetItemVOById(this.buildRes.itemId1);
            let item2 = ItemModel.Inst.GetItemVOById(this.buildRes.itemId2);
            if (item1) {
                let res = item1.itemRes.res;
                for (let i = 0; i < res.length; i++) {
                    let resItem = res[i];
                    this._teShuPeiJianArr.indexOf(resItem) == -1 && this._teShuPeiJianArr.push(resItem);
                }
            }
            if (item2) {
                let res = item2.itemRes.res;
                for (let i = 0; i < res.length; i++) {
                    let resItem = res[i];
                    this._teShuPeiJianArr.indexOf(resItem) == -1 && this._teShuPeiJianArr.push(resItem);
                }
            }
        }
        get buildPos() {
            return this._buildPos;
        }
        get upgradeEffPos() {
            return this._upgradeEffPos;
        }
        get advancedEffPos() {
            return this._advancedEffPos;
        }
        get scriptCls() {
            return BuildScript;
        }
        set zwCount(v) {
            this._zwCount = v;
            this.buildSeatRes = bulidSeatResource.getItemBy2Keys(this.buildRes.id, this._zwCount);
            this.nextBuildSeatRes = bulidSeatResource.getItemBy2Keys(this.buildRes.id, this._zwCount + 1);
            StorageUtils.Inst.SetBuilZuowei(this.buildRes.id, v);
        }
        get zwCount() {
            return this._zwCount;
        }
        set stage(v) {
            if (this.IsMaxStage())
                return;
            this._stage = v;
            this.buildUpRes = bulidupResource.getItemBy2Keys(this.buildRes.id, v);
            StorageUtils.Inst.SetBuildStage(this.buildRes.id, v);
        }
        get stage() {
            return this._stage;
        }
        set level(v) {
            if (this._level == v)
                return;
            this._level = v;
            StorageUtils.Inst.SetBuildLv(this.buildRes.id, v);
            let build = EntityModel.Inst.GetBuild(this.objId);
            if (build) {
                build.PlayClickAni();
            }
            GameDispatcher.Inst.event(UIGameEvent.BUILD_LEVEL_UP, this);
            if (this.level > 1) {
                SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.facilityUpgrade));
                GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [TaskType.TASKTYPE_UP_BUILD, this.buildRes.id]);
                GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [TaskType.TASKTYPE_BUILDLEVEL, this.buildRes.id]);
            }
            else if (this.level == 1) {
                SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.fx_congratulation_facility));
                GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [TaskType.TASKTYPE_UNLOCK_BUILD, this.buildRes.id]);
            }
            GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [TaskType.TASKTYPE_BUILDSCORE, this.buildRes.id]);
            if (this.isMatou) {
                GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, TaskType.TASKTYPE_ADDMATOUNUM);
            }
            if (this.buildRes.type == BuildVO.TYPE_MAOWO) {
            }
            else {
                if (this.buildUpRes && this.GetCurUpUnlockLv() >= this.GetNeedUpMaxLv()) {
                    this.stage += 1;
                    if (build) {
                        build.PlayStageUp();
                        GameDispatcher.Inst.event(UIGameEvent.BUILD_STAGE_UP, build.buildVO);
                    }
                }
            }
            GameDispatcher.Inst.event(UIGameEvent.UPDATE_GOLD_RATE);
        }
        get level() {
            return this._level;
        }
        get createAniName() {
            if (this.buildRes.type == BuildVO.TYPE_MAOWO) {
                let upRes = MaowuUpResource.getItemByKey(this.stage);
                return upRes ? upRes.levelAni : "";
            }
            else {
                let upRes = bulidupResource.getItemBy2Keys(this.buildRes.id, 0);
                return upRes ? upRes.levelAni : "";
            }
        }
        get stageUpAniName() {
            if (this.buildRes.type == BuildVO.TYPE_MAOWO) {
                let upRes = MaowuUpResource.getItemByKey(this.stage);
                return upRes ? upRes.levelAni : "";
            }
            else {
                let upRes = bulidupResource.getItemBy2Keys(this.buildRes.id, this.stage);
                return upRes ? upRes.levelAni : "";
            }
        }
        get catServerArr() {
            return this._catServerArr;
        }
        AddServer(catId) {
            let index = this._catServerArr.indexOf(catId);
            if (this._catServerArr.length < 4 && index == -1) {
                this._catServerArr.push(catId);
                StorageUtils.Inst.SetBuilCatSerArr(this.buildRes.id, this._catServerArr);
                GameDispatcher.Inst.event(UIGameEvent.BUILD_UPDATE_SERVER, this);
                GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [TaskType.TASKTYPE_ADDSERVER, this.buildRes.id]);
            }
        }
        AddServerAt(catId, index) {
            let i = this._catServerArr.indexOf(catId);
            if (this._catServerArr.length <= 4 && i == -1 && index < 4) {
                this._catServerArr[index] = catId;
                StorageUtils.Inst.SetBuilCatSerArr(this.buildRes.id, this._catServerArr);
                GameDispatcher.Inst.event(UIGameEvent.BUILD_UPDATE_SERVER, this);
                GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [TaskType.TASKTYPE_ADDSERVER, this.buildRes.id]);
            }
        }
        IsHasServer(catId) {
            return this._catServerArr.indexOf(catId) != -1;
        }
        IsMaxStage() {
            if (this.buildRes.type == BuildVO.TYPE_MAOWO) {
                return BuildModel.Inst.isMaowuMaxLevel;
            }
            else {
                let buildUpRes = bulidupResource.getItemBy2Keys(this.buildRes.id, this.stage + 1);
                return buildUpRes == null;
            }
        }
        GetNextStage() {
            let nextBuildUpRes = bulidupResource.getItemBy2Keys(this.buildRes.id, this.stage + 1);
            return nextBuildUpRes;
        }
        GetNextUpRate() {
            let nextBuildUpRes = bulidupResource.getItemBy2Keys(this.buildRes.id, this.stage + 1);
            if (nextBuildUpRes == null)
                return 0;
            return nextBuildUpRes.addon - this.buildUpRes.addon;
        }
        GetNeedUpMaxLv() {
            let nextBuildUpRes = bulidupResource.getItemBy2Keys(this.buildRes.id, this.stage + 1);
            if (nextBuildUpRes == null)
                return 0;
            return nextBuildUpRes.unlockLvl - this.buildUpRes.unlockLvl;
        }
        GetCurUpUnlockLv() {
            return this.level - this.buildUpRes.unlockLvl;
        }
        GetCurLvScore() {
            return this.GetScore(this.level);
        }
        GetScore(lv) {
            return this.buildRes.baseScore + this.buildRes.score * (lv - 1);
        }
        GetTime() {
            return this.buildRes.time;
        }
        GetTimeMs() {
            return this.buildRes.time * 1000;
        }
        GetAllAddGold() {
            return this.GetAddGold(this.level);
        }
        GetAddGoldRate() {
            let gold = this.GetAllAddGold();
            let time = this.GetTime();
            let rate;
            if (!(gold instanceof BigNumber)) {
                gold = new BigNumber(gold);
            }
            if (time == 0) {
                return new BigNumber(0);
            }
            rate = gold.multipliedBy(1 / time);
            return rate;
        }
        GetLvUpCostGold(lv = NaN) {
            if (this.isMatou) {
                let gold = BuildModel.Inst.GetMatouLvCostGold(lv);
                return this.bigNumber.multipliedBy(0).plus(gold);
            }
            return this.GetCostGOld(this.level);
        }
        GetCostGOld(lv) {
            let addRate = 1 + BuildModel.Inst.costlvlUp * 0.0001;
            let bigNum = new BigNumber(this.buildRes.costPara);
            if (this.isJingYuBanjia) {
                bigNum = bigNum.multipliedBy(lv);
            }
            else {
                bigNum = bigNum.multipliedBy(Math.pow(addRate, lv));
            }
            return bigNum;
        }
        GetZuoweiCostGold() {
            if (this.zwCount == 1) {
                return this.GetCostGOld(this.buildRes.positionPrice1);
            }
            else if (this.zwCount == 2) {
                return this.GetCostGOld(this.buildRes.positionPrice2);
            }
        }
        GetServerCostGold(serverIndex = 0) {
            let costGold = 0;
            if (serverIndex == 0) {
                costGold = this.buildRes.servePrice1;
            }
            else if (serverIndex == 1) {
                costGold = this.buildRes.servePrice2;
            }
            else if (serverIndex == 2) {
                costGold = this.buildRes.servePrice3;
            }
            else {
                costGold = this.buildRes.servePrice4;
            }
            if (this.catServerArr.length > serverIndex) {
                if (this.catServerArr[serverIndex] != null) {
                    return new BigNumber(0);
                }
            }
            return this.GetCostGOld(costGold);
        }
        IsShowZuowei() {
            if (this.nextBuildSeatRes) {
                return this.buildRes.positionNum > 0 && this.zwCount < this.nextBuildSeatRes.lv && this.level >= this.nextBuildSeatRes.setunlock;
            }
            return this.buildRes.positionNum > 0 && this.zwCount < this.buildRes.positionNum;
        }
        GetAddGold(lv) {
            if (lv == 0 || !this.buildUpRes)
                return 0;
            let addRate = 1 + this.buildUpRes.addonlvlUp * 0.0001;
            this.addLvGold = this.addLvGold.multipliedBy(0).plus(this.buildRes.getPara).multipliedBy(Math.pow(addRate, lv));
            let catAddRate = CatModel.Inst.GetAllRate();
            let allGold = this.addLvGold.multipliedBy((1 + catAddRate * 0.0001));
            allGold = allGold.multipliedBy(this.buildUpRes.addon).multipliedBy(this.buildRes.time).multipliedBy(this.GetPeijianRate()).multipliedBy(0.0001);
            return allGold;
        }
        GetPeijianRate() {
            let rate = 1;
            let item1 = ItemModel.Inst.GetItemVOById(this.buildRes.itemId1);
            if (item1)
                rate += BuildModel.Inst.addonItem;
            let item2 = ItemModel.Inst.GetItemVOById(this.buildRes.itemId2);
            if (item2)
                rate += BuildModel.Inst.addonItem;
            return rate;
        }
        CheckAddGold() {
            let time = Laya.timer.currTimer - this.startTime;
            if (time >= this.GetTimeMs()) {
                let lastTime = (Laya.timer.currTimer - this.startTime) % this.GetTimeMs();
                this.startTime = Laya.timer.currTimer - lastTime;
                return true;
            }
            return false;
        }
        get partName() {
            if (this.buildRes.type == BuildVO.TYPE_MAOWO) {
                let baseRes = BuildModel.Inst.maowuUpRes;
                return baseRes ? baseRes.partName : "";
            }
            else if (this.buildRes.type == BuildVO.TYPE_MATOU) {
                return this.buildUpRes ? this.buildUpRes.partName : "";
            }
            else {
                return this.buildUpRes ? this.buildUpRes.partName : "";
            }
        }
        get showParts() {
            if (this.buildRes.type == BuildVO.TYPE_MAOWO) {
                let baseRes = BuildModel.Inst.maowuUpRes;
                return baseRes ? baseRes.showParts : [];
            }
            else if (this.buildRes.type == BuildVO.TYPE_MATOU) {
                return this.buildUpRes ? this.buildUpRes.showParts : [];
            }
            else {
                return this.buildUpRes ? this.buildUpRes.showParts : [];
            }
        }
        get addParts() {
            if (this.buildRes.type == BuildVO.TYPE_MAOWO) {
                let baseRes = BuildModel.Inst.maowuUpRes;
                return baseRes ? baseRes.addParts : [];
            }
            else if (this.buildRes.type == BuildVO.TYPE_MATOU) {
                return this.buildUpRes ? this.buildUpRes.addParts : [];
            }
            else {
                return this.buildUpRes ? this.buildUpRes.addParts : [];
            }
        }
        get zuoweiAddParts() {
            return this.buildSeatRes ? this.buildSeatRes.addParts : [];
        }
        get teShuPeiJianAddParts() {
            return this._teShuPeiJianArr;
        }
        IsUnLock() {
            return this.level > 0;
        }
        IsCanUnlock() {
            if (this.buildRes.openOrder == 11) {
                return false;
            }
            if (this.IsUnLock())
                return false;
            if (this.buildRes.openOrder > 0) {
                let prevBuild = BuildModel.Inst.GetPrevOderBuildVO(this.buildRes.openOrder);
                if (prevBuild && !prevBuild.IsUnLock()) {
                    return false;
                }
            }
            let landLv = StorageUtils.Inst.GetLandLv();
            if (landLv < this.buildRes.Belong) {
                return false;
            }
            let openId = this.buildRes.openId;
            if (openId == 0)
                return true;
            return StorageUtils.Inst.HasBuild(openId);
        }
        get land() {
            return this.buildRes.Belong;
        }
        PreIsUnLock() {
            let builds = BuildModel.Inst.GetCurBuildVos();
            let index = builds.indexOf(this);
            if (index <= 0)
                return true;
            index--;
            return builds[index].IsUnLock();
        }
        NeedGold() {
            let bigNum = new BigNumber(this.buildRes.unlockGold);
            return bigNum;
        }
        IsMaxLv() {
            if (this.isCatHouse) {
                let lv = this.level;
                return lv >= BuildModel.Inst.maowuUpRes.houseMaxLvl;
            }
            else if (this.isMatou) {
                return this.level >= LandModel.Inst.boatNum;
            }
            else if (this.isJingYuBanjia) {
                return this.level >= LandModel.Inst.banjiaLevel;
            }
            else {
                return this.level >= LandModel.Inst.maxLvl;
            }
        }
        get curLandMaxLevel() {
            if (this.isCatHouse) {
                return BuildModel.Inst.maowuUpRes.houseMaxLvl;
            }
            else if (this.isMatou) {
                return LandModel.Inst.boatNum;
            }
            else if (this.isJingYuBanjia) {
                return LandModel.Inst.banjiaLevel;
            }
            else {
                return LandModel.Inst.maxLvl;
            }
        }
        get isCatHouse() {
            return this.buildRes.type == BuildVO.TYPE_MAOWO;
        }
        get isMatou() {
            return this.buildRes.type == BuildVO.TYPE_MATOU;
        }
        get isMaowo() {
            return this.buildRes.type == BuildVO.TYPE_MAOWO;
        }
        get isSheshi() {
            return this.buildRes.type == BuildVO.TYPE_XIUXIANSHESHI || this.buildRes.type == BuildVO.TYPE_HUANGJINGSHESHI ||
                this.buildRes.type == BuildVO.TYPE_YULESHESHI || this.buildRes.type == BuildVO.TYPE_CANYINSHESHI;
        }
        get isJingYuBanjia() {
            return this.buildRes.type == BuildVO.TYPE_BANJIASHESHI;
        }
        get isDiaoyuchan() {
            return this.buildRes.id == 101;
        }
        CatUseBuild(catId) {
            if (this.catServerArr.indexOf(catId) != -1) {
                let itemVo = ItemModel.Inst.GetItemVOById(ItemModel.ITEM_CHURU);
                ItemCtrl.Inst.AddItemVO(itemVo.itemId);
            }
        }
        HasKongZuowei() {
            if (this.curZuowei == null)
                return false;
            if (!this.IsUnLock())
                return false;
            if (this.isCatHouse) {
                return false;
            }
            let count = this.curZuowei.HasCount();
            return count < this.zwCount;
        }
    }
    BuildVO.TYPE_MAOWO = 1;
    BuildVO.TYPE_MATOU = 2;
    BuildVO.TYPE_XIUXIANSHESHI = 3;
    BuildVO.TYPE_HUANGJINGSHESHI = 4;
    BuildVO.TYPE_YULESHESHI = 5;
    BuildVO.TYPE_CANYINSHESHI = 6;
    BuildVO.TYPE_BANJIASHESHI = 7;
    BuildVO.matou_state_guanggao = "matou_guanggao";
    BuildVO.matou_state_likai = "matou_likai";
    BuildVO.matou_state_shangdao = "matou_shangdao";
    BuildVO.matou_state_xunzhao = "matou_xunzhao";

    class LandModel {
        constructor() {
            this._level = 0;
            this.isLandRes = null;
            this.nextLandRes = null;
            this.GetLevelInfo();
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new LandModel();
            return this._Inst;
        }
        GetLevelInfo() {
            this._level = StorageUtils.Inst.GetLandLv();
            this.isLandRes = islandResource.getItemByKey(this._level);
            this.nextLandRes = islandResource.getItemByKey(this._level + 1);
        }
        set level(value) {
            if (value == this._level)
                return;
            this._level = value;
            this.isLandRes = islandResource.getItemByKey(this._level);
            StorageUtils.Inst.SetLandLv(value);
            GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [TaskType.TASKTYPE_UNLOCKISLAND, this.isLandRes.id]);
        }
        get level() {
            return this._level;
        }
        get banjiaLevel() {
            return this.isLandRes ? this.isLandRes.moveLlv : 0;
        }
        get name() {
            return this.isLandRes ? this.isLandRes.name : "";
        }
        get icon() {
            return this.isLandRes ? this.isLandRes.icon : "";
        }
        get maxLvl() {
            return this.isLandRes ? this.isLandRes.maxLvl : 0;
        }
        get houseMaxLvl() {
            return this.isLandRes ? this.isLandRes.houseMaxLvl : 0;
        }
        get boatNum() {
            return this.isLandRes ? this.isLandRes.boatNum : 0;
        }
        IsMax() {
            let isLandRes = islandResource.getItemByKey(this.level + 1);
            return isLandRes == null;
        }
        IsUnlockServer() {
            return this.level > 1;
        }
        get cameraTweenStart() {
            return this.isLandRes.cameraTweenStart;
        }
        CanUnlockNextLand() {
            let vo = BuildModel.Inst.GetBuildVoByType(BuildVO.TYPE_BANJIASHESHI, this.level);
            return vo && vo.level >= this.banjiaLevel;
        }
        get nextName() {
            return this.nextLandRes && this.nextLandRes.name;
        }
        get nextIcon() {
            return this.nextLandRes && this.nextLandRes.icon;
        }
        get NextLandRes() {
            return this.nextLandRes;
        }
        moveHouse() {
            this.level += 1;
            this.GetLevelInfo();
            BuildModel.Inst.LandUp();
        }
    }
    LandModel._Inst = null;

    class MapVO extends BaseVO {
        constructor() {
            super();
        }
        Parse(bytes) {
            super.Parse(bytes);
        }
    }

    class TreeVO extends BaseVO {
        constructor() {
            super();
            this.landLv = 0;
        }
        Parse(bytes) {
            this.landLv = bytes.readUint16();
            super.Parse(bytes);
        }
    }

    class PathVO {
        constructor() {
            this.pathPointVO = null;
            this.objId = 0;
            this.weight = 0;
        }
    }

    class PathPointVO extends BaseVO {
        constructor() {
            super();
            this.nextPaths = null;
            this.nextPathDic = null;
            this.landId = 0;
            this.catId = 0;
        }
        Parse(bytes) {
            this.landId = bytes.readUint16();
            this.objId = bytes.readUint16();
            this.position.setValue(-bytes.readFloat32(), bytes.readFloat32(), bytes.readFloat32());
            let size = bytes.readUint16();
            this.nextPaths = new Array(size);
            this.nextPathDic = new Dictionary();
            for (let i = 0; i < size; i++) {
                let pathVO = new PathVO();
                pathVO.objId = bytes.readUint16();
                pathVO.weight = bytes.readUint16();
                this.nextPaths[i] = pathVO;
                this.nextPathDic[pathVO.objId] = pathVO;
            }
        }
        addPaths() {
            let landLv = LandModel.Inst.level;
            for (const pathVO of this.nextPaths) {
                if (pathVO.pathPointVO == null) {
                    let pathPointVO = PathPointModel.Inst.GetPathVOById(pathVO.objId);
                    if (pathPointVO.landId <= landLv)
                        pathVO.pathPointVO = pathPointVO;
                }
            }
        }
        CheckHasPath(objId) {
            return objId in this.nextPathDic;
        }
        GetPos(result = null) {
            if (result == null)
                result = new Laya.Vector3();
            this.position.cloneTo(result);
            return result;
        }
        GetDistanceSquared(value) {
            return Laya.Vector3.distanceSquared(value, this.position);
        }
        GetRandomNextPoint() {
            let pathVOArrs = this.nextPaths;
            let jv = Math.random();
            let onJv = 0;
            let max = 0;
            let pathVO = null;
            for (let i = 0; i < pathVOArrs.length; i++) {
                pathVO = pathVOArrs[i];
                if (pathVO.pathPointVO == null) {
                    continue;
                }
                max += pathVO.weight;
            }
            for (let i = 0; i < pathVOArrs.length; i++) {
                pathVO = pathVOArrs[i];
                if (pathVO.pathPointVO == null) {
                    continue;
                }
                let curJv = onJv + pathVO.weight / max;
                if (jv > onJv && jv <= curJv) {
                    return pathVO.pathPointVO;
                }
                onJv = curJv;
            }
            return null;
        }
        GetNearPathPointVO(toPos, onPathPointVO) {
            if (this.nextPaths.length == 1)
                return this.nextPaths[0].pathPointVO;
            let dis = Number.MAX_VALUE;
            let onPathVO = null;
            let curDis = 0;
            for (const pathVO of this.nextPaths) {
                if (pathVO.pathPointVO == onPathPointVO)
                    continue;
                if (pathVO.pathPointVO == null)
                    continue;
                curDis = Laya.Vector3.distanceSquared(pathVO.pathPointVO.position, toPos);
                if (curDis < dis) {
                    dis = curDis;
                    onPathVO = pathVO.pathPointVO;
                }
            }
            return onPathVO;
        }
    }

    class SignVO extends BaseVO {
        constructor() {
            super();
            this.buildId = 0;
        }
        Parse(bytes) {
            this.buildId = bytes.readUint16();
            super.Parse(bytes);
            this.buildRes = bulidResource.getItemByKey(this.buildId);
        }
    }

    class PathNavigateVO extends PathPointVO {
        constructor() {
            super();
            this.nearPathPointVO = null;
        }
        Parse(bytes) {
            super.Parse(bytes);
        }
        addPaths() {
            for (const pathVO of this.nextPaths) {
                if (pathVO.pathPointVO == null) {
                    let pathPointVO = PathNavigateModel.Inst.GetPathVOById(pathVO.objId);
                    pathVO.pathPointVO = pathPointVO;
                }
            }
        }
        GetDistanceSquared(value) {
            return Laya.Vector3.distanceSquared(value, this.position);
        }
        GetNearPath() {
            if (this.nearPathPointVO == null) {
                this.nearPathPointVO = PathPointModel.Inst.GetShortDistance(this.position);
            }
            return this.nearPathPointVO;
        }
        GetNearPathPointVO2(toPos, onPathPointVO) {
            let dis = Number.MAX_VALUE;
            let onPathVO = null;
            let curDis = 0;
            for (const pathVO of this.nextPaths) {
                if (pathVO.pathPointVO == onPathPointVO)
                    continue;
                if (pathVO.pathPointVO == null)
                    continue;
                curDis = Laya.Vector3.distanceSquared(pathVO.pathPointVO.position, toPos);
                if (curDis < dis) {
                    dis = curDis;
                    onPathVO = pathVO.pathPointVO;
                }
            }
            return onPathVO;
        }
    }

    class SceneModel {
        constructor() {
            this.m_maps = new Array();
            this.m_cats = new Array();
            this.m_trees = new Array();
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new SceneModel();
            return this._Inst;
        }
        Parse(bytes) {
            bytes.pos = 0;
            try {
                let length = bytes.readUint16();
                for (let i = 0; i < length; i++) {
                    var vo = null;
                    let type = bytes.readUint16();
                    switch (type) {
                        case BaseVO.MAP:
                            vo = new MapVO();
                            vo.type = type;
                            vo.Parse(bytes);
                            this.m_maps.push(vo);
                            break;
                        case BaseVO.TREE:
                            vo = new TreeVO();
                            vo.type = type;
                            vo.Parse(bytes);
                            this.m_trees.push(vo);
                            break;
                        case BaseVO.CAT:
                            vo = new CatVO();
                            vo.type = type;
                            vo.Parse(bytes);
                            this.m_cats.push(vo);
                            break;
                        case BaseVO.BUILD:
                            vo = new BuildVO();
                            vo.type = type;
                            vo.Parse(bytes);
                            BuildModel.Inst.AddBuildVO(vo);
                            break;
                        case BaseVO.PATH_POINT:
                            vo = new PathPointVO();
                            vo.type = type;
                            vo.Parse(bytes);
                            PathPointModel.Inst.AddPathPoint(vo);
                            break;
                        case BaseVO.NAVIGATE:
                            vo = new PathNavigateVO();
                            vo.type = type;
                            vo.Parse(bytes);
                            PathNavigateModel.Inst.AddPathNavigate(vo);
                            break;
                        case BaseVO.SIGN:
                            vo = new SignVO();
                            vo.type = type;
                            vo.Parse(bytes);
                            SignModel.Inst.AddSingVO(vo);
                            break;
                        default:
                            break;
                    }
                }
            }
            catch (error) {
            }
            GameDispatcher.Inst.event(UIGameEvent.UPDATE_GOLD_RATE);
        }
        AddToMap() {
            let landLv = LandModel.Inst.level;
            PathPointModel.Inst.UnlockPathPoint(landLv);
            PathNavigateModel.Inst.UnlockPathPoint();
            for (const vo of this.m_maps) {
                MapQuadTree.Inst.AddVO(vo, true);
            }
            for (const vo of this.m_trees) {
                MapQuadTree.Inst.AddVO(vo);
            }
            for (const vo of this.m_cats) {
                MapQuadTree.Inst.AddVO(vo);
            }
            let builds = BuildModel.Inst.GetBuildVOs();
            for (const vo of builds) {
                MapQuadTree.Inst.AddVO(vo);
            }
        }
        AddToCats() {
            let cats = CatModel.Inst.GetAllCatVOs();
            for (const vo of cats) {
                if (vo.IsHas())
                    MapQuadTree.Inst.AddVO(vo);
            }
            BuildCtrl.Inst.CheckAllBuild();
            CatEventCtrl.Inst.Init();
        }
    }
    SceneModel._Inst = null;

    class CameraScript extends Laya.Script3D {
        constructor() {
            super();
            this.camera = null;
            this.transform = null;
            this.transformParent = null;
            this.animator = null;
            this.rect = new Laya.Rectangle();
            this.MOVE_RATE = 0.1;
            this.FOV_RATE = 0.1;
            this.MOVE_TOP = 100;
            this.MOVE_BOTTOM = -100;
            this.MOVE_LEFT = -100;
            this.MOVE_RIGHT = 100;
            this.FOV_MIN = 40;
            this.FOV_MAX = 60;
            this.FOV_NORMAL = 50;
            this.initY = 73;
            this.onPosition = new Laya.Vector3();
            this.onRotationEuler = new Laya.Vector3();
        }
        onAwake() {
            this.animator = this.owner.getComponent(Laya.Animator);
            this.StopAni();
            this.camera = this.owner;
            this.camera.enableHDR = false;
            this.camera.nearPlane = 1;
            this.camera.farPlane = 500;
            this.transform = this.camera.transform;
            this.transformParent = this.owner.parent.transform;
            let MOVE_TOP = ConfigValue.getItemByKey("MOVE_TOP");
            if (MOVE_TOP)
                this.MOVE_TOP = parseFloat(MOVE_TOP.value);
            let MOVE_BOTTOM = ConfigValue.getItemByKey("MOVE_BOTTOM");
            if (MOVE_BOTTOM)
                this.MOVE_BOTTOM = parseFloat(MOVE_BOTTOM.value);
            let MOVE_LEFT = ConfigValue.getItemByKey("MOVE_LEFT");
            if (MOVE_LEFT)
                this.MOVE_LEFT = -parseFloat(MOVE_LEFT.value);
            let MOVE_RIGHT = ConfigValue.getItemByKey("MOVE_RIGHT");
            if (MOVE_RIGHT)
                this.MOVE_RIGHT = -parseFloat(MOVE_RIGHT.value);
            let FOV_MIN = ConfigValue.getItemByKey("FOV_MIN");
            if (FOV_MIN)
                this.FOV_MIN = parseFloat(FOV_MIN.value);
            let FOV_MAX = ConfigValue.getItemByKey("FOV_MAX");
            if (FOV_MAX)
                this.FOV_MAX = parseFloat(FOV_MAX.value);
            let FOV_NORMAL = ConfigValue.getItemByKey("FOV_NORMAL");
            if (FOV_NORMAL)
                this.FOV_NORMAL = parseFloat(FOV_NORMAL.value);
            let MOVE_RATE = ConfigValue.getItemByKey("MOVE_RATE");
            if (MOVE_RATE)
                this.MOVE_RATE = parseFloat(MOVE_RATE.value);
            let FOV_RATE = ConfigValue.getItemByKey("FOV_RATE");
            if (FOV_RATE)
                this.FOV_MAX = parseFloat(FOV_RATE.value);
            this.OnChangeScale(0);
        }
        SetMaxFovToNormal() {
            this.FOV_MAX = this.FOV_NORMAL;
            if (this.camera.fieldOfView > this.FOV_MAX) {
                this.camera.fieldOfView = this.FOV_MAX;
                GlobalConfig.scene_scale = (this.FOV_NORMAL - (this.camera.fieldOfView - this.FOV_NORMAL)) / this.FOV_NORMAL;
                GameDispatcher.Inst.event(UIGameEvent.SCENE_RESIZE);
            }
        }
        ChangeToZhengzhou(isZhengzhou) {
            let transformParent = this.transformParent;
            let camera = this.camera;
            let cameraTransform = camera.transform;
            cameraTransform.position.cloneTo(this.onPosition);
            cameraTransform.rotationEuler.cloneTo(this.onRotationEuler);
            let parentRotationEuler = transformParent.rotationEuler;
            parentRotationEuler.setValue(0, isZhengzhou ? -36.397 : 0, 0);
            transformParent.rotationEuler = parentRotationEuler;
            let parentPosition = transformParent.position;
            parentPosition.setValue(0, 0, 0);
            transformParent.position = parentPosition;
            let rotationEuler = this.transform.rotationEuler;
            this.onRotationEuler.cloneTo(rotationEuler);
            this.transform.rotationEuler = rotationEuler;
            let position = this.transform.position;
            this.onPosition.cloneTo(position);
            this.transform.position = position;
            cameraTransform.position = this.onPosition;
            cameraTransform.rotationEuler = this.onRotationEuler;
        }
        OnChangeMove(offsetX, offsetY) {
            if (SceneMgr.Inst.GetCameraBuildScript().isFollowCat)
                return;
            if (CatModel.Inst.selectCatVO != null)
                return;
            this.ChangeToZhengzhou(true);
            let position = this.transform.localPosition;
            let scale = 1 + (1 - GlobalConfig.scene_scale);
            let rateOffestY = this.transform.position.y / this.initY;
            scale *= rateOffestY;
            position.x += scale * offsetX * this.MOVE_RATE;
            position.z += scale * offsetY * this.MOVE_RATE;
            if (position.x < this.MOVE_LEFT)
                position.x = this.MOVE_LEFT;
            else if (position.x > this.MOVE_RIGHT)
                position.x = this.MOVE_RIGHT;
            if (position.z < this.MOVE_BOTTOM)
                position.z = this.MOVE_BOTTOM;
            else if (position.z > this.MOVE_TOP)
                position.z = this.MOVE_TOP;
            this.transform.localPosition = position;
            this.ChangeToZhengzhou(false);
            GameDispatcher.Inst.event(UIGameEvent.SCENE_RESIZE);
        }
        CameraRestTween() {
            return;
            if (SceneMgr.Inst.GetCameraBuildScript().isFollowCat)
                return;
            if (CatModel.Inst.selectCatVO != null)
                return;
            this.ChangeToZhengzhou(true);
            let position = this.transform.localPosition.clone();
            let startPos = this.transform.localPosition.clone();
            let scale = 1 + (1 - GlobalConfig.scene_scale);
            let rateOffestY = this.transform.position.y / this.initY;
            scale *= rateOffestY;
            let bol = false;
            if (position.x < this.MOVE_LEFT + 10)
                position.x = this.MOVE_LEFT + 10;
            else if (position.x > this.MOVE_RIGHT - 10)
                position.x = this.MOVE_RIGHT - 10;
            else
                bol = true;
            let bol2 = false;
            if (position.z < this.MOVE_BOTTOM + 10)
                position.z = this.MOVE_BOTTOM + 10;
            else if (position.z > this.MOVE_TOP - 10)
                position.z = this.MOVE_TOP - 10;
            else
                bol2 = true;
            let bol3 = false;
            let fov = this.camera.fieldOfView;
            fov = Math.max(this.FOV_MIN + 10, Math.min(fov, this.FOV_MAX - 10));
            bol3 = fov == this.camera.fieldOfView;
            if (bol && bol2 && bol3)
                return;
            this.posTween && Laya.Tween.clear(this.posTween);
            this.posTween = Laya.Tween.to(startPos, {
                "x": position.x, "z": position.z, "update": Laya.Handler.create(this, () => {
                    this.ChangeToZhengzhou(true);
                    this.transform.localPosition = startPos;
                    this.ChangeToZhengzhou(false);
                    GameDispatcher.Inst.event(UIGameEvent.SCENE_RESIZE);
                }, [], false)
            }, 300, Laya.Ease.circOut);
            if (bol3)
                return;
            this.fovTween && Laya.Tween.clear(this.fovTween);
            this.fovTween = Laya.Tween.to(this.camera, {
                "fieldOfView": fov, "update": Laya.Handler.create(this, () => {
                    this.ResetSceneScale();
                    GameDispatcher.Inst.event(UIGameEvent.SCENE_RESIZE);
                }, [], false)
            }, 300, Laya.Ease.circOut);
        }
        ClearResetAni() {
            this.posTween && Laya.Tween.clear(this.posTween);
            this.fovTween && Laya.Tween.clear(this.fovTween);
        }
        ResetSceneScale() {
            GlobalConfig.scene_scale = (this.FOV_NORMAL - (this.camera.fieldOfView - this.FOV_NORMAL)) / this.FOV_NORMAL;
        }
        CheckHideIcon() {
            return this.camera.fieldOfView > 60;
        }
        OnChangeScale(offset) {
            let fov = this.camera.fieldOfView + offset * this.FOV_RATE;
            if (fov < this.FOV_MIN)
                fov = this.FOV_MIN;
            else if (fov > this.FOV_MAX)
                fov = this.FOV_MAX;
            this.camera.fieldOfView = fov;
            GlobalConfig.scene_scale = (this.FOV_NORMAL - (this.camera.fieldOfView - this.FOV_NORMAL)) / this.FOV_NORMAL;
            GameDispatcher.Inst.event(UIGameEvent.SCENE_RESIZE);
        }
        StopAni() {
            this.animator.enabled = false;
            Laya.timer.clear(this, this.playComplete);
        }
        PlayState(actionName, handler = null, speed = 1) {
            this.animator.enabled = true;
            this.animator.speed = speed;
            this.animator.play(actionName, 0, 0);
            if (handler == null)
                return;
            Laya.timer.frameLoop(1, this, this.playComplete, [handler, actionName]);
        }
        playComplete(handler, actionName) {
            if (this.animator == null)
                return;
            let playState = this.animator.getControllerLayer().getCurrentPlayState();
            if (playState != null && playState.animatorState.name == actionName && playState.normalizedTime >= 1) {
                this.StopAni();
                if (handler != null)
                    handler.run();
            }
        }
    }

    class CameraSelectedScript extends Laya.Script3D {
        constructor() {
            super();
            this.camera = null;
            this._transform = null;
            this.onPosition = null;
            this.onRotationEuler = null;
            this.onFov = 0;
            this.isTween = false;
            this.fllowTarget = null;
            this.ROTATE_RATE = 0.5;
            this.ROTATE_MIN_X = -95;
            this.ROTATE_MAX_X = -20;
            this.FOV_RATE = 0.1;
            this.FOV_MIN = 40;
            this.FOV_MAX = 120;
            this.FOV_NORMAL = 44;
            this.norDis = 40;
            this.buildVo = null;
            this.lookCat = false;
            this.minCatDis = 5;
            this.maxCatDis = 40;
            this.minBuildDis = 13;
            this.maxBuildDis = 50;
            this.scaleRate = 0.1;
            this.resizeTime = 0;
            this.targetRot = new Laya.Vector3(-47.027, 180 - 36.397, 0);
            this._isPanningFollowCat = false;
            this.out = new Laya.Vector3();
            this.initRotationEuler = new Laya.Vector3(-47.027, 180 - 36.397, 0);
            this.initPosition = new Laya.Vector3(25.47, 73.68, -48.15);
        }
        onAwake() {
            this.camera = this.owner.getChildByName("Main Camera");
            this.camera.enableHDR = false;
            this._transform = this.owner.transform;
            let FOV_MIN = ConfigValue.getItemByKey("FOV_MIN");
            if (FOV_MIN)
                this.FOV_MIN = parseFloat(FOV_MIN.value);
            let FOV_MAX = ConfigValue.getItemByKey("FOV_MAX");
            if (FOV_MAX)
                this.FOV_MAX = parseFloat(FOV_MAX.value);
            let FOV_NORMAL = ConfigValue.getItemByKey("FOV_NORMAL");
            if (FOV_NORMAL)
                this.FOV_NORMAL = parseFloat(FOV_NORMAL.value);
            let FOV_RATE = ConfigValue.getItemByKey("FOV_RATE");
            if (FOV_RATE)
                this.FOV_RATE = parseFloat(FOV_RATE.value);
            let CAMERA_CAT_MINDIS = ConfigValue.getItemByKey("CAMERA_CAT_MINDIS");
            if (CAMERA_CAT_MINDIS)
                this.minCatDis = parseFloat(CAMERA_CAT_MINDIS.value);
            let CAMERA_CAT_MAXDIS = ConfigValue.getItemByKey("CAMERA_CAT_MAXDIS");
            if (CAMERA_CAT_MAXDIS)
                this.maxCatDis = parseFloat(CAMERA_CAT_MAXDIS.value);
            let CAMERA_BUILD_MINDIS = ConfigValue.getItemByKey("CAMERA_BUILD_MINDIS");
            if (CAMERA_BUILD_MINDIS)
                this.minBuildDis = parseFloat(CAMERA_BUILD_MINDIS.value);
            let CAMERA_BUILD_MAXDIS = ConfigValue.getItemByKey("CAMERA_BUILD_MAXDIS");
            if (CAMERA_BUILD_MAXDIS)
                this.maxBuildDis = parseFloat(CAMERA_BUILD_MAXDIS.value);
            let CAMERA_SELECT_SCALE_RATE = ConfigValue.getItemByKey("CAMERA_SELECT_SCALE_RATE");
            if (CAMERA_SELECT_SCALE_RATE)
                this.scaleRate = parseFloat(CAMERA_SELECT_SCALE_RATE.value);
        }
        get transform() {
            return this._transform;
        }
        onUpdate() {
            if (this.fllowTarget == null)
                return;
            if (this.selectBuildVo)
                return;
            let position = this.transform.position;
            this.fllowTarget.position.cloneTo(position);
            this.transform.position = position;
            if (this.isPanningFollowCat) {
                if (Laya.timer.currTimer - this.resizeTime > 30) {
                    this.resizeTime = Laya.timer.currTimer;
                    GameDispatcher.Inst.event(UIGameEvent.SCENE_RESIZE);
                }
            }
        }
        SelectedBuild(buildVO, isMainEnter = true) {
            if (this.isTween)
                return;
            if (isMainEnter) {
                this.OverFollowCat();
            }
            this.fllowTarget = null;
            let build = EntityModel.Inst.GetBuild(buildVO.objId);
            if (build) {
                this.isTween = true;
                this.buildVo = buildVO;
                let position = buildVO.buildRes.position;
                this.ROTATE_MIN_X = -95;
                this.ROTATE_MAX_X = -20;
                this.lookCat = false;
                let toPos = new Laya.Vector3();
                toPos.setValue(-parseFloat(position[0]), parseFloat(position[1]), parseFloat(position[2]));
                this.SelectBuildTween(buildVO.buildRes, isMainEnter);
            }
        }
        SelectBuildTween(buildRes, isMainEnter = true) {
            GameDispatcher.Inst.event(UIGameEvent.ISHIDE_ICON, true);
            Laya.Tween.clearAll(this.camera);
            let cameraParPosStr = buildRes.position;
            let centerPos = this.GetPosByStr(cameraParPosStr);
            if (isMainEnter) {
                GameDispatcher.Inst.event(UIGameEvent.CAMERA_TWEEN_START);
                this.ResetCameraParent();
                let cameraParRotStr = buildRes.selectCameraParRot;
                let cameraParRot = this.GetRotByStr(cameraParRotStr);
                let cameraTrans = this.camera.transform;
                let cameraPos = cameraTrans.position.clone();
                let cameraRot = cameraTrans.rotationEuler.clone();
                cameraParRot.y = -parseFloat(cameraParRotStr[1]);
                this.transform.position = centerPos;
                this.transform.rotationEuler = cameraParRot;
                this.camera.transform.position = cameraPos;
                this.camera.transform.rotationEuler = cameraRot;
                let selectCameraPos = buildRes.selectCameraPos;
                let cameraNewPos = this.GetPosByStr(selectCameraPos);
                let selectCameraRotStr = buildRes.selectCameraRot;
                let cameraNewRot = this.GetRotByStr(selectCameraRotStr);
                Laya.Tween.to(this.camera, {
                    "localPosition": cameraNewPos,
                    "localRotationEuler": cameraNewRot,
                    "fieldOfView": 55
                }, 500, Laya.Ease.circOut, Laya.Handler.create(this, () => {
                    let cameraTrans = this.camera.transform;
                    let cameraPos = cameraTrans.position.clone();
                    let cameraRot = cameraTrans.rotationEuler.clone();
                    let cameraParRotE = this.transform.rotationEuler;
                    cameraParRotE.y = cameraRot.y;
                    cameraParRotE.x = cameraRot.x;
                    this.transform.rotationEuler = cameraParRotE;
                    cameraTrans.position = cameraPos;
                    cameraTrans.rotationEuler = cameraRot;
                    this.onTweenComplete();
                    GlobalConfig.scene_scale = (this.FOV_NORMAL - (this.camera.fieldOfView - this.FOV_NORMAL)) / this.FOV_NORMAL;
                    GameDispatcher.Inst.event(UIGameEvent.SCENE_RESIZE);
                })).update = Laya.Handler.create(this, this.SelectBuildTweenUpdate, null, false);
            }
            else {
                this.transform.position = centerPos;
                this.onTweenComplete();
            }
        }
        SelectBuildTweenUpdate() {
            this.ResetSceneScale();
        }
        OverBuildSelected() {
            if (!this.buildVo)
                return;
            Laya.Tween.clearAll(this.camera);
            SceneMgr.Inst.GetCameraScript().ChangeToZhengzhou(false);
            let previewCameraPosStr = this.buildVo.buildRes.previewCameraPos;
            let pos = this.GetPosByStr(previewCameraPosStr);
            let cameraRotE = this.camera.transform.rotationEuler;
            cameraRotE = MathUtil.ClampRotationEulerRange(cameraRotE, this.targetRot);
            this.camera.transform.rotationEuler = cameraRotE;
            GameDispatcher.Inst.event(UIGameEvent.CAMERA_TWEEN_START);
            Laya.Tween.to(this.camera, {
                "position": pos,
                "rotationEuler": this.targetRot,
                "fieldOfView": parseFloat(previewCameraPosStr[3])
            }, 500, Laya.Ease.circOut, Laya.Handler.create(this, this.OnBuildOverSelectedComp)).update = Laya.Handler.create(this, this.SelectBuildTweenUpdate, null, false);
            this.buildVo = null;
        }
        ResetSelectBuild() {
            this.buildVo = null;
        }
        OnBuildOverSelectedComp() {
            let isHideIcon = false;
            GameDispatcher.Inst.event(UIGameEvent.ISHIDE_ICON, isHideIcon);
            GlobalConfig.scene_scale = (this.FOV_NORMAL - (this.camera.fieldOfView - this.FOV_NORMAL)) / this.FOV_NORMAL;
            GameDispatcher.Inst.event(UIGameEvent.SCENE_RESIZE);
            this.onTweenComplete();
        }
        StopCameraTween() {
            this.buildVo = null;
            Laya.Tween.clearAll(this.camera);
        }
        FollowCat(cat) {
            if (cat == null)
                return;
            let catTransform = cat.transform;
            this.ResetTransform(catTransform);
            this.fllowTarget = catTransform;
        }
        get isFollowCat() {
            return this.fllowTarget != null;
        }
        OverFollowCat() {
            this._isPanningFollowCat = false;
            this.fllowTarget = null;
            this.ResetCameraParent();
            CatModel.Inst.selectCatVO = null;
        }
        ResetTransform(catTransform) {
            let transformParent = this.transform;
            let camera = this.camera;
            let cameraTransform = camera.transform;
            let onPosition = cameraTransform.position.clone();
            let onRotationEuler = cameraTransform.rotationEuler.clone();
            let parentPosition = transformParent.position;
            catTransform.position.cloneTo(parentPosition);
            transformParent.position = parentPosition;
            let rotationEuler = cameraTransform.rotationEuler;
            onRotationEuler.cloneTo(rotationEuler);
            cameraTransform.rotationEuler = rotationEuler;
            let position = cameraTransform.position;
            onPosition.cloneTo(position);
            cameraTransform.position = position;
        }
        get isPanningFollowCat() {
            return this._isPanningFollowCat;
        }
        PanningFollowCat(catVO) {
            if (this.selectBuildVo)
                return;
            if (CatModel.Inst.selectCatVO)
                return;
            let cat = EntityModel.Inst.GetCat(catVO.objId);
            if (!cat)
                return;
            this.OverFollowCat();
            CatModel.Inst.selectCatVO = catVO;
            let catTransform = cat.transform;
            this.fllowTarget = catTransform;
            let camera = this.camera;
            let onRotationEuler = camera.transform.rotationEuler.clone();
            let onPosition = camera.transform.position.clone();
            let parentPosition = this.transform.position;
            catTransform.position.cloneTo(parentPosition);
            this.transform.position = parentPosition;
            let parentRotationEuler = this.transform.rotationEuler.clone();
            parentRotationEuler.setValue(0, 0, 0);
            this.transform.rotationEuler = parentRotationEuler;
            let rotationEuler = camera.transform.rotationEuler;
            onRotationEuler.cloneTo(rotationEuler);
            camera.transform.rotationEuler = rotationEuler;
            let position = camera.transform.position;
            onPosition.cloneTo(position);
            camera.transform.position = position;
            this._isPanningFollowCat = true;
            let offx = -39.8;
            let offy = 72.18;
            let offz = -51.1;
            let offrx = 47.027;
            let offry = 36.397;
            let offrz = 0;
            let fov = 21;
            let obj = {
                "x": camera.transform.localPosition.x, "y": camera.transform.localPosition.y, "z": camera.transform.localPosition.z,
                "rx": camera.transform.localRotationEuler.x, "ry": camera.transform.localRotationEuler.y, "rz": camera.transform.localRotationEuler.z,
                "fov": this.onFov
            };
            GameDispatcher.Inst.event(UIGameEvent.CAMERA_TWEEN_START);
            Laya.Tween.to(obj, {
                "x": -offx, "y": offy, "z": offz,
                "rx": -offrx, "ry": 180 - offry, "rz": offrz, "fov": fov
            }, 500, Laya.Ease.circOut, Laya.Handler.create(this, this.PanningFollowCatComp, [cat])).update = Laya.Handler.create(this, this.onCameraUpdate, [camera.transform, obj], false);
        }
        PanningFollowCatComp(cat) {
            this.FollowCat(cat);
            GameDispatcher.Inst.event(UIGameEvent.CAMERA_MOVE_COMPLETE);
        }
        get selectVo() {
            return this.buildVo || CatModel.Inst.selectCatVO;
        }
        get selectBuildVo() {
            return this.buildVo;
        }
        cameraIsTween() {
            return this.isTween;
        }
        ClearOnCat() {
            let onCatVO = CatModel.Inst.selectCatVO;
            if (onCatVO == null)
                return;
            if (onCatVO.isForciblyShow) {
                onCatVO.isForciblyShow = false;
                let cat = EntityModel.Inst.GetCat(onCatVO.objId);
                if (cat != null) {
                    cat.OnUpdateSelectedCat();
                }
            }
        }
        SelectedCat(catVO, isForciblyAni = false) {
            if (this.isTween || catVO == null)
                return;
            if (this.onPosition == null) {
                this.OverFollowCat();
            }
            this.ClearOnCat();
            CatModel.Inst.selectCatVO = catVO;
            let cat = EntityModel.Inst.GetCat(catVO.objId);
            if (cat) {
                this.isTween = true;
                catVO.isForciblyShow = true;
                cat.OnUpdateSelectedCat();
                this.ROTATE_MIN_X = -5;
                this.ROTATE_MAX_X = 74.427;
                this.lookCat = true;
                let catTransform = cat.transform;
                this.fllowTarget = catTransform;
                this.SelectedEntity(catTransform.position, 0.05649567, 2.81, -21.74921, 11.856, 0, 0, 50, isForciblyAni);
            }
        }
        SelectedEntity(toPos, offx, offy, offz, offrx, offry, offrz, fov, isForciblyAni) {
            GameDispatcher.Inst.event(UIGameEvent.ISHIDE_ICON, true);
            let camera = this.camera;
            if (this.onPosition == null) {
                this.onPosition = camera.transform.position.clone();
                this.onRotationEuler = camera.transform.rotationEuler.clone();
                this.onFov = camera.fieldOfView;
                isForciblyAni = true;
            }
            if (isForciblyAni) {
                let parentPosition = this.transform.position;
                toPos.cloneTo(parentPosition);
                let parentRotationEuler = this.transform.rotationEuler.clone();
                parentRotationEuler.setValue(0, -36.397, 0);
                this.transform.rotationEuler = MathUtil.ClampRotationEulerRange(this.transform.rotationEuler, parentRotationEuler);
                Laya.Tween.to(this.transform, {
                    "position": parentPosition, "rotationEuler": parentRotationEuler
                }, 500, Laya.Ease.circOut);
                let obj = {
                    "x": camera.transform.localPosition.x, "y": camera.transform.localPosition.y, "z": camera.transform.localPosition.z,
                    "rx": camera.transform.localRotationEuler.x, "ry": camera.transform.localRotationEuler.y, "rz": camera.transform.localRotationEuler.z,
                    "fov": this.onFov
                };
                GameDispatcher.Inst.event(UIGameEvent.CAMERA_TWEEN_START);
                Laya.Tween.to(obj, {
                    "x": -offx, "y": offy, "z": offz,
                    "rx": -offrx, "ry": 180 - offry, "rz": offrz, "fov": fov
                }, 500, Laya.Ease.circOut, Laya.Handler.create(this, this.onTweenComplete)).update = Laya.Handler.create(this, this.onCameraUpdate, [camera.transform, obj], false);
            }
            else {
                let parentPosition = this.transform.position;
                toPos.cloneTo(parentPosition);
                this.transform.position = parentPosition;
                this.isTween = false;
                let localRotationEuler = camera.transform.localRotationEuler;
                localRotationEuler.x = -offrx;
                camera.transform.localRotationEuler = localRotationEuler;
                let localPosition = camera.transform.localPosition;
                localPosition.setValue(-offx, offy, offz);
                camera.transform.localPosition = localPosition;
                camera.fieldOfView = fov;
                this.onTweenComplete();
            }
        }
        onTweenComplete() {
            this.isTween = false;
            this.norDis = Laya.Vector3.distance(this.camera.transform.position, this.transform.position);
            GameDispatcher.Inst.event(UIGameEvent.CAMERA_MOVE_COMPLETE, this.buildVo);
        }
        onCameraUpdate(transform, obj) {
            let rotationEuler = transform.localRotationEuler;
            rotationEuler.setValue(obj["rx"], obj["ry"], obj["rz"]);
            let position = transform.localPosition;
            position.setValue(obj["x"], obj["y"], obj["z"]);
            transform.localRotationEuler = rotationEuler;
            transform.localPosition = position;
            this.camera.fieldOfView = obj["fov"];
        }
        ResetSceneScale() {
            GlobalConfig.scene_scale = (this.FOV_NORMAL - (this.camera.fieldOfView - this.FOV_NORMAL)) / this.FOV_NORMAL;
            GameDispatcher.Inst.event(UIGameEvent.SCENE_RESIZE);
        }
        ClearSelectedCat() {
            this.ClearOnCat();
            CatModel.Inst.selectCatVO = null;
            this.fllowTarget = null;
        }
        OverSelected() {
            if (CatModel.Inst.selectCatVO == null)
                return;
            if (this.onPosition == null)
                return;
            this.ClearSelectedCat();
            let camera = this.camera;
            let transform = camera.transform;
            let curPosition = transform.position.clone();
            let curRotationEuler = transform.rotationEuler.clone();
            let curFov = camera.fieldOfView;
            let position = this.transform.position;
            position.setValue(0, 0, 0);
            let rotationE = this.transform.rotationEuler;
            rotationE.setValue(0, 0, 0);
            this.transform.rotationEuler = MathUtil.ClampRotationEulerRange(this.transform.rotationEuler, rotationE);
            Laya.Tween.to(this.transform, {
                "position": position, "rotationEuler": rotationE
            }, 500, Laya.Ease.circOut);
            let obj = {
                x: transform.position.x, y: transform.position.y, z: transform.position.z,
                rx: transform.rotationEuler.x, ry: transform.rotationEuler.y, rz: transform.rotationEuler.z,
                fov: curFov
            };
            Laya.Tween.to(obj, {
                "x": this.onPosition.x, "y": this.onPosition.y, "z": this.onPosition.z,
                "rx": this.onRotationEuler.x, "ry": this.onRotationEuler.y, "rz": this.onRotationEuler.z,
                "fov": this.onFov
            }, 500, Laya.Ease.circOut, Laya.Handler.create(this, this.onOverComplete)).update = Laya.Handler.create(this, this.onCameraOverUpdate, [this.camera.transform, obj], false);
            this.onPosition = null;
            this.onRotationEuler = null;
            this.onFov = 0;
        }
        onCameraOverUpdate(transform, obj) {
            let rotationEuler = transform.rotationEuler;
            rotationEuler.setValue(obj["rx"], obj["ry"], obj["rz"]);
            let position = transform.position;
            position.setValue(obj["x"], obj["y"], obj["z"]);
            transform.rotationEuler = rotationEuler;
            transform.position = position;
            this.camera.fieldOfView = obj["fov"];
            GlobalConfig.scene_scale = (this.FOV_NORMAL - (this.camera.fieldOfView - this.FOV_NORMAL)) / this.FOV_NORMAL;
            GameDispatcher.Inst.event(UIGameEvent.SCENE_RESIZE);
        }
        onOverComplete() {
            this.isTween = false;
            this.fllowTarget = null;
            GlobalConfig.scene_scale = (this.FOV_NORMAL - (this.camera.fieldOfView - this.FOV_NORMAL)) / this.FOV_NORMAL;
            GameDispatcher.Inst.event(UIGameEvent.SCENE_RESIZE);
            let isHideIcon = false;
            GameDispatcher.Inst.event(UIGameEvent.ISHIDE_ICON, isHideIcon);
        }
        ChangeRotationEuler(adddx, addy) {
            let rate = this.ROTATE_RATE;
            let minX = this.ROTATE_MIN_X;
            let maxX = this.ROTATE_MAX_X;
            let rotationEuler = this.transform.rotationEuler;
            rotationEuler = MathUtil.changeToM180To180(rotationEuler);
            if (this.lookCat) {
                addy = -addy;
            }
            rotationEuler.y += adddx * rate;
            rotationEuler.x -= addy * rate;
            if (rotationEuler.x < minX && addy > 0) {
                rotationEuler.x = minX;
            }
            else if (rotationEuler.x >= maxX && addy < 0) {
                rotationEuler.x = maxX;
            }
            this.transform.rotationEuler = rotationEuler;
            GlobalConfig.scene_scale = (this.FOV_NORMAL - (this.camera.fieldOfView - this.FOV_NORMAL)) / this.FOV_NORMAL;
            GameDispatcher.Inst.event(UIGameEvent.SCENE_RESIZE);
        }
        get minDis() {
            if (this.buildVo) {
                return this.minBuildDis;
            }
            return this.minCatDis;
        }
        get maxDis() {
            if (this.buildVo) {
                return this.maxBuildDis;
            }
            return this.maxCatDis;
        }
        OnChangeScale(offest) {
            let cameraTrans = this.camera.transform;
            let targetTrans = this.transform;
            let dis = Laya.Vector3.distance(cameraTrans.position, targetTrans.position);
            if (offest < 0 && dis < this.minDis || offest > 0 && dis > this.maxDis) {
                offest = 0;
            }
            let out = this.out;
            Laya.Vector3.subtract(targetTrans.position, cameraTrans.position, out);
            Laya.Vector3.normalize(out, out);
            Laya.Vector3.scale(out, -offest * this.scaleRate, out);
            let offLen = Laya.Vector3.scalarLength(out);
            if (offest > 0) {
                offLen = Math.min(this.maxDis - dis, offLen);
            }
            else if (offest < 0) {
                offLen = Math.min(dis - this.minDis, offLen);
            }
            Laya.Vector3.normalize(out, out);
            Laya.Vector3.scale(out, offLen, out);
            cameraTrans.translate(out, false);
            GlobalConfig.scene_scale = (this.FOV_NORMAL - (this.camera.fieldOfView - this.FOV_NORMAL)) / this.FOV_NORMAL * (this.norDis / dis);
            GameDispatcher.Inst.event(UIGameEvent.SCENE_RESIZE);
        }
        ForwardCat(catVO, offx, offy, offz, offrx, offry, offrz, fov) {
            let cat = EntityModel.Inst.GetCat(catVO.objId);
            if (cat == null)
                return;
            let transform = this.camera.transform;
            let catTrans = cat.transform;
            let catPos = catTrans.position;
            let catRotE = catTrans.rotationEuler.clone();
            catRotE.y -= 180;
            GameDispatcher.Inst.event(UIGameEvent.CAMERA_TWEEN_START);
            this.transform.rotationEuler = MathUtil.ClampRotationEulerRange(this.transform.rotationEuler, catRotE);
            Laya.Tween.to(this.transform, {
                "rotationEuler": { x: catRotE.x, y: catRotE.y, z: catRotE.z },
                "position": catPos
            }, 1000, Laya.Ease.circOut);
            let obj = {
                x: transform.localPosition.x, y: transform.localPosition.y, z: transform.localPosition.z,
                "rx": transform.localRotationEuler.x, "ry": transform.localRotationEuler.y, "rz": transform.localRotationEuler.z,
                "fov": this.onFov
            };
            Laya.Tween.to(obj, {
                "x": -offx, "y": offy, "z": offz,
                "rx": -offrx, "ry": 180 - offry, "rz": offrz, "fov": fov
            }, 1000, Laya.Ease.circOut, Laya.Handler.create(this, () => {
                GameDispatcher.Inst.event(UIGameEvent.CAMERA_MOVE_COMPLETE);
            })).update = Laya.Handler.create(this, this.onCameraUpdate, [transform, obj], false);
        }
        ResetCameraParent() {
            this.onPosition = null;
            this.onRotationEuler = null;
            this.onFov = 0;
            SceneMgr.Inst.GetCameraScript().StopAni();
            let camera = this.camera;
            let cameraTransform = camera.transform;
            let curPosition = cameraTransform.position.clone();
            let curRotationEuler = cameraTransform.rotationEuler.clone();
            let parentRotationEuler = this.transform.rotationEuler;
            parentRotationEuler.setValue(0, 0, 0);
            this.transform.rotationEuler = parentRotationEuler;
            let parentPosition = this.transform.position;
            parentPosition.setValue(0, 0, 0);
            this.transform.position = parentPosition;
            cameraTransform.position = curPosition;
            cameraTransform.rotationEuler = MathUtil.changeToM180To180(curRotationEuler);
            ;
        }
        CameraTweenTo(complete = null, roteteEuler = null, pos = null, fov = 44, time = 500) {
            if (roteteEuler == null) {
                roteteEuler = this.initRotationEuler;
            }
            if (pos == null) {
                pos = this.initPosition;
            }
            let transform = this.camera.transform;
            let toObj = {
                "x": pos.x, "y": pos.y, "z": pos.z,
                "rx": roteteEuler.x, "ry": roteteEuler.y, "rz": roteteEuler.z,
                "fov": fov
            };
            if (time > 0) {
                let obj = {
                    x: transform.position.x, y: transform.position.y, z: transform.position.z,
                    rx: transform.rotationEuler.x, ry: transform.rotationEuler.y, rz: transform.rotationEuler.z,
                    fov: this.camera.fieldOfView
                };
                GameDispatcher.Inst.event(UIGameEvent.CAMERA_TWEEN_START);
                Laya.Tween.to(obj, toObj, time, Laya.Ease.circOut, Laya.Handler.create(this, () => {
                    complete && complete.run();
                    GameDispatcher.Inst.event(UIGameEvent.CAMERA_MOVE_COMPLETE);
                })).update = Laya.Handler.create(this, this.onCameraTweenOverUpdate, [this.camera.transform, obj], false);
            }
            else {
                this.onCameraTweenOverUpdate(this.camera.transform, toObj);
            }
        }
        onCameraTweenOverUpdate(transform, obj) {
            let rotationEuler = transform.rotationEuler;
            rotationEuler.setValue(obj["rx"], obj["ry"], obj["rz"]);
            let position = transform.position;
            position.setValue(obj["x"], obj["y"], obj["z"]);
            transform.rotationEuler = rotationEuler;
            transform.position = position;
            this.camera.fieldOfView = obj["fov"];
            GameDispatcher.Inst.event(UIGameEvent.SCENE_RESIZE);
        }
        GetPosByStr(strArr) {
            return new Laya.Vector3(-parseFloat(strArr[0]), parseFloat(strArr[1]), parseFloat(strArr[2]));
        }
        GetRotByStr(strArr) {
            return new Laya.Vector3(-parseFloat(strArr[0]), 180 - parseFloat(strArr[1]), parseFloat(strArr[2]));
        }
        MoveBuild(buildVO) {
            if (buildVO == null)
                return;
            if (this.buildVo)
                return;
            this.isTween = false;
            this.fllowTarget = null;
            Laya.Tween.clearAll(this.camera);
            SceneMgr.Inst.GetCameraScript().ChangeToZhengzhou(false);
            let previewCameraPosStr = buildVO.buildRes.previewCameraPos;
            let pos = this.GetPosByStr(previewCameraPosStr);
            let cameraRotE = this.camera.transform.rotationEuler;
            cameraRotE.x = MathUtil.clampAngle(cameraRotE.x, this.targetRot.x - 180);
            cameraRotE.x = MathUtil.MinNearAngle(cameraRotE.x, this.targetRot.x);
            cameraRotE.y = MathUtil.clampAngle(cameraRotE.y, this.targetRot.y - 180);
            cameraRotE.y = MathUtil.MinNearAngle(cameraRotE.y, this.targetRot.y);
            cameraRotE.z = MathUtil.clampAngle(cameraRotE.z, this.targetRot.z - 180);
            cameraRotE.z = MathUtil.MinNearAngle(cameraRotE.z, this.targetRot.z);
            this.camera.transform.rotationEuler = cameraRotE;
            GameDispatcher.Inst.event(UIGameEvent.CAMERA_TWEEN_START);
            Laya.Tween.to(this.camera, {
                "position": pos,
                "rotationEuler": this.targetRot,
                "fieldOfView": parseFloat(previewCameraPosStr[3])
            }, 500, Laya.Ease.circOut, Laya.Handler.create(this, this.onMoveBuildComplete)).update = Laya.Handler.create(this, this.onMoveBuildUpdate, null, false);
        }
        onMoveBuildComplete() {
            GlobalConfig.scene_scale = (this.FOV_NORMAL - (this.camera.fieldOfView - this.FOV_NORMAL)) / this.FOV_NORMAL;
            GameDispatcher.Inst.event(UIGameEvent.SCENE_RESIZE);
            this.onTweenComplete();
        }
        onMoveBuildUpdate() {
            GlobalConfig.scene_scale = (this.FOV_NORMAL - (this.camera.fieldOfView - this.FOV_NORMAL)) / this.FOV_NORMAL;
            GameDispatcher.Inst.event(UIGameEvent.SCENE_RESIZE);
        }
        IsSelected() {
            if (this.buildVo)
                return true;
            if (CatModel.Inst.selectCatVO)
                return true;
            return false;
        }
    }

    class CloudScript extends Laya.Script3D {
        constructor() {
            super();
            this.transform = null;
        }
        onAwake() {
            super.onAwake();
            this.transform = this.owner.transform;
        }
        onLateUpdate() {
            this.transform.localRotationEulerY += Laya.timer.delta / 2000;
        }
    }

    class ToonWaterMaterial extends Laya.Material {
        constructor() {
            super();
            this.setShaderName("ToonWaterShader");
        }
        get NoisTexture() {
            return this._shaderValues.getTexture(ToonWaterMaterial.NOISTEXTURE);
        }
        set NoisTexture(value) {
            this._shaderValues.setTexture(ToonWaterMaterial.NOISTEXTURE, value);
        }
        get TilingOffset() {
            return this._shaderValues.getVector3(ToonWaterMaterial.TILINGOFFSET);
        }
        set TilingOffset(value) {
            this._shaderValues.setVector3(ToonWaterMaterial.TILINGOFFSET, value);
        }
        get WaterColor_1() {
            return this._shaderValues.getVector3(ToonWaterMaterial.WATERCOLOR_1);
        }
        set WaterColor_1(value) {
            this._shaderValues.setVector3(ToonWaterMaterial.WATERCOLOR_1, value);
        }
        get WaterColor_2() {
            return this._shaderValues.getVector3(ToonWaterMaterial.WATERCOLOR_2);
        }
        set WaterColor_2(value) {
            this._shaderValues.setVector3(ToonWaterMaterial.WATERCOLOR_2, value);
        }
        get Water_time() {
            return this._shaderValues.getNumber(ToonWaterMaterial.WATERTIME);
        }
        set Water_time(value) {
            this._shaderValues.setNumber(ToonWaterMaterial.WATERTIME, value);
        }
        static initShader() {
            var attributeMap = {
                'a_Position': Laya.VertexMesh.MESH_POSITION0,
                'a_Normal': Laya.VertexMesh.MESH_NORMAL0,
                'a_Texcoord': Laya.VertexMesh.MESH_TEXTURECOORDINATE0,
                'a_Color': Laya.VertexMesh.MESH_COLOR0
            };
            var uniformMap = {
                'u_MvpMatrix': Laya.Shader3D.PERIOD_SPRITE,
                'u_WorldMat': Laya.Shader3D.PERIOD_SPRITE,
                'u_CameraPos': Laya.Shader3D.PERIOD_CAMERA,
                'u_Texture': Laya.Shader3D.PERIOD_MATERIAL,
                'u_Time': Laya.Shader3D.PERIOD_SCENE,
                'TilingOffset': Laya.Shader3D.PERIOD_MATERIAL,
                'u_WaterColor_1': Laya.Shader3D.PERIOD_MATERIAL,
                'u_WaterColor_2': Laya.Shader3D.PERIOD_MATERIAL,
                'u_Water_time': Laya.Shader3D.PERIOD_MATERIAL,
            };
            var vs = `
        
        #include "Lighting.glsl";
        attribute vec4 a_Position;
        attribute vec2 a_Texcoord;
        attribute vec3 a_Normal;
        attribute vec4 a_Color;
        uniform mat4 u_MvpMatrix;
        uniform vec3 u_CameraPos;
        uniform mat4 u_WorldMat;
        //uniform float u_Time;
        varying vec2 v_Texcoord;
        varying vec3 v_Normal;
        varying vec3 v_ViewDir;
        
        
        varying vec4 v_Color;
        varying vec3 v_PositionWorld;
       
        void main()
        {
            
                gl_Position=u_MvpMatrix * a_Position;
                mat3 worldMat=mat3(u_WorldMat);
                v_Normal=worldMat*a_Normal;
          
                v_Texcoord=a_Texcoord;
            
                v_Color = a_Color;
                v_PositionWorld=(u_WorldMat*a_Position).xyz;            
          
           
            v_ViewDir = u_CameraPos- v_PositionWorld;    //相机视角

            gl_Position=remapGLPositionZ(gl_Position); 
        }`;
            var ps = `
        #ifdef FSHIGHPRECISION
            precision highp float;
        #else
            precision mediump float;
        #endif
        #include "Lighting.glsl";
            varying vec2 v_Texcoord;
            varying vec3  v_Normal;
            varying vec3 v_ViewDir;
            varying mat3 ModelViewMatrix;
            varying vec4 v_Color;
            uniform sampler2D u_Texture;
            uniform float u_Time;
            uniform vec3 TilingOffset;
            uniform vec3 u_WaterColor_1;  
            uniform vec3 u_WaterColor_2; 
            uniform float u_Water_time;
        void main()
        {    
            // float T = fract(u_Time*0.1);
            float T = u_Water_time;
            vec3 ViewDir = normalize(v_ViewDir);
            vec3 normal = normalize(v_Normal);
            float F0  = clamp(dot(ViewDir,normal),0.0,1.0);
            vec3 Speed = vec3(-1.0,2.0,0.0);
           vec3 WaterColor =   v_Color.rgb;
           vec3 WaterNois0 =  texture2D(u_Texture,v_Texcoord*TilingOffset.xy+(Speed.xy*T*-1.0)).rgb;
           vec3 WaterNois1 =  texture2D(u_Texture,v_Texcoord*TilingOffset.xy+(WaterNois0.x*0.3)).rgb;
           vec3 F1WaterColor =  WaterColor+smoothstep(0.98,1.0,v_Color.a*(WaterNois0.x+1.0))*u_WaterColor_1;
           vec3 F2WaterColor =  smoothstep(TilingOffset.z,0.5,(WaterNois1.z*1.3)*WaterNois0.x)*u_WaterColor_2+F1WaterColor;
          
             gl_FragColor =  vec4(F2WaterColor,0.75);
        }`;
            var customShader = Laya.Shader3D.add("ToonWaterShader");
            var subShader = new Laya.SubShader(attributeMap, uniformMap);
            customShader.addSubShader(subShader);
            var ShaderPass = subShader.addShaderPass(vs, ps);
            ShaderPass.renderState.cull = Laya.RenderState.CULL_BACK;
            ShaderPass.renderState.blend = Laya.RenderState.BLEND_ENABLE_ALL;
            ShaderPass.renderState.srcBlend = Laya.RenderState.BLENDPARAM_SRC_ALPHA;
            ShaderPass.renderState.dstBlend = Laya.RenderState.BLENDPARAM_ONE_MINUS_SRC_ALPHA;
            ShaderPass.renderState.depthWrite = false;
            ShaderPass.renderState.depthTest = Laya.RenderState.DEPTHTEST_LEQUAL;
        }
    }
    ToonWaterMaterial.NOISTEXTURE = Laya.Shader3D.propertyNameToID("u_Texture");
    ToonWaterMaterial.TILINGOFFSET = Laya.Shader3D.propertyNameToID("TilingOffset");
    ToonWaterMaterial.WATERCOLOR_1 = Laya.Shader3D.propertyNameToID("u_WaterColor_1");
    ToonWaterMaterial.WATERCOLOR_2 = Laya.Shader3D.propertyNameToID("u_WaterColor_2");
    ToonWaterMaterial.WATERTIME = Laya.Shader3D.propertyNameToID("u_Water_time");

    class WaterScript extends Laya.Script3D {
        constructor() {
            super();
            this.curTime = 0;
            this.material = null;
        }
        onAwake() {
            super.onAwake();
            LoadQueue.Inst.loadTexture(UrlUtils.GetShaderPngUrl("water_nois"), Laya.Handler.create(this, this.onLoadComplete));
        }
        onLoadComplete(tex) {
            let meshSprite3D = this.owner;
            if (meshSprite3D == null)
                return;
            let material = new ToonWaterMaterial();
            material.TilingOffset = new Laya.Vector3(1.5, 1.5, 0.3);
            material.NoisTexture = tex;
            material.WaterColor_1 = new Laya.Vector3(0.75, 0.75, 0.75);
            material.WaterColor_2 = new Laya.Vector3(0.15, 0.15, 0.15);
            meshSprite3D.meshRenderer.sharedMaterial = material;
            this.material = material;
            material.alphaTest = false;
            material.renderQueue = Laya.Material.RENDERQUEUE_TRANSPARENT;
        }
        onUpdate() {
            if (this.material == null)
                return;
            let time = Laya.timer.currTimer * 0.0001;
            time = time - Math.floor(time);
            this.material.Water_time = time;
        }
    }

    class SceneEntitys extends Laya.Sprite3D {
        constructor(name, isStatic) {
            super(name, isStatic);
            this.dic = null;
            this.dic = new Dictionary();
        }
        Add(key, entity) {
            let sp = this.dic[key];
            if (sp == null) {
                sp = new Laya.Sprite3D();
                this.addChild(sp);
                this.dic[key] = sp;
            }
            sp.addChild(entity);
        }
        Remove(key, entity) {
            let sp = this.dic[key];
            if (sp == null || entity == null) {
                return;
            }
            sp.removeChild(entity);
        }
        Get(key) {
            let sp = this.dic[key];
            if (sp == null) {
                sp = new Laya.Sprite3D();
                this.addChild(sp);
                this.dic[key] = sp;
            }
            return sp;
        }
    }

    class LevelMgr {
        constructor() {
            this._IsOver = false;
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new LevelMgr();
            return this._Inst;
        }
        Clear() {
            this._IsOver = false;
        }
        IsOver() {
            return this._IsOver;
        }
        SetResult(isSuccess) {
            if (this._IsOver)
                return;
            this._IsOver = true;
            this.onDelay(isSuccess);
        }
        onDelay(isSuccess) {
        }
    }
    LevelMgr._Inst = null;
    LevelMgr.TYPE_GUANQIA = 1;
    LevelMgr.TYPE_WUXIANMOSHI = 2;
    LevelMgr.TYPE_JISU = 3;

    class PrecompiledMgr {
        constructor() {
            this.parentNode = new Laya.Sprite3D();
        }
        static get Inst() {
            if (PrecompiledMgr._Inst == null)
                PrecompiledMgr._Inst = new PrecompiledMgr();
            return PrecompiledMgr._Inst;
        }
        ParseTxt(url = "../pre_urls.txt") {
            Laya.Shader3D.debugMode = true;
            let lf = this;
            Laya.loader.load(url, Laya.Handler.create(this, function (txt) {
                let urlArr = txt.split("\n");
                lf.ParseUrl(urlArr, 0);
            }), null, Laya.Loader.TEXT);
        }
        ParseUrl(urlArr, index) {
            if (index >= urlArr.length) {
                this.onComplete();
                return;
            }
            let url = urlArr[index];
            let binIndex = url.indexOf("bin");
            if (binIndex != -1)
                url = url.substr(binIndex + 4);
            console.log(url);
            url = MathUtil.replaceAll(url, "\\", "/");
            if (url.lastIndexOf(".ls") != -1) {
                Laya.loader.load(url, Laya.Handler.create(this, this.onSceneComplete), null, Laya.Loader.HIERARCHY);
            }
            else {
                Laya.Sprite3D.load(url, Laya.Handler.create(this, this.onSpriteComplete));
            }
            index++;
            Laya.timer.frameOnce(5, this, this.ParseUrl, [urlArr, index]);
        }
        onSceneComplete(scene) {
            let camera = this.GetSprite3D(scene, Laya.Camera);
            if (camera)
                camera.active = false;
            for (let i = 0; i < scene.numChildren; i++) {
                const node = scene.getChildAt(i);
                SceneMgr.Inst.GetScene().addChild(node);
            }
            let position = this.parentNode.transform.position;
            position.y = 3;
            this.parentNode.transform.position = position;
            SceneMgr.Inst.GetScene().addChild(this.parentNode);
        }
        onSpriteComplete(sp) {
            this.parentNode.addChild(sp);
        }
        onComplete() {
            Laya.timer.once(10000, this, this.OnCompile);
        }
        OnCompile() {
            let shaderObj = {};
            let arr;
            for (let i = 0; i < Laya.Shader3D.debugShaderVariantCollection.variantCount; i++) {
                let shadervariant = Laya.Shader3D.debugShaderVariantCollection.getByIndex(i);
                let shaderName = shadervariant.shader.name;
                if (!shaderObj[shaderName])
                    shaderObj[shaderName] = [];
                arr = shaderObj[shaderName];
                let obj = {};
                obj.defineNames = shadervariant.defineNames;
                obj.passIndex = shadervariant.passIndex;
                obj.subShaderIndex = shadervariant.subShaderIndex;
                arr.push(obj);
            }
            console.log("预编译信息：");
            console.log(JSON.stringify(shaderObj));
        }
        GetSprite3D(sp, cls) {
            if (sp == null)
                return null;
            if (sp instanceof Laya.Camera) {
                return sp;
            }
            let length = sp.numChildren;
            for (let i = 0; i < length; i++) {
                const element = sp.getChildAt(i);
                let node = this.GetSprite3D(element, cls);
                if (node != null)
                    return node;
            }
            return null;
        }
    }
    PrecompiledMgr._Inst = null;

    class SceneMgr {
        constructor() {
            this.m_scene = null;
            this.curGameArr = null;
            this.camera = null;
            this.m_cmera = null;
            this.m_build_cmera = null;
            this.m_map = null;
            this.m_build = null;
            this.m_Effect = null;
            this.m_tree = null;
            this.m_fecal = null;
            this.m_gifts = null;
            this.m_event = null;
            this.curGameArr = new Array();
        }
        static get Inst() {
            if (this._inst == null) {
                this._inst = new SceneMgr();
            }
            return this._inst;
        }
        GetCamera() {
            return this.camera;
        }
        GetScene() {
            return this.m_scene;
        }
        reset() {
            this.clear();
        }
        clear() {
            this.clearData();
        }
        clearData() {
            LevelMgr.Inst.Clear();
            MapQuadTree.Inst.Clear();
            this.curGameArr.length = 0;
            Laya.timer.clearAll(this);
            console.log("清理场景...........");
        }
        InitScene() {
            let lf = this;
            if (lf.m_scene == null) {
                LoadQueue.Inst.loadScene3D(UrlUtils.GetSceneUrl("CatIsland_EditScenes"), Laya.Handler.create(lf, function (scene) {
                    Logger.debug(lf, "=====>场景加载完成....");
                    scene.input.multiTouchEnabled = false;
                    lf.m_scene = scene;
                    GameLayer.sceneLayer.addChild(scene);
                    lf.camera = MathUtil.FindSprite3DByName(scene, "Main Camera");
                    lf.m_cmera = lf.camera.addComponent(CameraScript);
                    let cat2_sky_02 = scene.getChildByName("CatIsland_Scenes").getChildByName("sky").getChildByName("cat2_sky").getChildByName("cat2_sky_02");
                    cat2_sky_02.addComponent(CloudScript);
                    let build_camera = MathUtil.FindSprite3DByName(scene, "Camera");
                    lf.m_build_cmera = build_camera.addComponent(CameraSelectedScript);
                    if (lf.m_build == null) {
                        lf.m_build = new SceneEntitys();
                    }
                    scene.addChild(lf.m_build);
                    if (lf.m_map == null) {
                        lf.m_map = new SceneEntitys();
                    }
                    scene.addChild(lf.m_map);
                    if (lf.m_tree == null) {
                        lf.m_tree = new SceneEntitys();
                    }
                    scene.addChild(lf.m_tree);
                    if (lf.m_Effect == null) {
                        lf.m_Effect = new SceneEntitys();
                    }
                    scene.addChild(lf.m_Effect);
                    lf.m_fecal = lf.m_fecal || new SceneEntitys();
                    scene.addChild(lf.m_fecal);
                    lf.m_gifts = lf.m_gifts || new SceneEntitys();
                    scene.addChild(lf.m_gifts);
                    if (lf.m_event == null) {
                        lf.m_event = new SceneEntitys();
                    }
                    scene.addChild(lf.m_event);
                    if (GlobalConfig.stat && GlobalConfig.isRunShader) {
                        PrecompiledMgr.Inst.ParseTxt();
                    }
                    GlobalConfig.isLoadScene = true;
                    GameDispatcher.Inst.event(UIGameEvent.LOAD_SCENE_COMPLETE);
                }));
            }
            LoadQueue.Inst.loadSprite3D(UrlUtils.GetBuildUrl("cat2_boat"));
            LoadQueue.Inst.loadSprite3D(UrlUtils.GetCatUrl("ashort"));
        }
        InitLoadSceneConfig() {
            MapQuadTree.Inst.SetMapInit();
            let url = UrlUtils.GetLevelUrl("CatIsland_EditScenes");
            LoadQueue.Inst.loadBin(url, Laya.Handler.create(this, this.onLoadComplete, [url]));
        }
        onLoadComplete(url, byte) {
            SceneModel.Inst.Parse(byte);
            this.SceneLoadingComplete();
        }
        SceneLoadingComplete() {
            SceneModel.Inst.AddToMap();
            MapQuadTree.Inst.Start();
            this.UpdateLand(LandModel.Inst.level);
        }
        UpdateLand(landLv) {
            let waterParent = MathUtil.FindSprite3DByName(this.GetScene(), "water");
            let waters = ["cat2_island01_sea", "cat2_island02_sea", "cat2_island03_sea"];
            for (let i = 0; i < waters.length; i++) {
                let waterNode = waterParent && waterParent.getChildByName(waters[i]);
                if (waterNode == null)
                    continue;
                if (i == (landLv - 1)) {
                    waterNode.active = true;
                    let sc = waterNode.getComponent(WaterScript);
                    if (sc == null)
                        waterNode.addComponent(WaterScript);
                }
                else {
                    waterNode.active = false;
                    ;
                }
            }
            let Stone = MathUtil.FindSprite3DByName(this.GetScene(), "Stone");
            let Stones = ["land01_Stone", "land02_Stone", "land03_Stone"];
            for (let i = 0; i < Stones.length; i++) {
                let stoneNode = Stone && Stone.getChildByName(Stones[i]);
                if (stoneNode == null)
                    continue;
                stoneNode.active = i < landLv;
            }
            let Ground = MathUtil.FindSprite3DByName(this.GetScene(), "Ground");
            let Grounds = ["Cat2_Scenes_Island01", "Cat2_Scenes_Island02", "Cat2_Scenes_Island03"];
            for (let i = 0; i < Grounds.length; i++) {
                let groundNode = Ground && Ground.getChildByName(Grounds[i]);
                if (groundNode == null)
                    continue;
                groundNode.active = i < landLv;
            }
            let underwater = MathUtil.FindSprite3DByName(this.GetScene(), "underwater");
            let underwaters = ["cat2_underwater_01", "cat2_underwater_02", "cat2_underwater_03"];
            for (let i = 0; i < Grounds.length; i++) {
                let underwaterNode = underwater && underwater.getChildByName(underwaters[i]);
                if (underwaterNode == null)
                    continue;
                underwaterNode.active = i < landLv;
            }
            if (LandModel.Inst.IsMax()) {
                GameDispatcher.Inst.event(UIGameEvent.LAND_PROMPT_ICON, [null, false]);
            }
            else {
                GameDispatcher.Inst.event(UIGameEvent.LAND_PROMPT_ICON, [islandResource.getItemByKey(landLv + 1), true]);
            }
        }
        Get3DTo2D(inPos, outPos = null) {
            if (outPos == null) {
                outPos = new Laya.Vector4();
            }
            let camera = SceneMgr.Inst.GetCamera();
            camera && camera.viewport.project(inPos, camera.projectionViewMatrix, outPos);
            outPos.x = outPos.x / Laya.stage.clientScaleX;
            outPos.y = outPos.y / Laya.stage.clientScaleY;
            return outPos;
        }
        GetCameraScript() {
            return this.m_cmera;
        }
        GetCameraBuildScript() {
            return this.m_build_cmera;
        }
        AddMap(key, entity) {
            return this.m_map.Add(key, entity);
        }
        RemoveMap(key, entity) {
            return this.m_map.Remove(key, entity);
        }
        GetMap(key) {
            return this.m_map.Get(key);
        }
        GetEffects(key) {
            return this.m_Effect.Get(key);
        }
        AddEffect(key, entity) {
            return this.m_Effect.Add(key, entity);
        }
        GetTree(key) {
            return this.m_tree.Get(key);
        }
        AddTree(key, entity) {
            return this.m_tree.Add(key, entity);
        }
        GetBuild(key) {
            return this.m_build.Get(key);
        }
        AddBuild(key, entity) {
            return this.m_build.Add(key, entity);
        }
        AddFecalBall(key, entity) {
            this.m_fecal.Add(key, entity);
        }
        GetFecalBall(key) {
            return this.m_fecal.Get(key);
        }
        RemoveFecalBall(key, entity) {
            this.m_fecal.Remove(key, entity);
            if (entity && !entity.destroyed) {
                entity.destroy();
            }
        }
        AddGift(key, entity) {
            this.m_gifts.Add(key, entity);
        }
        GetGift(key) {
            return this.m_gifts.Get(key);
        }
        RemoveGift(key, entity) {
            this.m_gifts.Remove(key, entity);
        }
        AddEvent(key, entity) {
            this.m_event.Add(key, entity);
        }
        GetEvent(key) {
            return this.m_event.Get(key);
        }
        RemoveEvent(key, entity) {
            this.m_event.Remove(key, entity);
        }
    }

    class UILiXianView extends UIBaseWindow {
        constructor() {
            super("main", "UILiXianView");
            this.txt_desc = null;
            this.txt_gold = null;
            this.btn_get = null;
            this.c1 = null;
            this.btn_adGet = null;
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_CENTER;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            this.c1.selectedIndex = data;
            this.UpdateData();
            this.showBanner();
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                if (this.c1.selectedIndex == 1) {
                    this.btn_close.visible = this.btn_adGet.isShowAd = false;
                    let p2 = AdModel.Inst.getPolicy("p2") && AdModel.Inst.getPolicy("p2").replace(/s/g, "");
                    if (p2 == null) {
                        p2 = 0;
                    }
                    let time = parseInt(p2);
                    if (isNaN(time))
                        time = 0;
                    Laya.timer.once(time * 1000, this, this.onCloseBtns);
                }
            }
            else if ((Laya.Browser.onVVMiniGame || Laya.Browser.onKGMiniGame) && GuideModel.Inst.guideVO == null) {
                if (this.c1.selectedIndex == 1) {
                    this.hideBanner();
                    AdCtrl.Inst.DelayShowInsertAd();
                }
            }
            else if (Laya.Browser.onQGMiniGame) {
                if (this.c1.selectedIndex == 1) {
                    this.hideBanner();
                    AdCtrl.Inst.ShowNatiEmbed(this);
                }
            }
        }
        onCloseBtns() {
            this.btn_close.visible = this.btn_adGet.isShowAd = true;
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_get) {
                this.GetAward(true);
            }
            else if (btn == this.btn_adGet) {
                let handler = Laya.Handler.create(this, this.GetAward);
                PlatformMgr.Inst.showAd(AdType.ad_13, handler);
            }
        }
        GetAward(isSuccess) {
            if (!isSuccess) {
                return;
            }
            let relTime = 0;
            if (this.c1.selectedIndex == 0) {
                relTime = Math.floor(StorageUtils.Inst.GetOfflineInterValTime() / 1000);
            }
            else if (this.c1.selectedIndex == 1) {
                relTime = Math.floor(StorageUtils.Inst.GetlixianAd() / 1000);
                GameDispatcher.Inst.event(UIGameEvent.GET_ADLIXIAN);
            }
            let offlinetime = Math.min(relTime, 2 * 60 * 60);
            let gold = StorageUtils.Inst.GetlixianGoldAward().multipliedBy(offlinetime);
            MoneyModel.gold.add(gold);
            UIAwardView.awardVO.setData({ "count": gold, "tempId": MoneyVo.GOLD });
            UISystem.Inst.CreateWindowView(UIAwardView, UIAwardView.awardVO);
            this.close();
        }
        UpdateData() {
            let relTime = Math.floor(StorageUtils.Inst.GetOfflineInterValTime() / 1000);
            if (this.c1.selectedIndex == 0) {
                this.txt_desc.text = "在离开的" + Math.min(120, Math.floor(relTime / 60)) + "分钟里收集了一些金币!\n(最多2小时)";
            }
            else if (this.c1.selectedIndex == 1) {
                relTime = Math.floor(StorageUtils.Inst.GetlixianAd() / 1000);
                this.txt_desc.text = "是否收看广告领取额外离线奖励?";
            }
            let offlinetime = Math.min(relTime, 2 * 60 * 60);
            let gold = StorageUtils.Inst.GetlixianGoldAward().multipliedBy(offlinetime);
            this.txt_gold.text = MathUtil.goldFormat(gold);
        }
        Reset() {
            super.Reset();
            Laya.timer.clear(this, this.onCloseBtns);
        }
    }

    class OffLineCtr {
        constructor() {
            this.updateOffLine = true;
            Laya.timer.loop(10000, this, this.UpDateLiXianTime);
        }
        static get Inst() {
            return this.inst = this.inst || new OffLineCtr();
        }
        GetLiXian() {
            let offLineTime = StorageUtils.Inst.GetOfflineInterValTime();
            let gold = BuildModel.Inst.GetAllBuildGoldRate(offLineTime / 1000, false);
            let bol = false;
            if (GuideModel.Inst.guideVO == null) {
                if (offLineTime) {
                    let sec = CVModel.Inst.GetValue(CVModel.leaveTime, CVModel.valType_number);
                    if (offLineTime > 1000 * sec && !gold.isZero()) {
                        bol = true;
                        StorageUtils.Inst.SetlixianGoldAward(BuildModel.Inst.GetAllBuildGoldRate());
                        UISystem.Inst.CreateWindowView(UILiXianView, 0);
                        StorageUtils.Inst.SetlixianAd(Math.floor(offLineTime));
                    }
                }
            }
            return bol;
        }
        UpDateLiXianTime() {
            if (this.updateOffLine) {
                StorageUtils.Inst.SetOfflineTime();
            }
        }
        OnActive(bol) {
            if (bol) {
                StorageUtils.Inst.ResetOffLine();
                StorageUtils.Inst.loginTime2 = StorageUtils.getCurTimer();
            }
            else {
                StorageUtils.Inst.SetOfflineTime();
            }
            this.updateOffLine = bol;
        }
    }
    OffLineCtr.inst = null;

    class TaskCtr {
        constructor() {
        }
        static get Inst() {
            return this.inst = this.inst || new TaskCtr();
        }
        init() {
        }
        SetCurMainTask(mainTask) {
            TaskModel.Inst.mainTask = mainTask;
            let mainTaskVos = TaskModel.Inst.GetMainTaskVos();
            let mainVo = null;
            for (let i = 0; i < mainTaskVos.length; i++) {
                mainVo = mainTaskVos[i];
                if (mainTask.tempId == mainVo.taskRes.id) {
                    TaskModel.Inst.mainTaskIndex = i;
                }
            }
            let mainTaskType = mainTask.taskRes.taskType;
            let id = mainTask.taskRes.otherCost;
            mainTask.state = BaseTaskVO.STATE_NORMAL;
            mainTask.pro = 0;
            GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [mainTaskType, id, 0]);
            StorageUtils.Inst.updateMainTask(mainTask.tempId, mainTask.curPro, mainTask.state);
            GameDispatcher.Inst.event(UIGameEvent.UPDATE_TASK, mainTask.type);
        }
        SetNextMainTask() {
            TaskModel.Inst.SetNextMainTask();
        }
        GetDayTaskAward(tasvo) {
            if (!tasvo.canAward)
                return;
            let itemId = tasvo.taskRes.item;
            let num = tasvo.taskRes.num;
            let itemType = itemResource.getItemByKey(itemId).type;
            this.GetAward(itemId, num, itemType);
            tasvo.state = BaseTaskVO.STATE_COMPLETED;
            StorageUtils.Inst.SaveDayTask(tasvo.tempId, tasvo.pro, tasvo.state);
            GameDispatcher.Inst.event(UIGameEvent.UPDATE_TASK, tasvo.type);
            TaskModel.Inst.CheckIsHideBtnRed();
        }
        GetMainTaskAward(mainTaskVo) {
            if (!mainTaskVo.canAward)
                return;
            let itemId = mainTaskVo.taskRes.item;
            let num = mainTaskVo.taskRes.num;
            let itemType = itemResource.getItemByKey(itemId).type;
            this.GetAward(itemId, num, itemType);
            mainTaskVo.state = BaseTaskVO.STATE_COMPLETED;
            this.SetNextMainTask();
            StorageUtils.Inst.updateMainTask(TaskModel.Inst.mainTask.tempId, TaskModel.Inst.mainTask.curPro, TaskModel.Inst.mainTask.state);
            TaskModel.Inst.CheckIsHideBtnRed();
        }
        GetAllDayTaskCompAward() {
            if (TaskModel.Inst.isGetDaysTask || TaskModel.Inst.allDayTaskState != BaseTaskVO.STATE_CAN_AWARD)
                return;
            let itemId = TaskModel.Inst.DAYTASKS_COMP_AWARDID;
            let num = TaskModel.Inst.DAYTASKS_COMP_DIMA;
            let itemType = itemResource.getItemByKey(itemId).type;
            this.GetAward(itemId, num, itemType);
            TaskModel.Inst.isGetDayTaskAllComp = true;
            StorageUtils.Inst.SaveAllDayTaskComp(TaskModel.Inst.isGetDaysTask);
            GameDispatcher.Inst.event(UIGameEvent.UPDATE_TASK);
            TaskModel.Inst.CheckIsHideBtnRed();
        }
        OnMainTaskComp(type, itemID, isAdd = 1) {
            let mainTask = TaskModel.Inst.mainTask;
            let taskRes = mainTask.taskRes;
            let mainTaskType = taskRes.taskType;
            let id = taskRes.otherCost;
            if (mainTaskType == TaskModel.TASKTYPE_OTHER) {
            }
            else if (mainTaskType == type) {
                if (type == TaskType.TASKTYPE_UNLOCK_BUILD) {
                    let buildVos = BuildModel.Inst.GetUnlockBuilds();
                    for (let i = 0; i < buildVos.length; i++) {
                        if (buildVos[i].buildRes.id == id && id > 0) {
                            mainTask.curPro += 1;
                            break;
                        }
                    }
                    if (id <= 0) {
                        mainTask.curPro += isAdd;
                    }
                }
                else if (type == TaskType.TASKTYPE_UP_BUILD) {
                    if (id > 0) {
                        if (itemID == id) {
                            mainTask.curPro += isAdd;
                        }
                    }
                    else {
                        mainTask.curPro += isAdd;
                    }
                    let buildVos = BuildModel.Inst.GetUnlockBuilds();
                    let ismax = true;
                    for (let i = 0; i < buildVos.length; i++) {
                        if (buildVos[i].buildRes.type == BuildVO.TYPE_MATOU)
                            continue;
                        if (!buildVos[i].IsMaxLv()) {
                            ismax = false;
                        }
                        if (buildVos[i].buildRes.id == id && id > 0 && (buildVos[i].IsMaxLv())) {
                            mainTask.curPro = mainTask.maxCount;
                            return;
                        }
                    }
                    if (ismax) {
                        mainTask.curPro = mainTask.maxCount;
                    }
                }
                else if (type == TaskType.TASKTYPE_INVITE_CAT) {
                    let catVos = CatModel.Inst.GetHasCatVOs();
                    for (let i = 0; i < catVos.length; i++) {
                        if (catVos[i].catRes.id == id && id > 0) {
                            mainTask.curPro += 1;
                            break;
                        }
                    }
                    if (id <= 0) {
                        mainTask.curPro += isAdd;
                    }
                }
                else if (type == TaskType.TASKTYPE_UP_CAT) {
                    if (id <= 0) {
                        mainTask.curPro += isAdd;
                    }
                    else {
                        if (id == itemID) {
                            mainTask.curPro += isAdd;
                        }
                    }
                }
                else if (type == TaskType.TASKTYPE_GET_CAT) {
                    mainTask.curPro += isAdd;
                }
                else if (type == TaskType.TASKTYPE_MO_CAT) {
                    mainTask.curPro += isAdd;
                }
                else if (type == TaskType.TASKTYPE_GET_MAOQIU) {
                    mainTask.curPro += isAdd;
                    CatFecalBallCtrl.Inst.MakeFecals(mainTask.maxCount - mainTask.curPro);
                }
                else if (type == TaskType.TASKTYPE_WEIMAO) {
                    mainTask.curPro += isAdd;
                    if (AddFishModel.Inst.HasFish()) {
                        mainTask.curPro += 1;
                    }
                }
                else if (type == TaskType.TASKTYPE_LOCKDRAW) {
                    mainTask.curPro += isAdd;
                }
                else if (type == TaskType.TASKTYPE_DRESSUP) {
                    mainTask.curPro += isAdd;
                }
                else if (type == TaskType.TASKTYPE_PAIZHAO) {
                    mainTask.curPro += isAdd;
                }
                else if (type == TaskType.TASKTYPE_DAYTASKCOMP) {
                    if (TaskModel.Inst.dayAllTaskComp) {
                        mainTask.curPro += 1;
                    }
                }
                else if (type == TaskType.TASKTYPE_GIVEDRESS) {
                    mainTask.curPro += isAdd;
                }
                else if (type == TaskType.TASKTYPE_BUILDLEVEL) {
                    let buildVos = BuildModel.Inst.GetUnlockBuilds();
                    let max = 0;
                    for (let i = 0; i < buildVos.length; i++) {
                        if (id > 0) {
                            if (buildVos[i].buildRes.id == id) {
                                max = buildVos[i].level;
                                break;
                            }
                        }
                        else if (buildVos[i].level > max) {
                            max = buildVos[i].level;
                        }
                    }
                    mainTask.curPro = max;
                }
                else if (type == TaskType.TASKTYPE_CATLEVEL) {
                    let catVos = CatModel.Inst.GetAllCatVOs();
                    let max = 0;
                    for (let i = 0; i < catVos.length; i++) {
                        if (id > 0) {
                            if (catVos[i].catRes.id == id) {
                                max = catVos[i].level;
                                break;
                            }
                        }
                        else if (catVos[i].level > max) {
                            max = catVos[i].level;
                        }
                    }
                    mainTask.curPro = max;
                }
                else if (type == TaskType.TASKTYPE_BUYDAOJU) {
                    if (id > 0) {
                        if (itemID instanceof Array) {
                            for (let i = 0; i < itemID.length; i++) {
                                if (itemID[i] instanceof ExchangeVO) {
                                    if (itemID[i].itemRes.id == id) {
                                        mainTask.curPro += isAdd;
                                        break;
                                    }
                                }
                            }
                        }
                        else {
                            if (itemID == id) {
                                mainTask.curPro += isAdd;
                            }
                        }
                    }
                    else {
                        mainTask.curPro += isAdd;
                    }
                }
                else if (type == TaskType.TASKTYPE_BUILDSCORE) {
                    let score = 0;
                    score = BuildModel.Inst.GetBuildScore(id);
                    mainTask.curPro = score;
                }
                else if (type == TaskType.TASKTYPE_GIVEQIULU) {
                    if (id <= 0) {
                        mainTask.curPro += isAdd;
                    }
                    else if (id == itemID) {
                        mainTask.curPro += isAdd;
                    }
                }
                else if (type == TaskType.TASKTYPE_UNLOCKISLAND) {
                    if (id <= 0) {
                        mainTask.curPro += isAdd;
                    }
                    else if (id <= LandModel.Inst.level) {
                        mainTask.curPro += 1;
                    }
                }
                else if (type == TaskType.TASKTYPE_ADDMATOUNUM) {
                    let num = BuildModel.Inst.matouBuildVO.level;
                    mainTask.curPro = num;
                }
                else if (type == TaskType.TASKTYPE_CATBAIZS) {
                    if (id <= 0) {
                        mainTask.curPro += isAdd;
                    }
                    else if (id == itemID) {
                        mainTask.curPro += isAdd;
                    }
                }
                else if (type == TaskType.TASKTYPE_ADDSERVER) {
                    if (id <= 0) {
                        mainTask.curPro += isAdd;
                    }
                    else if (id == itemID) {
                        mainTask.curPro += isAdd;
                    }
                }
                else if (type == TaskType.TASKTYPE_ADDCATHOME) {
                    if (BuildModel.Inst.isMaowuMaxLevel) {
                        mainTask.curPro += 1;
                    }
                    else {
                        mainTask.curPro += isAdd;
                    }
                    if (BuildModel.Inst.maowuBuildVO.stage >= 1) {
                        mainTask.curPro += 1;
                    }
                }
            }
        }
        GetAward(itemId, num, itemType) {
            if (itemId == 1) {
                MoneyModel.gold.add(num);
            }
            else if (itemId == 2) {
                MoneyModel.diamond.add(num);
            }
            else if (itemId == 13) {
                let goldNum = BuildModel.Inst.GetAllBuildGoldRate(num * 60 * 60, false);
                MoneyModel.gold.add(goldNum);
            }
            else if (itemType == ItemModel.TYPE_ITEM || itemType == ItemModel.TYPE_BUILD) {
                ItemCtrl.Inst.AddItemVO(itemId, num);
            }
            else if (itemType == ItemModel.TYPE_SKIN) {
                this.AddSkinAward(itemId, num);
            }
        }
        AddSkinAward(itemId, num) {
            let skinResArr = skinResource.items;
            for (let i = 0; i < skinResArr.length; i++) {
                let res = skinResArr[i];
                if (res.itemId == itemId) {
                    SkinModel.Inst.AddSkin(res.id, num);
                    break;
                }
            }
        }
    }
    TaskCtr.inst = null;

    class DriftingBottleTaskVo extends BaseTaskVO {
        constructor(tempId, taskRes, award) {
            super(tempId, 0);
            this._taskRes = taskRes;
            this._award = award;
        }
        setup(data) {
            this.pro = data.pro;
            this.state = data.state;
        }
        get award() {
            return this._award;
        }
        get taskRes() {
            return this._taskRes;
        }
        get curPro() {
            return this.pro;
        }
        set curPro(val) {
            if (this.pro == this.maxCount)
                return;
            this.pro = val;
            if (this.pro > this.maxCount) {
                this.pro = this.maxCount;
            }
            if (this.pro == this.maxCount) {
                this.state = BaseTaskVO.STATE_CAN_AWARD;
            }
            StorageUtils.Inst.updateBottleTask(this.tempId, this.pro, this.state);
        }
        get maxCount() {
            return this.taskRes.cost;
        }
    }

    class EventtaskResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint8();
            this.viceType = byteArray.readUint8();
            this.cost = byteArray.readUint8();
            this.desc = byteArray.readUTFString();
            this.item = byteArray.readUint16();
            this.num = byteArray.readUint8();
            this.weight = byteArray.readUint8();
            this.otherawardID = byteArray.readUint8();
            this.otherawardNum = byteArray.readUint8();
        }
        static getItemByKey(id) {
            return EventtaskResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return EventtaskResource.itemsDic[key1 + "," + key2];
        }
    }
    EventtaskResource.items = new Array();
    EventtaskResource.itemsDic = new Dictionary();

    class WarOrderResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.Grade = byteArray.readUint8();
            this.active = byteArray.readUint16();
            this.award = byteArray.readUint16();
            this.awardnum = byteArray.readUint16();
            this.awardicon = byteArray.readUTFString();
            this.senioraward = byteArray.readUint16();
            this.seniorawardnum = byteArray.readUint16();
            this.seniorawardicon = byteArray.readUTFString();
        }
        static getItemByKey(id) {
            return WarOrderResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return WarOrderResource.itemsDic[key1 + "," + key2];
        }
    }
    WarOrderResource.items = new Array();
    WarOrderResource.itemsDic = new Dictionary();

    class SummerTaskVo extends BaseTaskVO {
        constructor(tempId) {
            super(tempId, 0);
        }
        setup(data) {
            this.pro = data.pro;
            this.state = data.state;
        }
        get taskRes() {
            return EventtaskResource.getItemByKey(this.tempId);
        }
        get curPro() {
            return this.pro;
        }
        set curPro(val) {
            if (this.pro == this.maxCount)
                return;
            this.pro = val;
            if (this.pro > this.maxCount) {
                this.pro = this.maxCount;
            }
            if (this.pro == this.maxCount) {
                this.state = BaseTaskVO.STATE_CAN_AWARD;
            }
            StorageUtils.Inst.SaveActTask(this.tempId, this.pro, this.state);
        }
        get maxCount() {
            return this.taskRes.cost;
        }
        get exAward() {
            return this._exAward = this._exAward || new AwardVO(this.taskRes.otherawardID, { "count": this.taskRes.otherawardNum });
        }
        GetOtherAward() {
            this.GetItemAward(this._exAward.tempId, parseFloat(this._exAward.count.toString()));
        }
        GetAward() {
            if (this.state != BaseTaskVO.STATE_CAN_AWARD)
                return;
            let itemId = this.taskRes.item;
            let num = this.taskRes.num;
            this.GetItemAward(itemId, num);
            this.state = BaseTaskVO.STATE_COMPLETED;
            StorageUtils.Inst.SaveActTask(this.tempId, this.pro, this.state);
        }
        GetItemAward(itemId, num) {
            let itemType = itemResource.getItemByKey(itemId).type;
            if (itemId == 1) {
                MoneyModel.gold.add(num);
            }
            else if (itemId == 2) {
                MoneyModel.diamond.add(num);
            }
            else if (itemId == 13) {
                let goldNum = BuildModel.Inst.GetAllBuildGoldRate(num * 60 * 60);
                MoneyModel.gold.add(goldNum);
            }
            else if (itemType == ItemModel.TYPE_ITEM || itemType == ItemModel.TYPE_BUILD) {
                ItemCtrl.Inst.AddItemVO(itemId, num);
            }
            else if (itemType == ItemModel.TYPE_SKIN) {
                this.AddSkinAward(itemId, num);
            }
            else if (itemType == ItemModel.TYPE_ACTIVITY) {
                WarOrderModel.Inst.curActivity += num;
            }
        }
        AddSkinAward(itemId, num) {
            let skinResArr = skinResource.items;
            for (let i = 0; i < skinResArr.length; i++) {
                let res = skinResArr[i];
                if (res.itemId == itemId) {
                    SkinModel.Inst.AddSkin(res.id, num);
                    break;
                }
            }
        }
    }

    class WarOrderAwardVo {
        constructor(active, tempId, award, type = WarOrderAwardVo.TYPE_PUTONG) {
            this.state = NaN;
            this.type = type;
            this._award = award;
            this._tempId = tempId;
            this._active = active;
        }
        get active() {
            return this._active;
        }
        set active(val) {
            this._active = val;
        }
        get award() {
            return this._award;
        }
        get awardState() {
            if (isNaN(this.state)) {
                this.state = WarOrderModel.Inst.GetState(this._tempId, this.type);
            }
            return this.state;
        }
        set awardState(state) {
            if (this.awardState == state)
                return;
            this.state = state;
            WarOrderModel.Inst.SetState(this.state, this._tempId, this.type);
        }
        get canGetAward() {
            if (this.type == WarOrderAwardVo.TYPE_SENIOR && !WarOrderModel.Inst.isUnLockSenior) {
                return false;
            }
            return this.awardState == WarOrderAwardVo.state_canget;
        }
        get isComplete() {
            return this.awardState == WarOrderAwardVo.state_complete;
        }
        get isSenior() {
            return this.type == WarOrderAwardVo.TYPE_SENIOR;
        }
    }
    WarOrderAwardVo.state_normal = 0;
    WarOrderAwardVo.state_canget = 1;
    WarOrderAwardVo.state_complete = 2;
    WarOrderAwardVo.TYPE_PUTONG = 0;
    WarOrderAwardVo.TYPE_SENIOR = 1;

    class WarOrderVo {
        constructor(tempId) {
            this._tempId = tempId;
        }
        get warOrderRes() {
            return this.res = this.res || WarOrderResource.getItemByKey(this._tempId);
        }
        get prevRes() {
            return this._tempId - 1 > 0 ? WarOrderResource.getItemByKey(this._tempId - 1) : null;
        }
        get award0() {
            return this._award0 = this._award0 || new AwardVO(this.warOrderRes.award, { "count": this.warOrderRes.awardnum });
        }
        get award1() {
            return this._award1 = this._award1 || new AwardVO(this.warOrderRes.senioraward, { "count": this.warOrderRes.seniorawardnum });
        }
    }
    WarOrderVo.state_normal = 0;
    WarOrderVo.state_canget = 1;
    WarOrderVo.state_complete = 2;

    class WarOrderModel {
        constructor() {
            this.warOrderVos = [];
            this.warOrderMap = {};
            this._awardArr = [];
            this._seniorawardArr = [];
            this._isUnLock = false;
            this._taskArr = [];
            this._curTaskArr = [];
        }
        static get Inst() {
            return this.inst = this.inst || new WarOrderModel();
        }
        init() {
            this.warOrderData = StorageUtils.Inst.GetWarOrder();
            if (!this.warOrderData) {
                this.warOrderData = {};
            }
            let vo = null;
            let res;
            for (let i = 0; i < WarOrderResource.items.length; i++) {
                res = WarOrderResource.items[i];
                vo = new WarOrderVo(res.Grade);
                if (i == WarOrderResource.items.length - 1) {
                    this.warOrderVo = vo;
                    this.award = new WarOrderAwardVo(res.active, res.Grade, vo.award0, WarOrderAwardVo.TYPE_PUTONG);
                    continue;
                }
                this.warOrderVos.push(vo);
                this.warOrderMap[res.Grade] = vo;
                this._awardArr.push(new WarOrderAwardVo(res.active, res.Grade, vo.award0, WarOrderAwardVo.TYPE_PUTONG));
                this._seniorawardArr.push(new WarOrderAwardVo(res.active, res.Grade, vo.award1, WarOrderAwardVo.TYPE_SENIOR));
            }
            this._isUnLock = WarOrderModel.isUnLock in this.warOrderData ? this.warOrderData[WarOrderModel.isUnLock] : false;
            let taskRes;
            let taskVo;
            for (let i = 0; i < EventtaskResource.items.length; i++) {
                taskRes = EventtaskResource.items[i];
                taskVo = new SummerTaskVo(taskRes.id);
                this._taskArr.push(taskVo);
            }
            this.initTask();
            let obj = StorageUtils.Inst.getActTask();
            let objList = obj.list;
            for (let i = 0; i < objList.length; i++) {
                let objItem = objList[i];
                let taskItem = this.GetTaskVoById(objItem.taskId);
                taskItem.setup(objItem);
                this._curTaskArr.push(taskItem);
            }
            GameDispatcher.Inst.on(UIGameEvent.DAYTASK_COMP_PROP, this, this.OnDayTaskComp);
        }
        GetTaskVoById(id) {
            for (let i = 0; i < this._taskArr.length; i++) {
                const vo = this._taskArr[i];
                if (vo.tempId == id) {
                    return vo;
                }
            }
        }
        GetWarOrderVos() {
            return this.warOrderVos;
        }
        get awardArr() {
            return this._awardArr;
        }
        get seniorawardArr() {
            return this._seniorawardArr;
        }
        get maxActivity() {
            return this._awardArr[this._awardArr.length - 1].active;
        }
        get curActivity() {
            return WarOrderModel.curActivity in this.warOrderData ? this.warOrderData[WarOrderModel.curActivity] : 0;
        }
        set curActivity(val) {
            if (!this.warOrderData)
                return;
            if (val == this.warOrderData[WarOrderModel.curActivity])
                return;
            this.warOrderData[WarOrderModel.curActivity] = val;
            for (let i = 0; i < this.awardArr.length; i++) {
                const award = this.awardArr[i];
                if (award.active <= this.curActivity) {
                    if (!award.isComplete)
                        award.awardState = WarOrderAwardVo.state_canget;
                }
                else {
                    award.awardState = WarOrderAwardVo.state_normal;
                }
            }
            for (let i = 0; i < this.seniorawardArr.length; i++) {
                const award = this.seniorawardArr[i];
                if (award.active <= this.curActivity) {
                    if (!award.isComplete)
                        award.awardState = WarOrderAwardVo.state_canget;
                }
                else {
                    award.awardState = WarOrderAwardVo.state_normal;
                }
            }
            if (!this.award.isComplete) {
                if (this.award.active <= this.curActivity) {
                    if (!this.award.isComplete)
                        this.award.awardState = WarOrderAwardVo.state_canget;
                }
                else {
                    this.award.awardState = WarOrderAwardVo.state_normal;
                }
            }
            GameDispatcher.Inst.event(UIGameEvent.SUMMER_ACTIVE);
            this.SaveData();
        }
        GetStateObj(tempId) {
            if (WarOrderModel.state in this.warOrderData) {
            }
            else {
                this.warOrderData[WarOrderModel.state] = {};
            }
            let obj = this.warOrderData[WarOrderModel.state];
            if (tempId in obj) {
            }
            else {
                obj[tempId] = {};
            }
            return obj[tempId];
        }
        GetState(tempId, awardType = WarOrderAwardVo.TYPE_PUTONG) {
            let stateObj = this.GetStateObj(tempId);
            return awardType in stateObj ? stateObj[awardType] : 0;
        }
        SetState(state, tempId, awardType = WarOrderAwardVo.TYPE_PUTONG) {
            let stateObj = this.GetStateObj(tempId);
            stateObj[awardType] = state;
            this.SaveData();
        }
        get isUnLockSenior() {
            return this._isUnLock;
        }
        set isUnLockSenior(bol) {
            this._isUnLock = bol;
            this.warOrderData[WarOrderModel.isUnLock] = bol;
            this.SaveData();
        }
        SaveData() {
            StorageUtils.Inst.SaveWarOrder(this.warOrderData);
        }
        get loopAwardActive() {
            return WarOrderModel.loopAward in this.warOrderData ? this.warOrderData[WarOrderModel.loopAward] : this.warOrderVo.warOrderRes.active;
        }
        set loopAwardActive(active) {
            this.warOrderData[WarOrderModel.loopAward] = active;
            this.award.active = active;
            this.SaveData();
        }
        GetLoopAward(vo) {
            this.GetAward(vo);
            this.loopAwardActive += this.warOrderVo.warOrderRes.active;
            vo.awardState = WarOrderAwardVo.state_normal;
        }
        CheckShowRed() {
            let bol = false;
            for (let i = 0; i < this.awardArr.length; i++) {
                const award = this.awardArr[i];
                if (award.canGetAward) {
                    bol = true;
                    break;
                }
            }
            if (!bol) {
                for (let i = 0; i < this.seniorawardArr.length; i++) {
                    const award = this.seniorawardArr[i];
                    if (award.canGetAward) {
                        bol = true;
                        break;
                    }
                }
            }
            if (!bol) {
                for (let i = 0; i < this.taskVos.length; i++) {
                    let task = this.taskVos[i];
                    if (task.canAward) {
                        bol = true;
                        break;
                    }
                }
            }
            return bol;
        }
        GetAward(vo) {
            if (!vo.canGetAward)
                return;
            vo.awardState = WarOrderAwardVo.state_complete;
            let itemId = vo.award.tempId;
            let num = parseFloat(vo.award.count.toString());
            let item = itemResource.getItemByKey(itemId);
            let itemType = item && item.type;
            if (itemId == 1) {
                MoneyModel.gold.add(num);
            }
            else if (itemId == 2) {
                MoneyModel.diamond.add(num);
            }
            else if (itemId == 13) {
                let goldNum = BuildModel.Inst.GetAllBuildGoldRate(num * 60 * 60, false);
                MoneyModel.gold.add(goldNum);
            }
            else if (itemType == ItemModel.TYPE_ITEM || itemType == ItemModel.TYPE_BUILD) {
                ItemCtrl.Inst.AddItemVO(itemId, num);
            }
            else if (itemType == ItemModel.TYPE_SKIN) {
                this.AddSkinAward(itemId, num);
            }
        }
        AddSkinAward(itemId, num) {
            let skinResArr = skinResource.items;
            for (let i = 0; i < skinResArr.length; i++) {
                let res = skinResArr[i];
                if (res.itemId == itemId) {
                    SkinModel.Inst.AddSkin(res.id, num);
                    break;
                }
            }
        }
        get taskVos() {
            return this._curTaskArr;
        }
        RefreshTask() {
            StorageUtils.Inst.ClearActTask();
            this._curTaskArr.length = 0;
            this.initTask();
            let obj = StorageUtils.Inst.getActTask();
            let objList = obj.list;
            for (let i = 0; i < objList.length; i++) {
                let objItem = objList[i];
                let taskItem = this.GetTaskVoById(objItem.taskId);
                taskItem.setup(objItem);
                this._curTaskArr.push(taskItem);
            }
        }
        initTask() {
            let obj = StorageUtils.Inst.getActTask();
            if (obj == null) {
                let taskItems = EventtaskResource.items;
                let taskResArr = MathUtil.GetRandomItemByWeight(taskItems, 4);
                let list = [];
                for (let i = 0; i < taskResArr.length; i++) {
                    let taskRes = taskResArr[i];
                    let taskObj = { "taskId": taskRes.id, "pro": 0, "state": BaseTaskVO.STATE_NORMAL };
                    list.push(taskObj);
                }
                StorageUtils.Inst.InitActTaskList(list);
            }
        }
        OnDayTaskComp(viceType) {
            for (let i = 0; i < this.taskVos.length; i++) {
                let item = this.taskVos[i];
                if (item.taskRes.viceType == viceType) {
                    item.curPro++;
                    if (viceType == TaskVO.JIANQIU) {
                        CatFecalBallCtrl.Inst.MakeFecals(item.maxCount - item.curPro);
                    }
                    break;
                }
            }
        }
    }
    WarOrderModel.inst = null;
    WarOrderModel.curActivity = "curActivity";
    WarOrderModel.state = "state";
    WarOrderModel.loopAward = "loopAward";
    WarOrderModel.isUnLock = "isUnLock";

    class DriftingBottleModel {
        constructor() {
            this.yaoQingCatID = 10047;
        }
        static get Inst() {
            return this.inst = this.inst || new DriftingBottleModel();
        }
        init() {
            this._isThrowBottle = StorageUtils.Inst.isThrowBottle;
            this._bottleState = StorageUtils.Inst.bottleState;
            this.InitTask();
            GameDispatcher.Inst.on(UIGameEvent.MAINTASK_COMP_PROP, this, this.OnTaskComp);
        }
        InitTask() {
            let maxCount = parseFloat(ConfigValue.getItemByKey("bTaskGoal1").value);
            let awardStr = ConfigValue.getItemByKey("bTaskAward1").value;
            let strArr = awardStr.split(",");
            let award0 = new AwardVO(parseInt(strArr[0]), { "count": parseFloat(strArr[1]) });
            let desc0 = "如果你的岛上不干净我可不信任你……岛上的毛球都收拾完了吗？去捡干净吧！";
            let icon0 = ConfigValue.getItemByKey("bTaskIcon1") ? ConfigValue.getItemByKey("bTaskIcon1").value : "rw_0006";
            let obj1 = { "cost": maxCount, "taskType": TaskType.TASKTYPE_GET_MAOQIU, "desc": desc0, "icon": icon0 };
            this.task0 = new DriftingBottleTaskVo(0, obj1, award0);
            let taskStr = ConfigValue.getItemByKey("bTaskGoal2").value;
            let arr = taskStr.split(",");
            let awardStr1 = ConfigValue.getItemByKey("bTaskAward2").value;
            let strArr1 = awardStr1.split(",");
            let award1 = new AwardVO(parseInt(strArr1[0]), { "count": parseFloat(strArr1[1]) });
            let desc2 = "去提升阳光浴床的等级吧！要升到30级哦！";
            let icon1 = ConfigValue.getItemByKey("bTaskIcon2") ? ConfigValue.getItemByKey("bTaskIcon2").value : "rw_0007";
            let obj2 = { "cost": parseInt(arr[1]), "taskType": TaskType.TASKTYPE_BUILDLEVEL, "id": parseInt(arr[0]), "desc": desc2, "icon": icon1, "otherCost": parseInt(arr[0]) };
            this.task1 = new DriftingBottleTaskVo(1, obj2, award1);
            let obj = StorageUtils.Inst.getBottleTask();
            let curTask;
            if (obj["taskId"] == 0) {
                curTask = this.task0;
            }
            else {
                curTask = this.task1;
            }
            curTask.setup(obj);
            this._curTask = curTask;
        }
        get isUnLock() {
            let buildVo = BuildModel.Inst.GetBuildVOBy(DriftingBottleModel.unLockId);
            return buildVo && buildVo.IsUnLock() && this.driftingBottleState != DriftingBottleModel.yaoQingCompState;
        }
        get curTask() {
            return this._curTask;
        }
        get isThrowBottle() {
            return this._isThrowBottle;
        }
        set isThrowBottle(val) {
            if (this._isThrowBottle == val)
                return;
            this._isThrowBottle = val;
            StorageUtils.Inst.isThrowBottle = val;
        }
        get driftingBottleState() {
            return this._bottleState;
        }
        set driftingBottleState(val) {
            if (this._bottleState == val)
                return;
            this._bottleState = val;
            StorageUtils.Inst.bottleState = val;
        }
        OnTaskComp(type, itemID, isAdd = 1) {
            if (this.driftingBottleState != DriftingBottleModel.taskState)
                return;
            let mainTask = this.curTask;
            let taskRes = mainTask.taskRes;
            let mainTaskType = taskRes.taskType;
            let id = taskRes.otherCost;
            if (mainTaskType == TaskModel.TASKTYPE_OTHER) {
            }
            else if (mainTaskType == type) {
                if (type == TaskType.TASKTYPE_UNLOCK_BUILD) {
                    let buildVos = BuildModel.Inst.GetUnlockBuilds();
                    for (let i = 0; i < buildVos.length; i++) {
                        if (buildVos[i].buildRes.id == id && id > 0) {
                            mainTask.curPro += 1;
                            break;
                        }
                    }
                    if (id <= 0) {
                        mainTask.curPro += isAdd;
                    }
                }
                else if (type == TaskType.TASKTYPE_UP_BUILD) {
                    if (id > 0) {
                        if (itemID == id) {
                            mainTask.curPro += isAdd;
                        }
                    }
                    else {
                        mainTask.curPro += isAdd;
                    }
                    let buildVos = BuildModel.Inst.GetUnlockBuilds();
                    let ismax = true;
                    for (let i = 0; i < buildVos.length; i++) {
                        if (buildVos[i].buildRes.type == BuildVO.TYPE_MATOU)
                            continue;
                        if (!buildVos[i].IsMaxLv()) {
                            ismax = false;
                        }
                        if (buildVos[i].buildRes.id == id && id > 0 && (buildVos[i].IsMaxLv())) {
                            mainTask.curPro = mainTask.maxCount;
                            return;
                        }
                    }
                    if (ismax) {
                        mainTask.curPro = mainTask.maxCount;
                    }
                }
                else if (type == TaskType.TASKTYPE_INVITE_CAT) {
                    let catVos = CatModel.Inst.GetHasCatVOs();
                    for (let i = 0; i < catVos.length; i++) {
                        if (catVos[i].catRes.id == id && id > 0) {
                            mainTask.curPro += 1;
                            break;
                        }
                    }
                    if (id <= 0) {
                        mainTask.curPro += isAdd;
                    }
                }
                else if (type == TaskType.TASKTYPE_UP_CAT) {
                    if (id <= 0) {
                        mainTask.curPro += isAdd;
                    }
                    else {
                        if (id == itemID) {
                            mainTask.curPro += isAdd;
                        }
                    }
                }
                else if (type == TaskType.TASKTYPE_GET_CAT) {
                    mainTask.curPro += isAdd;
                }
                else if (type == TaskType.TASKTYPE_MO_CAT) {
                    mainTask.curPro += isAdd;
                }
                else if (type == TaskType.TASKTYPE_GET_MAOQIU) {
                    mainTask.curPro += isAdd;
                    CatFecalBallCtrl.Inst.MakeFecals(mainTask.maxCount - mainTask.curPro);
                }
                else if (type == TaskType.TASKTYPE_WEIMAO) {
                    mainTask.curPro += isAdd;
                    if (AddFishModel.Inst.HasFish()) {
                        mainTask.curPro += 1;
                    }
                }
                else if (type == TaskType.TASKTYPE_LOCKDRAW) {
                    mainTask.curPro += isAdd;
                }
                else if (type == TaskType.TASKTYPE_DRESSUP) {
                    mainTask.curPro += isAdd;
                }
                else if (type == TaskType.TASKTYPE_PAIZHAO) {
                    mainTask.curPro += isAdd;
                }
                else if (type == TaskType.TASKTYPE_DAYTASKCOMP) {
                    if (TaskModel.Inst.dayAllTaskComp) {
                        mainTask.curPro += 1;
                    }
                }
                else if (type == TaskType.TASKTYPE_GIVEDRESS) {
                    mainTask.curPro += isAdd;
                }
                else if (type == TaskType.TASKTYPE_BUILDLEVEL) {
                    let buildVos = BuildModel.Inst.GetUnlockBuilds();
                    let max = 0;
                    for (let i = 0; i < buildVos.length; i++) {
                        if (id > 0) {
                            if (buildVos[i].buildRes.id == id) {
                                max = buildVos[i].level;
                                break;
                            }
                        }
                        else if (buildVos[i].level > max) {
                            max = buildVos[i].level;
                        }
                    }
                    mainTask.curPro = max;
                }
                else if (type == TaskType.TASKTYPE_CATLEVEL) {
                    let catVos = CatModel.Inst.GetAllCatVOs();
                    let max = 0;
                    for (let i = 0; i < catVos.length; i++) {
                        if (id > 0) {
                            if (catVos[i].catRes.id == id) {
                                max = catVos[i].level;
                                break;
                            }
                        }
                        else if (catVos[i].level > max) {
                            max = catVos[i].level;
                        }
                    }
                    mainTask.curPro = max;
                }
                else if (type == TaskType.TASKTYPE_BUYDAOJU) {
                    if (id > 0) {
                        if (itemID instanceof Array) {
                            for (let i = 0; i < itemID.length; i++) {
                                if (itemID[i] instanceof ExchangeVO) {
                                    if (itemID[i].itemRes.id == id) {
                                        mainTask.curPro += isAdd;
                                        break;
                                    }
                                }
                            }
                        }
                        else {
                            if (itemID == id) {
                                mainTask.curPro += isAdd;
                            }
                        }
                    }
                    else {
                        mainTask.curPro += isAdd;
                    }
                }
                else if (type == TaskType.TASKTYPE_BUILDSCORE) {
                    let score = 0;
                    score = BuildModel.Inst.GetBuildScore(id);
                    mainTask.curPro = score;
                }
                else if (type == TaskType.TASKTYPE_GIVEQIULU) {
                    if (id <= 0) {
                        mainTask.curPro += isAdd;
                    }
                    else if (id == itemID) {
                        mainTask.curPro += isAdd;
                    }
                }
                else if (type == TaskType.TASKTYPE_UNLOCKISLAND) {
                    if (id <= 0) {
                        mainTask.curPro += isAdd;
                    }
                    else if (id <= LandModel.Inst.level) {
                        mainTask.curPro += 1;
                    }
                }
                else if (type == TaskType.TASKTYPE_ADDMATOUNUM) {
                    let num = LandModel.Inst.boatNum;
                    mainTask.curPro = num;
                }
                else if (type == TaskType.TASKTYPE_CATBAIZS) {
                    if (id <= 0) {
                        mainTask.curPro += isAdd;
                    }
                    else if (id == itemID) {
                        mainTask.curPro += isAdd;
                    }
                }
                else if (type == TaskType.TASKTYPE_ADDSERVER) {
                    if (id <= 0) {
                        mainTask.curPro += isAdd;
                    }
                    else if (id == itemID) {
                        mainTask.curPro += isAdd;
                    }
                }
                else if (type == TaskType.TASKTYPE_ADDCATHOME) {
                    if (BuildModel.Inst.isMaowuMaxLevel) {
                        mainTask.curPro += 1;
                    }
                    else {
                        mainTask.curPro += isAdd;
                    }
                }
            }
        }
        GetAward(vo) {
            if (!vo.canAward)
                return;
            vo.state = BaseTaskVO.STATE_COMPLETED;
            let itemId = vo.award.tempId;
            let num = parseFloat(vo.award.count.toString());
            let item = itemResource.getItemByKey(itemId);
            let itemType = item && item.type;
            if (itemId == 1) {
                MoneyModel.gold.add(num);
            }
            else if (itemId == 2) {
                MoneyModel.diamond.add(num);
            }
            else if (itemId == 13) {
                let goldNum = BuildModel.Inst.GetAllBuildGoldRate(num * 60 * 60, false);
                MoneyModel.gold.add(goldNum);
            }
            else if (itemType == ItemModel.TYPE_ITEM || itemType == ItemModel.TYPE_BUILD) {
                ItemCtrl.Inst.AddItemVO(itemId, num);
            }
            else if (itemType == ItemModel.TYPE_SKIN) {
                this.AddSkinAward(itemId, num);
            }
            else if (itemType == ItemModel.TYPE_ACTIVITY) {
                WarOrderModel.Inst.curActivity += num;
            }
            if (vo == this.task0) {
                this._curTask = this.task1;
            }
            else if (vo == this.task1) {
                DriftingBottleModel.Inst.driftingBottleState = DriftingBottleModel.taskCompState;
            }
            StorageUtils.Inst.updateBottleTask(this._curTask.tempId, this._curTask.pro, this._curTask.state);
        }
        AddSkinAward(itemId, num) {
            let skinResArr = skinResource.items;
            for (let i = 0; i < skinResArr.length; i++) {
                let res = skinResArr[i];
                if (res.itemId == itemId) {
                    SkinModel.Inst.AddSkin(res.id, num);
                    break;
                }
            }
        }
    }
    DriftingBottleModel.inst = null;
    DriftingBottleModel.unLockId = 103;
    DriftingBottleModel.throwBottleState = 0;
    DriftingBottleModel.taskState = 1;
    DriftingBottleModel.taskCompState = 2;
    DriftingBottleModel.shangdaoState = 3;
    DriftingBottleModel.shangdaoCompState = 4;
    DriftingBottleModel.yaoQingCompState = 5;

    class GrowUpResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint8();
            this.name = byteArray.readUTFString();
            this.cost = byteArray.readUTFString();
            this.item = byteArray.readUint8();
            this.num = byteArray.readFloat64();
            this.extra = byteArray.readUint8();
            this.extranumber = byteArray.readUint16();
        }
        static getItemByKey(id) {
            return GrowUpResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return GrowUpResource.itemsDic[key1 + "," + key2];
        }
    }
    GrowUpResource.items = new Array();
    GrowUpResource.itemsDic = new Dictionary();

    class packageResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint8();
            this.name = byteArray.readUTFString();
            this.type = byteArray.readUint8();
            this.icon = byteArray.readUTFString();
            this.time = byteArray.readUint16();
            this.desc = byteArray.readUTFString();
            this.item1 = ConfigLib.ReadArray(byteArray);
            this.item2 = ConfigLib.ReadArray(byteArray);
            this.item3 = ConfigLib.ReadArray(byteArray);
            this.unlockCon = byteArray.readUint8();
            this.unlockNum = byteArray.readUint16();
            this.adNum = byteArray.readUint8();
            this.adCd = byteArray.readUint8();
            this.buyNum = byteArray.readUint8();
        }
        static getItemByKey(id) {
            return packageResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return packageResource.itemsDic[key1 + "," + key2];
        }
    }
    packageResource.items = new Array();
    packageResource.itemsDic = new Dictionary();

    class GrowthGiftPackVO {
        constructor(tempId) {
            this._res = null;
            this._isGet = null;
            this._canGetScore = null;
            this._tempId = tempId;
            this._isGet = StorageUtils.Inst.GetGrowthUpGiftPack(this._tempId);
        }
        get exAward() {
            return new AwardVO(this.growUpRes.extra, { "count": this.growUpRes.extranumber });
        }
        GetAward(isGetEx = false) {
            if (!this.canGet)
                return;
            let arr = [{ "id": this.growUpRes.item, "num": this.growUpRes.num }];
            if (isGetEx) {
                arr.push({ "id": this.growUpRes.extra, "num": this.growUpRes.extranumber });
            }
            for (let i = 0; i < arr.length; i++) {
                let itemId = arr[i].id;
                let num = arr[i].num;
                let itemType = itemResource.getItemByKey(itemId).type;
                if (itemId == 1) {
                    MoneyModel.gold.add(num);
                }
                else if (itemId == 2) {
                    MoneyModel.diamond.add(num);
                }
                else if (itemId == 13) {
                    let goldNum = BuildModel.Inst.GetAllBuildGoldRate(num * 60 * 60, false);
                    MoneyModel.gold.add(goldNum);
                }
                else if (itemType == ItemModel.TYPE_ITEM || itemType == ItemModel.TYPE_BUILD) {
                    ItemCtrl.Inst.AddItemVO(itemId, num);
                }
                else if (itemType == ItemModel.TYPE_SKIN) {
                    this.AddSkinAward(itemId, num);
                }
            }
            this.isGet = true;
        }
        AddSkinAward(itemId, num) {
            let skinResArr = skinResource.items;
            for (let i = 0; i < skinResArr.length; i++) {
                let res = skinResArr[i];
                if (res.itemId == itemId) {
                    SkinModel.Inst.AddSkin(res.id, num);
                    break;
                }
            }
        }
        get canGet() {
            if (this.isGet) {
                return false;
            }
            return BuildModel.Inst.GetAllBuildGoldRate(10, false).isGreaterThanOrEqualTo(this.canGetScore);
        }
        get canGetScore() {
            return this._canGetScore = this._canGetScore || new BigNumber(this.growUpRes.cost);
        }
        get curPro() {
            return BuildModel.Inst.GetAllBuildGoldRate(10, false);
        }
        get isGet() {
            return this._isGet;
        }
        set isGet(bol) {
            if (this._isGet == bol)
                return;
            this._isGet = bol;
            StorageUtils.Inst.SetGrowthUpGiftPack(this._tempId, this._isGet);
        }
        get growUpRes() {
            return this._res = this._res || GrowUpResource.getItemByKey(this._tempId);
        }
        get state() {
            if (this.isGet) {
                return BaseTaskVO.STATE_COMPLETED;
            }
            else if (this.canGet) {
                return BaseTaskVO.STATE_CAN_AWARD;
            }
            else {
                return BaseTaskVO.STATE_NORMAL;
            }
        }
    }

    class PackageVO {
        constructor(tempId) {
            this._res = null;
            this.awards = [];
            this._curAdNum = NaN;
            this._adTime = NaN;
            this._buyNum = NaN;
            this._pkgShowTime = NaN;
            this._tempId = tempId;
        }
        get pkgAwardVos() {
            if (this.awards.length == 0) {
                let items = this.pkgRes.item1;
                if (this.curAdNum == 0) {
                    items = this.pkgRes.item1;
                }
                else if (this.curAdNum == 1) {
                    if (this.pkgRes.item2 && this.pkgRes.item2.length > 0) {
                        items = this.pkgRes.item2;
                    }
                }
                else {
                    if (this.pkgRes.item3 && this.pkgRes.item3.length > 0) {
                        items = this.pkgRes.item3;
                    }
                }
                for (let i = 0; i < items.length; i++) {
                    let item = items[i];
                    let awardVo = new AwardVO(item[0], { "count": item[1] });
                    this.awards.push(awardVo);
                }
            }
            return this.awards;
        }
        GetAward() {
            if (this.curAdNum == this.maxAdNum - 1) {
                this.buyNum++;
            }
            this.adTime = Laya.timer.currTimer;
            let awards = this.pkgAwardVos;
            for (let i = 0; i < awards.length; i++) {
                let award = awards[i];
                if (award.IsItem) {
                    let itemId = award.tempId;
                    let num = parseFloat(award.count.toString());
                    let itemType = itemResource.getItemByKey(itemId).type;
                    if (itemId == 1) {
                        MoneyModel.gold.add(num);
                    }
                    else if (itemId == 2) {
                        MoneyModel.diamond.add(num);
                    }
                    else if (itemId == 13) {
                        let goldNum = BuildModel.Inst.GetAllBuildGoldRate(num * 60 * 60, false);
                        MoneyModel.gold.add(goldNum);
                    }
                    else if (itemType == ItemModel.TYPE_ITEM || itemType == ItemModel.TYPE_BUILD) {
                        ItemCtrl.Inst.AddItemVO(itemId, num);
                    }
                    else if (itemType == ItemModel.TYPE_SKIN) {
                        this.AddSkinAward(itemId, num);
                    }
                }
            }
            this.awards.length = 0;
            this.curAdNum++;
        }
        AddSkinAward(itemId, num) {
            let skinResArr = skinResource.items;
            for (let i = 0; i < skinResArr.length; i++) {
                let res = skinResArr[i];
                if (res.itemId == itemId) {
                    SkinModel.Inst.AddSkin(res.id, num);
                    break;
                }
            }
        }
        get curAdNum() {
            if (isNaN(this._curAdNum)) {
                this._curAdNum = StorageUtils.Inst.GetGiftPKGAdCount(this._tempId, false);
            }
            return this._curAdNum;
        }
        set curAdNum(val) {
            if (val == this.curAdNum)
                return;
            if (this.curAdNum == this.maxAdNum)
                return;
            this._curAdNum = val;
            if (this._curAdNum > this.maxAdNum) {
                this._curAdNum = this.maxAdNum;
            }
            StorageUtils.Inst.SetGiftPKGAdCount(this._tempId, this.curAdNum, false);
        }
        get adTime() {
            if (isNaN(this._adTime)) {
                this._adTime = StorageUtils.Inst.LastGetGiftPKGTime(this._tempId);
            }
            return this._adTime;
        }
        set adTime(val) {
            if (this._adTime == val)
                return;
            this._adTime = val;
            StorageUtils.Inst.SetLastGetGiftPKGTime(this._tempId, val);
        }
        get pkgName() {
            return this.pkgRes.name;
        }
        get time() {
            return this.pkgRes.time;
        }
        get pkgShowTime() {
            if (isNaN(this._pkgShowTime)) {
                this._pkgShowTime = StorageUtils.Inst.LastShowGiftPKGTime(this._tempId);
            }
            return this._pkgShowTime;
        }
        set pkgShowTime(time) {
            this._pkgShowTime = time;
            StorageUtils.Inst.SetLastShowGiftPKGTime(this._tempId, time);
        }
        get isCanShow() {
            if (this.pkgShowTime == 0)
                this.pkgShowTime = Laya.timer.currTimer;
            return (Laya.timer.currTimer - this.pkgShowTime) / 1000 <= this.time;
        }
        get isCanGet() {
            return this.isUnLock && this.buyNum < this.pkgRes.buyNum;
        }
        get buyNum() {
            if (this.pkgRes.buyNum == 0) {
                return -1;
            }
            if (isNaN(this._buyNum)) {
                this._buyNum = StorageUtils.Inst.GetGiftPKGBuyCount(this._tempId, false);
            }
            return this._buyNum;
        }
        set buyNum(val) {
            if (this._buyNum == val)
                return;
            if (this.pkgRes.buyNum != 0) {
                this._buyNum = val;
                StorageUtils.Inst.SetGiftPKGBuyCount(this._tempId, this._buyNum, false);
            }
        }
        get isCd() {
            return (Laya.timer.currTimer - this.adTime) < this.adCd;
        }
        get adCd() {
            return this.pkgRes.adCd * 1000;
        }
        get maxAdNum() {
            return this.pkgRes.adNum;
        }
        get pkgRes() {
            return this._res = this._res || packageResource.getItemByKey(this._tempId);
        }
        get isUnLock() {
            if (this.pkgRes.unlockCon == PackageVO.unlockCon1) {
                return true;
            }
            else if (this.pkgRes.unlockCon == PackageVO.unlockCon2) {
                let vo = BuildModel.Inst.GetBuildVOBy(this.pkgRes.unlockNum);
                return vo && vo.IsUnLock();
            }
            else if (this.pkgRes.unlockCon == PackageVO.unlockCon3) {
                return this.pkgRes.unlockNum == StorageUtils.Inst.loginDayCount;
            }
            else if (this.pkgRes.unlockCon == PackageVO.unlockCon4) {
                return this.pkgRes.unlockNum == LandModel.Inst.level;
            }
            return false;
        }
        get isShopGiftBag() {
            return this.pkgRes.type == GiftPKGModel.TYPE_SHOP;
        }
        get isMainGiftBag() {
            return this.pkgRes.type == GiftPKGModel.TYPE_MAINVIEW;
        }
    }
    PackageVO.unlockCon1 = 1;
    PackageVO.unlockCon2 = 2;
    PackageVO.unlockCon3 = 3;
    PackageVO.unlockCon4 = 4;

    class GiftPKGModel {
        constructor() {
            this.pkgVos = [];
            this.pkgMap = {};
            this.growthGiftVos = [];
            this.growthGiftMap = {};
            this._mainGiftBag = null;
        }
        static get Inst() {
            return this.inst = this.inst || new GiftPKGModel();
        }
        init() {
            let growthvo = null;
            let growthres;
            for (let i = 0; i < GrowUpResource.items.length; i++) {
                growthres = GrowUpResource.items[i];
                growthvo = new GrowthGiftPackVO(growthres.id);
                this.growthGiftVos.push(growthvo);
                this.growthGiftMap[growthres.id] = growthvo;
            }
            let vo = null;
            let res;
            for (let i = 0; i < packageResource.items.length; i++) {
                res = packageResource.items[i];
                vo = new PackageVO(res.id);
                this.pkgVos.push(vo);
                this.pkgMap[res.id] = vo;
            }
            if (this.mainGiftBag && !this.mainGiftBag.pkgShowTime) {
                this.mainGiftBag.pkgShowTime = Laya.timer.currTimer;
            }
        }
        GetGrowthGiftVos() {
            this.growthGiftVos.sort((a, b) => {
                if (a.isGet && !b.isGet)
                    return 1;
                if (!a.isGet && b.isGet)
                    return -1;
                return 0;
            });
            return this.growthGiftVos;
        }
        get canGrowthAward() {
            let vos = this.growthGiftVos;
            if (!vos)
                return false;
            for (let i = 0; i < vos.length; i++) {
                if (vos[i].canGet) {
                    return true;
                }
            }
            return false;
        }
        GetPkgVos(type = 0, unLock = 0) {
            let arr = [];
            for (let i = 0; i < this.pkgVos.length; i++) {
                let vo = this.pkgVos[i];
                if (type == 0) {
                    if (unLock == 0) {
                        arr.push(vo);
                    }
                    else if (unLock == 1) {
                        if (vo.isUnLock) {
                            arr.push(vo);
                        }
                    }
                    else if (unLock == 2) {
                        if (!vo.isUnLock) {
                            arr.push(vo);
                        }
                    }
                }
                else if (type == 1 && vo.isShopGiftBag) {
                    if (unLock == 0) {
                        arr.push(vo);
                    }
                    else if (unLock == 1) {
                        if (vo.isUnLock) {
                            arr.push(vo);
                        }
                    }
                    else if (unLock == 2) {
                        if (!vo.isUnLock) {
                            arr.push(vo);
                        }
                    }
                }
                else if (type == 2 && vo.isMainGiftBag) {
                    if (unLock == 0) {
                        arr.push(vo);
                    }
                    else if (unLock == 1) {
                        if (vo.isUnLock) {
                            arr.push(vo);
                        }
                    }
                    else if (unLock == 2) {
                        if (!vo.isUnLock) {
                            arr.push(vo);
                        }
                    }
                }
            }
            return arr;
        }
        get mainGiftBag() {
            if (GuideModel.Inst.guideVO != null && !GuideModel.Inst.guideVO.isFunc)
                return;
            if (this._mainGiftBag && (!this._mainGiftBag.isCanGet || !this._mainGiftBag.isCanShow)) {
                this._mainGiftBag = null;
            }
            if (!this._mainGiftBag) {
                let vos = this.GetPkgVos(2, 1);
                for (let i = 0; i < vos.length; i++) {
                    let vo = vos[i];
                    if (vo.isCanShow && vo.isCanGet) {
                        this._mainGiftBag = vo;
                        break;
                    }
                }
            }
            return this._mainGiftBag;
        }
    }
    GiftPKGModel.TYPE_SHOP = 1;
    GiftPKGModel.TYPE_MAINVIEW = 2;
    GiftPKGModel.inst = null;

    class LiHeModel {
        constructor() {
            this.reEnterGame = true;
        }
        static get Inst() {
            return this.inst = this.inst || new LiHeModel();
        }
        init() {
            this._award0 = this.GetConfigValueAward("giftbox1");
            this._award1 = this.GetConfigValueAward("giftbox2");
            this._award2 = this.GetConfigValueAward("giftbox3");
            this._state1 = StorageUtils.Inst.GetLiHeState(0);
            this._state2 = StorageUtils.Inst.GetLiHeState(1);
            this._state3 = StorageUtils.Inst.GetLiHeState(2);
            this._isAddGame = StorageUtils.Inst.isAddGame;
        }
        GetConfigValueAward(key) {
            let itemstr1 = ConfigValue.getItemByKey(key) && ConfigValue.getItemByKey(key).value;
            if (!itemstr1)
                return;
            let strArr1 = itemstr1.split(",");
            let award = new AwardVO(parseInt(strArr1[0]), { "count": parseInt(strArr1[1]) });
            return award;
        }
        get isAddGame() {
            return this._isAddGame;
        }
        set isAddGame(bol) {
            if (this._isAddGame == bol)
                return;
            this._isAddGame = bol;
            StorageUtils.Inst.isAddGame = bol;
        }
        get award1() {
            return this._award0;
        }
        get award2() {
            return this._award1;
        }
        get award3() {
            return this._award2;
        }
        get state1() {
            return this._state1;
        }
        get state2() {
            return this._state2;
        }
        get state3() {
            return this._state3;
        }
        set state1(val) {
            if (this._state1 == val)
                return;
            this._state1 = val;
            StorageUtils.Inst.SetLiHeState(0, val);
        }
        set state2(val) {
            if (this._state2 == val)
                return;
            this._state2 = val;
            StorageUtils.Inst.SetLiHeState(1, val);
        }
        set state3(val) {
            if (this._state3 == val)
                return;
            this._state3 = val;
            StorageUtils.Inst.SetLiHeState(2, val);
        }
        GetAward(awardVo) {
            this.GetAwardItem(awardVo.tempId, parseInt(awardVo.count.toString()), awardVo.itemRes.type);
        }
        GetAwardItem(itemId, num, itemType) {
            if (itemId == 1) {
                MoneyModel.gold.add(num);
            }
            else if (itemId == 2) {
                MoneyModel.diamond.add(num);
            }
            else if (itemId == 13) {
                let goldNum = BuildModel.Inst.GetAllBuildGoldRate(num * 60 * 60, false);
                MoneyModel.gold.add(goldNum);
            }
            else if (itemType == ItemModel.TYPE_ITEM || itemType == ItemModel.TYPE_BUILD) {
                ItemCtrl.Inst.AddItemVO(itemId, num);
            }
            else if (itemType == ItemModel.TYPE_SKIN) {
                this.AddSkinAward(itemId, num);
            }
        }
        AddSkinAward(itemId, num) {
            let skinResArr = skinResource.items;
            for (let i = 0; i < skinResArr.length; i++) {
                let res = skinResArr[i];
                if (res.itemId == itemId) {
                    SkinModel.Inst.AddSkin(res.id, num);
                    break;
                }
            }
        }
        get canHide() {
            if (PlatformMgr.Inst.isWX) {
                if (this.state1 != LiHeModel.comp)
                    return false;
                if (this.state2 != LiHeModel.comp)
                    return false;
                if (this.state3 != LiHeModel.comp)
                    return false;
                return true;
            }
            else {
                return true;
            }
        }
    }
    LiHeModel.inst = null;
    LiHeModel.libaoName1 = "添加桌面礼包";
    LiHeModel.libaoName2 = "订阅礼包";
    LiHeModel.libaoName3 = "推送礼包";
    LiHeModel.normal = 0;
    LiHeModel.canAward = 1;
    LiHeModel.comp = 2;

    class LoginResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint8();
            this.next = byteArray.readUint8();
            this.type = byteArray.readUint8();
            this.award = byteArray.readUint16();
            this.number = byteArray.readUint16();
        }
        static getItemByKey(id) {
            return LoginResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return LoginResource.itemsDic[key1 + "," + key2];
        }
    }
    LoginResource.items = new Array();
    LoginResource.itemsDic = new Dictionary();

    class LoginVO {
        constructor(id, day = 0) {
            this._id = 0;
            this._loginRes = null;
            this._day = 0;
            this._id = id;
            this._day = day;
        }
        get loginRes() {
            if (!this._loginRes) {
                this._loginRes = LoginResource.getItemByKey(this._id);
            }
            return this._loginRes;
        }
        get day() {
            return this._day;
        }
        get isCat() {
            return this.loginRes.type == LoginVO.CAT;
        }
        get isDaoJu() {
            return this.loginRes.type == LoginVO.DAOJU;
        }
    }
    LoginVO.DAOJU = 1;
    LoginVO.CAT = 2;

    class LoginModel {
        constructor() {
            this.loginVos = [];
            this.signMap = {};
        }
        static get Inst() {
            return this.inst = this.inst || new LoginModel();
        }
        init() {
            this._signCount = StorageUtils.Inst.signCount;
            this._canGetDima = StorageUtils.Inst.canGetDima;
            let vo = null;
            let res;
            for (let i = 0; i < LoginResource.items.length; i++) {
                res = LoginResource.items[i];
                vo = new LoginVO(res.id, i % 7);
                this.loginVos.push(vo);
                this.signMap[res.id] = vo;
            }
        }
        get signCount() {
            return this._signCount;
        }
        set signCount(val) {
            if (this._signCount == val)
                return;
            this._signCount = val;
            StorageUtils.Inst.signCount = val;
        }
        get canGetDima() {
            return this._canGetDima;
        }
        set canGetDima(bol) {
            if (this.canGetDima == bol)
                return;
            this._canGetDima = bol;
            StorageUtils.Inst.canGetDima = bol;
        }
        get canSign() {
            return StorageUtils.Inst.canSign;
        }
        get SevenLoginVos() {
            let arr = [];
            let start = 7;
            if (this.signCount / 7 < 1 || (this.signCount / 7 == 1) && !this.canSign) {
                start = 0;
            }
            for (let i = 0; i < 7; i++) {
                let loginVo = this.loginVos[start + i];
                arr.push(loginVo);
            }
            return arr;
        }
        get SixLoginVos() {
            let arr = this.SevenLoginVos;
            arr.pop();
            return arr;
        }
        get SevenLoginVo() {
            return this.SevenLoginVos[6];
        }
        get curLoginVo() {
            return this.SevenLoginVos[this.signCount % 7];
        }
        get preLoginVo() {
            return this.signCount - 1 >= 0 && this.SevenLoginVos[(this.signCount - 1) % 7];
        }
        GetAward() {
            let vo = this.curLoginVo;
            let id = vo.loginRes.award;
            let num = vo.loginRes.number;
            if (vo.isCat) {
                let catVo = CatModel.Inst.GetCatVOById(id);
                if (!catVo)
                    return;
                if (catVo.level == 0) {
                    catVo.level++;
                }
                else {
                    console.log("已拥有");
                }
            }
            else if (vo.isDaoJu) {
                let itemType = itemResource.getItemByKey(id).type;
                if (id == 1) {
                    MoneyModel.gold.add(num);
                }
                else if (id == 2) {
                    MoneyModel.diamond.add(num);
                }
                else if (id == 13) {
                    let goldNum = BuildModel.Inst.GetAllBuildGoldRate(num * 60 * 60, false);
                    MoneyModel.gold.add(goldNum);
                }
                else if (itemType == ItemModel.TYPE_ITEM || itemType == ItemModel.TYPE_BUILD) {
                    ItemCtrl.Inst.AddItemVO(id, num);
                }
                else if (itemType == ItemModel.TYPE_SKIN) {
                    this.AddSkinAward(id, num);
                }
            }
            LoginModel.Inst.signCount++;
            GameDispatcher.Inst.event(UIGameEvent.SHOW_SIGINBTN_RED, LoginModel.Inst.canSign || LoginModel.Inst.canGetDima);
        }
        AddSkinAward(itemId, num) {
            let skinResArr = skinResource.items;
            for (let i = 0; i < skinResArr.length; i++) {
                let res = skinResArr[i];
                if (res.itemId == itemId) {
                    SkinModel.Inst.AddSkin(res.id, num);
                    break;
                }
            }
        }
    }
    LoginModel.inst = null;

    class YuYueModel {
        constructor() {
        }
        static get Inst() {
            return this.inst = this.inst || new YuYueModel();
        }
        init() {
            let itemstr1 = ConfigValue.getItemByKey("subscribe1").value;
            let strArr1 = itemstr1.split(",");
            this._award1 = new AwardVO(parseInt(strArr1[0]), { "count": parseInt(strArr1[1]) });
            let itemstr2 = ConfigValue.getItemByKey("subscribe2").value;
            let strArr2 = itemstr2.split(",");
            this._award2 = new AwardVO(parseInt(strArr2[0]), { "count": parseInt(strArr2[1]) });
            this._time = parseFloat(ConfigValue.getItemByKey("subscribe2time").value);
            this._state = StorageUtils.Inst.canyuyue;
            this.startGetAwardTime = TimeModel.Inst.LocalTime;
        }
        get award1() {
            return this._award1;
        }
        get award2() {
            return this._award2;
        }
        get time() {
            return this._time;
        }
        get canGet() {
            return this.state == YuYueModel.state2;
        }
        get state() {
            if (PlatformMgr.Inst.isWX) {
                return this._state;
            }
            else {
                return YuYueModel.state5;
            }
        }
        set state(val) {
            this._state = val;
            StorageUtils.Inst.canyuyue = val;
        }
        get canShowRed() {
            return this._state == YuYueModel.state0 || this._state == YuYueModel.state2 || this._state == YuYueModel.state4;
        }
    }
    YuYueModel.inst = null;
    YuYueModel.state0 = 0;
    YuYueModel.state1 = 1;
    YuYueModel.state2 = 2;
    YuYueModel.state3 = 3;
    YuYueModel.state4 = 4;
    YuYueModel.state5 = 5;

    class UIBagInfoItem extends TListButtonItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
            this.c3 = this.getController("c3");
            this.c4 = this.getController("c4");
        }
        InitData(data) {
            super.InitData(data);
            if (data instanceof ItemVO) {
                this.part = 0;
                this.setItemIcon(data.itemUrl);
                this.isShowCat = data.catRes != null;
                if (this.isShowCat) {
                    this.setCatIcon(data.catHeadUrl);
                }
                this.quality = data.itemRes.quality;
                this.txt_count.text = "x" + data.count;
                this.txt_name.text = data.itemRes.name;
            }
            else if (data instanceof SkinVO) {
                this.setItemIcon(data.iconUrl);
                this.quality = data.itemRes.quality;
                this.txt_name.text = data.itemRes.name;
                this.isShowCat = false;
                this.txt_count.text = "";
                if (data.hasOwned) {
                    this.part = 0;
                }
                else {
                    this.part = data.part;
                }
            }
            else if (data instanceof itemResource) {
                this.isShowCat = false;
                this.quality = data.quality;
                this.txt_name.text = data.name;
                this.part = 0;
                this.setItemIcon(ItemVO.GetItemUrl(data));
            }
        }
        setItemIcon(url) {
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                this.img_icon.url = url;
            }));
        }
        setCatIcon(url) {
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                this.img_cat.icon = url;
            }));
        }
        set part(value) {
            this.c3.selectedIndex = value;
        }
        get part() {
            return this.c3.selectedIndex;
        }
        set isShowCat(value) {
            if (this.Data instanceof ItemVO && this.Data.catRes != null) {
                this.c2.selectedIndex = value ? 1 : 0;
            }
            else {
                this.c2.selectedIndex = 0;
            }
        }
        get isShowCat() {
            return this.c2.selectedIndex == 1;
        }
        set quality(value) {
            this.c1.selectedIndex = value;
        }
        get quality() {
            return this.c1.selectedIndex;
        }
        set hasOwned(value) {
            this.c4.selectedIndex = value ? 1 : 0;
        }
        get hasOwned() {
            return this.c4.selectedIndex == 1;
        }
        get itemVO() {
            return this.Data instanceof ItemVO ? this.Data : null;
        }
        get skinVO() {
            return this.Data instanceof SkinVO ? this.Data : null;
        }
        get itemRes() {
            if (this.Data instanceof ItemVO) {
                return this.Data.itemRes;
            }
            if (this.Data instanceof SkinVO) {
                return this.Data.itemRes;
            }
            else if (this.Data instanceof itemResource) {
                return this.Data;
            }
            return null;
        }
        Reset() {
            super.Reset();
        }
    }

    class UIItemInfoView extends UIBaseWindow {
        constructor() {
            super("bag", "UIItemInfoView");
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("bag", "UIBagInfoItem", UIBagInfoItem);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_TOP;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            if (data instanceof ItemVO) {
                this.item_good.InitData(data);
                this.txt_desc.text = data.itemRes.desc;
            }
            else if (data instanceof SkinVO) {
                this.item_good.InitData(data);
                this.txt_desc.text = data.itemRes.desc;
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
        }
        Reset() {
            super.Reset();
        }
    }

    class UIBagView extends UIBaseWindow {
        constructor() {
            super("bag", "UIBagView");
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
            this.c2 = this._view.getController("c2");
            this.c3 = this._view.getController("c3");
            this.eglist_items.setVirtual();
            this.eglist_suits.setVirtual();
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onTabClick, this, this.tab);
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onTab1Click, this, this.tab1);
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onItemClick, this, this.eglist_items);
        }
        InitData(data) {
            super.InitData(data);
            this.cb1.selected = GlobalConfig.isShowCat;
            this.tab.selectedIndex = 0;
            this.tab1.selectedIndex = 0;
            this.showItems();
            this.hideBanner();
        }
        onEffectEnd() {
            if (PlatformMgr.Inst.isWX) {
                if (AdModel.Inst.wxIsShowMuiltGrid) {
                    this.hideBanner();
                    this.showMgridAd();
                }
                else {
                    this.showBanner();
                }
            }
            else if (Laya.Browser.onHWMiniGame) {
                this.hideBanner();
                PlatformMgr.Inst.showAd(AdType.ad_3);
            }
            else if (Laya.Browser.onQQMiniGame) {
                this.hideBanner();
                PlatformMgr.Inst.showAd(AdType.ad_3);
            }
            else if (PlatformMgr.Inst.isMZ) {
                this.hideBanner();
                PlatformMgr.Inst.showAd(AdType.ad_3);
            }
            else if ((Laya.Browser.onVVMiniGame || Laya.Browser.onKGMiniGame) && GuideModel.Inst.guideVO == null) {
                this.hideBanner();
                AdCtrl.Inst.DelayShowInsertAd();
            }
            else if (Laya.Browser.onQGMiniGame) {
                this.hideBanner();
                AdCtrl.Inst.ShowNatiEmbed(this);
            }
            else {
                this.showBanner();
            }
        }
        showMgridAd() {
            this.ad_mgrid.InitData({ "adId": AdType.ad_29 });
        }
        onTabClick() {
            this.c2.selectedIndex = this.tab.selectedIndex;
            this.eglist_items.x = this.c2.selectedIndex == 0 ? this.n0.x + 15 : this.n0.x + 133;
            this.eglist_items.y = this.tab.y + 85;
            this.c3.selectedIndex = 0;
            this.showItems();
        }
        onTab1Click() {
            this.c3.selectedIndex = this.tab1.selectedIndex;
            this.showItems();
        }
        showItems() {
            this.eglist_items.numItems = 0;
            if (this.tab.selectedIndex == 0) {
                this.eglist_items.Data = ItemModel.Inst.GetAllItems();
            }
            else if (this.tab.selectedIndex == 1) {
                if (this.tab1.selectedIndex == 7) {
                    this.eglist_suits.Data = SkinModel.Inst.GetAllSuitVos();
                }
                else {
                    let skinVos = SkinModel.Inst.GetAllSkinVos(this.tab1.selectedIndex);
                    this.eglist_items.Data = skinVos;
                }
            }
            this.eglist_items.scrollToTop();
            this.eglist_suits.scrollToTop();
        }
        onItemClick(item) {
            if (item) {
                UISystem.Inst.CreateWindowView(UIItemInfoView, item.Data);
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.cb1) {
                this.showCat();
            }
        }
        showCat() {
            GlobalConfig.isShowCat = this.cb1.selected;
            let item = null;
            for (let i = 0; i < this.eglist_items.numChildren; i++) {
                item = this.eglist_items.getChildAt(i);
                if (item) {
                    item.isShowCat = GlobalConfig.isShowCat;
                }
            }
        }
        onResize() {
            super.onResize();
            this._view.setSize(Laya.stage.width, Laya.stage.height);
        }
        Reset() {
            super.Reset();
            Laya.timer.clearAll(this);
        }
    }

    class UIBoxView extends UIBaseWindow {
        constructor() {
            super("baoxiang", "UIBoxView");
            this.maxValue = 100;
            this.curVal = 0;
            this.reduceSpeed = 1;
            this.addSpeed = 10;
            this.showBannerVal = 70;
            this.isSaveCloseTime = true;
            this.enterBannerState = 0;
            this.isTween = false;
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = true;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.ON_ACTIVE, this.OnActive, this);
            Laya.timer.frameLoop(1, this, this.OnUpdate);
        }
        InitData(data) {
            super.InitData(data);
            if (typeof data === "boolean") {
                this.isSaveCloseTime = data;
            }
            this.enterBannerState = 0;
            this.pro.value = 0;
            this.pro.max = this.maxValue;
            this.c1.selectedIndex = 0;
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                this.btn_close.visible = false;
                this.DeleteShowCloseBtn();
            }
            this.hideBanner();
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_open) {
                this.OnClickBtnOpen();
            }
        }
        CheckShowBanner() {
            if (this.pro.value >= this.showBannerVal) {
                if (!this.isShowBanner) {
                    this.showBanner({ "top": (this.btn_open.y + this.btn_open.height - PlatformModel.Inst.bannerSizeH) * PlatformModel.Inst.scaleY });
                }
            }
        }
        OnClickBtnOpen() {
            if (this.isOpen)
                return;
            let endVal = this.curVal + this.addSpeed;
            this.isTween = true;
            fgui.GTween.kill(this.pro);
            fgui.GTween.to(this.curVal, endVal, 0.4).setTarget(this.pro).setEase(fgui.EaseType.ElasticOut).onUpdate((tw) => {
                this.curVal = tw.value.x;
                this.curVal = Math.min(this.curVal, this.maxValue);
            }).onComplete(() => {
                this.isTween = false;
            });
            this.UpdateProVal();
            this.Shake();
        }
        Shake() {
            this.loader_box.pivotY = 1;
            this.loader_box.pivotX = 0.5;
            fgui.GTween.shake(this.loader_box.x, this.loader_box.y, 3, 0.4).setTarget(this.loader_box)
                .onUpdate((tw) => {
                let target = tw.target;
                this.loader_box.rotation = target.x - tw.value.x;
                target.x = tw.value.x;
                target.y = tw.value.y;
            }, this);
        }
        OnUpdate() {
            if (this.isOpen)
                return;
            if (this.curVal > 0 && !this.isTween) {
                this.curVal -= this.reduceSpeed;
                this.curVal = Math.max(0, this.curVal);
            }
            this.UpdateProVal();
        }
        UpdateProVal() {
            this.pro.value = this.curVal;
            if (this.pro.value >= this.maxValue) {
                this.OpenBox();
            }
            else {
                this.CheckShowBanner();
            }
        }
        get isOpen() {
            return this.c1.selectedIndex == 1;
        }
        OpenBox() {
            this.c1.selectedIndex = 1;
            Laya.timer.once(800, this, () => {
                this.GetAward();
                this.onBtnCloseClick();
            });
        }
        OnActive(bol) {
            if (!this.isShowBanner)
                return;
            if (!bol) {
                if (this.enterBannerState == 0)
                    this.enterBannerState = 1;
            }
            else {
                if (this.enterBannerState == 1)
                    this.enterBannerState = 2;
                if (this.enterBannerState == 2)
                    this.OpenBox();
            }
        }
        GetAward() {
            let vo = AdModel.Inst.boxAward;
            if (vo instanceof AwardVO) {
                this.GetAwardItem(vo.tempId, vo.getCount(), vo.itemRes.type);
                UISystem.Inst.CreateWindowView(UIAwardView, vo);
            }
        }
        GetAwardItem(itemId, num, itemType) {
            if (itemId == 1) {
                MoneyModel.gold.add(num);
            }
            else if (itemId == 2) {
                MoneyModel.diamond.add(num);
            }
            else if (itemId == 13) {
                let goldNum = BuildModel.Inst.GetAllBuildGoldRate(num * 60 * 60, false);
                MoneyModel.gold.add(goldNum);
            }
            else if (itemType == ItemModel.TYPE_ITEM || itemType == ItemModel.TYPE_BUILD) {
                ItemCtrl.Inst.AddItemVO(itemId, num);
            }
            else if (itemType == ItemModel.TYPE_SKIN) {
                this.AddSkinAward(itemId, num);
            }
        }
        AddSkinAward(itemId, num) {
            let skinResArr = skinResource.items;
            for (let i = 0; i < skinResArr.length; i++) {
                let res = skinResArr[i];
                if (res.itemId == itemId) {
                    SkinModel.Inst.AddSkin(res.id, num);
                    break;
                }
            }
        }
        onBtnCloseClick() {
            if (this.isSaveCloseTime) {
                AdCtrl.Inst.SetBoxViewCloseTime();
            }
            GameDispatcher.Inst.event(UIGameEvent.ONCLOSEBOXVIEW);
            super.onBtnCloseClick();
        }
        DeleteShowCloseBtn() {
            this.HideCloseBtn();
            Laya.timer.once(AdModel.Inst.boxCloseBtnDelayShowTime, this, this.ShowCloseBtn);
        }
        ShowCloseBtn() {
            this.btn_close.visible = true;
        }
        HideCloseBtn() {
            this.btn_close.visible = false;
        }
        onResize() {
            super.onResize();
            this._view.setSize(Laya.stage.width, Laya.stage.height);
        }
        Reset() {
            super.Reset();
            Laya.timer.clear(this, this.ShowCloseBtn);
            Laya.timer.clear(this, this.OnUpdate);
            this.curVal = 0;
        }
    }

    class RedButton extends TBaseButton {
        constructor() {
            super();
        }
        constructFromXML(xml) {
            super.constructFromXML(xml);
        }
        InitUI() {
            super.InitUI();
            this.c2 = this.getController("c2");
            this.c3 = this.getController("c3");
            this.c4 = this.getController("c4");
            this.t2 = this.getTransition("t2");
            this.touchable = true;
        }
        setRed(value, spineName = null) {
            if (this.c2) {
                this.c2.selectedIndex = value;
            }
            if (spineName && value == 1) {
                if (spineName == "kezhaohuan") {
                    this.c4.selectedIndex = 1;
                }
                else if (spineName == "xinlibao") {
                    this.c4.selectedIndex = 2;
                }
                if (!this.t2.playing) {
                    this.t2.play();
                }
            }
            else {
                this.c4.selectedIndex = 0;
            }
        }
        get isRed() {
            return this.c2 && this.c2.selectedIndex > 0;
        }
        set isDouble(value) {
            if (this.c3) {
                this.c3.selectedIndex = value ? 1 : 0;
            }
        }
        get isDouble() {
            return this.c3 && this.c3.selectedIndex > 0;
        }
    }

    class UIAdNativeBannerView extends UIBaseWindow {
        constructor() {
            super("ad", "UINativeBannerView");
            this.sp_icon = null;
            this.img_logo = null;
            this.clickHandler = null;
            this.adRes = null;
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_AD;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            AdCtrl.Inst.SetBannerCloseBtnScale(this.btn_close);
        }
        InitData(param) {
            let adObj = param[0];
            let showHandler = param[1];
            this.clickHandler = param[2];
            this.adRes = param[3];
            if (showHandler != null)
                showHandler.run();
            let lf = this;
            let url = adObj["icon"];
            let iconArr = adObj["imgUrlList"];
            if (iconArr != null && iconArr.length > 0)
                url = iconArr[0];
            else {
                iconArr = adObj["logoUrlList"];
                if (iconArr != null && iconArr.length > 0)
                    url = iconArr[0];
            }
            if (!url)
                return;
            if (adObj && adObj.title && this.txt_title)
                this.txt_title.text = adObj.title;
            if (adObj && adObj.desc && this.txt_desc)
                this.txt_desc.text = adObj.desc;
            this.visible = false;
            this.sp_icon && AssetCache.LoadIcon(this.sp_icon, url, Laya.Handler.create(this, this.onLoadComplete1, [url]));
            this.img_logo && AssetCache.LoadIcon(this.img_logo, adObj.logoUrl);
        }
        onLoadComplete1() {
            this.visible = true;
        }
        AddListener() {
            super.AddListener();
            this.btn_dowload.onClick(this, this.onBtnOkClick);
        }
        RemoveListener() {
            super.RemoveListener();
            this.btn_dowload.offClick(this, this.onBtnOkClick);
        }
        onBtnOkClick() {
            if (this.clickHandler != null) {
                this.clickHandler.run();
            }
        }
        onBtnCloseClick() {
            if (this.adRes != null)
                PlatformMgr.Inst.hideAd(this.adRes.id);
            super.onBtnCloseClick();
        }
        onResize() {
            super.onResize();
        }
        Reset() {
            this.adRes = null;
            if (this.clickHandler) {
                this.clickHandler.recover();
            }
            this.clickHandler = null;
        }
    }

    class AdResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.platform = byteArray.readUTFString();
            this.id = byteArray.readUint8();
            this.name = byteArray.readUTFString();
            this.type = byteArray.readUint8();
            this.adIds = ConfigLib.ReadArray(byteArray);
            this.params = ConfigLib.ReadArray(byteArray);
            this.result0 = ConfigLib.ReadArray(byteArray);
            this.result1 = ConfigLib.ReadArray(byteArray);
        }
        static getItemByKey(id) {
            return AdResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return AdResource.itemsDic[key1 + "," + key2];
        }
    }
    AdResource.items = new Array();
    AdResource.itemsDic = new Dictionary();

    class UINativeIconAd extends TListButtonItem {
        constructor() {
            super();
            this.adDataArr = [];
            this.nativeAd = null;
            this.adId = 0;
            this.nativeIconId = "";
        }
        InitUI() {
            super.InitUI();
            this.visible = false;
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.CLOSE_AD, this.onCloseAd, this);
        }
        RemoveListener() {
            super.RemoveListener();
        }
        InitData(data) {
            this.Reset();
            super.InitData(data);
            data = data || {};
            this.adId = data.adId || AdType.ad_4;
            this.nativeIconId = data.nativeIconId || PlatformMgr.Inst.nativeIconId;
            if (this.nativeIconId) {
                this.visible = false;
                if (PlatformMgr.Inst.isTT) {
                    return;
                }
                else if (Laya.Browser.onQQMiniGame) {
                    let obj = window["qq"].getSystemInfoSync();
                    let windowWidth = obj.windowWidth;
                    let windowHeight = obj.windowHeight;
                    let scaleX = windowWidth / Laya.stage.width;
                    let scaleY = windowHeight / Laya.stage.height;
                    let left = Math.ceil(this.x * scaleX);
                    let top = Math.ceil(this.y * scaleY);
                    if (windowWidth - left < 83) {
                        left = windowWidth - 83;
                    }
                }
                else if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                    this.reshow30();
                    Laya.timer.loop(30000, this, this.reshow30);
                }
                else {
                    this.reshow30();
                    let nativeIconUptime = 30;
                    if (nativeIconUptime > 0) {
                        Laya.timer.loop(nativeIconUptime * 1000, this, this.reshow30);
                    }
                }
                this.onClick(this, this.clickSelf);
            }
            else {
                this.Reset();
            }
        }
        onCloseAd(res) {
            if (res && res.id == this.adId) {
                Laya.timer.once(30000, this, this.reshow30);
            }
        }
        reshow30() {
            Laya.timer.clear(this, this.showNativeAd);
            this.nativeAd && this.nativeAd.destroy && this.nativeAd.destroy();
            this.nativeAd = null;
            PlatformMgr.Inst.hideAd(this.adId, true);
            this.creatNativeAd([this.nativeIconId]);
        }
        creatNativeAd(posIds) {
            let self = this;
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                let scaleX = PlatformModel.Inst.scaleX;
                let scaleY = PlatformModel.Inst.scaleY;
                let left = Math.ceil(this.x * scaleX);
                let top = Math.ceil(this.y * scaleY);
                PlatformMgr.Inst.showAd(this.adId, null, { "left": left, "top": top });
                return;
            }
            if (!posIds || posIds.length <= 0)
                return;
            let posId = posIds.pop();
            if (this.nativeAd == null) {
                if (Laya.Browser.onQGMiniGame) {
                    self.nativeAd = qg.createNativeAd({ "adUnitId": posId });
                }
                else if (Laya.Browser.onVVMiniGame) {
                    self.nativeAd = qg.createNativeAd({ "posId": posId });
                }
                if (this.nativeAd) {
                    this.nativeAd.onError(function (err) {
                        Logger.log(self, "----->原生ICON广告创建失败posId:" + posId + " err: " + JSON.stringify(err));
                        return;
                    });
                    function onLoad(res) {
                        self.showNativeAd(res);
                    }
                    this.nativeAd.onLoad(onLoad);
                    this.nativeAd.load();
                }
            }
        }
        showNativeAd(res) {
            let self = this;
            let adList = res.adList;
            let adData = null;
            for (adData of adList) {
                if ((adData.iconUrlList != null && adData.iconUrlList.length >= 1) ||
                    (adData.imgUrlList != null && adData.imgUrlList.length >= 1)) {
                    self.showAd(adData);
                    break;
                }
            }
        }
        shake() {
            let self = this;
            fgui.GTween.shake(self.x, self.y, 5, 0.6).setTarget(this)
                .onUpdate(function t2(tw) {
                self.x = tw.value.x;
                self.y = tw.value.y;
            }, this);
        }
        showAd(nativeData) {
            this.adData = nativeData;
            if (nativeData == null) {
                return;
            }
            const pics = [];
            if (nativeData.icon && nativeData.icon.length > 0) {
                pics.push(nativeData.icon);
            }
            if (nativeData.logoUrl) {
                pics.push(nativeData.logoUrl);
            }
            if (nativeData.iconUrlList) {
                for (let i = 0; i < nativeData.iconUrlList.length; i++) {
                    if (nativeData.iconUrlList[i]) {
                        pics.push(nativeData.iconUrlList[i]);
                    }
                }
            }
            if (nativeData.imgUrlList) {
                for (let i = 0; i < nativeData.imgUrlList.length; i++) {
                    if (nativeData.imgUrlList[i]) {
                        pics.push(nativeData.imgUrlList[i]);
                    }
                }
            }
            this.visible = true;
            nativeData.pics = pics;
            let title = nativeData.title;
            let desc = nativeData.desc;
            this.icon = pics[0];
            PlatformMgr.Inst.reportNativeShow(this.adData, this.nativeAd);
            this.shake();
        }
        clickSelf() {
            if (this.adData) {
                PlatformMgr.Inst.reportNativeClick(this.adData, this.nativeAd);
            }
        }
        Reset() {
            super.Reset();
            this.offClick(this, this.clickSelf);
            let adRes = AdResource.getItemBy2Keys(PlatformMgr.Inst.GetPlatformName(), this.adId);
            adRes && PlatformMgr.Inst.hideAd(this.adId, false);
            this.adId = 0;
            this.nativeAd && this.nativeAd.destroy && this.nativeAd.destroy();
            this.nativeAd = null;
            Laya.timer.clear(this, this.reshow30);
            fgui.GTween.kill(this, true);
            Laya.timer.clear(this, this.showNativeAd);
            PlatformMgr.Inst.isWX && PlatformMgr.Inst.hideBlock();
        }
    }

    class FlyStar extends TListItem {
        constructor() {
            super();
            this.speedV2 = new Laya.Vector2();
            this.fromPos = new Laya.Point();
        }
        InitUI() {
            super.InitUI();
            this.t0 = this.getTransition("t0");
            this.touchable = false;
        }
        InitData(data) {
            super.InitData(data);
        }
        flyFrom(fromX, fromY) {
            this.fromPos.setTo(fromX, fromY);
            this.speedV2.setValue(this.x - fromX, this.y - fromY);
            Laya.Vector2.normalize(this.speedV2, this.speedV2);
            this.t0.play(null, -1);
            let rt = MathUtil.makeRandom(30, 200);
            Laya.timer.once(rt, this, () => {
                Laya.timer.frameLoop(3, this, this.move);
            });
        }
        move() {
            this.setXY(this.x + this.speedV2.x, this.y + this.speedV2.y);
        }
        Reset() {
            super.Reset();
            this.t0.stop();
            fgui.GTween.kill(this);
            Laya.timer.clearAll(this);
        }
    }

    class UIGetItemView extends UIBaseWindow {
        constructor() {
            super("bag", "UIGetItemView");
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowMaskBg = false;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            this.item_good.InitData(data);
            this.item_good.isLock = false;
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_ok) {
                SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.Click_OK));
                let count = parseInt(this.txt_count.text);
                if (count < 1) {
                    count = 1;
                }
                if (this.item_good.Data instanceof ItemVO) {
                    ItemCtrl.Inst.AddItemVO(this.item_good.itemRes.id, count);
                    UISystem.Inst.CreateWindowView(UITipView, `获得物品[${this.item_good.itemRes.name}x${count}]`);
                    UISystem.Inst.CreateWindowView(UIAwardView, this.item_good.Data);
                }
                else if (this.item_good.Data instanceof SkinVO) {
                    SkinModel.Inst.AddSkin(this.item_good.Data);
                    UISystem.Inst.CreateWindowView(UITipView, `获得皮肤[${this.item_good.itemRes.name}x${count}]`);
                    UISystem.Inst.CreateWindowView(UIAwardView, this.item_good.Data);
                }
                else if (this.item_good.Data instanceof itemResource) {
                    if (this.item_good.Data.type == ItemModel.TYPE_MONEY) {
                        if (this.item_good.Data.id == ItemModel.ITEM_GOLD) {
                            MoneyModel.gold.add(count);
                        }
                        else if (this.item_good.Data.id == ItemModel.ITEM_GEM) {
                            MoneyModel.diamond.add(count);
                        }
                        UISystem.Inst.CreateWindowView(UITipView, `获得[${this.item_good.Data.name}x${count}]`);
                    }
                    else if (this.item_good.Data.type == ItemModel.TYPE_ITEM) {
                        ItemCtrl.Inst.AddItemVO(this.item_good.Data.id, count);
                        UISystem.Inst.CreateWindowView(UITipView, `获得物品[${this.item_good.Data.name}x${count}]`);
                    }
                    else if (this.item_good.Data.type == ItemModel.TYPE_SKIN) {
                        SkinModel.Inst.AddSkin(this.item_good.Data.id, count);
                        UISystem.Inst.CreateWindowView(UITipView, `获得皮肤[${this.item_good.itemRes.name}x${count}]`);
                    }
                    let item = new ItemVO();
                    item.itemId = this.item_good.Data.id;
                    item.count = count;
                    UISystem.Inst.CreateWindowView(UIAwardView, item);
                }
                else if (this.item_good.Data instanceof skinResource) {
                    SkinModel.Inst.AddSkin(this.item_good.Data.id);
                    UISystem.Inst.CreateWindowView(UITipView, `获得皮肤[${this.item_good.Data.name}x${count}]`);
                }
                this.close();
            }
        }
        Reset() {
            super.Reset();
        }
    }

    class UIGMSuitItem extends TListItem {
        constructor() {
            super();
            this.selectItem = null;
        }
        InitUI() {
            super.InitUI();
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onItemClick, this, this.eglist_items);
            this.AddGameListener(UIGameEvent.ADD_SKIN, this.onAddSkin, this);
        }
        InitData(data) {
            this.AddListener();
            super.InitData(data);
            if (data instanceof SuitVO) {
                this.txt_name.text = data.suitRes.name;
                let skinVos = data.skinVos;
                this.txt_count.text = data.skinOwnedCount + "/" + data.skinIds.length;
                this.eglist_items.Data = skinVos;
            }
        }
        onItemClick(item) {
            this.selectItem = item;
            if (item) {
                let skinVo = SkinModel.Inst.GetSkinVo(item.skinVO.tempId);
                if (skinVo == null || !skinVo.hasOwned) {
                    UISystem.Inst.CreateWindowView(UIAlertView, {
                        "text": `确定添加皮肤[${item.skinVO.itemRes.name}]?`, "okHandler": Laya.Handler.create(this, () => {
                            item.part = 0;
                            let skinVo = SkinModel.Inst.AddSkin(item.skinVO.tempId);
                            UISystem.Inst.CreateWindowView(UIAwardView, skinVo);
                        })
                    });
                }
                else {
                    UISystem.Inst.CreateWindowView(UIAlertView, {
                        "text": `确定移除皮肤[${item.skinVO.itemRes.name}]?`, "okHandler": Laya.Handler.create(this, () => {
                            UISystem.Inst.CreateWindowView(UITipView, `移除皮肤[${item.skinVO.itemRes.name}]`);
                            SkinModel.Inst.RemoveSkin(item.skinVO.tempId);
                            item.part = item.skinVO.part;
                        })
                    });
                }
            }
        }
        onAddSkin(skinVo) {
            if (this.selectItem && this.selectItem.skinId == skinVo.skinRes.id) {
                this.selectItem.InitData(skinVo);
            }
        }
        Reset() {
            super.Reset();
            this.RemoveListener();
            this.selectItem = null;
        }
    }

    class principalResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint8();
            this.taskType = byteArray.readUint8();
            this.icon = byteArray.readUTFString();
            this.task = byteArray.readUTFString();
            this.desc = byteArray.readUTFString();
            this.cost = byteArray.readUint16();
            this.finish = byteArray.readUint8();
            this.item = byteArray.readUint16();
            this.num = byteArray.readUint16();
            this.otherCost = byteArray.readUint16();
        }
        static getItemByKey(id) {
            return principalResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return principalResource.itemsDic[key1 + "," + key2];
        }
    }
    principalResource.items = new Array();
    principalResource.itemsDic = new Dictionary();

    class MainTaskVo extends BaseTaskVO {
        constructor(tempId) {
            super(tempId, 1);
        }
        setup(data) {
            this.pro = data.pro;
            this.state = data.state;
        }
        get curPro() {
            return this.pro;
        }
        set curPro(val) {
            if (this.pro == this.maxCount)
                return;
            this.pro = val;
            if (this.pro > this.maxCount) {
                this.pro = this.maxCount;
            }
            if (this.pro == this.maxCount) {
                this.state = BaseTaskVO.STATE_CAN_AWARD;
                GameDispatcher.Inst.event(UIGameEvent.SHOW_TASKBTN_RED, true);
            }
            StorageUtils.Inst.updateMainTask(this.tempId, this.curPro, this.state);
            GameDispatcher.Inst.event(UIGameEvent.UPDATE_TASK, this.type);
        }
        get maxCount() {
            return this.taskRes.cost;
        }
        get taskRes() {
            return principalResource.getItemByKey(this.tempId);
        }
    }

    class ProBar extends TListButtonItem {
        constructor() {
            super();
            this._max = 100;
            this._value = 0;
        }
        InitUI() {
            super.InitUI();
            this.bar_msk = this.bar.getChild("msk").asImage;
        }
        InitData(data) {
            super.InitData(data);
        }
        set max(value) {
            this._max = value;
            this.updateBar();
        }
        get max() {
            return this._max;
        }
        set value(val) {
            this._value = val;
            this.updateBar();
        }
        get value() {
            return this._value;
        }
        updateBar() {
            let bigNum = this._value instanceof BigNumber ? this._value : new BigNumber(this._value);
            bigNum = bigNum.dividedBy(this._max).multipliedBy(10000).toFixed();
            let pre = parseInt(bigNum) / 10000;
            pre = pre < 0 ? 0 : (pre > 1 ? 1 : pre);
            let bw = Math.ceil(pre * this.bar.width);
            this.bar_msk.width = bw;
            this.bar.visible = this.bar_msk.width >= 1;
        }
        Reset() {
            super.Reset();
            this._value = 0;
            this.bar_msk.width = 0;
            this._max = 100;
        }
    }

    class UIDayTaskItem extends TListItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
        }
        InitData(data) {
            super.InitData(data);
            this.UpdateData();
        }
        AddListener() {
            this.item.onClick(this, this.GetAward);
        }
        GetAward() {
            let vo = this.Data;
            if (!vo)
                return;
            if (vo.canAward) {
                TaskCtr.Inst.GetDayTaskAward(vo);
                UITaskView.itemVo.itemId = vo.taskRes.item;
                UITaskView.itemVo.count = vo.taskRes.num;
                UISystem.Inst.CreateWindowView(UIAwardView, UITaskView.itemVo);
            }
        }
        UpdateData() {
            let vo = this.Data;
            if (!vo)
                return;
            let taskRes = vo.taskRes;
            this.txt_title.text = taskRes.desc;
            this.probar.max = vo.maxCount;
            this.probar.value = vo.curPro;
            let item = itemResource.getItemByKey(taskRes.item);
            let iconName = item.icon;
            let url;
            if (item.type == 3) {
                url = UrlUtils.GetIconPartUrl(iconName);
            }
            else {
                url = UrlUtils.GetIconItemsUrl(iconName);
            }
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (iconurl) => {
                this.item.icon = iconurl;
            }));
            this.item.title = taskRes.num + "";
            this.item.SetState(vo.state);
            this.SetState(vo.state);
        }
        SetState(state) {
            if (state == BaseTaskVO.STATE_NORMAL) {
                this.c1.selectedIndex = 0;
            }
            else if (state == BaseTaskVO.STATE_CAN_AWARD) {
                this.c1.selectedIndex = 1;
            }
            else if (state == BaseTaskVO.STATE_COMPLETED) {
                this.c1.selectedIndex = 2;
            }
        }
        RemoveListener() {
            this.item.offClick(this, this.GetAward);
        }
        Reset() {
            super.Reset();
        }
    }

    class UITaskView extends UIBaseWindow {
        constructor() {
            super("task", "UITaskView");
            this.txt_time = null;
            this.dayCompAward = null;
            this.dayCompCtr = null;
            this.c1 = null;
            this.c2 = null;
            this.c3 = null;
            this.date = TimeModel.Inst.getDate();
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("task", "UITaskAwardItem", UITaskAwardItem);
            PanelRegister.registerClass("task", "UITaskAwardItem2", UITaskAwardItem);
            PanelRegister.registerClass("task", "UIDayTaskItem", UIDayTaskItem);
            PanelRegister.registerClass("task", "UIDayTaskItem2", UIDayTaskItem);
            PanelRegister.registerClass("task", "ProBar", ProBar);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
            this.txt_time = this.item_day.getChild("txt_time").asTextField;
            this.dayCompAward = this.item_day.getChild("item");
            this.dayCompCtr = this.item_day.getController("c1");
            this.c1 = this._view.getController("c1");
            this.c2 = this._view.getController("c2");
            if (this._btns.indexOf(this.dayCompAward) == -1) {
                this._btns.push(this.dayCompAward);
            }
        }
        InitData(param) {
            super.InitData(param);
            this.UpdateData();
            Laya.timer.loop(1000, this, this.OnUpdateTime);
            this.OnUpdateTime();
            if (TaskModel.Inst.isOpenDayTask) {
                this.c1.selectedIndex = 0;
                this.item_day.visible = true;
            }
            else {
                this.c1.selectedIndex = 1;
                this.item_day.visible = false;
            }
        }
        onEffectEnd() {
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                if (AdModel.Inst.wxIsShowMuiltGrid) {
                    this.hideBanner();
                    this.showMgridAd();
                }
                else {
                    let group_mainY = (Laya.stage.height - PlatformModel.Inst.bannerSizeH / PlatformModel.Inst.scaleY) - this.group_main.height;
                    this.group_main.y = Math.min(this.group_main.y, group_mainY);
                    this.showBanner();
                }
            }
            else if ((Laya.Browser.onVVMiniGame || Laya.Browser.onKGMiniGame) && GuideModel.Inst.guideVO == null) {
                this.hideBanner();
                AdCtrl.Inst.DelayShowInsertAd();
            }
            else if (PlatformModel.Inst.windowHeight > 720) {
                this.showBanner();
            }
        }
        showMgridAd() {
            this.ad_mgrid.InitData({ "adId": AdType.ad_27 });
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.UPDATE_TASK, this.UpdateData, this);
            this.AddGameListener(UIGameEvent.ON_SHOW, this.onShow, this);
            this.AddGameListener(UIGameEvent.ON_HIDE, this.onHide, this);
        }
        RemoveListener() {
            super.RemoveListener();
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.item_main) {
                this.OnMainTaskAwardClick();
            }
            else if (btn == this.dayCompAward) {
                this.OnAllDayTaskCompClick();
            }
        }
        onShow(panelName) {
            if (panelName == "UIAwardView") {
                if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                    if (AdModel.Inst.wxIsShowMuiltGrid) {
                        this.ad_mgrid.Reset();
                    }
                    else {
                        this.hideBanner();
                    }
                }
                else {
                    this.hideBanner();
                }
            }
        }
        onHide(panelName) {
            if (panelName == "UIAwardView") {
                if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                    if (AdModel.Inst.wxIsShowMuiltGrid) {
                        this.showMgridAd();
                    }
                    else {
                        this.showBanner();
                    }
                }
                else {
                    this.showBanner();
                }
            }
        }
        OnAllDayTaskCompClick() {
            if (TaskModel.Inst.allDayTaskState == BaseTaskVO.STATE_CAN_AWARD) {
                TaskCtr.Inst.GetAllDayTaskCompAward();
                UITaskView.itemVo.itemId = TaskModel.Inst.DAYTASKS_COMP_AWARDID;
                UITaskView.itemVo.count = TaskModel.Inst.DAYTASKS_COMP_DIMA;
                UISystem.Inst.CreateWindowView(UIAwardView, UITaskView.itemVo);
            }
        }
        OnMainTaskAwardClick() {
            let mainTaskVo = TaskModel.Inst.mainTask;
            if (mainTaskVo && mainTaskVo.canAward) {
                TaskCtr.Inst.GetMainTaskAward(mainTaskVo);
                UITaskView.itemVo.itemId = mainTaskVo.taskRes.item;
                UITaskView.itemVo.count = mainTaskVo.taskRes.num;
                UISystem.Inst.CreateWindowView(UIAwardView, UITaskView.itemVo);
            }
        }
        UpdateData(type = -1) {
            if (type == -1) {
                this.UpdateMainTaskDate();
                this.UpdateDayTaskDate();
            }
            else if (type == 0) {
                this.UpdateDayTaskDate();
            }
            else if (type == 1) {
                this.UpdateMainTaskDate();
            }
        }
        OnUpdateTime() {
            this.date.setTime(TimeModel.Inst.LocalTime);
            let hours = 23 - this.date.getHours();
            let min = 59 - this.date.getMinutes();
            let sec = 59 - this.date.getSeconds();
            let offestX = hours < 10 ? "0" : "";
            let offestY = min < 10 ? "0" : "";
            let offestZ = sec < 10 ? "0" : "";
            this.txt_time.text = `${offestX}${hours}:${offestY}${min}:${offestZ}${sec}`;
            if (TaskModel.Inst.isOpenDayTask) {
                this.c1.selectedIndex = 0;
                this.item_day.visible = true;
            }
            else {
                this.c1.selectedIndex = 1;
                this.item_day.visible = false;
            }
        }
        UpdateMainTaskDate() {
            let mainTaskVo = TaskModel.Inst.mainTask;
            if (!mainTaskVo)
                return;
            let mainTaskRes = mainTaskVo.taskRes;
            this.txt_task_name.text = mainTaskRes.task;
            this.txt_task_desc.text = mainTaskRes.desc;
            if (TaskModel.Inst.allMainTaskIsComped) {
                this.c2.selectedIndex = 1;
            }
            else {
                this.c2.selectedIndex = 0;
            }
            this.probar.max = mainTaskVo.maxCount;
            this.probar.value = mainTaskVo.curPro;
            this.probar.title = mainTaskVo.curPro + "/" + mainTaskVo.maxCount;
            let url1 = UrlUtils.GetIconTaskUrl(mainTaskRes.icon);
            PanelRegister.loadUrl(url1, Laya.Handler.create(this, (url) => {
                this.icon_task.url = url;
            }));
            let item = itemResource.getItemByKey(mainTaskRes.item);
            let iconName = item.icon;
            let url;
            if (item.type == 3) {
                url = UrlUtils.GetIconPartUrl(iconName);
            }
            else {
                url = UrlUtils.GetIconItemsUrl(iconName);
            }
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (iconUrl) => {
                this.item_main.icon = iconUrl;
            }));
            this.item_main.title = mainTaskRes.num + "";
            this.item_main.SetState(mainTaskVo.state);
        }
        UpdateDayTaskDate() {
            this.eglist_items.Data = TaskModel.Inst.dayTaskVOArr;
            let item = itemResource.getItemByKey(TaskModel.Inst.DAYTASKS_COMP_AWARDID);
            let iconName = item.icon;
            let url;
            if (item.type == 3) {
                url = UrlUtils.GetIconPartUrl(iconName);
            }
            else {
                url = UrlUtils.GetIconItemsUrl(iconName);
            }
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (iconurl) => {
                this.dayCompAward.icon = iconurl;
            }));
            this.dayCompAward.title = TaskModel.Inst.DAYTASKS_COMP_DIMA + "";
            let state = TaskModel.Inst.allDayTaskState;
            this.dayCompAward.SetState(state);
            if (state == BaseTaskVO.STATE_NORMAL) {
                this.dayCompCtr.selectedIndex = 0;
            }
            else if (state == BaseTaskVO.STATE_CAN_AWARD) {
                this.dayCompCtr.selectedIndex = 1;
            }
            else if (state == BaseTaskVO.STATE_COMPLETED) {
                this.dayCompCtr.selectedIndex = 2;
            }
        }
        onResize() {
            super.onResize();
            this._view.setSize(Laya.stage.width, Laya.stage.height);
            if (this.ad_mgrid) {
                this.ad_mgrid.onResize();
            }
        }
        Reset() {
            super.Reset();
            Laya.timer.clear(this, this.OnUpdateTime);
            Laya.timer.clearAll(this);
        }
    }
    UITaskView.itemVo = new ItemVO();

    class UIGMTaskItem extends TListItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
        }
        AddListener() {
            super.AddListener();
            this.onClick(this, this.OnClickBtn);
        }
        OnClickBtn() {
            if (this.Data instanceof MainTaskVo) {
                TaskCtr.Inst.SetCurMainTask(this.Data);
                UISystem.Inst.CreateWindowView(UITaskView);
            }
        }
        InitData(data) {
            this.AddListener();
            super.InitData(data);
            if (data instanceof MainTaskVo) {
                this.txt_task.text = data.taskRes.task;
                let item = itemResource.getItemByKey(data.taskRes.item);
                let iconUrl = UrlUtils.GetIconItemsUrl(item.icon);
                PanelRegister.loadUrl(iconUrl, Laya.Handler.create(this, (url) => {
                    this.loader_sp.url = url;
                }));
                this.txt_num.text = data.taskRes.num + "";
            }
        }
        Reset() {
            super.Reset();
            this.RemoveListener();
        }
    }

    class UIGMView extends UIBaseWindow {
        constructor() {
            super("common", "UIGMView");
            this.c3 = null;
            this.selectItem = null;
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("common", "UIGMSuitItem", UIGMSuitItem);
            PanelRegister.registerClass("common", "UIGMTaskItem", UIGMTaskItem);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = true;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
            this.c3 = this._view.getController("c3");
            this.eglist_suit.setVirtual();
            this.eglist_items.setVirtual();
            this.eglist_cat.setVirtual();
            this.eglist_task.setVirtual();
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onTabClick, this, this.tab);
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onTabClick1, this, this.tab1);
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onItemClick, this, this.eglist_items);
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onCatClick, this, this.eglist_cat);
            this.AddGameListener(UIGameEvent.ADD_SKIN, this.onAddSkin, this);
        }
        InitData(data) {
            super.InitData(data);
            this.tab.selectedIndex = 0;
            this.onTabClick();
        }
        onTabClick() {
            this.c3.selectedIndex = 0;
            if (this.tab.selectedIndex == 0) {
                this.eglist_items.Data = this.getItemResList();
            }
            else if (this.tab.selectedIndex == 1) {
                this.eglist_cat.Data = CatModel.Inst.GetAllCatVOs();
            }
            else if (this.tab.selectedIndex == 2) {
                this.tab1.selectedIndex = 0;
                this.onTabClick1();
            }
            else if (this.tab.selectedIndex == 3) {
                this.c3.selectedIndex = 1;
                this.eglist_task.Data = TaskModel.Inst.GetMainTaskVos();
            }
        }
        onTabClick1() {
            if (this.tab1.selectedIndex == 0) {
                this.eglist_items.Data = this.getSkinResList();
            }
            else {
                let suitVos = SkinModel.Inst.GetAllSuitVos();
                this.eglist_suit.Data = suitVos;
            }
            this.eglist_suit.scrollToTop();
            this.eglist_items.scrollToTop();
        }
        getSkinResList() {
            let arr = [];
            let skinRes = null;
            for (let i = 0; i < skinResource.items.length; i++) {
                skinRes = skinResource.items[i];
                if (skinRes.suitId == 0) {
                    arr.push(skinRes);
                }
            }
            return arr;
        }
        getItemResList() {
            let list = [];
            let res = null;
            for (let i = 0; i < itemResource.items.length; i++) {
                res = itemResource.items[i];
                if (res && (res.type == ItemModel.TYPE_ITEM || res.type == ItemModel.TYPE_MONEY)) {
                    list.push(res);
                }
            }
            return list;
        }
        onItemClick(item) {
            this.selectItem = item;
            UISystem.Inst.CreateWindowView(UIGetItemView, item.Data);
        }
        onCatClick(item) {
            CatCtrl.Inst.AddCat(item.catVO, false);
            item.UpdateData();
        }
        onAddSkin(skinVo) {
            if (this.selectItem && this.selectItem.skinId == skinVo.skinRes.id) {
                this.selectItem.InitData(skinVo);
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
        }
        Reset() {
            super.Reset();
        }
    }

    class UIDriftingBottleView extends UIBaseWindow {
        constructor() {
            super("driftingBottle", "UIDriftingBottleView");
            this.date = TimeModel.Inst.getDate();
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = false;
            this.viewStruct.isEffect = false;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
            this.c2 = this._view.getController("c2");
            this.c3 = this._view.getController("c3");
            this.loader_img.visible = false;
            this.item_award.isAd = false;
            this.item_award.isYellow = false;
        }
        InitData(param) {
            this.m_param = param;
            this.spine_bottle.play("bottle");
            this.UpdateTask();
            this.c3.selectedIndex = 0;
            if (DriftingBottleModel.Inst.curTask.isCompleted) {
                this.c1.selectedIndex = 3;
                this.c3.selectedIndex = 2;
                this.spine_tips.play("biaoti");
                this.LoadModel();
                this.hideBanner();
            }
            else if (DriftingBottleModel.Inst.isThrowBottle) {
                this.c1.selectedIndex = 2;
                this.hideBanner();
                this.c3.selectedIndex = 1;
                this.spine_tips.play("biaoti");
                this.LoadModel();
                if (DriftingBottleModel.Inst.curTask.taskRes.taskType == TaskType.TASKTYPE_GET_MAOQIU) {
                    this.btn_spine.play("qipao_1");
                }
                else {
                    this.btn_spine.play("qipao_4");
                }
            }
            else {
                this.c1.selectedIndex = 0;
                DriftingBottleModel.Inst.isThrowBottle = false;
                this.spine_tips.play("piaoliuping");
                this.showBanner();
            }
            Laya.timer.loop(1000, this, this.UpdateTime);
            this.UpdateTime();
            this.btn_waterSpine.play("water");
        }
        AddListener() {
            super.AddListener();
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_ren) {
                this.OnClickRen();
            }
            else if (btn == this.btn_come) {
                this.OnClickComeLand();
            }
            else if (btn == this.btn_get) {
                this.OnClickGetAward();
            }
            else if (btn == this.btn_yaoqing) {
                this.YaoqingShangDao();
            }
            else if (btn == this.btn_close1) {
                this.onBtnCloseClick();
            }
        }
        OnClickRen() {
            if (!this.spine_bottle)
                return;
            if (DriftingBottleModel.Inst.isThrowBottle)
                return;
            DriftingBottleModel.Inst.isThrowBottle = true;
            this.spine_bottle.PlayAni(1, false, Laya.Handler.create(this, this.ThrowBottle));
        }
        ThrowBottle() {
            this.c1.selectedIndex = 1;
            this.spine_tips && this.spine_tips.play("biaoti");
            this.btn_spine && this.btn_spine.play("qipao_3");
            let complete = Laya.Handler.create(this, (sp) => {
                this.sp3d_cat.PlayAni("event_Box", true, Laya.Handler.create(this, () => {
                    this.sp3d_cat.PlayAni(this.GetRandomAni(), true);
                }));
            });
            this.sp3d_cat.LoadCat("abyssinianB", complete);
        }
        GetRandomAni() {
            let strArr = CVModel.Inst.GetValue(CVModel.bActId, CVModel.valType_array);
            let ranIndex = MathUtil.makeRandom(0, strArr.length);
            let id = parseInt(strArr[ranIndex]);
            let ranActRes = actResource.getItemByKey(id);
            return ranActRes ? ranActRes.name : "idle_wait";
        }
        LoadModel() {
            let complete = Laya.Handler.create(this, (sp) => {
                if (DriftingBottleModel.Inst.driftingBottleState != DriftingBottleModel.shangdaoState) {
                    this.sp3d_cat.PlayAni(this.GetRandomAni(), true);
                }
                this.PlayLandAni(this.sp3d_cat);
            });
            this.sp3d_cat.LoadCat("abyssinianB", complete);
        }
        PlayLandAni(cat) {
            if (DriftingBottleModel.Inst.driftingBottleState == DriftingBottleModel.shangdaoState) {
                LoadQueue.Inst.loadSprite3D(UrlUtils.GetBuildUrl("cat2_boat"), Laya.Handler.create(this, this.onLoadComplete));
                this.sp3d_cat.PlayAni("idle_wait", true);
            }
        }
        onLoadComplete(sp) {
            this.sp3d_cat.AddSprite3d(sp);
            let catModel = this.sp3d_cat.catModel;
            let animator = sp.getComponent(Laya.Animator);
            if (animator) {
                animator.enabled = false;
            }
            this.sp3d_cat.ResetCatPos();
            sp.transform.localScale = new Laya.Vector3(0.6, 0.6, 0.6);
            catModel.transform.localScale = new Laya.Vector3(0.3, 0.3, 0.3);
            catModel.transform.localRotationEulerY = sp.transform.localRotationEulerY = -50;
            this.sp3d_cat.InitModel(sp, new Laya.Point(this.sp3d_cat.x, this.sp3d_cat.y));
            this.PlayShuiwen(sp);
        }
        PlayShuiwen(sp) {
            EffectMgr.Inst.playConfigEffect(EffectMgr.Eff_shuiwen, sp, null, null, null, null, 4000, true, false);
            Laya.timer.once(4000, this, this.PlayShuiwen, [sp]);
        }
        OnClickComeLand() {
            this.c1.selectedIndex = 2;
            this.c3.selectedIndex = 0;
            this.c3.selectedIndex = 1;
            this.btn_spine.play("qipao_1");
            DriftingBottleModel.Inst.driftingBottleState = DriftingBottleModel.taskState;
            GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [DriftingBottleModel.Inst.curTask.taskRes.taskType, -1, 0]);
            this.hideBanner();
        }
        OnClickGetAward() {
            let curTask = DriftingBottleModel.Inst.curTask;
            if (curTask.canAward) {
                DriftingBottleModel.Inst.GetAward(curTask);
                if (DriftingBottleModel.Inst.curTask.taskRes.taskType == TaskType.TASKTYPE_GET_MAOQIU) {
                    this.btn_spine.play("qipao_1");
                }
                else {
                    this.btn_spine.play("qipao_4");
                }
                this.CheckTaskComlete();
                let mainTaskType = DriftingBottleModel.Inst.curTask.taskRes.taskType;
                let id = DriftingBottleModel.Inst.curTask.taskRes.id;
                GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [mainTaskType, id, 0]);
                this.UpdateTask();
                UISystem.Inst.CreateWindowView(UIAwardView, curTask.award);
            }
        }
        CheckTaskComlete() {
            if (DriftingBottleModel.Inst.curTask.isCompleted) {
                this.c1.selectedIndex = 3;
                this.c3.selectedIndex = 0;
                this.c3.selectedIndex = 2;
                this.btn_spine.visible = false;
                this.YaoqingShangDao();
                this.hideBanner();
            }
        }
        YaoqingShangDao() {
            if (!this.sp3d_cat.catModel)
                return;
            if (DriftingBottleModel.Inst.driftingBottleState == DriftingBottleModel.taskCompState) {
                DriftingBottleModel.Inst.driftingBottleState = DriftingBottleModel.shangdaoState;
                PlatformMgr.Inst.aldSendEvent(EventType.piaoliupingyaoqing);
                this.btn_spine.visible = true;
                this.btn_spine.play("qipao_2");
                this.PlayLandAni(this.sp3d_cat);
            }
            else if (DriftingBottleModel.Inst.driftingBottleState == DriftingBottleModel.shangdaoCompState) {
                if (CatModel.Inst.GetLastMaowuCount() > 0) {
                    DriftingBottleModel.Inst.driftingBottleState = DriftingBottleModel.yaoQingCompState;
                    let catVo = CatModel.Inst.GetCatVOById(DriftingBottleModel.Inst.yaoQingCatID);
                    if (catVo.IsHas()) {
                        UIAwardView.awardVO.setData({ "tempId": catVo.catRes.item, "count": catVo.catRes.itemNum });
                        UISystem.Inst.CreateWindowView(UIAwardView, UIAwardView.awardVO);
                    }
                    else {
                        CatCtrl.Inst.AddCat(CatModel.Inst.GetCatVOById(DriftingBottleModel.Inst.yaoQingCatID));
                    }
                    this.close();
                }
                else {
                    I18nUtils.ShowI18nById("maowu_no_new");
                }
            }
        }
        UpdateTime() {
            if (DriftingBottleModel.Inst.driftingBottleState == DriftingBottleModel.shangdaoState) {
                this.btn_spine.visible = true;
                this.btn_spine.play("qipao_2");
                this.date.setTime(TimeModel.Inst.LocalTime);
                let hours = 23 - this.date.getHours();
                let min = 59 - this.date.getMinutes();
                let sec = 59 - this.date.getSeconds();
                let offestX = hours < 10 ? "0" : "";
                let offestY = min < 10 ? "0" : "";
                let offestZ = sec < 10 ? "0" : "";
                this.txt_djs.text = `喵星人正在赶来中！马上就可以迎接喵星人的到来了!\n\n猫猫上岛倒计时: ${offestX}${hours}:${offestY}${min}:${offestZ}${sec}`;
                this.btn_yaoqing.enabled = false;
            }
            else {
                this.btn_yaoqing.enabled = true;
            }
            if (DriftingBottleModel.Inst.driftingBottleState == DriftingBottleModel.shangdaoCompState) {
                this.txt_djs.text = "喵星人已经赶到了！赶快迎接喵星人吧！";
            }
        }
        UpdateTask() {
            let curTask = DriftingBottleModel.Inst.curTask;
            this.pro.max = curTask.maxCount;
            this.pro.value = curTask.curPro;
            this.txt_desc.text = curTask.taskRes.desc;
            this.txt_num.text = "x" + MathUtil.goldFormat(curTask.award.count);
            PanelRegister.loadUrl(curTask.award.itemUrl, Laya.Handler.create(this, (url) => {
                this.item_award.icon = url;
            }));
            this.item_award.title = MathUtil.goldFormat(curTask.award.count);
            if (curTask.canAward) {
                this.c2.selectedIndex = 1;
                this.btn_get.enabled = true;
                let c1 = this.btn_get.getController("c1");
                if (c1) {
                    c1.selectedIndex = 1;
                }
                this.item_award.SetState(BaseTaskVO.STATE_CAN_AWARD);
            }
            else {
                this.c2.selectedIndex = 0;
                this.btn_get.enabled = false;
                let c1 = this.btn_get.getController("c1");
                if (c1) {
                    c1.selectedIndex = 0;
                }
                this.item_award.SetState(BaseTaskVO.STATE_NORMAL);
            }
            let url1 = UrlUtils.GetIconTaskUrl(curTask.taskRes.icon);
            if (url1) {
                PanelRegister.loadUrl(url1, Laya.Handler.create(this, (url) => {
                    this.loder_task.url = url;
                }));
            }
        }
        RemoveListener() {
        }
        onResize() {
            super.onResize();
            this._view.setSize(Laya.stage.width, Laya.stage.height);
        }
        Reset() {
            super.Reset();
            Laya.timer.clear(this, this.UpdateTime);
            Laya.timer.clear(this, this.PlayShuiwen);
        }
    }

    class ExchangeModel {
        constructor() {
            this.selectedItems = [];
            this.selectedMap = {};
        }
        static get Inst() {
            return this.inst = this.inst || new ExchangeModel();
        }
        init() {
        }
        addExchangeItem(res, count, isBuy) {
            let vo = this.getSelectedItem(res.id);
            if (vo == null) {
                vo = new ExchangeVO();
                vo.tempId = res.id;
                vo.isBuy = isBuy;
                this.selectedItems.push(vo);
                this.selectedMap[res.id] = vo;
            }
            vo.count = count;
            return vo;
        }
        removeExchangeVO(tempId) {
            let vo = this.getSelectedItem(tempId);
            if (vo) {
                let index = this.selectedItems.indexOf(vo);
                if (index != -1) {
                    this.selectedItems.splice(index, 1);
                }
                delete this.selectedMap[tempId];
            }
        }
        getSelectedItem(tempId) {
            return this.selectedMap[tempId];
        }
        clear() {
            this.selectedItems.length = 0;
            this.selectedMap = {};
        }
        exchange(isBuy) {
            let vo;
            let nipCount = 0;
            for (let i = 0; i < this.selectedItems.length; i++) {
                vo = this.selectedItems[i];
                if (isBuy) {
                    ItemCtrl.Inst.AddItemVO(vo.changeRes.id, vo.count);
                    nipCount += vo.changeRes.buy * vo.count;
                }
                else {
                    ItemCtrl.Inst.RemoveItemVO(vo.changeRes.id, vo.count);
                    nipCount += vo.changeRes.sale * vo.count;
                }
            }
            if (isBuy) {
                ItemCtrl.Inst.RemoveItemVO(ItemModel.ITEM_NIP, nipCount);
                UISystem.Inst.CreateWindowView(UIAwardView, this.selectedItems.slice());
            }
            else {
                ItemCtrl.Inst.AddItemVO(ItemModel.ITEM_NIP, nipCount);
                UIAwardView.awardVO.setData({ "tempId": ItemModel.ITEM_NIP, "count": nipCount, "type": AwardVO.TYPE_ITEM });
                UISystem.Inst.CreateWindowView(UIAwardView, UIAwardView.awardVO);
            }
            this.clear();
            GameDispatcher.Inst.event(UIGameEvent.EXCHANGE_SUCCESS);
            GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [TaskType.TASKTYPE_BUYDAOJU, this.selectedItems]);
        }
    }
    ExchangeModel.inst = null;

    class UIExchangeGoodItem extends TListButtonItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
            this.c3 = this.getController("c3");
            this.c4 = this.getController("c4");
            this.c5 = this.getController("c5");
            this.c6 = this.getController("c6");
        }
        InitData(data) {
            super.InitData(data);
            if (data instanceof changeResource) {
                data = itemResource.getItemByKey(data.id);
            }
            else if (data instanceof ItemVO) {
                data = data.itemRes;
            }
            if (data instanceof itemResource) {
                this.isShowCat = GlobalConfig.isShowCat;
                let catRes = ItemModel.Inst.getCatRes(data.id);
                let catVo = catRes && CatModel.Inst.GetCatVOById(catRes.id);
                this.img_owned.visible = catVo && catVo.IsHas();
                let url = catRes ? PanelRegister.getUrl("icon_cat_head", catRes.icon) : "";
                PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                    this.img_cat.icon = url;
                }));
                this.quality = data.quality;
                this.txt_name.text = data.name;
                this.part = 0;
                PanelRegister.loadUrl(ItemVO.GetItemUrl(data), Laya.Handler.create(this, (url) => {
                    this.img_icon.url = url;
                }));
            }
        }
        set isShowCat(value) {
            this.c2.selectedIndex = value ? 1 : 0;
        }
        get isShowCat() {
            return this.c2.selectedIndex == 1;
        }
        set part(value) {
            this.c3.selectedIndex = value;
        }
        get part() {
            return this.c3.selectedIndex;
        }
        set quality(value) {
            this.c1.selectedIndex = value;
        }
        get quality() {
            return this.c1.selectedIndex;
        }
        Reset() {
            super.Reset();
        }
    }

    class UIExchangeItem extends TListButtonItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
            this.c3 = this.getController("c3");
            this.c4 = this.getController("c4");
            this.c5 = this.getController("c5");
            this.c6 = this.getController("c6");
        }
        InitData(data) {
            super.InitData(data);
            this.selected = false;
            this.isLock = false;
            this.isSuit = false;
            let isBuy = this.parent && this.parent.data && this.parent.data.tab == 0;
            let changeRes;
            if (data instanceof changeResource) {
                changeRes = data;
                data = itemResource.getItemByKey(data.id);
                this.txt_count.text = "";
            }
            else if (data instanceof ExchangeVO) {
                this.txt_count.text = "x" + data.count;
                changeRes = data.changeRes;
                data = data.itemRes;
            }
            else if (data instanceof ItemVO) {
                this.txt_count.text = "x" + data.count;
                changeRes = changeResource.getItemByKey(data.itemId);
                data = data.itemRes;
            }
            let catRes = ItemModel.Inst.getCatRes(data.id);
            let catVo = catRes && CatModel.Inst.GetCatVOById(catRes.id);
            this.img_owned.visible = catVo && catVo.IsHas();
            if (changeRes) {
                if (isBuy) {
                    this.txt_cost.text = "" + changeRes.buy;
                }
                else {
                    this.txt_cost.text = "" + changeRes.sale;
                }
            }
            if (data instanceof ItemVO) {
                this.part = 0;
                PanelRegister.loadUrl(data.itemUrl, Laya.Handler.create(this, (url) => {
                    this.img_icon.icon = url;
                }));
                this.isShowCat = GlobalConfig.isShowCat;
                if (this.isShowCat) {
                    let url = data.catHeadUrl;
                    PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                        this.img_cat.icon = url;
                    }));
                }
                this.quality = data.itemRes.quality;
                this.txt_count.text = "x" + data.count;
                this.txt_name.text = data.itemRes.name;
            }
            else if (data instanceof itemResource) {
                let url = catRes ? PanelRegister.getUrl("icon_cat_head", catRes.icon) : "";
                PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                    this.img_cat.icon = url;
                }));
                this.isShowCat = GlobalConfig.isShowCat;
                this.quality = data.quality;
                this.txt_name.text = data.name;
                this.part = 0;
                PanelRegister.loadUrl(ItemVO.GetItemUrl(data), Laya.Handler.create(this, (url) => {
                    this.img_icon.icon = url;
                }));
            }
        }
        get exchangeVo() {
            return this.Data;
        }
        get changeRes() {
            if (this.Data instanceof ExchangeVO) {
                return this.Data.changeRes;
            }
            return this.Data;
        }
        set part(value) {
            this.c3.selectedIndex = value;
        }
        get part() {
            return this.c3.selectedIndex;
        }
        set isShowCat(value) {
            this.c2.selectedIndex = value ? 1 : 0;
        }
        get isShowCat() {
            return this.c2.selectedIndex == 1;
        }
        set quality(value) {
            this.c1.selectedIndex = value;
        }
        get quality() {
            return this.c1.selectedIndex;
        }
        set hasOwned(value) {
            this.c4.selectedIndex = value ? 1 : 0;
        }
        get hasOwned() {
            return this.c4.selectedIndex == 1;
        }
        set isLock(value) {
            this.c5.selectedIndex = value ? 1 : 0;
        }
        get isLock() {
            return this.c5.selectedIndex == 1;
        }
        set isSuit(value) {
            this.c6.selectedIndex = value ? 1 : 0;
        }
        get isSuit() {
            return this.c6.selectedIndex == 1;
        }
        get skinId() {
            if (this.Data instanceof SkinVO) {
                return this.Data.skinRes.id;
            }
            else if (this.Data instanceof skinResource) {
                return this.Data.id;
            }
            return 0;
        }
        get itemRes() {
            if (this.Data instanceof ItemVO) {
                return this.Data.itemRes;
            }
            if (this.Data instanceof SkinVO) {
                return this.Data.itemRes;
            }
            else if (this.Data instanceof itemResource) {
                return this.Data;
            }
            else if (this.Data instanceof skinResource) {
                let itemRes = itemResource.getItemByKey(this.Data.id);
                return itemRes;
            }
            return null;
        }
        Reset() {
            super.Reset();
        }
    }

    class UIAddNipView extends UIBaseWindow {
        constructor() {
            super("exchange", "UIAddNipView");
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
            this.probar.max = 5;
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            let cnt = StorageUtils.Inst.GetNipAdCount();
            this.btn_get.enabled = cnt > 0;
            this.probar.value = 5 - cnt;
            this.txt_desc.text = `今日已获得${this.probar.value}/5次`;
            this.showBanner();
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_get) {
                PlatformMgr.Inst.showAd(AdType.ad_2, Laya.Handler.create(this, (isSuccess) => {
                    if (isSuccess) {
                        ItemCtrl.Inst.AddItemVO(ItemModel.ITEM_NIP, 50);
                        StorageUtils.Inst.AddNipAdCount(-1);
                        this.close();
                    }
                }));
            }
        }
        Reset() {
            super.Reset();
        }
    }

    class UIExchangeAlert extends UIBaseWindow {
        constructor() {
            super("exchange", "UIExchangeAlert");
            this.count = 1;
            this.maxCount = 99;
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            this.item.InitData(data.item);
            if (data.item instanceof changeResource) {
                let selectVo = ExchangeModel.Inst.getSelectedItem(data.item.id);
                this.count = selectVo ? selectVo.count : 1;
                this.updateCount();
                this.c1.selectedIndex = 0;
                this.maxCount = 99;
            }
            else if (data.item instanceof ItemVO) {
                this.count = data.item.count;
                this.updateCount();
                this.c1.selectedIndex = 1;
                this.maxCount = data.item.count;
            }
            this.showBanner();
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_add1) {
                this.count++;
                this.updateCount();
            }
            else if (btn == this.btn_add2) {
                this.count += 10;
                this.updateCount();
            }
            else if (btn == this.btn_red1) {
                this.count--;
                this.updateCount();
            }
            else if (btn == this.btn_red2) {
                this.count -= 10;
                this.updateCount();
            }
            else if (btn == this.btn_ok) {
                let okHandler = this.Data.okHandler;
                if (okHandler) {
                    okHandler.runWith([this.item.Data, this.count]);
                }
                this.close();
            }
        }
        updateCount() {
            if (this.count > this.maxCount) {
                this.count = this.maxCount;
            }
            else if (this.count < 1) {
                this.count = 1;
            }
            this.txt_count.text = `x${this.count}`;
        }
        Reset() {
            super.Reset();
        }
    }

    class UIExchangeBuyAlert extends UIBaseWindow {
        constructor() {
            super("exchange", "UIExchangeBuyAlert");
            this.isBuy = false;
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            this.isBuy = data;
            this.c1.selectedIndex = this.isBuy ? 0 : 1;
            this.eglist_items.Data = ExchangeModel.Inst.selectedItems;
            this.updateCost();
            this.showBanner();
        }
        updateCost() {
            let cost = 0;
            let item;
            for (let i = 0; i < this.eglist_items.numChildren; i++) {
                item = this.eglist_items.getChildAt(i);
                if (item) {
                    if (item.Data instanceof ExchangeVO || item.Data instanceof ItemVO) {
                        if (this.isBuy) {
                            cost += item.exchangeVo.changeRes.buy * item.Data.getCount();
                        }
                        else {
                            cost += item.exchangeVo.changeRes.sale * item.Data.getCount();
                        }
                    }
                }
            }
            this.comp_coin.title = "" + (this.isBuy ? -cost : cost);
            if (this.isBuy) {
                let count = ItemModel.Inst.GetItemCount(ItemModel.ITEM_NIP);
                this.btn_ok.enabled = count >= cost;
            }
            else {
                this.btn_ok.enabled = true;
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_ok) {
                ExchangeModel.Inst.exchange(this.isBuy);
                this.close();
            }
        }
        Reset() {
            super.Reset();
        }
    }

    class UIExchangeView extends UIBaseWindow {
        constructor() {
            super("exchange", "UIExchangeView");
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("exchange", "UIExchangeItem", UIExchangeItem);
            PanelRegister.registerClass("exchange", "UIExchangeGoodItem", UIExchangeGoodItem);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
            this.eglist_items.setVirtual();
            this.eglist_items2.setVirtual();
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onTabClick, this, this.tab);
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onItemClick, this, this.eglist_items);
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onItemClick2, this, this.eglist_items2);
            this.AddGameListener(UIGameEvent.EXCHANGE_SUCCESS, this.onExchangeSuccess, this);
            this.AddGameListener(UIGameEvent.ADD_ITEM, this.onAddItem, this);
            this.AddGameListener(UIGameEvent.ON_SHOW, this.onShow, this);
            this.AddGameListener(UIGameEvent.ON_HIDE, this.onHide, this);
        }
        InitData(data) {
            super.InitData(data);
            this.updateNips();
            this.cb.selected = GlobalConfig.isShowCat;
            this.tab.selectedIndex = 0;
            this.showItems();
            if (PlatformMgr.Inst.isWX) {
                if (AdModel.Inst.wxIsShowMuiltGrid) {
                    this.hideBanner();
                    this.showMgridAd();
                }
                else {
                    this.showBanner();
                }
            }
            else if (Laya.Browser.onHWMiniGame) {
                this.hideBanner();
                PlatformMgr.Inst.showAd(AdType.ad_3);
            }
            else if (Laya.Browser.onQQMiniGame) {
                this.hideBanner();
                PlatformMgr.Inst.showAd(AdType.ad_3);
            }
            else if (PlatformMgr.Inst.isMZ) {
                this.hideBanner();
                PlatformMgr.Inst.showAd(AdType.ad_3);
            }
            else if ((Laya.Browser.onVVMiniGame || Laya.Browser.onKGMiniGame) && GuideModel.Inst.guideVO == null) {
                this.hideBanner();
                AdCtrl.Inst.DelayShowInsertAd();
            }
            else {
                this.showBanner();
            }
        }
        onShow(panelName) {
            if (panelName == "UIAwardView" || panelName == "UIExchangeBuyAlert" || panelName == "UIAddNipView") {
                if (PlatformMgr.Inst.isWX) {
                    if (AdModel.Inst.wxIsShowMuiltGrid) {
                        this.ad_mgrid.Reset();
                    }
                    else {
                        this.hideBanner();
                    }
                }
                else {
                    this.hideBanner();
                }
            }
        }
        onHide(panelName) {
            if (panelName == "UIAwardView" || panelName == "UIExchangeBuyAlert" || panelName == "UIAddNipView") {
                if (PlatformMgr.Inst.isWX) {
                    if (AdModel.Inst.wxIsShowMuiltGrid) {
                        this.showMgridAd();
                    }
                    else {
                        this.showBanner();
                    }
                }
                else {
                    this.showBanner();
                }
            }
        }
        showMgridAd() {
            this.ad_mgrid.InitData({ "adId": AdType.ad_30 });
        }
        onAddItem(itemVO) {
            if (itemVO && itemVO.itemId == ItemModel.ITEM_NIP) {
                this.updateNips();
            }
        }
        updateNips() {
            let count = ItemModel.Inst.GetItemCount(ItemModel.ITEM_NIP);
            this.comp_coin1.title = "" + count;
        }
        onTabClick() {
            this.showItems();
        }
        showItems() {
            this.eglist_items.data = { "tab": this.tab.selectedIndex, "selected": false };
            this.eglist_items2.data = { "tab": this.tab.selectedIndex, "selected": true };
            if (this.tab.selectedIndex == 0) {
                this.eglist_items.Data = changeResource.items;
            }
            else if (this.tab.selectedIndex == 1) {
                this.eglist_items.Data = ItemModel.Inst.GetAllToyItems();
            }
            ExchangeModel.Inst.clear();
            this.eglist_items2.Data = ExchangeModel.Inst.selectedItems;
            this.comp_coin2.title = "0";
            this.eglist_items.scrollToTop();
            this.eglist_items2.scrollToTop();
            this.updateCost();
        }
        onItemClick(item) {
            UISystem.Inst.CreateWindowView(UIExchangeAlert, {
                "item": item.Data, "okHandler": Laya.Handler.create(this, (res, count) => {
                    if (res instanceof ItemVO) {
                        res = changeResource.getItemByKey(res.itemId);
                    }
                    if (res instanceof changeResource) {
                        ExchangeModel.Inst.addExchangeItem(res, count, this.tab.selectedIndex == 0);
                    }
                    this.eglist_items2.Data = ExchangeModel.Inst.selectedItems;
                    this.updateCost();
                })
            });
        }
        onItemClick2(item) {
            if (item && item.changeRes) {
                ExchangeModel.Inst.removeExchangeVO(item.changeRes.id);
                this.eglist_items2.Data = ExchangeModel.Inst.selectedItems;
                this.updateCost();
            }
        }
        updateCost() {
            let cost = 0;
            let item;
            for (let i = 0; i < this.eglist_items2.numChildren; i++) {
                item = this.eglist_items2.getChildAt(i);
                if (item) {
                    if (item.Data instanceof ExchangeVO) {
                        cost += this.tab.selectedIndex == 0 ? item.exchangeVo.changeRes.buy * item.Data.count : item.exchangeVo.changeRes.sale * item.Data.count;
                    }
                    else if (item.Data instanceof ItemVO) {
                        cost += this.tab.selectedIndex == 0 ? item.exchangeVo.changeRes.buy * item.Data.getCount() : item.exchangeVo.changeRes.sale * item.Data.getCount();
                    }
                }
            }
            this.comp_coin2.title = "" + (this.tab.selectedIndex == 0 ? -cost : cost);
            this.txt_desc.visible = this.eglist_items2.Data == null || this.eglist_items2.Data.length <= 0;
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_ok) {
                if (this.eglist_items2.numChildren <= 0) {
                    return;
                }
                UISystem.Inst.CreateWindowView(UIExchangeBuyAlert, this.tab.selectedIndex == 0);
            }
            else if (btn == this.cb) {
                GlobalConfig.isShowCat = this.cb.selected;
                this.updateCats();
            }
            else if (btn == this.btn_add) {
                UISystem.Inst.CreateWindowView(UIAddNipView);
            }
        }
        updateCats() {
            let item;
            for (let i = 0; i < this.eglist_items.numChildren; i++) {
                item = this.eglist_items.getChildAt(i);
                item.isShowCat = GlobalConfig.isShowCat;
            }
            for (let i = 0; i < this.eglist_items2.numChildren; i++) {
                item = this.eglist_items2.getChildAt(i);
                item.isShowCat = GlobalConfig.isShowCat;
            }
        }
        onExchangeSuccess() {
            let count = ItemModel.Inst.GetItemCount(ItemModel.ITEM_NIP);
            this.comp_coin1.title = "" + count;
            this.eglist_items2.Data = null;
            this.updateCost();
            if (this.tab.selectedIndex == 1) {
                this.eglist_items.Data = ItemModel.Inst.GetAllToyItems();
            }
        }
        Reset() {
            super.Reset();
        }
    }

    class LoginGiftBagListItem extends TListItem {
        constructor() {
            super();
            this.sp_item = null;
            this.txt_desc = null;
        }
        InitUI() {
            super.InitUI();
        }
        InitData(data) {
            super.InitData(data);
            this.UpdateData();
        }
        UpdateData() {
            let vo = this.Data;
            if (vo instanceof AwardVO) {
                let name = "";
                let num = 0;
                let desc = "";
                let iconUrl = "";
                if (vo.IsItem) {
                    name = vo.itemRes.name;
                    num = vo.count;
                    iconUrl = vo.itemUrl;
                }
                if (num > 0) {
                    desc = name + " x" + num;
                }
                else {
                    desc = name;
                }
                PanelRegister.loadUrl(iconUrl, Laya.Handler.create(this, (url) => {
                    this.sp_item.url = url;
                }));
                this.txt_desc.text = desc;
            }
        }
        Reset() {
            super.Reset();
        }
    }

    class UIGiftBagView extends UIBaseWindow {
        constructor() {
            super("giftBag", "UIGiftBagView");
            this.sp_catModel = null;
            this.c1 = null;
            this.c2 = null;
            this.c3 = null;
            this.txt_time = null;
            this.btn_sure = null;
            this.eglist_items = null;
            this.date = null;
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("giftBag", "LoginGiftBagListItem", LoginGiftBagListItem);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_CENTER;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isEffect = true;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
            this.c2 = this._view.getController("c2");
            this.c3 = this._view.getController("c3");
            this.date = new Date();
        }
        AddListener() {
            super.AddListener();
            Laya.timer.clear(this, this.onUpdate);
            Laya.timer.loop(1000, this, this.onUpdate);
        }
        InitData(data) {
            super.InitData(data);
            this.UpdateData();
            this.onUpdate();
            this.showBanner();
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_sure) {
                this.OnClickLinQuBtn();
            }
        }
        UpdateData() {
            let vo = this.Data;
            if (vo instanceof PackageVO) {
                this.eglist_items.Data = vo.pkgAwardVos;
                this.c2.selectedIndex = 0;
                this.c1.selectedIndex = 0;
                let isCat = true;
                let catId = null;
                if (vo.pkgName == "白色土耳其安哥拉猫礼包") {
                    this.c3.selectedIndex = 2;
                    catId = 10033;
                }
                else if (vo.pkgName == "棕色孟加拉猫礼包") {
                    this.c3.selectedIndex = 1;
                    catId = 10034;
                }
                else if (vo.pkgName == "蓝色阿比西尼猫") {
                    this.c3.selectedIndex = 3;
                    catId = 10045;
                }
                else {
                    this.c3.selectedIndex = 0;
                    this.c1.selectedIndex = 1;
                    isCat = false;
                }
                if (isCat) {
                    let parts = [];
                    for (let i = 0; i < vo.pkgAwardVos.length; i++) {
                        let award = vo.pkgAwardVos[i];
                        if (award.IsItem && award.itemRes.type == 4) {
                            let skins = skinResource.items;
                            for (let j = 0; j < skins.length; j++) {
                                if (award.itemRes.id == skins[j].itemId) {
                                    parts.push(skins[j].id);
                                    break;
                                }
                            }
                        }
                    }
                    this.LoadCat(catId, parts);
                }
            }
        }
        onUpdate() {
            let vo = this.Data;
            if (vo instanceof PackageVO) {
                if (!vo.isCanShow) {
                    this.close();
                    return;
                }
                if (vo.isCd) {
                    this.btn_sure.enabled = false;
                }
                else {
                    this.btn_sure.enabled = true;
                }
                let intervalTime = Laya.timer.currTimer - vo.pkgShowTime;
                let time = vo.time * 1000;
                this.date.setTime(time - intervalTime);
                let hours = this.date.getHours() - 8;
                if (hours < 0)
                    hours += 24;
                let min = this.date.getMinutes();
                let sec = this.date.getSeconds();
                let offestX = hours < 10 ? "0" : "";
                let offestY = min < 10 ? "0" : "";
                let offestZ = sec < 10 ? "0" : "";
                this.txt_time.text = `剩余时间 ${offestX}${hours}:${offestY}${min}:${offestZ}${sec}`;
            }
        }
        LoadCat(catId = 10033, parts = null) {
            let cat = catResource.getItemByKey(catId);
            let handler = Laya.Handler.create(this, () => {
                this.sp_catModel.AddCatPart(parts);
                this.sp_catModel.PlayAni("rub01_loop02", true, Laya.Handler.create(this, () => {
                }));
            });
            this.sp_catModel.LoadCat(cat.res, handler, this._view);
        }
        OnClickLinQuBtn() {
            let handler = Laya.Handler.create(this, (isSuccess) => {
                if (isSuccess) {
                    let vo = this.Data;
                    if (vo instanceof PackageVO) {
                        if (!this.CheckCanGet())
                            return;
                        let awards = vo.pkgAwardVos.slice();
                        vo.GetAward();
                        UISystem.Inst.CreateWindowView(UIAwardView, awards);
                        if (vo.curAdNum == vo.maxAdNum) {
                            this.close();
                        }
                        else {
                            this.UpdateData();
                        }
                    }
                }
            });
            PlatformMgr.Inst.showAd(AdType.ad_12, handler);
        }
        CheckCanGet() {
            let vo = this.Data;
            if (!(vo instanceof PackageVO))
                return;
            if (vo.buyNum >= vo.pkgRes.buyNum) {
                this.c3.selectedIndex = 2;
                return false;
            }
            if (vo.isCd) {
                I18nUtils.ShowI18nById("matou_ad_cd");
                return false;
            }
            return true;
        }
        Reset() {
            super.Reset();
            Laya.timer.clear(this, this.onUpdate);
        }
    }

    var exports$1 = {};(function(t,r){for(var n in r)t[n]=r[n];}(exports$1,function(t){var r={};function n(e){if(r[e])return r[e].exports;var o=r[e]={i:e,l:!1,exports:{}};return t[e].call(o.exports,o,o.exports,n),o.l=!0,o.exports}return n.m=t,n.c=r,n.d=function(t,r,e){n.o(t,r)||Object.defineProperty(t,r,{configurable:!1,enumerable:!0,get:e});},n.r=function(t){Object.defineProperty(t,"__esModule",{value:!0});},n.n=function(t){var r=t&&t.__esModule?function(){return t.default}:function(){return t};return n.d(r,"a",r),r},n.o=function(t,r){return Object.prototype.hasOwnProperty.call(t,r)},n.p="",n(n.s=1)}([function(t,r,n){"use strict";Object.defineProperty(r,"__esModule",{value:!0});var e=function(){function t(){}return t.lerr=function(){for(var t=[],r=0;r<arguments.length;r++)t[r]=arguments[r];t.unshift("====>"),console.error.apply(window,t);},t.llog=function(){for(var t=[],r=0;r<arguments.length;r++)t[r]=arguments[r];t.unshift("====>"),console.log.apply(window,t);},t.twoInt=function(t){return t<10?"0"+t:t},t.todayStr=function(){var r=new Date;return ""+r.getFullYear()+t.twoInt(r.getMonth()+1)+t.twoInt(r.getDate())},t.randomInt=function(t,r){return Math.floor(Math.random()*(r-t+1)+t)},t.saveData=function(t,r){var n=window.localStorage;n&&n.setItem("kcSdk_"+t,r);},t.loadData=function(t){var r=window.localStorage;if(r){var n=r.getItem("kcSdk_"+t);if(n)return n}return null},t.encodeUTF8=function(t){var r,n,e,o=[];for(r=0;r<t.length;r++)(n=t.charCodeAt(r))<128?o.push(n):n<2048?o.push(192+(n>>6&31),128+(63&n)):((e=55296^n)>>10==0?(n=(e<<10)+(56320^t.charCodeAt(++r))+65536,o.push(240+(n>>18&7),128+(n>>12&63))):o.push(224+(n>>12&15)),o.push(128+(n>>6&63),128+(63&n)));return o},t.s=function(r){var n,e,o,i=new Uint8Array(t.encodeUTF8(r)),a=16+(i.length+8>>>6<<4);for((r=new Uint8Array(a<<2)).set(new Uint8Array(i.buffer)),r=new Uint32Array(r.buffer),o=new DataView(r.buffer),n=0;n<a;n++)r[n]=o.getUint32(n<<2);r[i.length>>2]|=128<<24-8*(3&i.length),r[a-1]=i.length<<3;var u=[],s=[function(){return f[1]&f[2]|~f[1]&f[3]},function(){return f[1]^f[2]^f[3]},function(){return f[1]&f[2]|f[1]&f[3]|f[2]&f[3]},function(){return f[1]^f[2]^f[3]}],l=function(t,r){return t<<r|t>>>32-r},c=[1518500249,1859775393,-1894007588,-899497514],f=[1732584193,-271733879,null,null,-1009589776];for(f[2]=~f[0],f[3]=~f[1],n=0;n<r.length;n+=16){var h=f.slice(0);for(e=0;e<80;e++)u[e]=e<16?r[n+e]:l(u[e-3]^u[e-8]^u[e-14]^u[e-16],1),o=l(f[0],5)+s[e/20|0]()+f[4]+u[e]+c[e/20|0]|0,f[1]=l(f[1],30),f.pop(),f.unshift(o);for(e=0;e<5;e++)f[e]=f[e]+h[e]|0;}for(o=new DataView(new Uint32Array(f).buffer),n=0;n<5;n++)f[n]=o.getUint32(n<<2);return Array.prototype.map.call(new Uint8Array(new Uint32Array(f).buffer),function(t){return (t<16?"0":"")+t.toString(16)}).join("")},t.b=function(t){var r=[];for(var n in t)r.push(n);r=r.sort();for(var e="",o=0;o<r.length;o++){var i=t[r[o]];"object"==typeof i&&(i=this.b(i)),e+=r[o]+i;}return e},t.a=function(r,n,e){var o=[];for(var i in r)null!==r[i]&&void 0!==r[i]?o.push(i):delete r[i];o=o.sort();var a=t.b(r);return n&&(a+=n),a+=e||"pH0Xhyenu2e1zdUg",t.s(a)},t.jax=function(r,n,e){e||(e=function(r){if(r)return t.lerr(r)});var o=new XMLHttpRequest,i=(new Date).getTime();n.s=t.a(n,i),o.open("POST",r,!0),o.setRequestHeader("Content-Type","application/json; charset=utf-8"),o.setRequestHeader("ts",""+i);var a=JSON.stringify(n);o.send(a),o.onload=function(){if(200==o.status){var r={jaxERR:""+o.responseText};try{r=JSON.parse(o.responseText);}catch(r){return t.lerr(r),e(null,{re:"1111",err:"jsonErr:"+o.responseText})}return e(null,r)}return e(null,{re:"9"+o.status})};},t}();r.jax=e.jax,r.llog=e.llog,r.lerr=e.lerr,r.saveData=e.saveData,r.loadData=e.loadData,r.randomInt=e.randomInt,r.todayStr=e.todayStr;},function(t,r,n){"use strict";Object.defineProperty(r,"__esModule",{value:!0});var e=n(0),o=function(){function t(){this.version="1.2.0",this.navGap=3e3,this.lastNavT=0,this.htHost="https://dk.gametdd.com/htApi/ht_api/",this.picType="jpg";}return t.instance=function(){return t.me?t.me:(t.me=new t,t.me)},t.prototype.getUid=function(){return t.Uid?t.Uid:(t.Uid=e.loadData("uid"),t.Uid||"null")},t.prototype.setUid=function(r){r&&t.Uid!==r&&(t.Uid=r,e.saveData("uid",r));},t.prototype.jht=function(t,r,n){r.v=this.version,e.jax(this.htHost+t,r,function(t,r){return t||!r?(e.llog("jhtERR",t),n(t||new Error("no re"))):0!==r.re?(e.llog("jht FAILED!"+JSON.stringify(r)),n(new Error("err:"+r.re))):void n(null,r)});},t.prototype.init=function(r,n,o){var i=this;e.llog("HTS init v:"+this.version+","+r),t.Gid=r;var a=e.loadData("htHost");a&&(this.htHost=a);var u=e.loadData("picType");u&&(this.picType=u);var s={gid:r,uid:this.getUid(),query:n||"null"};e.llog("init start..."+this.picType+","+this.htHost),this.jht("init",s,function(r,n){if(r)o(r);else{e.llog("init done:"+n.re),i.setUid(n.uid),n.iconPre&&(t.IconPre=n.iconPre),n.picType&&(i.picType=n.picType,e.saveData("picType",n.picType)),n.navGap&&(i.navGap=n.navGap);for(var a=[],u=0;u<n.midArr.length;u++){var s=n.midArr[u],l="";n.vArr&&n.vArr.length===n.midArr.length&&0!==parseInt(n.vArr[u])&&(l="_"+n.vArr[u]),a.push({iconUrl:n.iconPre+s+l+"."+i.picType,mid:s});}o(null,a),n.htHost&&n.htHost!==i.htHost&&(i.htHost=n.htHost,e.saveData("htHost",n.htHost));}});},t.prototype.getBox=function(r,n){var o=this;if(!t.IconPre)return e.llog("htSdk 未初始化!!"+t.IconPre),n(new Error("htSdk 未初始化"));var i={bid:r};this.jht("getBox",i,function(e,i){if(e)return n(e);if(!i.arr||i.arr.length<=0)return n(new Error("服务配置有误bid:"+r));for(var a=[],u=0;u<i.arr.length;u++){for(var s=i.arr[u],l=[],c=0;c<s.length;c++){var f=s[c].split("@"),h="",p="";f.length>=4&&"0"!==f[3]&&(h="_"+f[3]),f.length>=5&&(p=f[4]),l.push({jumpKey:f[1],mid:f[0],iconUrl:t.IconPre+f[0]+h+"."+o.picType,txt:f[2],mark:p,query:"mid="+f[0]+"&bid="+r+"&uid="+t.Uid+"&po="+u});}a.push(l);}n(null,a);});},t.prototype.navTo=function(r,n,o,i){var a=this,u=(new Date).getTime();if(u-this.lastNavT<this.navGap)e.llog("navTo too");else{this.lastNavT=u;var s={gid:r||t.Gid,uid:this.getUid(),bid:n,mid:o};n?o?(i&&(s.fail="true"),e.llog("navTo",JSON.stringify(s)),this.jht("navTo",s,function(t,r){t||a.setUid(r.uid);})):e.llog("navTo 无mid"):e.llog("navTo 无bid");}},t.prototype.navFrom=function(r,n){var o=this;if(n){t.Gid=r;var i={gid:r,uid:this.getUid(),query:n||"null"};e.llog("navFrom",JSON.stringify(i)),this.jht("navFrom",i,function(t,r){t||o.setUid(r.uid);});}else e.llog("navFrom query is empty!"+n);},t.prototype.jlog=function(r,n,o){var i=this;if(t.Gid){var a={act:r,gid:t.Gid,uid:this.getUid(),data:n,user:o};e.llog("jlog",JSON.stringify(a)),this.jht("jlog",a,function(t,r){t||i.setUid(r.uid);});}else e.llog("jlog no gid",r,n);},t.prototype.getSharePic=function(t,r){this.jht("getSharePic",{game:t},r);},t.prototype.shareTo=function(t){this.jht("shareTo",{wxPicId:t},function(r){r?e.lerr(r):e.llog("shareTo OK,"+t);});},t.prototype.shareFrom=function(t){this.jht("shareFrom",{wxPicId:t},function(r){r?e.lerr(r):e.llog("shareFrom OK,"+t);});},t.prototype.jreq=function(t,r,n){e.jax(t,r,n);},t}();r.default=o.instance();}])));var htSdk = exports$1.default;

    class Hts {
        constructor() {
            this.htSdk = htSdk;
        }
        static instance() {
            if (Hts.me) {
                return Hts.me;
            }
            Hts.me = new Hts();
            return Hts.me;
        }
        getUid() {
            return this.htSdk.getUid();
        }
        init(gid, query, callback) {
            this.htSdk.init(gid, query, callback);
        }
        getBox(bid, callback) {
            this.htSdk.getBox(bid, callback);
        }
        navFrom(gid, query) {
            this.htSdk.navFrom(gid, query);
        }
        navTo(gid, bid, mid, isFail) {
            this.htSdk.navTo(gid, bid, mid, isFail);
        }
        jlog(act, data, user) {
            this.htSdk.jlog(act, data, user);
        }
        getSharePic(act, callback) {
            this.htSdk.getSharePic(act, callback);
        }
        shareTo(wxPicId) {
            this.htSdk.shareTo(wxPicId);
        }
        shareFrom(wxPicId) {
            this.htSdk.shareFrom(wxPicId);
        }
        jreq(url, data, callback) {
            this.htSdk.jreq(url, data, callback);
        }
    }
    const hts = Hts.instance();

    class GameHtMgr {
        constructor() {
            this.midMap = {};
            this.iconUrlmap = {};
            this.gid = "";
            this.isInited = false;
            this.bidListMap = {};
        }
        static get Inst() {
            return this._inst = this._inst || new GameHtMgr();
        }
        init() {
            let self = this;
            if (PlatformMgr.Inst.isTT || Laya.Browser.onPC || !GameHtMgr.isOpenHt) {
                self.isInited = true;
                return;
            }
            if (!Laya.Browser.onMiniGame) {
                return;
            }
            let query = self.getLauchFromQueryOppo();
            this.gid = GameHtMgr.wx_gid;
            if (Laya.Browser.onQGMiniGame) {
                this.gid = GameHtMgr.oppo_gid;
            }
            else if (Laya.Browser.onBDMiniGame) {
                this.gid = GameHtMgr.baidu_gid;
            }
            hts.init(this.gid, query, (err, mArr) => {
                if (err) {
                    return;
                }
                self.downloadPicsAndCache(mArr, (err, picsCount) => {
                    Logger.log(this, '---------互推初始化成功,完成图片缓存数:' + picsCount);
                    self.isInited = true;
                    GameDispatcher.Inst.event(UIGameEvent.GAME_HT_INIT);
                });
            });
            if (!query) {
                hts.navFrom(this.gid, self.getLauchFromQueryOppo());
            }
        }
        getLauchFromQueryOppo() {
            let query = null;
            const qg = Laya.Browser.onQGMiniGame ? window["qg"] : window["wx"];
            return query;
        }
        joinQuery(queryObj) {
            let out = '';
            for (const i in queryObj) {
                const el = queryObj[i];
                out += '&' + i + '=' + el;
            }
            if (out) {
                return '?' + out.substring(1);
            }
            return out;
        }
        downloadPicsAndCache(mArr, callback) {
            const pics = [];
            for (let i = 0; i < mArr.length; i++) {
                const one = mArr[i];
                if (this.midMap[one.mid]) {
                    continue;
                }
                this.midMap[one.mid] = { 'iconUrl': one.iconUrl };
                this.iconUrlmap[one.iconUrl] = one.mid;
                pics.push(one.iconUrl);
            }
            if (pics.length === 0) {
                callback(this, 0);
                return;
            }
            Laya.loader.load(pics, Laya.Handler.create(this, function loadPics(pics) {
                for (let i = 0; i < pics.length; i++) {
                    let onePic = pics[i];
                    const onePicFrame = Laya.loader.getRes(onePic);
                    this.midMap[this.iconUrlmap[onePic]].frame = onePicFrame;
                }
                callback(this, pics.length);
            }, [pics]));
        }
        createJumpInfoArr(boxArr, needDownPics, bid) {
            const jumpArr = [];
            for (let i = 0; i < boxArr.length; i++) {
                const oneArr = boxArr[i];
                const subArr = [];
                for (let j = 0; j < oneArr.length; j++) {
                    const oneBox = oneArr[j];
                    const jumpOne = {
                        'icon': null,
                        'jumpKey': oneBox.jumpKey,
                        'po': i,
                        'query': oneBox.query,
                        'txt': oneBox.txt,
                        'bid': bid,
                        'mid': oneBox.mid,
                        "mark": oneBox.mark
                    };
                    if (!this.midMap[oneBox.mid]) {
                        needDownPics.push({ 'iconUrl': oneBox.iconUrl, 'mid': oneBox.mid });
                    }
                    else {
                        jumpOne.icon = this.midMap[oneBox.mid].frame;
                    }
                    subArr.push(jumpOne);
                }
                jumpArr.push(subArr);
            }
            return jumpArr;
        }
        showHtView(bid) {
            this.showBox(bid);
        }
        showBox(bid) {
            let self = this;
            hts.getBox(bid, (err, boxArr) => {
                if (err) {
                    return;
                }
                const needDownPics = [];
                const jumpArr = self.createJumpInfoArr(boxArr, needDownPics, bid);
                let list = null;
                if (needDownPics.length <= 0) {
                    list = jumpArr.slice();
                    self.bidListMap[bid] = list;
                    GameDispatcher.Inst.event(UIGameEvent.GAME_HT, [list, bid]);
                }
                else {
                    self.downloadPicsAndCache(needDownPics, (err, picsCount) => {
                        list = self.createJumpInfoArr(boxArr, [], bid);
                        self.bidListMap[bid] = list;
                        GameDispatcher.Inst.event(UIGameEvent.GAME_HT, [list, bid]);
                    });
                }
            });
        }
        getList(bid) {
            return this.bidListMap[bid];
        }
        navigateToMiniGame(jumpInfo, cancel = null) {
            if (!jumpInfo) {
                return;
            }
            let o = ConfigValue.getItemByKey(jumpInfo.jumpKey);
            let query = o && o.value ? "&" + o.value : "";
            console.log("[GameHtMgr]=====>appId: " + jumpInfo.jumpKey + " path: " + query);
            let self = this;
            if (Laya.Browser.onQGMiniGame) {
                let data = {
                    pkgName: jumpInfo.jumpKey,
                    path: '?' + jumpInfo.query + query,
                    extraData: {
                        from: 'pageA'
                    },
                    success: function (res) {
                        Logger.log(self, "[GameHtMgr][navigateToMiniGame]----------------success: " + JSON.stringify(res));
                        hts.navTo(self.gid, jumpInfo.bid, jumpInfo.mid);
                    },
                    fail: function (res) {
                        Logger.log(this, "[GameHtMgr][navigateToMiniGame]----------------fail " + JSON.stringify(res));
                        if (cancel != null) {
                            cancel.run();
                        }
                    },
                    complete: function () {
                        Logger.log(self, "[GameHtMgr][navigateToMiniGame]----------------complete ");
                    }
                };
                if (qg["navigateToMiniGame"] != null) {
                    qg["navigateToMiniGame"](data);
                }
                else if (qg["navigateToMiniProgram"] != null) {
                    qg["navigateToMiniProgram"](data);
                }
            }
            else if (PlatformMgr.Inst.isWX) {
                let data = {
                    appId: jumpInfo.jumpKey,
                    path: '?' + jumpInfo.query + query,
                    envVersion: 'release',
                    extraData: {
                        from: 'pageA'
                    },
                    success: function (res) {
                        Logger.log(self, "[GameHtMgr][navigateToMiniGame]----------------success: " + JSON.stringify(res));
                        hts.navTo(self.gid, jumpInfo.bid, jumpInfo.mid, false);
                    },
                    fail: function (res) {
                        Logger.log(self, "[GameHtMgr][navigateToMiniGame]----------------fail " + JSON.stringify(res));
                        hts.navTo(self.gid, jumpInfo.bid, jumpInfo.mid, true);
                        if (cancel != null) {
                            cancel.run();
                        }
                    },
                    complete: function () {
                        Logger.log(self, "[GameHtMgr][navigateToMiniGame]----------------complete ");
                    }
                };
                wx.navigateToMiniProgram(data);
            }
            else if (Laya.Browser.onBDMiniGame) {
                let data = {
                    appKey: jumpInfo.jumpKey,
                    path: '?' + jumpInfo.query + query,
                    extraData: {},
                    success: function (res) {
                        Logger.log(self, "[GameHtMgr][navigateToMiniGame]----------------success: " + JSON.stringify(res));
                        hts.navTo(self.gid, jumpInfo.bid, jumpInfo.mid, false);
                    },
                    fail: function (error) {
                        Logger.log(self, "[GameHtMgr][navigateToMiniGame]----------------fail " + JSON.stringify(error));
                        hts.navTo(self.gid, jumpInfo.bid, jumpInfo.mid, true);
                        if (cancel != null) {
                            cancel.run();
                        }
                    }
                };
                let swan = window["swan"];
                if (swan) {
                    swan.navigateToMiniProgram(data);
                }
            }
        }
        showFriendHTView() {
        }
        showTTMoreBtn(createBtn = true) {
            const systemInfo = window["tt"].getSystemInfoSync();
            if (systemInfo.platform === "ios") {
                return;
            }
            let appLaunchOptions = JSON.parse(ConfigValue.getItemByKey("zj_ht").value);
            if (window["tt"].createMoreGamesButton && createBtn) {
                const btn = window["tt"].createMoreGamesButton({
                    type: "image",
                    image: "res/images/moregame.png",
                    style: {
                        left: 50,
                        top: 60,
                        width: 49,
                        height: 56,
                        borderWidth: 0,
                        borderColor: "#ff0000"
                    },
                    appLaunchOptions: appLaunchOptions,
                    onNavigateToMiniGame(res) {
                        console.log("跳转其他小游戏", res);
                        if (res && res.errCode == 0) {
                        }
                    }
                });
                btn.onTap(() => {
                    console.log("点击更多游戏");
                    showAlert();
                });
            }
            else if (!createBtn) {
                console.log("点击更多游戏");
                showAlert();
            }
            function showAlert() {
                const systemInfo = window["tt"].getSystemInfoSync();
                if (systemInfo.platform !== "ios") {
                    window["tt"].showMoreGamesModal({
                        appLaunchOptions: appLaunchOptions,
                        success(res) {
                            console.log("success", res.errMsg);
                        },
                        fail(res) {
                            console.log("fail", res.errMsg);
                        }
                    });
                }
            }
        }
        showAppBox() {
            let self = this;
            const systemInfo = window["tt"].getSystemInfoSync();
            let screenWidth = systemInfo.screenWidth;
            let screenHeight = systemInfo.screenHeight;
            let appLaunchOptions = JSON.parse(ConfigValue.getItemByKey("zj_ht").value);
            this.gamesBanner = window["tt"].createMoreGamesBanner({
                style: {
                    width: 150,
                    height: 40,
                    left: (screenWidth - 300) * 0.5,
                    top: (screenHeight - 80)
                },
                appLaunchOptions: appLaunchOptions,
                success(res) {
                    console.log("success", res.errMsg);
                },
                fail(res) {
                    console.log("fail", res.errMsg);
                }
            });
            this.gamesBanner.show();
            this.gamesBanner.onTap(() => {
                console.log("点击跳转游戏盒子");
                self.showTTMoreBtn(false);
            });
        }
        hideAppBox() {
            if (this.gamesBanner) {
                this.gamesBanner.offTap();
                this.gamesBanner.hide();
            }
        }
    }
    GameHtMgr._inst = null;
    GameHtMgr.wx_gid = '60e2e3c0c582ee361a30d1ef';
    GameHtMgr.oppo_gid = '600fb1e82e037758e9f50ed1';
    GameHtMgr.baidu_gid = '5e1c74e0f2454e4f90ff2b26';
    GameHtMgr.baidu_bid = "5e1c759ff2454e4f90ff2b27";
    GameHtMgr.wx_icon_bid = "60e2e3d3c582ee361a30d1f1";
    GameHtMgr.wx_more_bid = "60e2e3e1c582ee361a30d1f3";
    GameHtMgr.wx_box_bid = "5f3643166083fb4b5581b3c2";
    GameHtMgr.isOpenHt = true;
    GameHtMgr.OPPO_BID1 = "600fb2002e037758e9f50ed3";
    GameHtMgr.OPPO_BID2 = "600fb2202e037758e9f50ed5";

    class UIHTItem extends TListButtonItem {
        constructor() {
            super();
            this.icon_hot = null;
            this.currentIndex = 0;
            this.cancel = null;
        }
        InitUI() {
            super.InitUI();
        }
        InitData(data) {
            super.InitData(data);
            this.currentIndex = 0;
            if (data instanceof Array) {
                this.showData(data[0]);
            }
            else if (data) {
                this.showData(data);
            }
        }
        showData(data) {
            if (data == null) {
                return;
            }
            if (data.iconUrl) {
                this.icon = data.iconUrl;
            }
            else if (data.icon) {
                this._iconObject["__getResCompleted"](data.icon);
                this.title = data.txt;
            }
            if (this.icon_hot != null) {
                this.icon_hot.visible = true;
                if (data.mark == "NEW") {
                    this.icon_hot.icon = PanelRegister.getUrl("common", "hot");
                }
                else if (data.mark == "热门" || data.mark == "hot" || data.mark == "HOT") {
                    this.icon_hot.icon = PanelRegister.getUrl("common", "hot");
                }
                else {
                    this.icon_hot.icon = "";
                }
            }
        }
        shake() {
            let self = this;
            fgui.GTween.kill(this, true);
            fgui.GTween.shake(self.x, self.y, 5, 0.6).setTarget(this)
                .onUpdate(function t2(tw) {
                self.x = tw.value.x;
                self.y = tw.value.y;
            }, this);
        }
        stopShake() {
            fgui.GTween.kill(this, true);
        }
        getGameName() {
            let info = this.Data[this.currentIndex];
            return info ? info.txt : "";
        }
        clickThis(cancel = null) {
            let info = this.Data instanceof Array ? this.Data[this.currentIndex] : this.Data;
            if (info) {
                GameHtMgr.Inst.navigateToMiniGame(info, cancel);
            }
        }
        getCurData() {
            let info = this.Data instanceof Array ? this.Data[this.currentIndex] : this.Data;
            return info;
        }
        Reset() {
            super.Reset();
            if (this.cancel != null) {
                this.cancel.recover();
            }
            this.cancel = null;
            if (this.img_red) {
                this.img_red.visible = false;
            }
        }
    }

    class UIHTItem2 extends UIHTItem {
        constructor() {
            super();
            this.ht_list = null;
            this.ht_index = 0;
        }
        InitData(data) {
            this.visible = false;
            if (!GameHtMgr.Inst.isInited) {
                Logger.log(this, "=====>互推初始化未完成!!!");
                GameDispatcher.Inst.on(UIGameEvent.GAME_HT_INIT, this, this.gameHtInit);
            }
            else {
                this.gameHtInit();
            }
        }
        gameHtInit() {
            this.on(Laya.Event.CLICK, this, this.clickSelf);
            GameDispatcher.Inst.on(UIGameEvent.GAME_HT, this, this.gameHt);
            this.gameInit();
        }
        gameInit() {
            if (!GameHtMgr.isOpenHt) {
                return;
            }
            if (PlatformMgr.Inst.isWX) {
                GameHtMgr.Inst.showHtView(GameHtMgr.wx_icon_bid);
            }
            else if (Laya.Browser.onQGMiniGame) {
                GameHtMgr.Inst.showHtView(GameHtMgr.OPPO_BID1);
            }
            else if (Laya.Browser.onBDMiniGame) {
                GameHtMgr.Inst.showHtView(GameHtMgr.baidu_bid);
            }
            else if (PlatformMgr.Inst.isTT) {
            }
        }
        clickSelf() {
            this.clickThis(this.cancel);
        }
        gameHt(list, bid) {
            if (PlatformMgr.Inst.isWX && bid != GameHtMgr.wx_icon_bid) {
                return;
            }
            if (Laya.Browser.onQGMiniGame && bid != GameHtMgr.OPPO_BID1) {
                return;
            }
            if (this.ht_list != null) {
                return;
            }
            if (list[0] instanceof Array) {
                this.ht_list = list[0];
            }
            else {
                this.ht_list = list;
            }
            this.ht_index = 0;
            this.changeHtItem();
            Laya.timer.clear(this, this.changeHtItem);
            if (bid == GameHtMgr.wx_icon_bid) {
                Laya.timer.loop(3000, this, this.changeHtItem);
            }
            else if (bid == GameHtMgr.OPPO_BID1) {
                super.InitData(this.ht_list[this.ht_index]);
                Laya.timer.loop(3000, this, this.changeHtItem);
            }
            else if (bid == GameHtMgr.baidu_bid) {
            }
        }
        changeHtItem() {
            this.visible = true;
            let self = this;
            if (this.ht_list == null) {
                return;
            }
            this.ht_index++;
            if (this.ht_index >= this.ht_list.length) {
                this.ht_index = 0;
            }
            let data = this.ht_list[this.ht_index];
            if (data instanceof Array) {
                super.InitData(data[0]);
            }
            else {
                super.InitData(data);
            }
            this.shake();
        }
        Reset() {
            super.Reset();
            GameDispatcher.Inst.off(UIGameEvent.GAME_HT_INIT, this, this.gameHtInit);
            GameDispatcher.Inst.off(UIGameEvent.GAME_HT, this, this.gameHt);
            this.off(Laya.Event.CLICK, this, this.clickSelf);
            Laya.timer.clear(this, this.changeHtItem);
            this.ht_list = null;
        }
    }

    class UIHTMoreView extends UIBaseWindow {
        constructor() {
            super("ht", "UIHTMoreView");
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_AD;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isEffect = false;
            this.viewStruct.isExclusion = false;
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("ht", "UIHTItem", UIHTItem);
        }
        InitUI() {
            super.InitUI();
            this.eglist_ht.setVirtualAndLoop();
            this.eglist_ht.isAutoScroll = true;
            this.eglist_ht.autoHScrollSpeed = 2;
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.clickHtItem, this, this.eglist_ht);
            this.AddGameListener(UIGameEvent.GAME_HT, this.gameHt, this);
        }
        RemoveListener() {
            super.RemoveListener();
        }
        InitData(data) {
            super.InitData(data);
            if (PlatformMgr.Inst.isWX) {
                GameHtMgr.Inst.showHtView(GameHtMgr.wx_more_bid);
            }
            else if (Laya.Browser.onQGMiniGame) {
                GameHtMgr.Inst.showHtView(GameHtMgr.OPPO_BID2);
            }
            this.showBanner();
        }
        clickHtItem(item) {
            if (item instanceof UIHTItem) {
                item.clickThis(Laya.Handler.create(this, this.htCancel));
            }
        }
        htCancel() {
        }
        gameHt(list, bid) {
            if (bid == GameHtMgr.wx_more_bid) {
                this.eglist_ht.Data = list;
            }
            else if (bid == GameHtMgr.OPPO_BID2) {
                let arr = [];
                let d;
                let d2;
                for (d of list) {
                    if (d instanceof Array) {
                        for (d2 of d) {
                            arr.push(d2);
                        }
                    }
                    else {
                        arr.push(d);
                    }
                }
                list = ArrayUtil.shuffle(arr);
                this.eglist_ht.Data = list;
            }
        }
        Reset() {
            super.Reset();
        }
    }

    class BuildListItem extends TListButtonItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1_sp_sheshi = this.sp_sheshi.getController("c1");
            this.buildType = this.getController("buildType");
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
        }
        InitData(data) {
            super.InitData(data);
            this.UpdateData();
        }
        UpdateData() {
            let buildVo = this.Data;
            if (buildVo && buildVo.IsUnLock()) {
                this.c2.selectedIndex = 0;
                this.txt_buildName.text = buildVo.buildRes.name;
                this.txt_buildName1.text = buildVo.buildRes.name;
                let url = UrlUtils.GetIconBuildUrl(buildVo.buildRes.icon);
                PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                    this.sp_build.url = url;
                }));
                this.UpdateItem();
            }
            else {
                this.txt_buildName2.text = buildVo.buildRes.name;
                this.c1.selectedIndex = 2;
                this.c2.selectedIndex = 1;
                this.txt_Tips.text = buildVo.buildRes.desc;
            }
        }
        UpdateItem() {
            let vo = this.Data;
            if (!vo)
                return;
            let type = vo.buildRes.type;
            if (type == BuildVO.TYPE_MAOWO) {
                this.buildType.selectedIndex = 1;
                this.c1.selectedIndex = 0;
                let curBaseRes = baseResource.getItemByKey(vo.level);
                let catCount = CatModel.Inst.GetCatCount();
                this.txt_houseNum.text = curBaseRes.catMax + "";
                this.txt_catNum.text = catCount + "";
                this.txt_houseNum1.text = (curBaseRes.catMax - catCount) + "";
            }
            else if (type == BuildVO.TYPE_MATOU) {
                this.buildType.selectedIndex = 0;
                this.c1.selectedIndex = 0;
                this.txt_matouNum.text = LandModel.Inst.boatNum.toString();
            }
            else if (type == BuildVO.TYPE_BANJIASHESHI) {
                this.buildType.selectedIndex = 3;
                this.c1.selectedIndex = 1;
                this.txt_level.text = vo.level + "级";
                this.txt_desc.text = vo.buildRes.desc;
            }
            else {
                this.c1.selectedIndex = 1;
                this.buildType.selectedIndex = 2;
                this.UpdateSheShiItem();
            }
        }
        UpdateSheShiItem() {
            let vo = this.Data;
            if (!vo)
                return;
            let type = vo.buildRes.type;
            if (type == BuildVO.TYPE_XIUXIANSHESHI) {
                this.c1_sp_sheshi.selectedIndex = 0;
            }
            else if (type == BuildVO.TYPE_HUANGJINGSHESHI) {
                this.c1_sp_sheshi.selectedIndex = 1;
            }
            else if (type == BuildVO.TYPE_YULESHESHI) {
                this.c1_sp_sheshi.selectedIndex = 2;
            }
            else if (type == BuildVO.TYPE_CANYINSHESHI) {
                this.c1_sp_sheshi.selectedIndex = 3;
            }
            this.txt_level.text = vo.level + "级";
            this.txt_glodNum.text = MathUtil.goldFormat(vo.GetAllAddGold()) + "/" + vo.GetTime() + "秒";
            this.txt_natureNum.text = vo.GetCurLvScore().toString();
            this.txt_zuoweiNum.text = vo.zwCount.toString();
        }
        Reset() {
            super.Reset();
        }
    }

    class FacScoreItem extends fgui.GComponent {
        constructor() {
            super();
        }
        constructFromXML(xml) {
            super.constructFromXML(xml);
            FairyUtils.setVar(this, this);
            this.c1 = this.sp_sheshi.getController("c1");
        }
        UpdateData(type = 0) {
            let score = 0;
            this.c1.selectedIndex = type;
            if (type == 0) {
                this.txt_scoreName.text = "休闲设施分数";
                score = BuildModel.Inst.GetBuildScore(3);
            }
            else if (type == 1) {
                this.txt_scoreName.text = "环境设施分数";
                score = BuildModel.Inst.GetBuildScore(4);
            }
            else if (type == 2) {
                this.txt_scoreName.text = "娱乐设施分数";
                score = BuildModel.Inst.GetBuildScore(5);
            }
            else if (type == 3) {
                this.txt_scoreName.text = "餐饮设施分数";
                score = BuildModel.Inst.GetBuildScore(6);
            }
            this.txt_score.text = score.toString();
        }
    }

    class UIIslandInfoItem extends fgui.GComponent {
        constructor() {
            super();
            this.sp_xxscore = null;
            this.sp_hjscore = null;
            this.sp_ylscore = null;
            this.sp_cyscore = null;
            this.txt_gold = null;
            this.txt_catReward = null;
            this.txt_islandName = null;
            this.sp_island = null;
        }
        constructFromXML(xml) {
            super.constructFromXML(xml);
            FairyUtils.setVar(this, this);
            this.UpdateData();
        }
        UpdateData() {
            this.sp_xxscore.UpdateData(0);
            this.sp_hjscore.UpdateData(1);
            this.sp_ylscore.UpdateData(2);
            this.sp_cyscore.UpdateData(3);
            let goldNum = BuildModel.Inst.GetAllBuildGoldRate();
            this.txt_gold.text = "+" + MathUtil.goldFormat(goldNum);
            this.txt_catReward.text = Math.floor(CatModel.Inst.GetAllRate() * 0.01) + "%";
            this.txt_islandName.text = LandModel.Inst.name;
            let url = UrlUtils.GetIconBuildUrl(LandModel.Inst.icon);
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                this.sp_island.url = url;
            }));
        }
    }

    class UIMoveHouseTaskItem extends TListItem {
        constructor() {
            super();
            this.c1 = null;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
        }
        InitData(data) {
            super.InitData(data);
            if (data instanceof ConditionVO) {
                this.c1.selectedIndex = 1;
                this.cb.selected = data.IsFull;
                this.txt_task.text = data.Desc;
            }
            else if (typeof data === "string") {
                this.c1.selectedIndex = 0;
                this.txt_task.text = "✔ " + data;
            }
        }
        Reset() {
            super.Reset();
        }
    }

    class UIMoveHouseView extends UIBaseWindow {
        constructor() {
            super("IslandInfo", "UIMoveHouseView");
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_CENTER;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isEffect = true;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            this.UpdateData();
            this.showBanner();
        }
        UpdateData() {
            let landUrl = LandModel.Inst.nextIcon || LandModel.Inst.icon;
            let name = LandModel.Inst.nextName || LandModel.Inst.name;
            let url = UrlUtils.GetIconBuildUrl(landUrl);
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                this.sp_land.url = url;
            }));
            this.txt_landName.text = name;
            let res = LandModel.Inst.NextLandRes;
            if (res) {
                let count = 0;
                let vos = BuildModel.Inst.GetBuildVOs();
                for (let i = 0; i < vos.length; i++) {
                    if (vos[i].buildRes.Belong == res.id)
                        count++;
                }
                this.eglist_items.Data = ["可安装" + count + "个新设施", "可邀请" + res.houseMaxLvl + "只猫星人"];
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_sure) {
                this.OnBtnSureClick();
            }
        }
        OnBtnSureClick() {
            this.close();
            UISystem.Inst.RemoveWindowClass(UIIslandInfoView);
            LandCtrl.Inst.moveHouse();
        }
        Reset() {
            super.Reset();
        }
    }

    class UIIslandInfoView extends UIBaseWindow {
        constructor() {
            super("IslandInfo", "UIIslandInfoView");
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("IslandInfo", "FacScoreItem", FacScoreItem);
            PanelRegister.registerClass("IslandInfo", "UIIslandInfoItem", UIIslandInfoItem);
            PanelRegister.registerClass("IslandInfo", "buildListItem", BuildListItem);
            PanelRegister.registerClass("IslandInfo", "UIMoveHouseTaskItem", UIMoveHouseTaskItem);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_CENTER;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.ClickItem, this, this.eglist_builds);
        }
        InitData(data) {
            super.InitData(data);
            this.eglist_builds.Data = BuildModel.Inst.GetCurBuildVos();
            this.UpdateMoveHouseInfo();
            this.landInfoItem.UpdateData();
            if ((Laya.Browser.onVVMiniGame || Laya.Browser.onKGMiniGame) && GuideModel.Inst.guideVO == null) {
                this.hideBanner();
                AdCtrl.Inst.DelayShowInsertAd();
            }
            else {
                this.showBanner();
            }
        }
        UpdateMoveHouseInfo() {
            if (LandModel.Inst.IsMax()) {
                console.log("岛屿全部已解锁");
                return;
            }
            let icon = LandModel.Inst.nextIcon || LandModel.Inst.icon;
            let nextIconUrl = UrlUtils.GetIconBuildUrl(icon);
            if (LandModel.Inst.CanUnlockNextLand()) {
                this.c1.selectedIndex = 1;
                PanelRegister.loadUrl(nextIconUrl, Laya.Handler.create(this, (url) => {
                    this.sp_nextLand1.url = url;
                }));
                this.txt_islandName.text = LandModel.Inst.nextName || LandModel.Inst.name;
                let res = LandModel.Inst.NextLandRes;
                if (res) {
                    let count = 0;
                    let vos = BuildModel.Inst.GetBuildVOs();
                    for (let i = 0; i < vos.length; i++) {
                        if (vos[i].buildRes.Belong == res.id)
                            count++;
                    }
                    this.eglist_items.Data = ["可安装" + count + "个新设施", "可邀请" + res.houseMaxLvl + "只猫星人"];
                }
            }
            else {
                this.c1.selectedIndex = 0;
                this.sp_nextLand0.url = nextIconUrl;
                if (LandModel.Inst.level == 1) {
                    this.txt_lockLevelTips.text = "鲸鱼打包搬家设施等级超过Lv." + LandModel.Inst.banjiaLevel + "+进行搬家任务";
                }
                else {
                    this.txt_lockLevelTips.text = "鲸鱼大型打包搬家设施等级超过Lv." + LandModel.Inst.banjiaLevel + "+进行搬家任务";
                }
            }
        }
        RemoveListener() {
            super.RemoveListener();
            this.eglist_builds.offAll();
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_sure) {
                this.close();
            }
            else if (btn == this.btn_moveHouse) {
                this.OnBtnMoveHouseClick();
            }
        }
        OnBtnMoveHouseClick() {
            UISystem.Inst.CreateWindowView(UIMoveHouseView);
        }
        ClickItem(item) {
            let vo = item.Data;
            if (vo instanceof BuildVO) {
                if (vo.IsUnLock()) {
                    UISystem.Inst.CreateWindowView(UIBuildView, vo);
                }
                else {
                    SceneMgr.Inst.GetCameraBuildScript().MoveBuild(vo);
                    if (vo.PreIsUnLock()) {
                        UISystem.Inst.CreateWindowView(UIBuildCreateView, [vo, null]);
                    }
                    this.close();
                }
            }
        }
        Reset() {
            super.Reset();
        }
    }

    class LiHeItem extends TComponent {
        constructor() {
            super();
        }
        InitFairyGui() {
            super.InitFairyGui();
            FairyUtils.setVar(this, this);
        }
        InitUI() {
            super.InitUI();
        }
        InitData(data) {
            super.InitData(data);
        }
        AddListener() {
            super.AddListener();
        }
        SetLiBaoName(name) {
            this.txt_name.text = name;
        }
        SetAward(award) {
            this.award = award;
            PanelRegister.loadUrl(award.itemUrl, Laya.Handler.create(this, (url) => {
                this.loader_award.url = url;
            }));
            this.txt_num.text = MathUtil.goldFormat(award.count, 0);
        }
        Reset() {
            super.Reset();
            this.award = null;
        }
    }

    class UILiHeView extends UIBaseWindow {
        constructor() {
            super("lihe", "UILiHeView");
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("lihe", "LiHeItem", LiHeItem);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = LayerType.LAYOUT_CENTER;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isShowMaskBg = false;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
            this.c1.selectedIndex = 1;
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            if (!LiHeModel.Inst.isAddGame) {
                LiHeModel.Inst.isAddGame = true;
                LiHeModel.Inst.reEnterGame = false;
            }
            else if (LiHeModel.Inst.reEnterGame) {
                if (LiHeModel.Inst.state1 == LiHeModel.normal) {
                    LiHeModel.Inst.state1 = LiHeModel.canAward;
                }
            }
            this.UpdateData();
            this.showBanner();
        }
        UpdateData() {
            this.lihe1.SetAward(LiHeModel.Inst.award1);
            this.lihe1.SetLiBaoName(LiHeModel.libaoName1);
            this.lihe2.SetAward(LiHeModel.Inst.award2);
            this.lihe2.SetLiBaoName(LiHeModel.libaoName2);
            this.lihe3.SetAward(LiHeModel.Inst.award3);
            this.lihe3.SetLiBaoName(LiHeModel.libaoName3);
            let btnGet1c1 = this.btn_get1.getController("c1");
            if (LiHeModel.Inst.state1 == LiHeModel.canAward) {
                btnGet1c1.selectedIndex = 1;
            }
            else if (LiHeModel.Inst.state1 == LiHeModel.comp) {
                btnGet1c1.selectedIndex = 2;
            }
            else {
                btnGet1c1.selectedIndex = 0;
            }
            if (PlatformMgr.Inst.hasSubscribeApp && LiHeModel.Inst.state2 == LiHeModel.normal) {
                LiHeModel.Inst.state2 = LiHeModel.canAward;
            }
            let btnGet2c1 = this.btn_get2.getController("c1");
            if (LiHeModel.Inst.state2 == LiHeModel.canAward) {
                btnGet2c1.selectedIndex = 1;
            }
            else if (LiHeModel.Inst.state2 == LiHeModel.comp) {
                btnGet2c1.selectedIndex = 2;
            }
            else {
                btnGet2c1.selectedIndex = 0;
            }
            if (LiHeModel.Inst.state3 == LiHeModel.comp) {
                this.btn_get3.getController("c1").selectedIndex = 2;
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_get1) {
                this.OnClickAddDesktopBtn();
            }
            else if (btn == this.btn_get2) {
                this.OnClickDingYueBtn();
            }
            else if (btn == this.btn_get3) {
                this.OnClickTuiSongBtn();
            }
        }
        OnClickAddDesktopBtn() {
            if (LiHeModel.Inst.state1 == LiHeModel.canAward) {
                LiHeModel.Inst.GetAward(LiHeModel.Inst.award1);
                UISystem.Inst.CreateWindowView(UIAwardView, LiHeModel.Inst.award1);
                LiHeModel.Inst.state1 = LiHeModel.comp;
                GameDispatcher.Inst.event(UIGameEvent.ON_LIHE_GETAWARD);
                this.UpdateData();
            }
            else if (this.btn_get1.getController("c1").selectedIndex == 0) {
                this.c1.selectedIndex = 0;
            }
        }
        OnClickDingYueBtn() {
            if (LiHeModel.Inst.state2 == LiHeModel.canAward) {
                LiHeModel.Inst.GetAward(LiHeModel.Inst.award2);
                UISystem.Inst.CreateWindowView(UIAwardView, LiHeModel.Inst.award2);
                LiHeModel.Inst.state2 = LiHeModel.comp;
                GameDispatcher.Inst.event(UIGameEvent.ON_LIHE_GETAWARD);
                this.UpdateData();
            }
            else if (LiHeModel.Inst.state2 == LiHeModel.normal) {
                if (!PlatformMgr.Inst.hasSubscribeApp) {
                    PlatformMgr.Inst.subscribeAppMsg(Laya.Handler.create(this, (isSuccess = true) => {
                        if (isSuccess) {
                            StorageUtils.Inst.setForeverRecord(1, StorageUtils.qq_subscribe, true);
                            let count = StorageUtils.Inst.getForeverRecord(StorageUtils.qq_subscribe);
                            if (count == 1) {
                                LiHeModel.Inst.state2 = LiHeModel.canAward;
                                this.OnClickDingYueBtn();
                            }
                        }
                    }));
                }
                else {
                    LiHeModel.Inst.state2 = LiHeModel.canAward;
                    this.UpdateData();
                }
            }
            else if (this.btn_get2.getController("c1").selectedIndex == 0) {
                this.c1.selectedIndex = 2;
            }
        }
        OnClickTuiSongBtn() {
            if (LiHeModel.Inst.state3 == LiHeModel.comp)
                return;
            let handler = Laya.Handler.create(this, (isSuccess = true) => {
                if (isSuccess) {
                    LiHeModel.Inst.GetAward(LiHeModel.Inst.award3);
                    UISystem.Inst.CreateWindowView(UIAwardView, LiHeModel.Inst.award3);
                    LiHeModel.Inst.state3 = LiHeModel.comp;
                    GameDispatcher.Inst.event(UIGameEvent.ON_LIHE_GETAWARD);
                    this.UpdateData();
                }
            });
            PlatformMgr.Inst.onShareAppMessage(handler);
        }
        onBtnCloseClick() {
            if (this.c1.selectedIndex != 1) {
                this.c1.selectedIndex = 1;
            }
            else {
                this.close();
            }
        }
        onResize() {
            super.onResize();
            this._view.setSize(Laya.stage.width, Laya.stage.height);
        }
        Reset() {
            super.Reset();
        }
    }

    class UISettingView extends UIBaseWindow {
        constructor() {
            super("setting", "UISettingView");
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_CENTER;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            this.UpdateData();
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                PlatformMgr.Inst.showScreenAd();
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_music) {
                this.OnClickMusic();
            }
            else if (btn == this.btn_sound) {
                this.OnClickSound();
            }
        }
        UpdateData() {
            this.btn_music.getController("c1").selectedIndex = StorageUtils.Inst.GetMusic();
            this.btn_sound.getController("c1").selectedIndex = StorageUtils.Inst.GetSound();
        }
        OnClickMusic() {
            StorageUtils.Inst.SetMusic(1 - StorageUtils.Inst.GetMusic());
            this.UpdateData();
        }
        OnClickSound() {
            StorageUtils.Inst.SetSound(1 - StorageUtils.Inst.GetSound());
            this.UpdateData();
        }
        Reset() {
            super.Reset();
            PlatformMgr.Inst.hideInsert();
        }
    }

    class UIBuyGoldAlert extends UIBaseWindow {
        constructor() {
            super("shop", "UIBuyGoldAlert");
            this.btn_sure = null;
            this.txt_gold = null;
            this.c1 = null;
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = LayerType.LAYOUT_CENTER;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isEffect = true;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            this.UpdateData();
        }
        UpdateData() {
            let vo = this.Data;
            if (!vo)
                return;
            this.c1.selectedIndex = 0;
            if (vo.isGoldTimeShouyi) {
                this.txt_gold.text = MathUtil.goldFormat(BuildModel.Inst.GetAllBuildGoldRate(vo.shopRes.num * 60 * 60, false));
            }
            else if (vo.isMaobohe) {
                this.txt_gold.text = vo.shopRes.num + "";
                this.c1.selectedIndex = 1;
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_sure) {
                this.OnClickSureBtn();
            }
        }
        OnClickSureBtn() {
            let vo = this.Data;
            if (!vo)
                return;
            if (!vo.canLookAd) {
                console.log("已达到当日购买上限");
                return;
            }
            let isSuccess = MoneyModel.diamond.cost(vo.price, true, false);
            if (isSuccess) {
                vo.GetAward();
                let num = vo.shopRes.num;
                let goldNum = 0;
                let id = vo.shopRes.itemId;
                if (vo.isGoldTimeShouyi) {
                    goldNum = BuildModel.Inst.GetAllBuildGoldRate(num * 60 * 60, false).toNumber();
                    id = MoneyVo.GOLD;
                }
                else if (vo.isMaobohe) {
                    goldNum = num;
                }
                UITaskView.itemVo.itemId = id;
                UITaskView.itemVo.count = goldNum;
                UISystem.Inst.CreateWindowView(UIAwardView, UITaskView.itemVo);
                let view = UISystem.Inst.GetWindowView(UIBuyGoldAlert);
                UISystem.Inst.RemoveWindowView(view);
            }
        }
        Reset() {
            super.Reset();
        }
    }

    class UIBuyGoldTips extends UIBaseWindow {
        constructor() {
            super("shop", "UIBuyGoldTips");
            this.btn_sure = null;
            this.txt_gold = null;
            this.c1 = null;
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = LayerType.LAYOUT_CENTER;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isEffect = true;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            this.UpdateData();
        }
        UpdateData() {
            let vo = this.Data;
            if (!vo)
                return;
            this.c1.selectedIndex = 0;
            if (vo.isGoldTimeShouyi) {
                this.txt_gold.text = MathUtil.goldFormat(BuildModel.Inst.GetAllBuildGoldRate(vo.shopRes.num * 60 * 60, false));
                this.txt_tips.text = "是否购买(" + vo.shopRes.num + "h沙漏)的金币收益\n你将获得";
            }
            else if (vo.isMaobohe) {
                this.txt_gold.text = vo.shopRes.num + "";
                this.c1.selectedIndex = 1;
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_sure) {
                this.OnClickSureBtn();
            }
        }
        OnClickSureBtn() {
            let vo = this.Data;
            if (!vo)
                return;
            if (!vo.canLookAd) {
                return;
            }
            let isSuccess = MoneyModel.diamond.cost(vo.price, true, false);
            if (isSuccess) {
                vo.GetAward();
                let num = vo.shopRes.num;
                let goldNum = 0;
                let id = vo.shopRes.itemId;
                if (vo.isGoldTimeShouyi) {
                    goldNum = BuildModel.Inst.GetAllBuildGoldRate(num * 60 * 60, false).toNumber();
                    id = MoneyVo.GOLD;
                }
                else if (vo.isMaobohe) {
                    goldNum = num;
                }
                UITaskView.itemVo.itemId = id;
                UITaskView.itemVo.count = goldNum;
                UISystem.Inst.CreateWindowView(UIAwardView, UITaskView.itemVo);
                let view = UISystem.Inst.GetWindowView(UIBuyGoldTips);
                UISystem.Inst.RemoveWindowView(view);
            }
        }
        Reset() {
            super.Reset();
        }
    }

    class BuyItem extends TBaseButton {
        constructor() {
            super();
            TBaseButton;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
        }
        InitData(data) {
            super.InitData(data);
        }
        UpdateData(vo) {
            this.vo = vo;
            if (!vo)
                return;
            if (!vo.canLookAd) {
                this.txt_ggtime.text = "已达当日购买上限!";
                this.txt_ggtime.fontSize = 12;
            }
            else {
                this.txt_ggtime.fontSize = 20;
                if (vo.isMaobohe) {
                    this.txt_ggtime.text = vo.shopRes.num + "个猫薄荷";
                    let shopVoArr = ShopModel.Inst.GetShopVoByType(ShopVO.BUYMAOBOHE);
                    let i = shopVoArr.indexOf(vo);
                    this.c1.selectedIndex = Math.min(i + 3, this.c1.pageCount - 1);
                }
                else if (vo.isGoldTimeShouyi) {
                    this.txt_ggtime.text = vo.shopRes.num + "小时";
                    let shopVoArr = ShopModel.Inst.GetShopVoByType(ShopVO.GOLDTIMESHOUYI);
                    let i = shopVoArr.indexOf(vo);
                    this.c1.selectedIndex = Math.min(i, this.c1.pageCount - 1);
                }
            }
            this.txt_diaNum.text = vo.price + "";
        }
        AddListener() {
            this.onClick(this, this.OnClickBuy);
        }
        OnClickBuy() {
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.Click_OK));
            let vo = this.vo;
            if (vo instanceof ShopVO) {
                if (!vo.canLookAd) {
                    return;
                }
                if (vo.isGoldTimeShouyi) {
                    UISystem.Inst.CreateWindowView(UIBuyGoldTips, vo);
                }
                else {
                    UISystem.Inst.CreateWindowView(UIBuyGoldAlert, vo);
                }
            }
        }
        RemoveListener() {
            this.offClick(this, this.OnClickBuy);
        }
        Reset() {
            super.Reset();
            this.vo = null;
        }
    }

    class GiftBoxItem extends TComponent {
        constructor() {
            super();
            this.sp_budian = null;
            this.sp_budian_c1 = null;
            this.c1 = null;
            this.c2 = null;
            this.c3 = null;
            this.c4 = null;
            this.txt_ggtime = null;
            this.btn_mfzs = null;
            this.btn_mflwx = null;
            this.btn_mxrdj = null;
            this.btn_fs = null;
            this.shopVo = null;
            this.date = null;
        }
        InitUI() {
            super.InitUI();
            this.sp_budian_c1 = this.sp_budian.getController("c1");
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
            this.c3 = this.getController("c3");
            this.c4 = this.getController("c4");
            this.date = new Date();
        }
        InitFairyGui() {
            super.InitFairyGui();
            FairyUtils.setVar(this, this);
        }
        InitData(data) {
            super.InitData(data);
        }
        UpdateData(vo) {
            if (!vo)
                return;
            this.shopVo = vo;
            if (vo.isLookAd && vo.curCount >= vo.freeNum) {
                this.ShowVideoBtn();
            }
            if (vo.canLookAd) {
                this.c3.selectedIndex = 0;
            }
            else {
                this.c3.selectedIndex = 2;
            }
            this.OnUpdateAdTime();
        }
        AddListener() {
            this.btn_mfzs.onClick(this, this.OnClickMFZSBtn);
            this.btn_mflwx.onClick(this, this.OnClickMFLWXBtn);
            this.btn_mxrdj.onClick(this, this.OnClickMXRDJBtn);
            this.btn_fs.onClick(this, this.OnClickFSBtn);
            Laya.timer.loop(1000, this, this.OnUpdateAdTime);
        }
        OnUpdateAdTime() {
            let vo = this.shopVo;
            if (!vo)
                return;
            if (!vo.isLookAd)
                return;
            let offestTime = Laya.timer.currTimer - vo.adTime;
            if (offestTime > ShopModel.Inst.adTimeCD) {
                this.ShowTime(false);
            }
            else {
                this.ShowTime(true);
                this.date.setTime(ShopModel.Inst.adTimeCD - offestTime);
                let hours = this.date.getHours() - 8;
                if (hours < 0)
                    hours += 24;
                let min = this.date.getMinutes();
                let sec = this.date.getSeconds();
                let offestX = hours < 10 ? "0" : "";
                let offestY = min < 10 ? "0" : "";
                let offestZ = sec < 10 ? "0" : "";
                this.txt_ggtime.text = `${offestX}${hours}:${offestY}${min}:${offestZ}${sec}`;
            }
            this.CheckShowRed();
        }
        CheckShowRed() {
            let vo = this.shopVo;
            let showRed = 0;
            if (vo instanceof ShopVO) {
                if (!vo.isLookAd)
                    return;
                if (!vo.isCd && vo.canLookAd) {
                    showRed = 1;
                }
            }
            this.ShowRed(showRed);
        }
        ShowRed(val = 0) {
            if (val < this.c4.pageCount) {
                this.c4.selectedIndex = val;
            }
        }
        OnClickMFZSBtn() {
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.Click_OK));
            if (!this.shopVo)
                return;
            if (!this.CheckCanGet())
                return;
            let handler = Laya.Handler.create(this, this.GetAward);
            if (this.shopVo.curCount < this.shopVo.freeNum) {
                handler.run();
            }
            else {
                PlatformMgr.Inst.showAd(AdType.ad_16, handler);
            }
        }
        OnClickMFLWXBtn() {
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.Click_OK));
            if (!this.shopVo)
                return;
            if (!this.CheckCanGet())
                return;
            let handler = Laya.Handler.create(this, this.GetAward);
            if (this.shopVo.curCount < this.shopVo.freeNum) {
                handler.run();
            }
            else {
                PlatformMgr.Inst.showAd(AdType.ad_17, handler);
            }
        }
        GetAward(success = true) {
            if (!success)
                return;
            if (this.shopVo.curCount >= this.shopVo.freeNum) {
                this.shopVo.adTime = Laya.timer.currTimer;
            }
            let shopRes = this.shopVo.shopRes;
            let id = shopRes.itemId;
            let num = shopRes.num;
            if (id == 13) {
                id = 1;
                num = BuildModel.Inst.GetAllBuildGoldRate(num * 60 * 60, false).toNumber();
            }
            this.shopVo.GetAward();
            UITaskView.itemVo.itemId = id;
            UITaskView.itemVo.count = num;
            UISystem.Inst.CreateWindowView(UIAwardView, UITaskView.itemVo);
            this.UpdateData(this.shopVo);
            GameDispatcher.Inst.event(UIGameEvent.ON_BUYFREELIBAO);
        }
        CheckCanGet() {
            if (this.shopVo.curCount >= this.shopVo.maxCount) {
                console.log("已达到当日购买上限");
                this.c3.selectedIndex = 2;
                return false;
            }
            if (Laya.timer.currTimer - this.shopVo.adTime < ShopModel.Inst.adTimeCD) {
                I18nUtils.ShowI18nById("matou_ad_cd");
                return false;
            }
            return true;
        }
        OnClickMXRDJBtn() {
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.Click_OK));
            UISystem.Inst.CreateWindowView(UIStoreContentView, 0);
        }
        OnClickFSBtn() {
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.Click_OK));
            UISystem.Inst.CreateWindowView(UIStoreContentView, 1);
        }
        SetGiftType(type = 0) {
            this.c1.selectedIndex = type;
        }
        ShowVideoBtn(isShow = true) {
            this.c2.selectedIndex = isShow ? 1 : 0;
        }
        ShowTime(isShow = true) {
            let vo = this.shopVo;
            if (!vo)
                return;
            if (vo.canLookAd) {
                this.c3.selectedIndex = isShow ? 1 : 0;
            }
            else {
                this.c3.selectedIndex = 2;
            }
        }
        ShowBgCol(val = 0) {
            this.sp_budian_c1.selectedIndex = val;
        }
        RemoveListener() {
            this.btn_mfzs.offClick(this, this.OnClickMFZSBtn);
            this.btn_mflwx.offClick(this, this.OnClickMFLWXBtn);
            this.btn_mxrdj.offClick(this, this.OnClickMXRDJBtn);
            this.btn_fs.offClick(this, this.OnClickFSBtn);
            Laya.timer.clear(this, this.OnUpdateAdTime);
        }
        Reset() {
            super.Reset();
            this.shopVo = null;
        }
    }

    class UIGiftBagListItem extends TListItem {
        constructor() {
            super();
            this.c1 = null;
            this.txt_num = null;
            this.sp_bagitem0 = null;
            this.sp_bagitem1 = null;
            this.sp_nrs = null;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
        }
        InitData(data) {
            super.InitData(data);
            this.UpdateData();
        }
        UpdateData() {
            let vo = this.Data;
            if (vo instanceof AwardVO) {
                this.c1.selectedIndex = 0;
                if (vo.IsItem) {
                    PanelRegister.loadUrl(vo.itemUrl, Laya.Handler.create(this, (url) => {
                        this.sp_bagitem0.url = url;
                        this.sp_bagitem1.url = url;
                    }));
                    if (vo.itemRes.quality > 0) {
                        this.sp_nrs.visible = true;
                        this.sp_nrs.getController("c1").selectedIndex = (vo.itemRes.quality - 1);
                    }
                    else {
                        this.sp_nrs.visible = false;
                    }
                }
                this.txt_num.text = "x" + MathUtil.goldFormat(vo.count, 0);
            }
        }
        Reset() {
            super.Reset();
        }
    }

    class UISpecialGiftBagItem extends TListItem {
        constructor() {
            super();
            this.c1 = null;
            this.c2 = null;
            this.btn_buy = null;
            this.btn_watch = null;
            this.sp_cat = null;
            this.txt_buyPrice = null;
            this.txt_desc = null;
            this.txt_desc1 = null;
            this.txt_tips = null;
            this.txt_giftbagName = null;
            this.eglist_giftbag = null;
            this.sp_xiangou = null;
            this.c3 = null;
            this.txt_ggtime = null;
            this.cat_model = null;
            this.sp_tipsbg = null;
            this.vo = null;
            this.date = null;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
            this.c3 = this.getController("c3");
            this.date = new Date();
        }
        InitFairyGui() {
            super.InitFairyGui();
            FairyUtils.setVar(this, this);
        }
        InitData(data) {
            super.InitData(data);
            this.UpdateData(data);
        }
        AddListener() {
            this.btn_buy.onClick(this, this.OnClickBuyBtn);
            this.btn_watch.onClick(this, this.OnClickWatchBtn);
            Laya.timer.loop(1000, this, this.UpdateTime);
        }
        UpdateTime() {
            if (!this.vo)
                return;
            let vo = this.vo;
            if (this.vo.buyNum >= this.vo.pkgRes.buyNum) {
                this.c3.selectedIndex = 2;
                this.txt_tips.text = "已售罄";
                this.btn_watch.visible = false;
                this.txt_desc.visible = false;
                this.sp_tipsbg.y = 120;
                this.txt_tips.y = 129;
            }
            else {
                this.sp_tipsbg.y = 35;
                this.txt_tips.y = 44;
                let offestTime = (Laya.timer.currTimer - vo.adTime);
                if (offestTime > vo.adCd) {
                    if (vo.maxAdNum > 0) {
                        this.c3.selectedIndex = 2;
                        this.txt_tips.text = `还需观看${vo.maxAdNum - vo.curAdNum}次广告`;
                    }
                    else {
                        this.c3.selectedIndex = 0;
                    }
                }
                else {
                    this.c3.selectedIndex = 1;
                    this.date.setTime(vo.adCd - offestTime);
                    let hours = this.date.getHours() - 8;
                    let min = this.date.getMinutes();
                    let sec = this.date.getSeconds();
                    let offestX = hours < 10 ? "0" : "";
                    let offestY = min < 10 ? "0" : "";
                    let offestZ = sec < 10 ? "0" : "";
                    this.txt_ggtime.text = `${offestX}${hours}:${offestY}${min}:${offestZ}${sec}`;
                }
            }
        }
        UpdateData(data) {
            let vo = data;
            this.vo = data;
            if (vo instanceof PackageVO) {
                this.sp_xiangou.visible = vo.pkgRes.buyNum == 1;
                let isLookAd = true;
                if (isLookAd) {
                    this.c1.selectedIndex = 1;
                }
                this.txt_giftbagName.text = vo.pkgName;
                this.eglist_giftbag.Data = vo.pkgAwardVos;
                this.txt_desc.text = vo.pkgRes.desc;
                if (vo.pkgName == "钻石成长礼包" || vo.pkgName == "钻石新手礼包") {
                    this.c2.selectedIndex = 6;
                }
                else if (vo.pkgName == "建筑加成礼包") {
                    this.c2.selectedIndex = 3;
                }
                else if (vo.pkgName == "阶段成长礼包") {
                    this.c2.selectedIndex = 4;
                }
                else if (vo.pkgName == "加速金币礼包") {
                    this.c2.selectedIndex = 2;
                }
                else if (vo.pkgName == "丘比特装饰礼包") {
                    this.c2.selectedIndex = 5;
                }
                else if (vo.pkgName == "白色土耳其安哥拉猫礼包") {
                    this.c2.selectedIndex = 7;
                }
                else {
                    this.c2.selectedIndex = 1;
                }
            }
            this.UpdateTime();
        }
        OnClickBuyBtn() {
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.Click_OK));
            if (!this.vo)
                return;
            if (!this.CheckCanGet())
                return;
            this.GetAward();
        }
        OnClickWatchBtn() {
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.Click_OK));
            if (!this.vo)
                return;
            if (!this.CheckCanGet())
                return;
            let handler = Laya.Handler.create(this, this.GetAward);
            PlatformMgr.Inst.showAd(AdType.ad_18, handler);
        }
        GetAward(isSuccess = true) {
            if (!isSuccess || this.vo == null)
                return;
            let awards = this.vo.pkgAwardVos.slice();
            this.vo.GetAward();
            this.UpdateData(this.vo);
            PlatformMgr.Inst.aldSendEvent(EventType.shop, this.vo.pkgRes.name + "-" + "视频");
            UISystem.Inst.CreateWindowView(UIAwardView, awards);
            GameDispatcher.Inst.event(UIGameEvent.SHOW_LIBAO_RED);
        }
        CheckCanGet() {
            if (this.vo.buyNum >= this.vo.pkgRes.buyNum) {
                console.log("已达到购买上限");
                this.c3.selectedIndex = 2;
                this.txt_tips.text = "已售罄";
                this.btn_watch.visible = false;
                this.txt_desc.visible = false;
                return false;
            }
            if (Laya.timer.currTimer - this.vo.adTime < this.vo.adCd) {
                console.log("商城广告cd中");
                return false;
            }
            return true;
        }
        SetGiftType(type = 0) {
            this.c1.selectedIndex = type;
        }
        Reset() {
            super.Reset();
            Laya.timer.clear(this, this.UpdateTime);
            this.vo = null;
        }
    }

    class UIShopPanelView extends TComponent {
        constructor() {
            super();
            this.sp_mflwx1 = null;
            this.sp_mflwx2 = null;
            this.sp_manghe1 = null;
            this.sp_manghe2 = null;
            this.sp_buygold1 = null;
            this.sp_buygold2 = null;
            this.sp_buygold3 = null;
            this.list_bg = null;
            this.item0 = null;
            this.eglist_giftbag = null;
            this.sp_buymbh1 = null;
            this.sp_buymbh2 = null;
            this.sp_buymbh3 = null;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
        }
        InitFairyGui() {
            super.InitFairyGui();
            FairyUtils.setVar(this, this);
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.SHOPBUYUPPERLIMIT, this.CheckShopLimit, this);
            this.AddGameListener(UIGameEvent.SHOW_LIBAO_RED, this.CheckShowRed, this);
            this.AddGameListener(UIGameEvent.ON_BUYFREELIBAO, this.UpdateLiBaoCount, this);
        }
        scrollToGuide() {
            if (this.scrollPane) {
                this.scrollPane.scrollToView(this.sp_manghe1, true, false);
            }
        }
        UpdateData() {
            this.UpdateMFLWX();
            this.UpdateBuyGold();
            this.UpdateBuyMBH();
            this.UpdataChuXiaoLiBao();
            this.CheckShopLimit();
            this.CheckShowRed();
            this.sp_manghe1.SetGiftType(2);
            this.sp_manghe1.ShowBgCol(1);
            this.sp_manghe2.SetGiftType(3);
            this.sp_manghe2.ShowBgCol(2);
        }
        UpdateMFLWX() {
            let shopVos1 = ShopModel.Inst.GetShopVoByType(ShopVO.GETDIAMOND);
            this.sp_mflwx1.UpdateData(shopVos1[0]);
            this.sp_mflwx1.SetGiftType(0);
            this.sp_mflwx1.ShowBgCol(0);
            let shopVos2 = ShopModel.Inst.GetShopVoByType(ShopVO.GETQIULU);
            this.sp_mflwx2.UpdateData(shopVos2[0]);
            this.sp_mflwx2.SetGiftType(1);
            this.sp_mflwx2.ShowBgCol(0);
            this.UpdateLiBaoCount();
        }
        UpdateLiBaoCount() {
            let shopVos1 = ShopModel.Inst.GetShopVoByType(ShopVO.GETDIAMOND);
            this.txt_diaCount.text = `(${shopVos1[0].curCount}/${shopVos1[0].maxCount}次)`;
            let shopVos2 = ShopModel.Inst.GetShopVoByType(ShopVO.GETQIULU);
            this.txt_churuCount.text = `(${shopVos2[0].curCount}/${shopVos2[0].maxCount}次)`;
        }
        UpdateBuyGold() {
            let shopVoArr = ShopModel.Inst.GetShopVoByType(ShopVO.GOLDTIMESHOUYI);
            this.sp_buygold1.UpdateData(shopVoArr[0]);
            this.sp_buygold2.UpdateData(shopVoArr[1]);
            this.sp_buygold3.UpdateData(shopVoArr[2]);
        }
        UpdateBuyMBH() {
            let shopVoArr = ShopModel.Inst.GetShopVoByType(ShopVO.BUYMAOBOHE);
            this.sp_buymbh1.UpdateData(shopVoArr[0]);
            this.sp_buymbh2.UpdateData(shopVoArr[1]);
            this.sp_buymbh3.UpdateData(shopVoArr[2]);
        }
        UpdataChuXiaoLiBao() {
            let vos = GiftPKGModel.Inst.GetPkgVos(1, 1);
            if (vos.length < 1)
                return;
            let data = vos.shift();
            this.item0.UpdateData(data);
            this.eglist_giftbag.Data = vos;
            this.eglist_giftbag.height = vos.length * 237;
            this.list_bg.numItems = vos.length;
        }
        CheckShowRed() {
            let isShowRed = 0;
            let pkgVOs = GiftPKGModel.Inst.GetPkgVos(1, 1);
            for (let i = 0; i < pkgVOs.length; i++) {
                let pkg = pkgVOs[i];
                if (!pkg.isCanGet)
                    continue;
                if (!pkg.isCd) {
                    isShowRed = 1;
                    break;
                }
            }
            this.ShowChuXiaoRed(isShowRed);
            this.ShowMangHeRed();
        }
        ShowMangHeRed() {
            if (ShopLockDrawModel.Inst.CanGetJiFenAward(ShopLockDrawVO.TYPE_DAOJU)) {
                this.sp_manghe1.ShowRed(1);
            }
            else {
                this.sp_manghe1.ShowRed(0);
            }
            if (ShopLockDrawModel.Inst.CanGetJiFenAward(ShopLockDrawVO.TYPE_PIFU)) {
                this.sp_manghe2.ShowRed(1);
            }
            else {
                this.sp_manghe2.ShowRed(0);
            }
        }
        ShowChuXiaoRed(val = 0) {
            if (val < this.c1.pageCount)
                this.c1.selectedIndex = val;
        }
        CheckShopLimit(vo = null) {
            this.UpdateBuyGold();
            this.UpdateBuyMBH();
        }
        RemoveListener() {
        }
        Reset() {
            super.Reset();
        }
    }

    class UIShoppingView extends UIBaseWindow {
        constructor() {
            super("shop", "UIShoppingView");
            this.sp_shoppanel = null;
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("shop", "GiftBoxItem", GiftBoxItem);
            PanelRegister.registerClass("shop", "UIGiftBagListItem", UIGiftBagListItem);
            PanelRegister.registerClass("shop", "UISpecialGiftBagItem", UISpecialGiftBagItem);
            PanelRegister.registerClass("shop", "UIShopPanelView", UIShopPanelView);
            PanelRegister.registerClass("shop", "BuyItem", BuyItem);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_CENTER;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
            this.c2 = this._view.getController("c2");
            this.spine_shanghua.Reset();
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.GUIDE_SHOW, this.showGuide, this);
        }
        InitData(data) {
            super.InitData(data);
            this.sp_shoppanel.UpdateData();
            let panel = GuideModel.Inst.funGuideVO && GuideModel.Inst.funGuideVO.panel;
            if (panel == "UIShoppingView|spine_shanghua") {
                this.spine_shanghua.play("shanghua");
                this.c2.selectedIndex = 1;
            }
            else {
                if (panel == "UIShoppingView|sp_shoppanel|sp_manghe1") {
                    this.sp_shoppanel.scrollToGuide();
                }
                this.c2.selectedIndex = 0;
            }
            this.hideBanner();
            if (Laya.Browser.onHWMiniGame) {
                PlatformMgr.Inst.showAd(AdType.ad_3);
            }
            else if (Laya.Browser.onQQMiniGame) {
                PlatformMgr.Inst.showAd(AdType.ad_3);
            }
            else if (PlatformMgr.Inst.isMZ) {
                PlatformMgr.Inst.showAd(AdType.ad_3);
            }
            else if (Laya.Browser.onQGMiniGame) {
                this.hideBanner();
                AdCtrl.Inst.ShowNatiEmbed(this);
            }
            else if ((Laya.Browser.onVVMiniGame || Laya.Browser.onKGMiniGame) && GuideModel.Inst.guideVO == null) {
                this.hideBanner();
                AdCtrl.Inst.DelayShowInsertAd();
            }
        }
        showGuide(guideVO) {
            let panel = guideVO && guideVO.panel;
            if (panel == "UIShoppingView|sp_shoppanel|sp_manghe1") {
                this.c2.selectedIndex = 0;
                this.sp_shoppanel.scrollToGuide();
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
        }
        Reset() {
            super.Reset();
        }
    }

    class SignListItem extends TListItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
            this.c3 = this.getController("c3");
        }
        InitData(data) {
            super.InitData(data);
            this.UpdateData();
        }
        UpdateData() {
            let vo = this.Data;
            if (!vo)
                return;
            this.c1.selectedIndex = vo.day;
            if (this.c1.selectedIndex >= LoginModel.Inst.signCount % 7) {
                this.c2.selectedIndex = 0;
                if (LoginModel.Inst.signCount % 7 == 0) {
                    if (!LoginModel.Inst.canSign) {
                        this.c2.selectedIndex = 1;
                    }
                }
            }
            else {
                this.c2.selectedIndex = 1;
            }
            if (!LoginModel.Inst.canSign && LoginModel.Inst.preLoginVo && LoginModel.Inst.canGetDima && vo == LoginModel.Inst.preLoginVo) {
                this.c3.selectedIndex = 1;
            }
            else {
                this.c3.selectedIndex = 0;
            }
            let awardId = vo.loginRes.award;
            let num = vo.loginRes.number;
            if (vo.isDaoJu) {
                let item = itemResource.getItemByKey(awardId);
                let iconName = item.icon;
                let url;
                if (item.type == 3) {
                    url = UrlUtils.GetIconPartUrl(iconName);
                }
                else {
                    url = UrlUtils.GetIconItemsUrl(iconName);
                }
                if (num > 1) {
                    this.txt_name.text = item.name + " x" + num;
                }
                else {
                    this.txt_name.text = item.name;
                }
                if (!LoginModel.Inst.canSign && LoginModel.Inst.canGetDima && (LoginModel.Inst.signCount - 1) % 7 == vo.day) {
                    url = MoneyModel.diamond.iconUrl;
                    this.c2.selectedIndex = 0;
                    let diamondNum = CVModel.Inst.GetValue(CVModel.goldnumb, CVModel.valType_number);
                    this.txt_name.text = "钻石 x" + diamondNum;
                }
                PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                    this.sp_reward.icon = url;
                }));
            }
            else if (vo.isCat) {
                let cat = catResource.getItemByKey(awardId);
                if (cat) {
                    let iconName = cat.icon;
                    let url;
                    url = PanelRegister.getUrl("icon_cat_head", iconName);
                    if (num > 1) {
                        this.txt_name.text = cat.name + " x" + num;
                    }
                    else {
                        this.txt_name.text = cat.name;
                    }
                    if (!LoginModel.Inst.canSign && LoginModel.Inst.canGetDima && (LoginModel.Inst.signCount - 1) % 7 == vo.day) {
                        url = MoneyModel.diamond.iconUrl;
                        this.c2.selectedIndex = 0;
                        let diamondNum = CVModel.Inst.GetValue(CVModel.goldnumb, CVModel.valType_number);
                        this.txt_name.text = "钻石 x" + diamondNum;
                    }
                    PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                        this.sp_reward.icon = url;
                    }));
                }
            }
            if (LoginModel.Inst.canSign && vo == LoginModel.Inst.curLoginVo || (LoginModel.Inst.preLoginVo && !LoginModel.Inst.canSign && LoginModel.Inst.canGetDima && vo == LoginModel.Inst.preLoginVo)) {
                this.Shake();
            }
            else {
                fgui.GTween.kill(this.sp_reward);
            }
        }
        Shake() {
            fgui.GTween.kill(this.sp_reward);
            fgui.GTween.shake(this.sp_reward.x, this.sp_reward.y, 3, 0.5).setTarget(this.sp_reward).onUpdate((tw) => {
                let target = tw.target;
                target.rotation = target.x - tw.value.x;
                target.x = tw.value.x;
                target.y = tw.value.y;
            }, this).onComplete(this.Shake, this).setDelay(2);
        }
        Reset() {
            super.Reset();
            fgui.GTween.kill(this.sp_reward);
        }
    }

    class UISignView extends UIBaseWindow {
        constructor() {
            super("sign", "UISignView");
            this.eglist_days = null;
            this.group_sevenDay = null;
            this.txt_name = null;
            this.sp_reward = null;
            this.sp_sevenDayMask = null;
            this.btn_receive = null;
            this.btn_hyzs = null;
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_CENTER;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isEffect = true;
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("sign", "SignListItem", SignListItem);
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
            this.c2 = this._view.getController("c2");
            this.eglist_days.selectionMode = fgui.ListSelectionMode.Multiple;
            this.p1 = this._view.getController("p1");
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                this.p1.selectedPage = "wx";
            }
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.OnClickItem, this, this.eglist_days);
            this.group_sevenDay.onClick(this, this.onClickSevenDay);
        }
        InitData(data) {
            super.InitData(data);
            this.UpdateData();
            this.btn_receive.alpha = 0;
            this.btn_hyzs.alpha = 0;
            this.ShowEffect();
            GuideCtrl.Inst.showSignGuide();
            if ((Laya.Browser.onVVMiniGame || Laya.Browser.onKGMiniGame) && GuideModel.Inst.guideVO == null) {
                this.hideBanner();
                AdCtrl.Inst.DelayShowInsertAd();
            }
            else {
                this.ShowBannerAd();
            }
        }
        ShowBannerAd() {
            if (PlatformModel.Inst.windowHeight < 720) {
                this.hideBanner();
            }
            else {
                this.showBanner();
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_receive) {
                this.OnGetBtnClick();
            }
            else if (btn == this.btn_hyzs) {
                this.OnHYZSBtnClick();
            }
        }
        onClickSevenDay() {
            this.GetSignItem(LoginModel.Inst.SevenLoginVo);
        }
        OnClickItem(item) {
            let vo = item.Data;
            if (vo instanceof LoginVO) {
                this.GetSignItem(vo);
            }
        }
        GetSignItem(vo) {
            if (LoginModel.Inst.canSign && vo == LoginModel.Inst.curLoginVo) {
                this.SignSuccess();
            }
            else if (LoginModel.Inst.canGetDima && LoginModel.Inst.preLoginVo && vo == LoginModel.Inst.preLoginVo) {
                this.OnHYZSBtnClick();
            }
        }
        UpdateData() {
            if (LoginModel.Inst.canSign) {
                this.c1.selectedIndex = 1;
            }
            else if (LoginModel.Inst.canGetDima) {
                this.c1.selectedIndex = 2;
            }
            else {
                this.c1.selectedIndex = 0;
            }
            this.eglist_days.Data = LoginModel.Inst.SixLoginVos;
            this.sp_sevenDayMask.visible = (LoginModel.Inst.signCount % 7 == 6 && !LoginModel.Inst.canSign && !LoginModel.Inst.canGetDima) ? true : false;
            let sevenVo = LoginModel.Inst.SevenLoginVo;
            let awardId = sevenVo.loginRes.award;
            let num = sevenVo.loginRes.number;
            if (!LoginModel.Inst.canSign && LoginModel.Inst.preLoginVo && LoginModel.Inst.canGetDima && sevenVo == LoginModel.Inst.preLoginVo) {
                this.c2.selectedIndex = 1;
            }
            else {
                this.c2.selectedIndex = 0;
            }
            if (sevenVo.isDaoJu) {
                let item = itemResource.getItemByKey(awardId);
                let iconName = item.icon;
                let url;
                if (item.type == 3) {
                    url = UrlUtils.GetIconPartUrl(iconName);
                }
                else {
                    url = UrlUtils.GetIconItemsUrl(iconName);
                }
                if (num > 1) {
                    this.txt_name.text = item.name + "x" + num;
                }
                else {
                    this.txt_name.text = item.name;
                }
                if (!LoginModel.Inst.canSign && LoginModel.Inst.canGetDima && LoginModel.Inst.signCount % 7 == 0) {
                    url = MoneyModel.diamond.iconUrl;
                    let diamondNum = CVModel.Inst.GetValue(CVModel.goldnumb, CVModel.valType_number);
                    this.txt_name.text = "钻石 x" + diamondNum;
                }
                PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                    this.sp_reward.icon = url;
                }));
            }
            else if (sevenVo.isCat) {
                let cat = catResource.getItemByKey(awardId);
                if (cat) {
                    let iconName = cat.icon;
                    let url;
                    url = PanelRegister.getUrl("icon_cat_head", iconName);
                    if (num > 1) {
                        this.txt_name.text = cat.name + " x" + num;
                    }
                    else {
                        this.txt_name.text = cat.name;
                    }
                    if (!LoginModel.Inst.canSign && LoginModel.Inst.canGetDima && LoginModel.Inst.signCount % 7 == 0) {
                        url = MoneyModel.diamond.iconUrl;
                        let diamondNum = CVModel.Inst.GetValue(CVModel.goldnumb, CVModel.valType_number);
                        this.txt_name.text = "钻石 x" + diamondNum;
                    }
                    PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                        this.sp_reward.icon = url;
                    }));
                }
            }
            if (LoginModel.Inst.canSign && sevenVo == LoginModel.Inst.curLoginVo || (LoginModel.Inst.preLoginVo && !LoginModel.Inst.canSign && LoginModel.Inst.canGetDima && sevenVo == LoginModel.Inst.preLoginVo)) {
                this.Shake();
            }
            else {
                fgui.GTween.kill(this.sp_reward);
            }
        }
        Shake() {
            fgui.GTween.kill(this.sp_reward);
            fgui.GTween.shake(this.sp_reward.x, this.sp_reward.y, 3, 0.5).setTarget(this.sp_reward).onUpdate((tw) => {
                let target = tw.target;
                target.rotation = target.x - tw.value.x;
                target.x = tw.value.x;
                target.y = tw.value.y;
            }, this).onComplete(this.Shake, this).setDelay(2);
        }
        ShowEffect() {
            let arr = this.eglist_days.items;
            arr.push(this.group_sevenDay);
            EffectUtil.showListEffect(arr, 0.4, 0.1, Laya.Handler.create(this, this.ShowBtns));
        }
        ShowBtns() {
            fgui.GTween.to(this.btn_receive.alpha, 1, 0.5).setTarget(this.btn_receive, "alpha");
            fgui.GTween.to(this.btn_hyzs.alpha, 1, 0.5).setTarget(this.btn_hyzs, "alpha");
        }
        OnGetBtnClick() {
            if (LoginModel.Inst.canSign) {
                this.SignSuccess();
            }
            else {
            }
        }
        OnHYZSBtnClick() {
            if (!LoginModel.Inst.canGetDima)
                return;
            let handler = Laya.Handler.create(this, (success = true) => {
                if (success) {
                    let diamondNum = CVModel.Inst.GetValue(CVModel.goldnumb, CVModel.valType_number);
                    MoneyModel.diamond.add(diamondNum);
                    UIAwardView.awardVO.setData({ "tempId": MoneyModel.diamond.type, "type": AwardVO.TYPE_ITEM, "count": diamondNum });
                    LoginModel.Inst.canGetDima = false;
                    UISystem.Inst.CreateWindowView(UIAwardView, UIAwardView.awardVO);
                    GameDispatcher.Inst.event(UIGameEvent.SHOW_SIGINBTN_RED, LoginModel.Inst.canSign || LoginModel.Inst.canGetDima);
                    this.UpdateData();
                }
            });
            PlatformMgr.Inst.showAd(AdType.ad_8, handler);
        }
        SignSuccess(arr = []) {
            let vo = LoginModel.Inst.curLoginVo;
            LoginModel.Inst.GetAward();
            let id = vo.loginRes.award;
            let num = vo.loginRes.number;
            let data = null;
            if (vo.isDaoJu) {
                if (vo.loginRes.award == ItemModel.ITEM_SHALOU) {
                    id = ItemModel.ITEM_GOLD;
                    num = BuildModel.Inst.GetAllBuildGoldRate(num * 60 * 60, false).toNumber();
                }
                UITaskView.itemVo.itemId = id;
                UITaskView.itemVo.count = num;
                data = UITaskView.itemVo;
            }
            else if (vo.isCat) {
                data = CatModel.Inst.GetCatVOById(id);
            }
            arr.push(data);
            UISystem.Inst.CreateWindowView(UIAwardView, arr);
            this.UpdateData();
        }
        Reset() {
            super.Reset();
            fgui.GTween.kill(this.sp_reward);
        }
    }

    class UIBaikeGoodItem extends TListButtonItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
            this.c3 = this.getController("c3");
            this.c4 = this.getController("c4");
            this.c5 = this.getController("c5");
            this.c6 = this.getController("c6");
        }
        InitData(data) {
            super.InitData(data);
            if (data instanceof SkinVO) {
                this.showRes(data.skinRes);
            }
            else if (data instanceof skinResource) {
                this.showRes(data);
            }
        }
        showRes(data) {
            if (data instanceof skinResource) {
                let suitRes = suitResource.getItemByKey(data.suitId);
                let itemRes = itemResource.getItemByKey(data.itemId);
                let partName = SkinVO.GetPartName(data.icon);
                this.setIcon(SkinVO.GetSkinUrl(data));
                let cnt = StorageUtils.Inst.GetSkinBaike(data.id);
                this.txt_count.text = "x" + cnt;
                this.c1.selectedIndex = itemRes.quality;
                this.c2.selectedIndex = 0;
                this.c3.selectedIndex = 0;
                this.isLock = cnt <= 0;
                this.isSuit = data.suitId > 0;
                if (this.isSuit) {
                    this.txt_name.text = `【${suitRes.name}】\n${partName}`;
                }
                else {
                    this.txt_name.text = itemRes.name;
                }
            }
        }
        set part(value) {
            this.c4.selectedIndex = value;
        }
        get part() {
            return this.c4.selectedIndex;
        }
        set isLock(value) {
            this.c5.selectedIndex = value ? 1 : 0;
            this._iconObject.grayed = value;
            this.txt_count.visible = !value;
        }
        get isLock() {
            return this.c5.selectedIndex == 1;
        }
        set isSuit(value) {
            this.c6.selectedIndex = value ? 1 : 0;
        }
        get isSuit() {
            return this.c6.selectedIndex == 1;
        }
        get skinVo() {
            return this.Data;
        }
        Reset() {
            super.Reset();
        }
    }

    class UIBaikeSuitItem extends TListItem {
        constructor() {
            super();
            this.isCatBag = true;
        }
        InitUI() {
            super.InitUI();
        }
        InitData(data) {
            super.InitData(data);
            if (data instanceof SuitVO) {
                this.txt_name.text = data.suitRes.name;
                let skinVos = data.skinVos;
                this.eglist_items.Data = skinVos;
                this.txt_count.text = this.getUnlockCount() + "/" + data.skinIds.length;
            }
        }
        getUnlockCount() {
            let count = 0;
            let item;
            for (let i = 0; i < this.eglist_items.numChildren; i++) {
                item = this.eglist_items.getChildAt(i);
                if (!item.isLock) {
                    count++;
                }
            }
            return 0;
        }
        Reset() {
            super.Reset();
        }
    }

    class UISkinBaikeView extends UIBaseWindow {
        constructor() {
            super("skin", "UISkinBaikeView");
            this.cnt_1 = 0;
            this.cnt_2 = 0;
            this.cnt_3 = 0;
            this.cnt_total_1 = 0;
            this.cnt_total_2 = 0;
            this.cnt_total_3 = 0;
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("skin", "UIBaikeGoodItem", UIBaikeGoodItem);
            PanelRegister.registerClass("skin", "UIBaikeSuitItem", UIBaikeSuitItem);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
            this.eglist_suits.setVirtual();
            this.eglist_items.setVirtual();
            this.tb_0 = this.tab.getChildAt(0).asButton;
            this.tb_1 = this.tab.getChildAt(1).asButton;
            this.tb_2 = this.tab.getChildAt(2).asButton;
            this.tb_3 = this.tab.getChildAt(3).asButton;
            this.c1 = this._view.getController("c1");
            this.c2 = this._view.getController("c2");
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onTabClick, this, this.tab);
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onTab2Click, this, this.tab1);
        }
        InitData(data) {
            super.InitData(data);
            let res;
            let ownCnt = 0;
            for (let i = 0; i < skinResource.items.length; i++) {
                res = skinResource.items[i];
                ownCnt = StorageUtils.Inst.GetSkinBaike(res.id);
                if (res.award_type == 1) {
                    if (ownCnt > 0)
                        this.cnt_1++;
                    this.cnt_total_1++;
                }
                else if (res.award_type == 2) {
                    if (ownCnt > 0)
                        this.cnt_2++;
                    this.cnt_total_2++;
                }
                else if (res.award_type == 3) {
                    if (ownCnt > 0)
                        this.cnt_3++;
                    this.cnt_total_3++;
                }
            }
            this.tb_0.title = (this.cnt_1 + this.cnt_2 + this.cnt_3) + "/" + (this.cnt_total_1 + this.cnt_total_2 + this.cnt_total_3);
            this.tb_1.title = `${this.cnt_1}/${this.cnt_total_1}`;
            this.tb_2.title = `${this.cnt_2}/${this.cnt_total_2}`;
            this.tb_3.title = `${this.cnt_3}/${this.cnt_total_3}`;
            this.tab.selectedIndex = 0;
            this.tab1.selectedIndex = 0;
            Laya.timer.once(100, this, this.updateList);
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                if (AdModel.Inst.wxIsShowMuiltGrid) {
                    this.hideBanner();
                    this.showMgridAd();
                }
                else {
                    this.showBanner();
                }
            }
            else {
                this.showBanner();
            }
        }
        showMgridAd() {
            this.ad_mgrid.InitData({ "adId": AdType.ad_29 });
        }
        onTabClick() {
            this.updateList();
        }
        onTab2Click() {
            this.updateList();
        }
        updateList() {
            if (this.tab1.selectedIndex == 7) {
                this.eglist_suits.Data = SkinModel.Inst.GetAllSuitVosByAwardType(this.tab.selectedIndex);
            }
            else {
                this.eglist_items.Data = this.getSkinResItems(this.tab.selectedIndex, this.tab1.selectedIndex);
            }
            this.eglist_items.scrollToTop();
            this.eglist_suits.scrollToTop();
        }
        getSkinResItems(award_type = 0, part = 0) {
            if (award_type == 0 && part == 0) {
                return skinResource.items;
            }
            let arr = [];
            let skinRes = null;
            for (let i = 0; i < skinResource.items.length; i++) {
                skinRes = skinResource.items[i];
                if ((award_type == 0 || skinRes.award_type == award_type) && (part == 0 || part == skinRes.icon)) {
                    arr.push(skinRes);
                }
            }
            return arr;
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
        }
        get catVo() {
            return this.Data;
        }
        Reset() {
            super.Reset();
        }
    }

    class SubscribeAwardItem extends TComponent {
        constructor() {
            super();
        }
        InitFairyGui() {
            super.InitFairyGui();
            FairyUtils.setVar(this, this);
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
        }
        InitData(data) {
            super.InitData(data);
        }
        SetAward(award) {
            if (award.isGem) {
                this.c1.selectedIndex = 1;
            }
            else if (award.itemRes.id == ItemModel.ITEM_CHURU) {
                this.c1.selectedIndex = 0;
            }
            this.txt_tips.text = award.itemRes.name + " x" + award.count;
        }
        Reset() {
            super.Reset();
        }
    }

    class UISubscribeView extends UIBaseWindow {
        constructor() {
            super("subscribe", "UISubscribeView");
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("subscribe", "SubscribeAwardItem", SubscribeAwardItem);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isShowMaskBg = false;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            this.UpdateData();
            Laya.timer.loop(1000, this, this.UpdateTime);
            this.UpdateTime();
            this.showBanner();
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_time) {
                this.OnClickBtnTime();
            }
        }
        OnClickBtnTime() {
            if (YuYueModel.Inst.state == YuYueModel.state0) {
                if (PlatformMgr.Inst.isWX) {
                    if (!PlatformMgr.Inst.hasSubscribeApp) {
                        PlatformMgr.Inst.subscribeAppMsg(Laya.Handler.create(this, (isSuccess = true) => {
                            if (isSuccess) {
                                StorageUtils.Inst.setForeverRecord(1, StorageUtils.qq_subscribe, true);
                                let count = StorageUtils.Inst.getForeverRecord(StorageUtils.qq_subscribe);
                                if (count == 1) {
                                    YuYueModel.Inst.state = YuYueModel.state1;
                                    this.UpdateData();
                                    this.UpdateTime();
                                }
                            }
                        }));
                    }
                    else {
                        YuYueModel.Inst.state = YuYueModel.state1;
                    }
                }
                else {
                    YuYueModel.Inst.state = YuYueModel.state1;
                }
            }
            else if (YuYueModel.Inst.canGet) {
                YuYueModel.Inst.state = YuYueModel.state3;
                let award1 = YuYueModel.Inst.award1;
                this.GetAwardItem(award1.tempId, parseInt(award1.count.toString()), award1.itemRes.type);
                UISystem.Inst.CreateWindowView(UIAwardView, YuYueModel.Inst.award1);
                YuYueModel.Inst.startGetAwardTime = TimeModel.Inst.LocalTime;
            }
            else if (YuYueModel.Inst.state == YuYueModel.state4) {
                YuYueModel.Inst.state = YuYueModel.state5;
                let award2 = YuYueModel.Inst.award2;
                this.GetAwardItem(award2.tempId, parseInt(award2.count.toString()), award2.itemRes.type);
                UISystem.Inst.CreateWindowView(UIAwardView, YuYueModel.Inst.award2);
            }
            this.UpdateData();
            this.UpdateTime();
        }
        GetAwardItem(itemId, num, itemType) {
            if (itemId == 1) {
                MoneyModel.gold.add(num);
            }
            else if (itemId == 2) {
                MoneyModel.diamond.add(num);
            }
            else if (itemId == 13) {
                let goldNum = BuildModel.Inst.GetAllBuildGoldRate(num * 60 * 60, false);
                MoneyModel.gold.add(goldNum);
            }
            else if (itemType == ItemModel.TYPE_ITEM || itemType == ItemModel.TYPE_BUILD) {
                ItemCtrl.Inst.AddItemVO(itemId, num);
            }
            else if (itemType == ItemModel.TYPE_SKIN) {
                this.AddSkinAward(itemId, num);
            }
        }
        AddSkinAward(itemId, num) {
            let skinResArr = skinResource.items;
            for (let i = 0; i < skinResArr.length; i++) {
                let res = skinResArr[i];
                if (res.itemId == itemId) {
                    SkinModel.Inst.AddSkin(res.id, num);
                    break;
                }
            }
        }
        UpdateData() {
            let state = YuYueModel.Inst.state;
            let award;
            if (state == YuYueModel.state3 || state == YuYueModel.state4 || state == YuYueModel.state5) {
                award = YuYueModel.Inst.award2;
            }
            else {
                award = YuYueModel.Inst.award1;
            }
            this.c1.selectedIndex = 0;
            this.awardItem1.SetAward(award);
            if (state == YuYueModel.state0) {
                this.btn_time.getController("c1").selectedIndex = 0;
            }
            else if (state == YuYueModel.state1 || state == YuYueModel.state3) {
                this.btn_time.getController("c1").selectedIndex = 1;
            }
            else if (YuYueModel.Inst.canGet || state == YuYueModel.state4) {
                this.btn_time.getController("c1").selectedIndex = 2;
            }
            else if (state == YuYueModel.state5) {
                GameDispatcher.Inst.event(UIGameEvent.ONUPDATEYUYUEBTNSHOW);
                this.close();
            }
        }
        UpdateTime() {
            let state = YuYueModel.Inst.state;
            if (state == YuYueModel.state3) {
                let offestTime = YuYueModel.Inst.time - Math.floor((TimeModel.Inst.LocalTime - YuYueModel.Inst.startGetAwardTime) / 1000);
                this.btn_time.title = StringUtil.formatToTime(offestTime);
                if (offestTime < 0) {
                    Laya.timer.clear(this, this.UpdateTime);
                    YuYueModel.Inst.state = YuYueModel.state4;
                    this.UpdateData();
                }
                this.n2.visible = true;
            }
            else if (state == YuYueModel.state1) {
                let data = TimeModel.Inst.getDate(TimeModel.Inst.LocalTime);
                let hours = 23 - data.getHours();
                let min = 59 - data.getMinutes();
                let sec = 59 - data.getSeconds();
                let offestX = hours < 10 ? "0" : "";
                let offestY = min < 10 ? "0" : "";
                let offestZ = sec < 10 ? "0" : "";
                this.btn_time.title = `${offestX}${hours}:${offestY}${min}:${offestZ}${sec}`;
                this.n2.visible = true;
            }
            else {
                this.n2.visible = false;
            }
        }
        onResize() {
            super.onResize();
            this._view.setSize(Laya.stage.width, Laya.stage.height);
        }
        Reset() {
            super.Reset();
            Laya.timer.clear(this, this.UpdateTime);
        }
    }

    class UIProItem extends TListItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
        }
        InitData(data) {
            super.InitData(data);
            this.UpdateData();
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.SUMMER_ACTIVE, this.RefreshActive, this);
        }
        UpdateData() {
            let vo = this.Data;
            if (vo instanceof WarOrderVo) {
                this.txt_num.text = vo.warOrderRes.active + "";
                this.txt_level.text = vo.warOrderRes.Grade + "";
                this.RefreshActive();
            }
        }
        RefreshActive() {
            let vo = this.Data;
            if (vo instanceof WarOrderVo) {
                let preActive = 0;
                if (vo.prevRes) {
                    preActive = vo.prevRes.active;
                }
                let max = vo.warOrderRes.active - preActive;
                let curAct = WarOrderModel.Inst.curActivity - preActive;
                let offestVal = 0;
                this.pro_item0.max = max * 3 / 4;
                this.pro_item1.max = max / 4;
                this.pro_item0.value = Math.max(0, Math.min(curAct + offestVal, this.pro_item0.max));
                this.pro_item1.value = Math.min(this.pro_item1.max, Math.max(curAct - this.pro_item0.max, 0));
            }
        }
        Reset() {
            super.Reset();
        }
    }

    class UISummerActivTaskListItem extends TListItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
            this.item.isYellow = false;
            this.item2.isYellow = true;
        }
        InitData(data) {
            super.InitData(data);
            this.UpdateData();
        }
        AddListener() {
            this.item.onClick(this, this.GetAward);
            this.item2.onClick(this, this.GetExtraAward);
        }
        GetExtraAward() {
            let vo = this.Data;
            if (!vo)
                return;
            if (!vo.canAward)
                return;
            let handler = Laya.Handler.create(this, (success) => {
                if (success) {
                    let awardArr = [];
                    awardArr.push(vo.exAward);
                    vo.GetOtherAward();
                    this.GetAward(awardArr);
                }
            });
            PlatformMgr.Inst.showAd(AdType.ad_20, handler);
        }
        GetAward(arr = null) {
            let vo = this.Data;
            if (!vo)
                return;
            if (!(arr instanceof Array)) {
                arr = [];
            }
            if (vo.canAward) {
                vo.GetAward();
                if (arr.length > 0) {
                    UISystem.Inst.CreateWindowView(UIAwardView, arr);
                }
                this.UpdateData();
            }
        }
        UpdateData() {
            let vo = this.Data;
            if (!vo)
                return;
            let res = vo.taskRes;
            this.txt_title.text = res.desc;
            this.probar.max = vo.maxCount;
            this.probar.value = vo.curPro;
            let item = itemResource.getItemByKey(res.item);
            let iconName = item.icon;
            let url;
            if (item.type == 3) {
                url = UrlUtils.GetIconPartUrl(iconName);
            }
            else if (item.type == 5) {
                url = fgui.UIPackage.getItemURL("summerActivity", iconName);
            }
            else {
                url = UrlUtils.GetIconItemsUrl(iconName);
            }
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                this.item.icon = url;
            }));
            this.item.SetNum(MathUtil.goldFormat(res.num) + "");
            this.item.isAd = false;
            this.item.SetState(vo.state);
            this.SetState(vo.state);
            let award2 = vo.exAward;
            let item2 = itemResource.getItemByKey(award2.tempId);
            let iconName2 = item2.icon;
            let url2;
            if (item2.type == 3) {
                url2 = UrlUtils.GetIconPartUrl(iconName2);
            }
            else if (item2.type == 5) {
                url2 = fgui.UIPackage.getItemURL("summerActivity", iconName2);
            }
            else {
                url2 = UrlUtils.GetIconItemsUrl(iconName2);
            }
            PanelRegister.loadUrl(url2, Laya.Handler.create(this, (url) => {
                this.item2.icon = url2;
            }));
            this.item2.SetNum(MathUtil.goldFormat(award2.count) + "");
            this.item2.isAd = true;
            this.item2.SetState(vo.state);
        }
        SetState(state) {
            if (state == BaseTaskVO.STATE_NORMAL) {
                this.c1.selectedIndex = 0;
            }
            else if (state == BaseTaskVO.STATE_CAN_AWARD) {
                this.c1.selectedIndex = 1;
            }
            else if (state == BaseTaskVO.STATE_COMPLETED) {
                this.c1.selectedIndex = 2;
            }
        }
        RemoveListener() {
            this.item.offClick(this, this.GetAward);
        }
        Reset() {
            super.Reset();
        }
    }

    class UISummerAwardItem extends TListItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
            this.c3 = this.getController("c3");
        }
        InitData(data) {
            super.InitData(data);
            this.UpdateData();
        }
        SetData(data) {
            super.InitData(data);
            this.UpdateData();
        }
        AddListener() {
            super.AddListener();
        }
        UpdateData() {
            let vo = this.Data;
            if (vo instanceof WarOrderAwardVo) {
                let url = vo.award.itemUrl;
                let desc = vo.award.itemRes ? vo.award.itemRes.name : "";
                this.c1.selectedIndex = vo.awardState;
                if (vo.isSenior && !WarOrderModel.Inst.isUnLockSenior) {
                    this.c2.selectedIndex = 1;
                }
                else {
                    this.c2.selectedIndex = 0;
                }
                let catHeadRes = vo.award.itemRes ? ItemModel.Inst.getCatRes(vo.award.itemRes.id) : null;
                if (catHeadRes) {
                    this.c3.selectedIndex = 1;
                    PanelRegister.loadUrl(vo.award.catHeadUrl, Laya.Handler.create(this, (iconUrl) => {
                        this.btn_catHead.icon = iconUrl;
                    }));
                }
                else {
                    this.c3.selectedIndex = 0;
                }
                if (url) {
                    PanelRegister.loadUrl(url, Laya.Handler.create(this, (iconUrl) => {
                        this.loader_item.url = iconUrl;
                    }));
                }
                let str = MathUtil.goldFormat(vo.award.count, 0);
                if (str)
                    this.txt_num.text = "x" + str;
                this.txt_desc.text = desc;
            }
        }
        RemoveListener() {
            super.RemoveListener();
        }
        Reset() {
            super.Reset();
        }
    }

    class UIWarOrderItem extends TListItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.eglist_award0.selectionMode = fgui.ListSelectionMode.Multiple;
            this.eglist_award2.selectionMode = fgui.ListSelectionMode.Multiple;
        }
        InitData(data) {
            super.InitData(data);
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.SUMMER_ACTIVE, this.UpdateActive, this);
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.GetAward, this, this.eglist_award0);
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.GetAward, this, this.eglist_award2);
            this.AddGameListener(UIGameEvent.ONUNLOCKSUMMER, this.UpdateData, this);
            this.btn_totalAward.onClick(this, this.OnClickLoopAward);
            this.btn_buy.onClick(this, this.OnClickBuyBtn);
        }
        OnClickBuyBtn() {
            let handler = Laya.Handler.create(this, (isSuccess) => {
                if (isSuccess) {
                    WarOrderModel.Inst.isUnLockSenior = true;
                    this.UpdateData();
                }
            });
            PlatformMgr.Inst.showAd(AdType.ad_21, handler);
        }
        OnClickLoopAward() {
            let vo = this.btn_totalAward.Data;
            if (vo instanceof WarOrderAwardVo) {
                if (vo.canGetAward) {
                    WarOrderModel.Inst.GetLoopAward(vo);
                    this.btn_totalAward.UpdateData();
                    this.UpdateActive();
                    UISystem.Inst.CreateWindowView(UIAwardView, vo.award);
                }
            }
        }
        GetAward(item) {
            let vo = item.Data;
            if (vo instanceof WarOrderAwardVo) {
                if (vo.canGetAward) {
                    WarOrderModel.Inst.GetAward(vo);
                    item.UpdateData();
                    UISystem.Inst.CreateWindowView(UIAwardView, vo.award);
                }
            }
        }
        UpdateData() {
            let listData = WarOrderModel.Inst.GetWarOrderVos();
            if (!listData)
                return;
            this.btn_buy.visible = !WarOrderModel.Inst.isUnLockSenior;
            this.list_bg.numItems = listData.length - 4;
            this.list_bg.width = (listData.length - 4) * 216;
            this.eglist_award0.Data = WarOrderModel.Inst.awardArr;
            ;
            this.eglist_award2.Data = WarOrderModel.Inst.seniorawardArr;
            this.eglist_pro.Data = listData;
            this.UpdateActive();
        }
        UpdateActive() {
            this.UpdateBar();
            this.pro_total.max = WarOrderModel.Inst.warOrderVo.warOrderRes.active;
            this.btn_totalAward.SetData(WarOrderModel.Inst.award);
            this.txt_totalNum.text = WarOrderModel.Inst.award.active + "";
            let offestVal = WarOrderModel.Inst.curActivity - (WarOrderModel.Inst.award.active - WarOrderModel.Inst.warOrderVo.warOrderRes.active);
            this.pro_total.value = fgui.ToolSet.clamp(offestVal, 0, this.pro_total.max);
        }
        UpdateBar() {
            this.btn_bar.title = WarOrderModel.Inst.curActivity + "";
            let rate = WarOrderModel.Inst.curActivity / WarOrderModel.Inst.maxActivity;
            this.btn_bar.x = this.eglist_pro.x + 167 * WarOrderModel.Inst.GetWarOrderVos().length * rate;
            this.btn_bar.x = Math.max(this.btn_bar.x, 149);
        }
        Reset() {
            super.Reset();
            this.btn_totalAward.offClick(this, this.OnClickLoopAward);
        }
    }

    class UISummerExplainView extends UIBaseWindow {
        constructor() {
            super("summerActivity", "UISummerExplainView");
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = true;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                this.c1.selectedPage = "wx";
                this.btn_close.y = this.n0.y + 60;
            }
        }
        InitData(param) {
            this.m_param = param;
            this.UpdateData();
            if (PlatformModel.Inst.windowHeight < 720) {
                this.hideBanner();
            }
            else {
                this.showBanner();
            }
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.ONUNLOCKSUMMER, this.UpdateData, this);
        }
        UpdateData() {
            this.btn_get.visible = !WarOrderModel.Inst.isUnLockSenior;
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_get) {
                let handler = Laya.Handler.create(this, (success) => {
                    if (success) {
                        WarOrderModel.Inst.isUnLockSenior = true;
                        GameDispatcher.Inst.event(UIGameEvent.ONUNLOCKSUMMER);
                    }
                });
                PlatformMgr.Inst.showAd(AdType.ad_19, handler);
            }
        }
        RemoveListener() {
        }
        onResize() {
            super.onResize();
            this._view.setSize(Laya.stage.width, Laya.stage.height);
        }
        Reset() {
            super.Reset();
        }
    }

    class UISummerActivityView extends UIBaseWindow {
        constructor() {
            super("summerActivity", "UISummerActivityView");
            this.date = TimeModel.Inst.getDate();
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("summerActivity", "UIProItem", UIProItem);
            PanelRegister.registerClass("summerActivity", "UISummerActivTaskListItem", UISummerActivTaskListItem);
            PanelRegister.registerClass("summerActivity", "UISummerAwardItem", UISummerAwardItem);
            PanelRegister.registerClass("summerActivity", "UIWarOrderItem", UIWarOrderItem);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
            this.c1 = this._view.getController("c1");
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                this.c1.selectedPage = "wx";
            }
        }
        InitData(param) {
            this.m_param = param;
            this.item_warOrder.UpdateData();
            this.UpdateTask();
            Laya.timer.loop(1000, this, this.OnUpdateTime);
            this.OnUpdateTime();
            if ((Laya.Browser.onVVMiniGame || Laya.Browser.onKGMiniGame) && GuideModel.Inst.guideVO == null) {
                this.hideBanner();
                AdCtrl.Inst.DelayShowInsertAd();
            }
            else {
                this.ShowBannerAd();
            }
        }
        ShowBannerAd() {
            if (PlatformModel.Inst.windowHeight < 720) {
                this.hideBanner();
            }
            else {
                this.showBanner();
            }
        }
        UpdateTask() {
            this.eglist_items.Data = WarOrderModel.Inst.taskVos;
        }
        AddListener() {
            super.AddListener();
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_tips) {
                UISystem.Inst.CreateWindowView(UISummerExplainView);
            }
            else if (btn == this.btn_shuaxin) {
                this.OnClickShuaXing();
            }
        }
        OnClickShuaXing() {
            if (!StorageUtils.Inst.isShuaXing) {
                WarOrderModel.Inst.RefreshTask();
                this.UpdateTask();
                StorageUtils.Inst.isShuaXing = true;
            }
        }
        OnUpdateTime() {
            if (!StorageUtils.Inst.isShuaXing) {
                this.btn_shuaxin.text = "刷新";
                this.btn_shuaxin.enabled = true;
                return;
            }
            this.btn_shuaxin.enabled = false;
            this.date.setTime(TimeModel.Inst.LocalTime);
            let hours = 23 - this.date.getHours();
            let min = 59 - this.date.getMinutes();
            let sec = 59 - this.date.getSeconds();
            let offestX = hours < 10 ? "0" : "";
            let offestY = min < 10 ? "0" : "";
            let offestZ = sec < 10 ? "0" : "";
            this.btn_shuaxin.text = `${offestX}${hours}:${offestY}${min}:${offestZ}${sec}`;
        }
        RemoveListener() {
        }
        onResize() {
            super.onResize();
            this._view.setSize(Laya.stage.width, Laya.stage.height);
        }
        Reset() {
            super.Reset();
            Laya.timer.clear(this, this.OnUpdateTime);
        }
    }

    class UIGrowthGiftBagItem extends TListItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
            this.item.isYellow = false;
            this.item2.isYellow = true;
            this.bar = this.probar.getChild("bar").asCom;
            this.proMsk = this.bar.getChild("msk").asImage;
        }
        InitData(data) {
            super.InitData(data);
            this.UpdateData();
        }
        AddListener() {
            super.AddListener();
            this.item.onClick(this, this.onGetAward);
            this.item2.onClick(this, this.onGetExtraAward);
            Laya.timer.frameLoop(2, this, this.render);
            GameDispatcher.Inst.on(UIGameEvent.UPDATE_GROTH_COUNT, this, this.onUpdateGrothCount);
        }
        RemoveListener() {
            super.RemoveListener();
            this.item.offClick(this, this.onGetAward);
            this.item2.offClick(this, this.onGetExtraAward);
            Laya.timer.clear(this, this.render);
            GameDispatcher.Inst.off(UIGameEvent.UPDATE_GROTH_COUNT, this, this.onUpdateGrothCount);
        }
        onUpdateGrothCount() {
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                let p3 = AdModel.Inst.getPolicy("p3");
                let bol = p3 == "yes" && AdModel.Inst.gotGrothCount >= 4;
                this.item.isAd = bol;
            }
        }
        onGetExtraAward() {
            let vo = this.Data;
            if (!vo)
                return;
            if (!vo.canGet)
                return;
            let handler = Laya.Handler.create(this, (success) => {
                if (success) {
                    let awardArr = [];
                    awardArr.push(vo.exAward);
                    this.GetAward(awardArr);
                }
            });
            PlatformMgr.Inst.showAd(AdType.ad_22, handler);
        }
        onGetAward() {
            if (!(this.Data instanceof GrowthGiftPackVO) || !this.Data.canGet)
                return;
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                AdCtrl.Inst.addGrothCount();
            }
            this.GetAward(null);
        }
        GetAward(arr = null) {
            let vo = this.Data;
            if (!vo)
                return;
            arr = arr || [];
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                let p3 = AdModel.Inst.getPolicy("p3");
                let bol = p3 == "yes";
                if (AdModel.Inst.gotGrothCount >= 4 && bol) {
                    PlatformMgr.Inst.showAd(AdType.ad_22, Laya.Handler.create(this, this.GetAward2, [vo, arr]));
                }
                else {
                    this.GetAward2(vo, arr, true);
                }
            }
            else {
                this.GetAward2(vo, arr, true);
            }
        }
        GetAward2(vo, arr, isSuccess) {
            if (isSuccess && vo.canGet) {
                vo.GetAward(arr.length > 0);
                UITaskView.itemVo.itemId = vo.growUpRes.item;
                UITaskView.itemVo.count = vo.growUpRes.num;
                arr.push(UITaskView.itemVo);
                UISystem.Inst.CreateWindowView(UIAwardView, arr);
                this.UpdateData();
            }
        }
        render() {
            let vo = this.Data;
            if (!vo) {
                return;
            }
            let max = MathUtil.bignumerToString(vo.canGetScore);
            let value = MathUtil.bignumerToString(vo.curPro);
            let bigNum = vo.curPro.dividedBy(vo.canGetScore).multipliedBy(10000).toFixed();
            let pre = parseInt(bigNum) / 10000;
            pre = pre > 1 ? 1 : pre;
            this.probar.title = value + "/" + max;
            this.proMsk.width = Math.ceil(this.probar.width * pre);
            this.bar.visible = this.proMsk.width >= 1;
        }
        UpdateData() {
            let vo = this.Data;
            if (!vo)
                return;
            let growUpRes = vo.growUpRes;
            this.txt_title.text = `金币收益达到每10s获得${MathUtil.goldFormat(vo.canGetScore)}金币`;
            this.render();
            let item = itemResource.getItemByKey(growUpRes.item);
            let iconName = item.icon;
            let url;
            if (item.type == 3) {
                url = UrlUtils.GetIconPartUrl(iconName);
            }
            else {
                url = UrlUtils.GetIconItemsUrl(iconName);
            }
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                this.item.icon = url;
            }));
            this.item.title = MathUtil.goldFormat(growUpRes.num) + "";
            this.item.SetNum(MathUtil.goldFormat(growUpRes.num) + "");
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                let p3 = AdModel.Inst.getPolicy("p3");
                let bol = p3 == "yes" && AdModel.Inst.gotGrothCount >= 4;
                this.item.isAd = bol;
            }
            else {
                this.item.isAd = false;
            }
            this.item.SetState(vo.state);
            this.SetState(vo.state);
            let arr = GiftPKGModel.Inst.GetGrowthGiftVos();
            let index = arr.indexOf(vo);
            if (index != null) {
                if (index % 2 == 0) {
                    this.c2.selectedIndex = 1;
                }
                else {
                    this.c2.selectedIndex = 2;
                }
            }
            let item2 = itemResource.getItemByKey(growUpRes.extra);
            let iconName2 = item2.icon;
            let url2;
            if (item.type == 3) {
                url2 = UrlUtils.GetIconPartUrl(iconName2);
            }
            else {
                url2 = UrlUtils.GetIconItemsUrl(iconName2);
            }
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                this.item2.icon = url2;
            }));
            this.item2.title = MathUtil.goldFormat(growUpRes.extranumber) + "";
            this.item2.SetNum(MathUtil.goldFormat(growUpRes.extranumber) + "");
            this.item2.isAd = true;
            this.item2.SetState(vo.state);
        }
        SetState(state) {
            if (state == BaseTaskVO.STATE_NORMAL) {
                this.c1.selectedIndex = 0;
            }
            else if (state == BaseTaskVO.STATE_CAN_AWARD) {
                this.c1.selectedIndex = 1;
            }
            else if (state == BaseTaskVO.STATE_COMPLETED) {
                this.c1.selectedIndex = 2;
            }
        }
        Reset() {
            super.Reset();
            Laya.timer.clearAll(this);
        }
    }

    class UIGrowthGiftPackView extends UIBaseWindow {
        constructor() {
            super("growthGiftPack", "UIGrowthGiftPackView");
            this.eglist_items = null;
            this.btn_cat = null;
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("growthGiftPack", "UIGrowthGiftBagItem", UIGrowthGiftBagItem);
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_CENTER;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isEffect = true;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            this.btn_cat.play("czlb_cat");
            this.eglist_items.Data = GiftPKGModel.Inst.GetGrowthGiftVos();
            if (Laya.Browser.onHWMiniGame) {
                this.hideBanner();
                PlatformMgr.Inst.showAd(AdType.ad_3);
            }
            else if (Laya.Browser.onQQMiniGame) {
                this.hideBanner();
                PlatformMgr.Inst.showAd(AdType.ad_3);
            }
            else if (PlatformMgr.Inst.isMZ) {
                this.hideBanner();
                PlatformMgr.Inst.showAd(AdType.ad_3);
            }
            else if ((Laya.Browser.onVVMiniGame || Laya.Browser.onKGMiniGame) && GuideModel.Inst.guideVO == null) {
                this.hideBanner();
                AdCtrl.Inst.DelayShowInsertAd();
            }
            else {
                this.showBanner();
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
        }
        Reset() {
            super.Reset();
            AdModel.Inst.gotGrothCount = 0;
        }
    }

    class ChoutiItem extends TComponent {
        constructor() {
            super();
            this.endH = 0;
        }
        InitFairyGui() {
            super.InitFairyGui();
            FairyUtils.setVar(this, this);
            FairyUtils.setVar(this.leftBtns, this);
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            this.btn_table_c1 = this.btn_table.getController("c1");
            this.IsShowBtns();
        }
        InitData(data) {
            super.InitData(data);
        }
        get btnsComp() {
            return this.leftBtns;
        }
        AddListener() {
            super.AddListener();
            this.btn_table.onClick(this, this.OnClickTable);
        }
        OnClickTable() {
            this.IsShowBtns(1 - this.btn_table_c1.selectedIndex);
        }
        IsShowBtns(val = 0) {
            let startH;
            let endH;
            if (val == 1) {
                this.sortBtns();
                startH = this.n3.height;
                endH = this.endH;
                ;
            }
            else {
                startH = this.n3.height;
                endH = 0;
            }
            this.btn_table_c1.selectedIndex = val;
            fgui.GTween.kill(this.n3);
            fgui.GTween.to(startH, endH, 0.1).setEase(fgui.EaseType.Linear).setTarget(this.n3).onUpdate((tw) => {
                this.n3.height = tw.value.x;
            });
        }
        sortBtns() {
            let btn;
            let prev;
            let height = 0;
            for (let i = 0; i < this.leftBtns.numChildren; i++) {
                let child = this.leftBtns.getChildAt(i);
                if (child instanceof fgui.GButton) {
                    btn = child;
                    if (i == 0) {
                        prev = btn;
                        height += btn.height;
                    }
                    else if (btn.visible) {
                        height = height + btn.height + 20;
                        btn.y = prev.y + prev.height * (1 - prev.pivotY) + btn.height * btn.pivotY + 20;
                        prev = btn;
                    }
                }
            }
            if (prev)
                this.endH = height + 25;
            if (this.btn_table_c1.selectedIndex == 1) {
                this.n3.height = this.endH;
            }
        }
        RemoveListener() {
            super.RemoveListener();
            this.btn_table.offClick(this, this.OnClickTable);
        }
        Reset() {
            super.Reset();
        }
    }

    class CatFecalBallScript extends EntityBaseScript {
        constructor() {
            super();
        }
        onAwake() {
            super.onAwake();
        }
        SetVO(vo) {
            super.SetVO(vo);
        }
        onEnable() {
            super.onEnable();
        }
        Play() {
        }
        get target() {
            return this.owner;
        }
        PlayEffect() {
            let effectName = this.catVO.catRes.effcatball;
            let cat = this.catVO && CatFecalBallModel.Inst.GetFecal(this.catVO.objId);
            if (cat && cat.isSpeacil) {
                effectName = this.catVO.catRes.specialballEff;
            }
            if (effectName) {
                let url = UrlUtils.GetEffectUrl(effectName);
                LoadQueue.Inst.loadParticle(url, Laya.Handler.create(this, this.LoadParticle));
            }
        }
        LoadParticle(sp) {
            if (!sp || sp.transform == null)
                return;
            this.effectSp = sp;
            SceneMgr.Inst.AddEffect(sp.name, sp);
            let position = sp.transform.position;
            this.transform.position.cloneTo(position);
            position.y += 0.0345;
            sp.transform.position = position;
        }
        onClick() {
            let cat = this.catVO && CatFecalBallModel.Inst.GetFecal(this.catVO.objId);
            if (cat) {
                CatFecalBallCtrl.Inst.ClickFecal(cat);
            }
        }
        get catVO() {
            return this.vo;
        }
        onReset() {
            super.onReset();
        }
        onDestroy() {
            if (this.effectSp) {
                if (!this.effectSp.destroyed)
                    this.effectSp.destroy();
                this.effectSp = null;
            }
        }
    }

    class TouchPointID {
        constructor() {
            this.touchId = -1;
            this.touchDownX = 0;
            this.touchDownY = 0;
            this.touchMoveX = 0;
            this.touchMoveY = 0;
        }
    }
    class UICameraMoveScript extends TComponent {
        constructor() {
            super();
            this.touchPosArr = null;
            this.dic = new Dictionary();
            this._downPos = new Laya.Point();
            this._ray = new Laya.Ray(new Laya.Vector3(0, 0, 0), new Laya.Vector3(0, 0, 0));
            this.pos = new Laya.Vector2();
            this.outs = new Array();
        }
        InitUI() {
            this.touchPosArr = [new TouchPointID(), new TouchPointID()];
            Laya.MouseManager.multiTouchEnabled = true;
        }
        AddListener() {
            super.AddListener();
            this.on(Laya.Event.MOUSE_DOWN, this, this.onTouchDown);
            this.on(Laya.Event.MOUSE_OUT, this, this.OnTouchUp);
            this.on(Laya.Event.ROLL_OUT, this, this.OnTouchUp);
            this.onClick(this, this.onTouchClick);
            this.on(Laya.Event.MOUSE_WHEEL, this, this.mousewheel);
            this.AddGameListener(UIGameEvent.ON_ACTIVE, this.onActive, this);
        }
        mousewheel(e) {
            let camera = SceneMgr.Inst.GetCameraScript();
            camera.OnChangeScale(e.delta * 10);
        }
        onActive(isActive) {
            if (!isActive) {
                this.Clear();
            }
        }
        Clear() {
            for (const key in this.dic) {
                this.touchPosArr.push(this.dic[key]);
                delete this.dic[key];
            }
            this.off(Laya.Event.MOUSE_MOVE, this, this.OnTouchMove);
            this.off(Laya.Event.MOUSE_UP, this, this.OnTouchUp);
        }
        RemoveListener() {
            super.RemoveListener();
            this.off(Laya.Event.MOUSE_DOWN, this, this.onTouchDown);
            this.off(Laya.Event.MOUSE_MOVE, this, this.OnTouchMove);
            this.off(Laya.Event.MOUSE_UP, this, this.OnTouchUp);
            this.off(Laya.Event.MOUSE_OUT, this, this.OnTouchUp);
            this.off(Laya.Event.ROLL_OUT, this, this.OnTouchUp);
            this.offClick(this, this.onTouchClick);
            this.off(Laya.Event.MOUSE_WHEEL, this, this.mousewheel);
        }
        onTouchDown(evt) {
            if (!evt || evt.touchId == -1)
                return;
            let touchPos = this.dic[evt.touchId];
            if (!touchPos) {
                if (this.touchPosArr.length == 0)
                    return;
                touchPos = this.touchPosArr.pop();
                this.dic[evt.touchId] = touchPos;
            }
            if (touchPos) {
                touchPos.touchDownX = Laya.stage.mouseX;
                touchPos.touchDownY = Laya.stage.mouseY;
                touchPos.touchMoveX = Laya.stage.mouseX;
                touchPos.touchMoveY = Laya.stage.mouseY;
                this._downPos.setTo(touchPos.touchDownX, touchPos.touchDownY);
            }
            this.off(Laya.Event.MOUSE_MOVE, this, this.OnTouchMove);
            this.off(Laya.Event.MOUSE_UP, this, this.OnTouchUp);
            this.on(Laya.Event.MOUSE_MOVE, this, this.OnTouchMove);
            this.on(Laya.Event.MOUSE_UP, this, this.OnTouchUp);
            SceneMgr.Inst.GetCameraScript().ClearResetAni();
        }
        OnTouchUp(evt) {
            this.off(Laya.Event.MOUSE_MOVE, this, this.OnTouchMove);
            this.off(Laya.Event.MOUSE_UP, this, this.OnTouchUp);
            if (evt.touchId == -1)
                return;
            let touchPos = this.dic[evt.touchId];
            if (touchPos) {
                this.touchPosArr.push(touchPos);
                delete this.dic[evt.touchId];
            }
            if (this.GetTouchLength() == 0)
                SceneMgr.Inst.GetCameraScript().CameraRestTween();
        }
        GetTouchLength() {
            let length = 0;
            for (const key in this.dic) {
                length++;
            }
            return length;
        }
        OnTouchMove(evt) {
            if (GuideModel.Inst.guideVO && (GuideModel.Inst.guideVO.isClickCat || GuideModel.Inst.guideVO.isClickBuild)) {
                return;
            }
            let owner = this["$owner"];
            if (owner && !owner.IsCanTouch()) {
                return;
            }
            let touchPos = this.dic[evt.touchId];
            if (!touchPos) {
                return;
            }
            touchPos.touchMoveX = Laya.stage.mouseX;
            touchPos.touchMoveY = Laya.stage.mouseY;
            let camera = SceneMgr.Inst.GetCameraScript();
            if (!camera)
                return;
            if (this.GetTouchLength() < 2) {
                camera.OnChangeMove(touchPos.touchMoveX - touchPos.touchDownX, touchPos.touchMoveY - touchPos.touchDownY);
            }
            else {
                let onTouch = null;
                for (const key in this.dic) {
                    let curTouchPos = this.dic[key];
                    if (!curTouchPos)
                        continue;
                    if (onTouch == null) {
                        onTouch = curTouchPos;
                    }
                    else {
                        let downDis = MathUtil.distance(curTouchPos.touchDownX, curTouchPos.touchDownY, onTouch.touchDownX, onTouch.touchDownY);
                        let moveDis = MathUtil.distance(curTouchPos.touchMoveX, curTouchPos.touchMoveY, onTouch.touchMoveX, onTouch.touchMoveY);
                        camera.OnChangeScale(downDis - moveDis);
                    }
                }
            }
            if (touchPos) {
                touchPos.touchDownX = Laya.stage.mouseX;
                touchPos.touchDownY = Laya.stage.mouseY;
            }
        }
        onTouchClick() {
            let owner = this["$owner"];
            if (owner && !owner.IsCanTouch()) {
                return;
            }
            let distance = MathUtil.distance(this._downPos.x, this._downPos.y, Laya.stage.mouseX, Laya.stage.mouseY);
            if (distance > 20)
                return;
            this.pos.setValue(Laya.stage.mouseX * Laya.stage.clientScaleX, Laya.stage.mouseY * Laya.stage.clientScaleY);
            let camera = SceneMgr.Inst.GetCamera();
            if (!camera)
                return;
            camera && camera.viewportPointToRay(this.pos, this._ray);
            let isTrigger = SceneMgr.Inst.GetScene().physicsSimulation.rayCastAll(this._ray, this.outs, 1000, this.GetCollisonGroup(), this.GetCanCollideWith());
            if (!isTrigger)
                return;
            let fecalScript = this.GetHasCollision(this.outs, CatFecalBallScript);
            if (fecalScript) {
                fecalScript.onClick();
                return;
            }
            let giftScript = this.GetHasCollision(this.outs, CatGiftScript);
            if (giftScript) {
                giftScript.onClick();
                return;
            }
            let catScript = this.GetHasCollision(this.outs, CatScript);
            if (catScript != null && catScript.vo instanceof CatVO) {
                CatCtrl.Inst.ClickCatOnMainView(catScript.vo);
                return;
            }
            let basinScript = this.GetHasCollision(this.outs, CatBasinScript);
            if (basinScript) {
                basinScript.AddFish();
                return;
            }
            let buildScript = this.GetHasCollision(this.outs, BuildScript);
            if (buildScript != null && buildScript.vo != null) {
                if (GuideModel.Inst.guideVO) {
                    if (GuideModel.Inst.guideVO.isClickCat) {
                        return;
                    }
                    else if (buildScript.buildVO.buildRes.id == 101 && (GuideModel.Inst.guideVO.page < 6 && GuideModel.Inst.guideVO.guideRes.page != 2)) {
                        return;
                    }
                    else if (buildScript.buildVO.isMatou && GuideModel.Inst.guideVO.guideRes.id < GuideModel.GUID_CLICK_MATOU_ID) {
                        return;
                    }
                }
                UISystem.Inst.CreateWindowView(UIBuildView, buildScript.vo);
                GameDispatcher.Inst.event(UIGameEvent.CLICK_BUILD, buildScript.buildVO);
                return;
            }
            this.outs.length = 0;
        }
        GetHasCollision(outs, cls) {
            for (const out of outs) {
                if (out.succeeded) {
                    let sc = out.collider.owner.getComponent(cls);
                    if (sc != null)
                        return sc;
                }
            }
            return null;
        }
        GetCollisonGroup() {
            return CollisionType.TYPE_RAY_CLICK;
        }
        GetCanCollideWith() {
            return CollisionType.TYPE_BUILD | CollisionType.TYPE_CAT | CollisionType.TYPE_ENTITY;
        }
    }

    class UIDoubleAwardView extends UIBaseWindow {
        constructor() {
            super("main", "UIDoubleAwardView");
            this.btn_get = null;
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = LayerType.LAYOUT_CENTER;
            this.viewStruct.isShowMaskBg = true;
            this.viewStruct.isEffect = true;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            if ((Laya.Browser.onVVMiniGame || Laya.Browser.onKGMiniGame) && GuideModel.Inst.guideVO == null) {
                this.hideBanner();
                AdCtrl.Inst.DelayShowInsertAd();
            }
            else if (Laya.Browser.onQGMiniGame) {
                this.hideBanner();
                AdCtrl.Inst.ShowNatiEmbed(this);
            }
            else {
                this.showBanner();
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_get) {
                this.OnClickGetBtn();
            }
        }
        OnClickGetBtn() {
            PlatformMgr.Inst.showAd(AdType.ad_14, Laya.Handler.create(this, this.GetAward));
        }
        GetAward(isSuccess = true) {
            if (!isSuccess)
                return;
            GameDispatcher.Inst.event(UIGameEvent.ONGETDOUBLEAWARD);
            this.close();
        }
        Reset() {
            super.Reset();
        }
    }

    class ToolTip {
        static show(data, pos = null, url = "") {
            url = url || fgui.UIConfig.tooltipsWin;
            let tipView = ToolTip.tipPool[url];
            if (tipView == null) {
                tipView = fgui.UIPackage.createObjectFromURL(url).asLabel;
                ToolTip.tipPool[url] = tipView;
            }
            tipView.text = data;
            fgui.GRoot.inst.showTooltipsWin(tipView, pos);
        }
    }
    ToolTip.tipPool = {};

    class EButton extends TBaseButton {
        constructor() {
            super();
            this.m_param = null;
            this.playSound = true;
        }
        breath(range = 1.2) {
            fgui.GTween.to(1, range, 0.6).setTarget(this).setEase(fgui.EaseType.SineInOut).setRepeat(-1, true)
                .onUpdate((tw) => {
                this.setScale(tw.value.x, tw.value.x);
            }, this);
        }
        stopBreath() {
            this.setScale(1, 1);
            fgui.GTween.kill(this);
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            if (this._iconObject) {
                this._iconObject.touchable = false;
            }
            if (this._titleObject) {
                this._titleObject.touchable = false;
            }
        }
        InitData(param) {
            super.InitData(param);
            this.m_param = param;
        }
        get Data() {
            return this.m_param;
        }
        setTooltips(value) {
            this.off(Laya.Event.ROLL_OVER, this, this.rollOver);
            this.off(Laya.Event.ROLL_OUT, this, this.rollOut);
            this["_tooltips"] = value;
            if (this["_tooltips"]) {
                this.on(Laya.Event.ROLL_OVER, this, this.rollOver);
                this.on(Laya.Event.ROLL_OUT, this, this.rollOut);
            }
        }
        rollOver() {
            Laya.timer.once(100, this, this.showTooltip);
        }
        rollOut() {
            Laya.timer.clear(this, this.showTooltip);
            fgui.GRoot.inst.hideTooltips();
        }
        showTooltip(pos = null) {
            if (fgui.UIConfig.tooltipsWin) {
                ToolTip.show(this["_tooltips"], pos);
            }
        }
        set selectedPage(value) {
            this.c1 = this.c1 || this.getController("c1");
            if (this.c1 != null) {
                this.c1.selectedPage = value;
            }
            this.updateState();
        }
        get selectedPage() {
            return this.c1 && this.c1.selectedPage;
        }
        set selectedIndex(value) {
            if (this.c1) {
                this.c1.selectedIndex = value;
            }
            this.updateState();
        }
        get selectedIndex() {
            return this.c1 ? this.c1.selectedIndex : 0;
        }
        updateState() {
        }
        toTop(child) {
            child = child || this;
            if (this.numChildren > 1 && this.getChildIndex(child) != -1) {
                this.setChildIndex(child, this.numChildren - 1);
            }
        }
        toBottom(child) {
            child = child || this;
            if (this.getChildIndex(child) != -1) {
                this.setChildIndex(child, 0);
            }
        }
        get allVisible() {
            return this.internalVisible && this.internalVisible2 && this.internalVisible3;
        }
        showScaleEffect(btn) {
            if (btn == null)
                return;
            fgui.GTween.to3(0, 0, 0, 1, 1, 1, 0.3).setTarget(btn)
                .setEase(fgui.EaseType.SineInOut)
                .onUpdate(function onUpdate(tw) {
                btn.setScale(tw.value.x, tw.value.y);
                btn.alpha = tw.value.z;
            })
                .onComplete(function onComplete() {
            });
        }
        get privoGlobalPoint() {
            return this.localToGlobal(Math.ceil(this.pivotX * this.width), Math.ceil(this.pivotY * this.height));
        }
        RemoveListener() {
            super.RemoveListener();
            this.off(Laya.Event.ROLL_OVER, this, this.rollOver);
            this.off(Laya.Event.ROLL_OUT, this, this.rollOut);
        }
        Reset() {
            super.Reset();
            this.m_param = null;
            fgui.GTween.kill(this);
            Laya.Tween.clearAll(this);
        }
    }

    class UIFuncTipComp extends EButton {
        constructor() {
            super();
            this.prevTime = 0;
            this.moneyVo = new MoneyVo(MoneyVo.GOLD);
        }
        InitUI() {
            super.InitUI();
            this.moneyComp.isNoBg = true;
        }
        InitData(data) {
            super.InitData(data);
            this.visible = this.touchable = true;
            if (data instanceof BuildVO) {
                this.title = data.buildRes.name;
                this.txt_level.text = data.level + "级";
                let url = UrlUtils.GetIconBuildUrl(data.buildRes.icon);
                PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                    this.icon = url;
                }));
                let costGold = data.GetLvUpCostGold();
                this.moneyVo.setVal(costGold);
                this.moneyComp.InitData(this.moneyVo);
            }
            this.onClick(this, this.onClickLevelUpBuild);
            this.on(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            this.on(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            this.on(Laya.Event.MOUSE_OUT, this, this.onMouseUp);
        }
        onMouseDown(e) {
            Laya.timer.clear(this, this.loopClick);
            Laya.timer.once(300, this, this.onceDown);
        }
        onceDown() {
            if (GlobalConfig.isOpenAutoTouch) {
                Laya.timer.loop(GlobalConfig.autoTouchTimeGap, this, this.loopClick);
            }
        }
        onMouseUp(e) {
            Laya.timer.clear(this, this.loopClick);
            Laya.timer.clear(this, this.onceDown);
        }
        loopClick() {
            if (ObjectUtils.globalIsIn(this)) {
                this.onClickLevelUpBuild();
            }
        }
        static get clickCount() {
            if (!AdCtrl.Inst.wxIsLevelUp5)
                return 0;
            return UIFuncTipComp._clickCount;
        }
        static set clickCount(val) {
            UIFuncTipComp._clickCount = val;
        }
        onClickLevelUpBuild() {
            if (this.isShowAd) {
                this.onLevelUp5();
            }
            else {
                if (this.Data instanceof BuildVO && !this.Data.IsMaxLv()) {
                    BuildCtrl.Inst.BuildLevelUp(this.Data);
                }
                if (this.prevTime > 0 && (Laya.timer.currTimer - this.prevTime) > 200) {
                    UIFuncTipComp.clickCount = 0;
                }
                UIFuncTipComp.clickCount++;
                if (UIFuncTipComp.clickCount >= 5) {
                    this.isShowAd = true;
                }
            }
        }
        onLevelUp5() {
            BuildCtrl.Inst.BuildLevelUp(this.Data, 5, true, Laya.Handler.create(this, this.recover));
        }
        recover(isSuccess) {
            if (isSuccess) {
                UIFuncTipComp.clickCount = 0;
                this.isShowAd = false;
            }
        }
        backFromOther() {
            this.recover(true);
        }
        set isShowAd(value) {
            this.c1.selectedIndex = value ? 1 : 0;
        }
        get isShowAd() {
            return this.c1.selectedIndex == 1;
        }
        Reset() {
            super.Reset();
            this.visible = this.touchable = false;
            this.offClick(this, this.onClickLevelUpBuild);
            this.off(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            this.off(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            this.off(Laya.Event.MOUSE_OUT, this, this.onMouseUp);
            Laya.timer.clear(this, this.loopClick);
            Laya.timer.clear(this, this.onceDown);
            UIFuncTipComp.clickCount = 0;
            this.isShowAd = false;
        }
    }
    UIFuncTipComp._clickCount = 0;

    class UILogoView extends UIBaseWindow {
        constructor() {
            super("main", "UILogoView");
            this.sp_logo = null;
            this.sp_bgg = null;
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_WINDOW;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            super.InitUI();
        }
        SetAniEnd(delay = 0) {
            Laya.Tween.to(this.sp_logo, { alpha: 0 }, 500, null, Laya.Handler.create(this, this.close), delay);
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
            this.sp_logo.alpha = 0.5;
            this.sp_bgg.alpha = 0.7;
            Laya.Tween.to(this.sp_bgg, { alpha: 0 }, 2500);
            Laya.Tween.to(this.sp_logo, { alpha: 1 }, 1000);
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
        }
        Reset() {
            super.Reset();
        }
    }

    class UITouchBallScript extends TComponent {
        constructor() {
            super();
            this._ray = new Laya.Ray(new Laya.Vector3(0, 0, 0), new Laya.Vector3(0, 0, 0));
            this.pos = new Laya.Vector2();
            this.outs = new Array();
        }
        InitUI() {
        }
        AddListener() {
            super.AddListener();
            this.onClick(this, this.onTouchClick);
        }
        RemoveListener() {
            super.RemoveListener();
            this.offClick(this, this.onTouchClick);
        }
        onTouchClick() {
            this.pos.setValue(Laya.stage.mouseX * Laya.stage.clientScaleX, Laya.stage.mouseY * Laya.stage.clientScaleY);
            let camera = SceneMgr.Inst.GetCamera();
            if (!camera)
                return;
            camera && camera.viewportPointToRay(this.pos, this._ray);
            let isTrigger = SceneMgr.Inst.GetScene().physicsSimulation.rayCastAll(this._ray, this.outs, 1000, this.GetCollisonGroup(), this.GetCanCollideWith());
            if (!isTrigger)
                return;
            let fecalScript = this.GetHasCollision(this.outs, CatFecalBallScript);
            if (fecalScript) {
                fecalScript.onClick();
                return;
            }
            let giftScript = this.GetHasCollision(this.outs, CatGiftScript);
            if (giftScript) {
                giftScript.onClick();
                return;
            }
            this.outs.length = 0;
        }
        GetHasCollision(outs, cls) {
            for (const out of outs) {
                if (out.succeeded) {
                    let sc = out.collider.owner.getComponent(cls);
                    if (sc != null)
                        return sc;
                }
            }
            return null;
        }
        GetCollisonGroup() {
            return CollisionType.TYPE_RAY_CLICK;
        }
        GetCanCollideWith() {
            return CollisionType.TYPE_ENTITY;
        }
    }

    class UIMainView extends UIBaseWindow {
        constructor() {
            super("main", "UIMainView", UrlUtils.UI);
            this.btn_task = null;
            this.btn_miaoxingren = null;
            this.btn_daoyu = null;
            this.btn_beibao = null;
            this.btn_baike = null;
            this.btn_duihuansuo = null;
            this.btn_sign = null;
            this.btn_table = null;
            this.btn_matou = null;
            this.btn_shuangbei = null;
            this.btn_shangdian = null;
            this.btn_growUp = null;
            this.txt_gold = null;
            this.txt_diamond = null;
            this.btn_doubleani = null;
            this.txt_addGold = null;
            this.txt_rate = null;
            this.btn_huodong = null;
            this.btn_bottle = null;
            this.sp_touch = null;
            this.spine_dian = null;
            this.selectBuild = null;
            this.btn_libao = null;
            this.btn_lixian = null;
            this.date = TimeModel.Inst.getDate();
            this.hideTime = 0;
            this.showTime = 0;
            this.templete = null;
            this.clickAni = null;
            this.mainGuideCom = false;
            this.out = new Laya.Vector3();
            this.initCatBasinDis = NaN;
            this.goldP = new Laya.Point();
            this.gemP = new Laya.Point();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.layerType = LayerType.TYPE_MAIN;
            this.viewStruct.isShowMaskBg = false;
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("main", "RedButton", RedButton);
            PanelRegister.registerClass("main", "UITouchBallScript", UITouchBallScript);
            PanelRegister.registerClass("main", "UICameraMoveScript", UICameraMoveScript);
            PanelRegister.registerClass("main", "UIFuncTipComp", UIFuncTipComp);
            PanelRegister.registerClass("main", "UISceneIconView", UISceneIconView);
            PanelRegister.registerClass("main", "UISceneIconItem", UISceneIconItem);
            PanelRegister.registerClass("main", "UISceneCatPenIconItem", UISceneCatPenIconItem);
            PanelRegister.registerClass("main", "UISceneCatIcon", UISceneCatIcon);
            PanelRegister.registerClass("main", "UISceneCanUnlockIcon", UISceneCanUnlockIcon);
            PanelRegister.registerClass("main", "UISceneLandPromptIcon", UISceneLandPromptIcon);
            PanelRegister.registerClass("main", "UIFlyCoinItem", UIFlyCoinItem);
            PanelRegister.registerClass("main", "FlyStar", FlyStar);
            PanelRegister.registerClass("main", "UIHTItem2", UIHTItem2);
            PanelRegister.registerClass("main", "ChoutiItem", ChoutiItem);
        }
        InitUI() {
            super.InitUI();
            FairyUtils.setVar(this.btnChouti_leftTop.btnsComp, this, this._btns);
            this.btnChouti_leftTop.IsShowBtns(1);
            this.sp_rightRect.visible = false;
            this.group_tip.visible = false;
            this.c1 = this._view.getController("c1");
            this.c2 = this._view.getController("c2");
            this.c3 = this._view.getController("c3");
            this.t0 = this._view.getTransition("t0");
            this.t1 = this._view.getTransition("t1");
            this.t2 = this._view.getTransition("t2");
            this.t3 = this._view.getTransition("t3");
            this.t5 = this._view.getTransition("t5");
            this.t6 = this._view.getTransition("t6");
            this.btn_moreGame.setRed(1);
            SkinModel.Inst.init();
            ItemModel.Inst.init();
            TaskModel.Inst.init();
            ShopModel.Inst.init();
            CatFecalBallCtrl.Inst.init();
            GuideCtrl.Inst.init();
            let time = CVModel.Inst.GetValue(CVModel.goldtime, CVModel.valType_number);
            this.btn_shuangbei.title = Math.floor(time / 60) + "分钟";
            this.btn_camera.visible = false;
            this.txt_guide.visible = this.btn_gm.visible = GlobalConfig.isUseGm;
            this.sp_touch.touchable = false;
            this.sp_touch["$owner"] = this;
            SoundMgr.Inst.playMusic(UrlUtils.GetMusic(MusicType.BGM));
            this.btn_doubleani.Reset();
            this.btn_fengxiang.visible = false;
            if (PlatformMgr.Inst.isWX) {
                this.c1.selectedPage = "wx";
            }
            else if (Laya.Browser.onQGMiniGame) {
                this.c1.selectedPage = "oppo";
            }
            else if (Laya.Browser.onHWMiniGame) {
                this.c1.selectedPage = "hw";
            }
            else if (Laya.Browser.onQQMiniGame) {
                this.c1.selectedPage = "qq";
            }
            this.ad_grid.Reset();
            this.btn_club.Reset();
            this.sortRightBtns();
        }
        UpdateBtnTime() {
            if (DriftingBottleModel.Inst.driftingBottleState == DriftingBottleModel.yaoQingCompState) {
                this.btn_bottle.visible = false;
                Laya.timer.clear(this, this.UpdateBtnTime);
            }
            else if (DriftingBottleModel.Inst.driftingBottleState == DriftingBottleModel.shangdaoState) {
                this.date.setTime(TimeModel.Inst.LocalTime);
                let hours = 23 - this.date.getHours();
                let min = 59 - this.date.getMinutes();
                let sec = 59 - this.date.getSeconds();
                let offestX = hours < 10 ? "0" : "";
                let offestY = min < 10 ? "0" : "";
                let offestZ = sec < 10 ? "0" : "";
                this.btn_bottle.title = `${offestX}${hours}:${offestY}${min}:${offestZ}${sec}`;
            }
            else {
                this.btn_bottle.title = "";
            }
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.UPDATE_CURRENCY_DATA, this.UpdateCurrency, this);
            this.AddGameListener(UIGameEvent.UPDATE_GOLD_RATE, this.UpdateGoldRate, this);
            this.AddGameListener(UIGameEvent.CAT_UPDATE_QINMI, this.onCatQinmiUpdate, this);
            this.AddGameListener(UIGameEvent.CAT_WEAR_CLOTH, this.onCatWearCloth, this);
            this.AddGameListener(UIGameEvent.CAT_TAKEOFF_CLOTH, this.onTakeOffCloth, this);
            this.AddGameListener(UIGameEvent.CAT_STATE_STROKE_CHANGE, this.onCatStrokeChange, this);
            this.AddGameListener(UIGameEvent.CAT_START_ONLAND, this.onStartOnland, this);
            this.AddGameListener(UIGameEvent.CAT_COMPLETE_ONLAND, this.onCompleteOnland, this);
            this.AddGameListener(UIGameEvent.ON_SHOW, this.onShow, this);
            this.AddGameListener(UIGameEvent.ON_HIDE, this.onHide, this);
            this.AddGameListener(UIGameEvent.HIDE_BUILD_PROBAR, this.hideProbar, this);
            this.AddGameListener(UIGameEvent.CLICK_BUILD, this.onClickBuild, this);
            this.AddGameListener(UIGameEvent.SCENE_RESIZE, this.onSceneResize, this);
            this.AddGameListener(UIGameEvent.CAMERA_MOVE_COMPLETE, this.onCameraMoveComplete, this);
            this.AddGameListener(UIGameEvent.START_MOVE_HOUSE, this.onStartMoveHouse, this);
            this.AddGameListener(UIGameEvent.END_MOVE_HOUSE, this.onEndMoveHouse, this);
            this.AddGameListener(UIGameEvent.SHOW_TASKBTN_RED, this.onShowTaskBtnRed, this);
            this.AddGameListener(UIGameEvent.SHOW_SIGINBTN_RED, this.onShowSignBtnRed, this);
            this.AddGameListener(UIGameEvent.SHOW_BAGBTN_RED, this.onShowBagBtnRed, this);
            this.AddGameListener(UIGameEvent.BUILD_ADD_OR_REMOVE_ICON, this.onShowMaTouBtnState, this);
            GameDispatcher.Inst.event(UIGameEvent.SHOW_SIGINBTN_RED, LoginModel.Inst.canSign || LoginModel.Inst.canGetDima);
            this.AddGameListener(UIGameEvent.BLACK_TO_WHITE, this.onBlackToWhite, this);
            this.AddGameListener(UIGameEvent.WHITE_TO_BLACK, this.onWhiteToBlack, this);
            this.AddGameListener(UIGameEvent.WHITE_TO_MAIN, this.onWhiteToMain, this);
            this.AddGameListener(UIGameEvent.SHOW_MAIN_BTNS, this.onShowMainBtns, this);
            this.AddGameListener(UIGameEvent.GUIDE_SHOW, this.onShowGuide, this);
            this.AddGameListener(UIGameEvent.GUIDE_COMPELTE, this.onGuideComplete, this);
            this.AddGameListener(UIGameEvent.GUIDE_COMPELTE_ALL, this.onGuideCompletAll, this);
            this.AddGameListener(UIGameEvent.MAIN_GUIDE_COMPELTE_ALL, this.onMainGuideComplet, this);
            this.AddGameListener(UIGameEvent.GET_ADLIXIAN, this.OnGetAdLixian, this);
            this.AddGameListener(UIGameEvent.CAMERA_TWEEN_START, this.OnCameraTweenStart, this);
            this.AddGameListener(UIGameEvent.BUILD_LEVEL_LESS, this.onBuildLevelUp, this);
            this.AddGameListener(UIGameEvent.MAINTASK_COMP_PROP, TaskCtr.Inst.OnMainTaskComp, this);
            this.AddGameListener(UIGameEvent.BUILD_UNLOCK, this.OnBuildUnLock, this);
            Laya.stage.on(Laya.Event.CLICK, this, this.OnClickScree);
            this.AddGameListener(UIGameEvent.GAME_HT, this.onGameHt, this);
            this.AddGameListener(UIGameEvent.ON_ACTIVE, this.onActive, this);
            this.AddGameListener(UIGameEvent.ONGETDOUBLEAWARD, this.OnGetDoubleAward, this);
            this.AddGameListener(UIGameEvent.ONUPDATEYUYUEBTNSHOW, this.OnUpdateYuYueBtnShow, this);
            this.AddGameListener(UIGameEvent.ADD_BUILD_PARTS, BuildCtrl.Inst.BuildAddTeShuPeiJian, this);
            this.AddGameListener(UIGameEvent.ON_LIHE_GETAWARD, this.ChickIsHideLiHe, this);
            this.AddGameListener(UIGameEvent.ONCLOSEBOXVIEW, this.checkBoxBtnCanShow, this);
        }
        RemoveListener() {
            super.RemoveListener();
            Laya.stage.off(Laya.Event.CLICK, this, this.OnClickScree);
        }
        InitData(param) {
            super.InitData(param);
            this.btn_yuyue.play("mingtianlingqu");
            this.btn_box.play("baoxiang");
            this.UpdateCurrency();
            this.UpdateGoldRate();
            this.black.touchable = false;
            let guideVo = GuideModel.Inst.guideVO;
            this.sp_touch.touchable = guideVo && guideVo.isLessThan(10, 4) && !guideVo.isPlay && !guideVo.isStart;
            this.c2.selectedIndex = 0;
            Laya.timer.frameLoop(1, this, this.checkBlackCanTouch);
            TaskModel.Inst.CheckIsHideBtnRed();
            Laya.timer.loop(500, this, this.CheckCatBtn);
            this.CheckCatBtn();
            let spineBtn = this.btn_bottle.getChild("spine_bottle");
            spineBtn && spineBtn.play("icon_piaoliuping");
            if (CatModel.Inst.canBefall) {
                Laya.timer.loop(1000, this, this.CheckCatCanBefall);
            }
            if (PlatformMgr.Inst.isWX) {
                this.ht_0.InitData(null);
                this.ht_0.cancel = Laya.Handler.create(this, this.onHtCancel, null, false);
                Laya.timer.frameLoop(1, this, this.onGameHt);
            }
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                this._isShowbanner = false;
            }
            if (Laya.Browser.onQGMiniGame) {
                this.hideAddInstalledBtn();
                PlatformMgr.Inst.hasShortcutInstalled(Laya.Handler.create(this, this.showAddInstalledBtn));
                Laya.timer.loop(3000, this, this.ShakeAddDesktopBtn);
            }
            this.OnUpdateYuYueBtnShow();
            this.ChickIsHideLiHe();
            this.onResize();
        }
        ChickIsHideLiHe() {
            this.btn_lihe.visible = !LiHeModel.Inst.canHide;
            this.sortLeftTopBtns();
        }
        checkBoxBtnCanShow() {
            if (this.btn_box.visible != AdCtrl.Inst.canShowBoxBtn) {
                this.btn_box.visible = AdCtrl.Inst.canShowBoxBtn;
                this.sortRightBtns();
            }
        }
        checkBlackCanTouch() {
            if (UISystem.Inst.HasView(UICatDetailsView)) {
                this.black.touchable = false;
                return;
            }
            else if (!UISystem.Inst.HasView(UICatView) && UISystem.Inst.HasView(UICatInviteAlert)) {
                this.black.touchable = false;
                return;
            }
            if (!GuideModel.Inst.IsCanShowBtns()) {
                this.black.touchable = false;
                return;
            }
            if (this.c2.selectedIndex == 1) {
                if (this.group_right.alpha <= 0) {
                    this.black.touchable = true;
                }
                else {
                    this.black.touchable = false;
                }
            }
            else {
                this.black.touchable = true;
            }
        }
        start() {
            this.ShowDoubleGold();
            let bol = this.CheckShowOffLineAward();
            let vo = GiftPKGModel.Inst.mainGiftBag;
            if (vo && vo.isCanShow && vo.isCanGet && !vo.isCd && !bol) {
                UISystem.Inst.CreateWindowView(UIGiftBagView, vo);
            }
            this.sortLeftBtns();
            this.CheckShowBottleBtn();
            Laya.timer.loop(1000, this, this.UpdateBtnTime);
            this.UpdateBtnTime();
            this.onShowGuide(GuideModel.Inst.guideVO);
        }
        onGameHt() {
            if (this.ht_0.visible) {
                Laya.timer.clear(this, this.onGameHt);
                this.sortLeftTopBtns();
            }
        }
        onHtCancel() {
            UISystem.Inst.CreateWindowView(UIHTMoreView);
        }
        onActive(bol) {
            OffLineCtr.Inst.OnActive(bol);
            if (bol) {
                if (PlatformMgr.Inst.isWX && GuideModel.Inst.IsMainGuideAll()) {
                    UISystem.Inst.CreateWindowView(UIHTMoreView);
                }
            }
            let sec = CVModel.Inst.GetValue(CVModel.leaveTime, CVModel.valType_number);
            if (bol) {
                this.showTime = TimeModel.Inst.LocalTime;
            }
            else {
                this.hideTime = TimeModel.Inst.LocalTime;
            }
            if ((this.showTime - this.hideTime) / 1000 > sec) {
                this.CheckShowOffLineAward(bol);
            }
        }
        OnUpdateYuYueBtnShow() {
            this.btn_yuyue.visible = !(YuYueModel.Inst.state == YuYueModel.state5);
            this.sortLeftBtns();
        }
        CheckCatCanBefall() {
            if (GuideModel.Inst.guideVO != null)
                return;
            if (!CatModel.Inst.canBefall) {
                Laya.timer.clear(this, this.CheckCatCanBefall);
            }
            else {
                if (!UISystem.Inst.hasAnyViewOnMain([UIGuideView, UITipView, UINativeIconAd, UIAdNativeBannerView]) && this.btn_matou.touchable) {
                    CatModel.Inst.befallIntervalTime++;
                    if (CatModel.Inst.befallIntervalTime > CatModel.Inst.maxBefallIntervalTime) {
                        UISystem.Inst.CreateWindowView(UICatBefallView);
                    }
                }
                else {
                    CatModel.Inst.befallIntervalTime = 0;
                }
            }
        }
        OnBuildUnLock(buildVo) {
            if (buildVo && buildVo.buildRes.id == DriftingBottleModel.unLockId) {
                this.CheckShowBottleBtn();
            }
        }
        CheckShowBottleBtn() {
            if (DriftingBottleModel.Inst.driftingBottleState != DriftingBottleModel.yaoQingCompState) {
                this.btn_bottle.visible = DriftingBottleModel.Inst.isUnLock;
                this.sortLeftTopBtns();
            }
        }
        CheckShowOffLineAward(isActive = true) {
            if (isActive) {
                let bol = OffLineCtr.Inst.GetLiXian();
                this.UpdateLiXianBtnShow();
                this.btn_lixian.visible = !!StorageUtils.Inst.GetlixianAd();
                this.btn_lixian.setRed(1);
                this.sortLeftBtns();
                return bol;
            }
        }
        UpdateLiXianBtnShow() {
            let time = Math.floor(StorageUtils.Inst.GetlixianAd() / 1000);
            let offlinetime = Math.min(time, 2 * 60 * 60);
            let gold = StorageUtils.Inst.GetlixianGoldAward().multipliedBy(offlinetime);
            this.btn_lixian.title = MathUtil.goldFormat(gold);
        }
        OnGetAdLixian() {
            StorageUtils.Inst.SetlixianAd(0);
            this.btn_lixian.visible = false;
            this.sortLeftBtns();
        }
        CheckCatBtn() {
            this.onShowCatBtnRed(CatModel.Inst.CheckCanInvite());
            this.ShowLiBaoBtnRed();
            this.ShowDaoYuBtnRed();
            if (GiftPKGModel.Inst.canGrowthAward) {
                this.btn_growUp.setRed(1);
            }
            else {
                this.btn_growUp.setRed(0);
            }
            this.CheckShopBtnRed();
            if (WarOrderModel.Inst.CheckShowRed()) {
                this.btn_huodong.setRed(1);
            }
            else {
                this.btn_huodong.setRed(0);
            }
            if (DriftingBottleModel.Inst.curTask.canAward || DriftingBottleModel.Inst.driftingBottleState == DriftingBottleModel.shangdaoCompState) {
                this.btn_bottle.getController("c1").selectedIndex = 1;
            }
            else {
                this.btn_bottle.getController("c1").selectedIndex = 0;
            }
            if ((this.c2.selectedIndex == 1 || this.c2.selectedIndex == 2) && UISystem.Inst.hasAnyViewOnMain([UIGuideView, UITipView]) && GuideModel.Inst.IsCanShowBtns()) {
                this.ad_grid.Reset();
                this.btn_club.Reset();
            }
            if (UISystem.Inst.HasView(UISkinView) && this.c2.selectedIndex == 1) {
                this.hideBtns();
            }
            this.checkBoxBtnCanShow();
            if (this.c1.selectedIndex != 0) {
                this.onResize();
            }
        }
        OnClickScree() {
            CatModel.Inst.befallIntervalTime = 0;
            if (this.templete == null) {
                this.templete = new Laya.Templet();
                let url = UrlUtils.GetSpineUrl("clickstar");
                this.templete.loadAni(url);
                this.templete.once(Laya.Event.COMPLETE, this, () => {
                    this.clickAni = this.templete.buildArmature(0);
                    this.PlayClickScreenAni();
                });
            }
            else if (this.clickAni) {
                this.PlayClickScreenAni();
            }
        }
        CreateAni() {
            return this.templete && this.templete.buildArmature(0);
        }
        PlayClickScreenAni() {
            let skAni = Laya.Pool.getItemByCreateFun("clickstar", this.CreateAni, this);
            Laya.stage.addChild(skAni);
            skAni.pos(Laya.stage.mouseX, Laya.stage.mouseY);
            skAni.playbackRate(1.5);
            skAni.play(0, false);
            skAni.once(Laya.Event.STOPPED, this, () => {
                Laya.Pool.recover("clickstar", skAni);
            });
        }
        onShowTaskBtnRed(isShowRed = false) {
            let val = isShowRed ? 1 : 0;
            this.btn_task.setRed(val);
        }
        onShowSignBtnRed(isShowRed = false) {
            let val = isShowRed ? 1 : 0;
            this.btn_sign.setRed(val);
            this.sortLeftBtns();
        }
        onShowBagBtnRed(isShowRed = false) {
            let val = isShowRed ? 1 : 0;
            this.btn_beibao.setRed(val);
        }
        onShowCatBtnRed(isShowRed = false) {
            let val = isShowRed ? 1 : 0;
            this.btn_miaoxingren.setRed(val, "kezhaohuan");
        }
        CheckShopBtnRed() {
            let isShowRed = false;
            let vos = ShopModel.Inst.GetShopVos();
            for (let i = 0; i < vos.length; i++) {
                let vo = vos[i];
                if (!vo.isLookAd)
                    continue;
                if (!vo.isCd && vo.canLookAd) {
                    isShowRed = true;
                    break;
                }
            }
            if (!isShowRed) {
                let pkgVOs = GiftPKGModel.Inst.GetPkgVos(1, 1);
                for (let i = 0; i < pkgVOs.length; i++) {
                    let pkg = pkgVOs[i];
                    if (!pkg.isCanGet)
                        continue;
                    if (!pkg.isCd) {
                        isShowRed = true;
                        break;
                    }
                }
            }
            let val = isShowRed ? 1 : 0;
            this.btn_shangdian.setRed(val, "xinlibao");
        }
        onShowMaTouBtnState(buildVO, isAdd, matouState) {
            if (!buildVO || !buildVO.isMatou)
                return;
            if (isAdd) {
                if (matouState == BuildVO.matou_state_guanggao) {
                    this.btn_matou.setRed(2);
                }
                else if (matouState == BuildVO.matou_state_likai) {
                    this.btn_matou.setRed(3);
                }
                else if (matouState == BuildVO.matou_state_shangdao) {
                    this.btn_matou.setRed(4);
                }
                else if (matouState == BuildVO.matou_state_xunzhao) {
                    this.btn_matou.setRed(5);
                }
            }
            else {
                this.btn_matou.setRed(0);
            }
        }
        onShowGuide(guideVo) {
            if (!guideVo || !guideVo.guideRes) {
                this.txt_guide.text = "";
                return;
            }
            if (this.txt_guide.visible) {
                if (guideVo) {
                    this.txt_guide.text = `当前引导ID:${guideVo.guideRes.id}`;
                }
                else {
                    this.txt_guide.text = "";
                }
            }
            if (!GuideModel.Inst.IsCanShowBtns()) {
                this.group_left.visible = this.group_right.visible = this.group_leftTop.visible = false;
            }
            else {
                this.group_left.visible = this.group_right.visible = this.group_leftTop.visible = true;
                this.showBtns();
            }
        }
        onGuideComplete(guideVO) {
            if (guideVO.tempId == 1) {
                this.start();
            }
        }
        onMainGuideComplet() {
            if (Laya.Browser.onQGMiniGame) {
                !this.mainGuideCom && AdCtrl.Inst.AddDesktop(Laya.Handler.create(this, this.hideAddInstalledBtn));
                if (!this.mainGuideCom)
                    this.mainGuideCom = true;
            }
        }
        onGuideCompletAll() {
            this.txt_guide.visible = false;
        }
        onClickBuild(bs) {
            this.selectBuild = SceneMgr.Inst.GetCameraBuildScript().selectBuildVo;
        }
        OnCameraTweenStart() {
            fgui.GRoot.inst.touchable = false;
        }
        onCameraMoveComplete(buildVo) {
            fgui.GRoot.inst.touchable = true;
            SceneMgr.Inst.GetCameraBuildScript().ResetSceneScale();
            this.selectBuild = SceneMgr.Inst.GetCameraBuildScript().selectBuildVo;
            if (this.selectBuild) {
                this.probar_build.visible = this.selectBuild.GetTime() > 0;
                if (this.probar_build.visible) {
                    this.onSceneResize();
                    this.probar_build.max = this.selectBuild.GetTime() * 10;
                    this.probar_build.value = 0;
                    this.updateBuildProBar();
                    Laya.timer.clear(this, this.updateBuildProBar);
                    Laya.timer.loop(1000, this, this.updateBuildProBar);
                }
            }
        }
        updateBuildProBar() {
            let to = Math.ceil(this.probar_build.value) + 10;
            if (to > this.probar_build.max) {
                this.probar_build.value = 0;
                to = 10;
            }
            fgui.GTween.to(this.probar_build.value, to, 1).setTarget(this.probar_build, "value").setEase(fgui.EaseType.Linear)
                .onUpdate((tw) => {
                let txt = Math.ceil(this.probar_build.value) + "";
                if (txt.length < 2)
                    txt = "0" + txt;
                this.probar_build["_titleObject"].text = txt.substr(0, txt.length - 1) + "." + txt.charAt(txt.length - 1);
            }, this).onComplete(() => {
                if (to == this.probar_build.max && this.probar_build.visible) {
                    let num = this.selectBuild.GetAllAddGold();
                    let pos = new Laya.Point(this.probar_build.x, this.probar_build.y - 50);
                    EffectMgr.Inst.FlyGold(num, pos, UIFlyCoinItem.TYPE_NORMAL);
                }
            }, this);
        }
        onSceneResize() {
            if (this.probar_build.data instanceof CatBasinScript) {
                this.updateCatBasinXY();
            }
            else if (this.selectBuild != null) {
                let scale = GlobalConfig.scene_scale + 0.25;
                this.probar_build.setScale(scale, scale);
                let camera = SceneMgr.Inst.GetCamera();
                this.selectBuild.buildPos.cloneTo(this.out);
                this.out.y += 5;
                let svec = MathUtil.GetPointFrom3DWorld(camera, this.out);
                this.probar_build.x = svec.x;
                this.probar_build.y = svec.y;
            }
        }
        updateCatBasinXY() {
            if (this.probar_build.data instanceof CatBasinScript) {
                let scale = GlobalConfig.scene_scale;
                let catBasinPos = this.probar_build.data.catBasinPos;
                let camera = SceneMgr.Inst.GetCamera();
                camera.transform.getForward(this.out);
                let zz1 = new Laya.Vector3(-this.out.x, -this.out.y, -this.out.z);
                let curZ = MathUtil.ProjectDistance(catBasinPos, camera.transform.position, zz1);
                if (isNaN(this.initCatBasinDis)) {
                    this.initCatBasinDis = curZ;
                }
                scale *= (this.initCatBasinDis / curZ);
                scale = Math.max(0, scale);
                this.probar_build.setScale(scale, scale);
                catBasinPos.cloneTo(this.out);
                this.out.y += 4;
                let v2 = MathUtil.GetPointFrom3DWorld(camera, this.out);
                this.probar_build.setXY(v2.x, v2.y);
            }
        }
        showProBar(text, value, max, tweenStart = false, stay = 3, data = null) {
            this.probar_build.data = data;
            this.probar_build.max = max;
            this.probar_build.value = value;
            this.probar_build["_titleObject"].text = text;
            if (data instanceof Laya.Vector2 || data instanceof Laya.Point) {
                this.probar_build.setXY(data.x, data.y);
            }
            else {
                this.updateCatBasinXY();
            }
            if (tweenStart) {
                this.probar_build.visible = true;
                fgui.GTween.to(0, 1, 0.3).setTarget(this.probar_build, "alpha");
            }
            if (stay > 0) {
                fgui.GTween.delayedCall(stay).setTarget(this.probar_build).onComplete(() => {
                    fgui.GTween.to(1, 0, 0.3).setTarget(this.probar_build, "alpha").onComplete(this.hideProbar, this);
                }, this);
            }
        }
        hideProbar() {
            Laya.timer.clear(this, this.updateBuildProBar);
            fgui.GTween.kill(this.probar_build);
            this.probar_build.alpha = 1;
            this.probar_build.visible = false;
            this.probar_build.data = null;
            this.selectBuild = null;
        }
        onShow(panelName) {
            if (panelName == "UICatInviteAlert") {
                if (UISystem.Inst.HasView(UICatView)) {
                }
                else {
                    this.hideBtns();
                }
            }
            else if (panelName == "UICatDetailsView") {
                this.hideBtns();
            }
            else if (panelName == "UISkinView") {
                this.hideBtns();
            }
            else if (panelName == "UIBuildView") {
                this.hideBtns();
            }
            else {
                if (UISystem.Inst.hasAnyViewOnMain([UIGuideView, UITipView]) && GuideModel.Inst.IsCanShowBtns()) {
                    this.ad_grid.Reset();
                    this.btn_club.Reset();
                }
            }
            if (panelName != "UIGuideView" && panelName != "UITipView") {
                this.tipComp.backFromOther();
            }
        }
        onHide(panelName) {
            if (panelName == "UICatInviteAlert") {
                this.showBtns();
            }
            else if (panelName == "UICatDetailsView") {
                this.showBtns();
            }
            else if (panelName == "UISkinView") {
                this.showBtns();
            }
            else if (panelName == "UIBuildView") {
                this.showBtns();
                this.hideProbar();
            }
            else if (panelName == "UILogoView") {
                this.showBtns();
            }
            else if (panelName == "UILoadingView") {
            }
            else {
                if (!UISystem.Inst.hasAnyViewOnMain([UIGuideView, UITipView, UIAdNativeBannerView, UINativeIconAd])) {
                    this.showBtns();
                }
            }
            if (PlatformMgr.Inst.isWX) {
                this.hideBanner();
            }
            else if (Laya.Browser.onBDMiniGame) {
                this.hideBanner();
            }
        }
        showBtns() {
            if (UISystem.Inst.HasView(UICatDetailsView) || UISystem.Inst.HasView(UICatInviteAlert) || UISystem.Inst.HasView(UISkinView)
                || UISystem.Inst.HasView(UILogoView) || UISystem.Inst.HasView(UIBuildView)) {
                return;
            }
            if (GuideModel.Inst.guideVO && GuideModel.Inst.guideVO.guideRes.id < GuideModel.GUID_SHOWBTNS_ID && this.group_right.visible) {
                return;
            }
            if (!GuideModel.Inst.IsCanShowBtns()) {
                return;
            }
            this.setBtnsTouchEnabled(true);
            if (this.c2.selectedIndex != 1) {
                this.c2.selectedIndex = 1;
            }
            else if (this.group_right.alpha == 0) {
                this.t1.play();
            }
            if (this.black.alpha == 1) {
                this.black.touchable = false;
                this.t2.play();
            }
            if (this.group_top.alpha <= 0) {
                fgui.GTween.to(0, 1, 0.5).setTarget(this.group_top, "alpha").setEase(fgui.EaseType.Linear);
            }
            this.sortLeftTopBtns();
            this.sortRightBtns();
            if (!UISystem.Inst.hasAnyViewOnMain([UIGuideView, UITipView]) && GuideModel.Inst.IsCanShowBtns()) {
                this.showNativeGrid();
            }
            if (PlatformMgr.Inst.isWX) {
                this.hideBanner();
            }
            else if (Laya.Browser.onBDMiniGame) {
                this.hideBanner();
            }
            else {
                this.showBanner();
            }
            this.onResize();
        }
        showNativeGrid() {
            this.ad_grid.InitData(null);
            this.btn_club.InitData(null);
        }
        hideBtns() {
            this.setBtnsTouchEnabled(false);
            if (this.c2.selectedIndex != 4) {
                this.c2.selectedIndex = 4;
            }
            else if (this.group_left.alpha == 1) {
                this.t1.playReverse();
            }
            this.ad_grid.Reset();
            this.btn_club.Reset();
        }
        onBuildLevelUp(buildVO) {
            if (GuideModel.Inst.guideVO) {
                this.tipComp.Reset();
                return;
            }
            if (buildVO && this.group_left.alpha == 1) {
                this.tipComp.InitData(buildVO);
            }
            else {
                this.tipComp.Reset();
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn.alpha == 0 || !btn.visible) {
                return;
            }
            if (btn == this.btn_fengxiang) {
                GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, TaskType.TASKTYPE_PAIZHAO);
                GameDispatcher.Inst.event(UIGameEvent.DAYTASK_COMP_PROP, TaskVO.PAIZHAO);
            }
            else if (btn == this.btn_set) {
                this.onBtnSetClick();
            }
            else if (btn == this.btn_task) {
                UISystem.Inst.CreateWindowView(UITaskView);
            }
            else if (btn == this.btn_miaoxingren) {
                this.onBtnMaoxingrenClick();
            }
            else if (btn == this.btn_daoyu) {
                UISystem.Inst.CreateWindowView(UIIslandInfoView);
            }
            else if (btn == this.btn_beibao) {
                if (this.btn_beibao.isRed) {
                    this.btn_beibao.setRed(0);
                }
                UISystem.Inst.CreateWindowView(UIBagView);
            }
            else if (btn == this.btn_baike) {
                UISystem.Inst.CreateWindowView(UISkinBaikeView);
            }
            else if (btn == this.btn_duihuansuo) {
                UISystem.Inst.CreateWindowView(UIExchangeView);
            }
            else if (btn == this.btn_sign) {
                this.onBtnQiandaoClick();
            }
            else if (btn == this.btn_table) {
            }
            else if (btn == this.btn_matou) {
                this.onBtnMatouClick();
            }
            else if (btn == this.btn_shuangbei) {
                this.onBtnShuangbeiClick();
            }
            else if (btn == this.btn_shangdian) {
                UISystem.Inst.CreateWindowView(UIShoppingView);
            }
            else if (btn == this.btn_gm) {
                UISystem.Inst.CreateWindowView(UIGMView);
            }
            else if (btn == this.btn_camera) {
                CameraCtrl.Inst.MoveBuild(BuildModel.Inst.maowuBuildVO);
            }
            else if (btn == this.btn_skip) {
                this.sp_touch.touchable = true;
                this.onCompleteOnland(MatouModel.Inst.curUpCat);
                GameDispatcher.Inst.event(UIGameEvent.CAT_SKIP_ONLAND);
            }
            else if (btn == this.btn_libao) {
                UISystem.Inst.CreateWindowView(UIGiftBagView, GiftPKGModel.Inst.mainGiftBag);
            }
            else if (btn == this.btn_lixian) {
                this.OnBtnLiXianClick();
            }
            else if (btn == this.btn_growUp) {
                this.OnBtnClickGrowUpClick();
            }
            else if (btn == this.btn_huodong) {
                UISystem.Inst.CreateWindowView(UISummerActivityView);
            }
            else if (btn == this.btn_bottle) {
                UISystem.Inst.CreateWindowView(UIDriftingBottleView);
            }
            else if (btn == this.btn_moreGame) {
                if (PlatformMgr.Inst.isWX) {
                    UISystem.Inst.CreateWindowView(UIHTMoreView);
                }
                else if (Laya.Browser.onQGMiniGame) {
                    PlatformMgr.Inst.navigateTo(null);
                }
                else if (Laya.Browser.onQQMiniGame) {
                    PlatformMgr.Inst.showAd(AdType.ad_6);
                }
            }
            else if (btn == this.btn_yuyue) {
                UISystem.Inst.CreateWindowView(UISubscribeView);
            }
            else if (btn == this.btn_lihe) {
                UISystem.Inst.CreateWindowView(UILiHeView);
            }
            else if (btn == this.btn_box) {
                UISystem.Inst.CreateWindowView(UIBoxView, true);
            }
            else if (btn == this.btn_addInstalled) {
                AdCtrl.Inst.AddDesktop(Laya.Handler.create(this, this.hideAddInstalledBtn));
            }
        }
        hideAddInstalledBtn(isCreat = true) {
            this.btn_addInstalled.visible = !isCreat;
        }
        showAddInstalledBtn(isCreat = false) {
            this.btn_addInstalled.visible = !isCreat;
        }
        ShakeAddDesktopBtn() {
            let btn = this.btn_addInstalled;
            fgui.GTween.kill(btn, true);
            if (!btn.visible)
                return;
            btn.visible && fgui.GTween.shake(btn.x, btn.y, 5, 0.5).setTarget(this)
                .onUpdate(function t2(tw) {
                btn.x = tw.value.x;
                btn.y = tw.value.y;
            }, this);
        }
        OnBtnClickGrowUpClick() {
            UISystem.Inst.CreateWindowView(UIGrowthGiftPackView);
        }
        OnBtnLiXianClick() {
            UISystem.Inst.CreateWindowView(UILiXianView, 1);
        }
        ShowDaoYuBtnRed() {
            if (LandModel.Inst.CanUnlockNextLand()) {
                this.btn_daoyu.setRed(1);
            }
            else {
                this.btn_daoyu.setRed(0);
            }
        }
        ShowLiBaoBtnRed() {
            let bol = this.btn_libao.visible;
            let vo = GiftPKGModel.Inst.mainGiftBag;
            if (vo) {
                this.btn_libao.visible = vo.isCanGet && vo.isCanShow;
                if (vo.isCanGet && vo.isCanShow && !vo.isCd) {
                    this.btn_libao.setRed(1);
                }
                else {
                    this.btn_libao.setRed(0);
                }
                if (this.libaoBtnIcon != vo.pkgRes.icon) {
                    this.btn_libao.icon = PanelRegister.getUrl("main", vo.pkgRes.icon);
                    this.libaoBtnIcon = vo.pkgRes.icon;
                }
            }
            else {
                this.btn_libao.visible = false;
            }
            if (bol != this.btn_libao.visible) {
                this.sortLeftBtns();
            }
        }
        sortLeftBtns() {
            let btns = [this.btn_yuyue, this.btn_growUp, this.btn_lixian, this.btn_libao, this.btn_table, this.btn_shuangbei, this.btn_shangdian];
            let btn;
            let prev;
            for (let i = btns.length - 1; i >= 0; i--) {
                btn = btns[i];
                if (i == btns.length - 1) {
                    prev = btn;
                }
                else if (btn.visible) {
                    if (btn.pivotAsAnchor) {
                        btn.y = prev.y - btn.height * 0.5 - 25;
                    }
                    else {
                        btn.y = prev.y - btn.height - 25;
                    }
                    prev = btn;
                }
            }
        }
        sortLeftTopBtns() {
            this.group_leftTop.y = this.group_right.y;
            let btns = [this.ht_0, this.btnChouti_leftTop];
            let btn;
            let prev;
            for (let i = 0; i < btns.length; i++) {
                btn = btns[i];
                if (FairyUtils.targetIsAllVisible(btn)) {
                    if (prev == null) {
                        if (btn.pivotAsAnchor) {
                            btn.y = this.group_leftTop.y + btn.height * 0.5;
                        }
                        else {
                            btn.y = this.group_leftTop.y;
                        }
                    }
                    else {
                        btn.y = prev.y + prev.height * 0.5 + btn.height * 0.5 + 20;
                    }
                    prev = btn;
                }
            }
            this.btnChouti_leftTop.sortBtns();
        }
        sortRightBtns() {
            let btns = [this.btn_miaoxingren, this.btn_daoyu, this.btn_beibao, this.btn_baike, this.btn_duihuansuo, this.btn_matou, this.btn_box, this.btn_moreGame, this.btn_club];
            let prev = this.btn_task;
            let btn = null;
            for (let i = 0; i < btns.length; i++) {
                btn = btns[i];
                if (FairyUtils.targetIsAllVisible(btn)) {
                    let h1 = prev.pivotAsAnchor ? prev.height * 0.5 : prev.height;
                    let h2 = btn.pivotAsAnchor ? btn.height * 0.5 : 0;
                    btn.y = prev.y + h1 + h2 + 18;
                    prev = btn;
                }
            }
        }
        onCompleteOnland(cat) {
            if (MatouModel.Inst.curUpCat != cat) {
                return;
            }
            if (GuideModel.Inst.IsNewer()) {
                return;
            }
            this.black.touchable = false;
            SceneMgr.Inst.GetCameraBuildScript().ResetSceneScale();
            this.showBtns();
        }
        onBtnQiandaoClick() {
            UISystem.Inst.CreateWindowView(UISignView);
        }
        onBtnMaoxingrenClick() {
            UISystem.Inst.CreateWindowView(UICatView);
        }
        onBtnShuangbeiClick() {
            if (GlobalConfig.isDouble)
                return;
            UISystem.Inst.CreateWindowView(UIDoubleAwardView);
        }
        OnGetDoubleAward() {
            GlobalConfig.isDouble = true;
            StorageUtils.Inst.SetDoubleGoldTime(StorageUtils.getCurTimer());
            this.ShowDoubleGold();
            this.btn_doubleani.play("doublecoin", NaN, NaN, false, true, Laya.Handler.create(this, () => {
                this.btn_doubleani.Reset();
            }));
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.itemDrop));
        }
        ShowDoubleGold() {
            let time = CVModel.Inst.GetValue(CVModel.goldtime, CVModel.valType_number);
            let intervalTime = StorageUtils.Inst.GetDoubleGoldTime();
            this.pro_double.max = time;
            this.pro_double.min = 0;
            if (intervalTime != 0 && intervalTime < time) {
                GlobalConfig.isDouble = true;
                this.pro_double.value = intervalTime;
                GameDispatcher.Inst.event(UIGameEvent.UPDATE_GOLD_RATE);
                Laya.timer.loop(1000, this, this.UpdateDoubleProp);
                this.btn_shuangbei.visible = false;
            }
            else if (GlobalConfig.isDouble) {
                this.pro_double.value = 0;
                GameDispatcher.Inst.event(UIGameEvent.UPDATE_GOLD_RATE);
                Laya.timer.loop(1000, this, this.UpdateDoubleProp);
                this.btn_shuangbei.visible = false;
            }
            else {
                this.btn_shuangbei.visible = true;
            }
            this.sortLeftBtns();
        }
        UpdateDoubleProp() {
            this.pro_double.value++;
            if (this.pro_double.value >= this.pro_double.max) {
                GlobalConfig.isDouble = false;
                GameDispatcher.Inst.event(UIGameEvent.UPDATE_GOLD_RATE);
                Laya.timer.clear(this, this.UpdateDoubleProp);
                this.btn_shuangbei.visible = true;
                this.sortLeftBtns();
            }
        }
        onBtnMatouClick() {
            let matou = BuildModel.Inst.matouBuildVO;
            UISystem.Inst.CreateWindowView(UIBuildView, matou);
        }
        UpdateGoldRate() {
            this.pro_double.visible = GlobalConfig.isDouble;
            let preAdd = BuildModel.Inst.GetAllBuildGoldRate(10);
            preAdd = MathUtil.goldFormat(preAdd);
            this.txt_addGold.text = `+${preAdd}/10秒`;
        }
        UpdateCurrency() {
            this.txt_gold.text = MathUtil.goldFormat(MoneyModel.gold.num);
            this.txt_diamond.text = MathUtil.goldFormat(MoneyModel.diamond.num);
            this.moneyAdd();
        }
        onCatQinmiUpdate(catVo) {
            this.moneyAdd();
        }
        onCatWearCloth(catVo, skinId) {
            this.moneyAdd();
        }
        onTakeOffCloth() {
            this.moneyAdd();
        }
        moneyAdd() {
            let total = CatModel.Inst.GetAllRate();
            this.txt_rate.text = CatModel.getRate(total);
            this.UpdateGoldRate();
        }
        onCatStrokeChange(catVO, param = 0) {
            if (catVO == null || CatModel.Inst.selectCatVO == null || catVO.objId != CatModel.Inst.selectCatVO.objId) {
                return;
            }
            if (catVO.catStrokeVo.IsStroking() || catVO.catStrokeVo.IsStartStroke() || catVO.catStrokeVo.IsAnger()) {
                this.sp_touch.touchable = false;
            }
            else {
                this.sp_touch.touchable = true;
            }
        }
        onBtnSetClick() {
            UISystem.Inst.CreateWindowView(UISettingView);
        }
        onStartMoveHouse() {
            this.btn_club.Reset();
            this.setBtnsTouchEnabled(false);
            this.onMainToBlackToWhite(LandModel.Inst.name);
        }
        onEndMoveHouse() {
            this.c2.selectedIndex = 3;
            fgui.GTween.delayedCall(1).setTarget(this).onComplete(() => {
                this.showBtns();
                CameraCtrl.Inst.MoveToBorn();
            }, this);
        }
        onMainToBlackToWhite(text = "") {
            if (this.c2.selectedIndex == 1) {
                this.setBtnsTouchEnabled(false);
                this.c2.selectedIndex = 0;
                if (text) {
                    Laya.timer.once(1000, this, () => {
                        this.showTip(LandModel.Inst.name);
                    });
                }
            }
        }
        onBlackToWhite() {
            this.group_right.visible = this.group_left.visible = this.group_leftTop.visible = true;
            this.black.touchable = true;
            if (this.c2.selectedIndex == 3 || this.c2.selectedIndex == 4) {
                this.setBtnsTouchEnabled(false);
                this.c2.selectedIndex = 0;
            }
            else if (this.c2.selectedIndex != 2) {
                if (this.group_left.alpha >= 1) {
                    this.t0.playReverse();
                }
                this.setBtnsTouchEnabled(false);
                Laya.timer.once(500, this, () => {
                    this.t6.play();
                });
            }
        }
        onStartOnland() {
            this.setBtnsTouchEnabled(false);
            this.c2.selectedIndex = 2;
            this.black.touchable = true;
            this.btn_club.Reset();
        }
        onWhiteToBlack() {
            if (this.c2.selectedIndex == 0 || this.c2.selectedIndex == 4) {
                this.setBtnsTouchEnabled(false);
                this.c2.selectedIndex = 3;
            }
            else {
                this.setBtnsTouchEnabled(false);
                this.t0.playReverse();
                this.t5.play();
            }
        }
        onShowMainBtns(value = true) {
            if (value) {
                this.showBtns();
            }
            else {
                if (this.c2.selectedIndex == 1) {
                    Laya.timer.once(800, this, () => {
                        this.group_right.visible = this.group_left.visible = this.group_leftTop.visible = false;
                        this.hideBtns();
                    });
                }
            }
        }
        onWhiteToMain() {
            this.group_right.visible = this.group_left.visible = this.group_leftTop.visible = true;
            if (this.c2.selectedIndex != 1) {
                this.showBtns();
            }
            else {
                this.t0.play(Laya.Handler.create(this, this.setBtnsTouchEnabled, [true]));
            }
        }
        IsCanTouch() {
            if (!GuideModel.Inst.IsCanShowBtns()) {
                return true;
            }
            let bol = this.group_left.alpha == 1;
            if (this.group_left.alpha == 0 && (UISystem.Inst.HasView(UICatDetailsView) || UISystem.Inst.HasView(UICatInviteAlert)
                || UISystem.Inst.HasView(UISkinView) || UISystem.Inst.HasView(UIBuildView))) {
                bol = true;
            }
            return this.c2.selectedIndex == 1 && this.sp_touch.touchable && bol;
        }
        setBtnsTouchEnabled(value) {
            this.btn_addInstalled.touchable = this.btn_task.touchable = this.btn_miaoxingren.touchable = this.btn_daoyu.touchable = value;
            this.btn_box.touchable = this.btn_beibao.touchable = this.btn_baike.touchable = this.btn_duihuansuo.touchable = value;
            this.btn_lihe.touchable = this.btn_yuyue.touchable = this.btn_growUp.touchable = this.btn_matou.touchable = this.btn_shuangbei.touchable = this.btn_shangdian.touchable = value;
            this.btn_sign.touchable = this.btn_libao.touchable = this.btn_table.touchable = this.btn_lixian.touchable = value;
            this.btn_huodong.touchable = this.btn_bottle.touchable = value;
            let guideVO = GuideModel.Inst.guideVO;
            if (guideVO && guideVO.isLessThan(10, 4) && !guideVO.isPlay && !guideVO.isStart) {
                this.sp_touch.touchable = true;
            }
            else {
                this.sp_touch.touchable = value;
            }
        }
        showTip(text) {
            this.txt_tip.text = text;
            this.group_tip.visible = true;
            this.t3.play();
            Laya.timer.once(3000, this, this.hideTip);
        }
        hideTip() {
            Laya.timer.clear(this, this.hideTip);
            this.t3.playReverse(Laya.Handler.create(this, () => {
                this.group_tip.visible = false;
            }));
        }
        get GoldPoint() {
            return this._view.localToGlobal(this.txt_gold.x, this.txt_gold.y, this.goldP);
        }
        get GemPoint() {
            return this._view.localToGlobal(this.txt_diamond.x, this.txt_diamond.y, this.gemP);
        }
        showBanner(data = null) {
            if (this.c2.selectedIndex == 1) {
                super.showBanner(data);
            }
        }
        onResize() {
            super.onResize();
            let rightUpRect = PlatformModel.Inst.rightUpRect;
            if (rightUpRect) {
                let scaleY = 1;
                if (Laya.Browser.onQQMiniGame) {
                    scaleY = PlatformModel.Inst.scaleY / Laya.stage.clientScaleY;
                }
                this.group_top.y = rightUpRect.y * scaleY + rightUpRect.height * scaleY - 26;
                this.group_leftTop.y = this.group_right.y;
                this.sp_rightRect.setSize(rightUpRect.width, rightUpRect.height);
                this.sp_rightRect.setXY(rightUpRect.x, rightUpRect.y);
            }
            else {
                this.group_top.y = 70;
            }
            this.sortLeftTopBtns();
            if (this._isShowbanner) {
                let bannerRect = PlatformModel.Inst.bannerRect;
                if (bannerRect) {
                    this.btn_skip.y = bannerRect.y - this.btn_skip.height;
                    this.btn_shangdian.y = bannerRect.y - this.btn_shangdian.height;
                    this.tipComp.y = bannerRect.y - this.tipComp.height - 10;
                }
                else {
                    let ygap = 180;
                    this.btn_shangdian.y = Laya.stage.height - this.btn_shangdian.height - ygap;
                    this.tipComp.y = Laya.stage.height - this.tipComp.height - 73 - ygap;
                    this.btn_skip.y = this.tipComp.y;
                }
                this.sortLeftBtns();
            }
        }
        Reset() {
            super.Reset();
            fgui.GTween.kill(this.group_left);
            fgui.GTween.kill(this.group_right);
            fgui.GTween.kill(this.group_leftTop);
            fgui.GTween.kill(this.group_top);
            this.hideProbar();
            this.t0 && this.t0.stop();
            this.t1 && this.t1.stop();
            this.t2 && this.t2.stop();
            this.t3 && this.t3.stop();
            Laya.timer.clear(this, this.UpdateDoubleProp);
            Laya.timer.clear(this, this.CheckCatBtn);
            Laya.timer.clear(this, this.hideTip);
            Laya.timer.clear(this, this.UpdateBtnTime);
            Laya.timer.clear(this, this.onGameHt);
            Laya.timer.clearAll(this);
        }
    }

    class FlyGoldAction extends BaseAction {
        constructor() {
            super();
            this.goldNum = null;
            this.maxMake = 14;
            this.maxRadius = 100;
            this.fr = 0.7;
            this.makeRadius = 0;
            this.makeCount = 0;
            this.stars = [];
        }
        Run() {
            super.Run();
            this.makeRadius = this.maxRadius;
            this.makeCount = this.maxMake;
            let mainUI = UISystem.Inst.GetWindowView(UIMainView);
            if (this.startPos == null) {
                this.startPos = new Laya.Point(Laya.stage.width * 0.5, Laya.stage.height * 0.5);
            }
            if (this.endPos == null) {
                this.endPos = mainUI.GoldPoint;
            }
            let gold = MathUtil.goldFormat(this.goldNum);
            this.flyCoin = PanelRegister.createGObject("main", "FlyCoin");
            this.flyCoin.touchable = false;
            this.flyCoin.title = gold;
            this.flyCoin.setXY(this.startPos.x, this.startPos.y);
            GameLayer.diversionLayer.addChild(this.flyCoin);
            this.flyCoinTo(this.flyCoin, this.endPos);
        }
        flyCoinTo(target, targetPos) {
            let pointArr = [];
            let pointNum = MathUtil.makeRandom(1, 4);
            pointArr.push(new Laya.Point(target.x, target.y));
            for (let i = 0; i < pointNum; i++) {
                let x = MathUtil.makeRandom(0, Laya.stage.width);
                let y = MathUtil.makeRandom(0, Math.min(Laya.stage.height, target.y + 200));
                pointArr.push(new Laya.Point(x, y));
            }
            pointArr.push(targetPos);
            let bezier = new Bezier2D(target, 2600, 0, pointArr, Laya.Ease.sineIn, Laya.Handler.create(this, () => {
                this.m_isComplete = true;
            }));
            Laya.timer.once(200, this, () => {
                bezier.start();
            });
            Laya.timer.frameLoop(24, this, this.doMakeStars);
        }
        doMakeStars() {
            this.makeCount = Math.ceil(this.makeCount * this.fr) || 1;
            this.makeRadius = Math.ceil(this.makeRadius * 0.8);
            if (this.flyCoin) {
                this.makeStars(this.makeCount, this.flyCoin.x, this.flyCoin.y, this.makeRadius);
            }
        }
        makeStars(count, px, py, radius) {
            let star;
            let l;
            for (let i = 0; i < count; i++) {
                star = PanelRegister.createGObject("main", "FlyStar");
                l = MathUtil.makeRandom(0, 360);
                l = MathUtil.angleToRadian(l);
                let x = Math.ceil(radius * Math.random() * Math.cos(l));
                let y = Math.ceil(radius * Math.random() * Math.sin(l));
                star.setXY(px + x, py + y);
                let scale = MathUtil.makeRandom(5, 10) * 0.1;
                star.setScale(scale, scale);
                star.flyFrom(px, py);
                GameLayer.flayLayer.addChild(star);
                this.stars.push(star);
            }
        }
        Reset() {
            super.Reset();
            Laya.timer.clearAll(this);
            Laya.timer.clear(this, this.doMakeStars);
            PanelRegister.recover(this.flyCoin);
            this.flyCoin = null;
            let star;
            while (this.stars.length > 0) {
                star = this.stars.pop();
                star && star.Reset();
                PanelRegister.recover(star);
            }
            this.stars.length = 0;
        }
        GC() {
            ObjectPools.Inst.gcObj(FlyGoldAction.KEY, this);
        }
    }
    FlyGoldAction.KEY = "FlyGoldAction";

    class EffectMgr {
        constructor() {
            this.effects = new ActionNoQueue(6);
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new EffectMgr();
            return this._Inst;
        }
        playMatEffect(sp) {
            if (sp != null) {
                let action = ObjectPools.Inst.getObj(FlashMatAction.KEY, FlashMatAction);
                action.target = sp;
                action.loop = 3;
                this.effects.AddAction(action);
            }
        }
        playGoldEffect(goldNum, startPos, endPos = null) {
            let action = ObjectPools.Inst.getObj(FlyGoldAction.KEY, FlyGoldAction);
            action.goldNum = goldNum;
            action.startPos = startPos;
            action.endPos = endPos;
            this.effects.AddAction(action);
            GameDispatcher.Inst.event(UIGameEvent.FLY_GOLD, [goldNum, startPos]);
        }
        FlyGold(goldNum, pos, type = UIFlyCoinItem.TYPE_BEZIER, itemType = UIFlyCoinItem.ITEM_TYPE_GOLD, time = 1000, offestY = 80) {
            let item = UIFlyCoinItem.Create();
            item.SetGold(goldNum);
            GameLayer.flayLayer.addChild(item);
            item.x = pos.x;
            item.y = pos.y;
            if (type == 0) {
                item.FlyToTarget(new Laya.Point(Laya.stage.width / 2, 100));
            }
            else if (type == 1) {
                item.Showy(time, offestY);
            }
            item.SetType(itemType);
            return item;
        }
        playConfigEffect(id, parent, from = null, to = null, rot = null, loadSourceComplete = null, time = 5000, isDestroy = true, isLocalToStage = true) {
            let res = EffecResource.getItemByKey(id);
            if (res) {
                this.playEffect(parent, res.resName, from, to, rot, loadSourceComplete, isDestroy, time, 1, isLocalToStage);
            }
        }
        playEffect(parent, name, from = null, to = null, rot = null, loadSourceComplete = null, isDestroy = true, time = 5000, scale = 1, isLocalToStage = true) {
            let action = ObjectPools.Inst.getObj(EffectAction.KEY, EffectAction);
            action.parent = parent;
            action.resName = name;
            action.from = from;
            action.to = to;
            action.rot = rot;
            action.isDestroy = isDestroy;
            action.time = time;
            action.scale = scale;
            action.isLocalToStage = isLocalToStage;
            action.loadSourceComplete = loadSourceComplete;
            this.effects.AddAction(action);
            return action;
        }
        Play(parent, name, from = null, to = null, rot = null, loadSourceComplete = null, isDestroy = true, time = 3000, scale = 1, isLocalToStage = true) {
            if (parent == null)
                return;
            let lf = this;
            let url = UrlUtils.GetEffectUrl(name);
            LoadQueue.Inst.loadParticle(url, Laya.Handler.create(lf, lf.LoadParticle, [parent, name, from, to, rot, loadSourceComplete, isDestroy, time, scale, isLocalToStage]));
        }
        LoadParticle(parent, name, from = null, to = null, rot = null, loadSourceComplete = null, isDestroy, time, scale, isLocalToStage, sprite) {
            if (!sprite)
                return;
            loadSourceComplete && loadSourceComplete.run();
            parent && parent.addChild(sprite);
            if (to != null) {
                sprite.transform.position = to;
            }
            if (rot != null) {
                sprite.transform.rotationEuler = rot;
            }
            if (from != null) {
                sprite.transform.lookAt(from, Laya.Vector3["_Up"]);
            }
            if (isLocalToStage)
                MathUtil.localToGlobal(sprite, SceneMgr.Inst.GetEffects(name));
            let sScale = sprite.transform.localScale;
            sScale.setValue(scale, scale, scale);
            sprite.transform.setWorldLossyScale(sScale);
            if (isDestroy) {
                Laya.timer.once(time, this, this.onDestroy, [sprite], false);
            }
        }
        onDestroy(eff) {
            if (eff && !eff.destroyed) {
                eff.destroy();
            }
        }
    }
    EffectMgr._Inst = null;
    EffectMgr.Eff_fish = 1;
    EffectMgr.Eff_cat_ball_01 = 2;
    EffectMgr.Eff_cat_ball_02 = 3;
    EffectMgr.Eff_cat_ball_03 = 4;
    EffectMgr.Eff_cat_ball_04 = 5;
    EffectMgr.Eff_pubu = 6;
    EffectMgr.Eff_shengji_aixin = 7;
    EffectMgr.Eff_shengji_flower = 8;
    EffectMgr.Eff_shengji_glow1 = 9;
    EffectMgr.Eff_shengji_glow2 = 10;
    EffectMgr.Eff_shengji_jiantou = 11;
    EffectMgr.Eff_sheshi_glow = 12;
    EffectMgr.Eff_sheshi_lihua = 13;
    EffectMgr.Eff_sheshi_lihua2 = 14;
    EffectMgr.Eff_sheshi_smoke = 15;
    EffectMgr.Eff_sheshi_yanhua = 16;
    EffectMgr.Eff_shuihua = 17;
    EffectMgr.Eff_smoke = 18;
    EffectMgr.Eff_shuiwen = 19;
    EffectMgr.Eff_cat_ball_05 = 20;

    class CatFecalBall extends EntityAnimator {
        constructor(displayObject, isSpeacil) {
            super(displayObject);
            this._bornTime = 0;
            this.isSpeacil = false;
            this.onAwake();
            this.isSpeacil = isSpeacil;
        }
        onAwake() {
            this.fs = this.owner && this.owner.getChildAt(0).addComponent(CatFecalBallScript);
            super.onAwake();
        }
        InitAnimator() {
            this.animator = this.fs && this.fs.owner && this.fs.owner.getComponent(Laya.Animator);
            if (this.animator)
                this.animatorCtr = this.animator.getControllerLayer(0);
        }
        SetVO(vo) {
            super.SetVO(vo);
            this._bornTime = Laya.timer.currTimer;
            this.fs.SetVO(vo);
            this.InitPos();
            let aniName = "maoqiu01";
            if (this.catVO.catRes.catball == "cat2_Ball_02") {
                aniName = "maoqiu02";
            }
            else if (this.catVO.catRes.catball == "cat2_Ball_03") {
                aniName = "maoqiu03";
            }
            else if (this.catVO.catRes.catball == "cat2_Ball_04") {
                aniName = "maoqiu04";
            }
            if (this.isSpeacil) {
                if (this.catVO.catRes.specialball == "cat2_Ball_05") {
                    aniName = "maoqiu02";
                }
            }
            this.PlayState(aniName, Laya.Handler.create(this, this.PlayEffect));
        }
        InitPos() {
            let cat = EntityModel.Inst.GetCat(this._vo.objId);
            if (cat) {
                let pos = cat.transform.position.clone();
                pos.y += 0.35;
                this._displayObject.transform.position = pos;
            }
        }
        get catVO() {
            return this._vo;
        }
        get objId() {
            return this._vo.objId + "";
        }
        PlayEffect() {
            if (this.fs)
                this.fs.PlayEffect();
        }
        get BornTime() {
            return this._bornTime;
        }
        IsCanDisapper() {
            let livetime = CatFecalBallModel.Inst.livetime;
            return this._bornTime > 0 && Laya.timer.currTimer - this._bornTime >= livetime;
        }
        GetCollisonGroup() {
            return CollisionType.TYPE_ENTITY;
        }
        GetCanCollideWith() {
            return CollisionType.TYPE_RAY_CLICK;
        }
        dispose() {
            if (this.fs) {
                this.fs.destroy();
                this.fs = null;
            }
            super.dispose();
        }
    }

    class UISpecialCatBallAlert extends UIBaseWindow {
        constructor() {
            super("CatBall", "UISpecialCatBallAlert");
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_ALERT;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowImgBg = true;
            this.viewStruct.isEffect = true;
        }
        InitUI() {
            super.InitUI();
            this.c2 = this._view.getController("c2");
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.ON_HIDE, this.onHide, this);
        }
        InitData(data) {
            super.InitData(data);
            this.handler = data && data.handler;
            this.closeHandler = data && data.closeHandler;
            let count = StorageUtils.Inst.GetSpecialCatFecal();
            if (count == 0) {
                this.c2.selectedIndex = 0;
            }
            else {
                let isAd = MathUtil.random(CatFecalBallModel.Inst.HighHairballAdvertisement);
                this.c2.selectedIndex = isAd ? 1 : 0;
            }
            this.spine_maoqiu.play("maoqiu");
            StorageUtils.Inst.SetSpecialCatFecal(1);
            PlatformMgr.Inst.aldSendEvent(EventType.maoqiu, "高级毛球");
            if (PlatformMgr.Inst.isWX || Laya.Browser.onBDMiniGame) {
                if (AdCtrl.Inst.wxIsShowEmbedAd) {
                    this.hideBanner();
                    Laya.timer.once(1000, this, this.resizeBanner);
                }
                else {
                    this.showBanner();
                }
            }
            else if (Laya.Browser.onQGMiniGame) {
                this.hideBanner();
                AdCtrl.Inst.ShowNatiEmbed(this);
            }
            else if (Laya.Browser.onHWMiniGame) {
                if (MathUtil.random(3000)) {
                    this.hideBanner();
                    PlatformMgr.Inst.showAd(AdType.ad_3);
                }
                else {
                    this.showBanner();
                }
            }
            else if (Laya.Browser.onVVMiniGame) {
                if (Math.random() < 0.3) {
                    this.hideBanner();
                    PlatformMgr.Inst.showAd(AdType.ad_3);
                }
                else {
                    this.showBanner();
                }
            }
            else if (Laya.Browser.onQQMiniGame) {
                if (Math.random() < 0.3) {
                    this.hideBanner();
                    PlatformMgr.Inst.showAd(AdType.ad_3);
                }
                else {
                    this.showBanner();
                }
            }
            else if (PlatformMgr.Inst.isMZ) {
                if (Math.random() < 0.3) {
                    this.hideBanner();
                    PlatformMgr.Inst.showAd(AdType.ad_3);
                }
                else {
                    this.showBanner();
                }
            }
            else if (Laya.Browser.onKGMiniGame) {
                if (Math.random() < 0.3) {
                    this.hideBanner();
                    PlatformMgr.Inst.showAd(AdType.ad_3);
                }
                else {
                    this.showBanner();
                }
            }
            else {
                this.showBanner();
            }
        }
        resizeBanner() {
            if (GuideModel.Inst.IsMainGuideAll()) {
                let bottom = this.bg.y + this.bg.height;
                UISystem.Inst.CreateWindowView(UIWXBannerView, { "bottom": bottom });
            }
        }
        onHide(panelName) {
            if (panelName == "UITipView" || panelName == "UIGuideView") {
            }
            else {
                this.showBanner();
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_get) {
                this.getAward(false);
            }
            else if (btn == this.btn_get_ad) {
                PlatformMgr.Inst.showAd(AdType.ad_10, Laya.Handler.create(this, this.getAward, [true]));
            }
        }
        getAward(isAd = false, isSuccess = true) {
            if (isSuccess) {
                PlatformMgr.Inst.aldSendEvent(EventType.maoqiu, "高级毛球奖励-" + (isAd ? "视频" : "普遍"));
                this.handler && this.handler.run();
                this.close();
            }
        }
        onBtnCloseClick() {
            this.closeHandler && this.closeHandler.run();
            super.onBtnCloseClick();
        }
        Reset() {
            super.Reset();
            this.handler = null;
            this.closeHandler = null;
            AdCtrl.Inst.ClearShowNatiEmbedTimer();
        }
    }

    class CatFecalBallCtrl {
        constructor() {
            this.addMap = {};
            this.randomCatVO = null;
        }
        static get Inst() {
            return this.inst = this.inst || new CatFecalBallCtrl();
        }
        init() {
            let cats = EntityModel.Inst.GetCats();
            let cat;
            for (const objId in cats) {
                cat = cats[objId];
                cat && this.makeFecal(cat.catVO);
            }
            Laya.timer.loop(CatFecalBallModel.Inst.losttime, this, this.checkBallsMake);
        }
        checkBallsMake() {
            this.randomCatVO = CatModel.Inst.GetOwnedRandomOne();
            this.makeFecal(this.randomCatVO);
        }
        CheckFecal(cat) {
            this.makeFecal(cat.catVO);
        }
        makeFecal(catVo) {
            let self = this;
            if (catVo == null
                || !catVo.CanFecal()
                || this.addMap.hasOwnProperty(catVo.objId)
                || CatFecalBallModel.Inst.GetFecal(catVo.objId) != null
                || CatFecalBallModel.Inst.IsFull()) {
                return;
            }
            Logger.log(this, "---------制造粪球---------");
            let isSpeacil = MathUtil.random(CatFecalBallModel.Inst.HighHairball);
            let resName = catVo.catRes.catball;
            if (isSpeacil) {
                resName = catVo.catRes.specialball;
            }
            this.addMap[catVo.objId] = Laya.timer.currTimer;
            LoadQueue.Inst.loadSprite3D(UrlUtils.GetBuildUrl(resName), Laya.Handler.create(self, self.addbase, [catVo, isSpeacil]));
        }
        MakeFecals(count = 1) {
            let curCount = CatFecalBallModel.Inst.GetCount();
            if (curCount > count)
                return;
            let offestCount = count - curCount;
            for (let i = 0; i < offestCount; i++) {
                let catVo = CatModel.Inst.GetOwnedRandomOne();
                this.makeFecal(catVo);
            }
        }
        addbase(vo, isSpeacil, entity) {
            if (!entity || entity.destroyed)
                return;
            delete this.addMap[vo.objId];
            let catFecal = new CatFecalBall(entity, isSpeacil);
            catFecal.SetVO(vo);
            SceneMgr.Inst.AddFecalBall(catFecal.objId, entity);
            CatFecalBallModel.Inst.AddFecal(vo.objId, catFecal);
            if (CatFecalBallModel.Inst.GetCount() == 1) {
                Laya.timer.loop(10000, this, this.CheckDisapper);
            }
        }
        CheckDisapper() {
            let catFecal = null;
            let map = CatFecalBallModel.Inst.GetFecals();
            for (const key in map) {
                catFecal = map[key];
                if (catFecal && catFecal.IsCanDisapper()) {
                    SceneMgr.Inst.RemoveFecalBall(catFecal.objId, catFecal.owner);
                    CatFecalBallModel.Inst.RemoveFecal(catFecal.catVO.objId);
                    if (CatFecalBallModel.Inst.GetCount() <= 0) {
                        Laya.timer.clear(this, this.CheckDisapper);
                    }
                }
            }
        }
        ClickFecal(fecal) {
            if (!fecal)
                return;
            PlatformMgr.Inst.aldSendEvent(EventType.maoqiu, "所有点击");
            let from = new Laya.Point(Laya.stage.mouseX, Laya.stage.mouseY);
            if (!fecal.isSpeacil) {
                this.pickFecalBall(fecal, from);
                if (AdCtrl.Inst.PickupFecalBallShowBox) {
                    UISystem.Inst.CreateWindowView(UIBoxView, true);
                }
            }
            else {
                UISystem.Inst.CreateWindowView(UISpecialCatBallAlert, {
                    "handler": Laya.Handler.create(this, this.pickFecalBall, [fecal, from]),
                    "closeHandler": Laya.Handler.create(this, this.removeFecalBall, [fecal])
                });
            }
        }
        pickFecalBall(fecal, from) {
            let total = CatFecalBallModel.Inst.GetFecalGoldAward();
            if (fecal.isSpeacil) {
                total = total.multipliedBy(CatFecalBallModel.Inst.HighHairballmultiple);
            }
            MoneyModel.gold.add(total);
            EffectMgr.Inst.playGoldEffect(total, from);
            this.removeFecalBall(fecal);
            SoundMgr.Inst.playSound(UrlUtils.GetMusic(MusicType.getHairBall));
            GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, TaskType.TASKTYPE_GET_MAOQIU);
            GameDispatcher.Inst.event(UIGameEvent.DAYTASK_COMP_PROP, TaskVO.JIANQIU);
        }
        removeFecalBall(fecal) {
            if (fecal) {
                SceneMgr.Inst.RemoveFecalBall(fecal.catVO.objId + "", fecal.owner);
                CatFecalBallModel.Inst.RemoveFecal(fecal.catVO.objId);
                fecal.dispose();
            }
            if (CatFecalBallModel.Inst.GetCount() <= 0) {
                Laya.timer.clear(this, this.CheckDisapper);
            }
        }
    }
    CatFecalBallCtrl.inst = null;

    class TaskModel {
        constructor() {
            this.taskVos = [];
            this.taskMap = {};
            this.mainTaskVos = [];
            this.mainTaskMap = {};
            this.mainTaskVo = null;
            this.mainTaskIndex = 0;
            this.dayTaskVos = [];
            this.isGetDayTaskAllComp = false;
            this.DAYTASKS_COMP_DIMA = 10;
            this.DAYTASKS_COMP_AWARDID = 2;
            this.openDayTaskID = 15;
            this.moveHouseTaskID = 51;
        }
        static get Inst() {
            return this.inst = this.inst || new TaskModel();
        }
        init() {
            this.DAYTASKS_COMP_DIMA = parseInt(ConfigValue.getItemByKey("DailyTaskGem").value);
            let obj = StorageUtils.Inst.getMainTask();
            this.initDayTask();
            let vo = null;
            let res;
            for (let i = 0; i < taskResource.items.length; i++) {
                res = taskResource.items[i];
                vo = new TaskVO(res.id);
                this.taskVos.push(vo);
                this.taskMap[res.id] = vo;
            }
            let mainVo = null;
            let mainRes;
            for (let i = 0; i < principalResource.items.length; i++) {
                mainRes = principalResource.items[i];
                mainVo = new MainTaskVo(mainRes.id);
                this.mainTaskVos.push(mainVo);
                this.mainTaskMap[mainRes.id] = mainVo;
                if (obj.taskId == mainRes.id) {
                    this.mainTaskIndex = i;
                }
            }
            this.mainTaskVo = this.getTaskVoBy(obj.taskId, 1);
            this.mainTaskVo && this.mainTaskVo.setup(obj);
            obj = StorageUtils.Inst.getDayTask();
            let objList = obj.list;
            for (let i = 0; i < objList.length; i++) {
                let objItem = objList[i];
                let taskItem = this.getTaskVoBy(objItem.taskId, 0);
                taskItem.setup(objItem);
                this.dayTaskVos.push(taskItem);
            }
            this.isGetDayTaskAllComp = obj.award;
            GameDispatcher.Inst.on(UIGameEvent.DAYTASK_COMP_PROP, this, this.OnDayTaskComp);
        }
        get dayTaskVOArr() {
            return this.dayTaskVos;
        }
        get dayTaskPro() {
            let count = 0;
            for (let i = 0; i < this.dayTaskVOArr.length; i++) {
                let item = this.dayTaskVOArr[i];
                count += item.curPro;
            }
            return count;
        }
        get dayTaskNum() {
            let count = 0;
            for (let i = 0; i < this.dayTaskVOArr.length; i++) {
                count += this.dayTaskVOArr[i].maxCount;
            }
            return count;
        }
        get dayAllTaskComp() {
            for (let i = 0; i < this.dayTaskVOArr.length; i++) {
                let item = this.dayTaskVOArr[i];
                if (!(item.canAward || item.isCompleted)) {
                    return false;
                }
            }
            return true;
        }
        get isGetDaysTask() {
            return this.isGetDayTaskAllComp;
        }
        get allDayTaskState() {
            if (this.isGetDaysTask) {
                return BaseTaskVO.STATE_COMPLETED;
            }
            else if (this.dayAllTaskComp) {
                return BaseTaskVO.STATE_CAN_AWARD;
            }
            else {
                return BaseTaskVO.STATE_NORMAL;
            }
        }
        getTaskVoBy(tempId, type = 0) {
            if (type == 0) {
                return this.taskMap[tempId];
            }
            else {
                return this.mainTaskMap[tempId];
            }
        }
        get mainTask() {
            return this.mainTaskVo;
        }
        set mainTask(taskvo) {
            this.mainTaskVo = taskvo;
        }
        get allMainTaskIsComped() {
            return (this.mainTaskIndex >= this.mainTaskVos.length - 1) && (this.mainTask && this.mainTask.isCompleted);
        }
        SetNextMainTask() {
            if (this.mainTaskIndex >= this.mainTaskVos.length - 1) {
                GameDispatcher.Inst.event(UIGameEvent.UPDATE_TASK, this.mainTaskVo.type);
                return;
            }
            else {
                this.mainTaskIndex++;
                this.mainTaskVo = this.mainTaskVos[this.mainTaskIndex];
                if (this.mainTaskVo.taskRes.id == 13) {
                    this.mainTaskIndex++;
                    this.mainTaskVo = this.mainTaskVos[this.mainTaskIndex];
                }
            }
            if (!this.mainTaskVo)
                return;
            this.mainTaskVo.state = BaseTaskVO.STATE_NORMAL;
            this.mainTaskVo.pro = 0;
            StorageUtils.Inst.updateMainTask(this.mainTaskVo.tempId, this.mainTaskVo.pro, this.mainTaskVo.state);
            let mainTaskType = this.mainTask.taskRes.taskType;
            let id = this.mainTask.taskRes.otherCost;
            GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, [mainTaskType, id, 0]);
            GameDispatcher.Inst.event(UIGameEvent.UPDATE_TASK, this.mainTaskVo.type);
        }
        GetMainTaskVos() {
            return this.mainTaskVos;
        }
        initDayTask() {
            let obj = StorageUtils.Inst.getDayTask();
            if (obj == null) {
                let taskItems = taskResource.items;
                let taskResArr = MathUtil.GetRandomItemByWeight(taskItems, 3);
                let list = [];
                for (let i = 0; i < taskResArr.length; i++) {
                    let taskRes = taskResArr[i];
                    let taskObj = { "taskId": taskRes.id, "pro": 0, "state": BaseTaskVO.STATE_NORMAL };
                    list.push(taskObj);
                }
                StorageUtils.Inst.InitDayTaskList(list);
            }
        }
        get isOpenDayTask() {
            return !this.mainTask || this.mainTask.tempId >= this.openDayTaskID;
        }
        OnDayTaskComp(viceType) {
            if (!this.isOpenDayTask)
                return;
            for (let i = 0; i < this.dayTaskVOArr.length; i++) {
                let item = this.dayTaskVOArr[i];
                if (item.taskRes.viceType == viceType) {
                    item.curPro++;
                    if (viceType == TaskVO.JIANQIU) {
                        CatFecalBallCtrl.Inst.MakeFecals(item.maxCount - item.curPro);
                    }
                    break;
                }
            }
        }
        CheckIsHideBtnRed() {
            if (this.mainTask && this.mainTask.canAward) {
                GameDispatcher.Inst.event(UIGameEvent.SHOW_TASKBTN_RED, true);
                return;
            }
            for (let i = 0; i < this.dayTaskVOArr.length; i++) {
                if (this.dayTaskVOArr[i].canAward) {
                    GameDispatcher.Inst.event(UIGameEvent.SHOW_TASKBTN_RED, true);
                    return;
                }
            }
            if (this.allDayTaskState == BaseTaskVO.STATE_CAN_AWARD) {
                GameDispatcher.Inst.event(UIGameEvent.SHOW_TASKBTN_RED, true);
                return;
            }
            GameDispatcher.Inst.event(UIGameEvent.SHOW_TASKBTN_RED, false);
        }
        GetTaskIsComplete(taskId) {
            return this.mainTaskVo && this.mainTaskVo.tempId > taskId;
        }
    }
    TaskModel.inst = null;
    TaskModel.TASKTYPE_OTHER = 99999;

    class TaskVO extends BaseTaskVO {
        constructor(tempId) {
            super(tempId, 0);
        }
        setup(data) {
            this.pro = data.pro;
            this.state = data.state;
        }
        get taskRes() {
            return taskResource.getItemByKey(this.tempId);
        }
        get curPro() {
            return this.pro;
        }
        set curPro(val) {
            if (this.pro == this.maxCount)
                return;
            this.pro = val;
            if (this.pro > this.maxCount) {
                this.pro = this.maxCount;
            }
            if (this.pro == this.maxCount) {
                this.state = BaseTaskVO.STATE_CAN_AWARD;
                GameDispatcher.Inst.event(UIGameEvent.SHOW_TASKBTN_RED, true);
                if (TaskModel.Inst.dayAllTaskComp) {
                    StorageUtils.Inst.freeLockDrawCount++;
                    GameDispatcher.Inst.event(UIGameEvent.MAINTASK_COMP_PROP, TaskType.TASKTYPE_DAYTASKCOMP);
                }
            }
            StorageUtils.Inst.SaveDayTask(this.tempId, this.pro, this.state);
            GameDispatcher.Inst.event(UIGameEvent.UPDATE_TASK, this.type);
        }
        get maxCount() {
            return this.taskRes.cost;
        }
    }
    TaskVO.MOMAO = 1;
    TaskVO.JIANQIU = 2;
    TaskVO.SONGLI = 3;
    TaskVO.SONGPIFU = 4;
    TaskVO.KANGUANGGAO = 5;
    TaskVO.YAOQINGSHANGAN = 6;
    TaskVO.PAIZHAO = 7;

    class UIHWNativeBannerView extends UIBaseWindow {
        constructor() {
            super("ad", "UIHWNativeBannerView");
            this.adRes = null;
            this.adData = null;
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.ON_ACTIVE, this.onActive, this);
        }
        RemoveListener() {
            super.RemoveListener();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_AD;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowMaskBg = false;
            this.viewStruct.isExclusion = false;
            this.viewStruct.isEffect = false;
        }
        InitUI() {
            super.InitUI();
            this.btn_dowload.playSound = false;
            this.c1 = this._view.getController("c1");
        }
        InitData(data) {
            super.InitData(data);
            this.adRes = data ? data[0] : null;
            this.adData = data ? data[1] : null;
            this.nativeAd = data ? data[2] : null;
            this.updateAd(this.adRes, this.adData, this.nativeAd);
        }
        updateAd(adRes, adData, nativeAd) {
            this.adRes = adRes;
            this.adData = adData;
            this.nativeAd = nativeAd;
            this.showAd(adRes, adData);
        }
        showAd(adRes, nativeData) {
            if (nativeData == null) {
                return;
            }
            let self = this;
            const pics = [];
            if (nativeData.icon && nativeData.icon.length > 0) {
                if (typeof nativeData.icon === "string") {
                    pics.push(nativeData.icon);
                }
            }
            const imgUrlList = [];
            if (nativeData.imgUrlList) {
                for (let i = 0; i < nativeData.imgUrlList.length; i++) {
                    if (nativeData.imgUrlList[i]) {
                        imgUrlList.push(nativeData.imgUrlList[i]);
                    }
                }
            }
            if (imgUrlList && imgUrlList.length > 0) {
                if (this.img_urlListIcon)
                    this.img_urlListIcon.url = decodeURIComponent(imgUrlList[0]);
            }
            this.visible = true;
            let title = nativeData.title;
            this.c1.selectedIndex = pics.length > 0 ? 1 : 0;
            if (this.c1.selectedIndex == 1) {
                this.img_icon.url = decodeURIComponent(pics[0]);
            }
            this.txt_title.text = title;
            if (Laya.Browser.onHWMiniGame) {
                if (this.adData.source) {
                    this.txt_desc.text = this.adData.source;
                }
            }
            else {
                if (this.adData.desc) {
                    this.txt_desc.text = this.adData.desc;
                }
            }
            PlatformMgr.Inst.reportNativeShow(this.adData, this.nativeAd);
            PlatformModel.Inst.setBannerRect(this.group_banner.x, this.group_banner.y, this.group_banner.width, this.group_banner.height);
        }
        onBtnClick(btn) {
            if (btn == this.btn_close) {
                this.close();
                StorageUtils.Inst.setCloseNBannerCount();
                PlatformMgr.Inst.hideAd(AdType.ad_1, true);
                GameDispatcher.Inst.event(UIGameEvent.HIDE_NATIVE, this.adRes);
                AdModel.Inst.closeBannerTime = Laya.timer.currTimer;
            }
            else if (btn == this.btn_dowload) {
                if (this.adData) {
                    PlatformMgr.Inst.reportNativeClick(this.adData, this.nativeAd);
                    Laya.timer.once(600, this, this.close);
                }
            }
        }
        onActive(isActive) {
            if (isActive) {
                this.close();
            }
        }
        hideInsertAd() {
        }
        hideBanner() {
        }
        Reset() {
            this.isReset = true;
            fgui.GTween.kill(this._view);
            this.m_param = null;
            Laya.timer.clearAll(this);
            this.nativeAd = null;
            this.adData = null;
            this.img_icon.url = "";
        }
    }

    class UIHWNativeInsertView extends UIBaseWindow {
        constructor() {
            super("ad", "UIHWNativeInsertView");
            this.lab_des = null;
            this.lab_title = null;
            this.img_bg = null;
            this.icon_bg = null;
            this.btn_touming = null;
            this.group_center = null;
        }
        InitUI() {
            super.InitUI();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_AD;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowMaskBg = false;
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.ON_ACTIVE, this.onActive, this);
        }
        RemoveListener() {
            super.RemoveListener();
        }
        InitData(param) {
            super.InitData(param);
            console.log("[UIHWNativeInsertView]------->show : " + param);
            if (param == null) {
                return;
            }
            this.adRes = param[0];
            this.adData = param[1];
            this.nativeAd = param[2];
            this.showData(this.adData);
            this.hideBanner();
            Laya.timer.frameLoop(1, this, this.checkBannerShow);
        }
        checkBannerShow() {
            if (UISystem.Inst.HasView(UIHWNativeBannerView)) {
                UISystem.Inst.RemoveWindowClass(UIHWNativeBannerView);
            }
        }
        showData(data) {
            this.adData = data;
            if (this.adData) {
                let imgUrlList = this.adData.imgUrlList;
                if (imgUrlList && imgUrlList[0]) {
                    this.icon_bg.url = imgUrlList[0];
                }
                if (this.adData.title) {
                    this.lab_title.text = this.adData.title;
                }
                if (Laya.Browser.onHWMiniGame) {
                    if (this.adData.source) {
                        this.lab_des.text = this.adData.source;
                    }
                }
                else {
                    if (this.adData.desc) {
                        this.lab_des.text = this.adData.desc;
                    }
                }
                this.reportNativeShow(this.adData, this.nativeAd);
            }
        }
        onActive(bol) {
            if (bol) {
                this.close();
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_touming || btn == this.btn_xq) {
                this.reportNativeClick(this.adData, this.nativeAd);
                this.close();
            }
        }
        reportNativeShow(nativeData, nativeAd = null) {
            Logger.debug(this, "=====>原生插屏 上报显示!");
            PlatformMgr.Inst.reportNativeShow(nativeData, nativeAd);
        }
        onBtnCloseClick() {
            super.onBtnCloseClick();
        }
        reportNativeClick(nativeData, nativeAd = null) {
            Logger.debug(this, "=====>原生插屏 上报点击!");
            PlatformMgr.Inst.reportNativeClick(nativeData, nativeAd);
        }
        hideBanner() {
        }
        Reset() {
            super.Reset();
            Laya.timer.clear(this, this.checkBannerShow);
            PlatformMgr.Inst.hideAd(AdType.ad_3, true);
            this.nativeAd = null;
            this.adData = null;
            this.icon_bg.url = "";
        }
    }

    class PlatformCtrl {
        constructor() {
            this.nativeBannerShowTime = NaN;
        }
        static get Inst() {
            return this.inst = this.inst || new PlatformCtrl();
        }
        init() {
            GameDispatcher.Inst.on(UIGameEvent.SHOW_AD_COMPLETE, this, this.onShowAdComplete);
            if (PlatformMgr.Inst.isWX) {
                GameDispatcher.Inst.on(UIGameEvent.SHOW_BANNER, this, this.onShowBanner);
                GameDispatcher.Inst.on(UIGameEvent.HIDE_BANNER, this, this.onHideBanner);
                GameDispatcher.Inst.on(UIGameEvent.ON_SHOW, this, this.onShow);
                GameDispatcher.Inst.on(UIGameEvent.ON_HIDE, this, this.onHide);
            }
            else if (Laya.Browser.onQGMiniGame) {
                GameDispatcher.Inst.on(UIGameEvent.SHOW_BANNER, this, this.onShowNativeBanner);
                GameDispatcher.Inst.on(UIGameEvent.HIDE_BANNER, this, this.onHideNativeBanner);
                GameDispatcher.Inst.on(UIGameEvent.ON_SHOW, this, this.onShow);
                GameDispatcher.Inst.on(UIGameEvent.ON_HIDE, this, this.onHide);
                Laya.timer.loop(1000, this, this.CheckNativeBannerShow);
            }
            else if (Laya.Browser.onHWMiniGame) {
                GameDispatcher.Inst.on(UIGameEvent.SHOW_BANNER, this, this.onShowBanner);
                GameDispatcher.Inst.on(UIGameEvent.HIDE_BANNER, this, this.onHideBanner);
                GameDispatcher.Inst.on(UIGameEvent.ON_SHOW, this, this.onShow);
                GameDispatcher.Inst.on(UIGameEvent.ON_HIDE, this, this.onHide);
            }
            else if (Laya.Browser.onVVMiniGame) {
                GameDispatcher.Inst.on(UIGameEvent.SHOW_BANNER, this, this.onShowNativeBanner);
                GameDispatcher.Inst.on(UIGameEvent.HIDE_BANNER, this, this.onHideNativeBanner);
                GameDispatcher.Inst.on(UIGameEvent.ON_SHOW, this, this.onShow);
                GameDispatcher.Inst.on(UIGameEvent.ON_HIDE, this, this.onHide);
                Laya.timer.loop(1000, this, this.CheckHWNativeBannerShow);
            }
            else if (Laya.Browser.onQQMiniGame) {
                GameDispatcher.Inst.on(UIGameEvent.SHOW_BANNER, this, this.onShowBanner);
                GameDispatcher.Inst.on(UIGameEvent.HIDE_BANNER, this, this.onHideBanner);
                GameDispatcher.Inst.on(UIGameEvent.ON_SHOW, this, this.onShow);
                GameDispatcher.Inst.on(UIGameEvent.ON_HIDE, this, this.onHide);
            }
            else if (PlatformMgr.Inst.isMZ) {
                GameDispatcher.Inst.on(UIGameEvent.SHOW_BANNER, this, this.onShowBanner);
                GameDispatcher.Inst.on(UIGameEvent.HIDE_BANNER, this, this.onHideBanner);
                GameDispatcher.Inst.on(UIGameEvent.ON_SHOW, this, this.onShow);
                GameDispatcher.Inst.on(UIGameEvent.ON_HIDE, this, this.onHide);
            }
            else if (Laya.Browser.onBDMiniGame) {
                GameDispatcher.Inst.on(UIGameEvent.SHOW_BANNER, this, this.onShowBanner);
                GameDispatcher.Inst.on(UIGameEvent.HIDE_BANNER, this, this.onHideBanner);
                GameDispatcher.Inst.on(UIGameEvent.ON_SHOW, this, this.onShow);
                GameDispatcher.Inst.on(UIGameEvent.ON_HIDE, this, this.onHide);
            }
            else if (Laya.Browser.onKGMiniGame) {
                GameDispatcher.Inst.on(UIGameEvent.SHOW_BANNER, this, this.onShowBanner);
                GameDispatcher.Inst.on(UIGameEvent.HIDE_BANNER, this, this.onHideBanner);
                GameDispatcher.Inst.on(UIGameEvent.ON_SHOW, this, this.onShow);
                GameDispatcher.Inst.on(UIGameEvent.ON_HIDE, this, this.onHide);
            }
        }
        onShow(panelName) {
            if (!UISystem.Inst.hasAnyViewOnMain([UIGuideView, UITipView, UILoadingLeafView])) {
                this.reshowBanner();
            }
        }
        onHide(panelName) {
            if (!UISystem.Inst.hasAnyViewOnMain([UIGuideView, UITipView, UILoadingLeafView])) {
                this.reshowBanner();
            }
            if (Laya.Browser.onQGMiniGame)
                AdCtrl.Inst.ClearShowNatiEmbedTimer();
        }
        onShowBanner(view, data) {
            if (Laya.Browser.onPC) {
                return;
            }
            let topView = UISystem.Inst.getTopWindow([UIHWNativeBannerView, UIHWNativeInsertView, UIGuideView, UITipView, UILoadingLeafView]);
            if (topView instanceof UIBaseWindow && topView.isShowBanner && GuideModel.Inst.IsCanShowBtns()) {
                Logger.debug(this, "=====>显示banner view: " + topView.panelName);
                AdModel.Inst.closeBannerTime = 0;
                PlatformMgr.Inst.showBanner(data);
            }
        }
        onHideBanner(view) {
            this.reshowBanner();
        }
        reshowBanner(destroy = false) {
            let topView = UISystem.Inst.getTopWindow([UIGuideView, UITipView, UILoadingLeafView]);
            if (topView instanceof UIBaseWindow && topView.isShowBanner && GuideModel.Inst.IsCanShowBtns()) {
                topView.reshowBanner();
            }
            else {
                PlatformMgr.Inst.hideBanner(destroy);
            }
        }
        onShowAdComplete(adRes, isSuccess, data = null) {
            if (adRes) {
                if (adRes.type == AdType.TYPE_Banner) {
                    let time = data;
                    if (PlatformMgr.Inst.isWX) {
                        Laya.timer.clear(this, this.checkBannerShow);
                        Laya.timer.loop(30000, this, this.checkBannerShow, [time]);
                    }
                }
                else if (adRes.type == AdType.TYPE_RewardedVideo) {
                    if (isSuccess) {
                        GameDispatcher.Inst.event(UIGameEvent.DAYTASK_COMP_PROP, TaskVO.KANGUANGGAO);
                    }
                }
            }
        }
        checkBannerShow(time) {
            if (time > 0 && (TimeModel.Inst.LocalTime - time) > 60000) {
                console.log("banner60s时销毁重新拉取");
                PlatformMgr.Inst.hideBanner(true);
                this.reshowBanner();
            }
        }
        onShowNativeBanner() {
            if (Laya.Browser.onPC) {
                return;
            }
            let topView = UISystem.Inst.getTopWindow([UIGuideView, UITipView, UILoadingLeafView]);
            if (topView instanceof UIBaseWindow && topView.isShowBanner && GuideModel.Inst.IsCanShowBtns()) {
                PlatformMgr.Inst.showAd(AdType.TYPE_NativeBaner);
                this.nativeBannerShowTime = Laya.timer.currTimer;
                AdModel.Inst.closeBannerTime = 0;
            }
        }
        onHideNativeBanner() {
            PlatformMgr.Inst.hideAd(AdType.TYPE_NativeBaner, true);
            this.nativeBannerShowTime = Laya.timer.currTimer;
        }
        CheckNativeBannerShow() {
            if (!isNaN(this.nativeBannerShowTime) && Laya.timer.currTimer - this.nativeBannerShowTime > AdModel.Inst.nativeBannerUpdateTime && AdModel.Inst.nativeBannerUpdateTime != 0) {
                let topView = UISystem.Inst.getTopWindow([UIGuideView, UITipView, UILoadingLeafView, UIAdNativeBannerView, UINativeIconAd]);
                if (topView instanceof UIBaseWindow && topView.isShowBanner && GuideModel.Inst.IsCanShowBtns()) {
                    PlatformMgr.Inst.hideAd(AdType.TYPE_NativeBaner, true);
                    topView.reshowBanner();
                }
            }
        }
        CheckHWNativeBannerShow() {
            if (this.nativeBannerShowTime > 0 && Laya.timer.currTimer - this.nativeBannerShowTime > 40000) {
                let topView = UISystem.Inst.getTopWindow([UIGuideView, UITipView, UILoadingLeafView, UIAdNativeBannerView, UINativeIconAd]);
                if (topView instanceof UIBaseWindow) {
                    if (topView.isShowBanner) {
                        Logger.debug(this, "=======>banner 40s刷新 topView.name: " + topView.name);
                        if (GuideModel.Inst.IsCanShowBtns()) {
                            PlatformMgr.Inst.hideAd(AdType.TYPE_NativeBaner, true);
                            Laya.timer.once(600, this, () => {
                                topView.reshowBanner();
                            });
                        }
                    }
                    else {
                        PlatformMgr.Inst.hideAd(AdType.TYPE_NativeBaner, true);
                    }
                }
            }
            this.CheckBannerReShow();
        }
        CheckBannerReShow() {
            let topView = UISystem.Inst.getTopWindow([UIGuideView, UITipView, UILoadingLeafView, UIAdNativeBannerView, UINativeIconAd]);
            if (topView instanceof UIBaseWindow && topView.isShowBanner && GuideModel.Inst.IsCanShowBtns() && !UISystem.Inst.HasView(UIAdNativeBannerView)) {
                if (AdModel.Inst.closeBannerTime > 0 && (Laya.timer.currTimer - AdModel.Inst.closeBannerTime) > 30000) {
                    AdModel.Inst.closeBannerTime = 0;
                    PlatformMgr.Inst.hideAd(AdType.ad_1, true);
                    topView.reshowBanner();
                    Logger.debug(this, "=====>banner关闭后 30s刷新");
                }
            }
        }
    }
    PlatformCtrl.inst = null;

    class HttpUrl {
        constructor() {
            this.m_success = null;
            this.m_fail = null;
            this.m_url = null;
            this.index = 0;
            this.xhr = null;
        }
        sendGet(url, success = null, fail = null) {
            this.m_url = url;
            this.m_success = success;
            this.m_fail = fail;
            this.xhr = new Laya.HttpRequest();
            this.xhr.http.timeout = 10000;
            this.xhr.once(Laya.Event.COMPLETE, this, this.completeHandler);
            this.xhr.once(Laya.Event.ERROR, this, this.errorHandler);
            this.xhr.send(url, "", "get", "text");
        }
        static sendPost(url, params = null, success = null, fail = null) {
            let httpRequest = new Laya.HttpRequest();
            httpRequest.http.timeout = 30000;
            httpRequest.once(Laya.Event.COMPLETE, this, (data) => {
                if (data)
                    console.log("===>sendPost success: " + JSON.stringify(data));
                success && success.runWith(data);
            });
            httpRequest.once(Laya.Event.ERROR, this, (data) => {
                if (data)
                    console.log("===>sendPost fail: " + JSON.stringify(data));
                fail && fail.runWith(data);
            });
            httpRequest.send(url, params, "post", "json");
        }
        errorHandler(data) {
            this.clear();
            this.index++;
            if (this.index >= 3) {
                if (this.m_fail != null)
                    this.m_fail.run();
                return;
            }
            else {
                this.sendGet(this.m_url, this.m_success, this.m_fail);
            }
        }
        completeHandler(data) {
            this.clear();
            if (this.m_success != null)
                this.m_success.runWith(data);
        }
        clear() {
            this.xhr.off(Laya.Event.COMPLETE, this, this.completeHandler);
            this.xhr.off(Laya.Event.ERROR, this, this.completeHandler);
        }
    }

    class PhoneDeviceResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.brand = byteArray.readUTFString();
            this.model = byteArray.readUTFString();
            this.isLiuhai = byteArray.readUint8();
            this.platform = byteArray.readUTFString();
            this.hw_right_rect = ConfigLib.ReadArray(byteArray);
        }
        static getItemByKey(id) {
            return PhoneDeviceResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return PhoneDeviceResource.itemsDic[key1 + "," + key2];
        }
    }
    PhoneDeviceResource.items = new Array();
    PhoneDeviceResource.itemsDic = new Dictionary();

    class FpsPlatform {
        constructor() {
            this.isDownFps = false;
            this.curFps = 0;
            GameDispatcher.Inst.on(UIGameEvent.UPDATE_FPS, this, this.OnUpdateFps);
        }
        OnUpdateFps(fps) {
            console.log("切换帧频： " + fps);
            switch (fps) {
                case GlobalJugger.checkFps60:
                    this.SetChangeFps(this.isDownFps ? GlobalJugger.checkFps45 : fps);
                    break;
                case GlobalJugger.checkFps45:
                    this.SetChangeFps(this.isDownFps ? GlobalJugger.checkFps30 : fps);
                    break;
                case GlobalJugger.checkFps30:
                    this.SetChangeFps(this.isDownFps ? GlobalJugger.checkFps20 : fps);
                    break;
                case GlobalJugger.checkFps20:
                    this.SetChangeFps(this.isDownFps ? GlobalJugger.checkFps20 : fps);
                    break;
            }
            this.isDownFps = fps != GlobalJugger.checkFps60;
            this.SetSwitchFps(fps);
        }
        SetSwitchFps(fps) {
        }
        SetChangeFps(fps) {
            if (this.curFps == fps)
                return;
            this.curFps = fps;
            console.log("当前帧频： " + fps);
            switch (fps) {
                case GlobalJugger.checkFps60:
                    CatCtrl.Inst.SetCatMaxCount(catResource.items.length);
                    break;
                case GlobalJugger.checkFps45:
                    CatCtrl.Inst.SetCatMaxCount(20);
                    break;
                case GlobalJugger.checkFps30:
                    CatCtrl.Inst.SetCatMaxCount(15);
                    break;
                case GlobalJugger.checkFps20:
                    CatCtrl.Inst.SetCatMaxCount(10);
                    SceneMgr.Inst.GetCameraScript().SetMaxFovToNormal();
                    break;
            }
        }
    }

    class VideoPlatform extends FpsPlatform {
        constructor() {
            super();
            this.result = 0;
            this.openAd = 0;
            this.bannerCount = 0;
            this.bannerRate = 0;
            this.banner = "";
            this.video = [];
            this.insert = "";
            this.bannerId = "";
            this.native = [];
            this.box = "";
            this.nativeIconId = "";
            this.checkTime = 0;
            this.rewardVideo = null;
            this.rewardVideoIsLoad = false;
            this.handlers = null;
            this.handlers = Array();
        }
        StartCheck() {
            if (this.checkTime <= 0 || this.openAd == 0)
                return;
            Laya.timer.loop(this.checkTime, this, this.CheckRewardVideo, [null]);
        }
        CheckRewardVideo(handler) {
            if (this.rewardVideoIsLoad) {
                if (handler != null)
                    this.handlers.push(handler);
                return;
            }
            if (this.rewardVideo != null) {
                if (handler != null)
                    handler.run();
            }
            else {
                if (handler != null)
                    this.handlers.push(handler);
                this.rewardVideoIsLoad = true;
                this.LoadRewardVideo(handler, this.video.slice());
            }
        }
        LoadRewardVideo(handler, posIds) {
        }
        handlerRuns() {
            for (const iterator of this.handlers) {
                iterator.run();
            }
            this.handlers.length = 0;
        }
        Clear() {
            this.handlers.length = 0;
            this.rewardVideo = null;
            this.rewardVideoIsLoad = false;
        }
    }

    class BasePlatform extends VideoPlatform {
        constructor() {
            super();
            this.isInit = false;
            this.adHandlers = null;
            this.adDic = null;
            this.rate = 0;
            this.sizeWidth = 0;
            this.sizeHeight = 0;
            this.subNames = null;
            this.SDKVersion = "";
            this.systemInfo = null;
            this.appId = "";
            this.nativeInsertId = "";
            this.nativeBannerId = "";
            this.blockId = "";
            this.shareId = "";
            this.bannerAd = null;
            this.videoAd = null;
            this.rewardVideo = null;
            this.insertAd = null;
            this.nativeAd = null;
            this.blockAd = null;
            this.isLoadVideo = false;
            this.isShowRewardAd = false;
            this.isLoadInsert = false;
            this.isShowInsert = false;
            this._insertHandler = null;
            this.cacheBannerAds = [];
            this.cacheVideoAds = [];
            this.isCacheBanner = false;
            this.isCacheVideo = false;
            this.hasSubscribeApp = false;
            this.videoTime = 0;
            this.videoTimeMap = {};
            this.adHandlers = new Array();
            this.adDic = new Dictionary();
            this.subNames = new Dictionary();
            this.initAd(null);
        }
        GetPlatformName() {
            return null;
        }
        GetSubName(url) {
            for (const subName in this.subNames) {
                if (url.indexOf(subName) != -1) {
                    return this.subNames[subName];
                }
            }
            return null;
        }
        AddSubName(subName, resName) {
            if (!subName)
                return;
            if (resName == null)
                resName = subName;
            this.subNames[subName] = resName;
        }
        registeredSubNames() {
        }
        login(openId, channel, handler = null) {
        }
        setLoadingProgress(progress) {
            if (this.rate >= 100 || this.rate == progress)
                return;
            this.rate = progress;
            Logger.log(this, progress);
        }
        GetVersionName() {
            return "version.json";
        }
        navigateTo(value) {
        }
        onAddToFavorites(handler) {
        }
        GetAdVersionName() {
            return null;
        }
        initAd(handler) {
            let lf = this;
            if (lf.isInit) {
                handler.run();
                return;
            }
            if (handler != null)
                lf.adHandlers.push(handler);
            lf.isInit = true;
            for (const iterator of lf.adHandlers) {
                iterator.run();
            }
            lf.adHandlers.length = 0;
            let url = this.GetAdVersionName();
            if (url == null) {
                lf.StartCheck();
                return;
            }
            let http = new HttpUrl();
            http.sendGet(url, Laya.Handler.create(null, function (data) {
                console.log(data);
                let result = JSON.parse(data);
                this.onParseVersionJson(result);
                lf.StartCheck();
            }));
        }
        onParseVersionJson(result) {
        }
        hideAd(adId, destroy = true) {
            let adRes = AdResource.getItemBy2Keys(this.GetPlatformName(), adId);
            if (adRes == null) {
                if (!Laya.Browser.onPC) {
                    console.error("[BasePlatform]=====>广告表没有配置该广告ID: " + adId);
                }
                return;
            }
            if (destroy) {
                let ad = this.adDic[adRes.type];
                if (ad) {
                    ad && ad.destroy && ad.destroy();
                    delete this.adDic[adRes.type];
                    console.log("[BasePlatform]=====>销毁广告 adRes.type: " + adRes.type);
                }
            }
        }
        showAd(adId, handler = null, data = null) {
            let adRes = AdResource.getItemBy2Keys(this.GetPlatformName(), adId);
            if (adRes == null) {
                console.error("====>广告配置表没有配置该广告ID adId:" + adId, "请检查配置表!");
                handler && handler.runWith(false);
                return;
            }
            let delayTime = 0;
            let arr = adRes["result" + this.result];
            let jv = arr[0];
            if (MathUtil.random(jv)) {
                delayTime = arr[1];
            }
            if (delayTime > 0) {
                Laya.timer.once(delayTime * 1000, this, this.onDelay, [adRes, handler, data]);
            }
            else {
                this.onDelay(adRes, handler, data);
            }
        }
        onDelay(adRes, handler, data = null) {
            let lf = this;
            switch (adRes.type) {
                case AdType.TYPE_Banner:
                    lf.initAd(Laya.Handler.create(lf, lf.showBanner, [adRes, handler, data]));
                    break;
                case AdType.TYPE_RewardedVideo:
                    lf.initAd(Laya.Handler.create(lf, lf.showRewardedVideo, [adRes, handler]));
                    break;
                case AdType.TYPE_Insert:
                    lf.initAd(Laya.Handler.create(lf, lf.showInsert, [adRes, handler]));
                    break;
                case AdType.TYPE_NativeIcon:
                case AdType.TYPE_NativeBaner:
                case AdType.TYPE_NativeInsert:
                    lf.initAd(Laya.Handler.create(lf, lf.showNative, [adRes, handler, data]));
                    break;
                case AdType.TYPE_Box:
                    lf.initAd(Laya.Handler.create(lf, lf.showBox, [adRes, handler]));
                    break;
                case AdType.TYPE_Block:
                    lf.initAd(Laya.Handler.create(lf, lf.showBlock, [adRes, handler, data]));
                    break;
                default:
                    break;
            }
        }
        showBanner(adRes, handler, data = null) {
        }
        showRewardedVideo(adRes, handler) {
        }
        showInsert(adRes, handler) {
        }
        showNative(adRes, handler, data = null) {
        }
        showBox(adRes, handler) {
        }
        showBlock(adRes, handler, data = null) {
        }
        closeBanner() {
        }
        destroyBanner() {
        }
        hideInsert() {
            this._insertHandler = null;
        }
        reportNativeShow(nativeData, nativeAd = null) {
        }
        reportNativeClick(nativeData, nativeAd = null) {
        }
        vibrateShort() {
        }
        get canLookViewAd() {
            if (this.videoTime <= 0) {
                return true;
            }
            return Laya.timer.currTimer - this.videoTime >= 180000;
        }
        setLookVideoTime(adId, time = NaN) {
            let curTime = Laya.timer.currTimer;
            if (isNaN(time)) {
                this.videoTimeMap[adId] = curTime;
            }
            else {
                this.videoTimeMap[adId] = time;
            }
            let adRes = AdResource.getItemByKey(adId);
            if (adRes && adRes.type == AdType.TYPE_RewardedVideo) {
                this.videoTime = curTime;
            }
        }
        getLookVideoTime(adId) {
            let videoTime = this.videoTimeMap[adId];
            return !videoTime ? 0 : videoTime;
        }
        hasShortcutInstalled(handler) {
            if (handler != null)
                handler.run();
        }
        installShortcut(handler) {
            if (handler != null)
                handler.run();
        }
        getStorage(key, handler) {
            let data = Laya.LocalStorage.getJSON(key);
            if (data == null || data == "")
                data = {};
            if (handler != null)
                handler.runWith(data);
        }
        setStorage(key, value) {
            Laya.LocalStorage.setJSON(key, JSON.parse(value));
        }
        isShowNavigate() {
            return false;
        }
        isShowShare() {
            return false;
        }
        onShareAppMessage(handler) {
        }
        subscribeAppMsg(handler) {
        }
        checkNetwork() {
        }
        aldSendEvent(id, param = "") {
        }
        VisibleBaner(value) {
        }
        resizeBanner(bannerAd, data) {
            if (!bannerAd)
                return;
            let windowWidth = PlatformModel.Inst.windowWidth;
            let windowHeight = PlatformModel.Inst.windowHeight;
            let realWidth = bannerAd.style.realWidth || 320;
            let realHeight = bannerAd.style.realHeight || 111;
            let offsetX = data && data.hasOwnProperty("offsetX") ? data.offsetX : 0;
            let offsetY = data && data.hasOwnProperty("offsetY") ? data.offsetY : 0;
            if (data && data.hasOwnProperty("top")) {
                bannerAd.style.top = data.top;
            }
            else {
                bannerAd.style.top = windowHeight - realHeight + offsetY;
            }
            if (data && data.hasOwnProperty("isLeft") && data.isLeft == true) {
                bannerAd.style.left = offsetX;
            }
            else if (data && data.hasOwnProperty("isRight") && data.isRight == true) {
                bannerAd.style.left = windowWidth - realWidth + offsetX;
            }
            else {
                bannerAd.style.left = (windowWidth - realWidth) / 2 + offsetX;
            }
            PlatformModel.Inst.setBannerRect(bannerAd.style.left, bannerAd.style.top, realWidth, realHeight);
        }
        setKeepScreenOn(flag = true) {
        }
        get isLiuhai() {
            if (Laya.Browser.onPC) {
                return false;
            }
            let pres = PhoneDeviceResource.getItemBy2Keys(this.brand, this.model);
            if (pres) {
                return pres.isLiuhai == 1;
            }
            if (Laya.stage.screenMode == Laya.Stage.SCREEN_HORIZONTAL) {
                if ((Laya.Browser.clientWidth / Laya.Browser.clientHeight) > 2) {
                    return true;
                }
            }
            else {
                if ((Laya.Browser.clientHeight / Laya.Browser.clientWidth) > 2) {
                    return true;
                }
            }
            return false;
        }
        get brand() {
            return "";
        }
        get model() {
            return "";
        }
        get isReview() {
            return this.openAd == 0;
        }
        exitGame() {
        }
        compareVersion(v1, v2) {
            v1 = v1.split('.');
            v2 = v2.split('.');
            const len = Math.max(v1.length, v2.length);
            while (v1.length < len) {
                v1.push('0');
            }
            while (v2.length < len) {
                v2.push('0');
            }
            for (let i = 0; i < len; i++) {
                const num1 = parseInt(v1[i]);
                const num2 = parseInt(v2[i]);
                if (num1 > num2) {
                    return 1;
                }
                else if (num1 < num2) {
                    return -1;
                }
            }
            return 0;
        }
    }

    class PlatformType {
    }
    PlatformType.TYPE_NORMAL = "normal";
    PlatformType.TYPE_OPPO = "oppo";
    PlatformType.TYPE_VIVO = "vivo";
    PlatformType.TYPE_BAIDU = "baidu";
    PlatformType.TYPE_WEIXIN = "wx";
    PlatformType.TYPE_QQ = "qq";
    PlatformType.TYPE_TT = "tt";
    PlatformType.TYPE_MZ = "mz";
    PlatformType.TYPE_HUAWEI = "huawei";
    PlatformType.TYPE_XIAOMI = "xiaomi";
    PlatformType.TYPE_Android = "android";

    class AndroidPlatform extends BasePlatform {
        constructor() { super(); }
        GetPlatformName() {
            return PlatformType.TYPE_Android;
        }
        showAd(adId, handler = null, data = null) {
            if (handler != null)
                handler.run();
        }
        GetVersionName() {
            return "version.json";
        }
        hasShortcutInstalled(handler) {
        }
        installShortcut(handler) {
        }
    }

    class BaiduPlatform extends BasePlatform {
        constructor() {
            super();
            this.appSid = "";
            this.isLoadBanner = false;
            this.appId = "24670776";
            this.appSid = "dbf04f34";
            this.video = ["7636802"];
            this.bannerId = "7636801";
            this.enterMain();
        }
        registeredSubNames() {
        }
        enterMain() {
            let self = this;
            let obj = this.swan["getLaunchOptionsSync"]();
            let scene = obj.scene + "";
            console.log("=====>obj.scene: " + scene);
            HttpUrl.sendPost("https://ga.gametdd.com/adCtrl/agd/conf", { 'id': "60f124e3fac3fe0a5ccf7621", "sid": scene }, Laya.Handler.create(this, (res) => {
                console.log("====>策略 " + JSON.stringify(res));
                let data = res["data"];
                AdModel.Inst.addPolicy(data, this.GetPlatformName());
            }), Laya.Handler.create(this, () => {
            }));
            self.setKeepScreenOn(true);
            this.swan.onShow(function (res) {
                self.setKeepScreenOn(true);
                SoundMgr.Inst.Reset();
                GameDispatcher.Inst.event(UIGameEvent.ON_ACTIVE, true);
            });
            this.swan.onHide((res) => {
                SoundMgr.Inst.stop();
                GameDispatcher.Inst.event(UIGameEvent.ON_ACTIVE, false);
            });
            Laya.timer.once(2000, this, this.updateWindowSize);
        }
        updateWindowSize() {
            PlatformModel.Inst.platfrom = this.GetPlatformName();
            let obj = this.swan.getSystemInfoSync();
            PlatformModel.Inst.setWindowSize(obj.windowWidth, obj.windowHeight);
            if (this.swan["getMenuButtonBoundingClientRect"] != null) {
                let pos = this.swan["getMenuButtonBoundingClientRect"]();
                PlatformModel.Inst.setRect(pos.left, pos.top, pos.width, pos.height + 5);
            }
        }
        showBanner(adRes, handler, data = null) {
            let self = this;
            let offsetX = data && data.hasOwnProperty("offsetX") ? data.offsetX : 0;
            let offsetY = data && data.hasOwnProperty("offsetY") ? data.offsetY : 0;
            this.closeBanner();
            let obj = this.swan.getSystemInfoSync();
            let windowWidth = obj.windowWidth;
            let windowHeight = obj.windowHeight;
            let banObj = {
                adUnitId: self.bannerId,
                appSid: self.appSid,
                style: {
                    left: (windowWidth - 300) * 0.5,
                    top: windowHeight - 200,
                    width: 300,
                }
            };
            self.isLoadBanner = true;
            let bannerAd = this.swan.createBannerAd(banObj);
            bannerAd.onError(res => {
                if (res) {
                    Logger.log(self, 'bannerAd onError: ' + JSON.stringify(res));
                    console.log(JSON.stringify(res));
                }
                self.isLoadBanner = false;
                self.closeBanner();
            });
            bannerAd.onLoad(() => {
                self.isLoadBanner = false;
                bannerAd.show();
                if (handler != null) {
                    handler.runWith(true);
                }
            });
            resizeBanner();
            bannerAd.onResize(res => {
                console.log(res.width, res.height);
                resizeBanner();
            });
            function resizeBanner() {
                bannerAd.style.top = windowHeight - bannerAd.style.height + offsetY;
                if (data && data.hasOwnProperty("isLeft") && data.isLeft == true) {
                    bannerAd.style.left = offsetX;
                }
                else {
                    bannerAd.style.left = (windowWidth - bannerAd.style.width) / 2 + offsetX;
                }
            }
            this.bannerAd = bannerAd;
            this.adDic[adRes.type] = bannerAd;
            Laya.timer.clear(self, self.UpdateBanner);
            Laya.timer.loop(40000, self, self.UpdateBanner);
        }
        UpdateBanner() {
            let topView = UISystem.Inst.getTopWindow([UIGuideView, UITipView, UILoadingLeafView]);
            if (topView instanceof UIBaseWindow && topView.isShowBanner && GuideModel.Inst.IsCanShowBtns()) {
                this.closeBanner();
                topView.reshowBanner();
                console.log("banner广告40s刷新");
            }
        }
        closeBanner() {
            if (this.bannerAd != null) {
                try {
                    this.bannerAd.hide && this.bannerAd.hide();
                    this.bannerAd.destroy && this.bannerAd.destroy();
                }
                catch (e) {
                    Logger.log(this, "[BaiduPlatform]=====>关闭banner错误!");
                }
            }
            this.bannerAd = null;
            this.isLoadBanner = false;
        }
        showRewardedVideo(adRes, handler) {
            this.createRewardedVideo(adRes, handler, this.video.slice());
        }
        createRewardedVideo(adRes, handler, posIds) {
            let self = this;
            if (posIds.length <= 0 || self.isLoadVideo || self.isShowRewardAd) {
                return;
            }
            self.videoAd = this.swan.createRewardedVideoAd({ adUnitId: posIds.pop(), appSid: self.appSid });
            function onLoad() {
                self.videoAd.offLoad(onLoad);
                Logger.log(this, "激励视频加载成功");
                self.videoAd.show();
                self.isShowRewardAd = true;
                self.isLoadVideo = false;
                SoundMgr.Inst.stop();
                self.videoTime = StorageUtils.getCurTimer();
                self.setLookVideoTime(adRes.id);
            }
            self.videoAd.onLoad(onLoad);
            function onClose(res) {
                self.videoAd.offClose(onClose);
                GameDispatcher.Inst.event(UIGameEvent.SHOW_AD, [adRes, res.isEnded]);
                GameDispatcher.Inst.event(UIGameEvent.SHOW_AD_COMPLETE, [adRes, true, self.GetPlatformName()]);
                if (res.isEnded) {
                    Logger.log(this, '激励视频广告完成，发放奖励');
                }
                else {
                    Logger.log(this, '激励视频广告取消关闭，不发放奖励');
                    self.setLookVideoTime(adRes.id, 0);
                }
                if (handler != null) {
                    handler.runWith(res.isEnded);
                }
                self.isLoadVideo = false;
                self.isShowRewardAd = false;
                SoundMgr.Inst.Reset();
            }
            self.videoAd.onClose(onClose);
            function onError(err) {
                self.videoAd.offError(onError);
                if (posIds.length > 0) {
                    self.createRewardedVideo(adRes, handler, posIds);
                }
                else {
                    if (handler != null)
                        handler.runWith(false);
                    self.isShowRewardAd = false;
                    self.isLoadVideo = false;
                    SoundMgr.Inst.Reset();
                    I18nUtils.ShowI18nById("no_advertisement");
                }
            }
            self.videoAd.onError(onError);
            self.isLoadVideo = true;
            self.videoAd.load();
        }
        showInsert(adRes, handler) {
            let self = this;
            this._insertHandler = handler;
        }
        hideInsert() {
            this._insertHandler = null;
        }
        hideAd(adId, destroy = true) {
            let adRes = AdResource.getItemBy2Keys(this.GetPlatformName(), adId);
            if (adRes == null) {
                if (!Laya.Browser.onPC) {
                    console.error("[BasePlatform]=====>广告表没有配置该广告ID: " + adId);
                }
                return;
            }
            if (adRes.type == AdType.TYPE_Banner) {
                this.closeBanner();
            }
            else {
                super.hideAd(adId, destroy);
            }
        }
        onShareAppMessage(handler) {
            I18nUtils.ShowMsg("暂无分享");
        }
        GetPlatformName() {
            return PlatformType.TYPE_BAIDU;
        }
        vibrateShort() {
            this.swan.vibrateShort();
        }
        setKeepScreenOn(flag = true) {
            this.swan.setKeepScreenOn && this.swan.setKeepScreenOn({
                keepScreenOn: flag,
                success: function (result) {
                    console.info('设置屏幕常亮成功..');
                },
                fail() {
                    console.info('设置屏幕常亮失败..');
                }
            });
        }
        get swan() {
            return window["swan"];
        }
    }

    class UIAdNativeIconView extends UIBaseWindow {
        constructor() {
            super("ad", "UIAdNativeIconView");
            this.sp_icon = null;
            this.txt_title = null;
            this.btn_native = null;
            this.clickHandler = null;
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_MAIN;
            this.viewStruct.layout = 1;
            this.viewStruct.isExclusion = false;
        }
        InitUI() {
            this.sp_icon = this.btn_native.getChild("sp_icon").asLoader;
            this.txt_title = this.btn_native.getChild("txt_title").asTextField;
        }
        InitData(param) {
            let adObj = param[0];
            let showHandler = param[1];
            this.clickHandler = param[2];
            this.txt_title.text = adObj["title"];
            if (showHandler != null)
                showHandler.run();
            let lf = this;
            let url = adObj["icon"];
            if (!url) {
                let iconArr = adObj["logoUrl"];
                if (iconArr != null && iconArr.length > 0)
                    url = iconArr[0];
                else {
                    iconArr = adObj["imgUrlList"];
                    if (iconArr != null && iconArr.length > 0)
                        url = iconArr[0];
                }
            }
            if (!url)
                return;
            AssetCache.LoadIcon(this.sp_icon, url);
            Laya.timer.once(30000, this, this.onDelayShow, [adObj.adResId]);
        }
        onDelayShow(adResId) {
            PlatformMgr.Inst.hideAd(adResId);
            PlatformMgr.Inst.showAd(adResId);
        }
        AddListener() {
            super.AddListener();
            this.btn_native.onClick(this, this.onBtnOkClick);
        }
        RemoveListener() {
            super.RemoveListener();
            this.btn_native.offClick(this, this.onBtnOkClick);
        }
        onBtnOkClick() {
            if (this.clickHandler != null) {
                this.clickHandler.run();
            }
        }
        Reset() {
            super.Reset();
            Laya.timer.clear(this, this.onDelayShow);
            if (this.clickHandler) {
                this.clickHandler.recover();
                this.clickHandler = null;
            }
        }
    }

    class HWPlatform extends BasePlatform {
        constructor() {
            super();
            this.isShowAd = false;
            this.cacheNativeAds = [];
            this.nativeIsLoad = false;
            this.handler = null;
            this.nativeProbability = 10000;
            this.nativeData = null;
            this.showBannerTime = 0;
            this.cacheBannerAds = [];
            this.cacheVideoAds = [];
            this.bannerIds = [];
            this.updateBannerTime = 40;
            this.reshowBannerTime = 30;
            this.bigRect = new Laya.Rectangle(370, 180, 146, 49);
            this.smallRect = new Laya.Rectangle(508, 100, 194, 65);
            this.lastClickTime = 0;
            this.CLICK_INTERVAL = 500;
            this.cacheMap = {};
            this.bannerCloseTime = NaN;
            this.bannerCreateTime = NaN;
            this._prevNITime = 0;
            this.sendTimeMap = {};
            let lf = this;
            lf.result = 1;
            lf.openAd = 1;
            lf.checkTime = 0;
            lf.appId = "104584593";
            lf.nativeBannerId = "b3ouwudadt";
            lf.bannerIds = ["p8m0lsnm57"];
            lf.video = ["f6wwffjq30"];
            lf.nativeInsertId = "z4dhr30s92";
            this.enterMain();
        }
        registeredSubNames() {
        }
        enterMain() {
            let self = this;
            console.log("---------------------enterMain huawei------------" + hbs);
            this.systemInfo = qg && qg.getSystemInfoSync();
            this.setKeepScreenOn(true);
            let onHide = (qg && qg.onHide);
            let onShow = (qg && qg.onShow);
            onHide && onHide(function () {
                console.log("失去焦点");
                SoundMgr.Inst.stop();
                GlobalConfig.isActive = false;
                Laya.timer.pause();
                GameDispatcher.Inst.event(UIGameEvent.ON_ACTIVE, false);
            });
            onShow && onShow(function (res) {
                console.log("获得焦点");
                SoundMgr.Inst.Reset();
                GlobalConfig.isActive = true;
                Laya.timer.resume();
                self.setKeepScreenOn(true);
                GameDispatcher.Inst.event(UIGameEvent.ON_ACTIVE, true);
            });
            GameDispatcher.Inst.on(UIGameEvent.SHOW_NATIVE, this, this.showNativeAd);
            GameDispatcher.Inst.on(UIGameEvent.HIDE_NATIVE, this, this.hideNativeAd);
            Laya.timer.once(2000, this, this.updateWindowSize);
        }
        updateWindowSize() {
            PlatformModel.Inst.platfrom = this.GetPlatformName();
            if (this.systemInfo) {
                console.log("====>updateWindowSize obj: " + JSON.stringify(this.systemInfo));
            }
            if (qg && qg["getMenuButtonBoundingClientRect"] != null) {
                let pos = qg["getMenuButtonBoundingClientRect"]();
                console.log("====>右上角胶囊位置 left: " + pos.left + " top: " + pos.top + " width: " + pos.width + " height: " + pos.height);
                PlatformModel.Inst.setRect(pos.left, pos.top, pos.width, pos.height + 5);
            }
            else {
                if (PlatformModel.Inst.windowWidth > 720) {
                    console.log("=====> 大屏右上角位置 370, 180, 146, 49");
                    PlatformModel.Inst.setRect(this.bigRect.x, this.bigRect.y, this.bigRect.width, this.bigRect.height);
                }
                else {
                    console.log("=====> 小屏右上角 508, 100, 194, 65");
                    PlatformModel.Inst.setRect(this.smallRect.x, this.smallRect.y, this.smallRect.width, this.smallRect.height);
                }
            }
        }
        login(openId, channel, handler = null) {
            let self = this;
            handler && handler.runWith(true);
            this.cacheVideoAd();
        }
        showNativeAd(adRes, adData, nativeAd) {
            if (!adRes)
                return;
            console.log("-----------------showNativeAd adRes.id: " + adRes.id + " adRes.type: " + adRes.type);
            let self = this;
            if (adRes.type == AdType.TYPE_Banner || adRes.type == AdType.TYPE_NativeBaner) {
                if (UISystem.Inst.HasView(UIHWNativeInsertView)) {
                    return;
                }
                let view = UISystem.Inst.GetWindowView(UIHWNativeBannerView);
                if (view) {
                    if (view.parent) {
                        view.updateAd(adRes, adData, nativeAd);
                    }
                }
                else {
                    if (!UISystem.Inst.hasAnyShowBanner()) {
                        return;
                    }
                    UISystem.Inst.CreateWindowView(UIHWNativeBannerView, [adRes, adData, nativeAd]);
                    self.showBannerTime = Laya.timer.currTimer;
                }
            }
            else if (adRes.type == AdType.TYPE_Insert || adRes.type == AdType.TYPE_NativeInsert) {
                this.isShowInsert = true;
                UISystem.Inst.CreateWindowView(UIHWNativeInsertView, [adRes, adData, nativeAd]);
            }
        }
        hideNativeAd(adRes) {
            if (adRes && adRes.type == AdType.TYPE_Banner || adRes.type == AdType.TYPE_NativeBaner) {
                UISystem.Inst.RemoveWindowClass(UIHWNativeBannerView);
            }
            else if (adRes && adRes.type == AdType.TYPE_Insert || adRes.type == AdType.TYPE_NativeInsert) {
                this.isShowInsert = false;
                UISystem.Inst.RemoveWindowClass(UIHWNativeInsertView);
            }
        }
        setLoadingProgress(progress) {
            if (this.rate >= 100 || this.rate == progress)
                return;
            this.rate = progress;
            Logger.debug(this, "=====>setLoadingProgress progress: " + progress);
            qg && qg.setLoadingProgress && qg.setLoadingProgress({ progress: progress });
            if (this.rate >= 100) {
                qg && qg.loadingComplete && qg.loadingComplete({
                    complete: function (res) { }
                });
            }
        }
        get canLookViewAd() {
            if (this.videoTime <= 0) {
                return true;
            }
            return Laya.timer.currTimer - this.videoTime >= 120 * 1000;
        }
        IsFastClick() {
            if (!isNaN(this.lastClickTime) && this.lastClickTime != 0 && (new Date().getTime() - this.lastClickTime) < this.CLICK_INTERVAL) {
                return true;
            }
            this.lastClickTime = new Date().getTime();
            return false;
        }
        showRewardedVideo(adRes, handler) {
            if (this.isReview) {
                handler && handler.runWith(true);
                return;
            }
            if (this.IsFastClick()) {
                console.log("====>快速点击视频，播放失败!");
                return;
            }
            this.createRewardedVideo(adRes, handler, this.video.slice());
        }
        createRewardedVideo(adRes, handler, posIds) {
            let self = this;
            if (self.insertAd != null) {
                self.hideInsert();
            }
            let time = self.getLookVideoTime(adRes.id);
            if (time > 0 && (Laya.timer.currTimer - time) > 60000) {
                self.isLoadVideo = false;
                self.isShowRewardAd = false;
                if (self.rewardVideo != null) {
                    try {
                        self.rewardVideo.destroy();
                    }
                    catch (e) {
                        console.log("销毁视频错误！error: " + e);
                    }
                }
                self.rewardVideo = null;
            }
            console.log("[HWPlatform]=====> createRewardedVideo self.isLoadVideo: " + self.isLoadVideo +
                " self.isShowRewardAd: " + self.isShowRewardAd + " posIds.length: " + posIds.length + " this.cacheVideoAds.length: " + this.cacheVideoAds.length);
            if (posIds.length <= 0 || self.isLoadVideo || self.isShowRewardAd) {
                if (self.isLoadVideo) {
                    UISystem.Inst.CreateWindowView(UITipView, "广告加载中");
                }
                return;
            }
            let videoAd = this.cacheVideoAds.shift();
            if (videoAd != null) {
                self.showCacheVideoAd(videoAd, null, adRes, handler);
                return;
            }
            self.isLoadVideo = true;
            self.loadVideoAd(Laya.Handler.create(this, (videoAd) => {
                self.isLoadVideo = false;
                self.cacheVideoAd();
                if (videoAd == null) {
                    self.createRewardedVideo(adRes, handler, posIds);
                }
                else {
                    self.showCacheVideoAd(videoAd, null, adRes, handler);
                }
            }));
        }
        cacheVideoAd() {
            if (this.isReview) {
                return;
            }
            let self = this;
            if (this.isCacheVideo || this.cacheVideoAds.length >= 1) {
                return;
            }
            this.isCacheVideo = true;
            self.loadVideoAd(Laya.Handler.create(self, (videoAd) => {
                console.log('缓存激励视频加载成功');
                if (videoAd) {
                    self.cacheVideoAds.push(videoAd);
                }
                self.isCacheVideo = false;
                Laya.timer.once(10000, self, self.cacheVideoAd);
            }));
        }
        loadVideoAd(handler, posIds = null) {
            let self = this;
            posIds = posIds || this.video.slice();
            let posId = posIds.pop();
            console.log("加载视频广告id:", posId);
            if (posId) {
                let videoAd = qg.createRewardedVideoAd({ "adUnitId": posId });
                self.cacheMap[videoAd] = handler;
                function onLoad() {
                    videoAd.offLoad && videoAd.offLoad(onLoad);
                    console.log('激励视频加载成功');
                    let handler = self.cacheMap[videoAd];
                    if (handler) {
                        handler.runWith(videoAd);
                    }
                    delete self.cacheMap[videoAd];
                }
                videoAd.onLoad(onLoad);
                function onError(err) {
                    videoAd.offError && videoAd.offError(onLoad);
                    console.log('激励视频加载失败');
                    let handler = self.cacheMap[videoAd];
                    if (posIds.length > 0) {
                        self.loadVideoAd(handler, posIds);
                    }
                    else if (handler) {
                        handler.runWith(null);
                    }
                    delete self.cacheMap[videoAd];
                }
                videoAd.onError(onError);
                videoAd.load();
            }
        }
        showCacheVideoAd(videoAd, data, adRes, handler) {
            let self = this;
            console.log("显示的广告obj", videoAd);
            if (videoAd) {
                function onClose(res) {
                    videoAd.offClose && videoAd.offClose(onClose);
                    if (res.isEnded) {
                        Logger.log(self, '激励视频广告完成，发放奖励');
                    }
                    else {
                        Logger.log(self, '激励视频广告取消关闭，不发放奖励');
                        self.setLookVideoTime(adRes.id, 0);
                    }
                    self.playVideoError(null);
                    if (handler != null) {
                        handler.runWith(res.isEnded);
                        GameDispatcher.Inst.event(UIGameEvent.SHOW_AD, [adRes, res.isEnded]);
                        GameDispatcher.Inst.event(UIGameEvent.SHOW_AD_COMPLETE, [adRes, res.isEnded, self.GetPlatformName()]);
                    }
                }
                videoAd.onClose(onClose);
                videoAd.show();
                self.isShowRewardAd = true;
                self.isLoadVideo = false;
                SoundMgr.Inst.stop();
                self.videoTime = Laya.timer.currTimer;
                self.setLookVideoTime(adRes.id);
                Laya.timer.pause();
                self.rewardVideo = videoAd;
            }
        }
        playVideoError(handler) {
            this.isLoadVideo = false;
            this.isShowRewardAd = false;
            this.rewardVideo && this.rewardVideo.destroy && this.rewardVideo.destroy();
            this.rewardVideo = null;
            if (handler != null) {
                handler.runWith(false);
            }
            SoundMgr.Inst.Reset();
            Laya.timer.resume();
            this.cacheVideoAd();
        }
        showBanner(adRes, handler) {
            let lf = this;
            let ad = lf.adDic[adRes.type];
            if (ad) {
                ad.show && ad.show();
                return;
            }
            lf.CreateBanner(adRes, handler, lf.bannerIds.slice());
        }
        CreateBanner(adRes, handler, adIds) {
            let lf = this;
            var bannerAd = qg.createBannerAd({
                adUnitId: adIds.pop(),
                style: {
                    top: (lf.systemInfo.screenHeight - 57),
                    left: (lf.systemInfo.screenWidth - 360) / 2,
                    width: 360,
                    height: 57
                }
            });
            lf.adDic[adRes.type] = bannerAd;
            function onLoad() {
                bannerAd["offLoad"](onLoad);
                bannerAd.show();
                console.log('bannerAd onload success');
            }
            bannerAd["onLoad"] && bannerAd["onLoad"](onLoad);
            function onError(e) {
                bannerAd.offError(onError);
                console.log('bannerAd onError ' + JSON.stringify(e));
                lf.hideAd(AdType.ad_1);
                if (adIds.length > 0) {
                    console.log("重新创建banner:" + adIds.length);
                    lf.bannerCreateTime = NaN;
                    Laya.timer.once(5, lf, lf.CreateBanner, [adRes, handler, adIds]);
                }
            }
            bannerAd.onError && bannerAd.onError(onError);
            lf.bannerCloseTime = NaN;
            lf.bannerCreateTime = lf.GetLocalTime();
            function onClose(res) {
                bannerAd["offClose"](onClose);
                console.log("banner close");
                lf.bannerCloseTime = lf.GetLocalTime();
            }
            bannerAd["onClose"] && bannerAd["onClose"](onClose);
            Laya.timer.clear(lf, lf.CheckeUpdateTime);
            Laya.timer.loop(1000, lf, lf.CheckeUpdateTime);
        }
        GetLocalTime() {
            return new Date().getTime();
        }
        CheckeUpdateTime() {
            let isUpdateBanner = false;
            let localTime = this.GetLocalTime();
            if (!isNaN(this.bannerCloseTime) && localTime - this.bannerCloseTime > this.reshowBannerTime * 1000
                || !isNaN(this.bannerCreateTime) && localTime - this.bannerCreateTime > this.updateBannerTime * 1000) {
                isUpdateBanner = true;
            }
            if (isUpdateBanner) {
                this.hideAd(AdType.ad_1, true);
                GameDispatcher.Inst.event(UIGameEvent.SHOW_BANNER);
            }
        }
        closeBanner() {
            this.closePlatformBanner();
            UISystem.Inst.RemoveWindowClass(UIHWNativeBannerView);
            Laya.timer.clear(this, this.showBanner);
            delete this.sendTimeMap[this.bannerId];
        }
        closePlatformBanner() {
            if (this.bannerAd != null) {
                this.bannerAd.close && this.bannerAd.close();
                this.bannerAd.hide && this.bannerAd.hide();
                this.bannerAd.destroy && this.bannerAd.destroy();
            }
            this.bannerAd = null;
        }
        showInsert(adRes, handler) {
            if (this.isReview) {
                return;
            }
            if (MathUtil.random(this.nativeProbability)) {
                console.log("---->打开原生插屏广告");
                this.showNative(adRes, handler);
            }
        }
        hideInsert() {
            if (this.insertAd != null) {
                this.insertAd.destroy();
            }
            this.insertAd = null;
            this.isShowInsert = false;
            UISystem.Inst.RemoveWindowClass(UIHWNativeInsertView);
            delete this.sendTimeMap[this.nativeInsertId];
        }
        canShowInsert() {
            return true;
        }
        showNative(adRes, handler) {
            if (this.isReview) {
                return;
            }
            let posId = "";
            if (adRes && (adRes.type == AdType.TYPE_Banner || adRes.type == AdType.TYPE_NativeBaner)) {
                posId = this.nativeBannerId;
            }
            else if (adRes && (adRes.type == AdType.TYPE_Insert || adRes.type == AdType.TYPE_NativeInsert)) {
                posId = this.nativeInsertId;
            }
            if (posId) {
                this.createNative(adRes, posId, handler, false);
            }
        }
        createNative(adRes, posId, handler, isNext = false) {
            if (this.isReview) {
                return;
            }
            let nativeRes = AdModel.Inst.getNativeAd(posId);
            if (nativeRes) {
                GameDispatcher.Inst.event(UIGameEvent.SHOW_NATIVE, [adRes, nativeRes.adData, nativeRes.nativeAd]);
            }
            else {
                Logger.debug(this, "=====>加载原生广告 posId: " + posId);
                LoadQueue.Inst.loadNativeAd(posId, Laya.Handler.create(this, (adRes, nativeRes) => {
                    nativeRes = nativeRes || AdModel.Inst.getNativeAd(posId);
                    if (nativeRes) {
                        GameDispatcher.Inst.event(UIGameEvent.SHOW_NATIVE, [adRes, nativeRes.adData, nativeRes.nativeAd]);
                    }
                }, [adRes]));
            }
        }
        reportNativeShow(nativeData, nativeAd = null) {
            if (nativeData && nativeAd) {
                if (!AdModel.Inst.hasReportShow(nativeAd)) {
                    Logger.debug(this, "=====>reportNativeShow nativeData.adId: " + nativeData.adId);
                    nativeAd.reportAdShow({ "adId": nativeData.adId });
                    AdModel.Inst.reportAdShow(nativeAd);
                }
            }
        }
        reportNativeClick(nativeData, nativeAd = null) {
            if (nativeData && nativeAd) {
                Logger.debug(this, "=====>reportNativeClick nativeData.adId: " + nativeData.adId);
                nativeAd.reportAdClick({ "adId": nativeData.adId });
                AdCtrl.Inst.reportNativeClick(nativeAd);
            }
        }
        hideAd(adId, isDestroy = true) {
            let adRes = AdResource.getItemBy2Keys(this.GetPlatformName(), adId);
            if (adRes == null)
                return;
            if (adRes.type == AdType.TYPE_Banner) {
                let ad = this.adDic[adRes.type];
                if (ad) {
                    if (isDestroy) {
                        ad["destroy"] && ad["destroy"]();
                        this.adDic[adRes.type] = null;
                        delete this.adDic[adRes.type];
                    }
                    else {
                        ad["hide"] && ad["hide"]();
                    }
                }
            }
            else {
                if (adRes.type == AdType.TYPE_RewardedVideo) {
                    isDestroy = true;
                }
                if (adRes.type == AdType.TYPE_NativeIcon) {
                    UISystem.Inst.RemoveWindowClass(UIAdNativeIconView);
                }
                else if (adRes.type == AdType.TYPE_NativeBaner) {
                    this.closeBanner();
                    AdCtrl.Inst.destroyNative(this.nativeBannerId);
                }
                else if (adRes.type == AdType.TYPE_NativeInsert || adRes.type == AdType.TYPE_Insert) {
                    this.hideInsert();
                    AdCtrl.Inst.destroyNative(this.nativeInsertId);
                }
                super.hideAd(adId, isDestroy);
            }
        }
        exitGame() {
            qg.exitApplication && qg.exitApplication({
                success: function () {
                    console.log("exitApplication success");
                },
                fail: function () {
                    console.log("exitApplication fail");
                },
                complete: function () {
                    console.log("exitApplication complete");
                }
            });
        }
        checkNetwork() {
            let obj = {
                success: function (data) {
                    Logger.log(this, `handling success: ${data.type}`);
                    if (data.type == "none") {
                    }
                }
            };
            qg.getNetworkType(obj);
        }
        vibrateShort() {
            let obj = {
                success: function () {
                    console.log("vibrateShort success");
                },
                fail: function () {
                    console.log("vibrateShort fail");
                },
                complete: function () {
                    console.log("vibrateShort complete");
                }
            };
            let vs = qg["vibrateShort"];
            vs(obj);
        }
        vibrateLong() {
            let obj = {
                success: function () {
                    console.log("vibrateLong success");
                },
                fail: function () {
                    console.log("vibrateLong fail");
                },
                complete: function () {
                    console.log("vibrateLong complete");
                }
            };
            qg.vibrateLong(obj);
        }
        setKeepScreenOn(flag = true) {
            let obj = {
                keepScreenOn: flag,
                success: function (result) {
                    console.info('设置屏幕常亮成功..');
                },
                fail() {
                    console.info('设置屏幕常亮失败..');
                }
            };
            qg.setKeepScreenOn(obj);
        }
        get brand() {
            return this.systemInfo ? this.systemInfo.brand : "";
        }
        get model() {
            return this.systemInfo ? this.systemInfo.model : "";
        }
        GetPlatformName() {
            return PlatformType.TYPE_HUAWEI;
        }
        get qg() {
            return window["qg"];
        }
        hasShortcutInstalled(handler) {
            let self = this;
            qg && qg["hasShortcutInstalled"] && qg["hasShortcutInstalled"]({
                success: function (res) {
                    console.log("hasInstalledsuccess： " + res);
                    if (handler != null) {
                        handler.runWith(res);
                    }
                },
                fail: function (err) {
                    console.log(JSON.stringify(err));
                },
                complete: function () { }
            });
        }
        installShortcut(handler) {
            let self = this;
            qg["installShortcut"] && qg["installShortcut"]({
                success: function (ret) {
                    if (handler != null)
                        handler.run();
                },
                fail: function (erromsg, errocode) {
                    console.log('handling install fail: ' + erromsg + ", errocode: " + errocode);
                },
                complete: function () { }
            });
        }
    }

    class MzPlatform extends BasePlatform {
        constructor() {
            super();
            this.intervalTime = 30000;
            this.closeTime = NaN;
            this.updateIntervalTime = 40000;
            this.showTime = 0;
            let lf = this;
            lf.result = 1;
            lf.openAd = 1;
            lf.checkTime = 0;
            lf.video = ["3U2Jbt7Z"];
            lf.insert = "U1m3lRxZ";
            lf.bannerId = "7iqUzkej";
            this.enterMain();
        }
        updateWindowSize() {
            PlatformModel.Inst.platfrom = this.GetPlatformName();
            if (this.systemInfo) {
                console.log("====>updateWindowSize obj: " + JSON.stringify(this.systemInfo));
                PlatformModel.Inst.setWindowSize(this.systemInfo.windowWidth, this.systemInfo.windowHeight);
            }
            if (PlatformModel.Inst.windowWidth > 720) {
                console.log("=====> 大屏右上角位置 370, 170, 146, 49");
                PlatformModel.Inst.setRect(370, 170, 146, 49);
            }
            else {
                console.log("=====> 小屏右上角 508, 170, 194, 65");
                PlatformModel.Inst.setRect(508, 170, 194, 65);
            }
        }
        enterMain() {
            let self = this;
            qg.onHide(function () {
                Logger.log(self, "失去焦点");
                GameDispatcher.Inst.event(UIGameEvent.ON_HIDE);
            });
            self.setKeepScreenOn(true);
            qg.onShow(function (res) {
                self.setKeepScreenOn(true);
                SoundMgr.Inst.Reset();
                GameDispatcher.Inst.event(UIGameEvent.ON_SHOW);
            });
            this.systemInfo = qg && qg.getSystemInfoSync();
            Laya.timer.once(2000, this, this.updateWindowSize);
            Laya.timer.loop(1000, self, self.CheckBannerReShow);
        }
        initAd(handler) {
            let self = this;
            if (self.isInit) {
                handler.run();
                return;
            }
            if (handler != null)
                self.adHandlers.push(handler);
            self.isInit = true;
            for (const iterator of self.adHandlers) {
                iterator.run();
            }
            self.adHandlers.length = 0;
        }
        showAd(adId, handler = null) {
            super.showAd(adId, handler);
        }
        showBanner(adRes, handler) {
            let self = this;
            if (this.bannerAd != null) {
                this.bannerAd.show();
                console.log("bannerShow");
                return;
            }
            let screenHeight = qg.getSystemInfoSync().screenHeight;
            let screenWidth = qg.getSystemInfoSync().screenWidth;
            let bannerAd = qg.createBannerAd({
                "adUnitId": self.bannerId,
                "style": {
                    left: 0,
                    top: screenHeight - screenWidth / 6.7,
                    width: 1440,
                    height: screenWidth / 6.7
                }
            });
            bannerAd.onLoad(function () {
                console.log("banner 广告加载成功");
                bannerAd.show();
            });
            self.closeTime = NaN;
            self.showTime = TimeModel.Inst.LocalTime;
            bannerAd.onClose(function () {
                bannerAd.offClose();
                self.closeTime = TimeModel.Inst.LocalTime;
                self.bannerAd = null;
                console.log("banner 广告关闭");
            });
            bannerAd.onError(function (err) {
                console.log(err);
            });
            bannerAd.onResize((res) => {
                console.log("Banner 尺寸改变 screenWidth: " + screenWidth + " screenHeight: " + screenHeight);
                bannerAd.style.top = screenHeight - res.height;
                bannerAd.style.width = 1440;
                bannerAd.style.height = res.height;
                bannerAd.style.left = (screenWidth - bannerAd.style.width) * 0.5;
            });
            this.bannerAd = bannerAd;
        }
        hideAd(adId, destroy = true) {
            let adRes = AdResource.getItemBy2Keys(this.GetPlatformName(), adId);
            if (adRes == null) {
                if (!Laya.Browser.onPC) {
                    console.error("[BasePlatform]=====>广告表没有配置该广告ID: " + adId);
                }
                return;
            }
            if (adRes.type == AdType.TYPE_Banner) {
                if (destroy) {
                    this.bannerAd && this.bannerAd.hide && this.bannerAd.hide();
                    this.bannerAd && this.bannerAd.destory && this.bannerAd.destory();
                    this.bannerAd = null;
                }
                else {
                    this.bannerAd && this.bannerAd.hide && this.bannerAd.hide();
                }
            }
            else {
                super.hideAd(adId, destroy);
            }
        }
        CheckBannerReShow() {
            if (!isNaN(this.closeTime) && TimeModel.Inst.LocalTime - this.closeTime > this.intervalTime) {
                let topView = UISystem.Inst.getTopWindow([UIGuideView, UITipView, UILoadingLeafView]);
                if (topView instanceof UIBaseWindow && topView.isShowBanner && GuideModel.Inst.IsCanShowBtns()) {
                    if (this.bannerAd == null) {
                        PlatformMgr.Inst.showBanner();
                        this.closeTime = NaN;
                        console.log("banner广告关闭后30s刷新");
                    }
                }
            }
            else {
                this.UpdateBannerShow40();
            }
        }
        UpdateBannerShow40() {
            if (TimeModel.Inst.LocalTime - this.showTime > this.updateIntervalTime) {
                let topView = UISystem.Inst.getTopWindow([UIGuideView, UITipView, UILoadingLeafView]);
                if (topView instanceof UIBaseWindow && topView.isShowBanner && GuideModel.Inst.IsCanShowBtns()) {
                    this.closeBanner();
                    this.showTime = TimeModel.Inst.LocalTime;
                    if (this.bannerAd == null) {
                        PlatformMgr.Inst.showBanner();
                        console.log("banner广告40s刷新");
                    }
                }
            }
        }
        closeBanner() {
            if (this.bannerAd != null) {
                this.bannerAd.hide && this.bannerAd.hide();
            }
            this.bannerAd = null;
        }
        showRewardedVideo(adRes, handler) {
            this.createRewardedVideo(adRes, handler, this.video.slice());
        }
        createRewardedVideo(adRes, handler, posIds) {
            if (posIds.length <= 0)
                return;
            let self = this;
            let videoAd = qg.createRewardedVideoAd({ "adUnitId": posIds.pop() });
            videoAd.load();
            videoAd.onLoad(function () {
                videoAd.show();
                console.log('激励视频 广告显示成功');
                self.isShowRewardAd = true;
                self.isLoadVideo = false;
                SoundMgr.Inst.stop();
                self.videoTime = Laya.timer.currTimer;
                self.setLookVideoTime(adRes.id);
                console.log('激励视频 广告显示成功 end');
            });
            videoAd.onClose((res) => {
                console.log("onclose====>" + res);
                GameDispatcher.Inst.event(UIGameEvent.SHOW_AD_COMPLETE, [adRes, true, self.GetPlatformName()]);
                GameDispatcher.Inst.event(UIGameEvent.SHOW_AD, [adRes, true]);
                if (handler != null) {
                    handler.runWith(true);
                }
                self.isShowRewardAd = false;
                self.isLoadVideo = false;
                SoundMgr.Inst.Reset();
            });
            videoAd.onError(function (err) {
                console.log("onError====>" + JSON.stringify(err));
                if (posIds.length > 0) {
                    self.createRewardedVideo(adRes, handler, posIds);
                }
                else {
                    if (handler != null)
                        handler.runWith(false);
                    self.isShowRewardAd = false;
                    self.isLoadVideo = false;
                    SoundMgr.Inst.Reset();
                    UISystem.Inst.CreateWindowView(UITipView, "暂无广告");
                }
            });
        }
        showInsert(adRes, handler) {
            let self = this;
            let insertAd = qg.createInsertAd({ "adUnitId": self.insert });
            insertAd.onLoad(function () {
                console.log("insert 广告加载成功");
                insertAd.show();
            });
            insertAd.onError(function (err) {
                console.log(err);
            });
            insertAd.onClose(function () {
                console.log("关闭插屏广告");
            });
        }
        vibrateLong() {
            window["mz"].vibrateLong({});
        }
        vibrateShort() {
            window["mz"].vibrateShort({});
        }
        hasShortcutInstalled(handler) {
            let self = this;
            qg["hasShortcutInstalled"]({
                success: function (res) {
                    console.log("[MzPlatform] success=====>res: " + JSON.stringify(res));
                },
                fail: function (err) {
                    console.log("[MzPlatform] fail--->" + JSON.stringify(err));
                },
                complete: function (res) {
                    console.log("[MzPlatform]-complete-=====>res: " + res);
                    if (handler != null)
                        handler.runWith(res);
                }
            });
        }
        installShortcut(handler) {
            let self = this;
            qg["installShortcut"]({
                success: function (res) {
                    if (handler != null)
                        handler.run();
                },
                fail: function (err) {
                    console.log("[MzPlatform]=====>" + JSON.stringify(err));
                },
                complete: function () { }
            });
        }
        setKeepScreenOn(flag = true) {
            this.qg && this.qg.setKeepScreenOn && this.qg.setKeepScreenOn({
                keepScreenOn: flag,
                success: function (result) {
                    console.info('设置屏幕常亮成功..');
                },
                fail() {
                    console.info('设置屏幕常亮失败..');
                }
            });
        }
        get qg() {
            return window["qg"];
        }
        onShareAppMessage(handler) {
        }
        isShowShare() {
            return false;
        }
        GetPlatformName() {
            return PlatformType.TYPE_MZ;
        }
    }

    class NormalPlatform extends BasePlatform {
        constructor() { super(); }
        GetPlatformName() {
            return PlatformType.TYPE_NORMAL;
        }
        showAd(adId, handler = null, data = null) {
            let adRes = AdResource.getItemBy2Keys("wx", adId);
            if (adRes && adRes.type == AdType.TYPE_RewardedVideo) {
                GameDispatcher.Inst.event(UIGameEvent.SHOW_AD_COMPLETE, [adRes, true, "wx"]);
            }
            if (handler != null) {
                handler.runWith(true);
            }
        }
        GetVersionName() {
            return "version.json";
        }
    }

    class OppoPlatform extends BasePlatform {
        constructor() {
            super();
            this.isFirstVideo = true;
            this.isShow = true;
            this.insertTime = 0;
            this.nativeTime = 0;
            this.bannerTime = 0;
            this.curBannerCount = 0;
            this.navigateToAdId = "359114";
            this.lastShowRewardTime = NaN;
            this.showNativeInterValTime = 1000;
            this.showTime = 0;
            let lf = this;
            lf.result = 1;
            lf.openAd = 1;
            lf.checkTime = 10000;
            lf.banner = "";
            lf.video = ["358812"];
            lf.insert = "";
            lf.native = ["358808", "358806"];
            lf.setKeepScreenOn(true);
            var showf = function () {
                if (lf.isShow)
                    return;
                lf.isShow = true;
                console.log("获得焦点");
                SoundMgr.Inst.Reset();
                GlobalConfig.isActive = true;
                Laya.timer.resume();
                lf.setKeepScreenOn(true);
                GameDispatcher.Inst.event(UIGameEvent.ON_ACTIVE, true);
            };
            qg.onShow(showf);
            var hidef = function () {
                if (!lf.isShow)
                    return;
                lf.isShow = false;
                console.log("失去焦点");
                SoundMgr.Inst.stop();
                GlobalConfig.isActive = false;
                Laya.timer.pause();
                GameDispatcher.Inst.event(UIGameEvent.ON_ACTIVE, false);
            };
            qg.onHide(hidef);
            qg.setEnableDebug({
                enableDebug: false,
                success: function () {
                },
                complete: function () {
                },
                fail: function () {
                }
            });
            qg.onError((res) => {
                if (res) {
                    console.log("res.location：" + res.location);
                    console.log("res.message: " + res.message);
                    console.error("res.stack: " + res.stack);
                }
            });
            Laya.timer.once(2000, this, this.updateWindowSize);
        }
        login(openId, channel) {
            HttpUrl.sendPost("https://ga.gametdd.com/adCtrl/agd/conf", { 'id': "61010d3d24a0004a17cabd3c" }, Laya.Handler.create(this, (res) => {
                console.log(this, "====>策略 " + JSON.stringify(res), true);
                let data = res["data"];
                AdModel.Inst.addPolicy(data, this.GetPlatformName());
            }), Laya.Handler.create(this, () => {
            }));
        }
        updateWindowSize() {
            PlatformModel.Inst.platfrom = this.GetPlatformName();
            let obj = qg.getSystemInfoSync();
            PlatformModel.Inst.setWindowSize(obj.windowWidth, obj.windowHeight);
        }
        setLoadingProgress(progress) {
            if (this.rate >= 100 || this.rate == progress)
                return;
            if (progress <= this.rate)
                return;
            console.log("oppo加载进度:" + progress);
            this.rate = progress;
            qg.setLoadingProgress({ progress: progress });
            if (this.rate >= 100) {
                qg.loadingComplete({
                    complete: function (res) { }
                });
            }
        }
        registeredSubNames() {
        }
        GetPlatformName() {
            return PlatformType.TYPE_OPPO;
        }
        hasShortcutInstalled(handler) {
            qg["hasShortcutInstalled"]({
                success: function (status) {
                    if (status) {
                        console.log('已创建');
                        handler && handler.runWith(true);
                    }
                    else {
                        console.log('未创建');
                        handler && handler.runWith(false);
                    }
                }
            });
        }
        installShortcut(handler) {
            qg["installShortcut"]({
                success: function () {
                    console.log('创建成功');
                    handler && handler.run();
                },
                fail: function () {
                    console.log('创建失败');
                }
            });
        }
        hideAd(adId, idDestroy = true) {
            let adRes = AdResource.getItemBy2Keys(this.GetPlatformName(), adId);
            if (adRes == null)
                return;
            if (adRes.type == AdType.TYPE_NativeIcon) {
                UISystem.Inst.RemoveWindowClass(UIAdNativeIconView);
            }
            else if (adRes.type == AdType.TYPE_NativeBaner) {
                UISystem.Inst.RemoveWindowClass(UIAdNativeBannerView);
            }
            else if (adRes.type == AdType.TYPE_NativeInsert) {
                UISystem.Inst.RemoveWindowClass(UINatiEmbedView);
            }
            super.hideAd(adId, idDestroy);
        }
        canLookReward() {
            if (!isNaN(this.lastShowRewardTime) && TimeModel.Inst.LocalTime - this.lastShowRewardTime < 2000) {
                return false;
            }
            this.lastShowRewardTime = TimeModel.Inst.LocalTime;
            return true;
        }
        showRewardedVideo(adRes, handler) {
            this.createRewardVideo(adRes, handler, this.video.slice());
        }
        createRewardVideo(adRes, handler, posIds) {
            let lf = this;
            if (posIds.length <= 0)
                return;
            if (!lf.canLookReward())
                return;
            let rewardedAd;
            rewardedAd = qg.createRewardedVideoAd({
                posId: posIds.pop(),
                style: {}
            });
            function onClose(res) {
                GameDispatcher.Inst.event(UIGameEvent.SHOW_AD_COMPLETE, [adRes, res.isEnded, lf.GetPlatformName()]);
                rewardedAd.offError(onError);
                rewardedAd.offClose(onClose);
                handler.runWith(res.isEnded);
                if (res && res.isEnded) {
                    console.log("正常播放结束，可以下发游戏奖励");
                    if (handler != null)
                        handler.run();
                }
                else {
                    console.log("播放中途退出，不下发游戏奖励");
                    UISystem.Inst.CreateWindowView(UITipView, "视频没有播完，无法获得奖励");
                }
                if (!StorageUtils.Inst.IsCloseMusic())
                    SoundMgr.Inst.Reset();
            }
            rewardedAd.onClose(onClose);
            function onError(err) {
                rewardedAd.offError(onError);
                switch (err.errCode) {
                    case -3:
                        console.log("激励广告加载失败---调用太频繁", JSON.stringify(err));
                        UISystem.Inst.CreateWindowView(UITipView, "广告加载失败，请稍后再试");
                        break;
                    case -4:
                        console.log("激励广告加载失败--- 一分钟内不能重复加载", JSON.stringify(err));
                        if (posIds.length == 0) {
                            UISystem.Inst.CreateWindowView(UITipView, "您点的太快了，稍后再来哦");
                            if (!StorageUtils.Inst.IsCloseMusic())
                                SoundMgr.Inst.Reset();
                        }
                        else
                            lf.createRewardVideo(adRes, handler, posIds);
                        break;
                    case 30008:
                        break;
                    default:
                        console.log("激励广告展示失败code:    " + err.errCode);
                        UISystem.Inst.CreateWindowView(UITipView, "广告加载失败，请稍后再试");
                        break;
                }
            }
            rewardedAd.onError(onError);
            function onLoad() {
                rewardedAd.offLoad(onLoad);
                if (!StorageUtils.Inst.IsCloseMusic())
                    SoundMgr.Inst.stop();
                console.log("激励视频广告加载成功");
                let adshow = rewardedAd.show();
                adshow && adshow.catch((err) => {
                    console.log("激励广告展示失败" + JSON.stringify(err));
                    lf.createRewardVideo(adRes, handler, posIds);
                });
            }
            rewardedAd.onLoad(onLoad);
            rewardedAd.load();
        }
        showBanner(adRes, handler) {
            if (Laya.timer.currTimer <= (this.bannerTime + 30000)) {
                return;
            }
            this.bannerTime = Laya.timer.currTimer;
            let ad = this.adDic[adRes.type];
            if (ad != null)
                return;
            let nativeAd = this.adDic[AdType.TYPE_NativeBaner];
            if (nativeAd) {
                UISystem.Inst.RemoveWindowClass(UIAdNativeBannerView);
                nativeAd.destroy();
                delete this.adDic[AdType.TYPE_NativeBaner];
            }
            let self = this;
            let adId = adRes.adIds ? adRes.adIds[0] : self.banner;
            let bannerAd = qg.createBannerAd({ posId: adId, style: {} });
            self.adDic[adRes.type] = bannerAd;
            let adshow = bannerAd.show();
            adshow && adshow.then(() => {
                console.log("banner广告展示成功");
                if (handler != null) {
                    handler.run();
                }
            }).catch((err) => {
                switch (err.code) {
                    case 30003:
                        console.log("新用户7天内不能曝光Banner，请将手机时间调整为7天后，退出游戏重新进入");
                        break;
                    case 30009:
                        console.log("10秒内调用广告次数超过1次，10秒后再调用");
                        break;
                    case 30002:
                        console.log("加载广告失败，重新加载广告");
                        break;
                    case 30007:
                        console.log("Banner广告每日播放次数上限20次");
                        break;
                    default:
                        console.log("banner广告展示失败code:  " + err.code);
                        break;
                }
                self.showAd(adRes.id);
            });
        }
        showInsert(adRes, handler) {
            if (Laya.timer.currTimer <= (this.insertTime + 16000)) {
                return;
            }
            this.insertTime = Laya.timer.currTimer;
            let lf = this;
            let adId = adRes.adIds ? adRes.adIds[0] : lf.insert;
            var interstitialAd = qg["createInterstitialAd"]({
                posId: adId
            });
            var onError = function (err) {
                interstitialAd.offError(onError);
                switch (err.code) {
                    case 30003:
                        console.log("新用户7天内不能曝光插屏，请将手机时间调整为7天后，退出游戏重新进入");
                        break;
                    case 30009:
                        console.log("10秒内调用广告次数超过1次，10秒后再调用");
                        break;
                    case 30002:
                        console.log("load广告失败，重新加载广告");
                        break;
                    default:
                        console.log("插屏广告报错： " + err.code);
                        break;
                }
            };
            interstitialAd.onError(onError);
            var onLoad = function () {
                console.log("intersti广告开始展示");
                interstitialAd["offLoad"](onLoad);
                let adShow = interstitialAd.show();
                adShow && adShow.then(() => {
                    console.log("插屏广告展示成功");
                    if (handler != null)
                        handler.run();
                }).catch((err) => {
                    console.log("插屏广告展示失败");
                });
            };
            interstitialAd["onLoad"](onLoad);
        }
        CanShowNative() {
            if (this.showTime != 0 && new Date().getTime() - this.showTime < this.showNativeInterValTime)
                return false;
            this.showTime = new Date().getTime();
            return true;
        }
        showNative(adRes, handler) {
            let adIds = adRes.adIds ? adRes.adIds.slice() : this.native.slice();
            this.createNative(adRes, adIds, handler);
        }
        createNative(adRes, posIds, handler) {
            if (posIds.length <= 0)
                return;
            let ad = this.adDic[adRes.type];
            if (ad != null)
                return;
            if (!this.CanShowNative())
                return;
            let lf = this;
            var nativeAd = qg.createNativeAd({
                posId: posIds.pop()
            });
            lf.adDic[adRes.type] = nativeAd;
            nativeAd.onLoad(function (res) {
                let adList = res.adList;
                for (const iterator of adList) {
                    if ((iterator.logoUrl != null && iterator.logoUrl.length >= 1) ||
                        (iterator.imgUrlList != null && iterator.imgUrlList.length >= 1)) {
                        if (adRes.type == AdType.TYPE_NativeIcon) {
                            iterator.adResId = adRes.id;
                            UISystem.Inst.RemoveWindowClass(UIAdNativeIconView);
                            UISystem.Inst.CreateWindowView(UIAdNativeIconView, [iterator, Laya.Handler.create(null, showHandler, [iterator]),
                                Laya.Handler.create(null, clickHandler, [iterator], false)]);
                        }
                        else if (adRes.type == AdType.TYPE_NativeBaner) {
                            UISystem.Inst.RemoveWindowClass(UIAdNativeBannerView);
                            UISystem.Inst.CreateWindowView(UIAdNativeBannerView, [iterator, Laya.Handler.create(null, showHandler, [iterator]),
                                Laya.Handler.create(null, clickHandler, [iterator], false), adRes]);
                        }
                        else if (adRes.type == AdType.TYPE_NativeInsert) {
                            UISystem.Inst.RemoveWindowClass(UINatiEmbedView);
                            UISystem.Inst.CreateWindowView(UINatiEmbedView, [iterator, Laya.Handler.create(null, showHandler, [iterator]),
                                Laya.Handler.create(null, clickHandler, [iterator], false), adRes]);
                        }
                        break;
                    }
                }
            });
            nativeAd.onError(function (err) {
                lf.hideAd(adRes.id);
                console.log(err);
                if (posIds.length > 0) {
                    lf.createNative(adRes, posIds, handler);
                }
            });
            nativeAd.load();
            function showHandler(adObj) {
                nativeAd.reportAdShow({
                    adId: adObj["adId"]
                });
            }
            function clickHandler(adObj) {
                nativeAd.reportAdClick({
                    adId: adObj["adId"]
                });
            }
        }
        navigateTo(value) {
            this.CreateGamePortalAd(value);
        }
        CreateGamePortalAd(value) {
            let adUnitId = value && value["adUnitId"] || this.navigateToAdId;
            if (qg.getSystemInfoSync()["platformVersionCode"] >= 1076) {
                var gamePortalAd = qg["createGamePortalAd"]({
                    adUnitId: adUnitId
                });
                gamePortalAd.load().then(function () {
                    console.log('互推盒子九宫格广告加载成功');
                    gamePortalAd.show().then(function () {
                        console.log('show success');
                    }).catch(function (error) {
                        console.log('show fail with:' + error.errCode + ',' + error.errMsg);
                    });
                }).catch(function (error) {
                    console.log('load fail with:' + error.errCode + ',' + error.errMsg);
                });
            }
            else {
                console.log('快应用平台版本号低于1076，暂不支持互推盒子相关 API');
            }
        }
        checkNetwork() {
            let info = window.qg.getSystemInfoSync();
            if (info.platformVersionCode >= 1060) {
                window.qg.reportMonitor('game_scene', 0);
            }
            qg.getNetworkType({
                success: function (data) {
                    if (data.type == "none") {
                        UISystem.Inst.CreateWindowView(UITipView, "网络错误，请重新加载");
                    }
                }
            });
        }
        vibrateShort() {
            let obj = {};
            qg.vibrateShort();
        }
        vibrateLong() {
            let obj = {};
            qg.vibrateLong(obj);
        }
        setKeepScreenOn(flag = true) {
            let data = {
                keepScreenOn: flag,
                success: function (result) {
                    console.info('设置屏幕常亮成功..');
                },
                fail() {
                    console.info('设置屏幕常亮失败..');
                }
            };
            qg.setKeepScreenOn && qg.setKeepScreenOn(data);
        }
    }

    class QQPlatform extends BasePlatform {
        constructor() {
            super();
            this.endTime = 0;
            this.shareHandler = null;
            this.platform = "qq";
            this.isGroupManager = false;
            this.entryDataHash = "";
            this.hasSubscribeApp = false;
            this.insertStartIntervalTime = 30000;
            this.insertIntervalTime = 60000;
            this.insertAdShowTime = 0;
            let lf = this;
            this.qq.getSetting && this.qq.getSetting({
                withSubscriptions: true,
                success(res) {
                    console.log("--------------------getSetting----------------------");
                    if (res.authSetting) {
                        lf.hasSubscribeApp = res.authSetting["setting.appMsgSubscribed"];
                        console.log("res.authSetting: " + JSON.stringify(res.authSetting));
                        GameDispatcher.Inst.event(UIGameEvent.GET_Subscribe_INFO, lf.hasSubscribeApp);
                    }
                    if (res.subscriptionsSetting) {
                        console.log("res.subscriptionsSetting: " + JSON.stringify(res.subscriptionsSetting));
                    }
                }
            });
            window["qq"]["onHide"](function () {
                GlobalConfig.isActive = false;
                SoundMgr.Inst.stop();
                console.log("失去焦点");
                GameDispatcher.Inst.event(UIGameEvent.ON_ACTIVE, false);
            });
            lf.setKeepScreenOn(true);
            window["qq"]["onShow"](function () {
                lf.setKeepScreenOn(true);
                GlobalConfig.isActive = true;
                let localTime = TimeModel.Inst.GetLocalTime();
                SoundMgr.Inst.Reset();
                console.log("[QQPlatform]=====>获得焦点");
                if (lf.qq.getGroupInfo != null) {
                    lf.qq.getGroupInfo({
                        "entryDataHash": lf.entryDataHash,
                        "success": (res) => {
                            lf.isGroupManager = res.isGroupManager;
                            GameDispatcher.Inst.event(UIGameEvent.CHANGE_GROUP_INFO, lf.isGroupManager);
                        }
                    });
                }
                GameDispatcher.Inst.event(UIGameEvent.ON_ACTIVE, true);
                if (lf.endTime == 0 || lf.shareHandler == null) {
                    return;
                }
                else {
                    if (localTime >= lf.endTime) {
                        PlatformMgr.Inst.aldSendEvent(EventType.fenxiang, "成功");
                        lf.shareHandler.runWith(true);
                        if (this.openAd == 0) {
                            let res = AdResource.getItemBy2Keys(lf.GetPlatformName(), AdType.ad_2);
                            GameDispatcher.Inst.event(UIGameEvent.SHOW_AD_COMPLETE, [res, true]);
                        }
                    }
                    else {
                        UISystem.Inst.CreateWindowView(UITipView, "获取失败，请重新试试");
                        PlatformMgr.Inst.aldSendEvent(EventType.fenxiang, "失败");
                    }
                    lf.endTime = 0;
                    lf.shareHandler = null;
                }
            });
            lf.result = 1;
            lf.openAd = 1;
            lf.checkTime = 10000;
            lf.banner = "b64d727073d07bfdc513de4af31a7703";
            lf.video = ["967daab36192f3d06111a0e71651cf95"];
            lf.box = "50ff13a9fb47e94a0f594883fa6665e3";
            lf.insert = "4c593da8b4850fd36c28aeca98f5c284";
            if (this.qq.showShareMenu) {
                this.qq.showShareMenu({
                    withShareTicket: true, menus: ['qq', 'qzone', 'wechatFriends', 'wechatMoment'],
                    success(res) {
                        console.log('showShareMenu', res);
                    }
                });
            }
            let index = 0;
            let title = QQPlatform.titles[index] || QQPlatform.titles[0];
            if (this.qq["onShareAppMessage"]) {
                this.qq["onShareAppMessage"](function () {
                    return {
                        title: title,
                        imageUrl: QQPlatform.imageUrl,
                        query: ''
                    };
                });
            }
            let launchInfo = this.qq.getLaunchOptionsSync();
            this.entryDataHash = launchInfo.entryDataHash;
            this.systemInfo = this.qq.getSystemInfoSync();
            console.log(this.brand + " - " + this.model);
            this.qq.getSystemInfo && this.qq.getSystemInfo({
                "success": (res) => {
                    lf.platform = res.AppPlatform;
                    lf.SDKVersion = res.SDKVersion;
                    console.log("--->getSystemInfo res: " + JSON.stringify(res));
                    lf.qq.getGroupInfo && lf.qq.getGroupInfo({
                        "entryDataHash": lf.entryDataHash,
                        "success": (res) => {
                            lf.isGroupManager = res.isGroupManager;
                        }
                    });
                }, "fail": (res) => {
                }, "complete": (res) => {
                }
            });
            Laya.timer.once(2000, this, this.updateWindowSize);
        }
        registeredSubNames() {
            this.AddSubName("Scene", "Scene");
            this.AddSubName("music", "music");
        }
        addToGroup() {
            this.addGroup(this.entryDataHash);
        }
        addGroup(entryDataHash) {
            let self = this;
            if (this.platform != "qq" || this.qq.getGroupInfo == null) {
                return;
            }
            this.qq.getGroupInfo({
                "entryDataHash": entryDataHash,
                "success": (res) => {
                    self.isGroupManager = res.isGroupManager;
                    GameDispatcher.Inst.event(UIGameEvent.CHANGE_GROUP_INFO, res.isGroupManager);
                    if (self.isGroupManager && self.platform == "qq" && self.qq.addGroupApp != null) {
                        self.qq.addGroupApp({
                            "entryDataHash": entryDataHash,
                            "success": (res) => {
                                UISystem.Inst.CreateWindowView(UITipView, "添加群应用成功");
                                GameDispatcher.Inst.event(UIGameEvent.ADD_GROUP_SUCCESS);
                            },
                            "fail": (res) => {
                                switch (res.errCode) {
                                    case 41004:
                                        console.log('不是群主或管理员，注意：41004错误码安卓848版本开始支持');
                                        break;
                                    case 41004:
                                        console.log('超过可设置数量');
                                        break;
                                }
                            },
                            "complete": (res) => {
                                console.log('addGroupApp complete: ' + JSON.stringify(res));
                            }
                        });
                    }
                }, "fail": (res) => {
                }, "complete": (res) => {
                }
            });
        }
        login(openId, channel) {
            let self = this;
        }
        updateWindowSize() {
            PlatformModel.Inst.platfrom = this.GetPlatformName();
            try {
                const res = window["qq"].getSystemInfoSync();
                PlatformModel.Inst.setWindowSize(res.screenWidth, res.screenHeight);
                let pos = res.navbarPosition;
                pos && PlatformModel.Inst.setRect(pos.marginRight, pos.marginTop, pos.navbarWidth, pos.navbarHeight);
                console.log("右上角", JSON.stringify(pos));
            }
            catch (e) {
            }
        }
        hideAd(adId) {
            let adRes = AdResource.getItemBy2Keys(this.GetPlatformName(), adId);
            if (adRes == null)
                return;
            if (adRes.type == AdType.TYPE_Block) {
                let ad = this.adDic[adRes.id];
                ad && ad.destroy && ad.destroy();
                delete this.adDic[adRes.id];
            }
            else {
                super.hideAd(adId);
            }
        }
        showBanner(adRes, handler, data = null) {
            if (this.openAd == 0)
                return;
            let ad = this.adDic[adRes.type];
            if (ad != null)
                return;
            let lf = this;
            let info = window["qq"]["getSystemInfoSync"]();
            let adId = adRes.adIds ? adRes.adIds[0] : lf.banner;
            let bannerAd = window["qq"]["createBannerAd"]({
                adUnitId: adId,
                adIntervals: 40,
                style: {
                    top: info.screenHeight - info.screenWidth * 0.25,
                    left: 0,
                    width: info.screenWidth
                }
            });
            this.bannerAd = bannerAd;
            this.adDic[adRes.type] = bannerAd;
            this.adDic[adRes.type + "_time"] = TimeModel.Inst.LocalTime;
            function onResize(size) {
                bannerAd.offResize && bannerAd.offResize(onResize);
                lf.resizeBanner(bannerAd, data);
            }
            bannerAd.onResize(onResize);
            function onError(res) {
                if (res) {
                    console.log('bannerAd onError', JSON.stringify(res));
                }
            }
            bannerAd.onError(onError);
            function onLoad(res) {
                bannerAd.offLoad(onLoad);
                if (res) {
                    console.log(this, 'qq bannerAd onLoad: ' + JSON.stringify(res), true);
                }
                bannerAd.show().then(() => {
                    console.log('--------->bannerAd show ok.');
                    lf.setLookVideoTime(adRes.id);
                    lf.resizeBanner(bannerAd, data);
                }).catch((res) => {
                    console.log('bannerAd show error: ' + (res ? JSON.stringify(res) : ""));
                });
            }
            bannerAd.onLoad(onLoad);
        }
        showRewardedVideo(adRes, handler) {
            if (this.openAd == 0) {
                this.onShareAppMessage(handler);
                return;
            }
            let ad = this.adDic[adRes.type];
            if (ad != null)
                return;
            this.adDic[adRes.type] = true;
            this.CheckRewardVideo(Laya.Handler.create(this, this.createRewardedVideo, [adRes, handler]));
        }
        LoadRewardVideo(handler, posIds) {
            if (posIds.length <= 0)
                return;
            PlatformMgr.Inst.aldSendEvent(EventType.shipinguanggao, "开始加载");
            let lf = this;
            let videoAd = window["qq"]["createRewardedVideoAd"]({
                adUnitId: posIds.pop()
            });
            lf.rewardVideo = videoAd;
            function onError(res) {
                videoAd.offError(onError);
                delete lf.adDic[AdType.TYPE_RewardedVideo];
                lf.Clear();
                PlatformMgr.Inst.aldSendEvent(EventType.shipinguanggao, "报错" + (res["errCode"] ? res["errCode"] : ""));
                if (res)
                    console.log('videoAd onError', JSON.stringify(res));
            }
            videoAd.onError(onError);
            videoAd.load().then(() => {
                videoAd.offError(onError);
                console.log('激励视频加载成功');
                lf.rewardVideoIsLoad = false;
                lf.handlerRuns();
            }).catch((err) => {
                console.log('激励视频加载失败');
                if (posIds.length > 0) {
                    lf.LoadRewardVideo(handler, posIds);
                }
                else {
                    PlatformMgr.Inst.aldSendEvent(EventType.shipinguanggao, "加载失败");
                    delete lf.adDic[AdType.TYPE_RewardedVideo];
                    if (handler != null)
                        lf.onShareAppMessage(handler);
                }
            });
        }
        createRewardedVideo(adRes, handler, posIds) {
            if (this.rewardVideo == null)
                return;
            PlatformMgr.Inst.aldSendEvent(EventType.shipinguanggao, "开始播放");
            let lf = this;
            let videoAd = this.rewardVideo;
            function onClose(res) {
                GameDispatcher.Inst.event(UIGameEvent.SHOW_AD_COMPLETE, [adRes, res.isEnded, lf.GetPlatformName()]);
                handler && handler.runWith(res.isEnded);
                if (res.isEnded) {
                    PlatformMgr.Inst.aldSendEvent(EventType.shipinguanggao, "完成播放");
                    console.log('激励视频广告完成，发放奖励');
                }
                else {
                    PlatformMgr.Inst.aldSendEvent(EventType.shipinguanggao, "取消播放");
                    console.log('激励视频广告取消关闭，不发放奖励');
                }
                videoAd.offClose(onClose);
                delete lf.adDic[AdType.TYPE_RewardedVideo];
                lf.rewardVideo = null;
            }
            videoAd.onClose(onClose);
            lf.ShowAdReward(videoAd, 3);
        }
        ShowAdReward(videoAd, index) {
            let lf = this;
            index--;
            if (index <= 0) {
                PlatformMgr.Inst.aldSendEvent(EventType.shipinguanggao, "显示失败");
                return;
            }
            videoAd.show().then(() => {
                console.log('激励视频 广告显示成功');
                PlatformMgr.Inst.aldSendEvent(EventType.shipinguanggao, "显示成功");
            })
                .catch(err => {
                console.log('激励视频 广告显示失败');
                lf.ShowAdReward(videoAd, index);
            });
        }
        CanShowInsertAd() {
            let retBol = false;
            if (this.insertAdShowTime == 0 || isNaN(this.insertAdShowTime)) {
                if (TimeModel.Inst.LocalTime - StorageUtils.Inst.loginTime2 > this.insertStartIntervalTime)
                    retBol = true;
            }
            else {
                if (TimeModel.Inst.LocalTime - this.insertAdShowTime >= this.insertIntervalTime)
                    retBol = true;
            }
            if (retBol) {
                this.insertAdShowTime = TimeModel.Inst.LocalTime;
            }
            return retBol;
        }
        showInsert(adRes, handler) {
            if (this.openAd == 0 || !this.insert)
                return;
            let lf = this;
            if (!this.CanShowInsertAd())
                return;
            console.log("显示插屏广告");
            let adId = adRes.adIds ? adRes.adIds[0] : lf.insert;
            let interstitialAd = window["qq"]["createInterstitialAd"]({ adUnitId: adId });
            function onClose(res) {
                interstitialAd.offLoad(onLoad);
                interstitialAd.offError(onError);
                interstitialAd.offClose(onClose);
                interstitialAd.destroy && interstitialAd.destroy();
                console.log('interstitial onClose', JSON.stringify(res));
            }
            interstitialAd.onClose(onClose);
            function onError(res) {
                interstitialAd.offLoad(onLoad);
                interstitialAd.offError(onError);
                interstitialAd.offClose(onClose);
                console.log('interstitial onError', JSON.stringify(res));
            }
            interstitialAd.onError(onError);
            function onLoad(res) {
                interstitialAd.offLoad(onLoad);
                console.log('interstitial onLoad', JSON.stringify(res));
                interstitialAd.show();
            }
            interstitialAd.onLoad(onLoad);
            interstitialAd.load();
        }
        showBox(adRes, handler) {
            if (this.openAd == 0)
                return;
            let lf = this;
            let adId = adRes.adIds ? adRes.adIds[0] : lf.box;
            let appbox = window["qq"]["createAppBox"]({
                adUnitId: adId
            });
            appbox.load().then(() => {
                appbox.show();
            });
            this.adDic[adRes.type] = appbox;
        }
        showBlock(adRes = null, handler = null, data = null) {
            let self = this;
            this.hideBlock();
            let windowWidth = this.systemInfo.windowWidth;
            let windowHeight = this.systemInfo.windowHeight;
            if (this.qq.createBlockAd) {
                this.blockAd = this.qq.createBlockAd({ "adUnitId": this.blockId, "style": { "left": 100, "top": 100 }, "orientation": "vertical", "size": 2 });
                console.log("显示积木广告");
                this.blockAd.onLoad(() => {
                    console.log("显示积木广告 onLoad");
                    self.blockAd.show();
                });
                this.blockAd.onError((res) => {
                    console.log("显示积木广告 error: " + JSON.stringify(res));
                });
                this.blockAd.onResize((size) => {
                    console.log("显示积木广告 onResize");
                    self.blockAd.style.left = 0;
                    self.blockAd.style.top = (windowHeight - size.height) * 0.5;
                });
                this.adDic[adRes.id] = this.bannerAd;
            }
        }
        hideBlock() {
            if (this.blockAd) {
                this.blockAd.destroy && this.blockAd.destroy();
                this.blockAd = null;
            }
        }
        onShareAppMessage(handler) {
            this.shareHandler = handler;
            this.endTime = (new Date()).getTime() + 2000;
            window["qq"]["shareAppMessage"]({
                title: QQPlatform.titles[0],
                imageUrl: QQPlatform.imageUrl,
                success: function (res) {
                    console.log("分享成功=====", res);
                },
                fail: function (res) {
                    console.log("分享失败=====", res);
                }
            });
        }
        isShowShare() {
            return true;
        }
        hasShortcutInstalled(handler) {
            let self = this;
            if (this.qq["isMicroAppInstalled"]) {
                this.qq["isMicroAppInstalled"]({
                    success: function (res) {
                        console.log("----------isMicroAppInstalled success: " + JSON.stringify(res));
                        if (handler != null)
                            handler.runWith(res && res.installed);
                    },
                    fail: function (err) {
                        console.log("----------isMicroAppInstalled fail: " + JSON.stringify(err));
                        if (handler != null)
                            handler.runWith(false);
                    },
                    complete: function (res) {
                        console.log("----------isMicroAppInstalled complete: " + JSON.stringify(res));
                    }
                });
            }
        }
        installShortcut(handler) {
            let self = this;
            if (this.qq["saveAppToDesktop"]) {
                this.qq["saveAppToDesktop"]({
                    success: function (res) {
                        console.log(self, "创建桌面图标： " + JSON.stringify(res));
                        StorageUtils.Inst.setForeverRecord(1, StorageUtils.add_desktop, true);
                        if (handler != null)
                            handler.runWith(true);
                    },
                    fail: function (err) {
                        console.log("[QQPlatform]=====>installShortcut error: " + JSON.stringify(err));
                        if (handler != null)
                            handler.runWith(false);
                    },
                    complete: function () { }
                });
            }
        }
        subscribeAppMsg(handler = null) {
            let self = this;
            if (this.qq.subscribeAppMsg) {
                this.qq.subscribeAppMsg({
                    subscribe: true,
                    success(res) {
                        console.log("----subscribeAppMsg----success", res);
                        if (handler)
                            handler.runWith(true);
                    },
                    fail(res) {
                        console.log("----subscribeAppMsg----fail", res);
                        if (handler)
                            handler.runWith(false);
                        self.qq.openSetting({ "scope": "setting.appMsgSubscribed" });
                    }
                });
            }
        }
        setKeepScreenOn(flag = true) {
            this.qq.setKeepScreenOn && this.qq.setKeepScreenOn({
                keepScreenOn: flag,
                success: function (result) {
                    console.info('设置屏幕常亮成功..');
                },
                fail() {
                    console.info('设置屏幕常亮失败..');
                }
            });
        }
        addColorSign() {
            console.log("调用添加彩签方法");
            this.qq["addColorSign"]({
                success: function () {
                    console.log("添加彩签成功");
                },
                fail: function () {
                    console.log("添加彩签失败");
                },
                complete: function () {
                    console.log("添加彩签 complete ");
                }
            });
        }
        GetPlatformName() {
            return PlatformType.TYPE_QQ;
        }
        get brand() {
            return this.systemInfo && this.systemInfo.brand;
        }
        get model() {
            return this.systemInfo && this.systemInfo.model;
        }
        get qq() {
            return window["qq"];
        }
        vibrateShort() {
            window["qq"]["vibrateShort"]({});
        }
        report(params, type, complete = null, url = "https://report.173kw.com/") {
            console.log("===>report type: " + type + " params: " + (params ? JSON.stringify(params) : null));
            switch (type) {
                case "register":
                    HttpUrl.sendPost(url + "rpt/regaccount", params, complete);
                    break;
                case "create":
                    HttpUrl.sendPost(url + "rpt/regchar", params, complete);
                    break;
                case "login":
                    HttpUrl.sendPost(url + "rpt/login", params, complete);
                    break;
                case "ad":
                    HttpUrl.sendPost(url + "rpt/adpoint", params, complete);
                    break;
            }
        }
    }
    QQPlatform.imageUrlId = "d76mIFIXQpKDPEp1SjK5Ug==";
    QQPlatform.titles = ["快来体验在线撸猫的乐趣"];
    QQPlatform.imageUrl = "https://mmocgame.qpic.cn/wechatgame/LauSTzq43wDb4VfInL75BicPBoYPrPMtsYhoicGvkjicmpvEkVECCVxbicQuhyImg3mt/0";

    class TTPlatform extends BasePlatform {
        constructor() {
            super();
            this.endTime = 0;
            this.shareHandler = null;
            this.addDesc = "#喵咪度假村#";
            this.startTime = 0;
            this.isStopLuzhi = false;
            this.lztime = 0;
            this.isLuzhing = false;
            let lf = this;
            lf.result = 1;
            lf.openAd = 1;
            lf.checkTime = 0;
            lf.appId = "tt75b90304ea816af002";
            lf.video = ["p74kmq76defb0bc4c9"];
            lf.insert = "c7884fd4888f14o3bj";
            lf.bannerId = "1tvp01c7vuverj2he8";
            this.enterMain();
        }
        enterMain() {
            let self = this;
            this.systemInfo = this.tt.getSystemInfoSync();
            let shareObj = { withShareTicket: false };
            wx.showShareMenu(shareObj);
            wx["onHide"](function () {
                Logger.log(self, "失去焦点");
                SoundMgr.Inst.stop();
                GameDispatcher.Inst.event(UIGameEvent.ON_HIDE);
            });
            self.setKeepScreenOn(true);
            wx["onShow"](function () {
                self.setKeepScreenOn(true);
                SoundMgr.Inst.Reset();
                if (self.endTime == 0 || self.shareHandler == null) {
                }
                else {
                    if ((new Date()).getTime() >= self.endTime && MathUtil.random(9000, 10000)) {
                        self.shareHandler.runWith(true);
                    }
                    else {
                        self.shareHandler.runWith(false);
                        UISystem.Inst.CreateWindowView(UITipView, "分享失败！");
                    }
                    self.endTime = 0;
                    self.shareHandler = null;
                }
                GameDispatcher.Inst.event(UIGameEvent.ON_SHOW);
            });
        }
        showAd(adId, handler = null, data = null) {
            super.showAd(adId, handler, data);
        }
        hideAd(adId, destroy = true) {
            super.hideAd(adId, destroy);
        }
        showBanner(adRes, handler, data = null) {
            let self = this;
            if (this.isDouyin || this.isXigua || this.bannerId == "") {
                Logger.log(self, "抖音(西瓜)不支持banner广告!");
                return;
            }
            let offsetX = data && data.hasOwnProperty("offsetX") ? data.offsetX : 0;
            let offsetY = data && data.hasOwnProperty("offsetY") ? data.offsetY : 0;
            this.closeBanner();
            let targetBannerAdWidth = 180;
            let obj = this.tt.getSystemInfoSync();
            let windowWidth = obj.windowWidth;
            let windowHeight = obj.windowHeight;
            let bannerAd = this.tt.createBannerAd({
                adUnitId: self.bannerId,
                adIntervals: 30,
                style: {
                    width: targetBannerAdWidth
                }
            });
            function onResize(size) {
                bannerAd.offResize(onResize);
                Logger.log(self, 'Resize后正式宽高:' + size.width + "-" + size.height);
                bannerAd.style.top = windowHeight - size.height + offsetY;
                bannerAd.style.left = (windowWidth - size.width) / 2 + offsetX;
            }
            bannerAd.onResize(onResize);
            function onError(res) {
                bannerAd.offError(onError);
                Logger.log(self, 'bannerAd onError2: ' + JSON.stringify(res));
                if (handler != null) {
                    handler.runWith(false);
                }
                GameHtMgr.Inst.showAppBox();
            }
            bannerAd.onError(onError);
            function onLoad(res) {
                bannerAd.offLoad && bannerAd.offLoad(onLoad);
                Logger.log(self, 'bannerAd onLoad: ' + JSON.stringify(res));
                bannerAd.style.top = windowHeight - 104;
                bannerAd.style.left = (windowWidth - bannerAd.width) / 2;
                bannerAd.show().then(() => {
                    Logger.log(self, 'bannerAd show ok');
                    if (handler != null) {
                        handler.runWith(true);
                    }
                }).catch((res) => {
                    Logger.log(self, 'bannerAd show error1: ' + JSON.stringify(res));
                    if (handler != null) {
                        handler.runWith(false);
                    }
                });
            }
            bannerAd.onLoad(onLoad);
            self.bannerAd = bannerAd;
        }
        closeBanner() {
            let self = this;
            if (self.bannerAd != null) {
                try {
                    self.bannerAd.destroy();
                }
                catch (e) {
                    Logger.log(self, '[TTPlatform] closeBanner error.' + e);
                }
            }
            self.bannerAd = null;
            GameHtMgr.Inst.hideAppBox();
        }
        showRewardedVideo(adRes, handler) {
            if (this.tt.createRewardedVideoAd == null) {
                UISystem.Inst.CreateWindowView(UITipView, "播放广告错误！");
                return;
            }
            this.createRewardedVideo(adRes, handler, this.video.slice());
        }
        createRewardedVideo(adRes, handler, posIds) {
            let self = this;
            if (posIds.length <= 0 || self.isLoadVideo) {
                return;
            }
            let videoAd = this.tt.createRewardedVideoAd({ adUnitId: posIds.pop() });
            function onError(res) {
                Logger.log(self, 'videoAd onError: ' + JSON.stringify(res));
                if (posIds.length > 0) {
                    self.createRewardedVideo(adRes, handler, posIds);
                }
                else {
                    self.playVideoError(handler);
                }
            }
            videoAd.onError(onError);
            function onClose(res) {
                videoAd.offClose(onClose);
                if (res.isEnded) {
                    Logger.log(self, '激励视频广告完成，发放奖励');
                }
                else {
                    Logger.log(self, '激励视频广告取消关闭，不发放奖励');
                    self.setLookVideoTime(adRes.id, 0);
                }
                GameDispatcher.Inst.event(UIGameEvent.SHOW_AD, [adRes, res.isEnded]);
                if (handler != null) {
                    handler.runWith(res.isEnded);
                }
                self.isShowRewardAd = false;
                self.isLoadVideo = false;
                SoundMgr.Inst.Reset();
                Laya.timer.scale = 1;
            }
            videoAd.onClose(onClose);
            videoAd.load()
                .then(() => {
                Logger.log(self, '激励视频加载成功');
                videoAd.show().then(() => {
                    Logger.log(self, '激励视频 广告显示成功');
                    self.isShowRewardAd = true;
                    self.videoTime = StorageUtils.getCurTimer();
                    self.setLookVideoTime(adRes.id);
                    SoundMgr.Inst.stop();
                    Laya.timer.scale = 0;
                }).catch(err => {
                    Logger.log(self, '激励视频 广告显示失败');
                    self.playVideoError(handler);
                });
            })
                .catch(err => {
                Logger.log(self, '激励视频加载失败');
                self.playVideoError(handler);
            });
        }
        playVideoError(handler) {
            this.isShowRewardAd = false;
            this.isLoadVideo = false;
            UISystem.Inst.CreateWindowView(UITipView, "暂无广告");
            if (handler != null) {
                handler.runWith(false);
            }
            SoundMgr.Inst.Reset();
            Laya.timer.scale = 1;
        }
        showInsert(adRes, handler) {
            let self = this;
            let t = StorageUtils.Inst.getGameTime();
            Logger.log(self, "[TTPlatform]=====>showInsert this.isToutiao: " + this.isToutiao + " t:" + t + " adRes.id: " + adRes.id);
            if (this.isLoadInsert) {
                Logger.log(self, "[TTPlatform]=====>正在加载插屏广告");
                return;
            }
            if (!this.isToutiao) {
                Logger.log(self, "[TTPlatform]=====>头条才支持插屏广告!");
                return;
            }
            if (StorageUtils.Inst.LoginTime <= 0 || t < 60000) {
                return;
            }
            let prevTime = this.getLookVideoTime(adRes.id);
            if (prevTime > 0) {
                let gapTime = StorageUtils.getCurTimer() - prevTime;
                if (gapTime < 60000) {
                    return;
                }
            }
            self._insertHandler = handler;
            Logger.log(self, "[TTPlatform]=====>showInsert success self.insert: " + self.insert);
            if (this.tt.createInterstitialAd == null) {
                console.log("无法创建插屏广告");
                return;
            }
            let insertAd = this.tt.createInterstitialAd({ adUnitId: self.insert });
            function onClose(res) {
                insertAd.offClose(onClose);
                insertAd.destroy();
                self.isLoadInsert = false;
                self.insertAd = null;
                Logger.log(self, 'interstitial onClose' + JSON.stringify(res));
            }
            insertAd.onClose(onClose);
            function onError(res) {
                insertAd.offError(onError);
                Logger.log(self, 'interstitial onError' + JSON.stringify(res));
                if (self._insertHandler != null) {
                    self._insertHandler.runWith(false);
                }
                self._insertHandler = null;
                self.isLoadInsert = false;
                self.insertAd = null;
            }
            insertAd.onError(onError);
            insertAd.load().then(() => {
                Logger.log(self, 'interstitial onLoaded');
                insertAd.show();
                self.isLoadInsert = false;
                if (self._insertHandler != null) {
                    self._insertHandler.runWith(true);
                }
                self._insertHandler = null;
                self.insertAd = null;
            }).catch((err) => {
                Logger.log(self, 'interstitial onError');
                console.log(err);
                if (self._insertHandler != null) {
                    self._insertHandler.runWith(false);
                }
                self._insertHandler = null;
                self.isLoadInsert = false;
                self.insertAd = null;
            });
            self.isLoadInsert = true;
            self.insertAd = insertAd;
            self.setLookVideoTime(adRes.id);
        }
        hideInsert() {
            if (this.insertAd != null) {
                this.insertAd.destroy();
            }
            this.insertAd = null;
            this._insertHandler = null;
        }
        vibrateShort() {
            let self = this;
            this.tt.vibrateShort({
                success(res) {
                },
                fail(res) {
                    Logger.log(self, `vibrateShort调用失败`);
                }
            });
        }
        onShareAppMessage2(handler) {
            this.shareHandler = handler;
            this.endTime = (new Date()).getTime() + 2000;
            let index = MathUtil.makeRandom(0, QQPlatform.titles.length);
            let title = QQPlatform.titles[index] || QQPlatform.titles[0];
            let imageUrl = QQPlatform.imageUrl.replace("icon.png", "icon_" + MathUtil.makeRandom(0, 3) + ".jpg");
            if (wx && wx["aldShareAppMessage"] != null) {
                wx["aldShareAppMessage"]({
                    title: title,
                    imageUrlId: QQPlatform.imageUrlId,
                    imageUrl: imageUrl,
                    query: ''
                });
            }
        }
        onShareAppMessage(handler) {
            let self = this;
            if (TTPlatform.videoPath == null) {
                UISystem.Inst.CreateWindowView(UITipView, "请先录制有效视频");
                return;
            }
            let shareIds = [this.shareId];
            let index = 0;
            let shareid = shareIds[index];
            Logger.log(self, "开始分享 shareid: " + shareid + " index: " + index + " videoPath: " + TTPlatform.videoPath);
            this.tt.shareAppMessage({
                channel: "video",
                templateId: "",
                title: "真实战地模拟射击",
                desc: "战地模拟，飞机坦克应有尽有" + self.addDesc,
                query: "",
                extra: { videoPath: TTPlatform.videoPath },
                success() {
                    TTPlatform.videoPath = null;
                    Logger.log(self, "分享成功");
                    if (handler != null)
                        handler.runWith(true);
                },
                fail(e) {
                    console.log(self, "分享失败:    " + JSON.stringify(e));
                    UISystem.Inst.CreateWindowView(UITipView, "分享视频失败，请稍后再试");
                    if (handler != null)
                        handler.runWith(false);
                }
            });
        }
        startLuzhi() {
            let self = this;
            if (this.isLuzhing) {
                return;
            }
            this.isLuzhing = true;
            self.lztime = 0;
            Laya.timer.loop(1000, self, self.updateLuzhing);
            GameDispatcher.Inst.event(UIGameEvent.TT_START_LUZHI);
            this.isStopLuzhi = false;
            const recorder = window["tt"].getGameRecorderManager();
            recorder.onStart(res => {
                self.startTime = Laya.timer.currTimer;
            });
            recorder.onStop(res => {
                self.isLuzhing = false;
                TTPlatform.videoPath = res.videoPath;
                if ((Laya.timer.currTimer - self.startTime) < TTPlatform.MIN_TIME) {
                    TTPlatform.videoPath = null;
                    UISystem.Inst.CreateWindowView(UITipView, "录屏时间太短！");
                }
                else if (!res.videoPath) {
                    UISystem.Inst.CreateWindowView(UITipView, "录屏地址为空");
                }
                else {
                    TTPlatform.videoPath = res.videoPath;
                }
                self.isStopLuzhi = false;
                self.isLuzhing = false;
                self.lztime = 0;
                Laya.timer.clear(self, self.updateLuzhing);
                GameDispatcher.Inst.event(UIGameEvent.TT_STOP_LUZHI);
            });
            recorder.start({ duration: TTPlatform.MAX_TIME });
        }
        updateLuzhing() {
            this.lztime++;
            if (this.lztime > TTPlatform.MAX_TIME) {
                this.lztime = TTPlatform.MAX_TIME;
            }
            GameDispatcher.Inst.event(UIGameEvent.TT_UPDATE_LUZHI, this.lztime);
        }
        confimShare() {
            this.onShareAppMessage(Laya.Handler.create(this, this.shareSuccess));
        }
        shareSuccess(isSuccess) {
            if (isSuccess) {
                let shareCount = StorageUtils.Inst.getShareCount();
                if (shareCount <= 0) {
                    StorageUtils.Inst.setShareCount();
                }
            }
        }
        stopLuzhi() {
            this.isStopLuzhi = true;
            this.isLuzhing = false;
            const recorder = window["tt"].getGameRecorderManager();
            recorder.stop();
            this.lztime = 0;
            Laya.timer.clear(this, this.updateLuzhing);
        }
        hasShortcutInstalled(handler) {
            let self = this;
            this.tt && this.tt["checkShortcut"] && this.tt["checkShortcut"]({
                success: function (res) {
                    if (handler != null)
                        handler.runWith(res.status.exist);
                },
                fail: function (err) {
                    Logger.log(self, JSON.stringify(err));
                },
                complete: function () { }
            });
        }
        installShortcut(handler) {
            let self = this;
            this.tt && this.tt["addShortcut"] && this.tt["addShortcut"]({
                success: function (res) {
                    if (handler != null)
                        handler.run();
                },
                fail: function (err) {
                    Logger.log(self, JSON.stringify(err));
                },
                complete: function () { }
            });
        }
        get tt() {
            return window["tt"];
        }
        get isIOS() {
            const systemInfo = window["tt"].getSystemInfoSync();
            return systemInfo && (systemInfo.platform === "ios" || systemInfo.model.toLowerCase().indexOf("iphone") != -1);
        }
        get isToutiao() {
            return this.tt.getSystemInfoSync().appName === "Toutiao";
        }
        get isDouyin() {
            return this.tt.getSystemInfoSync().appName === "Douyin";
        }
        get isXigua() {
            return this.tt.getSystemInfoSync().appName === "XiGua";
        }
        setKeepScreenOn(flag = true) {
            this.tt.setKeepScreenOn && this.tt.setKeepScreenOn({
                keepScreenOn: flag,
                success: function (result) {
                    console.info('设置屏幕常亮成功..');
                },
                fail() {
                    console.info('设置屏幕常亮失败..');
                }
            });
        }
        getBlack() {
            return this.result;
        }
        get app_version() {
            return this.tt.getSystemInfoSync().version;
        }
        get brand() {
            return this.tt.getSystemInfoSync().brand;
        }
        get model() {
            return this.tt.getSystemInfoSync().model;
        }
        GetVersionName() {
            return "version.json";
        }
        GetPlatformName() {
            return PlatformType.TYPE_TT;
        }
        getClipboardData() {
            this.tt.getClipboardData({
                success(res) {
                    console.log(`${res.data}`);
                    GameDispatcher.Inst.event(UIGameEvent.JIANQIEBAN, res.data + "");
                },
                fail(res) {
                    console.log(`getClipboardData调用失败`);
                },
            });
        }
        setClipboardData(_data) {
            this.tt.setClipboardData({
                data: _data,
                success(res) {
                    console.log(`setClipboardData调用成功`);
                    GameDispatcher.Inst.event(UIGameEvent.JIANQIEBAN);
                    return true;
                },
                fail(res) {
                    console.log(`setClipboardData调用失败`);
                    return false;
                },
            });
        }
    }
    TTPlatform.MAX_TIME = 300000;
    TTPlatform.MIN_TIME = 3000;
    TTPlatform.videoPath = null;

    class UINativeInsertView extends UIBaseWindow {
        constructor() {
            super("ad", "UIVVNativeInsertView");
            this.lab_des = null;
            this.lab_title = null;
            this.img_bg = null;
            this.icon_bg = null;
            this.btn_touming = null;
            this.group_center = null;
        }
        InitUI() {
            super.InitUI();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_AD;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowMaskBg = false;
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.ON_ACTIVE, this.onActive, this);
        }
        RemoveListener() {
            super.RemoveListener();
        }
        InitData(param) {
            super.InitData(param);
            console.log("[UIHWNativeInsertView]------->show : " + param);
            if (param == null) {
                return;
            }
            this.adRes = param[0];
            this.adData = param[1];
            this.nativeAd = param[2];
            this.showData(this.adData);
            this.hideBanner();
            Laya.timer.frameLoop(1, this, this.checkBannerShow);
        }
        checkBannerShow() {
            if (UISystem.Inst.HasView(UIHWNativeBannerView)) {
                UISystem.Inst.RemoveWindowClass(UIHWNativeBannerView);
            }
        }
        showData(data) {
            this.adData = data;
            if (this.adData) {
                let imgUrlList = this.adData.imgUrlList;
                if (imgUrlList && imgUrlList[0]) {
                    this.icon_bg.url = imgUrlList[0];
                }
                if (this.adData.title) {
                    this.lab_title.text = this.adData.title;
                }
                if (Laya.Browser.onHWMiniGame) {
                    if (this.adData.source) {
                        this.lab_des.text = this.adData.source;
                    }
                }
                else {
                    if (this.adData.desc) {
                        this.lab_des.text = this.adData.desc;
                    }
                }
                this.reportNativeShow(this.adData, this.nativeAd);
            }
        }
        onActive(bol) {
            if (bol) {
                this.close();
            }
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_touming || btn == this.btn_xq) {
                this.reportNativeClick(this.adData, this.nativeAd);
                this.close();
            }
        }
        reportNativeShow(nativeData, nativeAd = null) {
            Logger.debug(this, "=====>原生插屏 上报显示!");
            PlatformMgr.Inst.reportNativeShow(nativeData, nativeAd);
        }
        onBtnCloseClick() {
            super.onBtnCloseClick();
        }
        reportNativeClick(nativeData, nativeAd = null) {
            Logger.debug(this, "=====>原生插屏 上报点击!");
            PlatformMgr.Inst.reportNativeClick(nativeData, nativeAd);
        }
        hideBanner() {
        }
        Reset() {
            super.Reset();
            Laya.timer.clear(this, this.checkBannerShow);
            PlatformMgr.Inst.hideAd(AdType.ad_3, true);
            this.nativeAd = null;
            this.adData = null;
            this.icon_bg.url = "";
        }
    }

    class VivoPlatform extends BasePlatform {
        constructor() {
            super();
            this.isFirstVideo = true;
            this.isShow = true;
            this.insertTime = 0;
            this.nativeTime = 0;
            this.bannerTime = 0;
            this.lastShowRewardTime = NaN;
            this.lastGetNativeTime = 0;
            this.GetNativeIntervalTime = 3000;
            let lf = this;
            lf.result = 1;
            lf.openAd = 1;
            lf.banner = "a33537e174ef41599a15d87ace9f308a";
            lf.video = ["1981beff5ae34b52a6819d9cc04d0968"];
            lf.insert = "c0ea532572f248ee96cccae18f7a2bf2";
            lf.native = ["76f2dc4a177f40878e74d102e3132b39"];
            var showf = function () {
                if (lf.isShow)
                    return;
                lf.isShow = true;
                console.log("获得焦点");
                SoundMgr.Inst.Reset();
                GlobalConfig.isActive = true;
                Laya.timer.resume();
                lf.setKeepScreenOn(true);
                GameDispatcher.Inst.event(UIGameEvent.ON_ACTIVE, true);
            };
            qg.onShow(showf);
            var hidef = function () {
                if (!lf.isShow)
                    return;
                lf.isShow = false;
                console.log("失去焦点");
                SoundMgr.Inst.stop();
                GlobalConfig.isActive = false;
                Laya.timer.pause();
                GameDispatcher.Inst.event(UIGameEvent.ON_ACTIVE, false);
            };
            qg.onHide(hidef);
            Laya.timer.once(2000, this, this.updateWindowSize);
            qg.onError((res) => {
                if (res) {
                    console.log("res.location：" + res.location);
                    console.log("res.message: " + res.message);
                    console.error("res.stack: " + res.stack);
                }
            });
        }
        updateWindowSize() {
            PlatformModel.Inst.platfrom = this.GetPlatformName();
            let obj = qg.getSystemInfoSync();
            PlatformModel.Inst.setWindowSize(obj.screenWidth, obj.screenHeight);
            if (qg["getMenuButtonBoundingClientRect"] != null) {
                let pos = qg["getMenuButtonBoundingClientRect"]();
                PlatformModel.Inst.setRect(pos.left, pos.top, pos.width, pos.height + 10);
            }
        }
        registeredSubNames() {
        }
        login(openId, channel) {
        }
        hasShortcutInstalled(handler) {
            qg["hasShortcutInstalled"]({
                success: function (status) {
                    if (status) {
                        console.log('已创建');
                    }
                    else {
                        console.log('未创建');
                        if (handler != null)
                            handler.run();
                    }
                }
            });
        }
        installShortcut(handler) {
            qg["installShortcut"]({
                success: function () {
                    console.log('创建成功');
                    if (handler != null)
                        handler.run();
                },
                fail: function () {
                    console.log('创建失败');
                }
            });
        }
        hideAd(adId) {
            let adRes = AdResource.getItemBy2Keys(this.GetPlatformName(), adId);
            if (adRes == null)
                return;
            if (adRes.type == AdType.TYPE_NativeIcon) {
                UISystem.Inst.RemoveWindowClass(UIAdNativeIconView);
            }
            else if (adRes.type == AdType.TYPE_NativeBaner) {
                UISystem.Inst.RemoveWindowClass(UIAdNativeBannerView);
            }
            else if (adRes.type == AdType.TYPE_NativeInsert) {
                UISystem.Inst.RemoveWindowClass(UINatiEmbedView);
            }
            super.hideAd(adId);
        }
        showBanner(adRes, handler) {
            let ad = this.adDic[adRes.type];
            if (ad != null)
                return;
            let lf = this;
            let nativeAd = this.adDic[AdType.TYPE_NativeBaner];
            if (nativeAd) {
                UISystem.Inst.RemoveWindowClass(UIAdNativeBannerView);
                nativeAd.destroy && nativeAd.destroy();
                delete this.adDic[AdType.TYPE_NativeBaner];
            }
            let adId = adRes.adIds ? adRes.adIds[0] : lf.banner;
            var bannerAd = qg.createBannerAd({
                posId: adId,
                style: {}
            });
            lf.adDic[adRes.type] = bannerAd;
            function onSize(size) {
                bannerAd["offSize"](onSize);
                if (size)
                    console.log('Resize后正式宽高:', size.width, size.height);
                lf.sizeWidth = size.width;
                lf.sizeHeight = size.height;
                GameLayer.onResize();
                bannerAd.show().then(() => {
                    console.log('bannerAd show ok');
                }).catch((res) => {
                    if (res)
                        console.log('bannerAd show error', JSON.stringify(res));
                });
            }
            bannerAd["onSize"](onSize);
            var onError = function (err) {
                bannerAd.offError(onError);
                bannerAd["offSize"](onSize);
                Laya.timer.once(11000, lf, lf.showBanner, [adRes, handler]);
                switch (err.code) {
                    case 30003:
                        console.log("新用户7天内不能曝光Banner，请将手机时间调整为7天后，退出游戏重新进入");
                        break;
                    case 30009:
                        console.log("10秒内调用广告次数超过1次，10秒后再调用");
                        break;
                    case 30002:
                        console.log("加载广告失败，重新加载广告");
                        break;
                    case 30007:
                        console.log("Banner广告每日播放次数上限20次");
                        break;
                    default:
                        console.log("banner广告展示失败code:  " + err.code);
                        break;
                }
            };
            bannerAd.onError(onError);
            var onClose = function () {
                bannerAd["offClose"](onClose);
                console.log("banner广告onClose");
                lf.hideAd(adRes.id);
            };
            bannerAd["onClose"](onClose);
            let adshow = bannerAd.show();
            adshow && adshow.then(() => {
                console.log("banner广告展示成功");
                bannerAd.offError(onError);
                if (handler != null)
                    handler.run();
            }).catch((err) => {
                console.log("banner广告展示失败", err);
            });
        }
        canLookReward() {
            if (!isNaN(this.lastShowRewardTime) && TimeModel.Inst.LocalTime - this.lastShowRewardTime < 2000) {
                return false;
            }
            this.lastShowRewardTime = TimeModel.Inst.LocalTime;
            return true;
        }
        showRewardedVideo(adRes, handler) {
            if (this.openAd == 0) {
                handler && handler.runWith(true);
                return;
            }
            this.createRewardVideo(adRes, handler, this.video.slice());
        }
        createRewardVideo(adRes, handler, posIds) {
            let lf = this;
            if (posIds.length <= 0) {
                return;
            }
            if (!lf.canLookReward())
                return;
            let rewardedAd = qg.createRewardedVideoAd({
                posId: posIds.pop(),
                style: {}
            });
            function onClose(res) {
                GameDispatcher.Inst.event(UIGameEvent.SHOW_AD_COMPLETE, [adRes, res.isEnded, lf.GetPlatformName()]);
                rewardedAd.offError(onError);
                rewardedAd.offClose(onClose);
                if (handler != null)
                    handler.runWith(res.isEnded);
                if (res && res.isEnded) {
                    console.log("正常播放结束，可以下发游戏奖励");
                }
                else {
                    console.log("播放中途退出，不下发游戏奖励");
                    UISystem.Inst.CreateWindowView(UITipView, "视频没有播完，无法获得奖励");
                }
                SoundMgr.Inst.Reset();
            }
            rewardedAd.onClose(onClose);
            function onError(err) {
                rewardedAd.offError(onError);
                rewardedAd.offLoad(onLoad);
                rewardedAd.offClose(onClose);
                switch (err.errCode) {
                    case -3:
                        console.log("激励广告加载失败---调用太频繁", JSON.stringify(err));
                        UISystem.Inst.CreateWindowView(UITipView, "广告加载失败，请稍后再试");
                        break;
                    case -4:
                        console.log("激励广告加载失败--- 一分钟内不能重复加载", JSON.stringify(err));
                        if (posIds.length == 0) {
                            UISystem.Inst.CreateWindowView(UITipView, "您点的太快了，稍后再来哦");
                            if (!StorageUtils.Inst.IsCloseMusic())
                                SoundMgr.Inst.Reset();
                        }
                        else
                            lf.createRewardVideo(adRes, handler, posIds);
                        break;
                    case 30008:
                        break;
                    default:
                        console.log("激励广告展示失败code:    " + err.errCode);
                        UISystem.Inst.CreateWindowView(UITipView, "广告加载失败，请稍后再试");
                        break;
                }
            }
            rewardedAd.onError(onError);
            function onLoad() {
                rewardedAd.offLoad(onLoad);
                SoundMgr.Inst.stop();
                console.log("激励视频广告加载成功");
                let adshow = rewardedAd.show();
                adshow && adshow.catch(err => {
                    console.log("激励广告展示失败" + JSON.stringify(err));
                    lf.createRewardVideo(adRes, handler, posIds);
                });
            }
            rewardedAd.onLoad(onLoad);
            if (!lf.isFirstVideo) {
                rewardedAd.load();
            }
            else {
                lf.isFirstVideo = false;
            }
        }
        showInsert(adRes, handler) {
            this.showNative(adRes, handler);
        }
        CanGetNative() {
            if (this.lastGetNativeTime != 0 && new Date().getTime() - this.lastGetNativeTime < this.GetNativeIntervalTime) {
                return false;
            }
            this.lastGetNativeTime = new Date().getTime();
            return true;
        }
        showNative(adRes, handler) {
            if (!this.CanGetNative())
                return;
            let adIds = adRes.adIds ? adRes.adIds : this.native;
            this.createNative(adRes, adIds.slice(), handler);
        }
        createNative(adRes, posIds, handler) {
            if (posIds.length <= 0)
                return;
            let ad = this.adDic[adRes.type];
            if (ad != null)
                return;
            let lf = this;
            var nativeAd = qg.createNativeAd({
                posId: posIds.pop()
            });
            lf.adDic[adRes.type] = nativeAd;
            function onLoad(res) {
                nativeAd.offError(onError);
                nativeAd.offLoad(onLoad);
                let adList = res.adList;
                for (const iterator of adList) {
                    if ((iterator.logoUrl != null && iterator.logoUrl.length >= 1) ||
                        (iterator.imgUrlList != null && iterator.imgUrlList.length >= 1)) {
                        if (adRes.type == AdType.TYPE_NativeIcon) {
                            iterator.adResId = adRes.id;
                            UISystem.Inst.RemoveWindowClass(UIAdNativeIconView);
                            UISystem.Inst.CreateWindowView(UIAdNativeIconView, [iterator, Laya.Handler.create(null, showHandler, [iterator]),
                                Laya.Handler.create(null, clickHandler, [iterator], false)]);
                        }
                        else if (adRes.type == AdType.TYPE_NativeBaner) {
                            UISystem.Inst.RemoveWindowClass(UIAdNativeBannerView);
                            UISystem.Inst.CreateWindowView(UIAdNativeBannerView, [iterator, Laya.Handler.create(null, showHandler, [iterator]),
                                Laya.Handler.create(null, clickHandler, [iterator], false), adRes]);
                        }
                        else if (adRes.type == AdType.TYPE_NativeInsert) {
                            UISystem.Inst.RemoveWindowClass(UINatiEmbedView);
                            UISystem.Inst.CreateWindowView(UINatiEmbedView, [iterator, Laya.Handler.create(null, showHandler, [iterator]),
                                Laya.Handler.create(null, clickHandler, [iterator], false), adRes]);
                        }
                        else if (adRes.type == AdType.TYPE_Insert) {
                            UISystem.Inst.RemoveWindowClass(UINativeInsertView);
                            UISystem.Inst.CreateWindowView(UINativeInsertView, [adRes, iterator, nativeAd]);
                        }
                        break;
                    }
                }
            }
            nativeAd.onLoad(onLoad);
            function onError(err) {
                nativeAd.offError(onError);
                nativeAd.offLoad(onLoad);
                lf.hideAd(adRes.id);
                console.log(err);
                if (posIds.length > 0) {
                    lf.createNative(adRes, posIds, handler);
                }
            }
            nativeAd.onError(onError);
            nativeAd.load();
            function showHandler(adObj) {
                nativeAd.reportAdShow({
                    adId: adObj["adId"]
                });
            }
            function clickHandler(adObj) {
                nativeAd.reportAdClick({
                    adId: adObj["adId"]
                });
            }
        }
        reportNativeShow(nativeData, nativeAd = null) {
            if (nativeAd && nativeData) {
                console.log("-------->上报显示原生广告 Id: " + nativeData.adId);
                nativeAd.reportAdShow({ "adId": nativeData.adId });
            }
        }
        reportNativeClick(nativeData, nativeAd = null) {
            if (nativeData && nativeAd) {
                console.log("-------->上报点击原生广告 Id: " + nativeData.adId);
                nativeAd.reportAdClick({ "adId": nativeData.adId });
            }
        }
        vibrateShort() {
            qg.vibrateShort();
        }
        getStorage(key, handler) {
            qg.getStorage({
                "key": key,
                success: function (data) {
                    let obj = null;
                    if (!data)
                        obj = {};
                    else
                        obj = JSON.parse(data);
                    if (handler != null)
                        handler.runWith(obj);
                },
                fail: function (data, code) {
                    console.log(`handling fail, code = ${code}`);
                    if (handler != null)
                        handler.runWith({});
                }
            });
        }
        setStorage(key, value) {
            qg.setStorage({
                "key": key,
                "value": value
            });
        }
        checkNetwork() {
            qg.getNetworkType({
                success: function (data) {
                    console.log(`handling success: ${data.type}`);
                    if (data.type == "none") {
                        UISystem.Inst.CreateWindowView(UITipView, "网络错误，请重新加载");
                    }
                }
            });
        }
        setKeepScreenOn(flag = true) {
            let data = {
                keepScreenOn: flag,
                success: function (result) {
                    console.info('设置屏幕常亮成功..');
                },
                fail() {
                    console.info('设置屏幕常亮失败..');
                }
            };
            qg.setKeepScreenOn && qg.setKeepScreenOn(data);
        }
        GetPlatformName() {
            return PlatformType.TYPE_VIVO;
        }
    }

    class EventResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUTFString();
            this.wx = byteArray.readUTFString();
        }
        static getItemByKey(id) {
            return EventResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return EventResource.itemsDic[key1 + "," + key2];
        }
    }
    EventResource.items = new Array();
    EventResource.itemsDic = new Dictionary();

    class WeixinPlatform extends BasePlatform {
        constructor() {
            super();
            this.endTime = 0;
            this.shareHandler = null;
            this.SubscribeKey = "kZcqWn0_Mx_ouCZcCJXm5BDT9ZsJn1xebKUere0F8Ds";
            this.isShowRewardAd = false;
            this.isLoadVideo = false;
            this.loadingVideo = null;
            this.loadingVideoTime = 0;
            this.catchVideos = [];
            this.showNativeIds = [];
            let lf = this;
            wx["showShareMenu"]({ withShareTicket: false });
            wx["uma"]["onShareAppMessage"](function () {
                PlatformMgr.Inst.aldSendEvent(EventType.fenxiang, "开始");
                return {
                    title: WeixinPlatform.title,
                    imageUrlId: WeixinPlatform.imageUrlId,
                    imageUrl: WeixinPlatform.imageUrl,
                    query: ''
                };
            });
            wx["onHide"](function () {
                GlobalConfig.isActive = false;
                SoundMgr.Inst.stop();
                console.log("失去焦点");
                GameDispatcher.Inst.event(UIGameEvent.ON_ACTIVE, false);
            });
            lf.setKeepScreenOn(true);
            wx["onShow"](function () {
                lf.setKeepScreenOn(true);
                GlobalConfig.isActive = true;
                let localTime = TimeModel.Inst.GetLocalTime();
                SoundMgr.Inst.Reset();
                GameDispatcher.Inst.event(UIGameEvent.ON_ACTIVE, true);
                if (lf.endTime == 0 || lf.shareHandler == null) {
                    return;
                }
                else {
                    if (localTime >= lf.endTime) {
                        PlatformMgr.Inst.aldSendEvent(EventType.fenxiang, "成功");
                        lf.shareHandler.runWith(true);
                        if (this.openAd == 0) {
                            let res = AdResource.getItemBy2Keys(lf.GetPlatformName(), AdType.ad_2);
                            GameDispatcher.Inst.event(UIGameEvent.SHOW_AD_COMPLETE, [res, true]);
                        }
                    }
                    else {
                        UISystem.Inst.CreateWindowView(UITipView, "获取失败，请重新试试");
                        PlatformMgr.Inst.aldSendEvent(EventType.fenxiang, "失败");
                    }
                    lf.endTime = 0;
                    lf.shareHandler = null;
                }
            });
            lf.result = 1;
            lf.openAd = 1;
            lf.checkTime = 0;
            lf.banner = "adunit-edc6bdd0fadc93f4";
            lf.video = ["adunit-59a90c71155d1980", "adunit-d5661494c7eb9b17", "adunit-b42cf0fec4b75998", "adunit-12e555bff1c46a51", "adunit-d6b463d2b5440a1c"];
            lf.insert = "adunit-6605c9996a465cfb";
            lf.native = [""];
            this.nativeIconId = "adunit-b5746b62d6ebf018";
            let obj = {
                withSubscriptions: true,
                success(res) {
                    console.log("---------wx.getSetting: " + JSON.stringify(res));
                    if (res.subscriptionsSetting && res.subscriptionsSetting.mainSwitch) {
                        if (res.itemSettings && res.itemSettings[lf.SubscribeKey] == "accept") {
                            lf.hasSubscribeApp = true;
                        }
                    }
                }
            };
            wx.getSetting && wx.getSetting(obj);
            this.LoadRewardVideo(null, this.video.slice());
        }
        registeredSubNames() {
            this.AddSubName("music", "music");
            this.AddSubName("Scene", "Scene");
        }
        SetSwitchFps(fps) {
            let frame = 0;
            if (fps == GlobalJugger.checkFps20) {
                frame = 30;
                GlobalConfig.FPS = 1 / 30;
            }
            else {
                frame = 60;
                GlobalConfig.FPS = 1 / 60;
            }
            if (wx["setPreferredFramesPerSecond"]) {
                wx["setPreferredFramesPerSecond"](frame);
            }
        }
        login(openId, channel) {
            wx["uma"].init({
                appKey: '60e273e426a57f101845bd30',
                useOpenid: true,
                autoGetOpenid: true,
                debug: false
            });
            wx.showShareMenu({
                withShareTicket: true, menus: ['shareAppMessage', 'shareTimeline'],
                success(res) {
                    console.log('showShareMenu', res);
                }
            });
            Laya.timer.once(2000, this, this.updateWindowSize);
            let obj = wx["getLaunchOptionsSync"]();
            let scene = obj.scene + "";
            console.log("=====>obj.scene: " + scene);
            HttpUrl.sendPost("https://ga.gametdd.com/adCtrl/agd/conf", { 'id': "60f124e3fac3fe0a5ccf7621", "sid": scene }, Laya.Handler.create(this, (res) => {
                console.log("====>策略 " + JSON.stringify(res));
                let data = res["data"];
                AdModel.Inst.addPolicy(data, this.GetPlatformName());
            }), Laya.Handler.create(this, () => {
            }));
            GameDispatcher.Inst.on(UIGameEvent.RESIZE_BANNER, this, this.onResizeBanner);
        }
        updateWindowSize() {
            PlatformModel.Inst.platfrom = this.GetPlatformName();
            let obj = wx.getSystemInfoSync();
            PlatformModel.Inst.setWindowSize(obj.windowWidth, obj.windowHeight);
            if (wx["getMenuButtonBoundingClientRect"] != null) {
                let pos = wx["getMenuButtonBoundingClientRect"]();
                PlatformModel.Inst.setRect(pos.left, pos.top, pos.width, pos.height + 5);
            }
        }
        hideAd(adId, destroy = false) {
            let adRes = AdResource.getItemBy2Keys(this.GetPlatformName(), adId);
            if (adRes && adRes.type == AdType.TYPE_Banner) {
                let time = this.adDic[adRes.type + "_time"];
                if (destroy || TimeModel.Inst.LocalTime - time > 60000) {
                    this.destroyBanner();
                    delete this.adDic[adRes.type];
                }
                else {
                    this.closeBanner();
                }
                return;
            }
            else if (adRes && adRes.type == AdType.TYPE_NativeIcon) {
                let time = this.adDic[adRes.id + "_time"];
                if (destroy || TimeModel.Inst.LocalTime - time > 60000) {
                    this.hideNative(adRes, true);
                }
                else {
                    this.hideNative(adRes);
                }
                return;
            }
            super.hideAd(adId);
            if (adRes && adRes.type == AdType.TYPE_Insert) {
                this.hideInsert();
            }
        }
        showBanner(adRes, handler, data = null) {
            let self = this;
            if (this.openAd == 0)
                return;
            let ad = this.adDic[adRes.type];
            if (ad != null) {
                ad.show();
                self.resizeBanner(ad, data);
                return;
            }
            let windowHeight = PlatformModel.Inst.windowHeight;
            let offsetX = data && data.hasOwnProperty("offsetX") ? data.offsetX : 0;
            let adId = adRes.adIds && adRes.adIds.length > 0 ? adRes.adIds[0] : this.banner;
            let bannerAd = wx["createBannerAd"]({
                adUnitId: adId,
                adIntervals: 30,
                style: {
                    top: windowHeight,
                    left: offsetX,
                    width: 320
                }
            });
            this.bannerAd = bannerAd;
            this.adDic[adRes.type] = bannerAd;
            this.adDic[adRes.type + "_time"] = TimeModel.Inst.LocalTime;
            PlatformMgr.Inst.aldSendEvent(EventType.bannerguangggao, "开始显示");
            function onResize(size) {
                bannerAd.offResize(onResize);
                self.resizeBanner(bannerAd, data);
            }
            bannerAd.onResize(onResize);
            function onError(res) {
                bannerAd.offError(onError);
            }
            bannerAd.onError(onError);
            bannerAd.show().then(() => {
                bannerAd.offError(onError);
                console.log('bannerAd show ok');
                PlatformMgr.Inst.aldSendEvent(EventType.bannerguangggao, "显示成功");
                let time = this.adDic[adRes.type + "_time"];
                GameDispatcher.Inst.event(UIGameEvent.SHOW_AD_COMPLETE, [adRes, true, time]);
            }).catch((res) => {
                bannerAd.offError(onError);
                if (res) {
                    console.log('bannerAd show error', JSON.stringify(res));
                    self.destroyBanner();
                    delete self.adDic[adRes.type];
                    PlatformMgr.Inst.aldSendEvent(EventType.bannerguangggao, "显示失败" + (res["errCode"] ? res["errCode"] : ""));
                }
            });
        }
        closeBanner() {
            if (this.bannerAd) {
                this.bannerAd.style.top = PlatformModel.Inst.windowHeight - 1;
            }
        }
        destroyBanner() {
            let self = this;
            self.bannerAd && self.bannerAd.destroy();
            self.bannerAd = null;
        }
        onResizeBanner(data) {
            let windowHeight = PlatformModel.Inst.windowHeight;
            data = data || {};
            let bottomY = data ? (data.bottomY || Laya.stage.height) : Laya.stage.height;
            let bannerAd = this.adDic[AdType.TYPE_Banner];
            if (bannerAd != null) {
                bannerAd.show();
                let realHeight = bannerAd.style.realHeight || 111;
                let top = windowHeight - realHeight;
                if (bottomY > 0) {
                    top = Math.ceil(PlatformModel.Inst.scaleY * bottomY) - realHeight;
                }
                data.top = top;
                this.resizeBanner(bannerAd, data);
            }
        }
        showRewardedVideo(adRes, handler) {
            if (this.openAd == 0) {
                this.onShareAppMessage(handler);
                return;
            }
            let ad = this.adDic[adRes.type];
            if (ad != null) {
                console.log("正在显示视频广告，播放失败！！！");
                return;
            }
            console.log("=====>showRewardedVideo: " + this.catchVideos.length);
            let videoAd = this.catchVideos.pop();
            if (videoAd) {
                this.loadVideoComplete(adRes, handler, videoAd);
            }
            else {
                this.LoadRewardVideo(Laya.Handler.create(this, this.loadVideoComplete, [adRes, handler]), this.video.slice());
            }
        }
        LoadRewardVideo(complete, posIds, loadCount = 3) {
            if (this.isLoadVideo) {
                if (this.loadingVideoTime > 0 && (Laya.timer.currTimer - this.loadingVideoTime) >= 60000) {
                    this.destroyVideoAd(this.loadingVideo);
                    this.loadingVideo = null;
                    this.loadingVideoTime = 0;
                }
                else {
                    console.log("视频广告加载中...");
                    return;
                }
            }
            if (posIds.length <= 0) {
                return;
            }
            let lf = this;
            let posId = posIds.shift();
            let videoAd = wx["createRewardedVideoAd"]({
                "adUnitId": posId, "multiton": true
            });
            if (videoAd["_destroyed"]) {
                console.error('videoAd[_destroyed] = true');
                return;
            }
            function onError(err) {
                let errCode = err ? err.errCode : "";
                switch (errCode) {
                    case 1005:
                        console.log("广告正在审核中无法展示...");
                        break;
                    case 1006:
                        console.log("广告正在审核失败无法展示...");
                        break;
                }
                console.log('激励视频 广告显示失败 err: ' + JSON.stringify(err));
                lf.playVideoError();
            }
            videoAd.onError(onError);
            lf.isLoadVideo = true;
            lf.loadingVideo = videoAd;
            lf.loadingVideoTime = Laya.timer.currTimer;
            loadVideoAd(videoAd, loadCount);
            function loadVideoAd(videoAd, count) {
                console.log("加载视频广告 posId: " + posId + " count: " + (4 - count));
                videoAd.load().then(() => {
                    console.log('激励视频加载成功');
                    if (complete) {
                        complete.runWith(videoAd);
                    }
                    else {
                        lf.catchVideos.push(videoAd);
                        console.log("添加视频广告到缓存列表 lf.catchVideos.length: " + lf.catchVideos.length);
                    }
                    lf.isLoadVideo = false;
                    lf.loadingVideo = null;
                    lf.loadingVideoTime = 0;
                }).catch((err) => {
                    console.log('=====>加载视频广告失败 err: ' + JSON.stringify(err));
                    loadCount--;
                    if (loadCount > 0) {
                        loadVideoAd(videoAd, loadCount);
                    }
                    else {
                        lf.destroyVideoAd(videoAd);
                        lf.isLoadVideo = false;
                        lf.loadingVideo = null;
                        lf.loadingVideoTime = 0;
                        if (posIds.length > 0) {
                            lf.LoadRewardVideo(complete, posIds, 3);
                        }
                        else {
                            PlatformMgr.Inst.aldSendEvent(EventType.shipinguanggao, "加载失败" + err.errCode);
                            I18nUtils.ShowI18nById("no_advertisement");
                        }
                    }
                });
            }
        }
        playVideoError(handler = null) {
            this.isShowRewardAd = false;
            delete this.adDic[AdType.TYPE_RewardedVideo];
            this.destroyVideoAd(this.rewardVideo);
            this.rewardVideo = null;
            I18nUtils.ShowI18nById("no_advertisement");
            if (handler != null) {
                handler.runWith(false);
            }
        }
        loadVideoComplete(adRes, handler, videoAd) {
            this.ShowAdReward(adRes, videoAd, 3, handler);
        }
        ShowAdReward(adRes, videoAd, index, handler = null) {
            let lf = this;
            if (index <= 0) {
                delete lf.adDic[AdType.TYPE_RewardedVideo];
                lf.rewardVideo = null;
                I18nUtils.ShowI18nById("no_advertisement");
                return;
            }
            this.rewardVideo = videoAd;
            lf.adDic[AdType.TYPE_RewardedVideo] = this.rewardVideo;
            function onClose(res) {
                GameDispatcher.Inst.event(UIGameEvent.SHOW_AD_COMPLETE, [adRes, res.isEnded, lf.GetPlatformName()]);
                handler && handler.runWith(res.isEnded);
                if (res == null || res.isEnded) {
                    PlatformMgr.Inst.aldSendEvent(EventType.shipinguanggao, adRes.name + "_播放完成");
                    console.log('激励视频广告完成，发放奖励');
                }
                else {
                    PlatformMgr.Inst.aldSendEvent(EventType.shipinguanggao, adRes.name + "_播放取消");
                    console.log('激励视频广告取消关闭，不发放奖励');
                }
                AdCtrl.Inst.clearGrothCount(res.isEnded);
                videoAd.offClose(onClose);
                delete lf.adDic[AdType.TYPE_RewardedVideo];
                lf.destroyVideoAd(lf.rewardVideo);
                lf.rewardVideo = null;
                lf.isShowRewardAd = false;
                SoundMgr.Inst.Reset();
                lf.LoadRewardVideo(null, lf.video.slice());
            }
            videoAd.onClose(onClose);
            videoAd.show().then(() => {
                console.log('=====>激励视频 广告显示成功');
                lf.isShowRewardAd = true;
                PlatformMgr.Inst.aldSendEvent(EventType.shipinguanggao, adRes.name + "_播放开始");
            }).catch((err) => {
                index--;
                if (index > 0) {
                    lf.ShowAdReward(adRes, videoAd, index, handler);
                }
                else {
                    I18nUtils.ShowI18nById("no_advertisement");
                    lf.destroyVideoAd(lf.rewardVideo);
                    lf.rewardVideo = null;
                    lf.isShowRewardAd = false;
                }
            });
        }
        destroyVideoAd(videoAd) {
            videoAd && videoAd.destroy && videoAd.destroy();
        }
        showInsert(adRes, handler) {
            if (this.openAd == 0 || !this.insert)
                return;
            let lf = this;
            const version = wx["getSystemInfoSync"]().SDKVersion;
            if (this.compareVersion(version, '2.8.0') < 0) {
                console.log("版本" + version + "低于2.8.0, 插屏不播放");
                return;
            }
            let adId = adRes.adIds ? adRes.adIds[0] || this.insert : this.insert;
            let interstitialAd = wx["createInterstitialAd"]({ "adUnitId": adId });
            function onClose(res) {
                interstitialAd.offLoad(onLoad);
                interstitialAd.offError(onError);
                interstitialAd.offClose(onClose);
                interstitialAd.destroy();
                console.log('interstitial onClose', JSON.stringify(res));
            }
            interstitialAd.onClose(onClose);
            function onError(res) {
                interstitialAd.offLoad(onLoad);
                interstitialAd.offError(onError);
                interstitialAd.offClose(onClose);
                console.log('interstitial onError', JSON.stringify(res));
            }
            interstitialAd.onError(onError);
            function onLoad(res) {
                interstitialAd.offLoad(onLoad);
                console.log('interstitial onLoad', JSON.stringify(res));
                interstitialAd.show();
            }
            interstitialAd.onLoad(onLoad);
            interstitialAd.load();
        }
        hideInsert() {
        }
        showNative(adRes, handler, data = null) {
            let adIds = adRes.adIds ? adRes.adIds : this.native;
            this.createNative(adRes, adIds.slice(), handler, data);
        }
        createNative(adRes, posIds, handler, data = null) {
            if (posIds.length <= 0)
                return;
            if (this.showNativeIds.indexOf(adRes.id) == -1) {
                this.showNativeIds.push(adRes.id);
            }
            let lf = this;
            let ad = lf.adDic[adRes.id];
            if (ad != null) {
                this.showCustomAd(ad, adRes, handler, data);
                return;
            }
            let left = data && data.hasOwnProperty("left") ? data.left : 0;
            let top = data && data.hasOwnProperty("top") ? data.top : 0;
            const customAd = wx["createCustomAd"] && wx["createCustomAd"]({
                adUnitId: posIds.pop(),
                style: {
                    left: left,
                    top: top,
                    fixed: false
                }
            });
            if (!customAd)
                return;
            lf.adDic[adRes.id] = customAd;
            lf.adDic[adRes.id + "_time"] = TimeModel.Inst.LocalTime;
            customAd && customAd.onError(function (err) {
                console.log(err);
                if (posIds.length > 0) {
                    lf.createNative(adRes, posIds, handler);
                }
                else {
                    lf.hideNative(adRes, true);
                }
            });
            function onLoad() {
                if (lf.showNativeIds.indexOf(adRes.id) != -1) {
                    lf.showCustomAd(customAd, adRes, handler, data);
                }
            }
            customAd && customAd.onLoad(onLoad);
            customAd && customAd.onClose(() => {
                console.log("原生模板广告关闭");
                lf.hideNative(adRes, true);
                GameDispatcher.Inst.event(UIGameEvent.CLOSE_AD, adRes);
            });
        }
        showCustomAd(customAd, adRes, handler = null, data = null) {
            let self = this;
            customAd && customAd.show().then(() => {
                console.warn("显示原生格子广告成功");
            }).catch((err) => {
                self.hideNative(adRes, true);
                self.showNative(adRes, handler, data);
            });
        }
        hideNative(adRes, destroy = false) {
            if (!adRes)
                return;
            let ad = this.adDic[adRes.id];
            if (destroy) {
                if (ad && !ad.isDestroyed()) {
                    ad.destroy();
                }
                delete this.adDic[adRes.id];
            }
            else {
                if (ad && !ad.isDestroyed()) {
                    ad.hide();
                }
            }
            let index = this.showNativeIds.indexOf(adRes.id);
            if (index != -1) {
                ArrayUtil.RemoveAt(this.showNativeIds, index);
            }
        }
        onShareAppMessage(handler) {
            this.shareHandler = handler;
            this.endTime = TimeModel.Inst.GetLocalTime() + 2000;
            wx["uma"] && wx["uma"]["shareAppMessage"]({
                title: WeixinPlatform.title,
                imageUrlId: WeixinPlatform.imageUrlId,
                imageUrl: WeixinPlatform.imageUrl,
                query: ''
            });
            PlatformMgr.Inst.aldSendEvent(EventType.fenxiang, "开始");
        }
        isShowShare() {
            return true;
        }
        aldSendEvent(id, param = null) {
            let eventRes = EventResource.getItemByKey(id);
            if (eventRes == null)
                return;
            if (param != null) {
                wx["uma"]["trackEvent"](eventRes.wx, { "config": param });
            }
            else {
                wx["uma"]["trackEvent"](eventRes.wx);
            }
        }
        navigateTo(value) {
            let self = this;
            let data = {
                fid: value["fid"],
                sceneName: value["sceneName"],
                channelName: ''
            };
            let options = {
                appId: value["appId"],
                envVersion: 'release',
                path: value["path"],
                extraData: {},
                fail: function () {
                    console.log('跳转失败:', arguments);
                },
                success: function () {
                    console.log('跳转成功');
                },
                complete: function () {
                }
            };
            wx.navigateToMiniProgram(options);
        }
        hasShortcutInstalled(handler) {
            if (StorageUtils.Inst.HasGetDesktop())
                return;
            let obj = wx["getLaunchOptionsSync"]();
            let scene = obj["scene"];
            if (isNaN(scene))
                scene = 0;
            if (handler != null)
                handler.run();
        }
        installShortcut(handler) {
        }
        onAddToFavorites(handler = null) {
            if (StorageUtils.Inst.HasAddToFavorites()) {
                handler.runWith(false);
                return;
            }
            wx["onAddToFavorites"](() => {
                StorageUtils.Inst.SetAddToFavorites();
                handler && handler.runWith(true);
                return {
                    imageUrl: WeixinPlatform.imageUrl
                };
            });
        }
        subscribeAppMsg(handler = null) {
            let self = this;
            if (!self.hasSubscribeApp) {
                wx["requestSubscribeMessage"] && wx["requestSubscribeMessage"]({
                    "tmplIds": [self.SubscribeKey],
                    "success": (res) => {
                        console.log("--------->requestSubscribeMessage: " + JSON.stringify(res));
                        if (res.errMsg == "requestSubscribeMessage:ok" && res[self.SubscribeKey] == "accept") {
                            self.hasSubscribeApp = true;
                            handler && handler.runWith(true);
                        }
                        else {
                            handler && handler.runWith(false);
                        }
                    },
                    "fail": (res) => {
                        console.log("---------fail: " + (res ? JSON.stringify(res) : null));
                    }
                });
            }
        }
        GetPlatformName() {
            return PlatformType.TYPE_WEIXIN;
        }
        vibrateShort() {
            wx["vibrateShort"]({});
        }
        setKeepScreenOn(flag = true) {
            let obj = {
                keepScreenOn: flag,
                success: function (result) {
                    console.info('设置屏幕常亮成功..');
                },
                fail() {
                    console.info('设置屏幕常亮失败..');
                }
            };
            wx.setKeepScreenOn && wx.setKeepScreenOn(obj);
        }
    }
    WeixinPlatform.imageUrlId = "d76mIFIXQpKDPEp1SjK5Ug==";
    WeixinPlatform.title = "快来体验在线撸猫的乐趣";
    WeixinPlatform.imageUrl = "https://mmocgame.qpic.cn/wechatgame/LauSTzq43wDb4VfInL75BicPBoYPrPMtsYhoicGvkjicmpvEkVECCVxbicQuhyImg3mt/0";

    class XMPlatform extends BasePlatform {
        constructor() {
            super();
            this.isFirstVideo = true;
            this.isShow = true;
            this.insertTime = 0;
            this.nativeTime = 0;
            this.bannerTime = 0;
            this.bigRect = new Laya.Rectangle(370, 180, 146, 49);
            this.smallRect = new Laya.Rectangle(508, 100, 194, 65);
            this.bannerCloseTime = NaN;
            this.bannerShowTime = NaN;
            this.bannerShowed = false;
            this.lastShowRewardTime = NaN;
            this.rewardedAd = null;
            this.lastGetNativeTime = 0;
            this.GetNativeIntervalTime = 3000;
            let lf = this;
            lf.result = 1;
            lf.openAd = 1;
            lf.banner = "a3185827d77f5a854820beae000e81b6";
            lf.video = ["a5c313d3b234adf832eb247f901e06bf"];
            lf.insert = "54b21f951ea0c84c2bc5feb78323f782";
            lf.native = [];
            var showf = function () {
                if (lf.isShow)
                    return;
                lf.isShow = true;
                console.log("获得焦点");
                SoundMgr.Inst.Reset();
                GlobalConfig.isActive = true;
                lf.setKeepScreenOn(true);
                GameDispatcher.Inst.event(UIGameEvent.ON_ACTIVE, true);
            };
            qg.onShow(showf);
            var hidef = function () {
                if (!lf.isShow)
                    return;
                lf.isShow = false;
                console.log("失去焦点");
                SoundMgr.Inst.stop();
                GlobalConfig.isActive = false;
                GameDispatcher.Inst.event(UIGameEvent.ON_ACTIVE, false);
            };
            qg.onHide(hidef);
            Laya.timer.once(2000, this, this.updateWindowSize);
            qg.onError((res) => {
                if (res) {
                }
            });
        }
        updateWindowSize() {
            PlatformModel.Inst.platfrom = this.GetPlatformName();
            let obj = qg.getSystemInfoSync();
            PlatformModel.Inst.setWindowSize(obj.screenWidth, obj.screenHeight);
            if (qg["getMenuButtonBoundingClientRect"] != null) {
                let pos = qg["getMenuButtonBoundingClientRect"]();
                PlatformModel.Inst.setRect(pos.left, pos.top, pos.width, pos.height);
                console.log("右上角位置:" + JSON.stringify(pos));
            }
        }
        registeredSubNames() {
        }
        login(openId, channel) {
        }
        hasShortcutInstalled(handler) {
            let shortcut = qg["getShortcut"]();
            shortcut.hasInstalled({
                success: function (status) {
                    if (status) {
                        console.log('已创建');
                    }
                    else {
                        console.log('未创建');
                        if (handler != null)
                            handler.run();
                    }
                }
            });
        }
        installShortcut(handler) {
            let shortcut = qg["getShortcut"]();
            shortcut.install({
                success: function () {
                    console.log('handling success');
                    console.log('创建成功');
                    if (handler != null)
                        handler.run();
                },
                fail: function () {
                    console.log('创建失败');
                }
            });
        }
        hideAd(adId, destroy = true) {
            let adRes = AdResource.getItemBy2Keys(this.GetPlatformName(), adId);
            if (adRes == null)
                return;
            if (adRes.type == AdType.TYPE_Banner) {
                let ad = this.adDic[adRes.type];
                if (ad) {
                    this.bannerShowed = false;
                    if (destroy) {
                        ad && ad.destroy && ad.destroy();
                        delete this.adDic[adRes.type];
                        console.log("[XMPlatform]=====>销毁广告 adRes.type: " + adRes.type);
                    }
                    else {
                        ad && ad.hide && ad.hide();
                        console.log("[XMPlatform]=====>隐藏广告 adRes.type: " + adRes.type);
                    }
                }
            }
            else {
                if (adRes.type == AdType.TYPE_NativeIcon) {
                    UISystem.Inst.RemoveWindowClass(UIAdNativeIconView);
                }
                else if (adRes.type == AdType.TYPE_NativeBaner) {
                    UISystem.Inst.RemoveWindowClass(UIAdNativeBannerView);
                }
                else if (adRes.type == AdType.TYPE_NativeInsert) {
                    UISystem.Inst.RemoveWindowClass(UINatiEmbedView);
                }
                super.hideAd(adId, destroy);
            }
        }
        canCreateBanner() {
            if (!isNaN(this.bannerCloseTime) && Laya.timer.currTimer - this.bannerCloseTime < 30000)
                return false;
            if (!isNaN(this.bannerShowTime) && Laya.timer.currTimer - this.bannerShowTime < 5000)
                return false;
            return true;
        }
        showBanner(adRes, handler, data = null) {
            let ad = this.adDic[adRes.type];
            if (ad != null) {
                if (!this.bannerShowed) {
                    this.bannerShowed = true;
                    ad.show && ad.show();
                    console.log("bannerShow");
                }
                return;
            }
            let lf = this;
            if (!this.canCreateBanner()) {
                console.log("banner广告关闭后30s后才能再次创建");
                return;
            }
            let nativeAd = this.adDic[AdType.TYPE_NativeBaner];
            if (nativeAd) {
                UISystem.Inst.RemoveWindowClass(UIAdNativeBannerView);
                nativeAd.destroy && nativeAd.destroy();
                delete this.adDic[AdType.TYPE_NativeBaner];
            }
            let adId = adRes.adIds ? adRes.adIds[0] : lf.banner;
            var bannerAd = qg.createBannerAd({
                adUnitId: adId
            });
            lf.adDic[adRes.type] = bannerAd;
            function onSize(size) {
                bannerAd["offResize"](onSize);
                lf.resizeBanner(bannerAd, data);
                if (size)
                    console.log('Resize后正式宽高:', size.width, size.height);
                lf.sizeWidth = size.width;
                lf.sizeHeight = size.height;
                GameLayer.onResize();
                bannerAd.show().then(() => {
                    lf.bannerShowed = true;
                    console.log('bannerAd show ok');
                }).catch((res) => {
                    if (res)
                        console.log('bannerAd show error', JSON.stringify(res));
                });
            }
            bannerAd["onResize"] && bannerAd["onResize"](onSize);
            var onError = function (err) {
                bannerAd.offError(onError);
                bannerAd["offSize"](onSize);
                lf.hideAd(AdType.ad_1, true);
                console.log("banner广告展示失败code:  " + err.code);
            };
            bannerAd.onError(onError);
            var onClose = function () {
                bannerAd["offClose"](onClose);
                console.log("banner广告onClose");
                lf.hideAd(adRes.id);
                lf.bannerCloseTime = Laya.timer.currTimer;
            };
            bannerAd["onClose"](onClose);
            let adshow = bannerAd.show();
            adshow && adshow.then(() => {
                console.log("banner广告展示成功");
                bannerAd.offError(onError);
                if (handler != null)
                    handler.run();
            }).catch((err) => {
                lf.hideAd(AdType.ad_1, true);
                console.log("banner广告展示失败", err);
            });
            lf.bannerShowTime = Laya.timer.currTimer;
            Laya.timer.clear(lf, lf.CheckUpdateBanner);
            Laya.timer.loop(1000, lf, lf.CheckUpdateBanner);
        }
        CheckUpdateBanner() {
            if (!isNaN(this.bannerShowTime) && Laya.timer.currTimer - this.bannerShowTime > 40000) {
                this.CheckShowBanner();
            }
        }
        CheckShowBanner() {
            let topView = UISystem.Inst.getTopWindow([UIGuideView, UITipView, UILoadingLeafView]);
            if (topView instanceof UIBaseWindow && topView.isShowBanner && GuideModel.Inst.IsCanShowBtns()) {
                this.hideAd(AdType.ad_1, true);
                topView.reshowBanner();
                this.bannerShowTime = Laya.timer.currTimer;
                console.log("banner广告刷新");
            }
        }
        showRewardedVideo(adRes, handler) {
            if (this.openAd == 0) {
                handler && handler.runWith(true);
                return;
            }
            this.createRewardVideo(adRes, handler, this.video.slice());
        }
        canLookReward() {
            if (!isNaN(this.lastShowRewardTime) && TimeModel.Inst.LocalTime - this.lastShowRewardTime < 2000) {
                return false;
            }
            this.lastShowRewardTime = TimeModel.Inst.LocalTime;
            return true;
        }
        createRewardVideo(adRes, handler, posIds) {
            let lf = this;
            if (posIds.length <= 0) {
                return;
            }
            if (!this.canLookReward())
                return;
            if (lf.rewardedAd) {
                return;
            }
            let rewardedAd = qg.createRewardedVideoAd({
                adUnitId: posIds.pop()
            });
            lf.rewardedAd = rewardedAd;
            function onClose(res) {
                lf.rewardedAd = null;
                GameDispatcher.Inst.event(UIGameEvent.SHOW_AD_COMPLETE, [adRes, res.isEnded, lf.GetPlatformName()]);
                rewardedAd.offError(onError);
                rewardedAd.offClose(onClose);
                if (handler != null)
                    handler.runWith(res.isEnded);
                if (res && res.isEnded) {
                    console.log("正常播放结束，可以下发游戏奖励");
                }
                else {
                    console.log("播放中途退出，不下发游戏奖励");
                    UISystem.Inst.CreateWindowView(UITipView, "视频没有播完，无法获得奖励");
                }
                SoundMgr.Inst.Reset();
            }
            rewardedAd.onClose(onClose);
            function onError(err) {
                rewardedAd.offError(onError);
                rewardedAd.offLoad(onLoad);
                rewardedAd.offClose(onClose);
                lf.rewardedAd = null;
                if (err) {
                    console.log(`激励视频error: errorMsg:${err.errMsg}, erroCode: ${err.errCode}`);
                }
                UISystem.Inst.CreateWindowView(UITipView, "广告加载失败，请稍后再试");
            }
            rewardedAd.onError(onError);
            function onLoad() {
                rewardedAd.offLoad(onLoad);
                SoundMgr.Inst.stop();
                console.log("激励视频广告加载成功");
                let adshow = rewardedAd.show();
                adshow && adshow.catch((err) => {
                    console.log("激励广告展示失败" + JSON.stringify(err));
                    lf.createRewardVideo(adRes, handler, posIds);
                });
            }
            rewardedAd.onLoad(onLoad);
            if (!lf.isFirstVideo) {
                rewardedAd.load();
            }
            else {
                lf.isFirstVideo = false;
            }
        }
        showInsert(adRes, handler) {
            if (!qg["createInterstitialAd"]) {
                console.log("createInterstitialAd==null");
                return;
            }
            if (Laya.timer.currTimer <= (this.insertTime + 30000)) {
                return;
            }
            let lf = this;
            let adId = adRes.adIds ? adRes.adIds[0] : lf.insert;
            var interstitialAd = qg["createInterstitialAd"]({
                adUnitId: adId
            });
            var onClose = function (data) {
                interstitialAd.offClose(onClose);
                lf.insertTime = Laya.timer.currTimer;
                console.log('close Insert');
            };
            interstitialAd.onClose(onClose);
            var onError = function (err) {
                interstitialAd.offError(onError);
                console.log(`插屏广告error: errorMsg:${err.errMsg}, erroCode: ${err.errCode}`);
            };
            interstitialAd.onError(onError);
            var onLoad = function () {
                console.log("intersti广告开始展示");
                interstitialAd["offLoad"](onLoad);
                let adShow = interstitialAd.show();
                adShow && adShow.then(() => {
                    console.log("插屏广告展示成功");
                    if (handler != null)
                        handler.run();
                }).catch((err) => {
                    console.log("插屏广告展示失败");
                });
            };
            interstitialAd["onLoad"](onLoad);
        }
        CanGetNative() {
            if (this.lastGetNativeTime != 0 && new Date().getTime() - this.lastGetNativeTime < this.GetNativeIntervalTime) {
                return false;
            }
            this.lastGetNativeTime = new Date().getTime();
            return true;
        }
        showNative(adRes, handler) {
            if (!this.CanGetNative())
                return;
            let adIds = adRes.adIds ? adRes.adIds : this.native;
            this.createNative(adRes, adIds.slice(), handler);
        }
        createNative(adRes, posIds, handler) {
            if (posIds.length <= 0)
                return;
            let ad = this.adDic[adRes.type];
            if (ad != null)
                return;
            let lf = this;
            var nativeAd = qg.createNativeAd({
                posId: posIds.pop()
            });
            lf.adDic[adRes.type] = nativeAd;
            function onLoad(res) {
                nativeAd.offError(onError);
                nativeAd.offLoad(onLoad);
                let adList = res.adList;
                for (const iterator of adList) {
                    if ((iterator.logoUrl != null && iterator.logoUrl.length >= 1) ||
                        (iterator.imgUrlList != null && iterator.imgUrlList.length >= 1)) {
                        if (adRes.type == AdType.TYPE_NativeIcon) {
                            iterator.adResId = adRes.id;
                            UISystem.Inst.RemoveWindowClass(UIAdNativeIconView);
                            UISystem.Inst.CreateWindowView(UIAdNativeIconView, [iterator, Laya.Handler.create(null, showHandler, [iterator]),
                                Laya.Handler.create(null, clickHandler, [iterator], false)]);
                        }
                        else if (adRes.type == AdType.TYPE_NativeBaner) {
                            UISystem.Inst.RemoveWindowClass(UIAdNativeBannerView);
                            UISystem.Inst.CreateWindowView(UIAdNativeBannerView, [iterator, Laya.Handler.create(null, showHandler, [iterator]),
                                Laya.Handler.create(null, clickHandler, [iterator], false), adRes]);
                        }
                        else if (adRes.type == AdType.TYPE_NativeInsert) {
                            UISystem.Inst.RemoveWindowClass(UINatiEmbedView);
                            UISystem.Inst.CreateWindowView(UINatiEmbedView, [iterator, Laya.Handler.create(null, showHandler, [iterator]),
                                Laya.Handler.create(null, clickHandler, [iterator], false), adRes]);
                        }
                        else if (adRes.type == AdType.TYPE_Insert) {
                            UISystem.Inst.RemoveWindowClass(UIHWNativeInsertView);
                            UISystem.Inst.CreateWindowView(UIHWNativeInsertView, [adRes, iterator, nativeAd]);
                        }
                        break;
                    }
                }
            }
            nativeAd.onLoad(onLoad);
            function onError(err) {
                nativeAd.offError(onError);
                nativeAd.offLoad(onLoad);
                lf.hideAd(adRes.id);
                console.log(err);
                if (posIds.length > 0) {
                    lf.createNative(adRes, posIds, handler);
                }
            }
            nativeAd.onError(onError);
            nativeAd.load();
            function showHandler(adObj) {
                nativeAd.reportAdShow({
                    adId: adObj["adId"]
                });
            }
            function clickHandler(adObj) {
                nativeAd.reportAdClick({
                    adId: adObj["adId"]
                });
            }
        }
        reportNativeShow(nativeData, nativeAd = null) {
            if (nativeAd && nativeData) {
                console.log("-------->上报显示原生广告 Id: " + nativeData.adId);
                nativeAd.reportAdShow({ "adId": nativeData.adId });
            }
        }
        reportNativeClick(nativeData, nativeAd = null) {
            if (nativeData && nativeAd) {
                console.log("-------->上报点击原生广告 Id: " + nativeData.adId);
                nativeAd.reportAdClick({ "adId": nativeData.adId });
            }
        }
        vibrateShort() {
            qg.vibrateShort && qg.vibrateShort();
        }
        checkNetwork() {
            qg.getNetworkType && qg.getNetworkType({
                success: function (data) {
                    console.log(`handling success: ${data.type}`);
                    if (data.type == "none") {
                        UISystem.Inst.CreateWindowView(UITipView, "网络错误，请重新加载");
                    }
                }
            });
        }
        setKeepScreenOn(flag = true) {
            let data = {
                keepScreenOn: flag,
                success: function (result) {
                    console.info('设置屏幕常亮成功..');
                },
                fail() {
                    console.info('设置屏幕常亮失败..');
                }
            };
            qg.setKeepScreenOn && qg.setKeepScreenOn(data);
        }
        GetPlatformName() {
            return PlatformType.TYPE_XIAOMI;
        }
    }

    class PlatformMgr {
        constructor() {
            this.curPlatform = null;
            this.onRegistered();
            this.curPlatform.registeredSubNames();
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new PlatformMgr();
            return this._Inst;
        }
        onRegistered() {
            if (Laya.Browser.onMiniGame) {
                this.curPlatform = new WeixinPlatform();
            }
            else if (Laya.Browser.onQGMiniGame) {
                this.curPlatform = new OppoPlatform();
            }
            else if (Laya.Browser.onVVMiniGame) {
                this.curPlatform = new VivoPlatform();
            }
            else if (Laya.Browser.onQQMiniGame) {
                this.curPlatform = new QQPlatform();
            }
            else if (this.isMZ) {
                this.curPlatform = new MzPlatform();
            }
            else if (this.isTT) {
                this.curPlatform = new TTPlatform();
            }
            else if (Laya.Browser.onHWMiniGame) {
                this.curPlatform = new HWPlatform();
            }
            else if (window["conch"]) {
                this.curPlatform = new AndroidPlatform();
            }
            else if (Laya.Browser.onBDMiniGame) {
                this.curPlatform = new BaiduPlatform();
            }
            else if (Laya.Browser.onKGMiniGame) {
                this.curPlatform = new XMPlatform();
            }
            else {
                this.curPlatform = new NormalPlatform();
            }
        }
        GetPlatformName() {
            return this.curPlatform.GetPlatformName();
        }
        GetSubName(url) {
            return this.curPlatform.GetSubName(url);
        }
        get isReview() {
            return this.curPlatform.isReview;
        }
        login(openId = null, channel = null, complete = null) {
            PlatformCtrl.Inst.init();
            this.curPlatform.login(openId, channel, complete);
        }
        showAd(adId, handler = null, data = null) {
            this.curPlatform.showAd(adId, handler, data);
        }
        hideAd(adId, destroy = true) {
            this.curPlatform.hideAd(adId, destroy);
        }
        showBanner(data = null) {
            this.showAd(AdType.ad_1, null, data);
        }
        hideBanner(destroy = false) {
            this.hideInsert();
            this.hideAd(AdType.ad_1, destroy);
        }
        showScreenAd() {
            let time = 0;
            if (Laya.Browser.onQGMiniGame || Laya.Browser.onVVMiniGame || this.isMZ) {
                this.hideBanner();
            }
            if (time > 0) {
                fgui.GTween.kill(this);
                fgui.GTween.delayedCall(time).setTarget(this).onComplete(this.delayScreenAd, this);
            }
            else {
                this.delayScreenAd();
            }
        }
        delayScreenAd() {
            this.showAd(AdType.TYPE_Insert);
        }
        hideInsert() {
            if (this.curPlatform) {
                this.curPlatform.hideInsert();
            }
        }
        showBox(isMust = false) {
            this.showAd(AdType.TYPE_Box, null);
        }
        hideBox() {
            this.hideAd(AdType.TYPE_Box);
        }
        showBlock(data = null) {
            this.showAd(AdType.TYPE_Block, null, data);
        }
        hideBlock() {
            this.hideAd(AdType.TYPE_Block);
        }
        reportNativeShow(nativeData, nativeAd) {
            if (this.curPlatform) {
                this.curPlatform.reportNativeShow(nativeData, nativeAd);
            }
        }
        reportNativeClick(nativeData, nativeAd = null) {
            if (this.curPlatform) {
                this.curPlatform.reportNativeClick(nativeData, nativeAd);
            }
        }
        get nativeIconId() {
            return this.curPlatform ? this.curPlatform.nativeIconId : "";
        }
        setLoadingProgress(progress) {
            this.curPlatform.setLoadingProgress(progress);
        }
        subscribeAppMsg(handler = null) {
            if (this.curPlatform) {
                this.curPlatform.subscribeAppMsg(handler);
            }
        }
        get hasSubscribeApp() {
            return this.curPlatform.hasSubscribeApp;
        }
        vibrateShort() {
            this.curPlatform.vibrateShort();
        }
        GetVersionName() {
            return this.curPlatform.GetVersionName();
        }
        navigateTo(value) {
            this.curPlatform.navigateTo(value);
        }
        hasShortcutInstalled(handler) {
            this.curPlatform.hasShortcutInstalled(handler);
        }
        installShortcut(handler) {
            this.curPlatform.installShortcut(handler);
        }
        getStorage(key, handler) {
            this.curPlatform.getStorage(key, handler);
        }
        setStorage(key, value) {
            this.curPlatform.setStorage(key, value);
        }
        isShowNavigate() {
            return this.curPlatform.isShowNavigate();
        }
        isShowShare() {
            return this.curPlatform.isShowShare();
        }
        onShareAppMessage(handler) {
            this.curPlatform.onShareAppMessage(handler);
        }
        checkNetwork() {
            this.curPlatform.checkNetwork();
        }
        aldSendEvent(id, param = "") {
            this.curPlatform.aldSendEvent(id, param);
        }
        VisibleBaner(value) {
            this.curPlatform.VisibleBaner(value);
        }
        onAddToFavorites(handler = null) {
            this.curPlatform.onAddToFavorites(handler);
        }
        exitGame() {
            this.curPlatform && this.curPlatform.exitGame();
        }
        get isWX() {
            return Laya.Browser.onMiniGame && typeof (wx) != "undefined";
        }
        get isTT() {
            return Laya.Browser.onMiniGame && typeof (wx) == "undefined";
        }
        get isMZ() {
            return window["mz_jsb"] != null;
        }
        get isWeb() {
            return this.curPlatform instanceof NormalPlatform;
        }
        get isAndroidApp() {
            return window["conch"] != null;
        }
    }
    PlatformMgr._Inst = null;

    class UrlUtils {
        constructor() {
        }
        static IsPlatform() {
            return Laya.Browser.onMiniGame || Laya.Browser.onTTMiniGame || Laya.Browser.onHWMiniGame ||
                Laya.Browser.onQGMiniGame || Laya.Browser.onVVMiniGame || Laya.Browser.onQQMiniGame ||
                Laya.Browser.onBLMiniGame || Laya.Browser.onTBMiniGame || Laya.Browser.onBDMiniGame;
        }
        static GetIconBuildUrl(resName) {
            return PanelRegister.getUrl("icon_build", resName);
        }
        static GetIconPartUrl(resName) {
            return PanelRegister.getUrl("icon_part", resName);
        }
        static GetIconItemsUrl(resName) {
            return PanelRegister.getUrl("icon_items", resName);
        }
        static GetIconTaskUrl(resName) {
            return PanelRegister.getUrl("icon_task", resName);
        }
        static GetAstarUrl(resName) {
            return UrlUtils.ASTAR + resName + UrlUtils.resLm;
        }
        static GetCatHead(resName) {
            return UrlUtils.CAT_HEADS + resName + UrlUtils.resJpg;
        }
        static GetSkyUrl(resName) {
            return UrlUtils.SKY + resName + UrlUtils.resLmat;
        }
        static GetShaderPngUrl(resName) {
            return UrlUtils.SHADER + resName + UrlUtils.resPng;
        }
        static GetEntityLhUrl(resName) {
            return UrlUtils.PREFAB + resName + UrlUtils.resLh;
        }
        static GetSceneUrl(resName) {
            return UrlUtils.SCENE + "Conventional/" + resName + UrlUtils.resLs;
        }
        static GetMapUrl(resName) {
            return UrlUtils.MAP + resName + UrlUtils.resLh;
        }
        static GetBuildUrl(resName) {
            if (Laya.Browser.onKGMiniGame) {
                return "DJC" + UrlUtils.Build + "Conventional/" + resName + UrlUtils.resLh;
            }
            return UrlUtils.Build + "Conventional/" + resName + UrlUtils.resLh;
        }
        static GetCatUrl(resName) {
            return UrlUtils.Cats + resName + UrlUtils.resLh;
        }
        static GetCatAnimationUrl(resName) {
            return UrlUtils.CAT_ANIMATION + resName + "-" + resName + UrlUtils.resLani;
        }
        static GetLevelUrl(resName) {
            return UrlUtils.LEVEL + resName + UrlUtils.resLm;
        }
        static GetTemplateUrl(resName) {
            return UrlUtils.TEMPLATE + resName + UrlUtils.resLm;
        }
        static GetEffectUrl(resName) {
            return UrlUtils.EFFECT + resName + UrlUtils.resLh;
        }
        static GetSkinUrl(resName) {
            return UrlUtils.SKIN + resName + UrlUtils.resJpg;
        }
        static GetUiUlr(resName) {
            return [
                { url: resName + "_atlas0" + UrlUtils.resPng, type: Laya.Loader.IMAGE },
                { url: resName + UrlUtils.resLm, type: Laya.Loader.BUFFER }
            ];
        }
        static GetMusic(resName, isWav = false) {
            if (isWav) {
                return UrlUtils.MUSIC + resName + UrlUtils.resWav;
            }
            else {
                return UrlUtils.MUSIC + resName + UrlUtils.resMP3;
            }
        }
        static GetSpineUrl(resName) {
            let url = UrlUtils.SPINE + resName + "/" + resName + UrlUtils.resSk;
            return url;
        }
        static GetSpinePngUrl(resName) {
            let url = UrlUtils.SPINE + resName + "/" + resName + UrlUtils.resPng;
            return url;
        }
        static get PREFAB() {
            return UrlUtils.PREFAB_CONVENTIONAL;
        }
        static getFairyPkg(resName, pkg = UrlUtils.UI) {
            let url = pkg + resName;
            let newUrl = PlatformMgr.Inst.GetSubName(url);
            if (newUrl) {
                return newUrl;
            }
            return url;
        }
    }
    UrlUtils.RES = "res/";
    UrlUtils.PREFAB_IOS = "Entity/iOS/";
    UrlUtils.PREFAB_ADNROID = "Entity/Android/";
    UrlUtils.PREFAB_CONVENTIONAL = "Entity/Conventional/";
    UrlUtils.LIBARARY = "Assets/modle/";
    UrlUtils.SCENE = "Scene/";
    UrlUtils.MAP = "Scene/Conventional/";
    UrlUtils.Build = "Build/";
    UrlUtils.Cats = "Cats/Conventional/";
    UrlUtils.SKIN = UrlUtils.RES + "Skin/";
    UrlUtils.CAT_ANIMATION = UrlUtils.Cats + "Assets/Animation/Character/AnimationClip/";
    UrlUtils.SKY = UrlUtils.SCENE + "Assets/Model/_res/mat/sky/";
    UrlUtils.UI = "uisub/";
    UrlUtils.TEMPLATE = UrlUtils.UI + "template/";
    UrlUtils.UI_COM = UrlUtils.RES + "uicom/";
    UrlUtils.ICON = UrlUtils.UI + "icon/";
    UrlUtils.LEVEL = UrlUtils.UI + "level/";
    UrlUtils.SPINE = UrlUtils.UI + "spine/";
    UrlUtils.UI_MAP = "uimap/";
    UrlUtils.EFFECT = "Effect/Conventional/";
    UrlUtils.MUSIC = "music/";
    UrlUtils.ASTAR = UrlUtils.RES + "astar/";
    UrlUtils.SHADER = UrlUtils.RES + "shader/";
    UrlUtils.ICON_BUILD = UrlUtils.ICON + "build/";
    UrlUtils.ICON_ITEM = UrlUtils.ICON + "item/";
    UrlUtils.CAT_HEADS = "CatHeads/";
    UrlUtils.resPng = ".png";
    UrlUtils.resJpg = ".jpg";
    UrlUtils.resMP3 = ".mp3";
    UrlUtils.resWav = ".wav";
    UrlUtils.resJson = ".json";
    UrlUtils.resLm = ".lm";
    UrlUtils.resLh = ".lh";
    UrlUtils.resLs = ".ls";
    UrlUtils.resSk = ".sk";
    UrlUtils.resLmat = ".lmat";
    UrlUtils.resLani = ".lani";

    class TSpineButton extends TBaseButton {
        constructor() {
            super();
            this.spineName = null;
            this.iRes = null;
            this.skeleton = null;
            this.isLoop = true;
            this.isOverRemove = false;
            this.handler = null;
            this.touchable = true;
        }
        InitUI() {
            super.InitUI();
            if (this._iconObject) {
                this._iconObject.visible = false;
            }
        }
        play(spineName, x = NaN, y = NaN, isLoop = true, isOverRemove = false, handler = null, aniName = null) {
            let sl = this;
            if (sl.spineName == spineName) {
                return;
            }
            sl.Reset();
            sl.visible = true;
            sl.spineName = spineName;
            sl.isLoop = isLoop;
            sl.isOverRemove = isOverRemove;
            sl.handler = handler;
            let url = UrlUtils.GetSpineUrl(spineName);
            LoadQueue.Inst.loadSpine(url, Laya.Handler.create(this, this.onComplete, [spineName, aniName, x, y]));
        }
        onComplete(spineName, aniName, x, y, iRes) {
            this.iRes = iRes;
            this.skeleton = iRes.GetRes().buildArmature(0);
            if (!this.skeleton || this.skeleton.destroyed) {
                console.error("spine资源已释放!!! spineName: " + spineName + " aniName: " + aniName);
                return;
            }
            if (!(this.displayObject && !this.displayObject.destroyed)) {
                return;
            }
            this.displayObject.addChildAt(this.skeleton, 0);
            x = isNaN(x) ? this.width * 0.5 : x;
            y = isNaN(y) ? this.height * 0.5 : y;
            this.skeleton.pos(x, y);
            if (!this.isLoop)
                this.skeleton.once(Laya.Event.STOPPED, this, this.completeHandler);
            this.skeleton.play(aniName == null ? 0 : aniName, this.isLoop);
        }
        PlayAni(aniName, isLoop = true, handler = null) {
            this.handler = handler;
            if (this.skeleton) {
                this.skeleton.play(aniName, isLoop);
                if (!isLoop) {
                    this.skeleton.once(Laya.Event.STOPPED, this, this.completeHandler);
                }
            }
        }
        completeHandler() {
            if (this.handler != null)
                this.handler.run();
            if (this.isOverRemove)
                this.Reset();
        }
        get isShow() {
            return this.visible && this.skeleton != null;
        }
        Reset() {
            super.Reset();
            this.spineName = null;
            this.isLoop = true;
            this.isOverRemove = false;
            this.handler = null;
            this.visible = false;
            if (this.skeleton != null) {
                this.skeleton.removeSelf();
                this.skeleton.off(Laya.Event.STOPPED, this, this.completeHandler, false);
                this.skeleton = null;
            }
            if (this.iRes != null) {
                this.iRes.Dispose();
                this.iRes = null;
            }
            this.displayObject && this.displayObject.removeChildren();
        }
    }

    class PreloadingResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.resName = byteArray.readUTFString();
            this.type = byteArray.readUint8();
        }
        static getItemByKey(id) {
            return PreloadingResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return PreloadingResource.itemsDic[key1 + "," + key2];
        }
    }
    PreloadingResource.items = new Array();
    PreloadingResource.itemsDic = new Dictionary();

    class Scene3DLightResource {
        constructor() {
        }
        parseItem(byteArray) {
            this.id = byteArray.readUint8();
            this.type = byteArray.readUint8();
            this.position = ConfigLib.ReadArray(byteArray);
            this.rotation = ConfigLib.ReadArray(byteArray);
            this.intensity = byteArray.readFloat64();
            this.color = ConfigLib.ReadArray(byteArray);
            this.range = byteArray.readFloat64();
        }
        static getItemByKey(id) {
            return Scene3DLightResource.itemsDic[id];
        }
        static getItemBy2Keys(key1, key2) {
            return Scene3DLightResource.itemsDic[key1 + "," + key2];
        }
    }
    Scene3DLightResource.items = new Array();
    Scene3DLightResource.itemsDic = new Dictionary();

    class InitTpl {
        constructor() {
        }
        static start() {
            InitTpl.TplMaping = [
                ["I18nResource", I18nResource, ["id"]],
                ["ConfigValue", ConfigValue, ["key"]],
                ["AdResource", AdResource, ["platform", "id"]],
                ["EventResource", EventResource, ["id"]],
                ["bulidResource", bulidResource, ["id"]],
                ["bulidupResource", bulidupResource, ["buildId", "stage"]],
                ["bulidSeatResource", bulidSeatResource, ["buildId", "lv"]],
                ["bulidEffResource", bulidEffResource, ["id"]],
                ["itemResource", itemResource, ["id"]],
                ["islandResource", islandResource, ["id"]],
                ["baseResource", baseResource, ["lvl"]],
                ["catResource", catResource, ["id"]],
                ["catlangResource", catlangResource, ["id"]],
                ["catActResource", catActResource, ["id"]],
                ["catUpResource", catUpResource, ["lvl"]],
                ["catloveResource", catloveResource, ["quality", "lvl"]],
                ["skinResource", skinResource, ["id"]],
                ["suitResource", suitResource, ["suitId"]],
                ["changeResource", changeResource, ["id"]],
                ["taskResource", taskResource, ["id"]],
                ["principalResource", principalResource, ["id"]],
                ["EffecResource", EffecResource, ["id"]],
                ["CatEventResource", CatEventResource, ["eventId"]],
                ["ActionResource", ActionResource, ["id"]],
                ["GiftResource", GiftResource, ["id"]],
                ["shopResource", shopResource, ["id"]],
                ["GuideResource", GuideResource, ["id"]],
                ["LoginResource", LoginResource, ["id"]],
                ["packageResource", packageResource, ["id"]],
                ["GrowUpResource", GrowUpResource, ["id"]],
                ["PreloadingResource", PreloadingResource, ["resName"]],
                ["Scene3DLightResource", Scene3DLightResource, ["id"]],
                ["PhoneDeviceResource", PhoneDeviceResource, ["brand", "model"]],
                ["MaowuUpResource", MaowuUpResource, ["stage"]],
                ["WarOrderResource", WarOrderResource, ["Grade"]],
                ["EventtaskResource", EventtaskResource, ["id"]],
                ["clothingdrawResource", clothingdrawResource, ["ID"]],
                ["propdrawResource", propdrawResource, ["ID"]],
                ["actResource", actResource, ["id"]],
            ];
        }
        static InitTpls(bytearray) {
            if (InitTpl.TplMaping == null) {
                InitTpl.start();
            }
            if (InitTpl.dic == null) {
                InitTpl.dic = new Dictionary();
                for (var i = 0; i < InitTpl.TplMaping.length; i++) {
                    var data = InitTpl.TplMaping[i];
                    InitTpl.dic[data[0]] = new MappingVO(data[1], data[2], data[3]);
                }
            }
            bytearray.endian = Laya.Byte.LITTLE_ENDIAN;
            while (bytearray.bytesAvailable) {
                var tplName = bytearray.readUTFString();
                var mapVO = InitTpl.dic[tplName];
                var len = bytearray.readUint32();
                var end = bytearray.pos + len;
                if (mapVO == null) {
                    bytearray.pos += len;
                    Logger.warning(this, "模板 : " + tplName + " 未解析！");
                }
                else {
                    Logger.warning(this, "解析模板 : " + tplName + "");
                    var cls = mapVO.tplCls;
                    while (bytearray.pos < end) {
                        var tplObj = new cls();
                        tplObj.parseItem(bytearray);
                        cls.items.push(tplObj);
                        var hashKey;
                        if (mapVO.idprop.length == 1) {
                            hashKey = tplObj[mapVO.idprop[0]];
                        }
                        else if (mapVO.idprop.length == 2) {
                            if (mapVO.hashFunction == null) {
                                hashKey = tplObj[mapVO.idprop[0]] + "," + tplObj[mapVO.idprop[1]];
                            }
                            else {
                                hashKey = mapVO.hashFunction(tplObj[mapVO.idprop[0]], tplObj[mapVO.idprop[1]]);
                            }
                        }
                        else if (mapVO.idprop.length == 3) {
                            hashKey = mapVO.hashFunction(tplObj[mapVO.idprop[0]], tplObj[mapVO.idprop[1]], tplObj[mapVO.idprop[2]]);
                        }
                        else {
                            throw new Error("暂时不支持!");
                        }
                        if (hashKey in cls.itemsDic) {
                        }
                        else {
                            cls.itemsDic[hashKey] = tplObj;
                        }
                    }
                }
            }
        }
    }
    class MappingVO {
        constructor(tplCls, id, hashFunction) {
            this.hashFunction = hashFunction;
            this.idprop = id;
            this.tplCls = tplCls;
        }
    }

    class MoneyCtrl {
        constructor() {
        }
        static get Inst() {
            return this.inst = this.inst || new MoneyCtrl();
        }
        init() {
            MoneyModel.Inst.init();
        }
    }
    MoneyCtrl.inst = null;

    class SceneCtrl {
        constructor() {
            this.imgBgCount = 0;
        }
        static get Inst() {
            if (this._Inst == null)
                this._Inst = new SceneCtrl();
            return this._Inst;
        }
        init() {
            GameDispatcher.Inst.on(UIGameEvent.UI_SHOW_IMGBG, this, this.onShowImgBg);
            GameDispatcher.Inst.on(UIGameEvent.UI_HIDE_IMGBG, this, this.onHideImgBg);
        }
        onShowImgBg() {
            this.imgBgCount++;
            let camera = SceneMgr.Inst.GetCamera();
            if (camera && camera.active) {
                camera.active = false;
            }
        }
        onHideImgBg() {
            this.imgBgCount--;
            if (this.imgBgCount <= 0) {
                this.imgBgCount = 0;
                let camera = SceneMgr.Inst.GetCamera();
                if (camera && !camera.active) {
                    camera.active = true;
                }
            }
        }
    }
    SceneCtrl._Inst = null;

    class TimeCtrl {
        constructor() {
        }
        static get Inst() {
            return this.inst = this.inst || new TimeCtrl();
        }
        init() {
            let self = this;
            TimeModel.Inst.init();
            Laya.timer.loop(1000, this, () => {
                let todayFirstTime = StorageUtils.Inst.todayFirstTime;
                if (!TimeModel.judgeIsOneDay(todayFirstTime)) {
                    self.setLoginTime();
                    GameDispatcher.Inst.event(UIGameEvent.SKIP_DAY);
                }
            });
        }
        setLoginTime() {
            StorageUtils.Inst.setLoginTime();
        }
    }
    TimeCtrl.inst = null;

    class UILoadingView extends UIBaseWindow {
        constructor() {
            super("loading", "UILoadingView", UrlUtils.UI_COM);
            this.sp_pro = null;
            this.initHandler = null;
            this.completeHandler = null;
            this.failHandler = null;
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_TOP;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isExclusion = false;
        }
        showLoadingLeaf() {
        }
        registerClasses() {
            super.registerClasses();
            PanelRegister.registerClass("loading", "SpineButton", TSpineButton, UrlUtils.UI_COM);
        }
        InitUI() {
            this.c1 = this._view.getController("c1");
            this.c2 = this._view.getController("c2");
            this.sp_pro.max = 100;
            this.sp_pro.min = 0;
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.LOGIN_FAIL, this.loginFail, this);
            this.AddGameListener(UIGameEvent.LOAD_SCENE_COMPLETE, this.loadSceneComplete, this);
        }
        InitData(param) {
            super.InitData(param);
            this.spine_loading.play("loading");
            this.sp_pro.value = 0.01;
            PlatformMgr.Inst.setLoadingProgress(0);
            this.txtUpdate();
            this.initHandler = param[0];
            this.completeHandler = param[1];
            LoadQueue.Inst.clearRate();
            Laya.timer.frameOnce(5, this, this.onDelayInit);
            this.hideBanner();
            if (Laya.Browser.onHWMiniGame) {
                this.c1.selectedPage = "hw";
            }
        }
        loginFail(handler = null) {
            console.log("登陆失败!");
            this.failHandler = handler;
            this.c2.selectedPage = "hw";
        }
        onDelayInit() {
            if (this.initHandler)
                this.initHandler.run();
            Laya.timer.frameLoop(1, this, this.onUpdate);
        }
        onUpdate() {
            if (this.sp_pro.value == 0)
                this.sp_pro.value = 0.01;
            let rate = LoadQueue.Inst.GetRate();
            if (rate == 0)
                rate = 0.01;
            Logger.log(this, "loading...	" + rate);
            if (rate >= 1) {
                Laya.timer.clearAll(this);
            }
            if (rate <= this.sp_pro.value * 0.01)
                return;
            this.sp_pro.tweenValue(rate * 100, 0.2)
                .onUpdate(this.txtUpdate, this)
                .onComplete(this.txtComplet, this);
        }
        txtUpdate() {
            let rate = Math.ceil(this.sp_pro.value);
            PlatformMgr.Inst.setLoadingProgress(rate);
        }
        txtComplet() {
            let lf = this;
            if (Math.ceil(lf.sp_pro.value) >= 100 && GlobalConfig.isLoadScene) {
                PlatformMgr.Inst.setLoadingProgress(100);
                Laya.timer.clearAll(lf);
                Laya.timer.frameOnce(3, lf, function () {
                    Logger.debug(lf, "completeHandler： " + lf.completeHandler);
                    if (lf.completeHandler != null) {
                        lf.completeHandler.run();
                    }
                });
            }
        }
        loadSceneComplete() {
            this.txtComplet();
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_start) {
                this.c1.selectedIndex = 0;
                PlatformMgr.Inst.login("", "", this.failHandler);
            }
        }
        onResize() {
            super.onResize();
        }
        Reset() {
            super.Reset();
            this.sp_pro.value = 0.01;
            Laya.timer.clearAll(this);
            this.initHandler = null;
            this.completeHandler = null;
        }
    }

    class UIAdAgreeView extends UIBaseWindow {
        constructor() {
            super("ad", "UIAdAgreeView");
        }
        registerClasses() {
            super.registerClasses();
        }
        SetViewStruct() {
            super.SetViewStruct();
            this.viewStruct.layerType = LayerType.TYPE_AD;
            this.viewStruct.layout = LayerType.LAYOUT_LEFT_TOP;
            this.viewStruct.isShowMaskBg = true;
        }
        InitUI() {
            super.InitUI();
        }
        AddListener() {
            super.AddListener();
        }
        InitData(data) {
            super.InitData(data);
        }
        onBtnClick(btn) {
            super.onBtnClick(btn);
            if (btn == this.btn_ok) {
                StorageUtils.Inst.isAgreeHWAd = true;
                UISystem.Inst.RemoveWindowClass(UILoadingView);
                let handler = this.Data && this.Data.handler;
                if (handler) {
                    handler.run();
                }
                this.close();
            }
        }
        onBtnCloseClick() {
            super.onBtnCloseClick();
            this.closeHandler();
        }
        closeHandler() {
            PlatformMgr.Inst.exitGame();
        }
        reshow() {
            this.visible = true;
        }
        Reset() {
            super.Reset();
            Laya.timer.clear(this, this.reshow);
        }
    }

    class UICameraRotateScript extends TComponent {
        constructor() {
            super(...arguments);
            this.touchPosArr = null;
            this.dic = new Dictionary();
            this._downPos = new Laya.Point();
            this._ray = new Laya.Ray(new Laya.Vector3(0, 0, 0), new Laya.Vector3(0, 0, 0));
            this.pos = new Laya.Vector2();
            this.outs = new Array();
        }
        InitUI() {
            this.touchPosArr = [new TouchPointID(), new TouchPointID()];
            Laya.MouseManager.multiTouchEnabled = true;
        }
        AddListener() {
            super.AddListener();
            this.on(Laya.Event.MOUSE_DOWN, this, this.onTouchDown);
            this.on(Laya.Event.MOUSE_MOVE, this, this.OnTouchMove);
            this.on(Laya.Event.MOUSE_UP, this, this.OnTouchUp);
            this.on(Laya.Event.MOUSE_OUT, this, this.OnTouchUp);
            this.onClick(this, this.onTouchClick);
            this.on(Laya.Event.MOUSE_WHEEL, this, this.OnMouseWheel);
        }
        RemoveListener() {
            super.RemoveListener();
            this.off(Laya.Event.MOUSE_DOWN, this, this.onTouchDown);
            this.off(Laya.Event.MOUSE_MOVE, this, this.OnTouchMove);
            this.off(Laya.Event.MOUSE_UP, this, this.OnTouchUp);
            this.off(Laya.Event.MOUSE_OUT, this, this.OnTouchUp);
            this.offClick(this, this.onTouchClick);
            this.off(Laya.Event.MOUSE_WHEEL, this, this.OnMouseWheel);
        }
        OnMouseWheel(e) {
            let camera = SceneMgr.Inst.GetCameraBuildScript();
            camera.OnChangeScale(e.delta * 5);
        }
        onTouchDown(evt) {
            if (!evt || evt.touchId == -1)
                return;
            let touchPos = this.dic[evt.touchId];
            if (!touchPos) {
                touchPos = this.touchPosArr.pop();
                this.dic[evt.touchId] = touchPos;
            }
            if (touchPos) {
                touchPos.touchDownX = Laya.stage.mouseX;
                touchPos.touchDownY = Laya.stage.mouseY;
                touchPos.touchMoveX = Laya.stage.mouseX;
                touchPos.touchMoveY = Laya.stage.mouseY;
                this._downPos.setTo(touchPos.touchDownX, touchPos.touchDownY);
            }
        }
        OnTouchUp(evt) {
            if (evt.touchId == -1)
                return;
            let touchPos = this.dic[evt.touchId];
            if (touchPos) {
                this.touchPosArr.push(touchPos);
                delete this.dic[evt.touchId];
            }
        }
        GetTouchLength() {
            let length = 0;
            for (const key in this.dic) {
                length++;
            }
            return length;
        }
        OnTouchMove(evt) {
            let touchPos = this.dic[evt.touchId];
            if (!touchPos) {
                return;
            }
            touchPos.touchMoveX = Laya.stage.mouseX;
            touchPos.touchMoveY = Laya.stage.mouseY;
            let camera = SceneMgr.Inst.GetCameraBuildScript();
            if (!camera)
                return;
            if (this.GetTouchLength() < 2) {
                let offestX = touchPos.touchMoveX - touchPos.touchDownX;
                let offestY = touchPos.touchMoveY - touchPos.touchDownY;
                camera.ChangeRotationEuler(-offestX, offestY);
            }
            else {
                let onTouch = null;
                for (const key in this.dic) {
                    let curTouchPos = this.dic[key];
                    if (!curTouchPos)
                        continue;
                    if (onTouch == null) {
                        onTouch = curTouchPos;
                    }
                    else {
                        let downDis = MathUtil.distance(curTouchPos.touchDownX, curTouchPos.touchDownY, onTouch.touchDownX, onTouch.touchDownY);
                        let moveDis = MathUtil.distance(curTouchPos.touchMoveX, curTouchPos.touchMoveY, onTouch.touchMoveX, onTouch.touchMoveY);
                        camera.OnChangeScale(downDis - moveDis);
                    }
                }
            }
            touchPos.touchDownX = Laya.stage.mouseX;
            touchPos.touchDownY = Laya.stage.mouseY;
        }
        onTouchClick() {
            let distance = MathUtil.distance(this._downPos.x, this._downPos.y, Laya.stage.mouseX, Laya.stage.mouseY);
            if (distance > 20)
                return;
            this.pos.setValue(Laya.stage.mouseX * Laya.stage.clientScaleX, Laya.stage.mouseY * Laya.stage.clientScaleY);
            let camera = SceneMgr.Inst.GetCamera();
            if (!camera)
                return;
            camera && camera.viewportPointToRay(this.pos, this._ray);
            let isTrigger = SceneMgr.Inst.GetScene().physicsSimulation.rayCastAll(this._ray, this.outs, 1000, this.GetCollisonGroup(), this.GetCanCollideWith());
            if (!isTrigger)
                return;
            let fecalScript = this.GetHasCollision(this.outs, CatFecalBallScript);
            if (fecalScript) {
                fecalScript.onClick();
                return;
            }
            let giftScript = this.GetHasCollision(this.outs, CatGiftScript);
            if (giftScript) {
                giftScript.onClick();
                return;
            }
        }
        GetHasCollision(outs, cls) {
            for (const out of outs) {
                if (out.succeeded) {
                    let sc = out.collider.owner.getComponent(cls);
                    if (sc != null)
                        return sc;
                }
            }
            return null;
        }
        GetCollisonGroup() {
            return CollisionType.TYPE_RAY_CLICK;
        }
        GetCanCollideWith() {
            return CollisionType.TYPE_ENTITY;
        }
    }

    class UICameraStrokeScript extends TComponent {
        constructor() {
            super(...arguments);
            this.touchPosArr = null;
            this.dic = new Dictionary();
            this._downPos = new Laya.Point();
            this.touchDownTime = 0;
            this._ray = new Laya.Ray(new Laya.Vector3(0, 0, 0), new Laya.Vector3(0, 0, 0));
            this.pos = new Laya.Vector2();
            this.outs = new Array();
            this.catScript = null;
        }
        InitUI() {
            this.touchPosArr = [new TouchPointID(), new TouchPointID()];
            Laya.MouseManager.multiTouchEnabled = true;
        }
        AddListener() {
            this.on(Laya.Event.MOUSE_DOWN, this, this.onTouchDown);
            this.on(Laya.Event.MOUSE_OUT, this, this.OnTouchUp);
            this.onClick(this, this.onTouchClick);
            this.on(Laya.Event.MOUSE_WHEEL, this, this.OnMouseWheel);
        }
        RemoveListener() {
            this.off(Laya.Event.MOUSE_DOWN, this, this.onTouchDown);
            this.off(Laya.Event.MOUSE_MOVE, this, this.OnTouchMove);
            this.off(Laya.Event.MOUSE_UP, this, this.OnTouchUp);
            this.off(Laya.Event.MOUSE_OUT, this, this.OnTouchUp);
            this.offClick(this, this.onTouchClick);
            this.off(Laya.Event.MOUSE_WHEEL, this, this.OnMouseWheel);
        }
        OnMouseWheel(e) {
            let camera = SceneMgr.Inst.GetCameraBuildScript();
            camera.OnChangeScale(e.delta * 5);
        }
        onTouchDown(evt) {
            if (!evt || evt.touchId == -1)
                return;
            let touchPos = this.dic[evt.touchId];
            if (!touchPos) {
                touchPos = this.touchPosArr.pop();
                this.dic[evt.touchId] = touchPos;
            }
            if (touchPos) {
                touchPos.touchDownX = Laya.stage.mouseX;
                touchPos.touchDownY = Laya.stage.mouseY;
                touchPos.touchMoveX = Laya.stage.mouseX;
                touchPos.touchMoveY = Laya.stage.mouseY;
                this._downPos.setTo(touchPos.touchDownX, touchPos.touchDownY);
            }
            this.touchDownTime = Laya.timer.currTimer;
            this.off(Laya.Event.MOUSE_MOVE, this, this.OnTouchMove);
            this.off(Laya.Event.MOUSE_UP, this, this.OnTouchUp);
            this.on(Laya.Event.MOUSE_MOVE, this, this.OnTouchMove);
            this.on(Laya.Event.MOUSE_UP, this, this.OnTouchUp);
        }
        GetTouchLength() {
            let length = 0;
            for (const key in this.dic) {
                length++;
            }
            return length;
        }
        OnTouchMove(evt) {
            let touchPos = this.dic[evt.touchId];
            if (!touchPos) {
                return;
            }
            touchPos.touchMoveX = Laya.stage.mouseX;
            touchPos.touchMoveY = Laya.stage.mouseY;
            let camera = SceneMgr.Inst.GetCameraBuildScript();
            if (!camera)
                return;
            if (this.GetTouchLength() < 2) {
                Logger.log(this, "=====>OnTouchMove");
                let offestX = touchPos.touchMoveX - touchPos.touchDownX;
                let offestY = touchPos.touchMoveY - touchPos.touchDownY;
                camera.ChangeRotationEuler(-offestX, offestY);
                let window = this.parent.parent;
                if (window instanceof UICatDetailsView) {
                    window.updateHandXY();
                }
            }
            else {
                let onTouch = null;
                for (const key in this.dic) {
                    let curTouchPos = this.dic[key];
                    if (!curTouchPos)
                        continue;
                    if (onTouch == null) {
                        onTouch = curTouchPos;
                    }
                    else {
                        let downDis = MathUtil.distance(curTouchPos.touchDownX, curTouchPos.touchDownY, onTouch.touchDownX, onTouch.touchDownY);
                        let moveDis = MathUtil.distance(curTouchPos.touchMoveX, curTouchPos.touchMoveY, onTouch.touchMoveX, onTouch.touchMoveY);
                        camera.OnChangeScale(downDis - moveDis);
                    }
                }
            }
            touchPos.touchDownX = Laya.stage.mouseX;
            touchPos.touchDownY = Laya.stage.mouseY;
        }
        OnTouchUp(evt) {
            this.off(Laya.Event.MOUSE_MOVE, this, this.OnTouchMove);
            this.off(Laya.Event.MOUSE_UP, this, this.OnTouchUp);
            if (evt.touchId == -1)
                return;
            let touchPos = this.dic[evt.touchId];
            if (touchPos) {
                this.touchPosArr.push(touchPos);
                delete this.dic[evt.touchId];
            }
            Logger.log(this, "=====>OnTouchUp touchId: " + evt.touchId + " " + evt.stageX + "," + evt.stageY);
        }
        onTouchClick() {
            let distance = MathUtil.distance(this._downPos.x, this._downPos.y, Laya.stage.mouseX, Laya.stage.mouseY);
            if (distance > 20)
                return;
            this.pos.setValue(Laya.stage.mouseX * Laya.stage.clientScaleX, Laya.stage.mouseY * Laya.stage.clientScaleY);
            let camera = SceneMgr.Inst.GetCamera();
            camera.viewportPointToRay(this.pos, this._ray);
            let isTrigger = SceneMgr.Inst.GetScene().physicsSimulation.rayCastAll(this._ray, this.outs, 1000, this.GetCollisonGroup(), this.GetCanCollideWith());
            if (!isTrigger)
                return;
            let fecalScript = this.GetHasCollision(this.outs, CatFecalBallScript);
            if (fecalScript) {
                fecalScript.onClick();
                return;
            }
            let giftScript = this.GetHasCollision(this.outs, CatGiftScript);
            if (giftScript) {
                giftScript.onClick();
                return;
            }
            let window = this.parent.parent;
            let curCatVo = window && window.Data;
            if (!curCatVo) {
                return;
            }
            if ((curCatVo.catStrokeVo.IsReady() || curCatVo.catStrokeVo.IsAnger()) && (Laya.timer.currTimer - this.touchDownTime) > 200) {
                return;
            }
            let preClickCatScript = this.catScript;
            this.catScript = this.GetHasCollision(this.outs, CatScript);
            if (this.catScript != null && this.catScript.vo instanceof CatVO) {
                if (curCatVo instanceof CatVO && curCatVo != this.catScript.vo) {
                    if (curCatVo.catStrokeVo.IsReady()) {
                        this.catScript = preClickCatScript;
                        return;
                    }
                    else {
                        CatCtrl.Inst.StopLumao(curCatVo);
                        window.InitData(this.catScript.vo);
                        return;
                    }
                }
                else {
                    if (curCatVo) {
                        if (window instanceof UICatDetailsView || window instanceof UICatInviteAlert) {
                            if (curCatVo.catStrokeVo.IsReady()) {
                                window.readyToStroke();
                            }
                            if (this.catScript.vo.catStrokeVo.IsStroked()) {
                                window.showTip("刚刚已摸过啦。", true);
                            }
                            else if (this.catScript.vo.catStrokeVo.IsBusy()) {
                                window.showTip("现在好像有些忙。", true);
                            }
                            else if (curCatVo.catStrokeVo.IsRefuse()) {
                                window.showTip("现在不太喜欢", true);
                            }
                        }
                    }
                }
                CatCtrl.Inst.ClickCat(this.catScript.vo);
            }
        }
        GetCatScreenPos() {
            if (this.catScript != null) {
                let camera = SceneMgr.Inst.GetCamera();
                let target = this.catScript.owner;
                return MathUtil.GetPointFrom3DWorld(camera, target.transform.position);
            }
            return null;
        }
        GetHasCollision(outs, cls) {
            for (const out of outs) {
                if (out.succeeded) {
                    let sc = out.collider.owner.getComponent(cls);
                    if (sc != null)
                        return sc;
                }
            }
            return null;
        }
        GetCollisonGroup() {
            return CollisionType.TYPE_RAY_CLICK;
        }
        GetCanCollideWith() {
            return CollisionType.TYPE_CAT | CollisionType.TYPE_ENTITY;
        }
        Reset() {
            this.catScript = null;
        }
    }

    class UIGameClubButton extends TListButtonItem {
        constructor() {
            super();
            this.tapHandler = null;
        }
        InitUI() {
            super.InitUI();
            this._iconObject.visible = false;
        }
        InitData(data) {
            super.InitData(data);
            if (PlatformMgr.Inst.isWX) {
                this.showGameClub(this.data);
            }
        }
        showGameClub(data = null, tapHandler = null) {
            this.tapHandler = tapHandler;
            let icon = "green";
            if (data && data.hasOwnProperty("icon")) {
                icon = data.icon;
            }
            this.tapHandler = data && data.hasOwnProperty("handler") ? data.handler : null;
            let scaleX = PlatformModel.Inst.scaleX;
            let scaleY = PlatformModel.Inst.scaleY;
            let style = { "width": Math.ceil(this.width * scaleX), "height": Math.ceil(this.height * scaleY) };
            let x = data && data.hasOwnProperty("x") ? data.x : this.x;
            let y = data && data.hasOwnProperty("y") ? data.x : this.y;
            let left = Math.ceil(x * scaleX);
            let top = Math.ceil(y * scaleY);
            if (data && data.hasOwnProperty("width")) {
                style.width = Math.ceil(data.width);
            }
            if (data && data.hasOwnProperty("height")) {
                style.height = Math.ceil(data.height);
            }
            if (data && data.hasOwnProperty("left")) {
                style.left = data.left;
            }
            else {
                let w = Math.ceil(style.width * 0.5);
                style.left = left - w;
            }
            if (data && data.hasOwnProperty("top")) {
                style.top = data.top;
            }
            else {
                let h = Math.ceil(style.height * 0.5);
                style.top = top - h;
            }
            if (UIGameClubButton.clubBtn == null) {
                let func = window["wx"]["createGameClubButton"];
                UIGameClubButton.clubBtn = func ? func({ "icon": icon, "style": style }) : null;
            }
            UIGameClubButton.clubBtn.style = style;
            UIGameClubButton.clubBtn.show();
            UIGameClubButton.clubBtn.onTap(this.tapFunc);
        }
        tapFunc() {
            this.tapHandler && this.tapHandler.run();
        }
        hideGameClub() {
            UIGameClubButton.clubBtn && UIGameClubButton.clubBtn.offTap(this.tapFunc);
            UIGameClubButton.clubBtn && UIGameClubButton.clubBtn.hide();
            UIGameClubButton.clubBtn = null;
            this.tapHandler = null;
        }
        onResize() {
        }
        Reset() {
            super.Reset();
            this.hideGameClub();
        }
    }
    UIGameClubButton.clubBtn = null;

    class UINativeMuiltGrid extends TListButtonItem {
        constructor() {
            super();
            this.adId = 0;
        }
        AddListener() {
            super.AddListener();
            this.AddGameListener(UIGameEvent.CLOSE_AD, this.onCloseAd, this);
        }
        RemoveListener() {
            super.RemoveListener();
        }
        InitUI() {
            super.InitUI();
        }
        InitData(data) {
            this.Reset();
            super.InitData(data);
            data = data || {};
            this.adId = data.adId || 0;
            if (this.adId > 0) {
                this.visible = false;
                if (PlatformMgr.Inst.isWX) {
                    this.reshow30();
                }
            }
            else {
                this.Reset();
            }
        }
        onCloseAd(res) {
            if (res && res.id == this.adId) {
                Laya.timer.once(30000, this, this.reshow30);
            }
        }
        reshow30() {
            if (PlatformMgr.Inst.isWX) {
                let windowWidth = PlatformModel.Inst.windowWidth;
                let scaleY = PlatformModel.Inst.scaleY;
                let left = Math.ceil((windowWidth - 360) * 0.5);
                let top = Math.ceil(this.y * scaleY);
                PlatformMgr.Inst.showAd(this.adId, null, { "left": left, "top": top });
            }
        }
        onResize() {
        }
        Reset() {
            super.Reset();
            if (this.adId > 0) {
                PlatformMgr.Inst.hideAd(this.adId, true);
            }
            this.adId = 0;
            Laya.timer.clear(this, this.reshow30);
            fgui.GTween.kill(this, true);
            PlatformMgr.Inst.hideBlock();
        }
    }

    class UIBagItem extends TListButtonItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
            this.c1 = this.getController("c1");
            this.c2 = this.getController("c2");
            this.c3 = this.getController("c3");
            this.c4 = this.getController("c4");
            this.c5 = this.getController("c5");
            this.c6 = this.getController("c6");
        }
        InitData(data) {
            super.InitData(data);
            this.img_owned.visible = false;
            this.selected = false;
            this.isLock = false;
            this.isSuit = false;
            if (data instanceof ItemVO) {
                this.part = 0;
                this.setItemIcon(data.itemUrl);
                this.setCatIcon(data.catHeadUrl);
                this.isShowCat = GlobalConfig.isShowCat;
                this.quality = data.itemRes.quality;
                this.txt_count.text = "x" + data.count;
                this.txt_name.text = data.itemRes.name;
            }
            else if (data instanceof SkinVO) {
                this.setItemIcon(data.iconUrl);
                this.quality = data.itemRes.quality;
                this.txt_name.text = data.itemRes.name;
                this.isShowCat = false;
                this.txt_count.text = "x" + data.count;
                if (data.hasOwned) {
                    this.part = 0;
                }
                else {
                    this.part = data.part;
                }
                this.isSuit = data.isSuit;
            }
            else if (data instanceof itemResource) {
                this.isShowCat = false;
                this.quality = data.quality;
                this.txt_name.text = data.name;
                this.part = 0;
                this.setItemIcon(ItemVO.GetItemUrl(data));
            }
            else if (data instanceof skinResource) {
                this.setItemIcon(SkinVO.GetSkinUrl(data));
                this.quality = data.quality;
                this.txt_name.text = data.name;
                this.isShowCat = false;
                this.txt_count.text = "";
                this.part = 0;
                let hasOwned = StorageUtils.Inst.HasSkin(data.id);
                this.isLock = !hasOwned;
                this.isSuit = data.suitId > 0;
            }
        }
        setItemIcon(url) {
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                this.img_icon.url = url;
            }));
        }
        setCatIcon(url) {
            PanelRegister.loadUrl(url, Laya.Handler.create(this, (url) => {
                this.img_cat.icon = url;
            }));
        }
        set part(value) {
            this.c3.selectedIndex = value;
        }
        get part() {
            return this.c3.selectedIndex;
        }
        set isShowCat(value) {
            if (this.Data instanceof ItemVO && this.Data.catRes != null) {
                this.c2.selectedIndex = value ? 1 : 0;
                let catRes = ItemModel.Inst.getCatRes(this.Data.itemId);
                let catVo = catRes && CatModel.Inst.GetCatVOById(catRes.id);
                this.img_owned.visible = catVo && catVo.IsHas();
            }
            else {
                this.img_owned.visible = false;
                this.c2.selectedIndex = 0;
            }
        }
        get isShowCat() {
            return this.c2.selectedIndex == 1;
        }
        set quality(value) {
            this.c1.selectedIndex = value;
        }
        get quality() {
            return this.c1.selectedIndex;
        }
        set hasOwned(value) {
            this.c4.selectedIndex = value ? 1 : 0;
        }
        get hasOwned() {
            return this.c4.selectedIndex == 1;
        }
        set isLock(value) {
            this.c5.selectedIndex = value ? 1 : 0;
        }
        get isLock() {
            return this.c5.selectedIndex == 1;
        }
        set isSuit(value) {
            this.c6.selectedIndex = value ? 1 : 0;
        }
        get isSuit() {
            return this.c6.selectedIndex == 1;
        }
        get itemVO() {
            return this.Data instanceof ItemVO ? this.Data : null;
        }
        get skinVO() {
            return this.Data instanceof SkinVO ? this.Data : null;
        }
        get skinId() {
            if (this.Data instanceof SkinVO) {
                return this.Data.skinRes.id;
            }
            else if (this.Data instanceof skinResource) {
                return this.Data.id;
            }
            return 0;
        }
        get itemRes() {
            if (this.Data instanceof ItemVO) {
                return this.Data.itemRes;
            }
            if (this.Data instanceof SkinVO) {
                return this.Data.itemRes;
            }
            else if (this.Data instanceof itemResource) {
                return this.Data;
            }
            else if (this.Data instanceof skinResource) {
                let itemRes = itemResource.getItemByKey(this.Data.id);
                return itemRes;
            }
            return null;
        }
        Reset() {
            super.Reset();
        }
    }

    class UIBagSuitItem extends TListItem {
        constructor() {
            super();
        }
        InitUI() {
            super.InitUI();
        }
        InitData(data) {
            super.InitData(data);
            if (data instanceof SuitVO) {
                this.txt_name.text = data.suitRes.name;
                let skinVos = data.skinVos;
                this.txt_count.text = data.skinOwnedCount + "/" + data.skinIds.length;
                this.eglist_items.Data = skinVos;
            }
            this.AddGameListener(fgui.Events.CLICK_ITEM, this.onClickBagItem, this, this.eglist_items);
        }
        onClickBagItem(item) {
            GameDispatcher.Inst.event(UIGameEvent.CLICK_SUIT_ITEM, item);
        }
        Reset() {
            super.Reset();
            this.RemoveGameListener(fgui.Events.CLICK_ITEM, this.onClickBagItem, this, this.eglist_items);
        }
    }

    class UI3DModel extends fgui.GComponent {
        constructor() {
            super();
            this.uiScene3D = null;
            this._translate = new Laya.Vector3();
            this.pos = new Laya.Point();
            this.nodeParent = null;
            this.role = null;
            this.role_url = null;
            this.showScale = 1;
            this.showOffsetX = 0;
            this.showOffsetY = 0;
            this.isTween = false;
            this.iRes = null;
        }
        constructFromXML(xml) {
            super.constructFromXML(xml);
            this.nodeParent = new Laya.Sprite3D();
        }
        updateLoop() {
            if (this.nodeParent == null)
                return;
            this.nodeParent.transform.localRotationEulerY += 1;
        }
        onResize() {
            if (!this.uiScene3D)
                return;
            this.localToGlobal(this.showOffsetX, this.showOffsetY, this.pos);
            let scale = 1 * this.scaleX * this.showScale;
            this.uiScene3D.GetCamera().convertScreenCoordToOrthographicCoord(new Laya.Vector3(this.pos.x, this.pos.y), this._translate);
            this.nodeParent.transform.position = this._translate;
            if (this.isTween) {
                let localScale = this.nodeParent.transform.localScale;
                localScale.setValue(0.2, 0.2, 0.2);
                this.nodeParent.transform.localScale = localScale;
                Laya.Tween.clearAll(this.nodeParent.transform);
                Laya.Tween.to(this.nodeParent.transform, { "localScaleX": scale, "localScaleY": scale, "localScaleZ": scale }, 500);
            }
            else {
                let localScale = this.nodeParent.transform.localScale;
                localScale.setValue(scale, scale, scale);
                this.nodeParent.transform.localScale = localScale;
            }
        }
        SetOffsetScale(scale, offsetX, offsetY) {
            this.showScale = scale;
            this.showOffsetX = offsetX;
            this.showOffsetY = offsetY;
            this.onResize();
            console.log("缩放：", scale, "偏移X：", offsetX, "偏移Y：", offsetY);
        }
        ResetRole() {
            Laya.timer.clear(this, this.updateLoop);
            Laya.Tween.clearAll(this.nodeParent.transform);
            if (this.role) {
                if (this.iRes) {
                    this.iRes.Dispose();
                    this.iRes = null;
                }
                this.nodeParent.removeSelf();
                this.role.removeSelf();
                this.role = null;
            }
            this.role_url = null;
            if (this.uiScene3D && !this.uiScene3D.destroyed) {
                this.uiScene3D.destroy();
                this.uiScene3D = null;
            }
        }
        Reset() {
            this.showScale = 1;
            this.ResetRole();
            this.role_url = null;
            this.isTween = false;
        }
    }

    class UIMoneyComp extends EButton {
        constructor() {
            super();
            this.sign = "";
            this.initX = 0;
            this.moneyVo = new MoneyVo(MoneyVo.GOLD);
        }
        InitUI() {
            super.InitUI();
            this.initX = this._titleObject.x;
        }
        InitData(data) {
            super.InitData(data);
            this.updateInfo();
        }
        updateInfo() {
            if (this.Data instanceof MoneyVo) {
                this.icon = this.Data.iconUrl;
                this.title = this.Data.getValueString(this.sign);
            }
            else if (this.Data instanceof AwardVO) {
                this.icon = this.Data.getCoinUrl(this.Data.rate, true);
                this.title = this.Data.getValueString(this.sign);
            }
        }
        setMoney(type, num) {
            this.moneyVo.type = type;
            this.moneyVo.setVal(num);
            this.InitData(this.moneyVo);
        }
        setSign(sign) {
            this.sign = sign;
            this.updateInfo();
        }
        set titleOffsetX(value) {
            if (this._titleObject) {
                this._titleObject.x = this.initX + value;
            }
        }
        set isNoBg(value) {
            this.c1.selectedIndex = value ? 1 : 0;
        }
        get isNoBg() {
            return this.c1.selectedIndex == 1;
        }
        get MoneyVo() {
            return this.Data;
        }
        get moneyPoint() {
            return this._iconObject && this._iconObject.localToGlobal();
        }
        shake() {
            if (this._iconObject == null)
                return;
            fgui.GTween.to(1, 1.2, 0.2)
                .setTarget(this._iconObject)
                .onUpdate((tw) => {
                this._iconObject.setScale(tw.value.x, tw.value.x);
            }, this).setRepeat(1, true).setEase(fgui.EaseType.Linear);
        }
        flyCoin() {
            if (this.Data) {
                EffectMgr.Inst.playGoldEffect(this.moneyVo.num, this.localToGlobal(0, 0));
            }
        }
        set bold(value) {
            if (this._titleObject instanceof fgui.GTextField) {
                this._titleObject.bold = value;
            }
        }
        get bold() {
            return this._titleObject instanceof fgui.GTextField && this._titleObject.bold;
        }
        get coinPoint() {
            return this._iconObject.localToGlobal();
        }
        Reset() {
            super.Reset();
        }
    }

    class UITypeIcon extends fgui.GComponent {
        constructor() {
            super();
        }
        SetIndex(value) {
            let ctr = this.getController("ctr");
            ctr.selectedIndex = value;
        }
    }

    class UIScene3D extends Laya.Scene3D {
        constructor() {
            super();
            this.m_camera = null;
            this.m_light = null;
            this.pointLight0 = null;
            this.pointLight1 = null;
            this.lightArr = null;
            this.Init();
        }
        Init() {
            let camera = new Laya.Camera(0, 0.1, 1000);
            this.addChild(camera);
            camera.transform.position = new Laya.Vector3(0, 0.39, 11.26);
            camera.transform.rotationEuler = new Laya.Vector3(-1.609, 0, 0);
            camera.clearFlag = Laya.CameraClearFlags.DepthOnly;
            camera.orthographic = false;
            camera.fieldOfView = 50;
            this.m_camera = camera;
            this.lightArr = [];
            let lightResArr = Scene3DLightResource.items;
            for (let i = 0; i < lightResArr.length; i++) {
                let res = lightResArr[i];
                let light;
                if (res.type == UIScene3D.TYPE_DIRLIGHT) {
                    light = new Laya.DirectionLight();
                    this.addChild(light);
                }
                else if (res.type == UIScene3D.TYPE_POINTLIGHT) {
                    light = new Laya.PointLight();
                    this.addChild(light);
                    if (res.range) {
                        light.range = res.range;
                    }
                }
                light.lightmapBakedType = Laya.LightSprite.LIGHTMAPBAKEDTYPE_REALTIME;
                if (res.position && res.position.length > 0) {
                    light.transform.position = MathUtil.GetPosByStr(res.position);
                }
                if (res.rotation && res.rotation.length > 0) {
                    light.transform.rotationEuler = MathUtil.GetRotByStr(res.rotation);
                }
                if (res.color && res.color.length > 0) {
                    light.color = MathUtil.GetColorByStr(res.color);
                }
                if (res.intensity) {
                    light.intensity = res.intensity;
                }
                this.lightArr.push(light);
            }
        }
        ResetPointLightPos(target) {
            if (!target || target.destroyed)
                return;
            let lightResArr = Scene3DLightResource.items;
            for (let i = 0; i < this.lightArr.length; i++) {
                let light = this.lightArr[i];
                let res = lightResArr[i];
                if (i > lightResArr.length - 1)
                    break;
                if (!res)
                    continue;
                if (res.type == UIScene3D.TYPE_POINTLIGHT && res.position && res.position.length > 0) {
                    light.transform.position = this.TransformPoint(target.transform, MathUtil.GetPosByStr(res.position));
                }
            }
        }
        TransformPoint(origin, point) {
            var out = new Laya.Vector3();
            Laya.Vector3.transformQuat(point, origin.rotation, out);
            Laya.Vector3.add(out, origin.position, out);
            return out;
        }
        GetCamera() {
            return this.m_camera;
        }
        GetLight() {
            return this.m_light;
        }
    }
    UIScene3D.TYPE_DIRLIGHT = 0;
    UIScene3D.TYPE_POINTLIGHT = 1;

    class UI3DScene extends TComponent {
        constructor() {
            super();
            this.scene3d = null;
            this.catModel = null;
            this._catName = null;
            this.catAni = null;
            this.aniScript = null;
            this.aniName = null;
            this.isLoop = false;
            this.complete = null;
            this.catSkinScript = null;
            this.showScale = 1;
            this.showOffsetX = 0;
            this.showOffsetY = 0;
            this.nodeParent = null;
            this._translate = new Laya.Vector3();
            this.isTween = false;
            this._view = null;
            this.catInitScale = 1;
            this.pos = new Laya.Point();
            this.out = new Laya.Vector3();
            this.pos1 = new Laya.Point();
        }
        InitFairyGui() {
        }
        InitUI() {
            super.InitUI();
            this.touchable = false;
            this.LoadScene();
        }
        InitData(param) {
            super.InitData(param);
            this.LoadScene();
        }
        LoadScene() {
            if (this.scene3d == null) {
                this.scene3d = new UIScene3D();
                if (this.scene3d.parent != this.displayListContainer) {
                    this.displayListContainer.addChild(this.scene3d);
                }
                this.InitScene();
            }
            else {
                this.InitScene();
            }
        }
        InitScene() {
            if (this.scene3d == null || this.scene3d.destroyed)
                return;
            this.nodeParent = new Laya.Sprite3D();
            this.scene3d.addChild(this.nodeParent);
            let light = this.scene3d.GetLight();
        }
        get catName() {
            return this._catName;
        }
        set catName(name) {
            if (this._catName == name)
                return;
            this._catName = name;
        }
        LoadCat(cat, complete = null, view = null) {
            this._view = view;
            if (this.catName == cat) {
                return;
            }
            Laya.timer.clear(this, this.onResize);
            Laya.timer.loop(1, this, this.onResize);
            this.catName = cat;
            if (this.catModel && !this.catModel.destroyed) {
                this.catModel.destroy();
                this.catModel = null;
                this.catAni = null;
            }
            let url = UrlUtils.GetCatUrl(cat);
            LoadQueue.Inst.loadSprite3D(url, Laya.Handler.create(this, (sp) => {
                if (sp == null)
                    return;
                sp = sp.clone();
                this.catModel = sp;
                if (!this.scene3d || this.scene3d.destroyed) {
                    this.scene3d = null;
                    this.LoadScene();
                }
                this.scene3d.addChild(sp);
                this.pos.setTo(this.x, this.y);
                this.catInitScale = sp.transform.localScale.x;
                this.InitModel(sp, this.pos);
                this.scene3d.ResetPointLightPos(sp);
                this.catSkinScript = sp.addComponent(CatSkinSctipt);
                this.aniScript = sp.addComponent(AnimatorScript);
                this.catAni = sp && sp.getComponent(Laya.Animator);
                complete && complete.runWith(sp);
            }));
        }
        InitModel(sp, pos) {
            if (!sp || !this.scene3d)
                return;
            this.out.setValue(pos.x, pos.y, -10);
            sp.transform.position = MathUtil.ScreenToWorld(this.scene3d.GetCamera(), this.out);
        }
        ResetCatPos() {
            if (!this.catModel)
                return;
            if (!this.scene3d)
                return;
            if (this.parent == null)
                return;
            let pos = this.parent.localToGlobal(this.x, this.y);
            this.out.setValue(pos.x, pos.y, -10);
            this.catModel.transform.position = MathUtil.ScreenToWorld(this.scene3d.GetCamera(), this.out);
        }
        PlayAni(aniName, isLoop = false, complete = null, speed = 1, transitionDuration = NaN) {
            if (this.aniScript) {
                if (isNaN(transitionDuration)) {
                    this.aniScript.PlayState(aniName, complete, speed);
                }
                else {
                    this.aniScript.CrossFadeState(aniName, complete, transitionDuration, speed);
                }
            }
            if (this.catAni) {
            }
        }
        PlayCatAni(aniName, isLoop = false, complete = null, speed = 1) {
            if (this.catAni && this.catAni.getControllerLayer(0).getCurrentPlayState().normalizedTime >= 1) {
                Laya.timer.clear(this, this.PlayCatAni);
                if (isLoop) {
                    this.PlayAni(aniName, isLoop, complete, speed);
                }
                else {
                    complete && complete.run();
                }
            }
        }
        AddCatPart(parts) {
            if (!this.catSkinScript)
                return;
            this.catSkinScript.ChangeSkin(parts);
        }
        AddModel(url, pos, scale = 1, rotationE = new Laya.Vector3()) {
            if (!url)
                return;
            if (this.scene3d == null || this.scene3d.destroyed) {
                this.scene3d = null;
                this.LoadScene();
            }
            LoadQueue.Inst.loadSprite3D(url, Laya.Handler.create(this, (sp) => {
                if (sp == null)
                    return;
                if (this.scene3d == null || this.scene3d.destroyed) {
                    this.scene3d = null;
                    this.LoadScene();
                }
                this.scene3d.addChild(sp);
                this.InitModel(sp, pos);
                sp.transform.localScale = new Laya.Vector3(scale, scale, scale);
                sp.transform.rotationEuler = rotationE;
            }));
        }
        AddSprite3d(sp) {
            if (!this.scene3d || this.scene3d.destroyed)
                return;
            if (!this.scene3d) {
                this.LoadScene();
            }
            this.scene3d.addChild(sp);
        }
        SetOffsetScale(scale, offsetX, offsetY) {
            this.showScale = scale;
            this.showOffsetX = offsetX;
            this.showOffsetY = offsetY;
            this.onResize();
            console.log("缩放：", scale, "偏移X：", offsetX, "偏移Y：", offsetY);
        }
        onResize() {
            if (!this.scene3d)
                return;
            if (!this.catModel || this.catModel.destroyed)
                return;
            this.ResetCatPos();
            let viewScale = 1;
            if (this._view && !this._view.isDisposed) {
                viewScale = this._view.scaleX;
            }
            let scale = 1 * this.scaleX * this.showScale * this.catInitScale * viewScale;
            if (this.isTween) {
                let localScale = this.catModel.transform.localScale;
                localScale.setValue(0.2, 0.2, 0.2);
                this.catModel.transform.localScale = localScale;
                Laya.Tween.clearAll(this.catModel.transform);
                Laya.Tween.to(this.catModel.transform, { "localScaleX": scale, "localScaleY": scale, "localScaleZ": scale }, 500);
            }
            else {
                let localScale = this.catModel.transform.localScale.clone();
                localScale.setValue(scale, scale, scale);
                this.catModel.transform.localScale = localScale;
            }
        }
        Reset() {
            Laya.timer.clear(this, this.PlayCatAni);
            Laya.timer.clear(this, this.onResize);
            this._view = null;
            if (this.nodeParent) {
                this.nodeParent.removeSelf();
            }
            this.nodeParent = null;
            if (this.catSkinScript) {
                this.catSkinScript.ChangeSkin([]);
            }
            if (this.scene3d && !this.scene3d.destroyed) {
                this.scene3d.destroy();
            }
            this.scene3d = null;
            if (this.catModel && !this.catModel.destroyed) {
                this.catModel.destroy();
            }
            this.catModel = null;
            this.catName = null;
            this.catAni = null;
            this.aniName = null;
            this.isLoop = false;
        }
    }

    class LoadAnimationAction extends BaseAction {
        constructor() {
            super();
            this.url = null;
            this.handler = null;
            this.level = LoadLevel.DEFAULT;
        }
        Run() {
            super.Run();
            LoadQueue;
            LoadQueue.Inst.loadAnimationClip(this.url, Laya.Handler.create(this, this.onLoadComplete), false, this.level);
        }
        onLoadComplete(sp) {
            if (this.handler) {
                this.handler.runWith(sp);
                this.handler = null;
            }
            this.m_isComplete = true;
        }
        Reset() {
            super.Reset();
            this.url = null;
            this.handler = null;
        }
        GC() {
            ObjectPools.Inst.gcObj(LoadAnimationAction.KEY, this);
        }
    }
    LoadAnimationAction.KEY = "LoadAnimationAction";

    class PreloadingMgr extends ActionQueue {
        constructor() {
            super();
        }
        static get Inst() {
            if (PreloadingMgr._Inst == null)
                PreloadingMgr._Inst = new PreloadingMgr();
            return PreloadingMgr._Inst;
        }
        Init() {
            for (const preLoadingRes of PreloadingResource.items) {
                switch (preLoadingRes.type) {
                    case PreloadingMgr.TYPE_BUILD:
                        let loadAction = ObjectPools.Inst.getObj(LoadSprite3dAction.KEY, LoadSprite3dAction);
                        loadAction.url = UrlUtils.GetBuildUrl(preLoadingRes.resName);
                        loadAction.level = LoadLevel.PROLOADING;
                        this.AddAction(loadAction);
                        break;
                    case PreloadingMgr.TYPE_CAT_ANIMATION:
                        let loadAction2 = ObjectPools.Inst.getObj(LoadAnimationAction.KEY, LoadAnimationAction);
                        loadAction2.url = UrlUtils.GetCatAnimationUrl(preLoadingRes.resName);
                        loadAction2.level = LoadLevel.PROLOADING;
                        this.AddAction(loadAction2);
                        break;
                    case PreloadingMgr.TYPE_CAT_MODEL:
                        if (Laya.Browser.onIOS)
                            break;
                        let loadAction3 = ObjectPools.Inst.getObj(LoadSprite3dAction.KEY, LoadSprite3dAction);
                        loadAction3.url = UrlUtils.GetCatUrl(preLoadingRes.resName);
                        loadAction3.level = LoadLevel.PROLOADING;
                        this.AddAction(loadAction3);
                        break;
                }
            }
        }
        advanceTime() {
            if (!LoadQueue.Inst.isEmpty)
                return;
            super.advanceTime();
        }
    }
    PreloadingMgr.TYPE_BUILD = 1;
    PreloadingMgr.TYPE_CAT_ANIMATION = 2;
    PreloadingMgr.TYPE_CAT_MODEL = 3;
    PreloadingMgr._Inst = null;

    class JSUtil {
        constructor() {
        }
        static getRequest(url = location.search) {
            if (JSUtil._rquest == null) {
                var theRequest = this.getRequestParams(url);
                JSUtil._rquest = theRequest;
            }
            return JSUtil._rquest;
        }
        static getRequestParams(url) {
            if (!url)
                return {};
            var theRequest = new Object();
            var index = url.indexOf("?");
            if (index != -1) {
                var str = url.substring(index + 1);
                var strs = str.split("&");
                for (var i = 0; i < strs.length; i++) {
                    var arr = strs[i].split("=");
                    theRequest[arr[0]] = arr[1];
                }
            }
            return theRequest;
        }
        static getHref(url = window.location.href) {
            let index = url.lastIndexOf("?");
            if (index != -1) {
                url = url.substring(0, index);
            }
            index = url.lastIndexOf(".html");
            if (index != -1) {
                index = url.lastIndexOf("/");
                url = url.substring(0, index);
            }
            else {
                let lastStr = url.substr(url.length - 1, 1);
                if (lastStr == "/") {
                    url = url.substr(0, url.length - 1);
                }
            }
            return url;
        }
        static openHttpUrl(url, target = "_self") {
            window.open(url, target);
        }
    }

    class TweenExt {
        static init() {
            Laya.Tween.prototype["_initProps"] = TweenExt["_initProps"];
            Laya.Tween.prototype["_updateEase"] = TweenExt["_updateEase"];
            Laya.Tween.prototype["complete"] = TweenExt["complete"];
            Laya.Tween.prototype["completData"] = TweenExt["completData"];
        }
        static _initProps(target, props, isTo) {
            for (let p in props) {
                if ((typeof (target[p]) == 'number')) {
                    let start = isTo ? target[p] : props[p];
                    let end = isTo ? props[p] : target[p];
                    this["_props"].push([p, start, end - start]);
                    if (!isTo)
                        target[p] = start;
                }
                if (target[p] instanceof Laya.Vector4) {
                    let v4_lerp = new Laya.Vector4();
                    let v4_start = isTo ? target[p].clone() : props[p];
                    let v4_end = isTo ? props[p] : target[p].clone();
                    if (!(v4_start instanceof Laya.Vector4)) {
                        v4_start = new Laya.Vector4(v4_start.x, v4_start.y, v4_start.z, v4_start.w);
                    }
                    v4_end = new Laya.Vector4(v4_end.x, v4_end.y, v4_end.z, v4_end.w);
                    Laya.Vector4.subtract(v4_end, v4_start, v4_lerp);
                    this["_props"].push([p, v4_start, v4_lerp, v4_end]);
                    if (!isTo)
                        target[p] = v4_start;
                }
                let trans = target instanceof Laya.Transform3D ? target : target.transform;
                if (trans && trans[p] instanceof Laya.Vector3) {
                    let v3_lerp = new Laya.Vector3();
                    let v3_start = isTo ? trans[p].clone() : props[p];
                    let v3_end = isTo ? props[p] : trans[p].clone();
                    if (!(v3_start instanceof Laya.Vector3)) {
                        v3_start = new Laya.Vector3(v3_start.x, v3_start.y, v3_start.z);
                    }
                    v3_end = new Laya.Vector3(v3_end.x, v3_end.y, v3_end.z);
                    Laya.Vector3.subtract(v3_end, v3_start, v3_lerp);
                    this["_props"].push([p, v3_start, v3_lerp, v3_end]);
                    if (!isTo)
                        target[p] = v3_start;
                }
            }
        }
        static _updateEase(time) {
            var target = this["_target"];
            if (!target)
                return;
            if (target.destroyed)
                return this["clearTween"](target);
            var usedTimer = this["_usedTimer"] = time - this["_startTimer"] - this["_delay"];
            if (usedTimer < 0)
                return;
            if (usedTimer >= this["_duration"])
                return this["complete"]();
            var ratio = usedTimer > 0 ? this["_ease"](usedTimer, 0, 1, this["_duration"]) : 0;
            var props = this["_props"];
            for (var i = 0, n = props.length; i < n; i++) {
                var prop = props[i];
                if (typeof target[prop[0]] === "number") {
                    target[prop[0]] = prop[1] + (ratio * prop[2]);
                }
                if (target[prop[0]] instanceof Laya.Vector4) {
                    let x = prop[1].x + (ratio * prop[2].x);
                    let y = prop[1].y + (ratio * prop[2].y);
                    let z = prop[1].z + (ratio * prop[2].z);
                    let w = prop[1].w + (ratio * prop[2].w);
                    target[prop[0]] = new Laya.Vector4(x, y, z, w);
                }
                if (target[prop[0]] instanceof Laya.Vector3) {
                    let x = prop[1].x + (ratio * prop[2].x);
                    let y = prop[1].y + (ratio * prop[2].y);
                    let z = prop[1].z + (ratio * prop[2].z);
                    target[prop[0]] = new Laya.Vector3(x, y, z);
                }
                if (target.transform && target.transform[prop[0]] instanceof Laya.Vector3) {
                    let x = prop[1].x + (ratio * prop[2].x);
                    let y = prop[1].y + (ratio * prop[2].y);
                    let z = prop[1].z + (ratio * prop[2].z);
                    target.transform[prop[0]] = new Laya.Vector3(x, y, z);
                }
            }
            if (this["update"])
                this["update"].run();
        }
        static complete() {
            if (!this["_target"])
                return;
            Laya.timer.runTimer(this, this["firstStart"]);
            var handler = this["_complete"];
            this.completData();
            if (this["update"])
                this["update"].run();
            this["_count"]++;
            if (this["repeat"] != 0 && this["_count"] >= this["repeat"]) {
                this["clear"]();
                handler && handler.run();
            }
            else {
                this["restart"]();
            }
        }
        static completData() {
            var target = this["_target"];
            var props = this["_props"];
            for (var i = 0, n = props.length; i < n; i++) {
                var prop = props[i];
                if (typeof target[prop[0]] === "number") {
                    target[prop[0]] = prop[1] + prop[2];
                }
                if (target[prop[0]] instanceof Laya.Vector4) {
                    target[prop[0]] = prop[3];
                }
                if (target[prop[0]] instanceof Laya.Vector3) {
                    target[prop[0]] = prop[3];
                }
                if (target.transform && target.transform[prop[0]] instanceof Laya.Vector3) {
                    target.transform[prop[0]] = prop[3];
                }
            }
        }
    }

    var Scene = Laya.Scene;
    var REG = Laya.ClassUtils.regClass;
    var ui;
    (function (ui) {
        var test;
        (function (test) {
            class GameSceneUI extends Scene {
                constructor() { super(); }
                createChildren() {
                    super.createChildren();
                    this.loadScene("test/GameScene");
                }
            }
            test.GameSceneUI = GameSceneUI;
            REG("ui.test.GameSceneUI", GameSceneUI);
        })(test = ui.test || (ui.test = {}));
    })(ui || (ui = {}));

    class GameUI extends ui.test.GameSceneUI {
        constructor() {
            super();
            this.startLoadTime = 0;
            ToonWaterMaterial.initShader();
            if (Laya.Browser.onPC) {
                let urlObj = JSUtil.getRequest();
                let key;
                for (key in urlObj) {
                    if (key.toLowerCase() == "isclear" && urlObj[key] == "true") {
                        StorageUtils.Inst.clear();
                    }
                    else if (key.toLowerCase() == "stat" && urlObj[key] == "true") {
                        Laya.Stat.show();
                        GlobalConfig.stat = urlObj[key] == "true";
                    }
                    else if (key.toLowerCase() == "debug" && urlObj[key] == "true") {
                    }
                    else if (key.toLowerCase() == "guide") {
                        GuideModel.Inst.isOpenGuide = urlObj[key] == "true";
                    }
                    else if (key.toLowerCase() == "showguidebg") {
                        GuideModel.Inst.showGuideBg = urlObj[key] == "true";
                    }
                    else if (key.toLowerCase() == "isusegm") {
                        GlobalConfig.isUseGm = urlObj[key] == "true";
                    }
                    else if (key.toLowerCase() == "shader") {
                        GlobalConfig.isRunShader = urlObj[key] == "true";
                    }
                    else if (key.toLowerCase() == "CheckCatEventTIme") {
                        GlobalConfig.CheckCatEventTIme = parseInt(urlObj[key]);
                    }
                }
            }
            this.addEvent();
        }
        addEvent() {
            this.on(Laya.Event.ADDED, this, this.onAdd);
        }
        onAdd() {
            TweenExt.init();
            this.off(Laya.Event.ADDED, this, this.onAdd);
            GameLayer.sceneLayer = this;
            GameLayer.start();
            this.loadInitAll();
            this.width = GlobalConfig.viewWidth;
            this.height = GlobalConfig.viewHeight;
        }
        initShader() {
            let shaderStr = '{"Unlit":[{"defineNames":["COLOR","ALBEDOTEXTURE"],"passIndex":0,"subShaderIndex":0},{"defineNames":["ALBEDOTEXTURE"],"passIndex":0,"subShaderIndex":0},{"defineNames":["ALPHATEST","ALBEDOTEXTURE"],"passIndex":0,"subShaderIndex":0},{"defineNames":["GPU_INSTANCE","COLOR","ALBEDOTEXTURE"],"passIndex":0,"subShaderIndex":0},{"defineNames":["GPU_INSTANCE","ALBEDOTEXTURE"],"passIndex":0,"subShaderIndex":0},{"defineNames":["GPU_INSTANCE","ALPHATEST","ALBEDOTEXTURE"],"passIndex":0,"subShaderIndex":0},{"defineNames":["COLOR","TILINGOFFSET","ALBEDOTEXTURE"],"passIndex":0,"subShaderIndex":0},{"defineNames":["BONE","ALPHATEST","ALBEDOTEXTURE"],"passIndex":0,"subShaderIndex":0}],"BLINNPHONG":[{"defineNames":["RECEIVESHADOW","DIFFUSEMAP","DIRECTIONLIGHT","UV"],"passIndex":0,"subShaderIndex":0},{"defineNames":["RECEIVESHADOW","DIFFUSEMAP","DIRECTIONLIGHT","UV","UV1","COLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["GPU_INSTANCE","RECEIVESHADOW","DIFFUSEMAP","DIRECTIONLIGHT","UV","UV1","COLOR","ALPHATEST"],"passIndex":0,"subShaderIndex":0},{"defineNames":["RECEIVESHADOW","DIFFUSEMAP","DIRECTIONLIGHT","UV","UV1","COLOR","ALPHATEST"],"passIndex":0,"subShaderIndex":0},{"defineNames":["RECEIVESHADOW","DIFFUSEMAP","DIRECTIONLIGHT","UV","ALPHATEST"],"passIndex":0,"subShaderIndex":0},{"defineNames":["RECEIVESHADOW","DIRECTIONLIGHT","UV","COLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["GPU_INSTANCE","RECEIVESHADOW","DIFFUSEMAP","DIRECTIONLIGHT","UV"],"passIndex":0,"subShaderIndex":0},{"defineNames":["GPU_INSTANCE","RECEIVESHADOW","DIFFUSEMAP","DIRECTIONLIGHT","UV","UV1","COLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["GPU_INSTANCE","RECEIVESHADOW","DIFFUSEMAP","DIRECTIONLIGHT","UV","COLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["GPU_INSTANCE","RECEIVESHADOW","DIFFUSEMAP","DIRECTIONLIGHT","UV","ALPHATEST"],"passIndex":0,"subShaderIndex":0},{"defineNames":["GPU_INSTANCE","RECEIVESHADOW","DIFFUSEMAP","DIRECTIONLIGHT","UV","COLOR","ALPHATEST"],"passIndex":0,"subShaderIndex":0},{"defineNames":["RECEIVESHADOW","DIRECTIONLIGHT","UV"],"passIndex":0,"subShaderIndex":0},{"defineNames":["RECEIVESHADOW","DIFFUSEMAP","DIRECTIONLIGHT","UV","COLOR","ALPHATEST"],"passIndex":0,"subShaderIndex":0},{"defineNames":["RECEIVESHADOW","DIFFUSEMAP","DIRECTIONLIGHT","UV","UV1","BONE"],"passIndex":0,"subShaderIndex":0},{"defineNames":["RECEIVESHADOW","DIFFUSEMAP","DIRECTIONLIGHT","UV","COLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["GPU_INSTANCE","RECEIVESHADOW","DIRECTIONLIGHT","UV","COLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["RECEIVESHADOW","DIFFUSEMAP","DIRECTIONLIGHT","UV","COLOR","TILINGOFFSET"],"passIndex":0,"subShaderIndex":0},{"defineNames":["RECEIVESHADOW","DIFFUSEMAP","DIRECTIONLIGHT","UV","BONE"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","DIRECTIONLIGHT","ALPHATEST"],"passIndex":0,"subShaderIndex":0},{"defineNames":["GPU_INSTANCE","RECEIVESHADOW","DIRECTIONLIGHT","UV"],"passIndex":0,"subShaderIndex":0},{"defineNames":["RECEIVESHADOW","DIFFUSEMAP","DIRECTIONLIGHT","UV","BONE","ALPHATEST"],"passIndex":0,"subShaderIndex":0},{"defineNames":["RECEIVESHADOW","DIFFUSEMAP","DIRECTIONLIGHT","UV","UV1","BONE","ALPHATEST"],"passIndex":0,"subShaderIndex":0},{"defineNames":["RECEIVESHADOW","DIRECTIONLIGHT","UV","BONE","ALPHATEST"],"passIndex":0,"subShaderIndex":0},{"defineNames":["RECEIVESHADOW","DIRECTIONLIGHT","UV","BONE"],"passIndex":0,"subShaderIndex":0},{"defineNames":["RECEIVESHADOW","DIRECTIONLIGHT","UV","ALPHATEST"],"passIndex":0,"subShaderIndex":0},{"defineNames":["SIMPLEBONE","RECEIVESHADOW","DIFFUSEMAP","DIRECTIONLIGHT","UV","BONE"],"passIndex":0,"subShaderIndex":0},{"defineNames":["SIMPLEBONE","RECEIVESHADOW","DIFFUSEMAP","DIRECTIONLIGHT","UV","BONE","ALPHATEST"],"passIndex":0,"subShaderIndex":0}],"PARTICLESHURIKEN":[{"defineNames":["TILINGOFFSET","ADDTIVEFOG","RENDERMODE_MESH","COLOROVERLIFETIME","SIZEOVERLIFETIMECURVE","SHAPE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","ADDTIVEFOG","SPHERHBILLBOARD","SIZEOVERLIFETIMECURVE","SHAPE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","ADDTIVEFOG","SPHERHBILLBOARD","COLOROVERLIFETIME","VELOCITYOVERLIFETIMERANDOMCONSTANT","SHAPE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","SPHERHBILLBOARD","COLOROVERLIFETIME","SIZEOVERLIFETIMECURVE","TEXTURESHEETANIMATIONCURVE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","TILINGOFFSET","ADDTIVEFOG","RENDERMODE_MESH","TEXTURESHEETANIMATIONCURVE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","SPHERHBILLBOARD","COLOROVERLIFETIME","VELOCITYOVERLIFETIMERANDOMCONSTANT","SIZEOVERLIFETIMECURVE","TEXTURESHEETANIMATIONCURVE","SHAPE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","ADDTIVEFOG","SPHERHBILLBOARD","COLOROVERLIFETIME","SIZEOVERLIFETIMECURVE","SHAPE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","RENDERMODE_MESH","COLOROVERLIFETIME","ROTATIONOVERLIFETIMECONSTANT","ROTATIONOVERLIFETIMESEPERATE"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","SPHERHBILLBOARD","COLOROVERLIFETIME","SIZEOVERLIFETIMECURVE","SHAPE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","ADDTIVEFOG","STRETCHEDBILLBOARD","COLOROVERLIFETIME","SIZEOVERLIFETIMECURVE","SHAPE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","TILINGOFFSET","ADDTIVEFOG","RENDERMODE_MESH","COLOROVERLIFETIME","SIZEOVERLIFETIMECURVESEPERATE","ROTATIONOVERLIFETIME","ROTATIONOVERLIFETIMECONSTANT","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","ADDTIVEFOG","STRETCHEDBILLBOARD","COLOROVERLIFETIME","SIZEOVERLIFETIMECURVE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","ADDTIVEFOG","SPHERHBILLBOARD","COLOROVERLIFETIME","SIZEOVERLIFETIMECURVE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","ADDTIVEFOG","SPHERHBILLBOARD","COLOROVERLIFETIME","SIZEOVERLIFETIMECURVE","TEXTURESHEETANIMATIONCURVE","SHAPE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","TILINGOFFSET","ADDTIVEFOG","RENDERMODE_MESH","SIZEOVERLIFETIMECURVE","ROTATIONOVERLIFETIME","ROTATIONOVERLIFETIMECONSTANT","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","ADDTIVEFOG","SPHERHBILLBOARD","COLOROVERLIFETIME","SIZEOVERLIFETIMECURVE","ROTATIONOVERLIFETIME","ROTATIONOVERLIFETIMECONSTANT","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","TILINGOFFSET","ADDTIVEFOG","SPHERHBILLBOARD","COLOROVERLIFETIME","SIZEOVERLIFETIMECURVE","SHAPE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","TILINGOFFSET","ADDTIVEFOG","STRETCHEDBILLBOARD","COLOROVERLIFETIME","SIZEOVERLIFETIMECURVE","SHAPE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","ADDTIVEFOG","SPHERHBILLBOARD","COLOROVERLIFETIME","TEXTURESHEETANIMATIONCURVE","SHAPE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","STRETCHEDBILLBOARD","COLOROVERLIFETIME","SIZEOVERLIFETIMECURVE","SHAPE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","SPHERHBILLBOARD","COLOROVERLIFETIME","ROTATIONOVERLIFETIMERANDOMCONSTANTS","SIZEOVERLIFETIMECURVE","ROTATIONOVERLIFETIME","SHAPE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","SPHERHBILLBOARD","COLOROVERLIFETIME","SIZEOVERLIFETIMECURVE","TEXTURESHEETANIMATIONCURVE","SHAPE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","SPHERHBILLBOARD","COLOROVERLIFETIME","TEXTURESHEETANIMATIONCURVE","SHAPE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","SPHERHBILLBOARD","COLOROVERLIFETIME","SIZEOVERLIFETIMECURVE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","RENDERMODE_MESH","COLOROVERLIFETIME","ROTATIONOVERLIFETIMECONSTANT","ROTATIONOVERLIFETIMESEPERATE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","TILINGOFFSET","SPHERHBILLBOARD","COLOROVERLIFETIME","SIZEOVERLIFETIMECURVE","SHAPE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","ADDTIVEFOG","SPHERHBILLBOARD","COLOROVERLIFETIME","ROTATIONOVERLIFETIMERANDOMCONSTANTS","SIZEOVERLIFETIMECURVE","ROTATIONOVERLIFETIME","SHAPE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","ADDTIVEFOG","STRETCHEDBILLBOARD","COLOROVERLIFETIME","TEXTURESHEETANIMATIONCURVE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","ADDTIVEFOG","SPHERHBILLBOARD","COLOROVERLIFETIME","VELOCITYOVERLIFETIMECONSTANT","SIZEOVERLIFETIMECURVE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","STRETCHEDBILLBOARD","COLOROVERLIFETIME","SIZEOVERLIFETIMECURVE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","ADDTIVEFOG","SPHERHBILLBOARD","COLOROVERLIFETIME","TEXTURESHEETANIMATIONCURVE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","ADDTIVEFOG","HORIZONTALBILLBOARD","COLOROVERLIFETIME","SIZEOVERLIFETIMECURVE","SHAPE","TINTCOLOR"],"passIndex":0,"subShaderIndex":0},{"defineNames":["DIFFUSEMAP","TILINGOFFSET","ADDTIVEFOG","RENDERMODE_MESH","TINTCOLOR"],"passIndex":0,"subShaderIndex":0}],"ToonWaterShader":[{"defineNames":[],"passIndex":0,"subShaderIndex":0}],"Effect":[{"defineNames":["TILINGOFFSET","ADDTIVEFOG","MAINTEXTURE"],"passIndex":0,"subShaderIndex":0},{"defineNames":["COLOR","TILINGOFFSET","ADDTIVEFOG","MAINTEXTURE"],"passIndex":0,"subShaderIndex":0}]}';
            if (!shaderStr)
                return;
            let shaderObj = JSON.parse(shaderStr);
            let shaderName = "";
            let shaderArrs = [];
            for (shaderName in shaderObj) {
                let arr = shaderObj[shaderName];
                for (let index = 0; index < arr.length; index++) {
                    let obj = arr[index];
                    obj["shaderName"] = shaderName;
                    shaderArrs.push(obj);
                }
            }
            this.CompileShader(shaderArrs, 0);
        }
        CompileShader(arr, index) {
            if (index >= arr.length)
                return;
            let obj = arr[index];
            Laya.Shader3D.compileShaderByDefineNames(obj["shaderName"], obj.subShaderIndex, obj.passIndex, obj.defineNames);
            index++;
            Laya.timer.frameOnce(1, this, this.CompileShader, [arr, index]);
        }
        loadInitAll() {
            this.startLoadTime = Laya.timer.currTimer;
            UISystem.Inst.CreateWindowView(UILoadingView, [Laya.Handler.create(this, this.loadingInit), Laya.Handler.create(this, this.enterScene)]);
        }
        loadingInit() {
            let lf = this;
            StorageUtils.Inst.initStorage(Laya.Handler.create(null, function () {
                StorageUtils.Inst.enterGame();
                TimeCtrl.Inst.init();
                MoneyCtrl.Inst.init();
                SceneMgr.Inst.InitScene();
                LoadQueue.Inst.loadBin(UrlUtils.GetTemplateUrl("ExcelsData"), Laya.Handler.create(lf, lf.onLoadTemplate));
                LoadQueue.Inst.loadUI(UrlUtils.UI + "common", null, false);
                LoadQueue.Inst.loadUI(UrlUtils.UI + "ad", null, false);
                LoadQueue.Inst.loadUI(UrlUtils.UI + "main", null, false);
                LoadQueue.Inst.loadUI(UrlUtils.UI + "guide", null, false);
                LoadQueue.Inst.loadFguiPkg("icon_cat_head", UrlUtils.UI, null, false);
                LoadQueue.Inst.loadFguiPkg("icon_pose", UrlUtils.UI, null, false);
                LoadQueue.Inst.loadFguiPkg("icon_pose_cirle", UrlUtils.UI, null, false);
                LoadQueue.Inst.loadFguiPkg("icon_pose_rect", UrlUtils.UI, null, false);
                LoadQueue.Inst.loadFguiPkg("icon_items", UrlUtils.UI, null, false);
                LoadQueue.Inst.loadFguiPkg("icon_task", UrlUtils.UI, null, false);
                LoadQueue.Inst.loadFguiPkg("icon_part", UrlUtils.UI, null, false);
                LoadQueue.Inst.loadFguiPkg("icon_build", UrlUtils.UI, null, false);
                LoadQueue.Inst.loadFguiPkg("icon_award_part", UrlUtils.UI, null, false);
                LoadQueue.Inst.loadSpine(UrlUtils.GetSpineUrl("loadingcircle"));
            }));
        }
        onLoadTemplate(byte) {
            InitTpl.InitTpls(byte);
            CameraModel.Inst.init();
            AddFishModel.Inst.init();
            LoginModel.Inst.init();
            ShopLockDrawModel.Inst.init();
            GiftPKGModel.Inst.init();
            CatFecalBallModel.Inst.init();
            CatLangModel.Inst.init();
            PlatformModel.Inst.init();
            WarOrderModel.Inst.init();
            DriftingBottleModel.Inst.init();
            YuYueModel.Inst.init();
            LiHeModel.Inst.init();
            PlatformMgr.Inst.aldSendEvent(EventType.kaishijiazai);
            AdCtrl.Inst.init();
        }
        enterScene() {
            PlatformMgr.Inst.aldSendEvent(EventType.jiazaiwancheng, "所有触发");
            let lastTime = Laya.timer.currTimer - this.startLoadTime;
            lastTime = Math.floor(lastTime * 0.001);
            if (lastTime >= 50) {
                PlatformMgr.Inst.aldSendEvent(EventType.jiazaiwancheng, "超过50s");
            }
            else {
                PlatformMgr.Inst.aldSendEvent(EventType.jiazaiwancheng, lastTime + "s");
            }
            PanelRegister.registerClass("common", "UI3DModel", UI3DModel);
            PanelRegister.registerClass("common", "UITypeIcon", UITypeIcon);
            PanelRegister.registerClass("common", "EButton", EButton);
            PanelRegister.registerClass("common", "SpineButton", TSpineButton);
            PanelRegister.registerClass("common", "UIMoneyComp", UIMoneyComp);
            PanelRegister.registerClass("common", "UIBagItem", UIBagItem);
            PanelRegister.registerClass("common", "UIBagSuitItem", UIBagSuitItem);
            PanelRegister.registerClass("common", "UICatHeadItem", UICatHeadItem);
            PanelRegister.registerClass("common", "UICameraRotateScript", UICameraRotateScript);
            PanelRegister.registerClass("common", "UICameraStrokeScript", UICameraStrokeScript);
            PanelRegister.registerClass("common", "UILoadingLeafView", UILoadingLeafView);
            PanelRegister.registerClass("common", "UI3DScene", UI3DScene);
            PanelRegister.registerClass("common", "Icon", Icon);
            PanelRegister.registerClass("common", "Icon2", Icon);
            PanelRegister.registerClass("common", "btn_yellow_ad", VideoButton);
            PanelRegister.registerClass("common", "btn_adGet", VideoButton);
            PanelRegister.registerClass("common", "UITaskAwardItem", UITaskAwardItem);
            PanelRegister.registerClass("common", "UITaskAwardItem2", UITaskAwardItem);
            PanelRegister.registerClass("ad", "NativeIconAd", UINativeIconAd);
            PanelRegister.registerClass("ad", "NativeMuiltGrid", UINativeMuiltGrid);
            PanelRegister.registerClass("ad", "UIGameClubButton", UIGameClubButton);
            this.loginGame();
        }
        loginGame() {
            if (Laya.Browser.onHWMiniGame) {
                PlatformMgr.Inst.login("", "", Laya.Handler.create(this, this.loginPlatform));
            }
            else {
                PlatformMgr.Inst.login();
                if (PlatformMgr.Inst.isMZ) {
                    Laya.timer.once(2000, this, this.enterGame);
                }
                else {
                    this.enterGame();
                }
            }
        }
        loginPlatform(isSuccess) {
            let self = this;
            if (isSuccess) {
                let isAgree = StorageUtils.Inst.isAgreeHWAd;
                if (Laya.Browser.onHWMiniGame && !isAgree) {
                    UISystem.Inst.CreateWindowView(UIAdAgreeView, { "handler": Laya.Handler.create(self, self.enterGame) });
                    return;
                }
                else {
                    this.enterGame();
                }
            }
            else {
                console.log("=====>登陆失败");
                GameDispatcher.Inst.event(UIGameEvent.LOGIN_FAIL, [Laya.Handler.create(this, this.loginPlatform)]);
            }
        }
        enterGame() {
            console.log("enterGame");
            let isFirstLogin = !StorageUtils.Inst.IsHasLogin();
            if (isFirstLogin)
                StorageUtils.Inst.SetLogin();
            UISystem.Inst.CreateWindowView(UIMainView);
            this.initShader();
            PlatformMgr.Inst.checkNetwork();
            if (Laya.Browser.onQGMiniGame && qg)
                qg["reportMonitor"]('game_scene', 0);
            LandCtrl.Inst.init();
            SceneMgr.Inst.InitLoadSceneConfig();
            Laya.timer.once(100, this, this.StartGameAni);
        }
        StartGameAni() {
            UISystem.Inst.RemoveWindowClass(UILoadingView);
            let camera = SceneMgr.Inst.GetCamera();
            camera.transform.position = new Laya.Vector3(25.47, 73.68, -48.15);
            camera.transform.rotationEuler = new Laya.Vector3(-47.027, 180 - 36.397, 0);
            camera.fieldOfView = 90;
            let logoView = UISystem.Inst.CreateWindowView(UILogoView);
            logoView.SetAniEnd(2700);
            if (GameHtMgr.isOpenHt) {
                GameHtMgr.Inst.init();
            }
            Laya.Tween.to(camera, { fieldOfView: CameraModel.FOV_NORMAL }, 3000, Laya.Ease.sineOut, Laya.Handler.create(this, () => {
                SceneModel.Inst.AddToCats();
                GuideCtrl.Inst.start();
                BuildCtrl.Inst.init();
                PreloadingMgr.Inst.Init();
                SceneCtrl.Inst.init();
            }));
        }
    }

    class GlobalConfig {
        static init() {
            var reg = Laya.ClassUtils.regClass;
            reg("script/GameUI.ts", GameUI);
        }
    }
    GlobalConfig.width = 720;
    GlobalConfig.height = 1280;
    GlobalConfig.scaleMode = Laya.Stage.SCALE_FIXED_AUTO;
    GlobalConfig.screenMode = Laya.Stage.SCREEN_VERTICAL;
    GlobalConfig.alignV = "top";
    GlobalConfig.alignH = "left";
    GlobalConfig.startScene = "test/GameScene.scene";
    GlobalConfig.sceneRoot = "";
    GlobalConfig.debug = false;
    GlobalConfig.stat = false;
    GlobalConfig.physicsDebug = false;
    GlobalConfig.exportSceneToJson = true;
    GlobalConfig.isUseGm = false;
    GlobalConfig.isPlay = true;
    GlobalConfig.isRunShader = false;
    GlobalConfig.isShowCat = false;
    GlobalConfig.isDouble = false;
    GlobalConfig.CheckCatEventTIme = 0;
    GlobalConfig.isLoadScene = false;
    GlobalConfig.scale = 1;
    GlobalConfig.viewWidth = 720;
    GlobalConfig.viewHeight = 1280;
    GlobalConfig.Font_Normal = "Microsoft YaHei";
    GlobalConfig.isOpenAutoTouch = true;
    GlobalConfig.autoTouchTimeGap = 100;
    GlobalConfig.scene_scale = 1;
    GlobalConfig.FPS = 1 / 60;
    GlobalConfig.isActive = true;
    GlobalConfig.init();

    class Main {
        constructor() {
            if (window["Laya3D"])
                Laya3D.init(GlobalConfig.width, GlobalConfig.height);
            else
                Laya.init(GlobalConfig.width, GlobalConfig.height, Laya["WebGL"]);
            Laya["Physics"] && Laya["Physics"].enable();
            Laya["DebugPanel"] && Laya["DebugPanel"].enable();
            Laya.stage.scaleMode = GlobalConfig.scaleMode;
            Laya.stage.screenMode = GlobalConfig.screenMode;
            Laya.stage.alignV = GlobalConfig.alignV;
            Laya.stage.alignH = GlobalConfig.alignH;
            Laya.URL.exportSceneToJson = GlobalConfig.exportSceneToJson;
            if (GlobalConfig.debug || Laya.Utils.getQueryString("debug") == "true")
                Laya.enableDebugPanel();
            if (GlobalConfig.physicsDebug && Laya["PhysicsDebugDraw"])
                Laya["PhysicsDebugDraw"].enable();
            if (GlobalConfig.stat) {
                Laya.Stat.show();
                Laya.alertGlobalError(true);
            }
            else {
            }
            Laya.stage.frameRate = Laya.Stage.FRAME_FAST;
            Laya.ResourceVersion.enable("version.json", Laya.Handler.create(this, this.onVersionLoaded), Laya.ResourceVersion.FILENAME_VERSION);
        }
        onVersionLoaded() {
            Laya.AtlasInfoManager.enable("fileconfig.json", Laya.Handler.create(this, this.onConfigLoaded));
        }
        onConfigLoaded() {
            GlobalConfig.startScene && Laya.Scene.open(GlobalConfig.startScene);
        }
    }
    new Main();

}());
//# sourceMappingURL=bundle.js.map
